@@ -122,6 +122,8 @@ enum Opcode : uint8_t {
     SExt8,
     SExt16,
     // Takes Int32 and returns Int64:
+    SExt8To64,
+    SExt16To64,
     SExt32,
     ZExt32,
     // Does a bitwise truncation of Int64->Int32 and Double->Float:
@@ -347,7 +349,10 @@ enum Opcode : uint8_t {
     // SIMD instructions
     VectorExtractLane,
     VectorReplaceLane,
-    VectorDupElement, // Currently only some architectures support this.
+
+    // Currently only some architectures support this.
+    // FIXME: Expand this to identical instructions for the other architectures as a macro.
+    VectorDupElement,
 
     VectorSplat,
 
@@ -412,6 +417,15 @@ enum Opcode : uint8_t {
     VectorMulSat,
     VectorSwizzle,
 
+    // Relaxed SIMD
+
+    VectorRelaxedSwizzle,
+
+    // Currently only some architectures support this.
+    // FIXME: Expand this to identical instructions for the other architectures as a macro.
+    VectorMulByElement,
+    VectorShiftByVector,
+
     // SSA support, in the style of DFG SSA.
     Upsilon, // This uses the UpsilonValue class.
     Phi,
