@@ -31,10 +31,9 @@
 #include "AirSpecial.h"
 #include "AirStackSlot.h"
 #include "B3Value.h"
-#include "FPRInfo.h"
 #include "GPRInfo.h"
 
-#if ASSERT_DISABLED
+#if !ASSERT_ENABLED
 IGNORE_RETURN_TYPE_WARNINGS_BEGIN
 #endif
 
@@ -71,9 +70,14 @@ bool Arg::usesTmp(Air::Tmp tmp) const
     return uses;
 }
 
+bool Arg::canRepresent(Type type) const
+{
+    return isBank(bankForType(type));
+}
+
 bool Arg::canRepresent(Value* value) const
 {
-    return isBank(bankForType(value->type()));
+    return canRepresent(value->type());
 }
 
 bool Arg::isCompatibleBank(const Arg& other) const
@@ -92,12 +96,14 @@ unsigned Arg::jsHash() const
     switch (m_kind) {
     case Invalid:
     case Special:
+    case SIMDInfo:
         break;
     case Tmp:
         result += m_base.internalValue();
         break;
     case Imm:
     case BitImm:
+    case ZeroReg:
     case CallArg:
     case RelCond:
     case ResCond:
@@ -106,6 +112,7 @@ unsigned Arg::jsHash() const
     case WidthArg:
         result += static_cast<unsigned>(m_offset);
         break;
+    case Imm64:
     case BigImm:
     case BitImm64:
         result += static_cast<unsigned>(m_offset);
@@ -125,6 +132,11 @@ unsigned Arg::jsHash() const
         result += m_base.internalValue();
         result += m_index.internalValue();
         break;
+    case PreIndex:
+    case PostIndex:
+        result += m_offset;
+        result += m_base.internalValue();
+        break;
     case Stack:
         result += static_cast<unsigned>(m_scale);
         result += stackSlot()->index();
@@ -146,6 +158,9 @@ void Arg::dump(PrintStream& out) const
     case Imm:
         out.print("$", m_offset);
         return;
+    case Imm64:
+        out.printf("$0x%llx", static_cast<long long unsigned>(m_offset));
+        return;
     case BigImm:
         out.printf("$0x%llx", static_cast<long long unsigned>(m_offset));
         return;
@@ -155,6 +170,9 @@ void Arg::dump(PrintStream& out) const
     case BitImm64:
         out.printf("$0x%llx", static_cast<long long unsigned>(m_offset));
         return;
+    case ZeroReg:
+        out.print("%xzr");
+        return;
     case SimpleAddr:
         out.print("(", base(), ")");
         return;
@@ -172,6 +190,12 @@ void Arg::dump(PrintStream& out) const
             out.print(",", scale());
         out.print(")");
         return;
+    case PreIndex:
+        out.print("(", base(), ",Pre($", offset(), "))");
+        return;
+    case PostIndex:
+        out.print("(", base(), ",Post($", offset(), "))");
+        return;
     case Stack:
         if (offset())
             out.print(offset());
@@ -200,6 +224,9 @@ void Arg::dump(PrintStream& out) const
     case WidthArg:
         out.print(width());
         return;
+    case SIMDInfo:
+        out.print("{ ", simdInfo().lane, ", ", simdInfo().signMode, " }");
+        return;
     }
 
     RELEASE_ASSERT_NOT_REACHED();
@@ -223,6 +250,9 @@ void printInternal(PrintStream& out, Arg::Kind kind)
     case Arg::Imm:
         out.print("Imm");
         return;
+    case Arg::Imm64:
+        out.print("Imm64");
+        return;
     case Arg::BigImm:
         out.print("BigImm");
         return;
@@ -232,6 +262,9 @@ void printInternal(PrintStream& out, Arg::Kind kind)
     case Arg::BitImm64:
         out.print("BitImm64");
         return;
+    case Arg::ZeroReg:
+        out.print("ZeroReg");
+        return;
     case Arg::SimpleAddr:
         out.print("SimpleAddr");
         return;
@@ -250,6 +283,12 @@ void printInternal(PrintStream& out, Arg::Kind kind)
     case Arg::Index:
         out.print("Index");
         return;
+    case Arg::PreIndex:
+        out.print("PreIndex");
+        return;
+    case Arg::PostIndex:
+        out.print("PostIndex");
+        return;
     case Arg::RelCond:
         out.print("RelCond");
         return;
@@ -268,6 +307,9 @@ void printInternal(PrintStream& out, Arg::Kind kind)
     case Arg::WidthArg:
         out.print("WidthArg");
         return;
+    case Arg::SIMDInfo:
+        out.print("SIMDInfo");
+        return;
     }
 
     RELEASE_ASSERT_NOT_REACHED();
@@ -378,7 +420,7 @@ void printInternal(PrintStream& out, Arg::Signedness signedness)
 
 } // namespace WTF
 
-#if ASSERT_DISABLED
+#if !ASSERT_ENABLED
 IGNORE_RETURN_TYPE_WARNINGS_END
 #endif
 
