@@ -68,6 +68,7 @@ public:
         // OSR exit and tail calls.
         // BitImm is an immediate for Bitwise operation (And, Xor, etc).
         Imm,
+        Imm64,
         BigImm,
         BitImm,
         BitImm64,
@@ -524,6 +525,14 @@ public:
         return result;
     }
 
+    static Arg imm64(int64_t value)
+    {
+        Arg result;
+        result.m_kind = Imm64;
+        result.m_offset = value;
+        return result;
+    }
+
     static Arg bigImm(int64_t value)
     {
         if constexpr (is32Bit())
@@ -781,11 +790,6 @@ public:
             && m_scale == other.m_scale;
     }
 
-    bool operator!=(const Arg& other) const
-    {
-        return !(*this == other);
-    }
-
     explicit operator bool() const { return *this != Arg(); }
 
     Kind kind() const
@@ -803,6 +807,11 @@ public:
         return kind() == Imm;
     }
 
+    bool isImm64() const
+    {
+        return kind() == Imm64;
+    }
+
     bool isBigImm() const
     {
         return kind() == BigImm;
@@ -827,6 +836,7 @@ public:
     {
         switch (kind()) {
         case Imm:
+        case Imm64:
         case BigImm:
         case BitImm:
         case BitImm64:
@@ -1128,6 +1138,7 @@ public:
     {
         switch (kind()) {
         case Imm:
+        case Imm64:
         case BigImm:
         case BitImm:
         case BitImm64:
@@ -1161,6 +1172,7 @@ public:
     {
         switch (kind()) {
         case Imm:
+        case Imm64:
         case BitImm:
         case BitImm64:
         case RelCond:
@@ -1193,6 +1205,7 @@ public:
     {
         switch (kind()) {
         case Imm:
+        case Imm64:
         case BitImm:
         case BitImm64:
         case Special:
@@ -1275,12 +1288,25 @@ public:
         if (isX86())
             return B3::isRepresentableAs<int32_t>(value);
         if (isARM64())
-            return isUInt12(value);
+            return isUInt12(value) || isUInt12(toTwosComplement(value));
         if (isARM_THUMB2())
             return isValidARMThumb2Immediate(value);
         return false;
     }
 
+    static bool isValidImm64Form(int64_t value)
+    {
+        if (isARM64()) {
+            if (isUInt12(value) || isUInt12(toTwosComplement(value)))
+                return true;
+            int64_t shifted = value >> 12;
+            if ((shifted << 12) == value)
+                return isUInt12(shifted) || isUInt12(toTwosComplement(shifted));
+            return false;
+        }
+        return false;
+    }
+
     static bool isValidBitImmForm(int64_t value)
     {
         if (isX86())
@@ -1304,7 +1330,7 @@ public:
     template<typename Int, typename = Value::IsLegalOffset<Int>>
     static bool isValidAddrForm(Air::Opcode opcode, Int offset, std::optional<Width> width = std::nullopt)
     {
-#if !CPU(ARM_THUM2)
+#if !CPU(ARM_THUMB2)
         UNUSED_PARAM(opcode);
 #endif
         if (isX86())
@@ -1327,7 +1353,7 @@ public:
             case Width64:
                 return isValidScaledUImm12<64>(offset);
             case Width128:
-                return false;
+                return isValidScaledUImm12<128>(offset);
             }
         }
 
@@ -1340,7 +1366,7 @@ public:
         case MoveFloat:
             if (!std::is_signed<Int>::value)
                 return !((offset & 3) || (offset > (255 * 4)));
-            return !((offset & 3) || (offset > (255 * 4)) || (static_cast<std::make_signed<Int>::type>(offset) < -(255 * 4)));
+            return !((offset & 3) || (offset > (255 * 4)) || (static_cast<typename std::make_signed<Int>::type>(offset) < -(255 * 4)));
         default:
             return false;
         }
@@ -1349,14 +1375,23 @@ public:
     }
 
     template<typename Int, typename = Value::IsLegalOffset<Int>>
-    static bool isValidIndexForm(unsigned scale, Int offset, std::optional<Width> width = std::nullopt)
+    static bool isValidIndexForm(Air::Opcode opcode, unsigned scale, Int offset, std::optional<Width> width = std::nullopt)
     {
         if (!isValidScale(scale, width))
             return false;
         if (isX86())
             return true;
-        if (isARM64() || isARM_THUMB2())
+        if (isARM64())
             return !offset;
+        if (isARM_THUMB2()) {
+            switch (opcode) {
+            case MoveFloat:
+            case MoveDouble:
+                return false;
+            default:
+                return !offset;
+            }
+        }
         return false;
     }
 
@@ -1380,6 +1415,8 @@ public:
             return true;
         case Imm:
             return isValidImmForm(value());
+        case Imm64:
+            return isValidImm64Form(value());
         case BigImm:
             return true;
         case BitImm:
@@ -1396,7 +1433,7 @@ public:
         case CallArg:
             return isValidAddrForm(opcode, offset(), width);
         case Index:
-            return isValidIndexForm(scale(), offset(), width);
+            return isValidIndexForm(opcode, scale(), offset(), width);
         case PreIndex:
         case PostIndex:
             return isValidIncrementIndexForm(offset());
@@ -1488,7 +1525,7 @@ public:
     {
         if constexpr (is32Bit())
             UNREACHABLE_FOR_PLATFORM();
-        ASSERT(isBigImm() || isBitImm64());
+        ASSERT(isBigImm() || isImm64() || isBitImm64());
         return MacroAssembler::TrustedImm64(value());
     }
 
