@@ -1,6 +1,6 @@
 #!/usr/bin/env ruby
 
-# Copyright (C) 2015-2016 Apple Inc. All rights reserved.
+# Copyright (C) 2015-2017 Apple Inc. All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
 # modification, are permitted provided that the following conditions
@@ -44,21 +44,60 @@ class Opcode
 end
 
 class Arg
-    attr_reader :role, :type, :width
+    attr_reader :role, :bank, :width
 
-    def initialize(role, type, width)
+    def initialize(role, bank, width)
         @role = role
-        @type = type
+        @bank = bank
         @width = width
     end
+    
+    def self.widthCode(width)
+        if width == "Ptr"
+            "pointerWidth()"
+        else
+            "Width#{width}"
+        end
+    end
 
     def widthCode
-        if width == "Ptr"
-            "Arg::pointerWidth()"
+        Arg.widthCode(width)
+    end
+    
+    def self.roleCode(role)
+        case role
+        when "U"
+            "Use"
+        when "D"
+            "Def"
+        when "ZD"
+            "ZDef"
+        when "UD"
+            "UseDef"
+        when "UZD"
+            "UseZDef"
+        when "UA"
+            "UseAddr"
+        when "S"
+            "Scratch"
+        when "ED"
+            "EarlyDef"
+        when "EZD"
+            "EarlyZDef"
+        when "LU"
+            "LateUse"
         else
-            "Arg::Width#{width}"
+            raise
         end
     end
+    
+    def roleCode
+        Arg.roleCode(role)
+    end
+    
+    def to_s
+        "#{role}:#{bank}:#{width}"
+    end
 end
 
 class Overload
@@ -182,7 +221,7 @@ def lex(str, fileName)
 end
 
 def isRole(token)
-    token =~ /\A((U)|(D)|(UD)|(ZD)|(UZD)|(UA)|(S))\Z/
+    token =~ /\A((U)|(D)|(UD)|(ZD)|(UZD)|(UA)|(S)|(ED)|(EZD)|(LU))\Z/
 end
 
 def isGF(token)
@@ -190,15 +229,15 @@ def isGF(token)
 end
 
 def isKind(token)
-    token =~ /\A((Tmp)|(Imm)|(BigImm)|(BitImm)|(BitImm64)|(Addr)|(Index)|(RelCond)|(ResCond)|(DoubleCond))\Z/
+    token =~ /\A((Tmp)|(Imm)|(Imm64)|(BigImm)|(BitImm)|(BitImm64)|(ZeroReg)|(SimpleAddr)|(Addr)|(ExtendedOffsetAddr)|(Index)|(PreIndex)|(PostIndex)|(RelCond)|(ResCond)|(DoubleCond)|(StatusCond)|(SIMDInfo))\Z/
 end
 
 def isArch(token)
-    token =~ /\A((x86)|(x86_32)|(x86_64)|(arm)|(armv7)|(arm64)|(32)|(64))\Z/
+    token =~ /\A((x86)|(x86_32)|(x86_64_avx)|(x86_64)|(arm)|(armv7)|(arm64e)|(arm64_lse)|(arm64)|(32)|(64))\Z/
 end
 
 def isWidth(token)
-    token =~ /\A((8)|(16)|(32)|(64)|(Ptr))\Z/
+    token =~ /\A((8)|(16)|(32)|(64)|(Ptr)|(128))\Z/
 end
 
 def isKeyword(token)
@@ -255,23 +294,23 @@ class Parser
         result
     end
 
-    def consumeType
+    def consumeBank
         result = token.string
-        parseError("Expected type (G or F)") unless isGF(result)
+        parseError("Expected bank (G or F)") unless isGF(result)
         advance
         result
     end
 
     def consumeKind
         result = token.string
-        parseError("Expected kind (Imm, BigImm, BitImm, BitImm64, Tmp, Addr, Index, RelCond, ResCond, or DoubleCond)") unless isKind(result)
+        parseError("Expected kind (Imm, Imm64, BigImm, BitImm, BitImm64, ZeroReg, Tmp, SimpleAddr, Addr, ExtendedOffsetAddr, Index, PreIndex, PostIndex, RelCond, ResCond, DoubleCond, or StatusCond)") unless isKind(result)
         advance
         result
     end
 
     def consumeWidth
         result = token.string
-        parseError("Expected width (8, 16, 32, or 64)") unless isWidth(result)
+        parseError("Expected width (8, 16, 32, 64, or 128)") unless isWidth(result)
         advance
         result
     end
@@ -289,16 +328,22 @@ class Parser
                 result << "X86"
             when "x86_64"
                 result << "X86_64"
+            when "x86_64_avx"
+                result << "X86_64_AVX"
             when "arm"
-                result << "ARMv7"
+                result << "ARM_THUMB2"
                 result << "ARM64"
             when "armv7"
-                result << "ARMv7"
+                result << "ARM_THUMB2"
             when "arm64"
                 result << "ARM64"
+            when "arm64e"
+                result << "ARM64E"
+            when "arm64_lse"
+                result << "ARM64_LSE"
             when "32"
                 result << "X86"
-                result << "ARMv7"
+                result << "ARM_THUMB2"
             when "64"
                 result << "X86_64"
                 result << "ARM64"
@@ -369,11 +414,11 @@ class Parser
                     loop {
                         role = consumeRole
                         consume(":")
-                        type = consumeType
+                        bank = consumeBank
                         consume(":")
                         width = consumeWidth
                         
-                        signature << Arg.new(role, type, width)
+                        signature << Arg.new(role, bank, width)
                         
                         break unless token == ","
                         consume(",")
@@ -426,14 +471,22 @@ class Parser
                         parseError("Form has wrong number of arguments for overload") unless kinds.length == signature.length
                         kinds.each_with_index {
                             | kind, index |
-                            if kind.name == "Imm" or kind.name == "BigImm" or kind.name == "BitImm" or kind.name == "BitImm64"
+                            if kind.name == "Imm" or kind.name == "Imm64" or kind.name == "BigImm" or kind.name == "BitImm" or kind.name == "BitImm64"
                                 if signature[index].role != "U"
                                     parseError("Form has an immediate for a non-use argument")
                                 end
-                                if signature[index].type != "G"
+                                if signature[index].bank != "G"
                                     parseError("Form has an immediate for a non-general-purpose argument")
                                 end
                             end
+                            if kind.name == "ZeroReg"
+                                if signature[index].role != "U"
+                                    parseError("Zero immediate must be a use argument")
+                                end
+                                if signature[index].bank != "G"
+                                    parseError("Zero immediate must be a general-purpose argument")
+                                end
+                            end
                         }
                         forms << Form.new(kinds, altName, intersectArchs(opcodeArchs, formArchs))
 
@@ -477,9 +530,10 @@ end
 
 writeH("Opcode") {
     | outp |
+    outp.puts "#if ENABLE(B3_JIT)"
     outp.puts "namespace JSC { namespace B3 { namespace Air {"
     outp.puts "enum Opcode : int16_t {"
-    $opcodes.keys.sort.each {
+    $opcodes.keys.each {
         | opcode |
         outp.puts "    #{opcode},"
     }
@@ -492,6 +546,7 @@ writeH("Opcode") {
     outp.puts "class PrintStream;"
     outp.puts "JS_EXPORT_PRIVATE void printInternal(PrintStream&, JSC::B3::Air::Opcode);"
     outp.puts "} // namespace WTF"
+    outp.puts "#endif // ENABLE(B3_JIT)"
 }
 
 # From here on, we don't try to emit properly indented code, since we're using a recursive pattern
@@ -530,14 +585,14 @@ def matchForms(outp, speed, forms, columnIndex, columnGetter, filter, callback)
     outp.puts "switch (#{columnGetter[columnIndex]}) {"
     groups.each_pair {
         | key, value |
-        outp.puts "#if USE(JSVALUE64)" if key == "BigImm" or key == "BitImm64"
+        outp.puts "#if USE(JSVALUE64)" if key == "Imm64" || key == "BitImm64"
         Kind.argKinds(key).each {
             | argKind |
             outp.puts "case Arg::#{argKind}:"
         }
         matchForms(outp, speed, value, columnIndex + 1, columnGetter, filter, callback)
         outp.puts "break;"
-        outp.puts "#endif // USE(JSVALUE64)" if key == "BigImm" or key == "BitImm64"
+        outp.puts "#endif // USE(JSVALUE64)" if key == "Imm64" || key == "BitImm64"
     }
     outp.puts "default:"
     outp.puts "break;"
@@ -545,10 +600,10 @@ def matchForms(outp, speed, forms, columnIndex, columnGetter, filter, callback)
 end
 
 def matchInstOverload(outp, speed, inst)
-    outp.puts "switch (#{inst}->opcode) {"
+    outp.puts "switch (#{inst}->kind.opcode) {"
     $opcodes.values.each {
         | opcode |
-        outp.puts "case #{opcode.name}:"
+        outp.puts "case Opcode::#{opcode.name}:"
         if opcode.custom
             yield opcode, nil
         else
@@ -593,27 +648,63 @@ def matchInstOverloadForm(outp, speed, inst)
     }
 end
 
+$runTimeArchs = {
+    "ARM64_LSE" => "ARM64",
+    "X86_64_AVX" => "X86_64"
+}
 def beginArchs(outp, archs)
     return unless archs
     if archs.empty?
         outp.puts "#if 0"
         return
     end
-    outp.puts("#if " + archs.map {
+    compileTime = []
+    runTime = []
+    archs.each {|arch|
+        if $runTimeArchs.has_key? arch
+            compileTime << $runTimeArchs[arch]
+        else
+            compileTime << arch
+        end
+    }
+    if compileTime.empty?
+        outp.puts("#if 1")
+    else
+        outp.puts("#if " + compileTime.map {
+                      | arch |
+                      "CPU(#{arch})"
+                  }.join(" || "))
+    end
+    outp.puts("if (" + archs.map {
                   | arch |
-                  "CPU(#{arch})"
-              }.join(" || "))
+                  "is#{arch}()"
+              }.join(" || ") + ") {")
 end
 
 def endArchs(outp, archs)
     return unless archs
+    outp.puts "}"
     outp.puts "#endif"
 end
 
+maxNumOperands = 0
+$opcodes.values.each {
+    | opcode |
+    next if opcode.custom
+    opcode.overloads.each {
+        | overload |
+        maxNumOperands = overload.signature.length if overload.signature.length > maxNumOperands
+    }
+}
+
+formTableWidth = (maxNumOperands + 1) * maxNumOperands / 2
+
 writeH("OpcodeUtils") {
     | outp |
+    outp.puts "#if ENABLE(B3_JIT)"
     outp.puts "#include \"AirCustom.h\""
     outp.puts "#include \"AirInst.h\""
+    outp.puts "#include \"AirFormTable.h\""
     outp.puts "namespace JSC { namespace B3 { namespace Air {"
     
     outp.puts "inline bool opgenHiddenTruth() { return true; }"
@@ -625,40 +716,33 @@ writeH("OpcodeUtils") {
     outp.puts "} while (false)"
 
     outp.puts "template<typename Functor>"
-    outp.puts "void Inst::forEachArg(const Functor& functor)"
+    outp.puts "ALWAYS_INLINE void Inst::forEachArg(const Functor& functor)"
     outp.puts "{"
-    matchInstOverload(outp, :fast, "this") {
-        | opcode, overload |
+    outp.puts "switch (kind.opcode) {"
+    $opcodes.values.each {
+        | opcode |
         if opcode.custom
-            outp.puts "#{opcode.name}Custom::forEachArg(*this, functor);"
-        else
-            overload.signature.each_with_index {
-                | arg, index |
-                
-                role = nil
-                case arg.role
-                when "U"
-                    role = "Use"
-                when "D"
-                    role = "Def"
-                when "ZD"
-                    role = "ZDef"
-                when "UD"
-                    role = "UseDef"
-                when "UZD"
-                    role = "UseZDef"
-                when "UA"
-                    role = "UseAddr"
-                when "S"
-                    role = "Scratch"
-                else
-                    raise
-                end
-
-                outp.puts "functor(args[#{index}], Arg::#{role}, Arg::#{arg.type}P, #{arg.widthCode});"
-            }
+            outp.puts "case Opcode::#{opcode.name}:"
         end
     }
+    outp.puts "forEachArgCustom(scopedLambdaRef<EachArgCallback>(functor));"
+    outp.puts "return;"
+    outp.puts "default:"
+    outp.puts "forEachArgSimple(functor);"
+    outp.puts "return;"
+    outp.puts "}"
+    outp.puts "}"
+    
+    outp.puts "template<typename Func>"
+    outp.puts "ALWAYS_INLINE void Inst::forEachArgSimple(const Func& func)"
+    outp.puts "{"
+    outp.puts "    size_t numOperands = args.size();"
+    outp.puts "    size_t formOffset = (numOperands - 1) * numOperands / 2;"
+    outp.puts "    const uint8_t* formBase = g_formTable + kind.opcode * #{formTableWidth} + formOffset;"
+    outp.puts "    for (size_t i = 0; i < numOperands; ++i) {"
+    outp.puts "        uint8_t form = formBase[i];"
+    outp.puts "        func(args[i], decodeFormRole(form), decodeFormBank(form), decodeFormWidth(form));"
+    outp.puts "    }"
     outp.puts "}"
 
     outp.puts "template<typename... Arguments>"
@@ -668,7 +752,7 @@ writeH("OpcodeUtils") {
     outp.puts "switch (opcode) {"
     $opcodes.values.each {
         | opcode |
-        outp.puts "case #{opcode.name}:"
+        outp.puts "case Opcode::#{opcode.name}:"
         if opcode.custom
             outp.puts "OPGEN_RETURN(#{opcode.name}Custom::isValidFormStatic(arguments...));"
         else
@@ -721,7 +805,7 @@ writeH("OpcodeUtils") {
     $opcodes.values.each {
         | opcode |
         if opcode.attributes[:terminal]
-            outp.puts "case #{opcode.name}:"
+            outp.puts "case Opcode::#{opcode.name}:"
             didFindTerminals = true
         end
     }
@@ -740,7 +824,7 @@ writeH("OpcodeUtils") {
     $opcodes.values.each {
         | opcode |
         if opcode.attributes[:return]
-            outp.puts "case #{opcode.name}:"
+            outp.puts "case Opcode::#{opcode.name}:"
             didFindReturns = true
         end
     }
@@ -753,11 +837,15 @@ writeH("OpcodeUtils") {
     outp.puts "}"
     
     outp.puts "} } } // namespace JSC::B3::Air"
+    outp.puts "#endif // ENABLE(B3_JIT)"
 }
 
 writeH("OpcodeGenerated") {
     | outp |
+    outp.puts "#if ENABLE(B3_JIT)"
     outp.puts "#include \"AirInstInlines.h\""
+    outp.puts "#include \"B3ProcedureInlines.h\""
+    outp.puts "#include \"CCallHelpers.h\""
     outp.puts "#include \"wtf/PrintStream.h\""
     outp.puts "namespace WTF {"
     outp.puts "using namespace JSC::B3::Air;"
@@ -766,7 +854,7 @@ writeH("OpcodeGenerated") {
     outp.puts "    switch (opcode) {"
     $opcodes.keys.each {
         | opcode |
-        outp.puts "    case #{opcode}:"
+        outp.puts "    case Opcode::#{opcode}:"
         outp.puts "        out.print(\"#{opcode}\");"
         outp.puts "        return;"
     }
@@ -775,6 +863,56 @@ writeH("OpcodeGenerated") {
     outp.puts "}"
     outp.puts "} // namespace WTF"
     outp.puts "namespace JSC { namespace B3 { namespace Air {"
+    
+    outp.puts "const uint8_t g_formTable[#{$opcodes.size * formTableWidth}] = {"
+    $opcodes.values.each {
+        | opcode |
+        overloads = [nil] * (maxNumOperands + 1)
+        unless opcode.custom
+            opcode.overloads.each {
+                | overload |
+                overloads[overload.signature.length] = overload
+            }
+        end
+        
+        (0..maxNumOperands).each {
+            | numOperands |
+            overload = overloads[numOperands]
+            if overload
+                outp.puts "// #{opcode.name} #{overload.signature.join(', ')}"
+                numOperands.times {
+                    | index |
+                    arg = overload.signature[index]
+                    outp.print "ENCODE_INST_FORM(Arg::#{arg.roleCode}, #{arg.bank}P, #{arg.widthCode}), "
+                }
+            else
+                outp.puts "// Invalid: #{opcode.name} with numOperands = #{numOperands}"
+                numOperands.times {
+                    outp.print "INVALID_INST_FORM, "
+                }
+            end
+            outp.puts
+        }
+    }
+    outp.puts "};"
+    
+    outp.puts "void Inst::forEachArgCustom(ScopedLambda<EachArgCallback> lambda)"
+    outp.puts "{"
+    outp.puts "switch (kind.opcode) {"
+    $opcodes.values.each {
+        | opcode |
+        if opcode.custom
+            outp.puts "case Opcode::#{opcode.name}:"
+            outp.puts "#{opcode.name}Custom::forEachArg(*this, lambda);"
+            outp.puts "break;"
+        end
+    }
+    outp.puts "default:"
+    outp.puts "dataLog(\"Bad call to forEachArgCustom, not custom opcode: \", kind, \"\\n\");"
+    outp.puts "RELEASE_ASSERT_NOT_REACHED();"
+    outp.puts "}"
+    outp.puts "}"
+    
     outp.puts "bool Inst::isValidForm()"
     outp.puts "{"
     matchInstOverloadForm(outp, :safe, "this") {
@@ -793,32 +931,51 @@ writeH("OpcodeGenerated") {
                 # Some kinds of Args reqire additional validation.
                 case kind.name
                 when "Tmp"
-                    outp.puts "if (!args[#{index}].tmp().is#{arg.type}P())"
+                    outp.puts "if (!args[#{index}].tmp().is#{arg.bank}P())"
                     outp.puts "OPGEN_RETURN(false);"
                 when "Imm"
                     outp.puts "if (!Arg::isValidImmForm(args[#{index}].value()))"
                     outp.puts "OPGEN_RETURN(false);"
+                when "Imm64"
+                    outp.puts "if (!Arg::isValidImm64Form(args[#{index}].value()))"
+                    outp.puts "OPGEN_RETURN(false);"
                 when "BitImm"
                     outp.puts "if (!Arg::isValidBitImmForm(args[#{index}].value()))"
                     outp.puts "OPGEN_RETURN(false);"
                 when "BitImm64"
                     outp.puts "if (!Arg::isValidBitImm64Form(args[#{index}].value()))"
                     outp.puts "OPGEN_RETURN(false);"
+                when "SimpleAddr"
+                    outp.puts "if (!args[#{index}].ptr().isGP())"
+                    outp.puts "OPGEN_RETURN(false);"
                 when "Addr"
                     if arg.role == "UA"
                         outp.puts "if (args[#{index}].isStack() && args[#{index}].stackSlot()->isSpill())"
                         outp.puts "OPGEN_RETURN(false);"
                     end
-                    
-                    outp.puts "if (!Arg::isValidAddrForm(args[#{index}].offset()))"
+                    outp.puts "if (!Arg::isValidAddrForm(this->kind.opcode, args[#{index}].offset()))"
                     outp.puts "OPGEN_RETURN(false);"
+                when "ExtendedOffsetAddr"
+                    if arg.role == "UA"
+                        outp.puts "if (args[#{index}].isStack() && args[#{index}].stackSlot()->isSpill())"
+                        outp.puts "OPGEN_RETURN(false);"
+                    end
                 when "Index"
-                    outp.puts "if (!Arg::isValidIndexForm(args[#{index}].scale(), args[#{index}].offset(), #{arg.widthCode}))"
+                    outp.puts "if (!Arg::isValidIndexForm(this->kind.opcode, args[#{index}].scale(), args[#{index}].offset(), #{arg.widthCode}))"
+                    outp.puts "OPGEN_RETURN(false);"
+                when "PreIndex"
+                    outp.puts "if (!Arg::isValidIncrementIndexForm(args[#{index}].offset()))"
+                    outp.puts "OPGEN_RETURN(false);"
+                when "PostIndex"
+                    outp.puts "if (!Arg::isValidIncrementIndexForm(args[#{index}].offset()))"
                     outp.puts "OPGEN_RETURN(false);"
                 when "BigImm"
                 when "RelCond"
                 when "ResCond"
                 when "DoubleCond"
+                when "StatusCond"
+                when "ZeroReg"
+                when "SIMDInfo"
                 else
                     raise "Unexpected kind: #{kind.name}"
                 end
@@ -836,10 +993,10 @@ writeH("OpcodeGenerated") {
 
     outp.puts "bool Inst::admitsStack(unsigned argIndex)"
     outp.puts "{"
-    outp.puts "switch (opcode) {"
+    outp.puts "switch (kind.opcode) {"
     $opcodes.values.each {
         | opcode |
-        outp.puts "case #{opcode.name}:"
+        outp.puts "case Opcode::#{opcode.name}:"
 
         if opcode.custom
             outp.puts "OPGEN_RETURN(#{opcode.name}Custom::admitsStack(*this, argIndex));"
@@ -976,14 +1133,32 @@ writeH("OpcodeGenerated") {
     outp.puts "return false;"
     outp.puts "}"
 
+    outp.puts "bool Inst::admitsExtendedOffsetAddr(unsigned argIndex)"
+    outp.puts "{"
+    outp.puts "switch (kind.opcode) {"
+    $opcodes.values.each {
+        | opcode |
+        if opcode.custom
+            outp.puts "case Opcode::#{opcode.name}:"
+            outp.puts "OPGEN_RETURN(#{opcode.name}Custom::admitsExtendedOffsetAddr(*this, argIndex));"
+            outp.puts "break;"
+        end
+    }
+    outp.puts "default:"
+    outp.puts "break;"
+    outp.puts "}"
+    outp.puts "return false;"
+    outp.puts "}"
+
+
     outp.puts "bool Inst::isTerminal()"
     outp.puts "{"
-    outp.puts "switch (opcode) {"
+    outp.puts "switch (kind.opcode) {"
     foundTrue = false
     $opcodes.values.each {
         | opcode |
         if opcode.attributes[:terminal]
-            outp.puts "case #{opcode.name}:"
+            outp.puts "case Opcode::#{opcode.name}:"
             foundTrue = true
         end
     }
@@ -993,7 +1168,7 @@ writeH("OpcodeGenerated") {
     $opcodes.values.each {
         | opcode |
         if opcode.custom
-            outp.puts "case #{opcode.name}:"
+            outp.puts "case Opcode::#{opcode.name}:"
             outp.puts "return #{opcode.name}Custom::isTerminal(*this);"
         end
     }
@@ -1004,12 +1179,14 @@ writeH("OpcodeGenerated") {
     
     outp.puts "bool Inst::hasNonArgNonControlEffects()"
     outp.puts "{"
-    outp.puts "switch (opcode) {"
+    outp.puts "if (kind.effects)"
+    outp.puts "return true;"
+    outp.puts "switch (kind.opcode) {"
     foundTrue = false
     $opcodes.values.each {
         | opcode |
         if opcode.attributes[:effects]
-            outp.puts "case #{opcode.name}:"
+            outp.puts "case Opcode::#{opcode.name}:"
             foundTrue = true
         end
     }
@@ -1019,7 +1196,7 @@ writeH("OpcodeGenerated") {
     $opcodes.values.each {
         | opcode |
         if opcode.custom
-            outp.puts "case #{opcode.name}:"
+            outp.puts "case Opcode::#{opcode.name}:"
             outp.puts "return #{opcode.name}Custom::hasNonArgNonControlEffects(*this);"
         end
     }
@@ -1030,12 +1207,14 @@ writeH("OpcodeGenerated") {
     
     outp.puts "bool Inst::hasNonArgEffects()"
     outp.puts "{"
-    outp.puts "switch (opcode) {"
+    outp.puts "if (kind.effects)"
+    outp.puts "return true;"
+    outp.puts "switch (kind.opcode) {"
     foundTrue = false
     $opcodes.values.each {
         | opcode |
         if opcode.attributes[:terminal] or opcode.attributes[:effects]
-            outp.puts "case #{opcode.name}:"
+            outp.puts "case Opcode::#{opcode.name}:"
             foundTrue = true
         end
     }
@@ -1045,7 +1224,7 @@ writeH("OpcodeGenerated") {
     $opcodes.values.each {
         | opcode |
         if opcode.custom
-            outp.puts "case #{opcode.name}:"
+            outp.puts "case Opcode::#{opcode.name}:"
             outp.puts "return #{opcode.name}Custom::hasNonArgEffects(*this);"
         end
     }
@@ -1082,25 +1261,37 @@ writeH("OpcodeGenerated") {
                 end
                 case kind.name
                 when "Tmp"
-                    if overload.signature[index].type == "G"
+                    if overload.signature[index].bank == "G"
                         outp.print "args[#{index}].gpr()"
                     else
                         outp.print "args[#{index}].fpr()"
                     end
                 when "Imm", "BitImm"
                     outp.print "args[#{index}].asTrustedImm32()"
-                when "BigImm", "BitImm64"
+                when "BigImm"
+                    outp.print "args[#{index}].asTrustedBigImm()"
+                when "Imm64", "BitImm64"
                     outp.print "args[#{index}].asTrustedImm64()"
-                when "Addr"
+                when "ZeroReg"
+                    outp.print "args[#{index}].asZeroReg()"
+                when "SimpleAddr", "Addr", "ExtendedOffsetAddr"
                     outp.print "args[#{index}].asAddress()"
                 when "Index"
                     outp.print "args[#{index}].asBaseIndex()"
+                when "PreIndex"
+                    outp.print "args[#{index}].asPreIndexAddress()"
+                when "PostIndex"
+                    outp.print "args[#{index}].asPostIndexAddress()"
                 when "RelCond"
                     outp.print "args[#{index}].asRelationalCondition()"
                 when "ResCond"
                     outp.print "args[#{index}].asResultCondition()"
                 when "DoubleCond"
                     outp.print "args[#{index}].asDoubleCondition()"
+                when "StatusCond"
+                    outp.print "args[#{index}].asStatusCondition()"
+                when "SIMDInfo"
+                    outp.print "args[#{index}].simdInfo()"
                 end
             }
 
@@ -1114,111 +1305,6 @@ writeH("OpcodeGenerated") {
     outp.puts "}"
 
     outp.puts "} } } // namespace JSC::B3::Air"
-}
-
-# This is a hack for JSAir. It's a joke.
-File.open("JSAir_opcode.js", "w") {
-    | outp |
-    outp.puts "\"use strict\";"
-    outp.puts "// Generated by opcode_generator.rb from #{$fileName} -- do not edit!"
-    
-    $opcodes.values.each {
-        | opcode |
-        outp.puts "const #{opcode.name} = Symbol(#{opcode.name.inspect});"
-    }
-    
-    outp.puts "function Inst_forEachArg(inst, func)"
-    outp.puts "{"
-    outp.puts "let replacement;"
-    outp.puts "switch (inst.opcode) {"
-    $opcodes.values.each {
-        | opcode |
-        outp.puts "case #{opcode.name}:"
-        if opcode.custom
-            outp.puts "#{opcode.name}Custom.forEachArg(inst, func);"
-        else
-            needOverloadSwitch = opcode.overloads.size != 1
-            outp.puts "switch (inst.args.length) {" if needOverloadSwitch
-            opcode.overloads.each {
-                | overload |
-                outp.puts "case #{overload.signature.length}:" if needOverloadSwitch
-                overload.signature.each_with_index {
-                    | arg, index |
-                    role = nil
-                    case arg.role
-                    when "U"
-                        role = "Use"
-                    when "D"
-                        role = "Def"
-                    when "ZD"
-                        role = "ZDef"
-                    when "UD"
-                        role = "UseDef"
-                    when "UZD"
-                        role = "UseZDef"
-                    when "UA"
-                        role = "UseAddr"
-                    when "S"
-                        role = "Scratch"
-                    else
-                        raise
-                    end
-                    
-                    outp.puts "inst.visitArg(#{index}, func, Arg.#{role}, #{arg.type}P, #{arg.width});"
-                }
-                outp.puts "break;"
-            }
-            if needOverloadSwitch
-                outp.puts "default:"
-                outp.puts "throw new Error(\"Bad overload\");"
-                outp.puts "break;"
-                outp.puts "}"
-            end
-        end
-        outp.puts "break;"
-    }
-    outp.puts "default:"
-    outp.puts "throw \"Bad opcode\";"
-    outp.puts "}"
-    outp.puts "}"
-    
-    outp.puts "function Inst_hasNonArgEffects(inst)"
-    outp.puts "{"
-    outp.puts "switch (inst.opcode) {"
-    foundTrue = false
-    $opcodes.values.each {
-        | opcode |
-        if opcode.attributes[:terminal] or opcode.attributes[:effects]
-            outp.puts "case #{opcode.name}:"
-            foundTrue = true
-        end
-    }
-    if foundTrue
-        outp.puts "return true;"
-    end
-    $opcodes.values.each {
-        | opcode |
-        if opcode.custom
-            outp.puts "case #{opcode.name}:"
-            outp.puts "return #{opcode.name}Custom.hasNonArgNonControlEffects(inst);"
-        end
-    }
-    outp.puts "default:"
-    outp.puts "return false;"
-    outp.puts "}"
-    outp.puts "}"
-    
-    outp.puts "function opcodeCode(opcode)"
-    outp.puts "{"
-    outp.puts "switch (opcode) {"
-    $opcodes.keys.sort.each_with_index {
-        | opcode, index |
-        outp.puts "case #{opcode}:"
-        outp.puts "return #{index}"
-    }
-    outp.puts "default:"
-    outp.puts "throw new Error(\"bad opcode\");"
-    outp.puts "}"
-    outp.puts "}"
+    outp.puts "#endif // ENABLE(B3_JIT)"
 }
 
