@@ -52,6 +52,7 @@ ExitMode mayExitImpl(Graph& graph, Node* node, StateType& state)
     case DoubleConstant:
     case LazyJSConstant:
     case Int52Constant:
+    case ConstantStoragePointer:
     case MovHint:
     case InitializeEntrypointArguments:
     case SetLocal:
@@ -65,6 +66,7 @@ ExitMode mayExitImpl(Graph& graph, Node* node, StateType& state)
     case LoopHint:
     case Phi:
     case Upsilon:
+    case ZombieHint:
     case ExitOK:
     case BottomValue:
     case PutHint:
@@ -102,8 +104,16 @@ ExitMode mayExitImpl(Graph& graph, Node* node, StateType& state)
     case PutByOffset:
     case PutClosureVar:
     case PutInternalField:
+    case PutGlobalVariable:
+    case GetByOffset:
+    case GetClosureVar:
+    case GetInternalField:
+    case GetGlobalLexicalVariable:
+    case GetGlobalVar:
     case RecordRegExpCachedResult:
     case NukeStructureAndSetButterfly:
+    case GetButterfly:
+    case GetIndexedPropertyStorage:
     case FilterCallLinkStatus:
     case FilterGetByStatus:
     case FilterPutByStatus:
@@ -111,8 +121,10 @@ ExitMode mayExitImpl(Graph& graph, Node* node, StateType& state)
     case FilterDeleteByStatus:
     case FilterCheckPrivateBrandStatus:
     case FilterSetPrivateBrandStatus:
-    case EnumeratorNextExtractMode:
-    case EnumeratorNextExtractIndex:
+    case ExtractFromTuple:
+    case CompareBelow:
+    case CompareBelowEq:
+    case CompareEqPtr:
         break;
 
     case EnumeratorNextUpdatePropertyName:
@@ -132,6 +144,7 @@ ExitMode mayExitImpl(Graph& graph, Node* node, StateType& state)
     case NewGeneratorFunction:
     case NewAsyncFunction:
     case NewAsyncGeneratorFunction:
+    case NewBoundFunction:
     case NewStringObject:
     case NewInternalFieldObject:
     case NewRegexp:
@@ -140,6 +153,9 @@ ExitMode mayExitImpl(Graph& graph, Node* node, StateType& state)
     case ToObject:
     case RegExpExecNonGlobalOrSticky:
     case RegExpMatchFastGlobal:
+    case CallWasm:
+    case AllocatePropertyStorage:
+    case ReallocatePropertyStorage:
         result = ExitsForExceptions;
         break;
 
@@ -148,6 +164,97 @@ ExitMode mayExitImpl(Graph& graph, Node* node, StateType& state)
             break;
         return Exits;
 
+    case ArithBitNot:
+        if (node->child1().useKind() == Int32Use)
+            break;
+        return Exits;
+
+    case ArithAbs:
+        if (node->arithMode() == Arith::Mode::Unchecked && node->child1().useKind() == Int32Use)
+            break;
+        return Exits;
+
+    case ArithMin:
+    case ArithMax:
+        if (graph.child(node, 0).useKind() == Int32Use)
+            break;
+        if (graph.child(node, 0).useKind() == DoubleRepUse)
+            break;
+        return Exits;
+
+    case ArithBitRShift:
+    case ArithBitLShift:
+    case BitURShift:
+    case ArithBitAnd:
+    case ArithBitOr:
+    case ArithBitXor:
+        if (node->isBinaryUseKind(Int32Use))
+            break;
+        return Exits;
+
+    case ArithClz32:
+        if (node->child1().useKind() == Int32Use || node->child1().useKind() == KnownInt32Use)
+            break;
+        return Exits;
+
+    case ArithAdd:
+    case ArithSub:
+    case ArithMul:
+        if (node->arithMode() == Arith::Mode::Unchecked && node->isBinaryUseKind(Int32Use))
+            break;
+        if (node->isBinaryUseKind(DoubleRepUse))
+            break;
+        return Exits;
+
+    case ArithNegate:
+        if (node->arithMode() == Arith::Mode::Unchecked && node->child1().useKind() == Int32Use)
+            break;
+        if (node->child1().useKind() == DoubleRepUse)
+            break;
+        return Exits;
+
+    case ArithDiv:
+    case ArithMod:
+        if (node->isBinaryUseKind(DoubleRepUse))
+            break;
+        return Exits;
+
+    case CompareEq:
+    case CompareStrictEq:
+    case CompareLess:
+    case CompareLessEq:
+    case CompareGreater:
+    case CompareGreaterEq:
+        if (node->isBinaryUseKind(Int32Use))
+            break;
+        if (node->isBinaryUseKind(DoubleRepUse))
+            break;
+        if (node->isBinaryUseKind(Int52RepUse))
+            break;
+        return Exits;
+
+    case ArithPow:
+        if (node->isBinaryUseKind(Int32Use))
+            break;
+        if (node->isBinaryUseKind(DoubleRepUse))
+            break;
+        return Exits;
+
+    case ArithRound:
+    case ArithFloor:
+    case ArithCeil:
+    case ArithTrunc:
+        if (node->child1().useKind() == DoubleRepUse && !producesInteger(node->arithRoundingMode()))
+            break;
+        return Exits;
+
+    case ArithSqrt:
+    case ArithUnary:
+    case ArithFRound:
+        if (node->child1().useKind() == DoubleRepUse)
+            break;
+        return Exits;
+
     default:
         // If in doubt, return true.
         return Exits;
