@@ -28,6 +28,7 @@
 
 #if ENABLE(DFG_JIT)
 
+#include "CacheableIdentifierInlines.h"
 #include "DFGGraph.h"
 #include "DFGPromotedHeapLocation.h"
 #include "DOMJITSignature.h"
@@ -265,6 +266,12 @@ void Node::convertToNewArrayWithSize()
     m_opInfo = indexingType;
 }
 
+void Node::convertToNewBoundFunction(FrozenValue* executable)
+{
+    m_op = NewBoundFunction;
+    m_opInfo = executable;
+}
+
 void Node::convertToDirectCall(FrozenValue* executable)
 {
     NodeType newOp = LastNodeType;
@@ -290,6 +297,12 @@ void Node::convertToDirectCall(FrozenValue* executable)
     m_opInfo = executable;
 }
 
+void Node::convertToCallWasm(FrozenValue* callee)
+{
+    m_op = CallWasm;
+    m_opInfo = callee;
+}
+
 void Node::convertToCallDOM(Graph& graph)
 {
     ASSERT(op() == Call);
@@ -341,6 +354,42 @@ void Node::convertToRegExpTestInline(FrozenValue* globalObject, FrozenValue* reg
     m_opInfo2 = regExp;
 }
 
+void Node::convertToGetByIdMaybeMegamorphic(Graph& graph, CacheableIdentifier identifier)
+{
+    ASSERT(op() == GetByVal || op() == GetByValMegamorphic);
+    bool isMegamorphic = op() == GetByValMegamorphic && canUseMegamorphicGetById(graph.m_vm, identifier.uid());
+    Edge base = graph.varArgChild(this, 0);
+    ASSERT(base.useKind() == ObjectUse);
+    for (unsigned i = 0; i < numChildren(); ++i) {
+        Edge& edge = graph.varArgChild(this, i);
+        edge = Edge();
+    }
+    setOpAndDefaultFlags(isMegamorphic ? GetByIdMegamorphic : GetById);
+    children.child1() = Edge(base.node(), CellUse);
+    children.child2() = Edge();
+    children.child3() = Edge();
+    m_opInfo = identifier;
+}
+
+void Node::convertToPutByIdMaybeMegamorphic(Graph& graph, CacheableIdentifier identifier)
+{
+    ASSERT(op() == PutByVal || op() == PutByValMegamorphic);
+    bool isMegamorphic = op() == PutByValMegamorphic && canUseMegamorphicPutById(graph.m_vm, identifier.uid());
+    Edge base = graph.child(this, 0);
+    Edge value = graph.child(this, 2);
+    ASSERT(base.useKind() == CellUse);
+    for (unsigned i = 0; i < numChildren(); ++i) {
+        Edge& edge = graph.varArgChild(this, i);
+        edge = Edge();
+    }
+
+    setOpAndDefaultFlags(isMegamorphic ? PutByIdMegamorphic : PutById);
+    children.child1() = Edge(base.node(), CellUse);
+    children.child2() = Edge(value.node());
+    children.child3() = Edge();
+    m_opInfo = identifier;
+}
+
 String Node::tryGetString(Graph& graph)
 {
     if (hasConstant())
