@@ -38,6 +38,7 @@
 #include "JSObjectInlines.h"
 #include "JSWebAssemblyInstance.h"
 #include "MathCommon.h"
+#include "NumberPrototype.h"
 #include "RegExpObject.h"
 #include "StringPrototypeInlines.h"
 #include "WasmCallingConvention.h"
@@ -393,10 +394,15 @@ private:
         }
 
         case MakeRope:
+        case MakeAtomString:
         case StrCat: {
             String leftString = m_node->child1()->tryGetString(m_graph);
             if (!leftString)
                 break;
+            if (!m_node->child2()) {
+                ASSERT(!m_node->child3());
+                break;
+            }
             String rightString = m_node->child2()->tryGetString(m_graph);
             if (!rightString)
                 break;
@@ -565,6 +571,15 @@ private:
 
             ASSERT(m_node->op() != RegExpMatchFast);
 
+            bool needLastIndexTypeCheck = false;
+            auto insertLastIndexTypeCheckIfNecessary = [&](NodeOrigin origin) {
+                if (needLastIndexTypeCheck) {
+                    ASSERT(m_node->op() != RegExpExecNonGlobalOrSticky);
+                    Node* lastIndex = m_insertionSet.insertNode(m_nodeIndex, SpecNone, GetRegExpObjectLastIndex, origin, Edge(regExpObjectNode, RegExpObjectUse));
+                    m_insertionSet.insertNode(m_nodeIndex, SpecNone, Check, origin, Edge(lastIndex, Int32Use));
+                }
+            };
+
             unsigned lastIndex = UINT_MAX;
             if (m_node->op() != RegExpExecNonGlobalOrSticky) {
                 // This will only work if we can prove what the value of lastIndex is. To do this
@@ -592,11 +607,30 @@ private:
                         break;
                 }
                 if (lastIndex == UINT_MAX) {
-                    if (verbose)
-                        dataLog("Giving up because the last index is not known.\n");
-                    break;
+                    // We cannot statically prove lastIndex. But still there is a chance.
+                    // If RegExp is not global and not sticky, then only thing we care is ToIntegerOrInfinity(regExp.lastIndex).
+                    // Thus, we can emit Int32Use check to protect further when conversion happens.
+                    if (regExp->globalOrSticky()) {
+                        dataLogLnIf(verbose, "Giving up because the last index is not known.");
+                        break;
+                    }
+
+                    if (m_graph.hasExitSite(m_node->origin.semantic, BadType)) {
+                        dataLogLnIf(verbose, "Giving up because the last index type check may fail.");
+                        break;
+                    }
+
+                    if (RegExpObject* object = regExpObjectNode->dynamicCastConstant<RegExpObject*>()) {
+                        if (!object->getLastIndex().isInt32()) {
+                            dataLogLnIf(verbose, "Giving up because the constant RegExpObject's lastIndex is not Int32 already.");
+                            break;
+                        }
+                    }
+
+                    needLastIndexTypeCheck = true;
                 }
             }
+
             if (!regExp->globalOrSticky())
                 lastIndex = 0;
 
@@ -686,8 +720,8 @@ private:
 
                 NodeOrigin origin = m_node->origin;
 
-                m_insertionSet.insertNode(
-                    m_nodeIndex, SpecNone, Check, origin, m_node->children.justChecks());
+                m_insertionSet.insertNode(m_nodeIndex, SpecNone, Check, origin, m_node->children.justChecks());
+                insertLastIndexTypeCheckIfNecessary(origin);
 
                 if (m_node->op() == RegExpExec || m_node->op() == RegExpExecNonGlobalOrSticky) {
                     if (result) {
@@ -833,7 +867,7 @@ private:
                 if (regExp->globalOrSticky())
                     return false;
 
-                if (regExp->unicode())
+                if (regExp->eitherUnicode())
                     return false;
 
                 auto jitCodeBlock = regExp->getRegExpJITCodeBlock();
@@ -856,8 +890,8 @@ private:
                     m_graph.m_parameterSlots = std::max(m_graph.m_parameterSlots, argumentCountForStackSize(alignedFrameSize));
 
                 NodeOrigin origin = m_node->origin;
-                m_insertionSet.insertNode(
-                    m_nodeIndex, SpecNone, Check, origin, m_node->children.justChecks());
+                m_insertionSet.insertNode(m_nodeIndex, SpecNone, Check, origin, m_node->children.justChecks());
+                insertLastIndexTypeCheckIfNecessary(origin);
                 m_node->convertToRegExpTestInline(m_graph.freeze(globalObject), m_graph.freeze(regExp));
                 m_changed = true;
                 return true;
@@ -871,9 +905,10 @@ private:
                     return false;
                 if (m_node->child3().useKind() != StringUse)
                     return false;
+
                 NodeOrigin origin = m_node->origin;
-                m_insertionSet.insertNode(
-                    m_nodeIndex, SpecNone, Check, origin, m_node->children.justChecks());
+                m_insertionSet.insertNode(m_nodeIndex, SpecNone, Check, origin, m_node->children.justChecks());
+                insertLastIndexTypeCheckIfNecessary(origin);
                 m_node->convertToRegExpExecNonGlobalOrStickyWithoutChecks(m_graph.freeze(regExp));
                 m_changed = true;
                 return true;
@@ -1109,6 +1144,46 @@ private:
             break;
         }
 
+        case GetByVal:
+        case GetByValMegamorphic: {
+            Edge& baseEdge = m_graph.child(m_node, 0);
+            Edge& keyEdge = m_graph.child(m_node, 1);
+            if (baseEdge.useKind() == ObjectUse && m_node->arrayMode().type() == Array::Generic) {
+                if (keyEdge->op() == MakeRope) {
+                    keyEdge->setOp(MakeAtomString);
+                    m_changed = true;
+                }
+            }
+            break;
+        }
+
+        case PutByVal:
+        case PutByValDirect:
+        case PutByValAlias:
+        case PutByValMegamorphic: {
+            Edge& baseEdge = m_graph.child(m_node, 0);
+            Edge& keyEdge = m_graph.child(m_node, 1);
+            if ((baseEdge.useKind() == CellUse || baseEdge.useKind() == KnownCellUse) && m_node->arrayMode().type() == Array::Generic) {
+                if (keyEdge->op() == MakeRope) {
+                    keyEdge->setOp(MakeAtomString);
+                    m_changed = true;
+                }
+            }
+            break;
+        }
+
+        case InByVal: {
+            Edge& baseEdge = m_graph.child(m_node, 0);
+            Edge& keyEdge = m_graph.child(m_node, 1);
+            if (baseEdge.useKind() == CellUse) {
+                if (keyEdge->op() == MakeRope) {
+                    keyEdge->setOp(MakeAtomString);
+                    m_changed = true;
+                }
+            }
+            break;
+        }
+
         case Call:
         case Construct:
         case TailCallInlinedCaller:
@@ -1191,6 +1266,7 @@ private:
                             success = false;
                         break;
                     }
+                    case Wasm::TypeKind::V128:
                     default: {
                         success = false;
                         break;
@@ -1212,6 +1288,7 @@ private:
                     case Wasm::TypeKind::F64: {
                         break;
                     }
+                    case Wasm::TypeKind::V128:
                     default: {
                         success = false;
                         break;
@@ -1274,6 +1351,7 @@ private:
                         m_graph.varArgChild(m_node, 2 + index) = Edge(result, DoubleRepUse);
                         break;
                     }
+                    case Wasm::TypeKind::V128:
                     default:
                         RELEASE_ASSERT_NOT_REACHED();
                     }
@@ -1299,6 +1377,7 @@ private:
                     case Wasm::TypeKind::F64: {
                         break;
                     }
+                    case Wasm::TypeKind::V128:
                     default: {
                         break;
                     }
@@ -1309,7 +1388,46 @@ private:
                 break;
             }
 #endif
-            
+
+            // We gave up inlining a wrapped function, but still, we can inline bound function's wrapper by extracting it.
+            // This also wipes bound-function thunk call which is suboptimal compared to directly calling a wrapped function here.
+            if (executable->intrinsic() == BoundFunctionCallIntrinsic && function && (m_node->op() == Call || m_node->op() == TailCall || m_node->op() == TailCallInlinedCaller)) {
+                JSBoundFunction* boundFunction = jsCast<JSBoundFunction*>(function);
+                if (JSFunction* targetFunction = jsDynamicCast<JSFunction*>(boundFunction->targetFunction())) {
+                    auto* targetExecutable = targetFunction->executable();
+                    if ((boundFunction->boundArgsLength() + m_node->numChildren()) <= Options::maximumDirectCallStackSize()) {
+                        if (FunctionExecutable* functionExecutable = jsDynamicCast<FunctionExecutable*>(targetExecutable)) {
+                            // We need to update m_parameterSlots before we get to the backend, but we don't
+                            // want to do too much of this.
+                            unsigned numAllocatedArgs = static_cast<unsigned>(functionExecutable->parameterCount()) + 1;
+                            if (numAllocatedArgs <= Options::maximumDirectCallStackSize())
+                                m_graph.m_parameterSlots = std::max(m_graph.m_parameterSlots, Graph::parameterSlotsForArgCount(numAllocatedArgs));
+                        }
+
+                        unsigned firstChild = m_graph.m_varArgChildren.size();
+                        m_graph.m_varArgChildren.append(m_insertionSet.insertConstant(m_nodeIndex, m_node->origin, targetFunction)); // |callee|.
+                        m_graph.m_varArgChildren.append(m_insertionSet.insertConstant(m_nodeIndex, m_node->origin, boundFunction->boundThis())); // |this|
+
+                        boundFunction->forEachBoundArg([&](JSValue argument) {
+                            m_graph.m_varArgChildren.append(m_insertionSet.insertConstant(m_nodeIndex, m_node->origin, argument));
+                            return IterationStatus::Continue;
+                        });
+
+                        // First one is |callee|, second one is |this|.
+                        for (unsigned index = 2; index < m_node->numChildren(); ++index)
+                            m_graph.m_varArgChildren.append(m_graph.child(m_node, index));
+
+                        m_node->children = AdjacencyList(AdjacencyList::Variable, firstChild, m_graph.m_varArgChildren.size() - firstChild);
+                        m_graph.m_parameterSlots = std::max(m_graph.m_parameterSlots, Graph::parameterSlotsForArgCount(m_node->numChildren() - 1));
+
+                        m_graph.m_plan.recordedStatuses().addCallLinkStatus(m_node->origin.semantic, CallLinkStatus(CallVariant(targetExecutable)));
+                        m_node->convertToDirectCall(m_graph.freeze(targetExecutable));
+                        m_changed = true;
+                        break;
+                    }
+                }
+            }
+
             if (FunctionExecutable* functionExecutable = jsDynamicCast<FunctionExecutable*>(executable)) {
                 if (m_node->op() == Construct && functionExecutable->constructAbility() == ConstructAbility::CannotConstruct)
                     break;
