@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2013-2020 Apple Inc. All rights reserved.
+ * Copyright (C) 2013-2022 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -28,6 +28,7 @@
 
 #if ENABLE(FTL_JIT)
 
+#include "AssemblyHelpersSpoolers.h"
 #include "BytecodeStructs.h"
 #include "CheckpointOSRExitSideState.h"
 #include "DFGOSRExitCompilerCommon.h"
@@ -43,6 +44,8 @@
 #include "OperandsInlines.h"
 #include "ProbeContext.h"
 
+#include <wtf/Scope.h>
+
 namespace JSC { namespace FTL {
 
 using namespace DFG;
@@ -100,7 +103,7 @@ static void reboxAccordingToFormat(
 
 static void compileRecovery(
     CCallHelpers& jit, const ExitValue& value,
-    Vector<B3::ValueRep>& valueReps,
+    const FixedVector<B3::ValueRep>& valueReps,
     char* registerScratch,
     const HashMap<ExitTimeObjectMaterialization*, EncodedJSValue*>& materializationToPointer)
 {
@@ -145,6 +148,15 @@ static void compileStub(VM& vm, unsigned exitID, JITCode* jitCode, OSRExit& exit
 
     CCallHelpers jit(codeBlock);
 
+    if (UNLIKELY(Options::printEachOSRExit())) {
+        SpeculationFailureDebugInfo* debugInfo = new SpeculationFailureDebugInfo;
+        debugInfo->codeBlock = jit.codeBlock();
+        debugInfo->kind = exit.m_kind;
+        debugInfo->bytecodeIndex = exit.m_codeOrigin.bytecodeIndex();
+
+        jit.debugCall(vm, operationDebugPrintSpeculationFailure, debugInfo);
+    }
+
     // The first thing we need to do is restablish our frame in the case of an exception.
     if (exit.isGenericUnwindHandler()) {
         RELEASE_ASSERT(vm.callFrameForCatch); // The first time we hit this exit, like at all other times, this field should be non-null.
@@ -179,7 +191,7 @@ static void compileStub(VM& vm, unsigned exitID, JITCode* jitCode, OSRExit& exit
         sizeof(EncodedJSValue) * (
             exit.m_descriptor->m_values.size() + numMaterializations + maxMaterializationNumArguments) +
         requiredScratchMemorySizeInBytes() +
-        codeBlock->calleeSaveRegisters()->size() * sizeof(uint64_t));
+        codeBlock->jitCode()->calleeSaveRegisters()->sizeOfAreaInBytes());
     EncodedJSValue* scratch = scratchBuffer ? static_cast<EncodedJSValue*>(scratchBuffer->dataBuffer()) : nullptr;
     EncodedJSValue* materializationPointers = scratch + exit.m_descriptor->m_values.size();
     EncodedJSValue* materializationArguments = materializationPointers + numMaterializations;
@@ -216,7 +228,7 @@ static void compileStub(VM& vm, unsigned exitID, JITCode* jitCode, OSRExit& exit
             // to set it here because compileFTLOSRExit() is only called on the first time
             // we exit from this site, but all subsequent exits will take this compiled
             // ramp without calling compileFTLOSRExit() first.
-            jit.store32(CCallHelpers::TrustedImm32(DoesGCCheck::encode(true, DoesGCCheck::Special::FTLOSRExit)), vm.heap.addressOfDoesGC());
+            jit.store64(CCallHelpers::TrustedImm64(DoesGCCheck::encode(true, DoesGCCheck::Special::FTLOSRExit)), vm.addressOfDoesGC());
         }
     }
 
@@ -239,8 +251,7 @@ static void compileStub(VM& vm, unsigned exitID, JITCode* jitCode, OSRExit& exit
     
     // Get the call frame and tag thingies.
     // Restore the exiting function's callFrame value into a regT4
-    jit.move(MacroAssembler::TrustedImm64(JSValue::NumberTag), GPRInfo::numberTagRegister);
-    jit.move(MacroAssembler::TrustedImm64(JSValue::NotCellMask), GPRInfo::notCellMaskRegister);
+    jit.emitMaterializeTagCheckRegisters();
     
     // Do some value profiling.
     if (exit.m_descriptor->m_profileDataFormat != DataFormatNone) {
@@ -251,8 +262,8 @@ static void compileStub(VM& vm, unsigned exitID, JITCode* jitCode, OSRExit& exit
         if (exit.m_kind == BadCache || exit.m_kind == BadIndexingType) {
             CodeOrigin codeOrigin = exit.m_codeOriginForExitProfile;
             CodeBlock* codeBlock = jit.baselineCodeBlockFor(codeOrigin);
-            if (ArrayProfile* arrayProfile = codeBlock->getArrayProfile(codeOrigin.bytecodeIndex())) {
-                const Instruction* instruction = codeBlock->instructions().at(codeOrigin.bytecodeIndex()).ptr();
+            if (ArrayProfile* arrayProfile = codeBlock->getArrayProfile(ConcurrentJSLocker(codeBlock->m_lock), codeOrigin.bytecodeIndex())) {
+                const auto* instruction = codeBlock->instructions().at(codeOrigin.bytecodeIndex()).ptr();
                 CCallHelpers::Jump skipProfile;
                 if (instruction->is<OpGetById>()) {
                     auto& metadata = instruction->as<OpGetById>().metadata(codeBlock);
@@ -283,7 +294,7 @@ static void compileStub(VM& vm, unsigned exitID, JITCode* jitCode, OSRExit& exit
         }
 
         if (exit.m_descriptor->m_valueProfile)
-            exit.m_descriptor->m_valueProfile.emitReportValue(jit, JSValueRegs(GPRInfo::regT0), GPRInfo::regT1);
+            exit.m_descriptor->m_valueProfile.emitReportValue(jit, jit.codeBlock(), JSValueRegs(GPRInfo::regT0), GPRInfo::regT1);
     }
 
     // Materialize all objects. Don't materialize an object until all
@@ -375,9 +386,65 @@ static void compileStub(VM& vm, unsigned exitID, JITCode* jitCode, OSRExit& exit
     // Save all state from wherever the exit data tells us it was, into the appropriate place in
     // the scratch buffer. This also does the reboxing.
     
-    for (unsigned index = exit.m_descriptor->m_values.size(); index--;) {
-        recoverValue(exit.m_descriptor->m_values[index]);
-        jit.store64(GPRInfo::regT0, scratch + index);
+    {
+        std::optional<GPRReg> undefinedGPR;
+        jit.move(CCallHelpers::TrustedImmPtr(scratch), GPRInfo::regT3);
+        CCallHelpers::CopySpooler spooler(jit, CCallHelpers::framePointerRegister, GPRInfo::regT3, GPRInfo::regT0, GPRInfo::regT1);
+        for (unsigned index = exit.m_descriptor->m_values.size(); index--;) {
+            auto& value = exit.m_descriptor->m_values[index];
+            if (value.dataFormat() == DataFormatJS) {
+                switch (value.kind()) {
+                case ExitValueDead:
+                    if (UNLIKELY(!undefinedGPR)) {
+                        jit.move(CCallHelpers::TrustedImm64(JSValue::encode(jsUndefined())), GPRInfo::regT4);
+                        undefinedGPR = GPRInfo::regT4;
+                    }
+                    spooler.copyGPR(undefinedGPR.value());
+                    spooler.storeGPR(index * sizeof(EncodedJSValue));
+                    break;
+
+                case ExitValueConstant: {
+                    EncodedJSValue currentConstant = JSValue::encode(value.constant());
+                    if (currentConstant == encodedJSUndefined()) {
+                        if (UNLIKELY(!undefinedGPR)) {
+                            jit.move(CCallHelpers::TrustedImm64(JSValue::encode(jsUndefined())), GPRInfo::regT4);
+                            undefinedGPR = GPRInfo::regT4;
+                        }
+                        spooler.copyGPR(undefinedGPR.value());
+                    } else
+                        spooler.moveConstant(currentConstant);
+                    spooler.storeGPR(index * sizeof(EncodedJSValue));
+                    break;
+                }
+
+                case ExitValueArgument:
+                    Location::forValueRep(exit.m_valueReps[value.exitArgument().argument()]).restoreInto(jit, registerScratch, GPRInfo::regT0);
+                    jit.store64(GPRInfo::regT0, CCallHelpers::Address(GPRInfo::regT3, index * sizeof(EncodedJSValue)));
+                    break;
+
+                case ExitValueInJSStack:
+                case ExitValueInJSStackAsInt32:
+                case ExitValueInJSStackAsInt52:
+                case ExitValueInJSStackAsDouble:
+                    spooler.loadGPR(value.virtualRegister().offset() * sizeof(EncodedJSValue));
+                    spooler.storeGPR(index * sizeof(EncodedJSValue));
+                    break;
+
+                case ExitValueMaterializeNewObject:
+                    jit.loadPtr(materializationToPointer.get(value.objectMaterialization()), GPRInfo::regT0);
+                    jit.store64(GPRInfo::regT0, CCallHelpers::Address(GPRInfo::regT3, index * sizeof(EncodedJSValue)));
+                    break;
+
+                default:
+                    RELEASE_ASSERT_NOT_REACHED();
+                    break;
+                }
+            } else {
+                recoverValue(value);
+                jit.store64(GPRInfo::regT0, CCallHelpers::Address(GPRInfo::regT3, index * sizeof(EncodedJSValue)));
+            }
+        }
+        spooler.finalizeGPR();
     }
     
     // Henceforth we make it look like the exiting function was called through a register
@@ -388,19 +455,25 @@ static void compileStub(VM& vm, unsigned exitID, JITCode* jitCode, OSRExit& exit
     
     // Before we start messing with the frame, we need to set aside any registers that the
     // FTL code was preserving.
-    for (unsigned i = codeBlock->calleeSaveRegisters()->size(); i--;) {
-        RegisterAtOffset entry = codeBlock->calleeSaveRegisters()->at(i);
-        jit.load64(
-            MacroAssembler::Address(MacroAssembler::framePointerRegister, entry.offset()),
-            GPRInfo::regT0);
-        jit.store64(GPRInfo::regT0, unwindScratch + i);
+    {
+        constexpr GPRReg srcBufferGPR = GPRInfo::regT2;
+        constexpr GPRReg destBufferGPR = GPRInfo::regT3;
+        jit.move(CCallHelpers::framePointerRegister, srcBufferGPR);
+        jit.move(CCallHelpers::TrustedImmPtr(unwindScratch), destBufferGPR);
+        CCallHelpers::CopySpooler spooler(CCallHelpers::CopySpooler::BufferRegs::AllowModification, jit, srcBufferGPR, destBufferGPR, GPRInfo::regT0, GPRInfo::regT1);
+        for (unsigned i = codeBlock->jitCode()->calleeSaveRegisters()->registerCount(); i--;) {
+            RegisterAtOffset entry = codeBlock->jitCode()->calleeSaveRegisters()->at(i);
+            spooler.loadGPR(entry.offset());
+            spooler.storeGPR(i * sizeof(uint64_t));
+        }
+        spooler.finalizeGPR();
     }
     
     CodeBlock* baselineCodeBlock = jit.baselineCodeBlockFor(exit.m_codeOrigin);
 
     // First set up SP so that our data doesn't get clobbered by signals.
     unsigned conservativeStackDelta =
-        (exit.m_descriptor->m_values.numberOfLocals() + baselineCodeBlock->calleeSaveSpaceAsVirtualRegisters()) * sizeof(Register) +
+        (exit.m_descriptor->m_values.numberOfLocals() + CodeBlock::calleeSaveSpaceAsVirtualRegisters(*baselineCodeBlock->jitCode()->calleeSaveRegisters())) * sizeof(Register) +
         maxFrameExtentForSlowPathCall;
     conservativeStackDelta = WTF::roundUpToMultipleOf(
         stackAlignmentBytes(), conservativeStackDelta);
@@ -409,127 +482,153 @@ static void compileStub(VM& vm, unsigned exitID, JITCode* jitCode, OSRExit& exit
         MacroAssembler::framePointerRegister, MacroAssembler::stackPointerRegister);
     jit.checkStackPointerAlignment();
 
-    RegisterSet allFTLCalleeSaves = RegisterSet::ftlCalleeSaveRegisters();
-    const RegisterAtOffsetList* baselineCalleeSaves = baselineCodeBlock->calleeSaveRegisters();
-    RegisterAtOffsetList* vmCalleeSaves = RegisterSet::vmCalleeSaveRegisterOffsets();
-    RegisterSet vmCalleeSavesToSkip = RegisterSet::stackRegisters();
-    if (exit.isExceptionHandler()) {
-        jit.loadPtr(&vm.topEntryFrame, GPRInfo::regT1);
-        jit.addPtr(CCallHelpers::TrustedImm32(EntryFrame::calleeSaveRegistersBufferOffset()), GPRInfo::regT1);
-    }
-
-    for (Reg reg = Reg::first(); reg <= Reg::last(); reg = reg.next()) {
-        if (!allFTLCalleeSaves.get(reg)) {
-            if (exit.isExceptionHandler())
-                RELEASE_ASSERT(!vmCalleeSaves->find(reg));
-            continue;
-        }
-        unsigned unwindIndex = codeBlock->calleeSaveRegisters()->indexOf(reg);
-        const RegisterAtOffset* baselineRegisterOffset = baselineCalleeSaves->find(reg);
-        RegisterAtOffset* vmCalleeSave = nullptr; 
-        if (exit.isExceptionHandler())
-            vmCalleeSave = vmCalleeSaves->find(reg);
-
-        if (reg.isGPR()) {
-            GPRReg regToLoad = baselineRegisterOffset ? GPRInfo::regT0 : reg.gpr();
-            RELEASE_ASSERT(regToLoad != GPRInfo::regT1);
-
-            if (unwindIndex == UINT_MAX) {
-                // The FTL compilation didn't preserve this register. This means that it also
-                // didn't use the register. So its value at the beginning of OSR exit should be
-                // preserved by the thunk. Luckily, we saved all registers into the register
-                // scratch buffer, so we can restore them from there.
-                jit.load64(registerScratch + offsetOfReg(reg), regToLoad);
-            } else {
-                // The FTL compilation preserved the register. Its new value is therefore
-                // irrelevant, but we can get the value that was preserved by using the unwind
-                // data. We've already copied all unwind-able preserved registers into the unwind
-                // scratch buffer, so we can get it from there.
-                jit.load64(unwindScratch + unwindIndex, regToLoad);
+    {
+        auto allFTLCalleeSaves = RegisterSetBuilder::ftlCalleeSaveRegisters();
+        const RegisterAtOffsetList* baselineCalleeSaves = baselineCodeBlock->jitCode()->calleeSaveRegisters();
+        auto iterateCalleeSavesImpl = [&](auto check, auto func) {
+            for (Reg reg = Reg::first(); reg <= Reg::last(); reg = reg.next()) {
+                if (!allFTLCalleeSaves.contains(reg, IgnoreVectors))
+                    continue;
+                if (!check(reg))
+                    continue;
+                unsigned unwindIndex = codeBlock->jitCode()->calleeSaveRegisters()->indexOf(reg);
+                const RegisterAtOffset* baselineRegisterOffset = baselineCalleeSaves->find(reg);
+                func(reg, unwindIndex, baselineRegisterOffset);
             }
+        };
 
-            if (baselineRegisterOffset)
-                jit.store64(regToLoad, MacroAssembler::Address(MacroAssembler::framePointerRegister, baselineRegisterOffset->offset()));
-            if (vmCalleeSave && !vmCalleeSavesToSkip.get(vmCalleeSave->reg()))
-                jit.store64(regToLoad, MacroAssembler::Address(GPRInfo::regT1, vmCalleeSave->offset()));
-        } else {
-            FPRReg fpRegToLoad = baselineRegisterOffset ? FPRInfo::fpRegT0 : reg.fpr();
+        auto iterateGPRCalleeSaves = [&](auto func) {
+            iterateCalleeSavesImpl([](Reg reg) { return reg.isGPR(); }, func);
+        };
 
-            if (unwindIndex == UINT_MAX)
-                jit.loadDouble(MacroAssembler::TrustedImmPtr(registerScratch + offsetOfReg(reg)), fpRegToLoad);
-            else
-                jit.loadDouble(MacroAssembler::TrustedImmPtr(unwindScratch + unwindIndex), fpRegToLoad);
+        auto iterateFPRCalleeSaves = [&](auto func) {
+            iterateCalleeSavesImpl([](Reg reg) { return reg.isFPR(); }, func);
+        };
 
-            if (baselineRegisterOffset)
-                jit.storeDouble(fpRegToLoad, MacroAssembler::Address(MacroAssembler::framePointerRegister, baselineRegisterOffset->offset()));
-            if (vmCalleeSave && !vmCalleeSavesToSkip.get(vmCalleeSave->reg()))
-                jit.storeDouble(fpRegToLoad, MacroAssembler::Address(GPRInfo::regT1, vmCalleeSave->offset()));
+        {
+            // unwindIndex == UINT_MAX indicates that the FTL compilation didn't preserve these registers.
+            // This means that it also didn't use them. Their values at the beginning of OSR exit should
+            // be the ones to retain. We saved all registers into the register scratch buffer at the beginning
+            // of the thunk. So we can restore them from there.
+            ASSERT(!allFTLCalleeSaves.contains(GPRInfo::regT3, IgnoreVectors));
+            ASSERT(!allFTLCalleeSaves.contains(GPRInfo::regT0, IgnoreVectors));
+            ASSERT(!allFTLCalleeSaves.contains(GPRInfo::regT1, IgnoreVectors));
+            ASSERT(!allFTLCalleeSaves.contains(FPRInfo::fpRegT0, IgnoreVectors));
+            ASSERT(!allFTLCalleeSaves.contains(FPRInfo::fpRegT1, IgnoreVectors));
+            jit.move(CCallHelpers::TrustedImmPtr(registerScratch), GPRInfo::regT3);
+            {
+                // Load from registerScratch buffer to callee-save registers.
+                CCallHelpers::LoadRegSpooler spooler(jit, GPRInfo::regT3);
+                iterateGPRCalleeSaves([&](Reg reg, unsigned unwindIndex, const RegisterAtOffset* baselineRegisterOffset) {
+                    if (unwindIndex == UINT_MAX && !baselineRegisterOffset)
+                        spooler.loadGPR({ reg, static_cast<ptrdiff_t>(offsetOfReg(reg)), conservativeWidthWithoutVectors(reg) });
+                });
+                spooler.finalizeGPR();
+                iterateFPRCalleeSaves([&](Reg reg, unsigned unwindIndex, const RegisterAtOffset* baselineRegisterOffset) {
+                    if (unwindIndex == UINT_MAX && !baselineRegisterOffset)
+                        spooler.loadFPR({ reg, static_cast<ptrdiff_t>(offsetOfReg(reg)), conservativeWidthWithoutVectors(reg) });
+                });
+                spooler.finalizeFPR();
+            }
+            {
+                // Copy from registerScratch buffer to call frame.
+                CCallHelpers::CopySpooler spooler(jit, GPRInfo::regT3, CCallHelpers::framePointerRegister, GPRInfo::regT0, GPRInfo::regT1, FPRInfo::fpRegT0, FPRInfo::fpRegT1);
+                iterateGPRCalleeSaves([&](Reg reg, unsigned unwindIndex, const RegisterAtOffset* baselineRegisterOffset) {
+                    if (unwindIndex == UINT_MAX && baselineRegisterOffset) {
+                        spooler.loadGPR(offsetOfReg(reg));
+                        spooler.storeGPR(baselineRegisterOffset->offset());
+                    }
+                });
+                spooler.finalizeGPR();
+                iterateFPRCalleeSaves([&](Reg reg, unsigned unwindIndex, const RegisterAtOffset* baselineRegisterOffset) {
+                    if (unwindIndex == UINT_MAX && baselineRegisterOffset) {
+                        spooler.loadFPR(offsetOfReg(reg));
+                        spooler.storeFPR(baselineRegisterOffset->offset());
+                    }
+                });
+                spooler.finalizeFPR();
+            }
+        }
+        {
+            // The FTL compilation preserved these registers. Their new values are therefore irrelevant,
+            // but we can get their values that were preserved by using the unwind data. We've already
+            // copied all unwind-able preserved registers into the unwind scratch buffer, so we can get
+            // the values to restore from there.
+            ASSERT((bitwise_cast<uintptr_t>(unwindScratch) - bitwise_cast<uintptr_t>(registerScratch)) == requiredScratchMemorySizeInBytes());
+            jit.addPtr(CCallHelpers::TrustedImm32(requiredScratchMemorySizeInBytes()), GPRInfo::regT3); // Change registerScratch to unwindScratch.
+            {
+                // Load from unwindScratch buffer to callee-save registers.
+                CCallHelpers::LoadRegSpooler spooler(jit, GPRInfo::regT3);
+                iterateGPRCalleeSaves([&](Reg reg, unsigned unwindIndex, const RegisterAtOffset* baselineRegisterOffset) {
+                    if (unwindIndex != UINT_MAX && !baselineRegisterOffset)
+                        spooler.loadGPR({ reg, static_cast<ptrdiff_t>(unwindIndex * sizeof(uint64_t)), conservativeWidthWithoutVectors(reg) });
+                });
+                spooler.finalizeGPR();
+                iterateFPRCalleeSaves([&](Reg reg, unsigned unwindIndex, const RegisterAtOffset* baselineRegisterOffset) {
+                    if (unwindIndex != UINT_MAX && !baselineRegisterOffset)
+                        spooler.loadFPR({ reg, static_cast<ptrdiff_t>(unwindIndex * sizeof(uint64_t)), conservativeWidthWithoutVectors(reg) });
+                });
+                spooler.finalizeFPR();
+            }
+            {
+                // Copy from unwindScratch buffer to call frame.
+                CCallHelpers::CopySpooler spooler(jit, GPRInfo::regT3, CCallHelpers::framePointerRegister, GPRInfo::regT0, GPRInfo::regT1, FPRInfo::fpRegT0, FPRInfo::fpRegT1);
+                iterateGPRCalleeSaves([&](Reg, unsigned unwindIndex, const RegisterAtOffset* baselineRegisterOffset) {
+                    if (unwindIndex != UINT_MAX && baselineRegisterOffset) {
+                        spooler.loadGPR(static_cast<ptrdiff_t>(unwindIndex * sizeof(uint64_t)));
+                        spooler.storeGPR(baselineRegisterOffset->offset());
+                    }
+                });
+                spooler.finalizeGPR();
+                iterateFPRCalleeSaves([&](Reg, unsigned unwindIndex, const RegisterAtOffset* baselineRegisterOffset) {
+                    if (unwindIndex != UINT_MAX && baselineRegisterOffset) {
+                        spooler.loadFPR(static_cast<ptrdiff_t>(unwindIndex * sizeof(uint64_t)));
+                        spooler.storeFPR(baselineRegisterOffset->offset());
+                    }
+                });
+                spooler.finalizeFPR();
+            }
         }
     }
 
-    if (exit.isExceptionHandler()) {
-        RegisterAtOffset* vmCalleeSave = vmCalleeSaves->find(GPRInfo::numberTagRegister);
-        jit.store64(GPRInfo::numberTagRegister, MacroAssembler::Address(GPRInfo::regT1, vmCalleeSave->offset()));
-
-        vmCalleeSave = vmCalleeSaves->find(GPRInfo::notCellMaskRegister);
-        jit.store64(GPRInfo::notCellMaskRegister, MacroAssembler::Address(GPRInfo::regT1, vmCalleeSave->offset()));
-    }
-
-    size_t baselineVirtualRegistersForCalleeSaves = baselineCodeBlock->calleeSaveSpaceAsVirtualRegisters();
+    size_t baselineVirtualRegistersForCalleeSaves = CodeBlock::calleeSaveSpaceAsVirtualRegisters(*baselineCodeBlock->jitCode()->calleeSaveRegisters());
 
     if (exit.m_codeOrigin.inlineStackContainsActiveCheckpoint()) {
-        JSValue* tmpScratch = reinterpret_cast<JSValue*>(scratch + exit.m_descriptor->m_values.tmpIndex(0));
-        VM* vmPtr = &vm;
-        jit.probe([=] (Probe::Context& context) {
-            auto addSideState = [&] (CallFrame* frame, BytecodeIndex index, size_t tmpOffset) {
-                std::unique_ptr<CheckpointOSRExitSideState> sideState = WTF::makeUnique<CheckpointOSRExitSideState>();
-
-                sideState->bytecodeIndex = index;
-                for (size_t i = 0; i < maxNumCheckpointTmps; ++i)
-                    sideState->tmps[i] = tmpScratch[i + tmpOffset];
-
-                vmPtr->addCheckpointOSRSideState(frame, WTFMove(sideState));
-            };
-
-            const CodeOrigin* codeOrigin;
-            CallFrame* callFrame = context.gpr<CallFrame*>(GPRInfo::callFrameRegister);
-            for (codeOrigin = &exit.m_codeOrigin; codeOrigin && codeOrigin->inlineCallFrame(); codeOrigin = codeOrigin->inlineCallFrame()->getCallerSkippingTailCalls()) {
-                BytecodeIndex callBytecodeIndex = codeOrigin->bytecodeIndex();
-                if (!callBytecodeIndex.checkpoint())
-                    continue;
-
-                auto* inlineCallFrame = codeOrigin->inlineCallFrame();
-                addSideState(reinterpret_cast<CallFrame*>(reinterpret_cast<char*>(callFrame) + inlineCallFrame->returnPCOffset() - sizeof(CPURegister)), callBytecodeIndex, inlineCallFrame->tmpOffset);
-            }
-
-            if (!codeOrigin)
-                return;
-
-            if (BytecodeIndex bytecodeIndex = codeOrigin->bytecodeIndex(); bytecodeIndex.checkpoint())
-                addSideState(callFrame, bytecodeIndex, 0);
-        });
+        EncodedJSValue* tmpScratch = scratch + exit.m_descriptor->m_values.tmpIndex(0);
+        jit.setupArguments<decltype(operationMaterializeOSRExitSideState)>(CCallHelpers::TrustedImmPtr(&vm), CCallHelpers::TrustedImmPtr(&exit), CCallHelpers::TrustedImmPtr(tmpScratch));
+        jit.prepareCallOperation(vm);
+        jit.move(AssemblyHelpers::TrustedImmPtr(tagCFunction<OperationPtrTag>(operationMaterializeOSRExitSideState)), GPRInfo::nonArgGPR0);
+        jit.call(GPRInfo::nonArgGPR0, OperationPtrTag);
     }
 
     // Now get state out of the scratch buffer and place it back into the stack. The values are
     // already reboxed so we just move them.
-    for (unsigned index = exit.m_descriptor->m_values.size(); index--;) {
-        Operand operand = exit.m_descriptor->m_values.operandForIndex(index);
+    {
+        constexpr GPRReg srcBufferGPR = GPRInfo::regT2;
+        constexpr GPRReg destBufferGPR = GPRInfo::regT3;
+        jit.move(CCallHelpers::TrustedImmPtr(scratch), srcBufferGPR);
+        jit.move(GPRInfo::callFrameRegister, destBufferGPR);
+        CCallHelpers::CopySpooler spooler(CCallHelpers::CopySpooler::BufferRegs::AllowModification, jit, srcBufferGPR, destBufferGPR, GPRInfo::regT0, GPRInfo::regT1);
+        for (unsigned index = exit.m_descriptor->m_values.size(); index--;) {
+            Operand operand = exit.m_descriptor->m_values.operandForIndex(index);
 
-        if (operand.isTmp())
-            continue;
+            if (operand.isTmp())
+                continue;
 
-        if (operand.isLocal() && operand.toLocal() < static_cast<int>(baselineVirtualRegistersForCalleeSaves))
-            continue;
+            if (operand.isLocal() && operand.toLocal() < static_cast<int>(baselineVirtualRegistersForCalleeSaves))
+                continue;
 
-        jit.load64(scratch + index, GPRInfo::regT0);
-        jit.store64(GPRInfo::regT0, AssemblyHelpers::addressFor(operand.virtualRegister()));
+            spooler.loadGPR(index * sizeof(EncodedJSValue));
+            spooler.storeGPR(operand.virtualRegister().offset() * sizeof(EncodedJSValue));
+        }
+        spooler.finalizeGPR();
     }
     
     handleExitCounts(vm, jit, exit);
     reifyInlinedCallFrames(jit, exit);
     adjustAndJumpToTarget(vm, jit, exit);
     
-    LinkBuffer patchBuffer(jit, codeBlock);
+    LinkBuffer patchBuffer(jit, codeBlock, LinkBuffer::Profile::FTLOSRExit);
     exit.m_code = FINALIZE_CODE_IF(
         shouldDumpDisassembly() || Options::verboseOSR() || Options::verboseFTLOSRExit(),
         patchBuffer, OSRExitPtrTag,
@@ -540,17 +639,18 @@ static void compileStub(VM& vm, unsigned exitID, JITCode* jitCode, OSRExit& exit
         );
 }
 
-extern "C" JIT_OPERATION void* operationCompileFTLOSRExit(CallFrame* callFrame, unsigned exitID)
+JSC_DEFINE_JIT_OPERATION(operationCompileFTLOSRExit, void*, (CallFrame* callFrame, unsigned exitID))
 {
     if (shouldDumpDisassembly() || Options::verboseOSR() || Options::verboseFTLOSRExit())
         dataLog("Compiling OSR exit with exitID = ", exitID, "\n");
 
     VM& vm = callFrame->deprecatedVM();
+    // Don't need an ActiveScratchBufferScope here because we DeferGCForAWhile below.
 
     if constexpr (validateDFGDoesGC) {
         // We're about to exit optimized code. So, there's no longer any optimized
         // code running that expects no GC.
-        vm.heap.setDoesGCExpectation(true, DoesGCCheck::Special::FTLOSRExit);
+        vm.setDoesGCExpectation(true, DoesGCCheck::Special::FTLOSRExit);
     }
 
     if (vm.callFrameForCatch)
@@ -563,10 +663,10 @@ extern "C" JIT_OPERATION void* operationCompileFTLOSRExit(CallFrame* callFrame,
     
     // It's sort of preferable that we don't GC while in here. Anyways, doing so wouldn't
     // really be profitable.
-    DeferGCForAWhile deferGC(vm.heap);
+    DeferGCForAWhile deferGC(vm);
 
     JITCode* jitCode = codeBlock->jitCode()->ftl();
-    OSRExit& exit = jitCode->osrExit[exitID];
+    OSRExit& exit = jitCode->m_osrExit[exitID];
     
     if (shouldDumpDisassembly() || Options::verboseOSR() || Options::verboseFTLOSRExit()) {
         dataLog("    Owning block: ", pointerDump(codeBlock), "\n");
@@ -590,7 +690,7 @@ extern "C" JIT_OPERATION void* operationCompileFTLOSRExit(CallFrame* callFrame,
     MacroAssembler::repatchJump(
         exit.codeLocationForRepatch(codeBlock), CodeLocationLabel<OSRExitPtrTag>(exit.m_code.code()));
     
-    return exit.m_code.code().executableAddress();
+    return exit.m_code.code().taggedPtr();
 }
 
 } } // namespace JSC::FTL
