@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012, 2016 Apple Inc. All rights reserved.
+ * Copyright (C) 2012-2021 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -26,21 +26,30 @@
 #include "config.h"
 #include "IncrementalSweeper.h"
 
-#include "Heap.h"
-#include "JSObject.h"
-#include "JSString.h"
+#include "DeferGCInlines.h"
+#include "HeapInlines.h"
 #include "MarkedBlock.h"
-#include "JSCInlines.h"
+#include "VM.h"
+
+#if !USE(SYSTEM_MALLOC)
+#include <bmalloc/BPlatform.h>
+#if BUSE(LIBPAS)
+#include <bmalloc/pas_debug_spectrum.h>
+#include <bmalloc/pas_fd_stream.h>
+#include <bmalloc/pas_heap_lock.h>
+#include <bmalloc/pas_thread_local_cache.h>
+#endif
+#endif
 
 namespace JSC {
 
-static const Seconds sweepTimeSlice = 10_ms; // seconds
-static const double sweepTimeTotal = .10;
-static const double sweepTimeMultiplier = 1.0 / sweepTimeTotal;
+static constexpr Seconds sweepTimeSlice = 10_ms;
+static constexpr double sweepTimeTotal = .10;
+static constexpr double sweepTimeMultiplier = 1.0 / sweepTimeTotal;
 
 void IncrementalSweeper::scheduleTimer()
 {
-    Base::scheduleTimer(sweepTimeSlice * sweepTimeMultiplier);
+    setTimeUntilFire(sweepTimeSlice * sweepTimeMultiplier);
 }
 
 IncrementalSweeper::IncrementalSweeper(Heap* heap)
@@ -49,14 +58,14 @@ IncrementalSweeper::IncrementalSweeper(Heap* heap)
 {
 }
 
-void IncrementalSweeper::doWork()
+void IncrementalSweeper::doWork(VM& vm)
 {
-    doSweep(MonotonicTime::now());
+    doSweep(vm, MonotonicTime::now());
 }
 
-void IncrementalSweeper::doSweep(MonotonicTime sweepBeginTime)
+void IncrementalSweeper::doSweep(VM& vm, MonotonicTime sweepBeginTime)
 {
-    while (sweepNextBlock()) {
+    while (sweepNextBlock(vm)) {
         Seconds elapsedTime = MonotonicTime::now() - sweepBeginTime;
         if (elapsedTime < sweepTimeSlice)
             continue;
@@ -65,6 +74,11 @@ void IncrementalSweeper::doSweep(MonotonicTime sweepBeginTime)
         return;
     }
 
+#if !USE(SYSTEM_MALLOC)
+#if BUSE(LIBPAS)
+    pas_thread_local_cache_flush_deallocation_log(pas_thread_local_cache_try_get(), pas_lock_is_not_held);
+#endif
+#endif
     if (m_shouldFreeFastMallocMemoryAfterSweeping) {
         WTF::releaseFastMallocFreeMemory();
         m_shouldFreeFastMallocMemoryAfterSweeping = false;
@@ -72,9 +86,9 @@ void IncrementalSweeper::doSweep(MonotonicTime sweepBeginTime)
     cancelTimer();
 }
 
-bool IncrementalSweeper::sweepNextBlock()
+bool IncrementalSweeper::sweepNextBlock(VM& vm)
 {
-    m_vm->heap.stopIfNecessary();
+    vm.heap.stopIfNecessary();
 
     MarkedBlock::Handle* block = nullptr;
     
@@ -85,26 +99,25 @@ bool IncrementalSweeper::sweepNextBlock()
     }
     
     if (block) {
-        DeferGCForAWhile deferGC(m_vm->heap);
+        DeferGCForAWhile deferGC(vm);
         block->sweep(nullptr);
-        m_vm->heap.objectSpace().freeOrShrinkBlock(block);
+        vm.heap.objectSpace().freeOrShrinkBlock(block);
         return true;
     }
 
-    return m_vm->heap.sweepNextLogicallyEmptyWeakBlock();
+    return vm.heap.sweepNextLogicallyEmptyWeakBlock();
 }
 
-void IncrementalSweeper::startSweeping()
+void IncrementalSweeper::startSweeping(Heap& heap)
 {
     scheduleTimer();
-    m_currentDirectory = m_vm->heap.objectSpace().firstDirectory();
+    m_currentDirectory = heap.objectSpace().firstDirectory();
 }
 
 void IncrementalSweeper::stopSweeping()
 {
     m_currentDirectory = nullptr;
-    if (m_vm)
-        cancelTimer();
+    cancelTimer();
 }
 
 } // namespace JSC
