@@ -43,6 +43,7 @@ StackVisitor::StackVisitor(CallFrame* startFrame, VM& vm)
 {
     m_frame.m_index = 0;
     m_frame.m_isWasmFrame = false;
+    m_frame.m_wasmDistanceFromDeepestInlineFrame = 0;
     CallFrame* topFrame;
     if (startFrame) {
         ASSERT(!vm.topCallFrame || reinterpret_cast<void*>(vm.topCallFrame) != vm.topEntryFrame);
@@ -73,7 +74,7 @@ void StackVisitor::gotoNextFrame()
 {
     m_frame.m_index++;
 #if ENABLE(DFG_JIT)
-    if (m_frame.isInlinedFrame()) {
+    if (m_frame.isInlinedDFGFrame()) {
         InlineCallFrame* inlineCallFrame = m_frame.inlineCallFrame();
         CodeOrigin* callerCodeOrigin = inlineCallFrame->getCallerSkippingTailCalls();
         if (!callerCodeOrigin) {
@@ -95,7 +96,7 @@ void StackVisitor::gotoNextFrame()
 void StackVisitor::unwindToMachineCodeBlockFrame()
 {
 #if ENABLE(DFG_JIT)
-    if (m_frame.isInlinedFrame()) {
+    if (m_frame.isInlinedDFGFrame()) {
         CodeOrigin codeOrigin = m_frame.inlineCallFrame()->directCaller;
         while (codeOrigin.inlineCallFrame())
             codeOrigin = codeOrigin.inlineCallFrame()->directCaller;
@@ -111,8 +112,8 @@ void StackVisitor::readFrame(CallFrame* callFrame)
         return;
     }
 
-    if (callFrame->isAnyWasmCallee()) {
-        readNonInlinedFrame(callFrame);
+    if (callFrame->isWasmFrame()) {
+        readInlinableWasmFrame(callFrame);
         return;
     }
 
@@ -164,22 +165,57 @@ void StackVisitor::readNonInlinedFrame(CallFrame* callFrame, CodeOrigin* codeOri
     m_frame.m_isWasmFrame = false;
     m_frame.m_callee = callFrame->callee();
 #if ENABLE(DFG_JIT)
-    m_frame.m_inlineCallFrame = nullptr;
+    m_frame.m_inlineDFGCallFrame = nullptr;
 #endif
+    m_frame.m_wasmDistanceFromDeepestInlineFrame = 0;
 
-    m_frame.m_codeBlock = callFrame->isAnyWasmCallee() ? nullptr : callFrame->codeBlock();
+    m_frame.m_codeBlock = callFrame->isWasmFrame() ? nullptr : callFrame->codeBlock();
     m_frame.m_bytecodeIndex = !m_frame.codeBlock() ? BytecodeIndex(0)
         : codeOrigin ? codeOrigin->bytecodeIndex()
         : callFrame->bytecodeIndex();
 
-#if ENABLE(WEBASSEMBLY)
-    if (callFrame->isAnyWasmCallee()) {
-        m_frame.m_isWasmFrame = true;
-        m_frame.m_codeBlock = nullptr;
+    RELEASE_ASSERT(!callFrame->isWasmFrame());
+}
 
-        if (m_frame.m_callee.isWasm())
-            m_frame.m_wasmFunctionIndexOrName = m_frame.m_callee.asWasmCallee()->indexOrName();
-    }
+void StackVisitor::readInlinableWasmFrame(CallFrame* callFrame)
+{
+#if ENABLE(WEBASSEMBLY)
+    auto depth = m_frame.m_wasmDistanceFromDeepestInlineFrame;
+    m_frame.m_isWasmFrame = true;
+    m_frame.m_callFrame = callFrame;
+    m_frame.m_argumentCountIncludingThis = callFrame->argumentCountIncludingThis();
+    m_frame.m_callerEntryFrame = m_frame.m_entryFrame;
+    m_frame.m_callerFrame = callFrame->callerFrame(m_frame.m_callerEntryFrame);
+    m_frame.m_callerIsEntryFrame = m_frame.m_callerEntryFrame != m_frame.m_entryFrame;
+    m_frame.m_callee = callFrame->callee();
+    m_frame.m_codeBlock = nullptr;
+    m_frame.m_wasmDistanceFromDeepestInlineFrame = 0;
+
+    const auto& callee = *m_frame.m_callee.asWasmCallee();
+    auto csi = callFrame->callSiteIndex().bits();
+
+    RELEASE_ASSERT(m_frame.m_callee.isWasm());
+    if (m_frame.m_callee.isWasm())
+        m_frame.m_wasmFunctionIndexOrName = m_frame.m_callee.asWasmCallee()->indexOrName();
+
+    bool canInline = isAnyOMG(callee.compilationMode());
+    if (!canInline)
+        return;
+
+    const auto& omgCallee = *static_cast<const Wasm::OptimizingJITCallee*>(&callee);
+    bool isInlined = false;
+    auto origin = omgCallee.getOrigin(csi, depth, isInlined);
+
+    if (!isInlined)
+        return;
+
+    // The callerFrame just needs to be non-null to indicate that we
+    // haven't reached the last frame yet.
+    m_frame.m_callerFrame = callFrame;
+    m_frame.m_wasmDistanceFromDeepestInlineFrame = depth + 1;
+    m_frame.m_wasmFunctionIndexOrName = origin;
+#else
+    UNUSED_PARAM(callFrame);
 #endif
 }
 
@@ -195,6 +231,7 @@ void StackVisitor::readInlinedFrame(CallFrame* callFrame, CodeOrigin* codeOrigin
 {
     ASSERT(codeOrigin);
     m_frame.m_isWasmFrame = false;
+    m_frame.m_wasmDistanceFromDeepestInlineFrame = 0;
 
     int frameOffset = inlinedFrameOffset(codeOrigin);
     bool isInlined = !!frameOffset;
@@ -202,7 +239,7 @@ void StackVisitor::readInlinedFrame(CallFrame* callFrame, CodeOrigin* codeOrigin
         InlineCallFrame* inlineCallFrame = codeOrigin->inlineCallFrame();
 
         m_frame.m_callFrame = callFrame;
-        m_frame.m_inlineCallFrame = inlineCallFrame;
+        m_frame.m_inlineDFGCallFrame = inlineCallFrame;
         if (inlineCallFrame->argumentCountRegister.isValid())
             m_frame.m_argumentCountIncludingThis = callFrame->r(inlineCallFrame->argumentCountRegister).unboxedInt32();
         else
@@ -254,7 +291,7 @@ std::optional<RegisterAtOffsetList> StackVisitor::Frame::calleeSaveRegistersForU
     if (!NUMBER_OF_CALLEE_SAVES_REGISTERS)
         return std::nullopt;
 
-    if (isInlinedFrame())
+    if (isInlinedDFGFrame())
         return std::nullopt;
 
 #if ENABLE(WEBASSEMBLY)
@@ -264,7 +301,9 @@ std::optional<RegisterAtOffsetList> StackVisitor::Frame::calleeSaveRegistersForU
             return std::nullopt;
         }
         Wasm::Callee* wasmCallee = callee().asWasmCallee();
-        return *wasmCallee->calleeSaveRegisters();
+        if (auto* calleeSaveRegisters = wasmCallee->calleeSaveRegisters())
+            return *calleeSaveRegisters;
+        return std::nullopt;
     }
 #endif // ENABLE(WEBASSEMBLY)
 
@@ -329,6 +368,28 @@ String StackVisitor::Frame::sourceURL() const
     return traceLine.isNull() ? emptyString() : traceLine;
 }
 
+String StackVisitor::Frame::preRedirectURL() const
+{
+    String traceLine;
+
+    switch (codeType()) {
+    case CodeType::Eval:
+    case CodeType::Module:
+    case CodeType::Function:
+    case CodeType::Global: {
+        String preRedirectURL = codeBlock()->ownerExecutable()->preRedirectURL();
+        if (!preRedirectURL.isEmpty())
+            traceLine = preRedirectURL.impl();
+        break;
+    }
+    case CodeType::Native:
+    case CodeType::Wasm:
+        break;
+    }
+
+    return traceLine.isNull() ? emptyString() : traceLine;
+}
+
 String StackVisitor::Frame::toString() const
 {
     String functionName = this->functionName();
@@ -365,9 +426,9 @@ ClonedArguments* StackVisitor::Frame::createArguments(VM& vm)
     else
         mode = ArgumentsMode::FakeValues;
 #if ENABLE(DFG_JIT)
-    if (isInlinedFrame()) {
-        ASSERT(m_inlineCallFrame);
-        arguments = ClonedArguments::createWithInlineFrame(globalObject, physicalFrame, m_inlineCallFrame, mode);
+    if (isInlinedDFGFrame()) {
+        ASSERT(m_inlineDFGCallFrame);
+        arguments = ClonedArguments::createWithInlineFrame(globalObject, physicalFrame, m_inlineDFGCallFrame, mode);
     } else 
 #endif
         arguments = ClonedArguments::createWithMachineFrame(globalObject, physicalFrame, mode);
@@ -413,30 +474,38 @@ void StackVisitor::Frame::setToEnd()
 {
     m_callFrame = nullptr;
 #if ENABLE(DFG_JIT)
-    m_inlineCallFrame = nullptr;
+    m_inlineDFGCallFrame = nullptr;
 #endif
     m_isWasmFrame = false;
 }
 
 bool StackVisitor::Frame::isImplementationVisibilityPrivate() const
 {
-    auto* executable = [&] () -> ExecutableBase* {
-        if (auto* codeBlock = this->codeBlock())
-            return codeBlock->ownerExecutable();
+    ImplementationVisibility implementationVisibility = [&] () -> ImplementationVisibility {
+        if (auto* codeBlock = this->codeBlock()) {
+            if (auto* executable = codeBlock->ownerExecutable())
+                return executable->implementationVisibility();
+            return ImplementationVisibility::Public;
+        }
+
+#if ENABLE(WEBASSEMBLY)
+        if (isWasmFrame())
+            return callee().asWasmCallee()->implementationVisibility();
+#endif
 
         if (callee().isCell()) {
             if (auto* callee = this->callee().asCell()) {
-                if (auto* jsFunction = jsDynamicCast<JSFunction*>(callee))
-                    return jsFunction->executable();
+                if (auto* jsFunction = jsDynamicCast<JSFunction*>(callee)) {
+                    if (auto* executable = jsFunction->executable())
+                        return executable->implementationVisibility();
+                    return ImplementationVisibility::Public;
+                }
             }
         }
 
-        return nullptr;
+        return ImplementationVisibility::Public;
     }();
-    if (!executable)
-        return false;
-
-    switch (executable->implementationVisibility()) {
+    switch (implementationVisibility) {
     case ImplementationVisibility::Public:
         return false;
 
@@ -478,10 +547,10 @@ void StackVisitor::Frame::dump(PrintStream& out, Indenter indent, WTF::Function<
 
         bool isInlined = false;
 #if ENABLE(DFG_JIT)
-        isInlined = isInlinedFrame();
-        out.print(indent, "isInlinedFrame: ", isInlinedFrame(), "\n");
-        if (isInlinedFrame())
-            out.print(indent, "InlineCallFrame: ", RawPointer(m_inlineCallFrame), "\n");
+        isInlined = isInlinedDFGFrame();
+        out.print(indent, "isInlinedDFGFrame: ", isInlinedDFGFrame(), "\n");
+        if (isInlinedDFGFrame())
+            out.print(indent, "InlineCallFrame: ", RawPointer(m_inlineDFGCallFrame), "\n");
 #endif
 
         out.print(indent, "callee: ", RawPointer(callee().rawPtr()), "\n");
