@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2008-2022 Apple Inc. All rights reserved.
+ * Copyright (C) 2008-2023 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -36,7 +36,6 @@
 #include "JSCPtrTag.h"
 #include "LLIntData.h"
 #include "Options.h"
-#include "SigillCrashAnalyzer.h"
 #include "StructureAlignedMemoryAllocator.h"
 #include "SuperSampler.h"
 #include "VMTraps.h"
@@ -86,6 +85,7 @@ void initialize()
             } else {
 #if CPU(ARM64E) && ENABLE(JIT)
                 g_jscConfig.arm64eHashPins.initializeAtStartup();
+                isARM64E_FPAC(); // Call this to initialize g_jscConfig.canUseFPAC.
 #endif
             }
             StructureAlignedMemoryAllocator::initializeStructureAddressSpace();
@@ -101,9 +101,6 @@ void initialize()
 
         JITOperationList::populatePointersInJavaScriptCore();
 
-        if (Options::useSigillCrashAnalyzer())
-            enableSigillCrashAnalyzer();
-
         AssemblyCommentRegistry::initialize();
         LLInt::initialize();
         DisallowGC::initialize();
@@ -122,14 +119,16 @@ void initialize()
         if (VM::isInMiniMode())
             WTF::fastEnableMiniMode();
 
-#if HAVE(MACH_EXCEPTIONS)
-        // JSLock::lock() can call registerThreadForMachExceptionHandling() which crashes if this has not been called first.
-        WTF::startMachExceptionHandlerThread();
-#endif
-        VMTraps::initializeSignals();
-#if ENABLE(WEBASSEMBLY)
-        Wasm::prepareSignalingMemory();
-#endif
+        if (Wasm::isSupported() || !Options::usePollingTraps()) {
+            // JSLock::lock() can call registerThreadForMachExceptionHandling() which crashes if this has not been called first.
+            initializeSignalHandling();
+
+            if (!Options::usePollingTraps())
+                VMTraps::initializeSignals();
+            if (Wasm::isSupported())
+                Wasm::prepareSignalingMemory();
+        } else
+            disableSignalHandling();
 
         WTF::compilerFence();
         RELEASE_ASSERT(!g_jscConfig.initializeHasBeenCalled);
