@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2009-2019 Apple Inc. All rights reserved.
+ * Copyright (C) 2009-2022 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -29,6 +29,7 @@
 #include "Debugger.h"
 #include "EvalCodeBlock.h"
 #include "FunctionCodeBlock.h"
+#include "FunctionExecutableInlines.h"
 #include "GlobalExecutable.h"
 #include "IsoCellSetInlines.h"
 #include "JIT.h"
@@ -44,13 +45,14 @@
 
 namespace JSC {
 
-const ClassInfo ScriptExecutable::s_info = { "ScriptExecutable", &ExecutableBase::s_info, nullptr, nullptr, CREATE_METHOD_TABLE(ScriptExecutable) };
+const ClassInfo ScriptExecutable::s_info = { "ScriptExecutable"_s, &ExecutableBase::s_info, nullptr, nullptr, CREATE_METHOD_TABLE(ScriptExecutable) };
 
-ScriptExecutable::ScriptExecutable(Structure* structure, VM& vm, const SourceCode& source, bool isInStrictContext, DerivedContextType derivedContextType, bool isInArrowFunctionContext, bool isInsideOrdinaryFunction, EvalContextType evalContextType, Intrinsic intrinsic)
+ScriptExecutable::ScriptExecutable(Structure* structure, VM& vm, const SourceCode& source, LexicalScopeFeatures lexicalScopeFeatures, DerivedContextType derivedContextType, bool isInArrowFunctionContext, bool isInsideOrdinaryFunction, EvalContextType evalContextType, Intrinsic intrinsic)
     : ExecutableBase(vm, structure)
     , m_source(source)
     , m_intrinsic(intrinsic)
-    , m_features(isInStrictContext ? StrictModeFeature : 0)
+    , m_features(NoFeatures)
+    , m_lexicalScopeFeatures(lexicalScopeFeatures)
     , m_hasCapturedVariables(false)
     , m_neverInline(false)
     , m_neverOptimize(false)
@@ -73,8 +75,8 @@ void ScriptExecutable::clearCode(IsoCellSet& clearableCodeSet)
 {
     m_jitCodeForCall = nullptr;
     m_jitCodeForConstruct = nullptr;
-    m_jitCodeForCallWithArityCheck = MacroAssemblerCodePtr<JSEntryPtrTag>();
-    m_jitCodeForConstructWithArityCheck = MacroAssemblerCodePtr<JSEntryPtrTag>();
+    m_jitCodeForCallWithArityCheck = CodePtr<JSEntryPtrTag>();
+    m_jitCodeForConstructWithArityCheck = CodePtr<JSEntryPtrTag>();
 
     switch (type()) {
     case FunctionExecutableType: {
@@ -85,20 +87,20 @@ void ScriptExecutable::clearCode(IsoCellSet& clearableCodeSet)
     }
     case EvalExecutableType: {
         EvalExecutable* executable = static_cast<EvalExecutable*>(this);
-        executable->m_evalCodeBlock.clear();
-        executable->m_unlinkedEvalCodeBlock.clear();
+        executable->m_codeBlock.clear();
+        executable->m_unlinkedCodeBlock.clear();
         break;
     }
     case ProgramExecutableType: {
         ProgramExecutable* executable = static_cast<ProgramExecutable*>(this);
-        executable->m_programCodeBlock.clear();
-        executable->m_unlinkedProgramCodeBlock.clear();
+        executable->m_codeBlock.clear();
+        executable->m_unlinkedCodeBlock.clear();
         break;
     }
     case ModuleProgramExecutableType: {
         ModuleProgramExecutable* executable = static_cast<ModuleProgramExecutable*>(this);
-        executable->m_moduleProgramCodeBlock.clear();
-        executable->m_unlinkedModuleProgramCodeBlock.clear();
+        executable->m_codeBlock.clear();
+        executable->m_unlinkedCodeBlock.clear();
         executable->m_moduleEnvironmentSymbolTable.clear();
         break;
     }
@@ -107,7 +109,7 @@ void ScriptExecutable::clearCode(IsoCellSet& clearableCodeSet)
         break;
     }
 
-    ASSERT(&VM::SpaceAndSet::setFor(*subspace()) == &clearableCodeSet);
+    ASSERT(&Heap::ScriptExecutableSpaceAndSets::clearableCodeSetFor(*subspace()) == &clearableCodeSet);
     clearableCodeSet.remove(this);
 }
 
@@ -130,8 +132,7 @@ void ScriptExecutable::installCode(VM& vm, CodeBlock* genericCodeBlock, CodeType
         
         ASSERT(kind == CodeForCall);
         
-        oldCodeBlock = ExecutableToCodeBlockEdge::deactivateAndUnwrap(executable->m_programCodeBlock.get());
-        executable->m_programCodeBlock.setMayBeNull(vm, this, ExecutableToCodeBlockEdge::wrapAndActivate(codeBlock));
+        oldCodeBlock = executable->replaceCodeBlockWith(vm, codeBlock);
         break;
     }
 
@@ -141,8 +142,7 @@ void ScriptExecutable::installCode(VM& vm, CodeBlock* genericCodeBlock, CodeType
 
         ASSERT(kind == CodeForCall);
 
-        oldCodeBlock = ExecutableToCodeBlockEdge::deactivateAndUnwrap(executable->m_moduleProgramCodeBlock.get());
-        executable->m_moduleProgramCodeBlock.setMayBeNull(vm, this, ExecutableToCodeBlockEdge::wrapAndActivate(codeBlock));
+        oldCodeBlock = executable->replaceCodeBlockWith(vm, codeBlock);
         break;
     }
 
@@ -152,8 +152,7 @@ void ScriptExecutable::installCode(VM& vm, CodeBlock* genericCodeBlock, CodeType
         
         ASSERT(kind == CodeForCall);
         
-        oldCodeBlock = ExecutableToCodeBlockEdge::deactivateAndUnwrap(executable->m_evalCodeBlock.get());
-        executable->m_evalCodeBlock.setMayBeNull(vm, this, ExecutableToCodeBlockEdge::wrapAndActivate(codeBlock));
+        oldCodeBlock = executable->replaceCodeBlockWith(vm, codeBlock);
         break;
     }
         
@@ -161,16 +160,7 @@ void ScriptExecutable::installCode(VM& vm, CodeBlock* genericCodeBlock, CodeType
         FunctionExecutable* executable = jsCast<FunctionExecutable*>(this);
         FunctionCodeBlock* codeBlock = static_cast<FunctionCodeBlock*>(genericCodeBlock);
         
-        switch (kind) {
-        case CodeForCall:
-            oldCodeBlock = ExecutableToCodeBlockEdge::deactivateAndUnwrap(executable->m_codeBlockForCall.get());
-            executable->m_codeBlockForCall.setMayBeNull(vm, this, ExecutableToCodeBlockEdge::wrapAndActivate(codeBlock));
-            break;
-        case CodeForConstruct:
-            oldCodeBlock = ExecutableToCodeBlockEdge::deactivateAndUnwrap(executable->m_codeBlockForConstruct.get());
-            executable->m_codeBlockForConstruct.setMayBeNull(vm, this, ExecutableToCodeBlockEdge::wrapAndActivate(codeBlock));
-            break;
-        }
+        oldCodeBlock = executable->replaceCodeBlockWith(vm, kind, codeBlock);
         break;
     }
     }
@@ -186,8 +176,8 @@ void ScriptExecutable::installCode(VM& vm, CodeBlock* genericCodeBlock, CodeType
         break;
     }
 
-    auto& clearableCodeSet = VM::SpaceAndSet::setFor(*subspace());
-    if (hasClearableCode(vm))
+    auto& clearableCodeSet = Heap::ScriptExecutableSpaceAndSets::clearableCodeSetFor(*subspace());
+    if (hasClearableCode())
         clearableCodeSet.add(this);
     else
         clearableCodeSet.remove(this);
@@ -195,6 +185,8 @@ void ScriptExecutable::installCode(VM& vm, CodeBlock* genericCodeBlock, CodeType
     if (genericCodeBlock) {
         RELEASE_ASSERT(genericCodeBlock->ownerExecutable() == this);
         RELEASE_ASSERT(JITCode::isExecutableScript(genericCodeBlock->jitType()));
+
+        genericCodeBlock->m_isJettisoned = false;
         
         dataLogLnIf(Options::verboseOSR(), "Installing ", *genericCodeBlock);
         
@@ -209,10 +201,10 @@ void ScriptExecutable::installCode(VM& vm, CodeBlock* genericCodeBlock, CodeType
     if (oldCodeBlock)
         oldCodeBlock->unlinkIncomingCalls();
 
-    vm.heap.writeBarrier(this);
+    vm.writeBarrier(this);
 }
 
-bool ScriptExecutable::hasClearableCode(VM& vm) const
+bool ScriptExecutable::hasClearableCode() const
 {
     if (m_jitCodeForCall
         || m_jitCodeForConstruct
@@ -220,33 +212,32 @@ bool ScriptExecutable::hasClearableCode(VM& vm) const
         || m_jitCodeForConstructWithArityCheck)
         return true;
 
-    if (structure(vm)->classInfo() == FunctionExecutable::info()) {
+    if (structure()->classInfoForCells() == FunctionExecutable::info()) {
         auto* executable = static_cast<const FunctionExecutable*>(this);
-        if (executable->m_codeBlockForCall || executable->m_codeBlockForConstruct)
+        if (executable->eitherCodeBlock())
             return true;
 
-    } else if (structure(vm)->classInfo() == EvalExecutable::info()) {
+    } else if (structure()->classInfoForCells() == EvalExecutable::info()) {
         auto* executable = static_cast<const EvalExecutable*>(this);
-        if (executable->m_evalCodeBlock || executable->m_unlinkedEvalCodeBlock)
+        if (executable->m_codeBlock || executable->m_unlinkedCodeBlock)
             return true;
 
-    } else if (structure(vm)->classInfo() == ProgramExecutable::info()) {
+    } else if (structure()->classInfoForCells() == ProgramExecutable::info()) {
         auto* executable = static_cast<const ProgramExecutable*>(this);
-        if (executable->m_programCodeBlock || executable->m_unlinkedProgramCodeBlock)
+        if (executable->m_codeBlock || executable->m_unlinkedCodeBlock)
             return true;
 
-    } else if (structure(vm)->classInfo() == ModuleProgramExecutable::info()) {
+    } else if (structure()->classInfoForCells() == ModuleProgramExecutable::info()) {
         auto* executable = static_cast<const ModuleProgramExecutable*>(this);
-        if (executable->m_moduleProgramCodeBlock
-            || executable->m_unlinkedModuleProgramCodeBlock
+        if (executable->m_codeBlock
+            || executable->m_unlinkedCodeBlock
             || executable->m_moduleEnvironmentSymbolTable)
             return true;
     }
     return false;
 }
 
-CodeBlock* ScriptExecutable::newCodeBlockFor(
-    CodeSpecializationKind kind, JSFunction* function, JSScope* scope, Exception*& exception)
+CodeBlock* ScriptExecutable::newCodeBlockFor(CodeSpecializationKind kind, JSFunction* function, JSScope* scope)
 {
     VM& vm = scope->vm();
     auto throwScope = DECLARE_THROW_SCOPE(vm);
@@ -254,60 +245,42 @@ CodeBlock* ScriptExecutable::newCodeBlockFor(
     ASSERT(vm.heap.isDeferred());
     ASSERT(endColumn() != UINT_MAX);
 
-    JSGlobalObject* globalObject = scope->globalObject(vm);
+    JSGlobalObject* globalObject = scope->globalObject();
 
-    if (classInfo(vm) == EvalExecutable::info()) {
+    if (classInfo() == EvalExecutable::info()) {
         EvalExecutable* executable = jsCast<EvalExecutable*>(this);
         RELEASE_ASSERT(kind == CodeForCall);
-        RELEASE_ASSERT(!executable->m_evalCodeBlock);
+        RELEASE_ASSERT(!executable->m_codeBlock);
         RELEASE_ASSERT(!function);
-        auto codeBlock = EvalCodeBlock::create(vm,
-            executable, executable->m_unlinkedEvalCodeBlock.get(), scope);
-        EXCEPTION_ASSERT(throwScope.exception() || codeBlock);
-        if (!codeBlock) {
-            exception = throwException(
-                globalObject, throwScope,
-                createOutOfMemoryError(globalObject));
-            return nullptr;
-        }
-        return codeBlock;
+
+        // FIXME: There might be a case that executable->unlinkedCodeBlock() will be a nullptr 
+        // since ScriptExecutable::clearCode might be triggered due to limited memory usage. 
+        // We should regenerate unlinkedCodeBlock if necessary for both EvalExecutable and ProgramExecutable.
+        // See similar problem for ModuleProgramExecutable in https://bugs.webkit.org/show_bug.cgi?id=255044.
+        RELEASE_AND_RETURN(throwScope, EvalCodeBlock::create(vm, executable, executable->unlinkedCodeBlock(), scope));
     }
-    
-    if (classInfo(vm) == ProgramExecutable::info()) {
+
+    if (classInfo() == ProgramExecutable::info()) {
         ProgramExecutable* executable = jsCast<ProgramExecutable*>(this);
         RELEASE_ASSERT(kind == CodeForCall);
-        RELEASE_ASSERT(!executable->m_programCodeBlock);
+        RELEASE_ASSERT(!executable->m_codeBlock);
         RELEASE_ASSERT(!function);
-        auto codeBlock = ProgramCodeBlock::create(vm,
-            executable, executable->m_unlinkedProgramCodeBlock.get(), scope);
-        EXCEPTION_ASSERT(throwScope.exception() || codeBlock);
-        if (!codeBlock) {
-            exception = throwException(
-                globalObject, throwScope,
-                createOutOfMemoryError(globalObject));
-            return nullptr;
-        }
-        return codeBlock;
+        RELEASE_AND_RETURN(throwScope, ProgramCodeBlock::create(vm, executable, executable->unlinkedCodeBlock(), scope));
     }
 
-    if (classInfo(vm) == ModuleProgramExecutable::info()) {
+    if (classInfo() == ModuleProgramExecutable::info()) {
         ModuleProgramExecutable* executable = jsCast<ModuleProgramExecutable*>(this);
         RELEASE_ASSERT(kind == CodeForCall);
-        RELEASE_ASSERT(!executable->m_moduleProgramCodeBlock);
+        RELEASE_ASSERT(!executable->m_codeBlock);
         RELEASE_ASSERT(!function);
-        auto codeBlock = ModuleProgramCodeBlock::create(vm,
-            executable, executable->m_unlinkedModuleProgramCodeBlock.get(), scope);
-        EXCEPTION_ASSERT(throwScope.exception() || codeBlock);
-        if (!codeBlock) {
-            exception = throwException(
-                globalObject, throwScope,
-                createOutOfMemoryError(globalObject));
-            return nullptr;
-        }
-        return codeBlock;
+
+        UnlinkedModuleProgramCodeBlock* unlinkedCodeBlock = executable->getUnlinkedCodeBlock(globalObject);
+        RETURN_IF_EXCEPTION(throwScope, nullptr);
+        ASSERT(executable->unlinkedCodeBlock());
+        RELEASE_AND_RETURN(throwScope, ModuleProgramCodeBlock::create(vm, executable, unlinkedCodeBlock, scope));
     }
 
-    RELEASE_ASSERT(classInfo(vm) == FunctionExecutable::info());
+    RELEASE_ASSERT(classInfo() == FunctionExecutable::info());
     RELEASE_ASSERT(function);
     FunctionExecutable* executable = jsCast<FunctionExecutable*>(this);
     RELEASE_ASSERT(!executable->codeBlockFor(kind));
@@ -328,15 +301,13 @@ CodeBlock* ScriptExecutable::newCodeBlockFor(
             executable->parseMode());
     recordParse(
         executable->m_unlinkedExecutable->features(), 
+        executable->m_unlinkedExecutable->lexicalScopeFeatures(),
         executable->m_unlinkedExecutable->hasCapturedVariables(),
-        lastLine(), endColumn()); 
+        lastLine(), endColumn());
     if (!unlinkedCodeBlock) {
-        exception = throwException(
-            globalObject, throwScope,
-            error.toErrorObject(globalObject, executable->source()));
+        throwException(globalObject, throwScope, error.toErrorObject(globalObject, executable->source()));
         return nullptr;
     }
-
     RELEASE_AND_RETURN(throwScope, FunctionCodeBlock::create(vm, executable, unlinkedCodeBlock, scope));
 }
 
@@ -344,7 +315,7 @@ CodeBlock* ScriptExecutable::newReplacementCodeBlockFor(
     CodeSpecializationKind kind)
 {
     VM& vm = this->vm();
-    if (classInfo(vm) == EvalExecutable::info()) {
+    if (classInfo() == EvalExecutable::info()) {
         RELEASE_ASSERT(kind == CodeForCall);
         EvalExecutable* executable = jsCast<EvalExecutable*>(this);
         EvalCodeBlock* baseline = static_cast<EvalCodeBlock*>(
@@ -355,7 +326,7 @@ CodeBlock* ScriptExecutable::newReplacementCodeBlockFor(
         return result;
     }
     
-    if (classInfo(vm) == ProgramExecutable::info()) {
+    if (classInfo() == ProgramExecutable::info()) {
         RELEASE_ASSERT(kind == CodeForCall);
         ProgramExecutable* executable = jsCast<ProgramExecutable*>(this);
         ProgramCodeBlock* baseline = static_cast<ProgramCodeBlock*>(
@@ -366,7 +337,7 @@ CodeBlock* ScriptExecutable::newReplacementCodeBlockFor(
         return result;
     }
 
-    if (classInfo(vm) == ModuleProgramExecutable::info()) {
+    if (classInfo() == ModuleProgramExecutable::info()) {
         RELEASE_ASSERT(kind == CodeForCall);
         ModuleProgramExecutable* executable = jsCast<ModuleProgramExecutable*>(this);
         ModuleProgramCodeBlock* baseline = static_cast<ModuleProgramCodeBlock*>(
@@ -377,7 +348,7 @@ CodeBlock* ScriptExecutable::newReplacementCodeBlockFor(
         return result;
     }
 
-    RELEASE_ASSERT(classInfo(vm) == FunctionExecutable::info());
+    RELEASE_ASSERT(classInfo() == FunctionExecutable::info());
     FunctionExecutable* executable = jsCast<FunctionExecutable*>(this);
     FunctionCodeBlock* baseline = static_cast<FunctionCodeBlock*>(
         executable->codeBlockFor(kind)->baselineVersion());
@@ -404,34 +375,41 @@ static void setupJIT(VM& vm, CodeBlock* codeBlock)
 #endif
 }
 
-Exception* ScriptExecutable::prepareForExecutionImpl(
-    VM& vm, JSFunction* function, JSScope* scope, CodeSpecializationKind kind, CodeBlock*& resultCodeBlock)
+void ScriptExecutable::prepareForExecutionImpl(VM& vm, JSFunction* function, JSScope* scope, CodeSpecializationKind kind, CodeBlock*& resultCodeBlock)
 {
     auto throwScope = DECLARE_THROW_SCOPE(vm);
-    DeferGCForAWhile deferGC(vm.heap);
+    DeferGCForAWhile deferGC(vm);
 
     if (UNLIKELY(vm.getAndClearFailNextNewCodeBlock())) {
-        JSGlobalObject* globalObject = scope->globalObject(vm);
-        return throwException(globalObject, throwScope, createError(globalObject, "Forced Failure"_s));
+        JSGlobalObject* globalObject = scope->globalObject();
+        throwException(globalObject, throwScope, createError(globalObject, "Forced Failure"_s));
+        return;
     }
 
-    Exception* exception = nullptr;
-    CodeBlock* codeBlock = newCodeBlockFor(kind, function, scope, exception);
+    CodeBlock* codeBlock = newCodeBlockFor(kind, function, scope);
+    RETURN_IF_EXCEPTION(throwScope, void());
+
+    ASSERT(codeBlock);
     resultCodeBlock = codeBlock;
-    EXCEPTION_ASSERT(!!throwScope.exception() == !codeBlock);
-    if (UNLIKELY(!codeBlock))
-        return exception;
-    
+
     if (Options::validateBytecode())
         codeBlock->validate();
-    
-    if (Options::useLLInt())
-        setupLLInt(codeBlock);
-    else
-        setupJIT(vm, codeBlock);
-    
+
+    bool installedUnlinkedBaselineCode = false;
+#if ENABLE(JIT)
+    if (RefPtr<BaselineJITCode> baselineRef = codeBlock->unlinkedCodeBlock()->m_unlinkedBaselineCode) {
+        codeBlock->setupWithUnlinkedBaselineCode(baselineRef.releaseNonNull());
+        installedUnlinkedBaselineCode = true;
+    }
+#endif
+    if (!installedUnlinkedBaselineCode) {
+        if (Options::useLLInt())
+            setupLLInt(codeBlock);
+        else
+            setupJIT(vm, codeBlock);
+    }
+
     installCode(vm, codeBlock, codeBlock->codeType(), codeBlock->specializationKind());
-    return nullptr;
 }
 
 ScriptExecutable* ScriptExecutable::topLevelExecutable()
@@ -452,7 +430,7 @@ JSArray* ScriptExecutable::createTemplateObject(JSGlobalObject* globalObject, JS
     TemplateObjectMap& templateObjectMap = ensureTemplateObjectMap(vm);
     TemplateObjectMap::AddResult result;
     {
-        auto locker = holdLock(cellLock());
+        Locker locker { cellLock() };
         result = templateObjectMap.add(descriptor->endOffset(), WriteBarrier<JSArray>());
     }
     if (JSArray* array = result.iterator->value.get())
@@ -494,40 +472,40 @@ CodeBlockHash ScriptExecutable::hashFor(CodeSpecializationKind kind) const
     return CodeBlockHash(source(), kind);
 }
 
-Optional<int> ScriptExecutable::overrideLineNumber(VM& vm) const
+std::optional<int> ScriptExecutable::overrideLineNumber(VM&) const
 {
-    if (inherits<FunctionExecutable>(vm))
+    if (inherits<FunctionExecutable>())
         return jsCast<const FunctionExecutable*>(this)->overrideLineNumber();
-    return WTF::nullopt;
+    return std::nullopt;
 }
 
 unsigned ScriptExecutable::typeProfilingStartOffset(VM& vm) const
 {
-    if (inherits<FunctionExecutable>(vm))
+    if (inherits<FunctionExecutable>())
         return jsCast<const FunctionExecutable*>(this)->typeProfilingStartOffset(vm);
-    if (inherits<EvalExecutable>(vm))
+    if (inherits<EvalExecutable>())
         return UINT_MAX;
     return 0;
 }
 
 unsigned ScriptExecutable::typeProfilingEndOffset(VM& vm) const
 {
-    if (inherits<FunctionExecutable>(vm))
+    if (inherits<FunctionExecutable>())
         return jsCast<const FunctionExecutable*>(this)->typeProfilingEndOffset(vm);
-    if (inherits<EvalExecutable>(vm))
+    if (inherits<EvalExecutable>())
         return UINT_MAX;
     return source().length() - 1;
 }
 
-void ScriptExecutable::recordParse(CodeFeatures features, bool hasCapturedVariables, int lastLine, unsigned endColumn)
+void ScriptExecutable::recordParse(CodeFeatures features, LexicalScopeFeatures lexicalScopeFeatures, bool hasCapturedVariables, int lastLine, unsigned endColumn)
 {
     switch (type()) {
     case FunctionExecutableType:
         // Since UnlinkedFunctionExecutable holds the information to calculate lastLine and endColumn, we do not need to remember them in ScriptExecutable's fields.
-        jsCast<FunctionExecutable*>(this)->recordParse(features, hasCapturedVariables);
+        jsCast<FunctionExecutable*>(this)->recordParse(features, lexicalScopeFeatures, hasCapturedVariables);
         return;
     default:
-        jsCast<GlobalExecutable*>(this)->recordParse(features, hasCapturedVariables, lastLine, endColumn);
+        jsCast<GlobalExecutable*>(this)->recordParse(features, lexicalScopeFeatures, hasCapturedVariables, lastLine, endColumn);
         return;
     }
 }
@@ -554,4 +532,62 @@ unsigned ScriptExecutable::endColumn() const
     return 0;
 }
 
+template<typename Visitor>
+void ScriptExecutable::runConstraint(const ConcurrentJSLocker& locker, Visitor& visitor, CodeBlock* codeBlock)
+{
+    ASSERT(codeBlock);
+    codeBlock->propagateTransitions(locker, visitor);
+    codeBlock->determineLiveness(locker, visitor);
+}
+
+template void ScriptExecutable::runConstraint(const ConcurrentJSLocker&, AbstractSlotVisitor&, CodeBlock*);
+template void ScriptExecutable::runConstraint(const ConcurrentJSLocker&, SlotVisitor&, CodeBlock*);
+
+template<typename Visitor>
+void ScriptExecutable::visitCodeBlockEdge(Visitor& visitor, CodeBlock* codeBlock)
+{
+    ASSERT(codeBlock);
+
+    ConcurrentJSLocker locker(codeBlock->m_lock);
+
+    if (codeBlock->shouldVisitStrongly(locker, visitor))
+        visitor.appendUnbarriered(codeBlock);
+
+    if (JITCode::isOptimizingJIT(codeBlock->jitType())) {
+        // If we jettison ourselves we'll install our alternative, so make sure that it
+        // survives GC even if we don't.
+        visitor.append(codeBlock->m_alternative);
+    }
+
+    // NOTE: There are two sides to this constraint, with different requirements for correctness.
+    // Because everything is ultimately protected with weak references and jettisoning, it's
+    // always "OK" to claim that something is dead prematurely and it's "OK" to keep things alive.
+    // But both choices could lead to bad perf - either recomp cycles or leaks.
+    //
+    // Determining CodeBlock liveness: This part is the most consequential. We want to keep the
+    // output constraint active so long as we think that we may yet prove that the CodeBlock is
+    // live but we haven't done it yet.
+    //
+    // Marking Structures if profitable: It's important that we do a pass of this. Logically, this
+    // seems like it is a constraint of CodeBlock. But we have always first run this as a result
+    // of the edge being marked even before we determine the liveness of the CodeBlock. This
+    // allows a CodeBlock to mark itself by first proving that all of the Structures it weakly
+    // depends on could be strongly marked. (This part is also called propagateTransitions.)
+    //
+    // As a weird caveat, we only fixpoint the constraints so long as the CodeBlock is not live.
+    // This means that we may overlook structure marking opportunities created by other marking
+    // that happens after the CodeBlock is marked. This was an accidental policy decision from a
+    // long time ago, but it is probably OK, since it's only worthwhile to keep fixpointing the
+    // structure marking if we still have unmarked structures after the first round. We almost
+    // never will because we will mark-if-profitable based on the owning global object being
+    // already marked. We mark it just in case that hadn't happened yet. And if the CodeBlock is
+    // not yet marked because it weakly depends on a structure that we did not yet mark, then we
+    // will keep fixpointing until the end.
+    visitor.appendUnbarriered(codeBlock->globalObject());
+    runConstraint(locker, visitor, codeBlock);
+}
+
+template void ScriptExecutable::visitCodeBlockEdge(AbstractSlotVisitor&, CodeBlock*);
+template void ScriptExecutable::visitCodeBlockEdge(SlotVisitor&, CodeBlock*);
+
 } // namespace JSC
