@@ -20,6 +20,9 @@
 #elif defined(ANDROID)
 #    define ANGLE_PLATFORM_ANDROID 1
 #    define ANGLE_PLATFORM_POSIX 1
+#elif defined(__ggp__)
+#    define ANGLE_PLATFORM_GGP 1
+#    define ANGLE_PLATFORM_POSIX 1
 #elif defined(__linux__) || defined(EMSCRIPTEN)
 #    define ANGLE_PLATFORM_LINUX 1
 #    define ANGLE_PLATFORM_POSIX 1
@@ -43,7 +46,6 @@
 #    endif
 
 #    include <intrin.h>
-#    include <windows.h>
 
 #    if defined(WINAPI_FAMILY) && (WINAPI_FAMILY != WINAPI_FAMILY_DESKTOP_APP)
 #        define ANGLE_ENABLE_WINDOWS_UWP 1
@@ -79,8 +81,22 @@
 #        endif
 #    endif
 
+// Include <windows.h> to ensure tests related files can be built when building
+// vulkan only backend ANGLE on windows.
+#    if defined(ANGLE_ENABLE_VULKAN)
+#        include <windows.h>
+#    endif
+
+// Macros 'near', 'far', 'NEAR' and 'FAR' are defined by 'shared/minwindef.h' in the Windows SDK.
+// Macros 'near' and 'far' are empty. They are not used by other Windows headers and are undefined
+// here to avoid identifier conflicts. Macros 'NEAR' and 'FAR' contain 'near' and 'far'. They are
+// used by other Windows headers and are cleared here to avoid compilation errors.
 #    undef near
 #    undef far
+#    undef NEAR
+#    undef FAR
+#    define NEAR
+#    define FAR
 #endif
 
 #if defined(_MSC_VER) && !defined(_M_ARM) && !defined(_M_ARM64)
@@ -96,10 +112,6 @@
 #    include <stddef.h>
 #endif
 
-// The MemoryBarrier function name collides with a macro under Windows
-// We will undef the macro so that the function name does not get replaced
-#undef MemoryBarrier
-
 // Macro for hinting that an expression is likely to be true/false.
 #if !defined(ANGLE_LIKELY) || !defined(ANGLE_UNLIKELY)
 #    if defined(__GNUC__) || defined(__clang__)
@@ -116,15 +128,65 @@
 #    if TARGET_OS_OSX
 #        define ANGLE_PLATFORM_MACOS 1
 #    elif TARGET_OS_IPHONE
-#        define ANGLE_PLATFORM_IOS 1
-#        define GLES_SILENCE_DEPRECATION
+#        define ANGLE_PLATFORM_IOS_FAMILY 1
 #        if TARGET_OS_SIMULATOR
-#            define ANGLE_PLATFORM_IOS_SIMULATOR 1
+#            define ANGLE_PLATFORM_IOS_FAMILY_SIMULATOR 1
 #        endif
-#        if TARGET_OS_MACCATALYST
-#            define ANGLE_PLATFORM_MACCATALYST
+#        if TARGET_OS_IOS
+#            define ANGLE_PLATFORM_IOS 1
+#            if TARGET_OS_MACCATALYST
+#                define ANGLE_PLATFORM_MACCATALYST 1
+#            endif
+#        elif TARGET_OS_WATCH
+#            define ANGLE_PLATFORM_WATCHOS 1
+#        elif TARGET_OS_TV
+#            define ANGLE_PLATFORM_APPLETV 1
 #        endif
 #    endif
+#    // Identify Metal API >= what shipped on macOS Catalina.
+#    if (ANGLE_PLATFORM_MACOS && __MAC_OS_X_VERSION_MAX_ALLOWED >= 101500) || \
+        (ANGLE_PLATFORM_IOS_FAMILY && __IPHONE_OS_VERSION_MAX_ALLOWED >= 130000)
+#        define ANGLE_WITH_MODERN_METAL_API 1
+#    endif
+#endif
+
+// Define ANGLE_WITH_ASAN macro.
+#if defined(__has_feature)
+#    if __has_feature(address_sanitizer)
+#        define ANGLE_WITH_ASAN 1
+#    endif
+#endif
+
+// Define ANGLE_WITH_MSAN macro.
+#if defined(__has_feature)
+#    if __has_feature(memory_sanitizer)
+#        define ANGLE_WITH_MSAN 1
+#    endif
+#endif
+
+// Define ANGLE_WITH_TSAN macro.
+#if defined(__has_feature)
+#    if __has_feature(thread_sanitizer)
+#        define ANGLE_WITH_TSAN 1
+#    endif
+#endif
+
+// Define ANGLE_WITH_UBSAN macro.
+#if defined(__has_feature)
+#    if __has_feature(undefined_behavior_sanitizer)
+#        define ANGLE_WITH_UBSAN 1
+#    endif
+#endif
+
+#if defined(ANGLE_WITH_ASAN) || defined(ANGLE_WITH_TSAN) || defined(ANGLE_WITH_UBSAN)
+#    define ANGLE_WITH_SANITIZER 1
+#endif  // defined(ANGLE_WITH_ASAN) || defined(ANGLE_WITH_TSAN) || defined(ANGLE_WITH_UBSAN)
+
+#include <stdint.h>
+#if INTPTR_MAX == INT64_MAX
+#    define ANGLE_IS_64_BIT_CPU 1
+#else
+#    define ANGLE_IS_32_BIT_CPU 1
 #endif
 
 #endif  // COMMON_PLATFORM_H_
