@@ -10,7 +10,11 @@
 #include "common/utilities.h"
 #include "compiler/translator/BuiltInFunctionEmulatorGLSL.h"
 #include "compiler/translator/OutputESSL.h"
+#include "compiler/translator/StaticType.h"
+#include "compiler/translator/tree_ops/DeclarePerVertexBlocks.h"
 #include "compiler/translator/tree_ops/RecordConstantPrecision.h"
+#include "compiler/translator/tree_util/ReplaceClipCullDistanceVariable.h"
+#include "compiler/translator/util.h"
 
 namespace sh
 {
@@ -35,10 +39,15 @@ bool TranslatorESSL::translate(TIntermBlock *root,
     TInfoSinkBase &sink = getInfoSink().obj;
 
     int shaderVer = getShaderVersion();  // Frontend shader version.
-    if (hasPixelLocalStorageUniforms() &&
-        ShPixelLocalStorageTypeUsesImages(compileOptions.pls.type))
+    if ((shaderVer > 100 &&
+         (getResources().EXT_clip_cull_distance || getResources().ANGLE_clip_cull_distance ||
+          getResources().NV_shader_noperspective_interpolation ||
+          getResources().OES_shader_multisample_interpolation)) ||
+        (hasPixelLocalStorageUniforms() &&
+         compileOptions.pls.type == ShPixelLocalStorageType::ImageLoadStore))
     {
-        // The backend translator emits shader image code. Use a minimum version of 310.
+        // The backend translator emits interface blocks or shader image code.
+        // Use a minimum version of 310.
         shaderVer = std::max(shaderVer, 310);
     }
     if (shaderVer > 100)
@@ -79,6 +88,57 @@ bool TranslatorESSL::translate(TIntermBlock *root,
         sink << "// END: Generated code for built-in function emulation\n\n";
     }
 
+    if (getShaderType() == GL_VERTEX_SHADER)
+    {
+        // Emulate GL_CLIP_DISTANCEi_EXT state if needed
+        if (hasClipDistance() && compileOptions.emulateClipDistanceState)
+        {
+            constexpr const ImmutableString kClipDistanceEnabledName("angle_ClipDistanceEnabled");
+
+            const TType *type = StaticType::Get<EbtUInt, EbpLow, EvqUniform, 1, 1>();
+            const TVariable *clipDistanceEnabled = new TVariable(
+                &getSymbolTable(), kClipDistanceEnabledName, type, SymbolType::AngleInternal);
+            const TIntermSymbol *clipDistanceEnabledSymbol = new TIntermSymbol(clipDistanceEnabled);
+
+            // AngleInternal variables don't get collected
+            if (shouldCollectVariables(compileOptions))
+            {
+                ShaderVariable uniform;
+                uniform.name          = kClipDistanceEnabledName.data();
+                uniform.mappedName    = kClipDistanceEnabledName.data();
+                uniform.type          = GLVariableType(*type);
+                uniform.precision     = GLVariablePrecision(*type);
+                uniform.staticUse     = true;
+                uniform.active        = true;
+                uniform.binding       = type->getLayoutQualifier().binding;
+                uniform.location      = type->getLayoutQualifier().location;
+                uniform.offset        = type->getLayoutQualifier().offset;
+                uniform.rasterOrdered = type->getLayoutQualifier().rasterOrdered;
+                uniform.readonly      = type->getMemoryQualifier().readonly;
+                uniform.writeonly     = type->getMemoryQualifier().writeonly;
+                mUniforms.push_back(uniform);
+            }
+            DeclareGlobalVariable(root, clipDistanceEnabled);
+            if (!ZeroDisabledClipDistanceAssignments(this, root, &getSymbolTable(), getShaderType(),
+                                                     clipDistanceEnabledSymbol))
+                return false;
+
+            // The previous operation always redeclares gl_ClipDistance
+            if (!DeclarePerVertexBlocks(this, root, &getSymbolTable()))
+                return false;
+        }
+        else if ((IsExtensionEnabled(getExtensionBehavior(), TExtension::EXT_clip_cull_distance) ||
+                  IsExtensionEnabled(getExtensionBehavior(),
+                                     TExtension::ANGLE_clip_cull_distance)) &&
+                 areClipDistanceOrCullDistanceRedeclared())
+        {
+            // When clip distance state emulation is not needed,
+            // the redeclared extension built-ins still should be moved to gl_PerVertex
+            if (!DeclarePerVertexBlocks(this, root, &getSymbolTable()))
+                return false;
+        }
+    }
+
     if (getShaderType() == GL_FRAGMENT_SHADER)
     {
         EmitEarlyFragmentTestsGLSL(*this, sink);
@@ -178,6 +238,18 @@ void TranslatorESSL::writeExtensionBehavior(const ShCompileOptions &compileOptio
                 ASSERT(compileOptions.emulateGLBaseVertexBaseInstance);
                 continue;
             }
+            else if (iter->first == TExtension::EXT_clip_cull_distance ||
+                     iter->first == TExtension::ANGLE_clip_cull_distance)
+            {
+                sink << "#extension GL_EXT_clip_cull_distance : " << GetBehaviorString(iter->second)
+                     << "\n";
+                if (areClipDistanceOrCullDistanceRedeclared() ||
+                    (hasClipDistance() && compileOptions.emulateClipDistanceState))
+                {
+                    sink << "#extension GL_EXT_shader_io_blocks : "
+                         << GetBehaviorString(iter->second) << "\n";
+                }
+            }
             else if (iter->first == TExtension::ANGLE_shader_pixel_local_storage)
             {
                 if (compileOptions.pls.type == ShPixelLocalStorageType::PixelLocalStorageEXT)
