@@ -223,7 +223,7 @@ GenMetalTraverser::GenMetalTraverser(const TCompiler &compiler,
       mMainUniformBufferIndex(compileOptions.metal.defaultUniformsBindingIndex),
       mDriverUniformsBindingIndex(compileOptions.metal.driverUniformsBindingIndex),
       mUBOArgumentBufferBindingIndex(compileOptions.metal.UBOArgumentBufferBindingIndex),
-      mRasterOrderGroupsSupported(compileOptions.pls.fragmentSynchronizationType ==
+      mRasterOrderGroupsSupported(compileOptions.pls.fragmentSyncType ==
                                   ShFragmentSynchronizationType::RasterOrderGroups_Metal)
 {}
 
@@ -517,6 +517,8 @@ static const char *GetOperatorString(TOperator op,
             return "metal::rint";
         case TOperator::EOpClamp:
             return "metal::clamp";  // TODO fast vs precise namespace
+        case TOperator::EOpSaturate:
+            return "metal::saturate";  // TODO fast vs precise namespace
         case TOperator::EOpMix:
             if (argType2 && argType2->getBasicType() == EbtBool)
                 return "ANGLE_mix_bool";
@@ -553,6 +555,15 @@ static const char *GetOperatorString(TOperator op,
         case TOperator::EOpInverse:
             return "ANGLE_inverse";
 
+        case TOperator::EOpInterpolateAtCentroid:
+            return "ANGLE_interpolateAtCentroid";
+        case TOperator::EOpInterpolateAtSample:
+            return "ANGLE_interpolateAtSample";
+        case TOperator::EOpInterpolateAtOffset:
+            return "ANGLE_interpolateAtOffset";
+        case TOperator::EOpInterpolateAtCenter:
+            return "ANGLE_interpolateAtCenter";
+
         case TOperator::EOpFloatBitsToInt:
         case TOperator::EOpFloatBitsToUint:
         case TOperator::EOpIntBitsToFloat:
@@ -644,6 +655,11 @@ static const char *GetOperatorString(TOperator op,
         case TOperator::EOpUnpackHalf2x16:
             return "ANGLE_unpack_half_2x16";
 
+        case TOperator::EOpNumSamples:
+            return "metal::get_num_samples";
+        case TOperator::EOpSamplePosition:
+            return "metal::get_sample_position";
+
         case TOperator::EOpBitfieldExtract:
         case TOperator::EOpBitfieldInsert:
         case TOperator::EOpBitfieldReverse:
@@ -658,7 +674,6 @@ static const char *GetOperatorString(TOperator op,
         case TOperator::EOpMemoryBarrier:
         case TOperator::EOpMemoryBarrierAtomicCounter:
         case TOperator::EOpMemoryBarrierBuffer:
-        case TOperator::EOpMemoryBarrierImage:
         case TOperator::EOpMemoryBarrierShared:
         case TOperator::EOpGroupMemoryBarrier:
         case TOperator::EOpAtomicAdd:
@@ -833,6 +848,20 @@ void GenMetalTraverser::emitPostQualifier(const EmitVariableDeclarationConfig &e
             }
             break;
 
+        case TQualifier::EvqSampleID:
+            if (evdConfig.isMainParameter)
+            {
+                mOut << " [[sample_id]]";
+            }
+            break;
+
+        case TQualifier::EvqSampleMaskIn:
+            if (evdConfig.isMainParameter)
+            {
+                mOut << " [[sample_mask]]";
+            }
+            break;
+
         default:
             break;
     }
@@ -941,6 +970,30 @@ void GenMetalTraverser::emitBareTypeName(const TType &type, const EmitTypeConfig
                     emitNameOf(env);
                 }
             }
+            else if (IsImage(basicType))
+            {
+                mOut << "metal::texture2d<";
+                switch (type.getBasicType())
+                {
+                    case EbtImage2D:
+                        mOut << "float";
+                        break;
+                    case EbtIImage2D:
+                        mOut << "int";
+                        break;
+                    case EbtUImage2D:
+                        mOut << "uint";
+                        break;
+                    default:
+                        UNIMPLEMENTED();
+                        break;
+                }
+                if (type.getMemoryQualifier().readonly || type.getMemoryQualifier().writeonly)
+                {
+                    UNIMPLEMENTED();
+                }
+                mOut << ", metal::access::read_write>";
+            }
             else
             {
                 UNIMPLEMENTED();
@@ -982,6 +1035,11 @@ void GenMetalTraverser::emitType(const TType &type, const EmitTypeConfig &etConf
         }
     }
 
+    if (type.isInterpolant())
+    {
+        mOut << "metal::interpolant<";
+    }
+
     if (type.isVector() || type.isMatrix())
     {
         mOut << "metal::";
@@ -1004,6 +1062,22 @@ void GenMetalTraverser::emitType(const TType &type, const EmitTypeConfig &etConf
              << static_cast<uint32_t>(type.getRows());
     }
 
+    if (type.isInterpolant())
+    {
+        mOut << ", metal::interpolation::";
+        switch (type.getQualifier())
+        {
+            case EvqNoPerspectiveIn:
+            case EvqNoPerspectiveCentroidIn:
+            case EvqNoPerspectiveSampleIn:
+                mOut << "no_";
+                break;
+            default:
+                break;
+        }
+        mOut << "perspective>";
+    }
+
     if (!isUBO)
     {
         if (type.isArray())
@@ -1070,6 +1144,50 @@ void GenMetalTraverser::emitFieldDeclaration(const TField &field,
             }
             break;
 
+        case TQualifier::EvqNoPerspectiveIn:
+            if (mPipelineStructs.fragmentIn.external == &parent && !type.isInterpolant())
+            {
+                mOut << " [[center_no_perspective]]";
+            }
+            break;
+
+        case TQualifier::EvqCentroidIn:
+            if (mPipelineStructs.fragmentIn.external == &parent && !type.isInterpolant())
+            {
+                mOut << " [[centroid_perspective]]";
+            }
+            break;
+
+        case TQualifier::EvqSampleIn:
+            if (mPipelineStructs.fragmentIn.external == &parent && !type.isInterpolant())
+            {
+                mOut << " [[sample_perspective]]";
+            }
+            break;
+
+        case TQualifier::EvqNoPerspectiveCentroidIn:
+            if (mPipelineStructs.fragmentIn.external == &parent && !type.isInterpolant())
+            {
+                mOut << " [[centroid_no_perspective]]";
+            }
+            break;
+
+        case TQualifier::EvqNoPerspectiveSampleIn:
+            if (mPipelineStructs.fragmentIn.external == &parent && !type.isInterpolant())
+            {
+                mOut << " [[sample_no_perspective]]";
+            }
+            break;
+
+        case TQualifier::EvqFragColor:
+            mOut << " [[color(0)]]";
+            break;
+
+        case TQualifier::EvqSecondaryFragColorEXT:
+        case TQualifier::EvqSecondaryFragDataEXT:
+            mOut << " [[color(0), index(1)]]";
+            break;
+
         case TQualifier::EvqFragmentOut:
         case TQualifier::EvqFragmentInOut:
         case TQualifier::EvqFragData:
@@ -1078,7 +1196,7 @@ void GenMetalTraverser::emitFieldDeclaration(const TField &field,
                 if ((type.isVector() &&
                      (basic == TBasicType::EbtInt || basic == TBasicType::EbtUInt ||
                       basic == TBasicType::EbtFloat)) ||
-                    type.getQualifier() == EvqFragData)
+                    qual == EvqFragData)
                 {
                     // The OpenGL ES 3.0 spec says locations must be specified
                     // unless there is only a single output, in which case the
@@ -1086,9 +1204,25 @@ void GenMetalTraverser::emitFieldDeclaration(const TField &field,
                     // will have been rejected if locations are not specified
                     // and there is more than one output.
                     const TLayoutQualifier &layoutQualifier = type.getLayoutQualifier();
-                    size_t index = layoutQualifier.locationsSpecified ? layoutQualifier.location
-                                                                      : annotationIndices.color++;
-                    mOut << " [[color(" << index << ")";
+                    if (layoutQualifier.locationsSpecified)
+                    {
+                        mOut << " [[color(" << layoutQualifier.location << ")";
+                        ASSERT(layoutQualifier.index >= -1 && layoutQualifier.index <= 1);
+                        if (layoutQualifier.index == 1)
+                        {
+                            mOut << ", index(1)";
+                        }
+                    }
+                    else if (qual == EvqFragData)
+                    {
+                        mOut << " [[color(" << annotationIndices.color++ << ")";
+                    }
+                    else
+                    {
+                        // Either the only output or EXT_blend_func_extended is used;
+                        // actual assignment will happen in UpdateFragmentShaderOutputs.
+                        mOut << " [[" << sh::kUnassignedFragmentOutputString;
+                    }
                     if (mRasterOrderGroupsSupported && qual == TQualifier::EvqFragmentInOut)
                     {
                         // Put fragment inouts in their own raster order group for better
@@ -1104,18 +1238,51 @@ void GenMetalTraverser::emitFieldDeclaration(const TField &field,
             break;
 
         case TQualifier::EvqFragDepth:
-            mOut << " [[depth(any), function_constant(" << sh::mtl::kDepthWriteEnabledConstName
-                 << ")]]";
+            mOut << " [[depth(";
+            switch (type.getLayoutQualifier().depth)
+            {
+                case EdGreater:
+                    mOut << "greater";
+                    break;
+                case EdLess:
+                    mOut << "less";
+                    break;
+                default:
+                    mOut << "any";
+                    break;
+            }
+            mOut << "), function_constant(" << sh::mtl::kDepthWriteEnabledConstName << ")]]";
             break;
 
         case TQualifier::EvqSampleMask:
-            mOut << " [[sample_mask, function_constant(" << sh::mtl::kCoverageMaskEnabledConstName
-                 << ")]]";
+            if (field.symbolType() == SymbolType::AngleInternal)
+            {
+                mOut << " [[sample_mask, function_constant("
+                     << sh::mtl::kMultisampledRenderingConstName << ")]]";
+            }
             break;
 
         default:
             break;
     }
+
+    if (IsImage(type.getBasicType()))
+    {
+        if (type.getArraySizeProduct() != 1)
+        {
+            UNIMPLEMENTED();
+        }
+        mOut << " [[";
+        if (type.getLayoutQualifier().rasterOrdered)
+        {
+            mOut << "raster_order_group(0), ";
+        }
+        mOut << "texture(" << mMainTextureIndex << ")]]";
+        TranslatorMetalReflection *reflection = mtl::getTranslatorMetalReflection(&mCompiler);
+        reflection->addRWTextureBinding(type.getLayoutQualifier().binding,
+                                        static_cast<int>(mMainTextureIndex));
+        ++mMainTextureIndex;
+    }
 }
 
 static std::map<Name, size_t> BuildExternalAttributeIndexMap(
@@ -1161,13 +1328,15 @@ static std::map<Name, size_t> BuildExternalAttributeIndexMap(
             ++attributeIndex;  // TODO: Might need to increment more if shader var type is a matrix.
         }
 
-        const size_t cols = internalType.isMatrix() ? internalType.getCols() : 1;
+        const size_t cols =
+            (internalType.isMatrix() && !externalFields[externalIndex]->type()->isMatrix())
+                ? internalType.getCols()
+                : 1;
 
         for (size_t c = 0; c < cols; ++c)
         {
             const TField &externalField = *externalFields[externalIndex];
             const Name externalName     = Name(externalField);
-            ASSERT(!externalField.type()->isMatrix());
 
             externalNameToAttributeIndex[externalName] = attributeIndex;
 
@@ -1280,21 +1449,6 @@ void GenMetalTraverser::emitStructDeclaration(const TType &type)
         mOut << ";\n";
     }
 
-    if (!mPipelineStructs.matches(structure, true, true))
-    {
-        MetalLayoutOfConfig layoutConfig;
-        layoutConfig.treatSamplersAsTextureEnv = true;
-        Layout layout                          = MetalLayoutOf(type, layoutConfig);
-        size_t pad = (kDefaultStructAlignmentSize - layout.sizeOf) % kDefaultStructAlignmentSize;
-        if (pad != 0)
-        {
-            emitIndentation();
-            mOut << "char ";
-            EmitName(mOut, mIdGen.createNewName("pad"));
-            mOut << "[" << pad << "];\n";
-        }
-    }
-
     emitCloseBrace();
 }
 
@@ -1313,6 +1467,12 @@ void GenMetalTraverser::emitOrdinaryVariableDeclaration(
         ASSERT(type.getBasicType() == TBasicType::EbtFloat);
         mOut << "float";
     }
+    else if (type.getQualifier() == TQualifier::EvqSampleID && evdConfig.isMainParameter)
+    {
+        // Metal's [[sample_id]] must be unsigned
+        ASSERT(type.getBasicType() == TBasicType::EbtInt);
+        mOut << "uint32_t";
+    }
     else
     {
         emitType(type, etConfig);
@@ -1815,13 +1975,17 @@ void GenMetalTraverser::emitFunctionReturn(const TFunction &func)
     if (isMain)
     {
         const TStructure *structure = returnType.getStruct();
-        ASSERT(structure != nullptr);
         if (mPipelineStructs.fragmentOut.matches(*structure))
         {
+            if (mCompiler.isEarlyFragmentTestsSpecified())
+            {
+                mOut << "[[early_fragment_tests]]\n";
+            }
             mOut << "fragment ";
         }
         else if (mPipelineStructs.vertexOut.matches(*structure))
         {
+            ASSERT(structure != nullptr);
             mOut << "vertex __VERTEX_OUT(";
             isVertexMain = true;
         }
@@ -1891,7 +2055,7 @@ void GenMetalTraverser::emitFunctionParameter(const TFunction &func, const TVari
         {
             mOut << " [[texture(" << (mMainTextureIndex) << ")]]";
             const std::string originalName = reflection->getOriginalName(param.uniqueId().get());
-            reflection->addTextureBinding(originalName, mMainSamplerIndex);
+            reflection->addTextureBinding(originalName, mMainTextureIndex);
             mMainTextureIndex += type.getArraySizeProduct();
         }
         else if (Name(param) == Pipeline{Pipeline::Type::InstanceId, nullptr}.getStructInstanceName(
@@ -1922,7 +2086,8 @@ bool GenMetalTraverser::visitFunctionDefinition(Visit, TIntermFunctionDefinition
     {
         const TType &returnType     = func.getReturnType();
         const TStructure *structure = returnType.getStruct();
-        isTraversingVertexMain      = (mPipelineStructs.vertexOut.matches(*structure));
+        isTraversingVertexMain      = (mPipelineStructs.vertexOut.matches(*structure)) &&
+                                 mCompiler.getShaderType() == GL_VERTEX_SHADER;
     }
     emitIndentation();
     emitFunctionSignature(func);
@@ -1981,6 +2146,9 @@ GenMetalTraverser::FuncToName GenMetalTraverser::BuildFuncToName()
     putAngle("textureProjLodOffset");
     putAngle("textureProjOffset");
     putAngle("textureSize");
+    putAngle("imageLoad");
+    putAngle("imageStore");
+    putAngle("memoryBarrierImage");
 
     return map;
 }
@@ -2087,11 +2255,9 @@ bool GenMetalTraverser::visitAggregate(Visit, TIntermAggregate *aggregateNode)
                     return nullptr;
                 };
 
-                ASSERT(!args.empty());
                 const TType *argType0 = getArgType(0);
                 const TType *argType1 = getArgType(1);
                 const TType *argType2 = getArgType(2);
-                ASSERT(argType0);
 
                 const char *opName = GetOperatorString(op, retType, argType0, argType1, argType2);
 
