@@ -78,8 +78,6 @@ struct SystemInfo
     bool isAMDSwitchable = false;
     // Only true on dual-GPU Mac laptops.
     bool isMacSwitchable = false;
-    // Only true on Apple Silicon Macs when running in macCatalyst.
-    bool needsEAGLOnMac = false;
 
     // Only available on Android
     std::string machineManufacturer;
@@ -109,6 +107,7 @@ constexpr VendorID kVendorID_ImgTec    = 0x1010;
 constexpr VendorID kVendorID_Intel     = 0x8086;
 constexpr VendorID kVendorID_NVIDIA    = 0x10DE;
 constexpr VendorID kVendorID_Qualcomm  = 0x5143;
+constexpr VendorID kVendorID_Samsung   = 0x144D;
 constexpr VendorID kVendorID_VMWare    = 0x15ad;
 constexpr VendorID kVendorID_Apple     = 0x106B;
 constexpr VendorID kVendorID_Microsoft = 0x1414;
@@ -136,6 +135,7 @@ bool IsIntel(VendorID vendorId);
 bool IsKazan(VendorID vendorId);
 bool IsNVIDIA(VendorID vendorId);
 bool IsQualcomm(VendorID vendorId);
+bool IsSamsung(VendorID vendorId);
 bool IsGoogle(VendorID vendorId);
 bool IsSwiftshader(VendorID vendorId);
 bool IsVeriSilicon(VendorID vendorId);
@@ -159,17 +159,6 @@ void PrintSystemInfo(const SystemInfo &info);
 VersionInfo ParseNvidiaDriverVersion(uint32_t version);
 VersionInfo ParseMesaDriverVersion(uint32_t version);
 
-#if defined(ANGLE_PLATFORM_MACOS) || defined(ANGLE_PLATFORM_MACCATALYST)
-// Helper to get the active GPU ID from a given Core Graphics display ID.
-uint64_t GetGpuIDFromDisplayID(uint32_t displayID);
-
-// Helper to get the active GPU ID from an OpenGL display mask.
-uint64_t GetGpuIDFromOpenGLDisplayMask(uint32_t displayMask);
-
-// Get VendorID from metal device's registry ID
-VendorID GetVendorIDFromMetalDeviceRegistryID(uint64_t registryID);
-#endif
-
 uint64_t GetSystemDeviceIdFromParts(uint32_t highPart, uint32_t lowPart);
 uint32_t GetSystemDeviceIdHighPart(uint64_t systemDeviceId);
 uint32_t GetSystemDeviceIdLowPart(uint64_t systemDeviceId);
