@@ -650,8 +650,7 @@ static bool DetermineDepthTextureANGLESupport(const TextureCapsMap &textureCaps)
 {
     constexpr GLenum requiredFormats[] = {
         GL_DEPTH_COMPONENT16,
-#if !defined(ANGLE_PLATFORM_IOS) && \
-    (!defined(ANGLE_PLATFORM_MACCATALYST) || !defined(ANGLE_CPU_ARM64))
+#if !ANGLE_PLATFORM_IOS_FAMILY
         // anglebug.com/6082
         // TODO(dino): Temporarily Removing the need for GL_DEPTH_COMPONENT32_OES
         // because it is not supported on iOS.
@@ -669,8 +668,7 @@ static bool DetermineDepthTextureOESSupport(const TextureCapsMap &textureCaps)
 {
     constexpr GLenum requiredFormats[] = {
         GL_DEPTH_COMPONENT16,
-#if !defined(ANGLE_PLATFORM_IOS) && \
-    (!defined(ANGLE_PLATFORM_MACCATALYST) || !defined(ANGLE_CPU_ARM64))
+#if !ANGLE_PLATFORM_IOS_FAMILY
         // anglebug.com/6082
         // TODO(dino): Temporarily Removing the need for GL_DEPTH_COMPONENT32_OES
         // because it is not supported on iOS.
@@ -835,7 +833,40 @@ static bool DetermineStencilIndex8Support(const TextureCapsMap &textureCaps)
         GL_STENCIL_INDEX8,
     };
 
-    return GetFormatSupport(textureCaps, requiredFormats, false, false, true, false, false);
+    return GetFormatSupport(textureCaps, requiredFormats, true, false, true, false, false);
+}
+
+// Checks for GL_QCOM_render_shared_exponent support
+static bool DetermineRenderSharedExponentSupport(const TextureCapsMap &textureCaps)
+{
+    constexpr GLenum requiredFormats[] = {
+        GL_RGB9_E5,
+    };
+
+    return GetFormatSupport(textureCaps, requiredFormats, false, false, true, true, true);
+}
+
+static bool DetermineRenderSnormSupport(const TextureCapsMap &textureCaps, bool textureNorm16EXT)
+{
+    constexpr GLenum requiredSnorm8Formats[] = {
+        GL_R8_SNORM,
+        GL_RG8_SNORM,
+        GL_RGBA8_SNORM,
+    };
+
+    constexpr GLenum requiredSnorm16Formats[] = {
+        GL_R16_SNORM_EXT,
+        GL_RG16_SNORM_EXT,
+        GL_RGBA16_SNORM_EXT,
+    };
+
+    if (textureNorm16EXT &&
+        !GetFormatSupport(textureCaps, requiredSnorm16Formats, false, false, true, true, true))
+    {
+        return false;
+    }
+
+    return GetFormatSupport(textureCaps, requiredSnorm8Formats, false, false, true, true, true);
 }
 
 void Extensions::setTextureExtensionSupport(const TextureCapsMap &textureCaps)
@@ -896,6 +927,8 @@ void Extensions::setTextureExtensionSupport(const TextureCapsMap &textureCaps)
     textureCompressionPvrtcIMG          = DeterminePVRTCTextureSupport(textureCaps);
     pvrtcSRGBEXT                        = DeterminePVRTCsRGBTextureSupport(textureCaps);
     textureStencil8OES                  = DetermineStencilIndex8Support(textureCaps);
+    renderSharedExponentQCOM            = DetermineRenderSharedExponentSupport(textureCaps);
+    renderSnormEXT = DetermineRenderSnormSupport(textureCaps, textureNorm16EXT);
 }
 
 TypePrecision::TypePrecision() = default;
@@ -1280,6 +1313,7 @@ std::vector<std::string> DisplayExtensions::getStrings() const
     InsertExtensionString("EGL_ANGLE_timestamp_surface_attribute",               timestampSurfaceAttributeANGLE,     &extensionStrings);
     InsertExtensionString("EGL_ANDROID_recordable",                              recordable,                         &extensionStrings);
     InsertExtensionString("EGL_ANGLE_power_preference",                          powerPreference,                    &extensionStrings);
+    InsertExtensionString("EGL_ANGLE_wait_until_work_scheduled",                 waitUntilWorkScheduled,             &extensionStrings);
     InsertExtensionString("EGL_ANGLE_image_d3d11_texture",                       imageD3D11Texture,                  &extensionStrings);
     InsertExtensionString("EGL_ANDROID_create_native_client_buffer",             createNativeClientBufferANDROID,    &extensionStrings);
     InsertExtensionString("EGL_ANDROID_get_native_client_buffer",                getNativeClientBufferANDROID,       &extensionStrings);
