@@ -138,9 +138,9 @@ bool ImageSibling::isYUV() const
     return mTargetOf.get() && mTargetOf->isYUV();
 }
 
-bool ImageSibling::isCreatedWithAHB() const
+bool ImageSibling::isExternalImageWithoutIndividualSync() const
 {
-    return mTargetOf.get() && mTargetOf->isCreatedWithAHB();
+    return mTargetOf.get() && mTargetOf->isExternalImageWithoutIndividualSync();
 }
 
 bool ImageSibling::hasFrontBufferUsage() const
@@ -183,7 +183,7 @@ void ExternalImageSibling::onDestroy(const egl::Display *display)
     mImplementation->onDestroy(display);
 }
 
-Error ExternalImageSibling::initialize(const egl::Display *display)
+Error ExternalImageSibling::initialize(const egl::Display *display, const gl::Context *context)
 {
     return mImplementation->initialize(display);
 }
@@ -295,7 +295,6 @@ ImageState::ImageState(ImageID id,
       size(),
       samples(),
       levelCount(1),
-      sourceType(target),
       colorspace(
           static_cast<EGLenum>(attribs.get(EGL_GL_COLORSPACE, EGL_GL_COLORSPACE_DEFAULT_EXT))),
       hasProtectedContent(static_cast<bool>(attribs.get(EGL_PROTECTED_CONTENT_EXT, EGL_FALSE)))
@@ -337,7 +336,7 @@ void Image::onDestroy(const Display *display)
         mState.source->removeImageSource(this);
 
         // If the source is an external object, delete it
-        if (IsExternalImageTarget(mState.sourceType))
+        if (IsExternalImageTarget(mState.target))
         {
             ExternalImageSibling *externalSibling = rx::GetAs<ExternalImageSibling>(mState.source);
             externalSibling->onDestroy(display);
@@ -379,7 +378,7 @@ angle::Result Image::orphanSibling(const gl::Context *context, ImageSibling *sib
     if (mState.source == sibling)
     {
         // The external source of an image cannot be redefined so it cannot be orphaned.
-        ASSERT(!IsExternalImageTarget(mState.sourceType));
+        ASSERT(!IsExternalImageTarget(mState.target));
 
         // If the sibling is the source, it cannot be a target.
         ASSERT([&] {
@@ -406,45 +405,12 @@ const gl::Format &Image::getFormat() const
 
 bool Image::isRenderable(const gl::Context *context) const
 {
-    if (IsTextureTarget(mState.sourceType))
-    {
-        return mState.format.info->textureAttachmentSupport(context->getClientVersion(),
-                                                            context->getExtensions());
-    }
-    else if (IsRenderbufferTarget(mState.sourceType))
-    {
-        return mState.format.info->renderbufferSupport(context->getClientVersion(),
-                                                       context->getExtensions());
-    }
-    else if (IsExternalImageTarget(mState.sourceType))
-    {
-        ASSERT(mState.source != nullptr);
-        return mState.source->isRenderable(context, GL_NONE, gl::ImageIndex());
-    }
-
-    UNREACHABLE();
-    return false;
+    return mIsRenderable;
 }
 
 bool Image::isTexturable(const gl::Context *context) const
 {
-    if (IsTextureTarget(mState.sourceType))
-    {
-        return mState.format.info->textureSupport(context->getClientVersion(),
-                                                  context->getExtensions());
-    }
-    else if (IsRenderbufferTarget(mState.sourceType))
-    {
-        return true;
-    }
-    else if (IsExternalImageTarget(mState.sourceType))
-    {
-        ASSERT(mState.source != nullptr);
-        return rx::GetAs<ExternalImageSibling>(mState.source)->isTextureable(context);
-    }
-
-    UNREACHABLE();
-    return false;
+    return mIsTexturable;
 }
 
 bool Image::isYUV() const
@@ -452,9 +418,10 @@ bool Image::isYUV() const
     return mState.yuv;
 }
 
-bool Image::isCreatedWithAHB() const
+bool Image::isExternalImageWithoutIndividualSync() const
 {
-    return mState.target == EGL_NATIVE_BUFFER_ANDROID;
+    // Only Vulkan images are individually synced.
+    return IsExternalImageTarget(mState.target) && mState.target != EGL_VULKAN_IMAGE_ANGLE;
 }
 
 bool Image::hasFrontBufferUsage() const
@@ -513,12 +480,12 @@ rx::ImageImpl *Image::getImplementation() const
     return mImplementation;
 }
 
-Error Image::initialize(const Display *display)
+Error Image::initialize(const Display *display, const gl::Context *context)
 {
-    if (IsExternalImageTarget(mState.sourceType))
+    if (IsExternalImageTarget(mState.target))
     {
         ExternalImageSibling *externalSibling = rx::GetAs<ExternalImageSibling>(mState.source);
-        ANGLE_TRY(externalSibling->initialize(display));
+        ANGLE_TRY(externalSibling->initialize(display, context));
 
         mState.hasProtectedContent = externalSibling->hasProtectedContent();
         mState.levelCount          = externalSibling->getLevelCount();
@@ -541,7 +508,7 @@ Error Image::initialize(const Display *display)
         mState.format = gl::Format(nonLinearFormat);
     }
 
-    if (!IsExternalImageTarget(mState.sourceType))
+    if (!IsExternalImageTarget(mState.target))
     {
         // Account for the fact that GL_ANGLE_yuv_internal_format extension maybe enabled,
         // in which case the internal format itself could be YUV.
@@ -551,12 +518,42 @@ Error Image::initialize(const Display *display)
     mState.size    = mState.source->getAttachmentSize(mState.imageIndex);
     mState.samples = mState.source->getAttachmentSamples(mState.imageIndex);
 
-    if (IsTextureTarget(mState.sourceType))
+    if (IsTextureTarget(mState.target))
     {
         mState.size.depth = 1;
     }
 
-    return mImplementation->initialize(display);
+    Error error = mImplementation->initialize(display);
+    if (error.isError())
+    {
+        return error;
+    }
+
+    if (IsTextureTarget(mState.target))
+    {
+        mIsTexturable = true;
+        mIsRenderable = mState.format.info->textureAttachmentSupport(context->getClientVersion(),
+                                                                     context->getExtensions());
+    }
+    else if (IsRenderbufferTarget(mState.target))
+    {
+        mIsTexturable = true;
+        mIsRenderable = mState.format.info->renderbufferSupport(context->getClientVersion(),
+                                                                context->getExtensions());
+    }
+    else if (IsExternalImageTarget(mState.target))
+    {
+        ASSERT(mState.source != nullptr);
+        mIsTexturable = rx::GetAs<ExternalImageSibling>(mState.source)->isTextureable(context);
+        mIsRenderable = rx::GetAs<ExternalImageSibling>(mState.source)
+                            ->isRenderable(context, GL_NONE, gl::ImageIndex());
+    }
+    else
+    {
+        UNREACHABLE();
+    }
+
+    return NoError();
 }
 
 bool Image::orphaned() const
