@@ -458,6 +458,20 @@ class FlattenUniformVisitor : public sh::VariableNameVisitor
             {
                 linkedUniform.setParentArrayIndex(variable.parentArrayIndex());
             }
+
+            std::vector<unsigned int> arrayDims = arraySizes;
+            ASSERT(variable.arraySizes.size() == 1 || variable.arraySizes.size() == 0);
+            arrayDims.push_back(variable.arraySizes.empty() ? 1 : variable.arraySizes[0]);
+
+            size_t numDimensions = arraySizes.size();
+            uint32_t arrayStride = 1;
+            for (size_t dimension = numDimensions; dimension > 0;)
+            {
+                --dimension;
+                arrayStride *= arrayDims[dimension + 1];
+                linkedUniform.outerArrayOffset += arrayStride * mArrayElementStack[dimension];
+            }
+
             if (mMarkActive)
             {
                 linkedUniform.setActive(mShaderType, true);
@@ -505,6 +519,18 @@ class FlattenUniformVisitor : public sh::VariableNameVisitor
         sh::VariableNameVisitor::exitStructAccess(structVar, isRowMajor);
     }
 
+    void enterArrayElement(const sh::ShaderVariable &arrayVar, unsigned int arrayElement) override
+    {
+        mArrayElementStack.push_back(arrayElement);
+        sh::VariableNameVisitor::enterArrayElement(arrayVar, arrayElement);
+    }
+
+    void exitArrayElement(const sh::ShaderVariable &arrayVar, unsigned int arrayElement) override
+    {
+        mArrayElementStack.pop_back();
+        sh::VariableNameVisitor::exitArrayElement(arrayVar, arrayElement);
+    }
+
     ShaderUniformCount getCounts() const { return mUniformCount; }
 
   private:
@@ -525,6 +551,7 @@ class FlattenUniformVisitor : public sh::VariableNameVisitor
     std::vector<LinkedUniform> *mAtomicCounterUniforms;
     std::vector<LinkedUniform> *mInputAttachmentUniforms;
     std::vector<UnusedUniform> *mUnusedUniforms;
+    std::vector<unsigned int> mArrayElementStack;
     ShaderUniformCount mUniformCount;
     unsigned int mStructStackSize = 0;
 };
@@ -1347,17 +1374,17 @@ void InterfaceBlockLinker::defineInterfaceBlock(const GetBlockSizeFunc &getBlock
     size_t blockSize = 0;
     std::vector<unsigned int> blockIndexes;
 
-    int blockIndex = static_cast<int>(mBlocksOut->size());
+    const int blockIndex = static_cast<int>(mBlocksOut->size());
     // Track the first and last block member index to determine the range of active block members in
     // the block.
-    size_t firstBlockMemberIndex = getCurrentBlockMemberIndex();
+    const size_t firstBlockMemberIndex = getCurrentBlockMemberIndex();
 
     std::unique_ptr<sh::ShaderVariableVisitor> visitor(
         getVisitor(getMemberInfo, interfaceBlock.fieldPrefix(), interfaceBlock.fieldMappedPrefix(),
                    shaderType, blockIndex));
     sh::TraverseShaderVariables(interfaceBlock.fields, false, visitor.get());
 
-    size_t lastBlockMemberIndex = getCurrentBlockMemberIndex();
+    const size_t lastBlockMemberIndex = getCurrentBlockMemberIndex();
 
     for (size_t blockMemberIndex = firstBlockMemberIndex; blockMemberIndex < lastBlockMemberIndex;
          ++blockMemberIndex)
@@ -1365,7 +1392,7 @@ void InterfaceBlockLinker::defineInterfaceBlock(const GetBlockSizeFunc &getBlock
         blockIndexes.push_back(static_cast<unsigned int>(blockMemberIndex));
     }
 
-    unsigned int firstFieldArraySize = interfaceBlock.fields[0].getArraySizeProduct();
+    const unsigned int firstFieldArraySize = interfaceBlock.fields[0].getArraySizeProduct();
 
     for (unsigned int arrayElement = 0; arrayElement < interfaceBlock.elementCount();
          ++arrayElement)
@@ -1387,11 +1414,11 @@ void InterfaceBlockLinker::defineInterfaceBlock(const GetBlockSizeFunc &getBlock
         // ESSL 3.10 section 4.4.4 page 58:
         // Any uniform or shader storage block declared without a binding qualifier is initially
         // assigned to block binding point zero.
-        int blockBinding =
+        const int blockBinding =
             (interfaceBlock.binding == -1 ? 0 : interfaceBlock.binding + arrayElement);
         InterfaceBlock block(interfaceBlock.name, interfaceBlock.mappedName,
-                             interfaceBlock.isArray(), arrayElement, firstFieldArraySize,
-                             blockBinding);
+                             interfaceBlock.isArray(), interfaceBlock.isReadOnly, arrayElement,
+                             firstFieldArraySize, blockBinding);
         block.memberIndexes = blockIndexes;
         block.setActive(shaderType, interfaceBlock.active);
 
@@ -1485,17 +1512,17 @@ ProgramLinkedResources::ProgramLinkedResources() = default;
 
 ProgramLinkedResources::~ProgramLinkedResources() = default;
 
-LinkingVariables::LinkingVariables(const ProgramState &state)
+LinkingVariables::LinkingVariables(const Context *context, const ProgramState &state)
 {
     for (ShaderType shaderType : kAllGraphicsShaderTypes)
     {
         Shader *shader = state.getAttachedShader(shaderType);
         if (shader)
         {
-            outputVaryings[shaderType] = shader->getOutputVaryings();
-            inputVaryings[shaderType]  = shader->getInputVaryings();
-            uniforms[shaderType]       = shader->getUniforms();
-            uniformBlocks[shaderType]  = shader->getUniformBlocks();
+            outputVaryings[shaderType] = shader->getOutputVaryings(context);
+            inputVaryings[shaderType]  = shader->getInputVaryings(context);
+            uniforms[shaderType]       = shader->getUniforms(context);
+            uniformBlocks[shaderType]  = shader->getUniformBlocks(context);
             isShaderStageUsedBitset.set(shaderType);
         }
     }
@@ -1530,7 +1557,8 @@ void ProgramLinkedResources::init(std::vector<InterfaceBlock> *uniformBlocksOut,
     atomicCounterBufferLinker.init(atomicCounterBuffersOut);
 }
 
-void ProgramLinkedResourcesLinker::linkResources(const ProgramState &programState,
+void ProgramLinkedResourcesLinker::linkResources(const Context *context,
+                                                 const ProgramState &programState,
                                                  const ProgramLinkedResources &resources) const
 {
     // Gather uniform interface block info.
@@ -1540,7 +1568,7 @@ void ProgramLinkedResourcesLinker::linkResources(const ProgramState &programStat
         Shader *shader = programState.getAttachedShader(shaderType);
         if (shader)
         {
-            uniformBlockInfo.getShaderBlockInfo(shader->getUniformBlocks());
+            uniformBlockInfo.getShaderBlockInfo(shader->getUniformBlocks(context));
         }
     }
 
@@ -1565,7 +1593,7 @@ void ProgramLinkedResourcesLinker::linkResources(const ProgramState &programStat
         Shader *shader = programState.getAttachedShader(shaderType);
         if (shader)
         {
-            shaderStorageBlockInfo.getShaderBlockInfo(shader->getShaderStorageBlocks());
+            shaderStorageBlockInfo.getShaderBlockInfo(shader->getShaderStorageBlocks(context));
         }
     }
     auto getShaderStorageBlockSize = [&shaderStorageBlockInfo](const std::string &name,
@@ -2017,7 +2045,9 @@ bool LinkValidateBuiltInVaryings(const std::vector<sh::ShaderVariable> &outputVa
             if (sizeClipDistance != varying.getOutermostArraySize())
             {
                 infoLog << "If either shader redeclares the built-in arrays gl_ClipDistance[] the "
-                           "array must have the same size in both shaders.";
+                           "array must have the same size in both shaders. "
+                        << "Output size " << sizeClipDistance << ", input size "
+                        << varying.getOutermostArraySize() << ".";
                 return false;
             }
         }
@@ -2026,7 +2056,10 @@ bool LinkValidateBuiltInVaryings(const std::vector<sh::ShaderVariable> &outputVa
             if (sizeCullDistance != varying.getOutermostArraySize())
             {
                 infoLog << "If either shader redeclares the built-in arrays gl_CullDistance[] the "
-                           "array must have the same size in both shaders.";
+                           "array must have the same size in both shaders. "
+                        << "Output size " << sizeCullDistance << ", input size "
+                        << varying.getOutermostArraySize() << ".";
+
                 return false;
             }
         }
