@@ -172,6 +172,10 @@ class State : angle::NonCopyable
     void setCullMode(CullFaceMode mode);
     void setFrontFace(GLenum front);
 
+    // EXT_depth_clamp
+    bool isDepthClampEnabled() const { return mRasterizer.depthClamp; }
+    void setDepthClamp(bool enabled);
+
     // Depth test state manipulation
     bool isDepthTestEnabled() const { return mDepthStencil.depthTest; }
     bool isDepthWriteEnabled() const { return mDepthStencil.depthTest && mDepthStencil.depthMask; }
@@ -181,14 +185,11 @@ class State : angle::NonCopyable
     float getNearPlane() const { return mNearZ; }
     float getFarPlane() const { return mFarZ; }
 
-    // Clip control extension
-    void setClipControl(GLenum origin, GLenum depth);
-    bool isClipControlDepthZeroToOne() const { return mClipControlDepth == GL_ZERO_TO_ONE_EXT; }
-    gl::ClipSpaceOrigin getClipSpaceOrigin() const
-    {
-        return mClipControlOrigin == GL_UPPER_LEFT_EXT ? ClipSpaceOrigin::UpperLeft
-                                                       : ClipSpaceOrigin::LowerLeft;
-    }
+    // EXT_clip_control
+    void setClipControl(ClipOrigin origin, ClipDepthMode depth);
+    ClipOrigin getClipOrigin() const { return mClipOrigin; }
+    ClipDepthMode getClipDepthMode() const { return mClipDepthMode; }
+    bool isClipDepthModeZeroToOne() const { return mClipDepthMode == ClipDepthMode::ZeroToOne; }
 
     // Blend state manipulation
     bool isBlendEnabled() const { return mBlendStateExt.getEnabledMask().test(0); }
@@ -213,6 +214,11 @@ class State : angle::NonCopyable
 
     // Stencil state maniupulation
     bool isStencilTestEnabled() const { return mDepthStencil.stencilTest; }
+    bool isStencilWriteEnabled() const
+    {
+        return mDepthStencil.stencilTest &&
+               !(mDepthStencil.isStencilNoOp() && mDepthStencil.isStencilBackNoOp());
+    }
     void setStencilTest(bool enabled);
     void setStencilParams(GLenum stencilFunc, GLint stencilRef, GLuint stencilMask);
     void setStencilBackParams(GLenum stencilBackFunc, GLint stencilBackRef, GLuint stencilBackMask);
@@ -230,7 +236,7 @@ class State : angle::NonCopyable
     // Depth bias/polygon offset state manipulation
     bool isPolygonOffsetFillEnabled() const { return mRasterizer.polygonOffsetFill; }
     void setPolygonOffsetFill(bool enabled);
-    void setPolygonOffsetParams(GLfloat factor, GLfloat units);
+    void setPolygonOffsetParams(GLfloat factor, GLfloat units, GLfloat clamp);
 
     // Multisample coverage state manipulation
     bool isSampleAlphaToCoverageEnabled() const { return mSampleAlphaToCoverage; }
@@ -484,13 +490,13 @@ class State : angle::NonCopyable
     {
         return mBoundUniformBuffersMask;
     }
-    const angle::BitSet<gl::IMPLEMENTATION_MAX_ATOMIC_COUNTER_BUFFER_BINDINGS>
-        &getAtomicCounterBuffersMask() const
+    const angle::BitSet<gl::IMPLEMENTATION_MAX_ATOMIC_COUNTER_BUFFER_BINDINGS> &
+    getAtomicCounterBuffersMask() const
     {
         return mBoundAtomicCounterBuffersMask;
     }
-    const angle::BitSet<gl::IMPLEMENTATION_MAX_SHADER_STORAGE_BUFFER_BINDINGS>
-        &getShaderStorageBuffersMask() const
+    const angle::BitSet<gl::IMPLEMENTATION_MAX_SHADER_STORAGE_BUFFER_BINDINGS> &
+    getShaderStorageBuffersMask() const
     {
         return mBoundShaderStorageBuffersMask;
     }
@@ -711,7 +717,7 @@ class State : angle::NonCopyable
         DIRTY_BIT_SAMPLE_SHADING,
         DIRTY_BIT_PATCH_VERTICES,
         DIRTY_BIT_EXTENDED,  // clip distances, mipmap generation hint, derivative hint,
-                             // EXT_clip_control
+                             // EXT_clip_control, EXT_depth_clamp
         DIRTY_BIT_INVALID,
         DIRTY_BIT_MAX = DIRTY_BIT_INVALID,
     };
@@ -722,6 +728,7 @@ class State : angle::NonCopyable
     {
         EXTENDED_DIRTY_BIT_CLIP_CONTROL,            // EXT_clip_control
         EXTENDED_DIRTY_BIT_CLIP_DISTANCES,          // clip distances
+        EXTENDED_DIRTY_BIT_DEPTH_CLAMP_ENABLED,     // EXT_depth_clamp
         EXTENDED_DIRTY_BIT_MIPMAP_GENERATION_HINT,  // mipmap generation hint
         EXTENDED_DIRTY_BIT_SHADER_DERIVATIVE_HINT,  // shader derivative hint
         EXTENDED_DIRTY_BIT_SHADING_RATE,            // QCOM_shading_rate
@@ -766,9 +773,8 @@ class State : angle::NonCopyable
 
     using ExtendedDirtyBits = angle::BitSet32<EXTENDED_DIRTY_BIT_MAX>;
     const ExtendedDirtyBits &getExtendedDirtyBits() const { return mExtendedDirtyBits; }
-    // TODO(https://anglebug.com/5631): Handle extended dirty bits on non-vulkan backends
-    ExtendedDirtyBits getAndResetExtendedDirtyBits() const;
     void clearExtendedDirtyBits() { mExtendedDirtyBits.reset(); }
+    void clearExtendedDirtyBits(const ExtendedDirtyBits &bitset) { mExtendedDirtyBits &= ~bitset; }
 
     using DirtyObjects = angle::BitSet<DIRTY_OBJECT_MAX>;
     void clearDirtyObjects() { mDirtyObjects.reset(); }
@@ -1096,8 +1102,8 @@ class State : angle::NonCopyable
     float mNearZ;
     float mFarZ;
 
-    GLenum mClipControlOrigin;
-    GLenum mClipControlDepth;
+    ClipOrigin mClipOrigin;
+    ClipDepthMode mClipDepthMode;
 
     Framebuffer *mReadFramebuffer;
     Framebuffer *mDrawFramebuffer;
@@ -1188,7 +1194,7 @@ class State : angle::NonCopyable
     // GL_KHR_parallel_shader_compile
     GLuint mMaxShaderCompilerThreads;
 
-    // GL_APPLE_clip_distance/GL_EXT_clip_cull_distance
+    // GL_APPLE_clip_distance / GL_EXT_clip_cull_distance / GL_ANGLE_clip_cull_distance
     ClipDistanceEnableBits mClipDistancesEnabled;
 
     // GL_EXT_tessellation_shader
@@ -1236,6 +1242,9 @@ class State : angle::NonCopyable
     // QCOM_shading_rate
     bool mShadingRatePreserveAspectRatio;
     ShadingRate mShadingRate;
+
+    // GL_ARM_shader_framebuffer_fetch
+    bool mFetchPerSample;
 };
 
 ANGLE_INLINE angle::Result State::syncDirtyObjects(const Context *context,
