@@ -202,6 +202,8 @@ class ContextImpl : public GLImplFactory
     virtual angle::Result syncState(const gl::Context *context,
                                     const gl::State::DirtyBits &dirtyBits,
                                     const gl::State::DirtyBits &bitMask,
+                                    const gl::State::ExtendedDirtyBits &extendedDirtyBits,
+                                    const gl::State::ExtendedDirtyBits &extendedBitMask,
                                     gl::Command command) = 0;
 
     // Disjoint timer queries
@@ -213,11 +215,11 @@ class ContextImpl : public GLImplFactory
     virtual angle::Result onUnMakeCurrent(const gl::Context *context);
 
     // Native capabilities, unmodified by gl::Context.
-    virtual gl::Caps getNativeCaps() const                                 = 0;
-    virtual const gl::TextureCapsMap &getNativeTextureCaps() const         = 0;
-    virtual const gl::Extensions &getNativeExtensions() const              = 0;
-    virtual const gl::Limitations &getNativeLimitations() const            = 0;
-    virtual ShPixelLocalStorageType getNativePixelLocalStorageType() const = 0;
+    virtual gl::Caps getNativeCaps() const                                              = 0;
+    virtual const gl::TextureCapsMap &getNativeTextureCaps() const                      = 0;
+    virtual const gl::Extensions &getNativeExtensions() const                           = 0;
+    virtual const gl::Limitations &getNativeLimitations() const                         = 0;
+    virtual const ShPixelLocalStorageOptions &getNativePixelLocalStorageOptions() const = 0;
 
     virtual angle::Result dispatchCompute(const gl::Context *context,
                                           GLuint numGroupsX,
