@@ -25,7 +25,7 @@
 #include "libANGLE/renderer/gl/formatutilsgl.h"
 #include "libANGLE/renderer/gl/renderergl_utils.h"
 #include "libANGLE/renderer/renderer_utils.h"
-#include "platform/FeaturesGL.h"
+#include "platform/FeaturesGL_autogen.h"
 
 using angle::Vector2;
 
@@ -62,7 +62,7 @@ angle::Result CheckLinkStatus(const gl::Context *context,
     return angle::Result::Continue;
 }
 
-class ScopedGLState : angle::NonCopyable
+class [[nodiscard]] ScopedGLState : angle::NonCopyable
 {
   public:
     enum
@@ -85,6 +85,10 @@ class ScopedGLState : angle::NonCopyable
         }
         stateManager->setViewport(viewport);
         stateManager->setDepthRange(0.0f, 1.0f);
+        stateManager->setClipControl(gl::ClipOrigin::LowerLeft,
+                                     gl::ClipDepthMode::NegativeOneToOne);
+        stateManager->setClipDistancesEnable(gl::State::ClipDistanceEnableBits());
+        stateManager->setDepthClampEnabled(false);
         stateManager->setBlendEnabled(false);
         stateManager->setColorMask(true, true, true, true);
         stateManager->setSampleAlphaToCoverageEnabled(false);
@@ -94,6 +98,7 @@ class ScopedGLState : angle::NonCopyable
         stateManager->setCullFaceEnabled(false);
         stateManager->setPolygonOffsetFillEnabled(false);
         stateManager->setRasterizerDiscardEnabled(false);
+        stateManager->setLogicOpEnabled(false);
 
         stateManager->pauseTransformFeedback();
         return stateManager->pauseAllQueries(context);
@@ -199,6 +204,19 @@ angle::Result PrepareForClear(StateManagerGL *stateManager,
     return angle::Result::Continue;
 }
 
+angle::Result UnbindAttachment(const gl::Context *context,
+                               const FunctionsGL *functions,
+                               GLenum framebufferTarget,
+                               GLenum attachment)
+{
+    // Always use framebufferTexture2D as a workaround for an Nvidia driver bug. See
+    // https://anglebug.com/5536 and FeaturesGL.alwaysUnbindFramebufferTexture2D
+    ANGLE_GL_TRY(context,
+                 functions->framebufferTexture2D(GL_FRAMEBUFFER, attachment, GL_TEXTURE_2D, 0, 0));
+
+    return angle::Result::Continue;
+}
+
 angle::Result UnbindAttachments(const gl::Context *context,
                                 const FunctionsGL *functions,
                                 GLenum framebufferTarget,
@@ -206,8 +224,7 @@ angle::Result UnbindAttachments(const gl::Context *context,
 {
     for (GLenum bindTarget : bindTargets)
     {
-        ANGLE_GL_TRY(context, functions->framebufferRenderbuffer(framebufferTarget, bindTarget,
-                                                                 GL_RENDERBUFFER, 0));
+        ANGLE_TRY(UnbindAttachment(context, functions, framebufferTarget, bindTarget));
     }
     return angle::Result::Continue;
 }
@@ -256,9 +273,10 @@ BlitGL::~BlitGL()
         mScratchFBO = 0;
     }
 
-    if (mVAO != 0)
+    if (mOwnsVAOState)
     {
         mStateManager->deleteVertexArray(mVAO);
+        SafeDelete(mVAOState);
         mVAO = 0;
     }
 }
@@ -279,10 +297,16 @@ angle::Result BlitGL::copyImageToLUMAWorkaroundTexture(const gl::Context *contex
     GLenum format   = gl::GetUnsizedFormat(internalFormat);
     GLenum readType = source->getImplementationColorReadType(context);
 
+    // getImplementationColorReadType aligns the type with ES client version
+    if (readType == GL_HALF_FLOAT_OES && mFunctions->standard == STANDARD_GL_DESKTOP)
+    {
+        readType = GL_HALF_FLOAT;
+    }
+
     gl::PixelUnpackState unpack;
-    mStateManager->setPixelUnpackState(unpack);
-    mStateManager->setPixelUnpackBuffer(
-        context->getState().getTargetBuffer(gl::BufferBinding::PixelUnpack));
+    ANGLE_TRY(mStateManager->setPixelUnpackState(context, unpack));
+    ANGLE_TRY(mStateManager->setPixelUnpackBuffer(
+        context, context->getState().getTargetBuffer(gl::BufferBinding::PixelUnpack)));
     ANGLE_GL_TRY_ALWAYS_CHECK(
         context,
         mFunctions->texImage2D(ToGLenum(target), static_cast<GLint>(level), internalFormat,
@@ -314,6 +338,12 @@ angle::Result BlitGL::copySubImageToLUMAWorkaroundTexture(const gl::Context *con
     GLenum readFormat = source->getImplementationColorReadFormat(context);
     GLenum readType   = source->getImplementationColorReadType(context);
 
+    // getImplementationColorReadType aligns the type with ES client version
+    if (readType == GL_HALF_FLOAT_OES && mFunctions->standard == STANDARD_GL_DESKTOP)
+    {
+        readType = GL_HALF_FLOAT;
+    }
+
     nativegl::CopyTexImageImageFormat copyTexImageFormat =
         nativegl::GetCopyTexImageImageFormat(mFunctions, mFeatures, readFormat, readType);
 
@@ -365,7 +395,7 @@ angle::Result BlitGL::copySubImageToLUMAWorkaroundTexture(const gl::Context *con
     ANGLE_GL_TRY(context, mFunctions->uniform1i(blitProgram->multiplyAlphaLocation, 0));
     ANGLE_GL_TRY(context, mFunctions->uniform1i(blitProgram->unMultiplyAlphaLocation, 0));
 
-    mStateManager->bindVertexArray(mVAO, 0);
+    ANGLE_TRY(setVAOState(context));
     ANGLE_GL_TRY(context, mFunctions->drawArrays(GL_TRIANGLES, 0, 3));
 
     // Copy the swizzled texture to the destination texture
@@ -388,6 +418,7 @@ angle::Result BlitGL::copySubImageToLUMAWorkaroundTexture(const gl::Context *con
 
     // Finally orphan the scratch textures so they can be GCed by the driver.
     ANGLE_TRY(orphanScratchTextures(context));
+    ANGLE_TRY(UnbindAttachment(context, mFunctions, GL_FRAMEBUFFER, GL_COLOR_ATTACHMENT0));
 
     ANGLE_TRY(scopedState.exit(context));
     return angle::Result::Continue;
@@ -416,8 +447,7 @@ angle::Result BlitGL::blitColorBufferWithShader(const gl::Context *context,
     angle::Result result = blitColorBufferWithShader(context, source, mScratchFBO, sourceAreaIn,
                                                      destAreaIn, filter, writeAlpha);
     // Unbind the texture from the the scratch framebuffer.
-    ANGLE_GL_TRY(context, mFunctions->framebufferRenderbuffer(GL_FRAMEBUFFER, GL_COLOR_ATTACHMENT0,
-                                                              GL_RENDERBUFFER, 0));
+    ANGLE_TRY(UnbindAttachment(context, mFunctions, GL_FRAMEBUFFER, GL_COLOR_ATTACHMENT0));
     return result;
 }
 
@@ -548,7 +578,7 @@ angle::Result BlitGL::blitColorBufferWithShader(const gl::Context *context,
 
     mStateManager->bindFramebuffer(GL_DRAW_FRAMEBUFFER, destFramebuffer);
 
-    mStateManager->bindVertexArray(mVAO, 0);
+    ANGLE_TRY(setVAOState(context));
     ANGLE_GL_TRY(context, mFunctions->drawArrays(GL_TRIANGLES, 0, 3));
 
     ANGLE_TRY(scopedState.exit(context));
@@ -663,8 +693,9 @@ angle::Result BlitGL::copySubTexture(const gl::Context *context,
                                                     unpackUnmultiplyAlpha));
     }
 
-    mStateManager->bindVertexArray(mVAO, 0);
+    ANGLE_TRY(setVAOState(context));
     ANGLE_GL_TRY(context, mFunctions->drawArrays(GL_TRIANGLES, 0, 3));
+    ANGLE_TRY(UnbindAttachment(context, mFunctions, GL_FRAMEBUFFER, GL_COLOR_ATTACHMENT0));
 
     *copySucceededOut = true;
     ANGLE_TRY(scopedState.exit(context));
@@ -703,10 +734,16 @@ angle::Result BlitGL::copySubTextureCPUReadback(const gl::Context *context,
     gl::Rectangle readPixelsArea = sourceArea;
 
     mStateManager->bindFramebuffer(GL_FRAMEBUFFER, mScratchFBO);
-    ANGLE_GL_TRY(context, mFunctions->framebufferTexture2D(
-                              GL_FRAMEBUFFER, GL_COLOR_ATTACHMENT0, ToGLenum(source->getType()),
-                              source->getTextureID(), static_cast<GLint>(sourceLevel)));
-    GLenum status = ANGLE_GL_TRY(context, mFunctions->checkFramebufferStatus(GL_FRAMEBUFFER));
+    bool supportExternalTarget =
+        source->getType() == gl::TextureType::External && context->getExtensions().YUVTargetEXT;
+    GLenum status = GL_FRAMEBUFFER_INCOMPLETE_MISSING_ATTACHMENT;
+    if (supportExternalTarget || source->getType() != gl::TextureType::External)
+    {
+        ANGLE_GL_TRY(context, mFunctions->framebufferTexture2D(
+                                  GL_FRAMEBUFFER, GL_COLOR_ATTACHMENT0, ToGLenum(source->getType()),
+                                  source->getTextureID(), static_cast<GLint>(sourceLevel)));
+        status = ANGLE_GL_TRY(context, mFunctions->checkFramebufferStatus(GL_FRAMEBUFFER));
+    }
     if (status != GL_FRAMEBUFFER_COMPLETE)
     {
         // The source texture cannot be read with glReadPixels. Copy it into another RGBA texture
@@ -779,8 +816,8 @@ angle::Result BlitGL::copySubTextureCPUReadback(const gl::Context *context,
 
     gl::PixelUnpackState unpack;
     unpack.alignment = 1;
-    mStateManager->setPixelUnpackState(unpack);
-    mStateManager->setPixelUnpackBuffer(nullptr);
+    ANGLE_TRY(mStateManager->setPixelUnpackState(context, unpack));
+    ANGLE_TRY(mStateManager->setPixelUnpackBuffer(context, nullptr));
     ANGLE_GL_TRY(context, mFunctions->readPixels(readPixelsArea.x, readPixelsArea.y,
                                                  readPixelsArea.width, readPixelsArea.height,
                                                  readPixelsFormat, GL_UNSIGNED_BYTE, sourceMemory));
@@ -797,8 +834,8 @@ angle::Result BlitGL::copySubTextureCPUReadback(const gl::Context *context,
 
     gl::PixelPackState pack;
     pack.alignment = 1;
-    mStateManager->setPixelPackState(pack);
-    mStateManager->setPixelPackBuffer(nullptr);
+    ANGLE_TRY(mStateManager->setPixelPackState(context, pack));
+    ANGLE_TRY(mStateManager->setPixelPackBuffer(context, nullptr));
 
     nativegl::TexSubImageFormat texSubImageFormat =
         nativegl::GetTexSubImageFormat(mFunctions, mFeatures, destFormat, destType);
@@ -809,6 +846,8 @@ angle::Result BlitGL::copySubTextureCPUReadback(const gl::Context *context,
                               destOffset.y, readPixelsArea.width, readPixelsArea.height,
                               texSubImageFormat.format, texSubImageFormat.type, destMemory));
 
+    ANGLE_TRY(UnbindAttachment(context, mFunctions, GL_FRAMEBUFFER, GL_COLOR_ATTACHMENT0));
+
     return angle::Result::Continue;
 }
 
@@ -843,6 +882,7 @@ angle::Result BlitGL::copyTexSubImage(const gl::Context *context,
                                                destOffset.x, destOffset.y, sourceArea.x,
                                                sourceArea.y, sourceArea.width, sourceArea.height));
 
+    ANGLE_TRY(UnbindAttachment(context, mFunctions, GL_FRAMEBUFFER, GL_COLOR_ATTACHMENT0));
     *copySucceededOut = true;
     return angle::Result::Continue;
 }
@@ -987,13 +1027,17 @@ angle::Result BlitGL::clearRenderbuffer(const gl::Context *context,
     return angle::Result::Continue;
 }
 
-angle::Result BlitGL::clearFramebuffer(const gl::Context *context, FramebufferGL *source)
+angle::Result BlitGL::clearFramebuffer(const gl::Context *context,
+                                       bool colorClear,
+                                       bool depthClear,
+                                       bool stencilClear,
+                                       FramebufferGL *source)
 {
     // initializeResources skipped because no local state is used
 
     // Clear all attachments
     GLbitfield clearMask = 0;
-    ANGLE_TRY(SetClearState(mStateManager, true, true, true, &clearMask));
+    ANGLE_TRY(SetClearState(mStateManager, colorClear, depthClear, stencilClear, &clearMask));
 
     mStateManager->bindFramebuffer(GL_FRAMEBUFFER, source->getFramebufferID());
     ANGLE_GL_TRY(context, mFunctions->clear(clearMask));
@@ -1022,62 +1066,156 @@ angle::Result BlitGL::clearRenderableTextureAlphaToOne(const gl::Context *contex
     ANGLE_GL_TRY(context, mFunctions->clear(GL_COLOR_BUFFER_BIT));
 
     // Unbind the texture from the the scratch framebuffer
-    ANGLE_GL_TRY(context, mFunctions->framebufferRenderbuffer(GL_FRAMEBUFFER, GL_COLOR_ATTACHMENT0,
-                                                              GL_RENDERBUFFER, 0));
+    ANGLE_TRY(UnbindAttachment(context, mFunctions, GL_FRAMEBUFFER, GL_COLOR_ATTACHMENT0));
 
     return angle::Result::Continue;
 }
 
+angle::Result BlitGL::generateSRGBMipmap(const gl::Context *context,
+                                         TextureGL *source,
+                                         GLuint baseLevel,
+                                         GLuint levelCount,
+                                         const gl::Extents &sourceBaseLevelSize)
+{
+    ANGLE_TRY(initializeResources(context));
+
+    const gl::TextureType sourceType     = gl::TextureType::_2D;
+    const gl::TextureTarget sourceTarget = gl::TextureTarget::_2D;
+
+    ScopedGLState scopedState;
+    ANGLE_TRY(scopedState.enter(
+        context, gl::Rectangle(0, 0, sourceBaseLevelSize.width, sourceBaseLevelSize.height)));
+    scopedState.willUseTextureUnit(context, 0);
+    mStateManager->activeTexture(0);
+
+    // Copy source to a linear intermediate texture.
+    GLuint linearTexture = mScratchTextures[0];
+    mStateManager->bindTexture(sourceType, linearTexture);
+    ANGLE_GL_TRY(context, mFunctions->texImage2D(
+                              ToGLenum(sourceTarget), 0, mSRGBMipmapGenerationFormat.internalFormat,
+                              sourceBaseLevelSize.width, sourceBaseLevelSize.height, 0,
+                              mSRGBMipmapGenerationFormat.format, mSRGBMipmapGenerationFormat.type,
+                              nullptr));
+
+    mStateManager->bindFramebuffer(GL_FRAMEBUFFER, mScratchFBO);
+    ANGLE_GL_TRY(context,
+                 mFunctions->framebufferTexture2D(GL_FRAMEBUFFER, GL_COLOR_ATTACHMENT0,
+                                                  ToGLenum(sourceTarget), linearTexture, 0));
+    mStateManager->setFramebufferSRGBEnabled(context, true);
+
+    // Use a shader to do the sRGB to linear conversion. glBlitFramebuffer does not always do this
+    // conversion for us.
+    BlitProgram *blitProgram = nullptr;
+    ANGLE_TRY(getBlitProgram(context, sourceType, GL_FLOAT, GL_FLOAT, &blitProgram));
+
+    mStateManager->useProgram(blitProgram->program);
+    ANGLE_GL_TRY(context, mFunctions->uniform1i(blitProgram->sourceTextureLocation, 0));
+    ANGLE_GL_TRY(context, mFunctions->uniform2f(blitProgram->scaleLocation, 1.0f, 1.0f));
+    ANGLE_GL_TRY(context, mFunctions->uniform2f(blitProgram->offsetLocation, 0.0f, 0.0f));
+    ANGLE_GL_TRY(context, mFunctions->uniform1i(blitProgram->multiplyAlphaLocation, 0));
+    ANGLE_GL_TRY(context, mFunctions->uniform1i(blitProgram->unMultiplyAlphaLocation, 0));
+
+    mStateManager->bindTexture(sourceType, source->getTextureID());
+    ANGLE_TRY(source->setMinFilter(context, GL_NEAREST));
+
+    ANGLE_TRY(setVAOState(context));
+    ANGLE_GL_TRY(context, mFunctions->drawArrays(GL_TRIANGLES, 0, 3));
+
+    // Generate mipmaps on the linear texture
+    mStateManager->bindTexture(sourceType, linearTexture);
+    ANGLE_GL_TRY_ALWAYS_CHECK(context, mFunctions->generateMipmap(ToGLenum(sourceTarget)));
+    ANGLE_GL_TRY(context, mFunctions->texParameteri(ToGLenum(sourceTarget), GL_TEXTURE_MIN_FILTER,
+                                                    GL_NEAREST));
+
+    // Copy back to the source texture from the mips generated in the linear texture
+    for (GLuint levelIdx = 0; levelIdx < levelCount; levelIdx++)
+    {
+        gl::Extents levelSize(std::max(sourceBaseLevelSize.width >> levelIdx, 1),
+                              std::max(sourceBaseLevelSize.height >> levelIdx, 1), 1);
+
+        ANGLE_GL_TRY(context, mFunctions->framebufferTexture2D(
+                                  GL_FRAMEBUFFER, GL_COLOR_ATTACHMENT0, ToGLenum(sourceTarget),
+                                  source->getTextureID(), baseLevel + levelIdx));
+        mStateManager->setViewport(gl::Rectangle(0, 0, levelSize.width, levelSize.height));
+
+        ANGLE_GL_TRY(context, mFunctions->texParameteri(ToGLenum(sourceTarget),
+                                                        GL_TEXTURE_BASE_LEVEL, levelIdx));
+
+        ANGLE_GL_TRY(context, mFunctions->drawArrays(GL_TRIANGLES, 0, 3));
+    }
+
+    ANGLE_TRY(orphanScratchTextures(context));
+    ANGLE_TRY(UnbindAttachment(context, mFunctions, GL_FRAMEBUFFER, GL_COLOR_ATTACHMENT0));
+
+    ANGLE_TRY(scopedState.exit(context));
+    return angle::Result::Continue;
+}
+
 angle::Result BlitGL::initializeResources(const gl::Context *context)
 {
+    if (mResourcesInitialized)
+    {
+        return angle::Result::Continue;
+    }
+
     for (size_t i = 0; i < ArraySize(mScratchTextures); i++)
     {
-        if (mScratchTextures[i] == 0)
-        {
-            ANGLE_GL_TRY(context, mFunctions->genTextures(1, &mScratchTextures[i]));
-        }
+        ANGLE_GL_TRY(context, mFunctions->genTextures(1, &mScratchTextures[i]));
     }
 
-    if (mScratchFBO == 0)
-    {
-        ANGLE_GL_TRY(context, mFunctions->genFramebuffers(1, &mScratchFBO));
-    }
+    ANGLE_GL_TRY(context, mFunctions->genFramebuffers(1, &mScratchFBO));
 
-    if (mVertexBuffer == 0)
-    {
-        ANGLE_GL_TRY(context, mFunctions->genBuffers(1, &mVertexBuffer));
-        mStateManager->bindBuffer(gl::BufferBinding::Array, mVertexBuffer);
+    ANGLE_GL_TRY(context, mFunctions->genBuffers(1, &mVertexBuffer));
+    mStateManager->bindBuffer(gl::BufferBinding::Array, mVertexBuffer);
 
-        // Use a single, large triangle, to avoid arithmetic precision issues where fragments
-        // with the same Y coordinate don't get exactly the same interpolated texcoord Y.
-        float vertexData[] = {
-            -0.5f, 0.0f, 1.5f, 0.0f, 0.5f, 2.0f,
-        };
+    // Use a single, large triangle, to avoid arithmetic precision issues where fragments
+    // with the same Y coordinate don't get exactly the same interpolated texcoord Y.
+    float vertexData[] = {
+        -0.5f, 0.0f, 1.5f, 0.0f, 0.5f, 2.0f,
+    };
 
-        ANGLE_GL_TRY(context, mFunctions->bufferData(GL_ARRAY_BUFFER, sizeof(float) * 6, vertexData,
-                                                     GL_STATIC_DRAW));
-    }
+    ANGLE_GL_TRY(context, mFunctions->bufferData(GL_ARRAY_BUFFER, sizeof(float) * 6, vertexData,
+                                                 GL_STATIC_DRAW));
 
-    if (mVAO == 0)
+    VertexArrayStateGL *defaultVAOState = mStateManager->getDefaultVAOState();
+    if (!mFeatures.syncVertexArraysToDefault.enabled)
     {
         ANGLE_GL_TRY(context, mFunctions->genVertexArrays(1, &mVAO));
-
-        mStateManager->bindVertexArray(mVAO, 0);
-        mStateManager->bindBuffer(gl::BufferBinding::Array, mVertexBuffer);
-
-        // Enable all attributes with the same buffer so that it doesn't matter what location the
-        // texcoord attribute is assigned
-        GLint maxAttributes = 0;
-        ANGLE_GL_TRY(context, mFunctions->getIntegerv(GL_MAX_VERTEX_ATTRIBS, &maxAttributes));
-
-        for (GLint i = 0; i < maxAttributes; i++)
-        {
-            ANGLE_GL_TRY(context, mFunctions->enableVertexAttribArray(i));
-            ANGLE_GL_TRY(context,
-                         mFunctions->vertexAttribPointer(i, 2, GL_FLOAT, GL_FALSE, 0, nullptr));
-        }
+        mVAOState     = new VertexArrayStateGL(defaultVAOState->attributes.size(),
+                                               defaultVAOState->bindings.size());
+        mOwnsVAOState = true;
+        ANGLE_TRY(setVAOState(context));
+        ANGLE_TRY(initializeVAOState(context));
+    }
+    else
+    {
+        mVAO          = mStateManager->getDefaultVAO();
+        mVAOState     = defaultVAOState;
+        mOwnsVAOState = false;
     }
 
+    constexpr GLenum potentialSRGBMipmapGenerationFormats[] = {
+        GL_RGBA16, GL_RGBA16F, GL_RGBA32F,
+        GL_RGBA8,  // RGBA8 can have precision loss when generating mipmaps of a sRGBA8 texture
+    };
+    for (GLenum internalFormat : potentialSRGBMipmapGenerationFormats)
+    {
+        if (nativegl::SupportsNativeRendering(mFunctions, gl::TextureType::_2D, internalFormat))
+        {
+            const gl::InternalFormat &internalFormatInfo =
+                gl::GetSizedInternalFormatInfo(internalFormat);
+
+            // Pass the 'format' instead of 'internalFormat' to make sure we use unsized formats
+            // when available to increase support.
+            mSRGBMipmapGenerationFormat =
+                nativegl::GetTexImageFormat(mFunctions, mFeatures, internalFormatInfo.format,
+                                            internalFormatInfo.format, internalFormatInfo.type);
+            break;
+        }
+    }
+    ASSERT(mSRGBMipmapGenerationFormat.internalFormat != GL_NONE);
+
+    mResourcesInitialized = true;
     return angle::Result::Continue;
 }
 
@@ -1087,14 +1225,36 @@ angle::Result BlitGL::orphanScratchTextures(const gl::Context *context)
     {
         mStateManager->bindTexture(gl::TextureType::_2D, texture);
         gl::PixelUnpackState unpack;
-        mStateManager->setPixelUnpackState(unpack);
-        mStateManager->setPixelUnpackBuffer(nullptr);
-        GLint swizzle[4] = {GL_RED, GL_GREEN, GL_BLUE, GL_ALPHA};
+        ANGLE_TRY(mStateManager->setPixelUnpackState(context, unpack));
+        ANGLE_TRY(mStateManager->setPixelUnpackBuffer(context, nullptr));
+        if (mFunctions->isAtLeastGL(gl::Version(3, 3)))
+        {
+            constexpr GLint swizzle[4] = {GL_RED, GL_GREEN, GL_BLUE, GL_ALPHA};
+            ANGLE_GL_TRY(context, mFunctions->texParameteriv(GL_TEXTURE_2D, GL_TEXTURE_SWIZZLE_RGBA,
+                                                             swizzle));
+        }
+        else if (mFunctions->isAtLeastGLES(gl::Version(3, 0)))
+        {
+            ANGLE_GL_TRY(context,
+                         mFunctions->texParameteri(GL_TEXTURE_2D, GL_TEXTURE_SWIZZLE_R, GL_RED));
+            ANGLE_GL_TRY(context,
+                         mFunctions->texParameteri(GL_TEXTURE_2D, GL_TEXTURE_SWIZZLE_G, GL_GREEN));
+            ANGLE_GL_TRY(context,
+                         mFunctions->texParameteri(GL_TEXTURE_2D, GL_TEXTURE_SWIZZLE_B, GL_BLUE));
+            ANGLE_GL_TRY(context,
+                         mFunctions->texParameteri(GL_TEXTURE_2D, GL_TEXTURE_SWIZZLE_A, GL_ALPHA));
+        }
+
+        ANGLE_GL_TRY(context, mFunctions->texParameteri(GL_TEXTURE_2D, GL_TEXTURE_BASE_LEVEL, 0));
+        ANGLE_GL_TRY(context, mFunctions->texParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAX_LEVEL, 1000));
+        ANGLE_GL_TRY(context, mFunctions->texParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER,
+                                                        GL_NEAREST_MIPMAP_LINEAR));
         ANGLE_GL_TRY(context,
-                     mFunctions->texParameteriv(GL_TEXTURE_2D, GL_TEXTURE_SWIZZLE_RGBA, swizzle));
+                     mFunctions->texParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR));
         ANGLE_GL_TRY(context, mFunctions->texImage2D(GL_TEXTURE_2D, 0, GL_RGBA, 0, 0, 0, GL_RGBA,
                                                      GL_UNSIGNED_BYTE, nullptr));
     }
+
     return angle::Result::Continue;
 }
 
@@ -1111,6 +1271,43 @@ angle::Result BlitGL::setScratchTextureParameter(const gl::Context *context,
     return angle::Result::Continue;
 }
 
+angle::Result BlitGL::setVAOState(const gl::Context *context)
+{
+    mStateManager->bindVertexArray(mVAO, mVAOState);
+    if (mFeatures.syncVertexArraysToDefault.enabled)
+    {
+        ANGLE_TRY(initializeVAOState(context));
+    }
+
+    return angle::Result::Continue;
+}
+
+angle::Result BlitGL::initializeVAOState(const gl::Context *context)
+{
+    mStateManager->bindBuffer(gl::BufferBinding::Array, mVertexBuffer);
+
+    ANGLE_GL_TRY(context, mFunctions->enableVertexAttribArray(mTexcoordAttribLocation));
+    ANGLE_GL_TRY(context, mFunctions->vertexAttribPointer(mTexcoordAttribLocation, 2, GL_FLOAT,
+                                                          GL_FALSE, 0, nullptr));
+
+    VertexAttributeGL &attribute = mVAOState->attributes[mTexcoordAttribLocation];
+    attribute.enabled            = true;
+    attribute.format             = &angle::Format::Get(angle::FormatID::R32G32_FLOAT);
+    attribute.pointer            = nullptr;
+
+    VertexBindingGL &binding = mVAOState->bindings[mTexcoordAttribLocation];
+    binding.stride           = 8;
+    binding.offset           = 0;
+    binding.buffer           = mVertexBuffer;
+
+    if (mFeatures.syncVertexArraysToDefault.enabled)
+    {
+        mStateManager->setDefaultVAOStateDirty();
+    }
+
+    return angle::Result::Continue;
+}
+
 angle::Result BlitGL::getBlitProgram(const gl::Context *context,
                                      gl::TextureType sourceTextureType,
                                      GLenum sourceComponentType,
@@ -1126,6 +1323,7 @@ angle::Result BlitGL::getBlitProgram(const gl::Context *context,
 
         // Depending on what types need to be output by the shaders, different versions need to be
         // used.
+        constexpr const char *texcoordAttribName = "a_texcoord";
         std::string version;
         std::string vsInputVariableQualifier;
         std::string vsOutputVariableQualifier;
@@ -1166,15 +1364,17 @@ angle::Result BlitGL::getBlitProgram(const gl::Context *context,
             // Compile the vertex shader
             std::ostringstream vsSourceStream;
             vsSourceStream << "#version " << version << "\n";
-            vsSourceStream << vsInputVariableQualifier << " vec2 a_texcoord;\n";
+            vsSourceStream << vsInputVariableQualifier << " vec2 " << texcoordAttribName << ";\n";
             vsSourceStream << "uniform vec2 u_scale;\n";
             vsSourceStream << "uniform vec2 u_offset;\n";
             vsSourceStream << vsOutputVariableQualifier << " vec2 v_texcoord;\n";
             vsSourceStream << "\n";
             vsSourceStream << "void main()\n";
             vsSourceStream << "{\n";
-            vsSourceStream << "    gl_Position = vec4((a_texcoord * 2.0) - 1.0, 0.0, 1.0);\n";
-            vsSourceStream << "    v_texcoord = a_texcoord * u_scale + u_offset;\n";
+            vsSourceStream << "    gl_Position = vec4((" << texcoordAttribName
+                           << " * 2.0) - 1.0, 0.0, 1.0);\n";
+            vsSourceStream << "    v_texcoord = " << texcoordAttribName
+                           << " * u_scale + u_offset;\n";
             vsSourceStream << "}\n";
 
             std::string vsSourceStr  = vsSourceStream.str();
@@ -1361,6 +1561,8 @@ angle::Result BlitGL::getBlitProgram(const gl::Context *context,
             ANGLE_GL_TRY(context, mFunctions->deleteShader(fs));
         }
 
+        ANGLE_GL_TRY(context, mFunctions->bindAttribLocation(
+                                  result.program, mTexcoordAttribLocation, texcoordAttribName));
         ANGLE_GL_TRY(context, mFunctions->linkProgram(result.program));
         ANGLE_TRY(CheckLinkStatus(context, mFunctions, result.program));
 
