@@ -886,9 +886,12 @@ angle::Result ContextGL::popDebugGroup(const gl::Context *context)
 angle::Result ContextGL::syncState(const gl::Context *context,
                                    const gl::State::DirtyBits &dirtyBits,
                                    const gl::State::DirtyBits &bitMask,
+                                   const gl::State::ExtendedDirtyBits &extendedDirtyBits,
+                                   const gl::State::ExtendedDirtyBits &extendedBitMask,
                                    gl::Command command)
 {
-    return mRenderer->getStateManager()->syncState(context, dirtyBits, bitMask);
+    return mRenderer->getStateManager()->syncState(context, dirtyBits, bitMask, extendedDirtyBits,
+                                                   extendedBitMask);
 }
 
 GLint ContextGL::getGPUDisjoint()
@@ -937,9 +940,9 @@ const gl::Limitations &ContextGL::getNativeLimitations() const
     return mRenderer->getNativeLimitations();
 }
 
-ShPixelLocalStorageType ContextGL::getNativePixelLocalStorageType() const
+const ShPixelLocalStorageOptions &ContextGL::getNativePixelLocalStorageOptions() const
 {
-    return mRenderer->getNativePixelLocalStorageType();
+    return mRenderer->getNativePixelLocalStorageOptions();
 }
 
 StateManagerGL *ContextGL::getStateManager()
@@ -1046,7 +1049,8 @@ angle::Result ContextGL::drawPixelLocalStorageEXTEnable(gl::Context *context,
                                                         const gl::PixelLocalStoragePlane planes[],
                                                         const GLenum loadops[])
 {
-    ASSERT(getNativePixelLocalStorageType() == ShPixelLocalStorageType::PixelLocalStorageEXT);
+    ASSERT(getNativePixelLocalStorageOptions().type ==
+           ShPixelLocalStorageType::PixelLocalStorageEXT);
 
     getFunctions()->enable(GL_SHADER_PIXEL_LOCAL_STORAGE_EXT);
 
@@ -1055,8 +1059,8 @@ angle::Result ContextGL::drawPixelLocalStorageEXTEnable(gl::Context *context,
     {
         const gl::PixelLocalStoragePlane &plane = planes[i];
         GLenum loadop                           = loadops[i];
-        bool preserved                          = loadop == GL_KEEP;
-        b.prependPlane(loadop != GL_DISABLE_ANGLE ? plane.getInternalformat() : GL_NONE, preserved);
+        bool preserved                          = loadop == GL_LOAD_OP_LOAD_ANGLE;
+        b.prependPlane(plane.getInternalformat(), preserved);
         if (preserved)
         {
             const gl::ImageIndex &idx = plane.getTextureImageIndex();
@@ -1084,14 +1088,17 @@ angle::Result ContextGL::drawPixelLocalStorageEXTDisable(gl::Context *context,
                                                          const gl::PixelLocalStoragePlane planes[],
                                                          const GLenum storeops[])
 {
-    ASSERT(getNativePixelLocalStorageType() == ShPixelLocalStorageType::PixelLocalStorageEXT);
+    ASSERT(getNativePixelLocalStorageOptions().type ==
+           ShPixelLocalStorageType::PixelLocalStorageEXT);
+
     GLsizei n = context->getState().getPixelLocalStorageActivePlanes();
 
     PLSProgramKeyBuilder b;
     for (GLsizei i = n - 1; i >= 0; --i)
     {
         const gl::PixelLocalStoragePlane &plane = planes[i];
-        bool preserved = plane.isActive() && !plane.isMemoryless() && storeops[i] == GL_KEEP;
+        bool preserved =
+            plane.isActive() && !plane.isMemoryless() && storeops[i] == GL_STORE_OP_STORE_ANGLE;
         b.prependPlane(plane.isActive() ? plane.getInternalformat() : GL_NONE, preserved);
         if (preserved)
         {
