@@ -59,6 +59,7 @@ struct ExternalContextState
     GLfloat lineWidth;
     GLfloat polygonOffsetFactor;
     GLfloat polygonOffsetUnits;
+    GLfloat polygonOffsetClamp;
     GLfloat sampleCoverageValue;
     bool sampleCoverageInvert;
     GLenum blendEquationRgb;
@@ -66,6 +67,7 @@ struct ExternalContextState
 
     bool enableDither;
     bool enablePolygonOffsetFill;
+    bool enableDepthClamp;
     bool enableSampleAlphaToCoverage;
     bool enableSampleCoverage;
     bool multisampleEnabled;
@@ -77,6 +79,8 @@ struct ExternalContextState
     GLenum blendDestAlpha;
     GLenum activeTexture;
     gl::Rectangle viewport;
+    GLenum clipOrigin;
+    GLenum clipDepthMode;
     bool scissorTest;
     gl::Rectangle scissorBox;
 
@@ -198,6 +202,7 @@ class StateManagerGL final : angle::NonCopyable
 
     void setViewport(const gl::Rectangle &viewport);
     void setDepthRange(float near, float far);
+    void setClipControl(gl::ClipOrigin origin, gl::ClipDepthMode depth);
 
     void setBlendEnabled(bool enabled);
     void setBlendEnabledIndexed(const gl::DrawBufferMask blendEnabledMask);
@@ -226,7 +231,8 @@ class StateManagerGL final : angle::NonCopyable
     void setCullFace(gl::CullFaceMode cullFace);
     void setFrontFace(GLenum frontFace);
     void setPolygonOffsetFillEnabled(bool enabled);
-    void setPolygonOffset(float factor, float units);
+    void setPolygonOffset(float factor, float units, float clamp);
+    void setDepthClampEnabled(bool enabled);
     void setRasterizerDiscardEnabled(bool enabled);
     void setLineWidth(float width);
 
@@ -273,7 +279,9 @@ class StateManagerGL final : angle::NonCopyable
 
     angle::Result syncState(const gl::Context *context,
                             const gl::State::DirtyBits &glDirtyBits,
-                            const gl::State::DirtyBits &bitMask);
+                            const gl::State::DirtyBits &bitMask,
+                            const gl::State::ExtendedDirtyBits &extendedDirtyBits,
+                            const gl::State::ExtendedDirtyBits &extendedBitMask);
 
     ANGLE_INLINE void updateMultiviewBaseViewLayerIndexUniform(
         const gl::Program *program,
@@ -329,6 +337,10 @@ class StateManagerGL final : angle::NonCopyable
     void syncSamplersState(const gl::Context *context);
     void syncTransformFeedbackState(const gl::Context *context);
 
+    void updateEmulatedClipDistanceState(const gl::ProgramExecutable *executable,
+                                         const gl::Program *program,
+                                         const gl::State::ClipDistanceEnableBits enables) const;
+
     void updateMultiviewBaseViewLayerIndexUniformImpl(
         const gl::Program *program,
         const gl::FramebufferState &drawFramebufferState) const;
@@ -444,6 +456,7 @@ class StateManagerGL final : angle::NonCopyable
     // TODO(jmadill): Convert to std::array when available
     std::vector<GLenum> mFramebuffers;
     GLuint mRenderbuffer;
+    GLuint mPlaceholderFbo;
 
     bool mScissorTestEnabled;
     gl::Rectangle mScissor;
@@ -451,6 +464,9 @@ class StateManagerGL final : angle::NonCopyable
     float mNear;
     float mFar;
 
+    gl::ClipOrigin mClipOrigin;
+    gl::ClipDepthMode mClipDepthMode;
+
     gl::ColorF mBlendColor;
     gl::BlendStateExt mBlendStateExt;
     const bool mIndependentBlendStates;
@@ -487,6 +503,8 @@ class StateManagerGL final : angle::NonCopyable
     bool mPolygonOffsetFillEnabled;
     GLfloat mPolygonOffsetFactor;
     GLfloat mPolygonOffsetUnits;
+    GLfloat mPolygonOffsetClamp;
+    bool mDepthClampEnabled;
     bool mRasterizerDiscardEnabled;
     float mLineWidth;
 
