@@ -80,10 +80,16 @@ class CommandQueue final : public WrappedObject<id<MTLCommandQueue>>, angle::Non
     AutoObjCPtr<id<MTLCommandBuffer>> makeMetalCommandBuffer(uint64_t *queueSerialOut);
     void onCommandBufferCommitted(id<MTLCommandBuffer> buf, uint64_t serial);
 
-    uint64_t getNextRenderEncoderSerial();
+    uint64_t allocateTimeElapsedEntry();
+    bool deleteTimeElapsedEntry(uint64_t id);
+    void setActiveTimeElapsedEntry(uint64_t id);
+    bool isTimeElapsedEntryComplete(uint64_t id);
+    double getTimeElapsedEntryInSeconds(uint64_t id);
 
   private:
-    void onCommandBufferCompleted(id<MTLCommandBuffer> buf, uint64_t serial);
+    void onCommandBufferCompleted(id<MTLCommandBuffer> buf,
+                                  uint64_t serial,
+                                  uint64_t timeElapsedEntry);
     using ParentClass = WrappedObject<id<MTLCommandQueue>>;
 
     struct CmdBufferQueueEntry
@@ -96,9 +102,30 @@ class CommandQueue final : public WrappedObject<id<MTLCommandQueue>>, angle::Non
     uint64_t mQueueSerialCounter = 1;
     std::atomic<uint64_t> mCommittedBufferSerial{0};
     std::atomic<uint64_t> mCompletedBufferSerial{0};
-    uint64_t mRenderEncoderCounter = 1;
+
+    // The bookkeeping for TIME_ELAPSED queries must be managed under
+    // the cover of a lock because it's accessed by multiple threads:
+    // the application, and the internal thread which dispatches the
+    // command buffer completed handlers. The QueryMtl object
+    // allocates and deallocates the IDs and associated storage.
+    // In-flight CommandBuffers might refer to IDs that have been
+    // deallocated. ID 0 is used as a sentinel.
+    struct TimeElapsedEntry
+    {
+        double elapsed_seconds          = 0.0;
+        int32_t pending_command_buffers = 0;
+        uint64_t id                     = 0;
+    };
+    angle::HashMap<uint64_t, TimeElapsedEntry> mTimeElapsedEntries;
+    uint64_t mTimeElapsedNextId   = 1;
+    uint64_t mActiveTimeElapsedId = 0;
 
     mutable std::mutex mLock;
+
+    void addCommandBufferToTimeElapsedEntry(std::lock_guard<std::mutex> &lg, uint64_t id);
+    void recordCommandBufferTimeElapsed(std::lock_guard<std::mutex> &lg,
+                                        uint64_t id,
+                                        double seconds);
 };
 
 class CommandBuffer final : public WrappedObject<id<MTLCommandBuffer>>, angle::NonCopyable
@@ -310,6 +337,8 @@ struct RenderCommandEncoderStates
     id<MTLDepthStencilState> depthStencilState;
     float depthBias, depthSlopeScale, depthClamp;
 
+    MTLDepthClipMode depthClipMode;
+
     uint32_t stencilFrontRef, stencilBackRef;
 
     Optional<MTLViewport> viewport;
@@ -346,6 +375,7 @@ class RenderCommandEncoder final : public CommandEncoder
 
     RenderCommandEncoder &setDepthStencilState(id<MTLDepthStencilState> state);
     RenderCommandEncoder &setDepthBias(float depthBias, float slopeScale, float clamp);
+    RenderCommandEncoder &setDepthClipMode(MTLDepthClipMode depthClipMode);
     RenderCommandEncoder &setStencilRefVals(uint32_t frontRef, uint32_t backRef);
     RenderCommandEncoder &setStencilRefVal(uint32_t ref);
 
@@ -508,8 +538,6 @@ class RenderCommandEncoder final : public CommandEncoder
     bool hasDrawCalls() const { return mHasDrawCalls; }
     bool hasPipelineState() const { return mPipelineStateSet; }
 
-    uint64_t getSerial() const { return mSerial; }
-
   private:
     // Override CommandEncoder
     id<MTLRenderCommandEncoder> get()
@@ -554,7 +582,6 @@ class RenderCommandEncoder final : public CommandEncoder
     RenderCommandEncoderStates mStateCache = {};
 
     bool mPipelineStateSet = false;
-    const uint64_t mSerial = 0;
 };
 
 class BlitCommandEncoder final : public CommandEncoder
