@@ -13,6 +13,7 @@
 #include "common/CircularBuffer.h"
 #include "common/vulkan/vk_headers.h"
 #include "libANGLE/renderer/SurfaceImpl.h"
+#include "libANGLE/renderer/vulkan/CommandProcessor.h"
 #include "libANGLE/renderer/vulkan/RenderTargetVk.h"
 #include "libANGLE/renderer/vulkan/vk_helpers.h"
 
@@ -20,7 +21,7 @@ namespace rx
 {
 class RendererVk;
 
-class SurfaceVk : public SurfaceImpl, public angle::ObserverInterface
+class SurfaceVk : public SurfaceImpl, public angle::ObserverInterface, public vk::Resource
 {
   public:
     angle::Result getAttachmentRenderTarget(const gl::Context *context,
@@ -50,6 +51,9 @@ class OffscreenSurfaceVk : public SurfaceVk
     egl::Error initialize(const egl::Display *display) override;
     void destroy(const egl::Display *display) override;
 
+    egl::Error unMakeCurrent(const gl::Context *context) override;
+    const vk::ImageHelper *getColorImage() const { return &mColorAttachment.image; }
+
     egl::Error swap(const gl::Context *context) override;
     egl::Error postSubBuffer(const gl::Context *context,
                              EGLint x,
@@ -181,13 +185,13 @@ struct SwapchainImage : angle::NonCopyable
     SwapchainImage(SwapchainImage &&other);
     ~SwapchainImage();
 
-    vk::ImageHelper image;
+    std::unique_ptr<vk::ImageHelper> image;
     vk::ImageViewHelper imageViews;
     vk::Framebuffer framebuffer;
     vk::Framebuffer fetchFramebuffer;
     vk::Framebuffer framebufferResolveMS;
 
-    uint64_t mFrameNumber = 0;
+    uint64_t frameNumber = 0;
 };
 }  // namespace impl
 
@@ -212,6 +216,9 @@ class WindowSurfaceVk : public SurfaceVk
     void destroy(const egl::Display *display) override;
 
     egl::Error initialize(const egl::Display *display) override;
+
+    egl::Error unMakeCurrent(const gl::Context *context) override;
+
     angle::Result getAttachmentRenderTarget(const gl::Context *context,
                                             GLenum binding,
                                             const gl::ImageIndex &imageIndex,
@@ -268,8 +275,6 @@ class WindowSurfaceVk : public SurfaceVk
                                         const SwapchainResolveMode swapchainResolveMode,
                                         vk::MaybeImagelessFramebuffer *framebufferOut);
 
-    const vk::Semaphore *getAndResetAcquireImageSemaphore();
-
     VkSurfaceTransformFlagBitsKHR getPreTransform() const
     {
         if (mEmulatedPreTransform != VK_SURFACE_TRANSFORM_IDENTITY_BIT_KHR)
@@ -372,6 +377,9 @@ class WindowSurfaceVk : public SurfaceVk
     // not ahead of the frame being rendered by *one* frame.
     angle::Result throttleCPU(ContextVk *contextVk, const QueueSerial &currentSubmitSerial);
 
+    // Finish all GPU operations on the surface
+    angle::Result finish(vk::Context *context);
+
     void updateOverlay(ContextVk *contextVk) const;
     bool overlayHasEnabledWidget(ContextVk *contextVk) const;
     angle::Result drawOverlay(ContextVk *contextVk, impl::SwapchainImage *image) const;
@@ -383,6 +391,7 @@ class WindowSurfaceVk : public SurfaceVk
     std::vector<vk::PresentMode> mPresentModes;
 
     VkSwapchainKHR mSwapchain;
+    vk::SwapchainStatus mSwapchainStatus;
     // Cached information used to recreate swapchains.
     vk::PresentMode mSwapchainPresentMode;         // Current swapchain mode
     vk::PresentMode mDesiredSwapchainPresentMode;  // Desired mode set through setSwapInterval()
@@ -391,6 +400,12 @@ class WindowSurfaceVk : public SurfaceVk
     VkSurfaceTransformFlagBitsKHR mEmulatedPreTransform;
     VkCompositeAlphaFlagBitsKHR mCompositeAlpha;
 
+    // Present modes that are compatible with the current mode.  If mDesiredSwapchainPresentMode is
+    // in this list, mode switch can happen without the need to recreate the swapchain.  Fast
+    // vector's size is 6, as there are currently only 6 possible present modes.
+    static constexpr uint32_t kMaxCompatiblePresentModes = 6;
+    angle::FixedVector<VkPresentModeKHR, kMaxCompatiblePresentModes> mCompatiblePresentModes;
+
     // A circular buffer that stores the serial of the submission fence of the context on every
     // swap. The CPU is throttled by waiting for the 2nd previous serial to finish.  This should
     // normally be a no-op, as the application should pace itself to avoid input lag, and is
@@ -433,10 +448,6 @@ class WindowSurfaceVk : public SurfaceVk
     // Before this wait, there were three acquire semaphores in use corresponding to frames i, i+1
     // and i+2.  Frame i+3 can reuse the semaphore of frame i.
     angle::CircularBuffer<vk::Semaphore, impl::kSwapHistorySize + 1> mAcquireImageSemaphores;
-    // A pointer to mAcquireImageSemaphores.  This is set when an image is acquired and is waited on
-    // by the next submission (which uses this image), at which point it is reset so future
-    // submissions don't wait on it until the next acquire.
-    const vk::Semaphore *mAcquireImageSemaphore;
 
     // There is no direct signal from Vulkan regarding when a Present semaphore can be be reused.
     // During window resizing when swapchains are recreated every frame, the number of in-flight
