@@ -24,7 +24,7 @@ namespace egl
 {
 EGLint ClientWaitSyncKHR(Thread *thread,
                          Display *display,
-                         Sync *syncObject,
+                         SyncID syncID,
                          EGLint flags,
                          EGLTimeKHR timeout)
 {
@@ -32,9 +32,10 @@ EGLint ClientWaitSyncKHR(Thread *thread,
                          GetDisplayIfValid(display), EGL_FALSE);
     gl::Context *currentContext = thread->getContext();
     EGLint syncStatus           = EGL_FALSE;
-    ANGLE_EGL_TRY_RETURN(
-        thread, syncObject->clientWait(display, currentContext, flags, timeout, &syncStatus),
-        "eglClientWaitSync", GetSyncIfValid(display, syncObject), EGL_FALSE);
+    Sync *sync                  = display->getSync(syncID);
+    ANGLE_EGL_TRY_RETURN(thread,
+                         sync->clientWait(display, currentContext, flags, timeout, &syncStatus),
+                         "eglClientWaitSync", GetSyncIfValid(display, syncID), EGL_FALSE);
 
     thread->setSuccess();
     return syncStatus;
@@ -134,7 +135,7 @@ EGLSyncKHR CreateSyncKHR(Thread *thread,
                          "eglCreateSyncKHR", GetDisplayIfValid(display), EGL_NO_SYNC);
 
     thread->setSuccess();
-    return static_cast<EGLSync>(syncObject);
+    return reinterpret_cast<EGLSync>(static_cast<uintptr_t>(syncObject->id().value));
 }
 
 EGLint DebugMessageControlKHR(Thread *thread,
@@ -170,23 +171,25 @@ EGLBoolean DestroyStreamKHR(Thread *thread, Display *display, Stream *streamObje
     return EGL_TRUE;
 }
 
-EGLBoolean DestroySyncKHR(Thread *thread, Display *display, Sync *syncObject)
+EGLBoolean DestroySyncKHR(Thread *thread, Display *display, SyncID syncID)
 {
+    Sync *sync = display->getSync(syncID);
     ANGLE_EGL_TRY_RETURN(thread, display->prepareForCall(), "eglDestroySync",
                          GetDisplayIfValid(display), EGL_FALSE);
-    display->destroySync(syncObject);
+    display->destroySync(sync);
 
     thread->setSuccess();
     return EGL_TRUE;
 }
 
-EGLint DupNativeFenceFDANDROID(Thread *thread, Display *display, Sync *syncObject)
+EGLint DupNativeFenceFDANDROID(Thread *thread, Display *display, SyncID syncID)
 {
     ANGLE_EGL_TRY_RETURN(thread, display->prepareForCall(), "eglDupNativeFenceFDANDROID",
                          GetDisplayIfValid(display), EGL_NO_NATIVE_FENCE_FD_ANDROID);
-    EGLint result = EGL_NO_NATIVE_FENCE_FD_ANDROID;
+    EGLint result    = EGL_NO_NATIVE_FENCE_FD_ANDROID;
+    Sync *syncObject = display->getSync(syncID);
     ANGLE_EGL_TRY_RETURN(thread, syncObject->dupNativeFenceFD(display, &result),
-                         "eglDupNativeFenceFDANDROID", GetSyncIfValid(display, syncObject),
+                         "eglDupNativeFenceFDANDROID", GetSyncIfValid(display, syncID),
                          EGL_NO_NATIVE_FENCE_FD_ANDROID);
 
     thread->setSuccess();
@@ -228,7 +231,7 @@ EGLDisplay GetPlatformDisplayEXT(Thread *thread,
 
 EGLBoolean GetSyncAttribKHR(Thread *thread,
                             Display *display,
-                            Sync *syncObject,
+                            SyncID syncObject,
                             EGLint attribute,
                             EGLint *value)
 {
@@ -527,11 +530,12 @@ void SetBlobCacheFuncsANDROID(Thread *thread,
     display->setBlobCacheFuncs(set, get);
 }
 
-EGLBoolean SignalSyncKHR(Thread *thread, Display *display, Sync *syncObject, EGLenum mode)
+EGLBoolean SignalSyncKHR(Thread *thread, Display *display, SyncID syncID, EGLenum mode)
 {
     gl::Context *currentContext = thread->getContext();
+    Sync *syncObject            = display->getSync(syncID);
     ANGLE_EGL_TRY_RETURN(thread, syncObject->signal(display, currentContext, mode),
-                         "eglSignalSyncKHR", GetSyncIfValid(display, syncObject), EGL_FALSE);
+                         "eglSignalSyncKHR", GetSyncIfValid(display, syncID), EGL_FALSE);
 
     thread->setSuccess();
     return EGL_TRUE;
@@ -636,11 +640,11 @@ EGLBoolean SwapBuffersWithDamageKHR(Thread *thread,
 
 EGLBoolean PrepareSwapBuffersANGLE(EGLDisplay dpy, EGLSurface surface)
 {
-    ANGLE_SCOPED_GLOBAL_SURFACE_LOCK();
-
-    egl::Display *dpyPacked = PackParam<egl::Display *>(dpy);
-    SurfaceID surfacePacked = PackParam<SurfaceID>(surface);
-    Thread *thread          = egl::GetCurrentThread();
+    egl::Display *dpyPacked        = PackParam<egl::Display *>(dpy);
+    SurfaceID surfacePacked        = PackParam<SurfaceID>(surface);
+    Thread *thread                 = egl::GetCurrentThread();
+    Surface *surfacePtr            = nullptr;
+    const egl::Surface *eglSurface = nullptr;
     {
         ANGLE_SCOPED_GLOBAL_LOCK();
 
@@ -652,22 +656,25 @@ EGLBoolean PrepareSwapBuffersANGLE(EGLDisplay dpy, EGLSurface surface)
 
         ANGLE_EGL_TRY_RETURN(thread, dpyPacked->prepareForCall(), "eglPrepareSwapBuffersANGLE",
                              GetDisplayIfValid(dpyPacked), EGL_FALSE);
+
+        surfacePtr = dpyPacked->getSurface(surfacePacked);
+        eglSurface = GetSurfaceIfValid(dpyPacked, surfacePacked);
     }
-    Surface *surfacePtr = dpyPacked->getSurface(surfacePacked);
     ANGLE_EGL_TRY_RETURN(thread, surfacePtr->prepareSwap(thread->getContext()), "prepareSwap",
-                         GetSurfaceIfValid(dpyPacked, surfacePacked), EGL_FALSE);
+                         eglSurface, EGL_FALSE);
 
     thread->setSuccess();
     return EGL_TRUE;
 }
 
-EGLint WaitSyncKHR(Thread *thread, Display *display, Sync *syncObject, EGLint flags)
+EGLint WaitSyncKHR(Thread *thread, Display *display, SyncID syncID, EGLint flags)
 {
     ANGLE_EGL_TRY_RETURN(thread, display->prepareForCall(), "eglWaitSync",
                          GetDisplayIfValid(display), EGL_FALSE);
     gl::Context *currentContext = thread->getContext();
+    Sync *syncObject            = display->getSync(syncID);
     ANGLE_EGL_TRY_RETURN(thread, syncObject->serverWait(display, currentContext, flags),
-                         "eglWaitSync", GetSyncIfValid(display, syncObject), EGL_FALSE);
+                         "eglWaitSync", GetSyncIfValid(display, syncID), EGL_FALSE);
 
     thread->setSuccess();
     return EGL_TRUE;
@@ -874,6 +881,16 @@ void ForceGPUSwitchANGLE(Thread *thread, Display *display, EGLint gpuIDHigh, EGL
     thread->setSuccess();
 }
 
+void WaitUntilWorkScheduledANGLE(Thread *thread, Display *display)
+{
+    ANGLE_EGL_TRY(thread, display->prepareForCall(), "eglWaitUntilWorkScheduledANGLE",
+                  GetDisplayIfValid(display));
+    ANGLE_EGL_TRY(thread, display->waitUntilWorkScheduled(), "eglWaitUntilWorkScheduledANGLE",
+                  GetDisplayIfValid(display));
+
+    thread->setSuccess();
+}
+
 EGLBoolean QueryDisplayAttribANGLE(Thread *thread,
                                    Display *display,
                                    EGLint attribute,
@@ -995,14 +1012,14 @@ EGLBoolean QueryDmaBufModifiersEXT(Thread *thread,
     return EGL_TRUE;
 }
 
-void *CopyMetalSharedEventANGLE(Thread *thread, Display *display, Sync *syncObject)
+void *CopyMetalSharedEventANGLE(Thread *thread, Display *display, SyncID syncID)
 {
     ANGLE_EGL_TRY_RETURN(thread, display->prepareForCall(), "eglCopyMetalSharedEventANGLE",
                          GetDisplayIfValid(display), nullptr);
-    void *result = nullptr;
+    void *result     = nullptr;
+    Sync *syncObject = display->getSync(syncID);
     ANGLE_EGL_TRY_RETURN(thread, syncObject->copyMetalSharedEventANGLE(display, &result),
-                         "eglCopyMetalSharedEventANGLE", GetSyncIfValid(display, syncObject),
-                         nullptr);
+                         "eglCopyMetalSharedEventANGLE", GetSyncIfValid(display, syncID), nullptr);
 
     thread->setSuccess();
     return result;
