@@ -15,7 +15,6 @@
 #include "tests/perf_tests/ANGLEPerfTestArgs.h"
 #include "tests/perf_tests/DrawCallPerfParams.h"
 #include "util/capture/frame_capture_test_utils.h"
-#include "util/capture/trace_interpreter.h"
 #include "util/capture/traces_export.h"
 #include "util/egl_loader_autogen.h"
 #include "util/png_utils.h"
@@ -140,6 +139,12 @@ class TracePerfTest : public ANGLERenderTest
                                     const EGLint *attrib_list);
     EGLBoolean onEglDestroyImage(EGLDisplay display, EGLImage image);
     EGLBoolean onEglDestroyImageKHR(EGLDisplay display, EGLImage image);
+    EGLSync onEglCreateSync(EGLDisplay dpy, EGLenum type, const EGLAttrib *attrib_list);
+    EGLSync onEglCreateSyncKHR(EGLDisplay dpy, EGLenum type, const EGLint *attrib_list);
+    EGLBoolean onEglDestroySync(EGLDisplay dpy, EGLSync sync);
+    EGLBoolean onEglDestroySyncKHR(EGLDisplay dpy, EGLSync sync);
+    EGLint onEglClientWaitSync(EGLDisplay dpy, EGLSync sync, EGLint flags, EGLTimeKHR timeout);
+    EGLint onEglClientWaitSyncKHR(EGLDisplay dpy, EGLSync sync, EGLint flags, EGLTimeKHR timeout);
     EGLint onEglGetError();
 
     void onReplayFramebufferChange(GLenum target, GLuint framebuffer);
@@ -226,7 +231,8 @@ class TracePerfTest : public ANGLERenderTest
     uint32_t mOffscreenFrameCount                                       = 0;
     uint32_t mTotalFrameCount                                           = 0;
     bool mScreenshotSaved                                               = false;
-    std::unique_ptr<TraceReplayInterface> mTraceReplay;
+    uint32_t mScreenshotFrame                                           = gScreenshotFrame;
+    std::unique_ptr<TraceLibrary> mTraceReplay;
 };
 
 TracePerfTest *gCurrentTracePerfTest = nullptr;
@@ -282,6 +288,41 @@ EGLBoolean KHRONOS_APIENTRY EglDestroyImageKHR(EGLDisplay display, EGLImage imag
     return gCurrentTracePerfTest->onEglDestroyImageKHR(display, image);
 }
 
+EGLSync KHRONOS_APIENTRY EglCreateSync(EGLDisplay dpy, EGLenum type, const EGLAttrib *attrib_list)
+{
+    return gCurrentTracePerfTest->onEglCreateSync(dpy, type, attrib_list);
+}
+
+EGLSync KHRONOS_APIENTRY EglCreateSyncKHR(EGLDisplay dpy, EGLenum type, const EGLint *attrib_list)
+{
+    return gCurrentTracePerfTest->onEglCreateSyncKHR(dpy, type, attrib_list);
+}
+
+EGLBoolean KHRONOS_APIENTRY EglDestroySync(EGLDisplay dpy, EGLSync sync)
+{
+    return gCurrentTracePerfTest->onEglDestroySync(dpy, sync);
+}
+
+EGLBoolean KHRONOS_APIENTRY EglDestroySyncKHR(EGLDisplay dpy, EGLSync sync)
+{
+    return gCurrentTracePerfTest->onEglDestroySyncKHR(dpy, sync);
+}
+
+EGLint KHRONOS_APIENTRY EglClientWaitSync(EGLDisplay dpy,
+                                          EGLSync sync,
+                                          EGLint flags,
+                                          EGLTimeKHR timeout)
+{
+    return gCurrentTracePerfTest->onEglClientWaitSync(dpy, sync, flags, timeout);
+}
+EGLint KHRONOS_APIENTRY EglClientWaitSyncKHR(EGLDisplay dpy,
+                                             EGLSync sync,
+                                             EGLint flags,
+                                             EGLTimeKHR timeout)
+{
+    return gCurrentTracePerfTest->onEglClientWaitSyncKHR(dpy, sync, flags, timeout);
+}
+
 EGLint KHRONOS_APIENTRY EglGetError()
 {
     return gCurrentTracePerfTest->onEglGetError();
@@ -599,6 +640,30 @@ angle::GenericProc KHRONOS_APIENTRY TraceLoadProc(const char *procName)
     {
         return reinterpret_cast<angle::GenericProc>(EglDestroyImageKHR);
     }
+    if (strcmp(procName, "eglCreateSync") == 0)
+    {
+        return reinterpret_cast<angle::GenericProc>(EglCreateSync);
+    }
+    if (strcmp(procName, "eglCreateSyncKHR") == 0)
+    {
+        return reinterpret_cast<angle::GenericProc>(EglCreateSyncKHR);
+    }
+    if (strcmp(procName, "eglDestroySync") == 0)
+    {
+        return reinterpret_cast<angle::GenericProc>(EglDestroySync);
+    }
+    if (strcmp(procName, "eglDestroySyncKHR") == 0)
+    {
+        return reinterpret_cast<angle::GenericProc>(EglDestroySyncKHR);
+    }
+    if (strcmp(procName, "eglClientWaitSync") == 0)
+    {
+        return reinterpret_cast<angle::GenericProc>(EglClientWaitSync);
+    }
+    if (strcmp(procName, "eglClientWaitSyncKHR") == 0)
+    {
+        return reinterpret_cast<angle::GenericProc>(EglClientWaitSyncKHR);
+    }
     if (strcmp(procName, "eglGetError") == 0)
     {
         return reinterpret_cast<angle::GenericProc>(EglGetError);
@@ -773,6 +838,30 @@ TracePerfTest::TracePerfTest(std::unique_ptr<const TracePerfParams> params)
       mStartFrame(0),
       mEndFrame(0)
 {
+    bool isAMD      = IsAMD() && !mParams->isSwiftshader();
+    bool isAMDLinux = isAMD && IsLinux();
+    // bool isAMDLinuxANGLE  = isAMDLinux && mParams->isANGLE();
+    bool isAMDLinuxNative = isAMDLinux && !mParams->isANGLE();
+    bool isAMDWin         = isAMD && IsWindows();
+    bool isAMDWinANGLE    = isAMDWin && mParams->isANGLE();
+    // bool isAMDWinNative   = isAMDWin && !mParams->isANGLE();
+
+    bool isIntel            = IsIntel() && !mParams->isSwiftshader();
+    bool isIntelLinux       = isIntel && IsLinux();
+    bool isIntelLinuxANGLE  = isIntelLinux && mParams->isANGLE();
+    bool isIntelLinuxNative = isIntelLinux && !mParams->isANGLE();
+    bool isIntelWin         = IsWindows() && isIntel;
+    bool isIntelWinANGLE    = isIntelWin && mParams->isANGLE();
+    bool isIntelWinNative   = isIntelWin && !mParams->isANGLE();
+
+    bool isNVIDIA            = IsNVIDIA() && !mParams->isSwiftshader();
+    bool isNVIDIALinux       = isNVIDIA && IsLinux();
+    bool isNVIDIALinuxANGLE  = isNVIDIALinux && mParams->isANGLE();
+    bool isNVIDIALinuxNative = isNVIDIALinux && !mParams->isANGLE();
+    bool isNVIDIAWin         = isNVIDIA && IsWindows();
+    bool isNVIDIAWinANGLE    = isNVIDIAWin && mParams->isANGLE();
+    bool isNVIDIAWinNative   = isNVIDIAWin && !mParams->isANGLE();
+
     if (!mParams->traceInfo.initialized)
     {
         failTest("Failed to load trace json.");
@@ -784,14 +873,36 @@ TracePerfTest::TracePerfTest(std::unique_ptr<const TracePerfParams> params)
         addExtensionPrerequisite(extension);
     }
 
-    if (IsWindows() && IsIntel() && mParams->isVulkan() && traceNameIs("manhattan_10"))
+    if (!mParams->traceInfo.keyFrames.empty())
+    {
+        // Only support one keyFrame for now
+        if (mParams->traceInfo.keyFrames.size() != 1)
+        {
+            WARN() << "Multiple keyframes detected, only using the first";
+        }
+
+        // Only use keyFrame if the user didn't specify a value.
+        if (gScreenshotFrame == kDefaultScreenshotFrame)
+        {
+            mScreenshotFrame = mParams->traceInfo.keyFrames[0];
+            INFO() << "Trace contains keyframe, using frame " << mScreenshotFrame
+                   << " for screenshot";
+        }
+        else
+        {
+            WARN() << "Ignoring keyframe, user requested frame " << mScreenshotFrame
+                   << " for screenshot";
+        }
+    }
+
+    if (isIntelWinANGLE && traceNameIs("manhattan_10"))
     {
         skipTest(
             "TODO: http://anglebug.com/4533 This fails after the upgrade to the 26.20.100.7870 "
             "driver");
     }
 
-    if (IsWindows() && IsIntel() && !mParams->isANGLE() && traceNameIs("angry_birds_2_1500"))
+    if (isIntelWinNative && traceNameIs("angry_birds_2_1500"))
     {
         skipTest("TODO: http://anglebug.com/4731 Fails on older Intel drivers. Passes in newer");
     }
@@ -809,12 +920,22 @@ TracePerfTest::TracePerfTest(std::unique_ptr<const TracePerfParams> params)
         // TODO: http://anglebug.com/4731 This extension is missing on older Intel drivers.
         addExtensionPrerequisite("GL_OES_EGL_image_external");
 
-        if (IsWindows() && IsIntel())
+        if (isIntelWin)
         {
             skipTest("http://anglebug.com/6568 Flaky on Intel/windows");
         }
     }
 
+    if (isIntelWinANGLE && traceNameIs("black_desert_mobile"))
+    {
+        skipTest("TODO: http://anglebug.com/7879 Non-deterministic image on 31.0.101.2111 driver");
+    }
+
+    if (isIntelWinANGLE && traceNameIs("the_gardens_between"))
+    {
+        skipTest("TODO: http://anglebug.com/7879 Non-deterministic image on 31.0.101.2111 driver");
+    }
+
     if (traceNameIs("brawl_stars"))
     {
         addExtensionPrerequisite("GL_EXT_shadow_samplers");
@@ -855,7 +976,7 @@ TracePerfTest::TracePerfTest(std::unique_ptr<const TracePerfParams> params)
     {
         addExtensionPrerequisite("GL_EXT_shadow_samplers");
 
-        if (IsLinux() && IsIntel() && mParams->isVulkan())
+        if (isIntelLinuxANGLE)
         {
             skipTest(
                 "TODO: https://anglebug.com/5517 Linux+Intel generates 'Framebuffer is incomplete' "
@@ -877,7 +998,7 @@ TracePerfTest::TracePerfTest(std::unique_ptr<const TracePerfParams> params)
         // Intel doesn't support external images.
         addExtensionPrerequisite("GL_OES_EGL_image_external");
 
-        if (IsLinux() && (IsIntel() || IsAMD()) && mParams->driver != GLESDriverType::AngleEGL)
+        if (isIntelLinuxNative || isAMDLinuxNative)
         {
             skipTest("http://anglebug.com/5822 Failing on Linux Intel and AMD due to invalid enum");
         }
@@ -895,7 +1016,7 @@ TracePerfTest::TracePerfTest(std::unique_ptr<const TracePerfParams> params)
 
     if (traceNameIs("efootball_pes_2021"))
     {
-        if (mParams->isVulkan() && IsLinux() && IsIntel())
+        if (isIntelLinuxANGLE)
         {
             skipTest(
                 "TODO: https://anglebug.com/5517 Linux+Intel generate 'Framebuffer is "
@@ -916,7 +1037,7 @@ TracePerfTest::TracePerfTest(std::unique_ptr<const TracePerfParams> params)
 
     if (traceNameIs("happy_color"))
     {
-        if (IsWindows() && IsAMD() && mParams->isVulkan())
+        if (isAMDWinANGLE)
         {
             skipTest("http://anglebug.com/5623 Generates incorrect results on AMD Windows Vulkan");
         }
@@ -924,7 +1045,7 @@ TracePerfTest::TracePerfTest(std::unique_ptr<const TracePerfParams> params)
 
     if (traceNameIs("bus_simulator_indonesia"))
     {
-        if (IsLinux() && (IsIntel() || IsAMD()) && !mParams->isVulkan())
+        if (isIntelLinuxNative || isAMDLinuxNative)
         {
             skipTest("TODO: https://anglebug.com/5629 native GLES generates GL_INVALID_OPERATION");
         }
@@ -932,7 +1053,7 @@ TracePerfTest::TracePerfTest(std::unique_ptr<const TracePerfParams> params)
 
     if (traceNameIs("messenger_lite"))
     {
-        if (IsWindows() && IsNVIDIA() && mParams->isVulkan() && !mParams->isSwiftshader())
+        if (isNVIDIAWinANGLE)
         {
             skipTest(
                 "https://anglebug.com/5663 Incorrect pixels on NVIDIA Windows for first frame");
@@ -946,13 +1067,13 @@ TracePerfTest::TracePerfTest(std::unique_ptr<const TracePerfParams> params)
 
     if (traceNameIs("car_parking_multiplayer"))
     {
-        if (IsNVIDIA() && !mParams->isVulkan())
+        if (isNVIDIAWinNative || isNVIDIALinuxNative)
         {
             skipTest(
                 "TODO: https://anglebug.com/5613 NVIDIA native driver spews undefined behavior "
                 "warnings");
         }
-        if (IsWindows() && IsIntel() && mParams->isVulkan())
+        if (isIntelWinANGLE)
         {
             skipTest("https://anglebug.com/5724 Device lost on Win Intel");
         }
@@ -960,7 +1081,7 @@ TracePerfTest::TracePerfTest(std::unique_ptr<const TracePerfParams> params)
 
     if (traceNameIs("fifa_mobile"))
     {
-        if (IsWindows() && IsIntel() && mParams->isVulkan())
+        if (isIntelWinANGLE)
         {
             skipTest(
                 "TODO: http://anglebug.com/5875 Intel Windows Vulkan flakily renders entirely "
@@ -975,7 +1096,7 @@ TracePerfTest::TracePerfTest(std::unique_ptr<const TracePerfParams> params)
 
     if (traceNameIs("plants_vs_zombies_2"))
     {
-        if (IsWindows() && IsAMD() && mParams->isVulkan())
+        if (isAMDWinANGLE)
         {
             skipTest("TODO: http://crbug.com/1187752 Corrupted image");
         }
@@ -997,7 +1118,7 @@ TracePerfTest::TracePerfTest(std::unique_ptr<const TracePerfParams> params)
         addExtensionPrerequisite("GL_OES_EGL_image_external");
         addExtensionPrerequisite("GL_KHR_texture_compression_astc_ldr");
 
-        if (IsLinux() && IsIntel() && mParams->isVulkan())
+        if (isIntelLinuxANGLE)
         {
             skipTest("TODO: http://anglebug.com/5815 Trace is crashing on Intel Linux");
         }
@@ -1012,7 +1133,7 @@ TracePerfTest::TracePerfTest(std::unique_ptr<const TracePerfParams> params)
     {
         addExtensionPrerequisite("GL_OES_EGL_image_external");
 
-        if ((IsLinux() && IsIntel()) && mParams->isVulkan())
+        if (isIntelLinuxANGLE)
         {
             skipTest(
                 "TODO: http://anglebug.com/5807 Intel Linux errors with 'Framebuffer is "
@@ -1020,10 +1141,11 @@ TracePerfTest::TracePerfTest(std::unique_ptr<const TracePerfParams> params)
         }
     }
 
-    if (traceNameIs("hill_climb_racing") || traceNameIs("dead_trigger_2"))
+    if (traceNameIs("hill_climb_racing") || traceNameIs("dead_trigger_2") ||
+        traceNameIs("disney_mirrorverse") || traceNameIs("cut_the_rope") ||
+        traceNameIs("geometry_dash"))
     {
-        if (IsAndroid() && (IsPixel4() || IsPixel4XL()) &&
-            mParams->driver == GLESDriverType::SystemEGL)
+        if (IsAndroid() && (IsPixel4() || IsPixel4XL()) && !mParams->isANGLE())
         {
             skipTest(
                 "http://anglebug.com/5823 Adreno gives a driver error with empty/small draw calls");
@@ -1037,7 +1159,7 @@ TracePerfTest::TracePerfTest(std::unique_ptr<const TracePerfParams> params)
 
     if (traceNameIs("professional_baseball_spirits"))
     {
-        if (IsLinux() && (IsAMD() || IsIntel()) && mParams->isVulkan() && !mParams->isSwiftshader())
+        if (isIntelLinuxANGLE || isAMDLinuxNative)
         {
             skipTest(
                 "TODO: https://anglebug.com/5827 Linux+Mesa/RADV Vulkan generates "
@@ -1048,7 +1170,7 @@ TracePerfTest::TracePerfTest(std::unique_ptr<const TracePerfParams> params)
 
     if (traceNameIs("call_break_offline_card_game"))
     {
-        if ((IsLinux() && IsIntel()) && mParams->isVulkan())
+        if (isIntelLinuxANGLE)
         {
             skipTest(
                 "TODO: http://anglebug.com/5837 Intel Linux Vulkan errors with 'Framebuffer is "
@@ -1063,7 +1185,7 @@ TracePerfTest::TracePerfTest(std::unique_ptr<const TracePerfParams> params)
 
     if (traceNameIs("summoners_war"))
     {
-        if (IsWindows() && IsIntel() && mParams->driver != GLESDriverType::AngleEGL)
+        if (isIntelWinANGLE)
         {
             skipTest("TODO: http://anglebug.com/5943 GL_INVALID_ENUM on Windows/Intel");
         }
@@ -1074,12 +1196,12 @@ TracePerfTest::TracePerfTest(std::unique_ptr<const TracePerfParams> params)
         addExtensionPrerequisite("GL_EXT_texture_cube_map_array");
         addExtensionPrerequisite("GL_KHR_texture_compression_astc_ldr");
 
-        if (IsLinux() && IsIntel() && mParams->isVulkan())
+        if (isIntelLinuxANGLE)
         {
             skipTest("TODO: http://anglebug.com/5989 Intel Linux crashing on teardown");
         }
 
-        if (IsWindows() && IsIntel() && mParams->isVulkan())
+        if (isIntelWinANGLE)
         {
             skipTest("TODO: http://anglebug.com/5994 Intel Windows timing out periodically");
         }
@@ -1095,12 +1217,12 @@ TracePerfTest::TracePerfTest(std::unique_ptr<const TracePerfParams> params)
     {
         addExtensionPrerequisite("GL_KHR_texture_compression_astc_ldr");
 
-        if (IsNVIDIA() && mParams->isVulkan())
+        if (isNVIDIAWinANGLE || isNVIDIALinuxANGLE)
         {
             skipTest("http://anglebug.com/7496 Nondeterministic noise between runs");
         }
 
-        if ((IsLinux() && IsIntel()) && mParams->isVulkan())
+        if (isIntelLinuxANGLE)
         {
             skipTest("TODO: http://anglebug.com/6029 Crashes on Linux Intel Vulkan");
         }
@@ -1113,7 +1235,7 @@ TracePerfTest::TracePerfTest(std::unique_ptr<const TracePerfParams> params)
 
     if (traceNameIs("mario_kart_tour"))
     {
-        if (IsLinux() && IsIntel() && !mParams->isVulkan())
+        if (isIntelLinuxNative)
         {
             skipTest("http://anglebug.com/6711 Fails on native Mesa");
         }
@@ -1123,7 +1245,7 @@ TracePerfTest::TracePerfTest(std::unique_ptr<const TracePerfParams> params)
     {
         addExtensionPrerequisite("GL_EXT_texture_buffer");
 
-        if (((IsWindows() && IsIntel()) || IsNVIDIA()) && !mParams->isVulkan())
+        if (isIntelWinNative || isNVIDIALinuxNative || isNVIDIAWinNative)
         {
             skipTest("TODO: http://anglebug.com/6240 Internal errors on Windows/Intel and NVIDIA");
         }
@@ -1131,7 +1253,7 @@ TracePerfTest::TracePerfTest(std::unique_ptr<const TracePerfParams> params)
 
     if (traceNameIs("sakura_school_simulator"))
     {
-        if (IsWindows() && IsIntel())
+        if (isIntelWin)
         {
             skipTest("http://anglebug.com/6294 Flaky on Intel");
         }
@@ -1145,7 +1267,7 @@ TracePerfTest::TracePerfTest(std::unique_ptr<const TracePerfParams> params)
     if (traceNameIs("world_of_kings"))
     {
         addExtensionPrerequisite("GL_OES_EGL_image_external");
-        if (IsWindows() && IsIntel())
+        if (isIntelWin)
         {
             skipTest("http://anglebug.com/6372 Flaky on Intel");
         }
@@ -1182,7 +1304,7 @@ TracePerfTest::TracePerfTest(std::unique_ptr<const TracePerfParams> params)
     {
         addExtensionPrerequisite("GL_KHR_texture_compression_astc_ldr");
 
-        if (IsLinux() && IsIntel() && !mParams->isVulkan())
+        if (isIntelLinuxNative)
         {
             skipTest("http://anglebug.com/6657 Native test timing out on Intel Linux");
         }
@@ -1190,8 +1312,7 @@ TracePerfTest::TracePerfTest(std::unique_ptr<const TracePerfParams> params)
 
     if (traceNameIs("zillow"))
     {
-        if ((IsLinux() || IsWindows()) && IsNVIDIA() &&
-            mParams->driver == GLESDriverType::AngleEGL && !mParams->isSwiftshader())
+        if (isNVIDIAWinANGLE || isNVIDIALinuxANGLE)
         {
             skipTest("http://anglebug.com/6658 Crashing in Vulkan backend");
         }
@@ -1209,7 +1330,7 @@ TracePerfTest::TracePerfTest(std::unique_ptr<const TracePerfParams> params)
 
     if (traceNameIs("pubg_mobile_launch"))
     {
-        if (IsLinux() && IsNVIDIA() && mParams->driver != GLESDriverType::AngleEGL)
+        if (isNVIDIALinuxNative)
         {
             skipTest("http://anglebug.com/6850 Crashing in Nvidia GLES driver");
         }
@@ -1241,7 +1362,7 @@ TracePerfTest::TracePerfTest(std::unique_ptr<const TracePerfParams> params)
 
     if (traceNameIs("marvel_strike_force"))
     {
-        if ((IsAndroid() && IsQualcomm()) && mParams->driver != GLESDriverType::AngleEGL)
+        if ((IsAndroid() && IsQualcomm()) && !mParams->isANGLE())
         {
             skipTest(
                 "http://anglebug.com/7017 Qualcomm native driver gets confused about the state of "
@@ -1268,17 +1389,17 @@ TracePerfTest::TracePerfTest(std::unique_ptr<const TracePerfParams> params)
 
     if (traceNameIs("car_chase"))
     {
-        if (IsWindows() && IsIntel())
+        if (isIntelWin)
         {
             skipTest("http://anglebug.com/7173 Fails on Intel HD 630 Mobile");
         }
 
-        if (IsLinux() && IsIntel())
+        if (isIntelLinux)
         {
             skipTest("http://anglebug.com/7125#c8 Flaky hang on UHD630 Mesa 20.0.8");
         }
 
-        if (IsNVIDIA() && mParams->isVulkan())
+        if (isNVIDIAWinANGLE || isNVIDIALinuxANGLE)
         {
             skipTest("http://anglebug.com/7125 Renders incorrectly on NVIDIA");
         }
@@ -1302,7 +1423,7 @@ TracePerfTest::TracePerfTest(std::unique_ptr<const TracePerfParams> params)
 
     if (traceNameIs("tessellation"))
     {
-        if (IsNVIDIA() && mParams->isVulkan())
+        if (isNVIDIAWinANGLE || isNVIDIALinuxANGLE)
         {
             skipTest("http://anglebug.com/7240 Tessellation driver bugs on Nvidia");
         }
@@ -1355,7 +1476,7 @@ TracePerfTest::TracePerfTest(std::unique_ptr<const TracePerfParams> params)
 
     if (traceNameIs("life_is_strange"))
     {
-        if (IsWindows() && IsNVIDIA() && mParams->isVulkan() && !mParams->isSwiftshader())
+        if (isNVIDIAWinANGLE)
         {
             skipTest("http://anglebug.com/7723 Renders incorrectly on Nvidia Windows");
         }
@@ -1366,12 +1487,12 @@ TracePerfTest::TracePerfTest(std::unique_ptr<const TracePerfParams> params)
 
     if (traceNameIs("survivor_io"))
     {
-        if (IsWindows() && IsNVIDIA() && mParams->isVulkan() && !mParams->isSwiftshader())
+        if (isNVIDIAWinANGLE)
         {
             skipTest("http://anglebug.com/7733 Renders incorrectly on Nvidia Windows");
         }
 
-        if (IsWindows() && IsIntel() && mParams->driver != GLESDriverType::AngleEGL)
+        if (isIntelWinNative)
         {
             skipTest(
                 "http://anglebug.com/7737 Programs fail to link on Intel Windows native driver, "
@@ -1379,6 +1500,99 @@ TracePerfTest::TracePerfTest(std::unique_ptr<const TracePerfParams> params)
         }
     }
 
+    if (traceNameIs("minetest"))
+    {
+        addExtensionPrerequisite("GL_EXT_texture_format_BGRA8888");
+        addIntegerPrerequisite(GL_MAX_TEXTURE_UNITS, 4);
+    }
+
+    if (traceNameIs("diablo_immortal"))
+    {
+        addExtensionPrerequisite("GL_EXT_shader_framebuffer_fetch");
+    }
+
+    if (traceNameIs("mu_origin_3"))
+    {
+        addExtensionPrerequisite("GL_EXT_texture_buffer");
+        addExtensionPrerequisite("GL_EXT_shader_framebuffer_fetch");
+        addExtensionPrerequisite("GL_OES_EGL_image_external");
+    }
+
+    if (traceNameIs("catalyst_black"))
+    {
+        addExtensionPrerequisite("GL_EXT_shader_framebuffer_fetch");
+    }
+
+    if (traceNameIs("five_nights_at_freddys"))
+    {
+        if (isIntelWinANGLE)
+        {
+            skipTest("http://anglebug.com/7929 Too slow on Win Intel Vulkan");
+        }
+    }
+
+    if (traceNameIs("pubg_mobile_launch"))
+    {
+        if (isIntelWinNative)
+        {
+            skipTest("http://anglebug.com/7929 Too slow on Win Intel native");
+        }
+    }
+
+    if (traceNameIs("beach_buggy_racing"))
+    {
+        if (isIntelWinANGLE)
+        {
+            skipTest("http://anglebug.com/7934 Flaky context lost on Win Intel Vulkan");
+        }
+    }
+
+    if (traceNameIs("aliexpress"))
+    {
+        if (isIntelWinNative)
+        {
+            skipTest("http://anglebug.com/7934 Flaky failure on Win Intel native");
+        }
+    }
+
+    if (traceNameIs("final_fantasy"))
+    {
+        if (IsAndroid() && IsPixel6() && !mParams->isANGLE())
+        {
+            skipTest("http://anglebug.com/7936 Crashes on Pixel 6 native");
+        }
+    }
+
+    if (traceNameIs("limbo"))
+    {
+        addExtensionPrerequisite("GL_EXT_shader_framebuffer_fetch");
+
+        // For LUMINANCE8_ALPHA8_EXT
+        addExtensionPrerequisite("GL_EXT_texture_storage");
+    }
+
+    if (traceNameIs("into_the_dead_2"))
+    {
+        if (isNVIDIAWinANGLE)
+        {
+            skipTest("http://anglebug.com/8042 Non-deterministic trace");
+        }
+    }
+
+    if (traceNameIs("arknights"))
+    {
+        // Intel doesn't support external images.
+        addExtensionPrerequisite("GL_OES_EGL_image_external");
+    }
+
+    if (traceNameIs("street_fighter_duel"))
+    {
+        if (isNVIDIAWinANGLE)
+        {
+            skipTest("https://anglebug.com/8074 NVIDIA Windows flaky diffs");
+        }
+    }
+
     // glDebugMessageControlKHR and glDebugMessageCallbackKHR crash on ARM GLES1.
     if (IsARM() && mParams->traceInfo.contextClientMajorVersion == 1)
     {
@@ -1394,6 +1608,27 @@ TracePerfTest::TracePerfTest(std::unique_ptr<const TracePerfParams> params)
     {
         mStepsToRun = frameCount();
     }
+
+    if (gWarmupSteps == kAllFrames)
+    {
+        mWarmupSteps = frameCount();
+    }
+
+    if (gRunToKeyFrame)
+    {
+        if (mParams->traceInfo.keyFrames.empty())
+        {
+            // If we don't have a keyFrame, run one step
+            INFO() << "No keyframe available for trace, running to frame 1";
+            mStepsToRun = 1;
+        }
+        else
+        {
+            int keyFrame = mParams->traceInfo.keyFrames[0];
+            INFO() << "Running to keyframe: " << keyFrame;
+            mStepsToRun = keyFrame;
+        }
+    }
 }
 
 void TracePerfTest::startTest()
@@ -1402,6 +1637,20 @@ void TracePerfTest::startTest()
     ASSERT(mCurrentFrame == mStartFrame);
 }
 
+std::string FindTraceGzPath(const std::string &traceName)
+{
+    std::stringstream pathStream;
+
+    char genDir[kMaxPath] = {};
+    if (!angle::FindTestDataPath("gen", genDir, kMaxPath))
+    {
+        return "";
+    }
+    pathStream << genDir << angle::GetPathSeparator() << "tracegz_" << traceName << ".gz";
+
+    return pathStream.str();
+}
+
 void TracePerfTest::initializeBenchmark()
 {
     const TraceInfo &traceInfo = mParams->traceInfo;
@@ -1415,14 +1664,24 @@ void TracePerfTest::initializeBenchmark()
 
     if (gTraceInterpreter)
     {
-        mTraceReplay.reset(new TraceInterpreter(traceInfo, testDataDir, gVerboseLogging));
+        mTraceReplay.reset(new TraceLibrary("angle_trace_interpreter", traceInfo));
+        if (strcmp(gTraceInterpreter, "gz") == 0)
+        {
+            std::string traceGzPath = FindTraceGzPath(traceInfo.name);
+            if (traceGzPath.empty())
+            {
+                failTest("Could not find trace gz.");
+                return;
+            }
+            mTraceReplay->setTraceGzPath(traceGzPath);
+        }
     }
     else
     {
         std::stringstream traceNameStr;
         traceNameStr << "angle_restricted_traces_" << traceInfo.name;
         std::string traceName = traceNameStr.str();
-        mTraceReplay.reset(new TraceLibrary(traceName.c_str()));
+        mTraceReplay.reset(new TraceLibrary(traceNameStr.str(), traceInfo));
     }
 
     LoadTraceEGL(TraceLoadProc);
@@ -1808,6 +2067,42 @@ EGLBoolean TracePerfTest::onEglDestroyImageKHR(EGLDisplay display, EGLImage imag
     return getGLWindow()->destroyImageKHR(image);
 }
 
+EGLSync TracePerfTest::onEglCreateSync(EGLDisplay dpy, EGLenum type, const EGLAttrib *attrib_list)
+{
+    return getGLWindow()->createSync(dpy, type, attrib_list);
+}
+
+EGLSync TracePerfTest::onEglCreateSyncKHR(EGLDisplay dpy, EGLenum type, const EGLint *attrib_list)
+{
+    return getGLWindow()->createSyncKHR(dpy, type, attrib_list);
+}
+
+EGLBoolean TracePerfTest::onEglDestroySync(EGLDisplay dpy, EGLSync sync)
+{
+    return getGLWindow()->destroySync(dpy, sync);
+}
+
+EGLBoolean TracePerfTest::onEglDestroySyncKHR(EGLDisplay dpy, EGLSync sync)
+{
+    return getGLWindow()->destroySyncKHR(dpy, sync);
+}
+
+EGLint TracePerfTest::onEglClientWaitSync(EGLDisplay dpy,
+                                          EGLSync sync,
+                                          EGLint flags,
+                                          EGLTimeKHR timeout)
+{
+    return getGLWindow()->clientWaitSync(dpy, sync, flags, timeout);
+}
+
+EGLint TracePerfTest::onEglClientWaitSyncKHR(EGLDisplay dpy,
+                                             EGLSync sync,
+                                             EGLint flags,
+                                             EGLTimeKHR timeout)
+{
+    return getGLWindow()->clientWaitSyncKHR(dpy, sync, flags, timeout);
+}
+
 EGLint TracePerfTest::onEglGetError()
 {
     return getGLWindow()->getEGLError();
@@ -2090,16 +2385,16 @@ void TracePerfTest::swap()
 {
     // Capture a screenshot if enabled.
     if (gScreenshotDir != nullptr && gSaveScreenshots && !mScreenshotSaved &&
-        static_cast<uint32_t>(gScreenshotFrame) == mCurrentIteration)
+        mScreenshotFrame == mCurrentIteration)
     {
         std::stringstream screenshotNameStr;
         screenshotNameStr << gScreenshotDir << GetPathSeparator() << "angle" << mBackend << "_"
                           << mStory;
 
         // Add a marker to the name for any screenshot that isn't start frame
-        if (mStartFrame != static_cast<uint32_t>(gScreenshotFrame))
+        if (mStartFrame != mScreenshotFrame)
         {
-            screenshotNameStr << "_frame" << gScreenshotFrame;
+            screenshotNameStr << "_frame" << mScreenshotFrame;
         }
 
         screenshotNameStr << ".png";
@@ -2161,7 +2456,7 @@ void RegisterTraceTests()
     GLESDriverType driverType = GetDriverTypeFromString(gUseGL, GLESDriverType::AngleEGL);
     GLenum platformType       = EGL_PLATFORM_ANGLE_TYPE_DEFAULT_ANGLE;
     GLenum deviceType         = EGL_PLATFORM_ANGLE_DEVICE_TYPE_HARDWARE_ANGLE;
-    if (driverType == GLESDriverType::AngleEGL)
+    if (IsANGLE(driverType))
     {
         platformType = GetPlatformANGLETypeFromArg(gUseANGLE, EGL_PLATFORM_ANGLE_TYPE_VULKAN_ANGLE);
         deviceType =
