@@ -15,10 +15,10 @@
 #include <map>
 
 #include "angle_gl.h"
+#include "common/base/anglebase/no_destructor.h"
 #include "common/debug.h"
 #include "common/platform.h"
 #include "common/system_utils.h"
-#include "common/third_party/base/anglebase/no_destructor.h"
 #include "gpu_info_util/SystemInfo.h"
 #include "test_utils/angle_test_configs.h"
 #include "util/EGLWindow.h"
@@ -38,7 +38,9 @@ namespace angle
 {
 namespace
 {
-bool IsAngleEGLConfigSupported(const PlatformParameters &param, OSWindow *osWindow)
+bool IsEGLConfigSupported(const PlatformParameters &param,
+                          OSWindow *osWindow,
+                          const char *eglLibraryName)
 {
     std::unique_ptr<angle::Library> eglLibrary;
 
@@ -58,6 +60,16 @@ bool IsAngleEGLConfigSupported(const PlatformParameters &param, OSWindow *osWind
     return result;
 }
 
+bool IsAngleEGLConfigSupported(const PlatformParameters &param, OSWindow *osWindow)
+{
+    return IsEGLConfigSupported(param, osWindow, ANGLE_EGL_LIBRARY_NAME);
+}
+
+bool IsAngleVulkanSecondariesEGLConfigSupported(const PlatformParameters &param, OSWindow *osWindow)
+{
+    return IsEGLConfigSupported(param, osWindow, ANGLE_VULKAN_SECONDARIES_EGL_LIBRARY_NAME);
+}
+
 bool IsSystemWGLConfigSupported(const PlatformParameters &param, OSWindow *osWindow)
 {
 #if defined(ANGLE_PLATFORM_WINDOWS) && defined(ANGLE_USE_UTIL_LOADER)
@@ -102,23 +114,7 @@ bool IsSystemEGLConfigSupported(const PlatformParameters &param, OSWindow *osWin
 
 bool IsZinkEGLConfigSupported(const PlatformParameters &param, OSWindow *osWindow)
 {
-    std::unique_ptr<angle::Library> eglLibrary;
-
-#if defined(ANGLE_USE_UTIL_LOADER)
-    eglLibrary.reset(
-        angle::OpenSharedLibrary(ANGLE_MESA_EGL_LIBRARY_NAME, angle::SearchType::ModuleDir));
-#endif
-
-    EGLWindow *eglWindow =
-        EGLWindow::New(param.clientType, param.majorVersion, param.minorVersion, param.profileMask);
-    ConfigParameters configParams;
-    bool result =
-        eglWindow->initializeGL(osWindow, eglLibrary.get(), angle::GLESDriverType::ZinkEGL,
-                                param.eglParameters, configParams);
-    eglWindow->destroyGL();
-    EGLWindow::Delete(&eglWindow);
-
-    return result;
+    return IsEGLConfigSupported(param, osWindow, ANGLE_MESA_EGL_LIBRARY_NAME);
 }
 
 bool IsAndroidDevice(const std::string &deviceName)
@@ -267,7 +263,7 @@ bool IsOSX()
 
 bool IsIOS()
 {
-#if defined(ANGLE_PLATFORM_IOS)
+#if ANGLE_PLATFORM_IOS_FAMILY
     return true;
 #else
     return false;
@@ -371,6 +367,11 @@ bool IsPixel4XL()
     return IsAndroidDevice("Pixel 4 XL");
 }
 
+bool IsPixel6()
+{
+    return IsAndroidDevice("Pixel 6");
+}
+
 bool IsNVIDIAShield()
 {
     return IsAndroidDevice("SHIELD Android TV");
@@ -477,6 +478,20 @@ bool IsConfigAllowlisted(const SystemInfo &systemInfo, const PlatformParameters
     }
 #endif
 
+    if (param.driver == GLESDriverType::AngleVulkanSecondariesEGL)
+    {
+        if (param.getRenderer() != EGL_PLATFORM_ANGLE_TYPE_VULKAN_ANGLE)
+        {
+            return false;
+        }
+        if (IsAndroid() &&
+            param.getDeviceType() == EGL_PLATFORM_ANGLE_DEVICE_TYPE_SWIFTSHADER_ANGLE)
+        {
+            return false;
+        }
+        return true;
+    }
+
     if (IsWindows())
     {
         switch (param.driver)
@@ -675,6 +690,9 @@ bool IsConfigSupported(const PlatformParameters &param)
             case GLESDriverType::AngleEGL:
                 result = IsAngleEGLConfigSupported(param, osWindow);
                 break;
+            case GLESDriverType::AngleVulkanSecondariesEGL:
+                result = IsAngleVulkanSecondariesEGLConfigSupported(param, osWindow);
+                break;
             case GLESDriverType::SystemEGL:
                 result = IsSystemEGLConfigSupported(param, osWindow);
                 break;
@@ -698,7 +716,7 @@ bool IsPlatformAvailable(const PlatformParameters &param)
     // Disable "null" device when not on ANGLE or in D3D9.
     if (param.getDeviceType() == EGL_PLATFORM_ANGLE_DEVICE_TYPE_NULL_ANGLE)
     {
-        if (param.driver != GLESDriverType::AngleEGL)
+        if (!IsANGLE(param.driver))
             return false;
         if (param.getRenderer() == EGL_PLATFORM_ANGLE_TYPE_D3D9_ANGLE)
             return false;
@@ -846,6 +864,11 @@ GLESDriverType GetDriverTypeFromString(const char *driverName, GLESDriverType de
         return GLESDriverType::AngleEGL;
     }
 
+    if (strcmp(driverName, "angle-vulkan-secondaries") == 0)
+    {
+        return GLESDriverType::AngleVulkanSecondariesEGL;
+    }
+
     if (strcmp(driverName, "zink") == 0)
     {
         return GLESDriverType::ZinkEGL;
