@@ -7,53 +7,71 @@
 //   Parser and interpreter for the C-based replays.
 //
 
-#define ANGLE_REPLAY_EXPORT
-
 #include "trace_interpreter.h"
 
+#include "anglebase/no_destructor.h"
 #include "common/gl_enum_utils.h"
 #include "common/string_utils.h"
 #include "trace_fixture.h"
 
+#define USE_SYSTEM_ZLIB
+#include "compression_utils_portable.h"
+
 namespace angle
 {
 namespace
 {
-bool ShouldSkipFile(const std::string &file)
+bool ShouldParseFile(const std::string &file)
 {
-    // Skip non-C-source files.
-    if (file.back() != 'c')
-    {
-        return true;
-    }
+    return EndsWith(file, ".c") || EndsWith(file, ".cpp");
+}
 
-    return false;
+void ReplayTraceFunction(const TraceFunction &func, const TraceFunctionMap &customFunctions)
+{
+    for (const CallCapture &call : func)
+    {
+        ReplayTraceFunctionCall(call, customFunctions);
+    }
 }
 
 class Parser : angle::NonCopyable
 {
   public:
-    Parser(const std::string &stream, bool verboseLogging)
-        : mStream(stream), mIndex(0), mVerboseLogging(verboseLogging)
+    Parser(const std::string &stream,
+           TraceFunctionMap &functionsIn,
+           TraceStringMap &stringsIn,
+           bool verboseLogging)
+        : mStream(stream),
+          mFunctions(functionsIn),
+          mStrings(stringsIn),
+          mIndex(0),
+          mVerboseLogging(verboseLogging)
     {}
 
-    void getFunctionsAndShaders(TraceFunctionMap &functionsOut, TraceShaderMap &shadersOut)
+    void parse()
     {
-        parse();
-        for (auto &iter : mFunctions)
+        while (mIndex < mStream.size())
         {
-            std::string name    = iter.first;
-            TraceFunction &func = iter.second;
-            functionsOut.emplace(std::move(name), std::move(func));
+            if (peek() == '#' || peek() == '/')
+            {
+                skipLine();
+            }
+            else if (peek() == 'v')
+            {
+                ASSERT(check("void "));
+                readFunction();
+            }
+            else if (peek() == 'c')
+            {
+                ASSERT(check("const "));
+                readMultilineString();
+            }
+            else
+            {
+                printf("Unexpected character: '%c'\n", peek());
+                UNREACHABLE();
+            }
         }
-        mFunctions.clear();
-        for (auto &iter : mShaders)
-        {
-            std::string name    = iter.first;
-            TraceShader &shader = iter.second;
-            shadersOut.emplace(std::move(name), std::move(shader));
-        }
-        mShaders.clear();
     }
 
   private:
@@ -99,7 +117,7 @@ class Parser : angle::NonCopyable
     }
 
     // In our simplified trace C, every line that begins with a } either ends a function or a
-    // shader. All lines inside the function begin with whitespace. So to find the end of the
+    // string. All lines inside the function begin with whitespace. So to find the end of the
     // function we just need to scan for a line beginning with }.
     void skipFunction()
     {
@@ -119,13 +137,21 @@ class Parser : angle::NonCopyable
             if (peek() == '\\')
             {
                 advance();
-                if (peek() == 'n')
+                switch (peek())
                 {
-                    *stringOut += '\n';
-                }
-                else
-                {
-                    UNREACHABLE();
+                    case 'n':
+                        *stringOut += '\n';
+                        break;
+                    case '\"':
+                        *stringOut += '\"';
+                        break;
+                    case '\\':
+                        *stringOut += '\\';
+                        break;
+                    default:
+                        printf("Unrecognized escape character: \\%c\n", peek());
+                        UNREACHABLE();
+                        break;
                 }
             }
             else
@@ -140,7 +166,9 @@ class Parser : angle::NonCopyable
     {
         size_t startIndex = mIndex;
         advanceTo(delim);
-        memcpy(token, &mStream[startIndex], mIndex - startIndex);
+        size_t tokenSize = mIndex - startIndex;
+        ASSERT(tokenSize < kMaxTokenSize);
+        memcpy(token, &mStream[startIndex], tokenSize);
         token[mIndex - startIndex] = 0;
     }
 
@@ -202,11 +230,13 @@ class Parser : angle::NonCopyable
                 // Skip casts.
                 if (peek() == ',' || (peek() == ')' && mIndex != tokenStart))
                 {
-                    ASSERT(numParams <= kMaxParameters);
+                    ASSERT(numParams < kMaxParameters);
+                    size_t tokenSize = mIndex - tokenStart;
+                    ASSERT(tokenSize < kMaxTokenSize);
                     Token &token = paramTokens[numParams++];
 
-                    memcpy(token, &mStream[tokenStart], mIndex - tokenStart);
-                    token[mIndex - tokenStart] = 0;
+                    memcpy(token, &mStream[tokenStart], tokenSize);
+                    token[tokenSize] = 0;
                     advance();
                     skipWhitespace();
                     skipCast();
@@ -233,8 +263,8 @@ class Parser : angle::NonCopyable
             //    printf(")\n");
             //}
 
-            // We pass in the shaders for specific use with C string array parameters.
-            CallCapture call = ParseCallCapture(nameToken, numParams, paramTokens, mShaders);
+            // We pass in the strings for specific use with C string array parameters.
+            CallCapture call = ParseCallCapture(nameToken, numParams, paramTokens, mStrings);
             func.push_back(std::move(call));
             skipLine();
         }
@@ -243,43 +273,43 @@ class Parser : angle::NonCopyable
         addFunction(funcName, func);
     }
 
-    void readShader()
+    void readMultilineString()
     {
         std::string name;
-        TraceShader shader;
+        TraceString traceStr;
 
         while (peek() != 'g')
         {
             advance();
         }
-        ASSERT(check("glShaderSource"));
+        ASSERT(check("glShaderSource") || check("glTransformFeedbackVaryings"));
 
         readStringAppend(&name, '[');
         if (mVerboseLogging)
         {
-            printf("shader: %s\n", name.c_str());
+            printf("string: %s\n", name.c_str());
         }
         skipLine();
-        std::string source;
+        std::string str;
         while (peek() != '}')
         {
             advance();
-            readStringAppend(&source, '\"');
+            readStringAppend(&str, '\"');
             advance();
             if (peek() == ',')
             {
-                shader.sources.push_back(std::move(source));
+                traceStr.strings.push_back(std::move(str));
             }
             skipLine();
         }
         skipLine();
 
-        for (const std::string &src : shader.sources)
+        for (const std::string &cppstr : traceStr.strings)
         {
-            shader.pointers.push_back(src.c_str());
+            traceStr.pointers.push_back(cppstr.c_str());
         }
 
-        mShaders[name] = std::move(shader);
+        mStrings[name] = std::move(traceStr);
     }
 
     void addFunction(const std::string &funcName, TraceFunction &func)
@@ -293,30 +323,10 @@ class Parser : angle::NonCopyable
         mFunctions[funcName] = std::move(func);
     }
 
-    void parse()
-    {
-        while (mIndex < mStream.size())
-        {
-            if (peek() == '#' || peek() == '/')
-            {
-                skipLine();
-            }
-            else if (peek() == 'v')
-            {
-                ASSERT(check("void "));
-                readFunction();
-            }
-            else
-            {
-                readShader();
-            }
-        }
-    }
-
     const std::string &mStream;
+    TraceFunctionMap &mFunctions;
+    TraceStringMap &mStrings;
     size_t mIndex;
-    TraceFunctionMap mFunctions;
-    TraceShaderMap mShaders;
     bool mVerboseLogging = false;
 };
 
@@ -351,11 +361,31 @@ void PackResourceID(ParamBuffer &params, const Token &token)
         gl::FramebufferID id = {value};
         params.addUnnamedParam(ParamType::TFramebufferID, id);
     }
+    else if (BeginsWith(token, "gSyncMap"))
+    {
+        gl::SyncID id = {value};
+        params.addUnnamedParam(ParamType::TSyncID, id);
+    }
     else if (BeginsWith(token, "gTransformFeedbackMap"))
     {
         gl::TransformFeedbackID id = {value};
         params.addUnnamedParam(ParamType::TTransformFeedbackID, id);
     }
+    else if (BeginsWith(token, "gVertexArrayMap"))
+    {
+        gl::VertexArrayID id = {value};
+        params.addUnnamedParam(ParamType::TVertexArrayID, id);
+    }
+    else if (BeginsWith(token, "gQueryMap"))
+    {
+        gl::QueryID id = {value};
+        params.addUnnamedParam(ParamType::TQueryID, id);
+    }
+    else if (BeginsWith(token, "gSamplerMap"))
+    {
+        gl::SamplerID id = {value};
+        params.addUnnamedParam(ParamType::TSamplerID, id);
+    }
     else
     {
         printf("Unknown resource map: %s\n", token);
@@ -382,31 +412,49 @@ void PackIntParameter(ParamBuffer &params, ParamType paramType, const Token &tok
     }
     else
     {
-        ASSERT(isdigit(token[0]));
-        value = static_cast<IntT>(atoi(token));
+        if (!isdigit(token[0]) && !(token[0] == '-' && isdigit(token[1])))
+        {
+            printf("Expected number, got %s\n", token);
+            UNREACHABLE();
+        }
+        if (token[0] == '0' && token[1] == 'x')
+        {
+            value = static_cast<IntT>(strtol(token, nullptr, 16));
+        }
+        else
+        {
+            value = static_cast<IntT>(atoi(token));
+        }
     }
 
     params.addUnnamedParam(paramType, value);
 }
 
+uint32_t GetStringArrayOffset(const Token &token, const char *prefixString)
+{
+    const char *offsetString = &token[strlen(prefixString)];
+    return atoi(offsetString);
+}
+
 template <typename PointerT>
-void PackMemPointer(ParamBuffer &params,
-                    ParamType paramType,
-                    const char *offsetString,
-                    uint8_t *mem)
+void PackMemPointer(ParamBuffer &params, ParamType paramType, uint32_t offset, uint8_t *mem)
 {
     ASSERT(gBinaryData);
-    uint32_t offset = atoi(offsetString);
     params.addUnnamedParam(paramType, reinterpret_cast<PointerT>(&mem[offset]));
 }
 
 template <typename T>
 void PackMutablePointerParameter(ParamBuffer &params, ParamType paramType, const Token &token)
 {
-    if (token[0] == '&')
+    if (token[0] == '0' && token[1] == 0)
+    {
+        params.addUnnamedParam(paramType, reinterpret_cast<T *>(0));
+    }
+    else if (token[0] == '&')
     {
         ASSERT(BeginsWith(token, "&gReadBuffer[") && EndsWith(token, "]"));
-        PackMemPointer<T *>(params, paramType, &token[strlen("&gReadBuffer[")], gReadBuffer);
+        uint32_t offset = GetStringArrayOffset(token, "&gReadBuffer[");
+        PackMemPointer<T *>(params, paramType, offset, gReadBuffer);
     }
     else if (token[0] == 'g')
     {
@@ -430,7 +478,25 @@ void PackConstPointerParameter(ParamBuffer &params, ParamType paramType, const T
     else if (token[0] == '&')
     {
         ASSERT(BeginsWith(token, "&gBinaryData[") && EndsWith(token, "]"));
-        PackMemPointer<const T *>(params, paramType, &token[strlen("&gBinaryData[")], gBinaryData);
+        uint32_t offset = GetStringArrayOffset(token, "&gReadBuffer[");
+        PackMemPointer<const T *>(params, paramType, offset, gBinaryData);
+    }
+    else if (token[0] == 'g')
+    {
+        if (strcmp(token, "gResourceIDBuffer") == 0)
+        {
+            params.addUnnamedParam(paramType, reinterpret_cast<const T *>(gResourceIDBuffer));
+        }
+        else if (BeginsWith(token, "gClientArrays"))
+        {
+            uint32_t offset = GetStringArrayOffset(token, "gClientArrays[");
+            params.addUnnamedParam(paramType, reinterpret_cast<const T *>(gClientArrays[offset]));
+        }
+        else
+        {
+            printf("Unexpected token: %s\n", token);
+            UNREACHABLE();
+        }
     }
     else
     {
@@ -440,45 +506,40 @@ void PackConstPointerParameter(ParamBuffer &params, ParamType paramType, const T
                                reinterpret_cast<const T *>(static_cast<uintptr_t>(offset)));
     }
 }
-}  // anonymous namespace
 
-TraceInterpreter::TraceInterpreter(const TraceInfo &traceInfo,
-                                   const char *testDataDir,
-                                   bool verboseLogging)
-    : mTraceInfo(traceInfo), mTestDataDir(testDataDir), mVerboseLogging(verboseLogging)
-{}
-
-TraceInterpreter::~TraceInterpreter() = default;
-
-bool TraceInterpreter::valid() const
+class TraceInterpreter : angle::NonCopyable
 {
-    return true;
-}
+  public:
+    TraceInterpreter()  = default;
+    ~TraceInterpreter() = default;
 
-void TraceInterpreter::setBinaryDataDir(const char *dataDir)
-{
-    SetBinaryDataDir(dataDir);
-}
+    void replayFrame(uint32_t frameIndex);
+    void setupReplay();
+    void resetReplay();
+    const char *getSerializedContextState(uint32_t frameIndex);
 
-void TraceInterpreter::setBinaryDataDecompressCallback(DecompressCallback decompressCallback,
-                                                       DeleteCallback deleteCallback)
-{
-    SetBinaryDataDecompressCallback(decompressCallback, deleteCallback);
-}
+  private:
+    void runTraceFunction(const char *name) const;
+    void parseTraceUncompressed();
+    void parseTraceGz();
+
+    TraceFunctionMap mTraceFunctions;
+    TraceStringMap mTraceStrings;
+    bool mVerboseLogging = true;
+};
 
 void TraceInterpreter::replayFrame(uint32_t frameIndex)
 {
     char funcName[kMaxTokenSize];
-    snprintf(funcName, kMaxTokenSize, "ReplayContext%dFrame%u", mTraceInfo.windowSurfaceContextId,
-             frameIndex);
+    snprintf(funcName, kMaxTokenSize, "ReplayFrame%u", frameIndex);
     runTraceFunction(funcName);
 }
 
-void TraceInterpreter::setupReplay()
+void TraceInterpreter::parseTraceUncompressed()
 {
-    for (const std::string &file : mTraceInfo.traceFiles)
+    for (const std::string &file : gTraceFiles)
     {
-        if (ShouldSkipFile(file))
+        if (!ShouldParseFile(file))
         {
             if (mVerboseLogging)
             {
@@ -492,7 +553,7 @@ void TraceInterpreter::setupReplay()
             printf("Parsing functions from %s\n", file.c_str());
         }
         std::stringstream pathStream;
-        pathStream << mTestDataDir << GetPathSeparator() << file;
+        pathStream << gBinaryDataDir << GetPathSeparator() << file;
         std::string path = pathStream.str();
 
         std::string fileData;
@@ -501,13 +562,65 @@ void TraceInterpreter::setupReplay()
             UNREACHABLE();
         }
 
-        Parser parser(fileData, mVerboseLogging);
-        parser.getFunctionsAndShaders(mTraceFunctions, mTraceShaders);
+        Parser parser(fileData, mTraceFunctions, mTraceStrings, mVerboseLogging);
+        parser.parse();
+    }
+}
+
+void TraceInterpreter::parseTraceGz()
+{
+    if (mVerboseLogging)
+    {
+        printf("Parsing functions from %s\n", gTraceGzPath.c_str());
+    }
+
+    FILE *fp = fopen(gTraceGzPath.c_str(), "rb");
+    if (fp == 0)
+    {
+        printf("Error loading trace (gz) from: %s\n", gTraceGzPath.c_str());
+        exit(1);
+    }
+
+    fseek(fp, 0, SEEK_END);
+    long size = ftell(fp);
+    fseek(fp, 0, SEEK_SET);
+
+    std::vector<uint8_t> compressedData(size);
+    (void)fread(compressedData.data(), 1, size, fp);
+
+    uint32_t uncompressedSize =
+        zlib_internal::GetGzipUncompressedSize(compressedData.data(), compressedData.size());
+
+    std::string uncompressedData(uncompressedSize, 0);
+    uLong destLen = uncompressedSize;
+    int zResult = zlib_internal::GzipUncompressHelper((uint8_t *)uncompressedData.data(), &destLen,
+                                                      compressedData.data(),
+                                                      static_cast<uLong>(compressedData.size()));
+
+    if (zResult != Z_OK)
+    {
+        printf("Failure to decompress gz trace: %s\n", gTraceGzPath.c_str());
+        exit(1);
+    }
+
+    Parser parser(uncompressedData, mTraceFunctions, mTraceStrings, mVerboseLogging);
+    parser.parse();
+}
+
+void TraceInterpreter::setupReplay()
+{
+    if (!gTraceGzPath.empty())
+    {
+        parseTraceGz();
+    }
+    else
+    {
+        parseTraceUncompressed();
     }
 
     if (mTraceFunctions.count("SetupReplay") == 0)
     {
-        printf("Did not find a SetupReplay function to run among %zu parsed functions.",
+        printf("Did not find a SetupReplay function to run among %zu parsed functions.\n",
                mTraceFunctions.size());
         exit(1);
     }
@@ -520,11 +633,6 @@ void TraceInterpreter::resetReplay()
     runTraceFunction("ResetReplay");
 }
 
-void TraceInterpreter::finishReplay()
-{
-    FinishReplay();
-}
-
 const char *TraceInterpreter::getSerializedContextState(uint32_t frameIndex)
 {
     // TODO: Necessary for complete self-testing. http://anglebug.com/7779
@@ -532,21 +640,28 @@ const char *TraceInterpreter::getSerializedContextState(uint32_t frameIndex)
     return nullptr;
 }
 
-void TraceInterpreter::setValidateSerializedStateCallback(ValidateSerializedStateCallback callback)
-{
-    SetValidateSerializedStateCallback(callback);
-}
-
 void TraceInterpreter::runTraceFunction(const char *name) const
 {
     auto iter = mTraceFunctions.find(name);
-    ASSERT(iter != mTraceFunctions.end());
+    if (iter == mTraceFunctions.end())
+    {
+        printf("Cannot find function: %s\n", name);
+        UNREACHABLE();
+    }
     const TraceFunction &func = iter->second;
     ReplayTraceFunction(func, mTraceFunctions);
 }
 
+TraceInterpreter &GetInterpreter()
+{
+    static angle::base::NoDestructor<std::unique_ptr<TraceInterpreter>> sTraceInterpreter(
+        new TraceInterpreter());
+    return *sTraceInterpreter.get()->get();
+}
+}  // anonymous namespace
+
 template <>
-void PackParameter<uint32_t>(ParamBuffer &params, const Token &token, const TraceShaderMap &shaders)
+void PackParameter<uint32_t>(ParamBuffer &params, const Token &token, const TraceStringMap &strings)
 {
     if (token[0] == 'g')
     {
@@ -559,7 +674,7 @@ void PackParameter<uint32_t>(ParamBuffer &params, const Token &token, const Trac
 }
 
 template <>
-void PackParameter<int32_t>(ParamBuffer &params, const Token &token, const TraceShaderMap &shaders)
+void PackParameter<int32_t>(ParamBuffer &params, const Token &token, const TraceStringMap &strings)
 {
     if (BeginsWith(token, "gUniformLocations"))
     {
@@ -576,7 +691,7 @@ void PackParameter<int32_t>(ParamBuffer &params, const Token &token, const Trace
 }
 
 template <>
-void PackParameter<void *>(ParamBuffer &params, const Token &token, const TraceShaderMap &shaders)
+void PackParameter<void *>(ParamBuffer &params, const Token &token, const TraceStringMap &strings)
 {
     void *value = 0;
     params.addUnnamedParam(ParamType::TvoidPointer, value);
@@ -585,13 +700,13 @@ void PackParameter<void *>(ParamBuffer &params, const Token &token, const TraceS
 template <>
 void PackParameter<const int32_t *>(ParamBuffer &params,
                                     const Token &token,
-                                    const TraceShaderMap &shaders)
+                                    const TraceStringMap &strings)
 {
     PackConstPointerParameter<int32_t>(params, ParamType::TGLintConstPointer, token);
 }
 
 template <>
-void PackParameter<void **>(ParamBuffer &params, const Token &token, const TraceShaderMap &shaders)
+void PackParameter<void **>(ParamBuffer &params, const Token &token, const TraceStringMap &strings)
 {
     UNREACHABLE();
 }
@@ -599,20 +714,20 @@ void PackParameter<void **>(ParamBuffer &params, const Token &token, const Trace
 template <>
 void PackParameter<int32_t *>(ParamBuffer &params,
                               const Token &token,
-                              const TraceShaderMap &shaders)
+                              const TraceStringMap &strings)
 {
-    UNREACHABLE();
+    PackMutablePointerParameter<int32_t>(params, ParamType::TGLintPointer, token);
 }
 
 template <>
-void PackParameter<uint64_t>(ParamBuffer &params, const Token &token, const TraceShaderMap &shaders)
+void PackParameter<uint64_t>(ParamBuffer &params, const Token &token, const TraceStringMap &strings)
 {
     params.addUnnamedParam(ParamType::TGLuint64,
                            static_cast<GLuint64>(std::strtoull(token, nullptr, 10)));
 }
 
 template <>
-void PackParameter<int64_t>(ParamBuffer &params, const Token &token, const TraceShaderMap &shaders)
+void PackParameter<int64_t>(ParamBuffer &params, const Token &token, const TraceStringMap &strings)
 {
     params.addUnnamedParam(ParamType::TGLint64,
                            static_cast<GLint64>(std::strtoll(token, nullptr, 10)));
@@ -621,7 +736,7 @@ void PackParameter<int64_t>(ParamBuffer &params, const Token &token, const Trace
 template <>
 void PackParameter<const int64_t *>(ParamBuffer &params,
                                     const Token &token,
-                                    const TraceShaderMap &shaders)
+                                    const TraceStringMap &strings)
 {
     UNREACHABLE();
 }
@@ -629,7 +744,7 @@ void PackParameter<const int64_t *>(ParamBuffer &params,
 template <>
 void PackParameter<int64_t *>(ParamBuffer &params,
                               const Token &token,
-                              const TraceShaderMap &shaders)
+                              const TraceStringMap &strings)
 {
     UNREACHABLE();
 }
@@ -637,7 +752,7 @@ void PackParameter<int64_t *>(ParamBuffer &params,
 template <>
 void PackParameter<uint64_t *>(ParamBuffer &params,
                                const Token &token,
-                               const TraceShaderMap &shaders)
+                               const TraceStringMap &strings)
 {
     UNREACHABLE();
 }
@@ -645,7 +760,7 @@ void PackParameter<uint64_t *>(ParamBuffer &params,
 template <>
 void PackParameter<const char *>(ParamBuffer &params,
                                  const Token &token,
-                                 const TraceShaderMap &shaders)
+                                 const TraceStringMap &strings)
 {
     if (token[0] == '"')
     {
@@ -667,7 +782,7 @@ void PackParameter<const char *>(ParamBuffer &params,
 template <>
 void PackParameter<const void *>(ParamBuffer &params,
                                  const Token &token,
-                                 const TraceShaderMap &shaders)
+                                 const TraceStringMap &strings)
 {
     PackConstPointerParameter<void>(params, ParamType::TvoidConstPointer, token);
 }
@@ -675,7 +790,7 @@ void PackParameter<const void *>(ParamBuffer &params,
 template <>
 void PackParameter<uint32_t *>(ParamBuffer &params,
                                const Token &token,
-                               const TraceShaderMap &shaders)
+                               const TraceStringMap &strings)
 {
     PackMutablePointerParameter<uint32_t>(params, ParamType::TGLuintPointer, token);
 }
@@ -683,59 +798,63 @@ void PackParameter<uint32_t *>(ParamBuffer &params,
 template <>
 void PackParameter<const uint32_t *>(ParamBuffer &params,
                                      const Token &token,
-                                     const TraceShaderMap &shaders)
+                                     const TraceStringMap &strings)
 {
     PackConstPointerParameter<uint32_t>(params, ParamType::TGLuintConstPointer, token);
 }
 
 template <>
-void PackParameter<float>(ParamBuffer &params, const Token &token, const TraceShaderMap &shaders)
+void PackParameter<float>(ParamBuffer &params, const Token &token, const TraceStringMap &strings)
 {
     params.addUnnamedParam(ParamType::TGLfloat, std::stof(token));
 }
 
 template <>
-void PackParameter<uint8_t>(ParamBuffer &params, const Token &token, const TraceShaderMap &shaders)
+void PackParameter<uint8_t>(ParamBuffer &params, const Token &token, const TraceStringMap &strings)
 {
     PackIntParameter<uint8_t>(params, ParamType::TGLubyte, token);
 }
 
 template <>
-void PackParameter<float *>(ParamBuffer &params, const Token &token, const TraceShaderMap &shaders)
+void PackParameter<float *>(ParamBuffer &params, const Token &token, const TraceStringMap &strings)
 {
-    UNREACHABLE();
+    PackMutablePointerParameter<float>(params, ParamType::TGLfloatPointer, token);
 }
 
 template <>
 void PackParameter<const float *>(ParamBuffer &params,
                                   const Token &token,
-                                  const TraceShaderMap &shaders)
+                                  const TraceStringMap &strings)
 {
     PackConstPointerParameter<float>(params, ParamType::TGLfloatConstPointer, token);
 }
 
 template <>
-void PackParameter<GLsync>(ParamBuffer &params, const Token &token, const TraceShaderMap &shaders)
+void PackParameter<GLsync>(ParamBuffer &params, const Token &token, const TraceStringMap &strings)
 {
-    UNREACHABLE();
+    PackResourceID(params, token);
 }
 
 template <>
 void PackParameter<const char *const *>(ParamBuffer &params,
                                         const Token &token,
-                                        const TraceShaderMap &shaders)
+                                        const TraceStringMap &strings)
 {
-    // Find the shader that corresponds to "token". Currently we only support shader string arrays.
-    auto iter = shaders.find(token);
-    ASSERT(iter != shaders.end());
-    const TraceShader &shader = iter->second;
-    params.addUnnamedParam(ParamType::TGLcharConstPointerPointer, shader.pointers.data());
+    // Find the string that corresponds to "token". Currently we only support string arrays.
+    auto iter = strings.find(token);
+    if (iter == strings.end())
+    {
+        printf("Could not find string: %s\n", token);
+        UNREACHABLE();
+    }
+    const TraceString &traceStr = iter->second;
+    params.addUnnamedParam(ParamType::TGLcharConstPointerPointer, traceStr.pointers.data());
 }
 
 template <>
 void PackParameter<const char **>(ParamBuffer &params,
                                   const Token &token,
-                                  const TraceShaderMap &shaders)
+                                  const TraceStringMap &strings)
 {
     UNREACHABLE();
 }
@@ -743,7 +862,7 @@ void PackParameter<const char **>(ParamBuffer &params,
 template <>
 void PackParameter<GLDEBUGPROCKHR>(ParamBuffer &params,
                                    const Token &token,
-                                   const TraceShaderMap &shaders)
+                                   const TraceStringMap &strings)
 {
     UNREACHABLE();
 }
@@ -751,7 +870,7 @@ void PackParameter<GLDEBUGPROCKHR>(ParamBuffer &params,
 template <>
 void PackParameter<EGLDEBUGPROCKHR>(ParamBuffer &params,
                                     const Token &token,
-                                    const TraceShaderMap &shaders)
+                                    const TraceStringMap &strings)
 {
     UNREACHABLE();
 }
@@ -759,7 +878,7 @@ void PackParameter<EGLDEBUGPROCKHR>(ParamBuffer &params,
 template <>
 void PackParameter<const struct AHardwareBuffer *>(ParamBuffer &params,
                                                    const Token &token,
-                                                   const TraceShaderMap &shaders)
+                                                   const TraceStringMap &strings)
 {
     UNREACHABLE();
 }
@@ -767,7 +886,7 @@ void PackParameter<const struct AHardwareBuffer *>(ParamBuffer &params,
 template <>
 void PackParameter<EGLSetBlobFuncANDROID>(ParamBuffer &params,
                                           const Token &token,
-                                          const TraceShaderMap &shaders)
+                                          const TraceStringMap &strings)
 {
     UNREACHABLE();
 }
@@ -775,13 +894,13 @@ void PackParameter<EGLSetBlobFuncANDROID>(ParamBuffer &params,
 template <>
 void PackParameter<EGLGetBlobFuncANDROID>(ParamBuffer &params,
                                           const Token &token,
-                                          const TraceShaderMap &shaders)
+                                          const TraceStringMap &strings)
 {
     UNREACHABLE();
 }
 
 template <>
-void PackParameter<int16_t>(ParamBuffer &params, const Token &token, const TraceShaderMap &shaders)
+void PackParameter<int16_t>(ParamBuffer &params, const Token &token, const TraceStringMap &strings)
 {
     PackIntParameter<int16_t>(params, ParamType::TGLshort, token);
 }
@@ -789,13 +908,13 @@ void PackParameter<int16_t>(ParamBuffer &params, const Token &token, const Trace
 template <>
 void PackParameter<const int16_t *>(ParamBuffer &params,
                                     const Token &token,
-                                    const TraceShaderMap &shaders)
+                                    const TraceStringMap &strings)
 {
     PackConstPointerParameter<int16_t>(params, ParamType::TGLshortConstPointer, token);
 }
 
 template <>
-void PackParameter<char *>(ParamBuffer &params, const Token &token, const TraceShaderMap &shaders)
+void PackParameter<char *>(ParamBuffer &params, const Token &token, const TraceStringMap &strings)
 {
     UNREACHABLE();
 }
@@ -803,15 +922,15 @@ void PackParameter<char *>(ParamBuffer &params, const Token &token, const TraceS
 template <>
 void PackParameter<unsigned char *>(ParamBuffer &params,
                                     const Token &token,
-                                    const TraceShaderMap &shaders)
+                                    const TraceStringMap &strings)
 {
-    UNREACHABLE();
+    PackMutablePointerParameter<GLubyte>(params, ParamType::TGLubytePointer, token);
 }
 
 template <>
 void PackParameter<const void *const *>(ParamBuffer &params,
                                         const Token &token,
-                                        const TraceShaderMap &shaders)
+                                        const TraceStringMap &strings)
 {
     UNREACHABLE();
 }
@@ -819,7 +938,7 @@ void PackParameter<const void *const *>(ParamBuffer &params,
 template <>
 void PackParameter<const uint64_t *>(ParamBuffer &params,
                                      const Token &token,
-                                     const TraceShaderMap &shaders)
+                                     const TraceStringMap &strings)
 {
     UNREACHABLE();
 }
@@ -828,7 +947,7 @@ void PackParameter<const uint64_t *>(ParamBuffer &params,
 template <>
 void PackParameter<EGLNativeDisplayType>(ParamBuffer &params,
                                          const Token &token,
-                                         const TraceShaderMap &shaders)
+                                         const TraceStringMap &strings)
 {
     UNREACHABLE();
 }
@@ -838,7 +957,7 @@ void PackParameter<EGLNativeDisplayType>(ParamBuffer &params,
 template <>
 void PackParameter<EGLNativeWindowType>(ParamBuffer &params,
                                         const Token &token,
-                                        const TraceShaderMap &shaders)
+                                        const TraceStringMap &strings)
 {
     UNREACHABLE();
 }
@@ -846,7 +965,7 @@ void PackParameter<EGLNativeWindowType>(ParamBuffer &params,
 template <>
 void PackParameter<EGLNativePixmapType>(ParamBuffer &params,
                                         const Token &token,
-                                        const TraceShaderMap &shaders)
+                                        const TraceStringMap &strings)
 {
     UNREACHABLE();
 }
@@ -856,19 +975,19 @@ void PackParameter<EGLNativePixmapType>(ParamBuffer &params,
 template <>
 void PackParameter<const long *>(ParamBuffer &params,
                                  const Token &token,
-                                 const TraceShaderMap &shaders)
+                                 const TraceStringMap &strings)
 {
     PackConstPointerParameter<int64_t>(params, ParamType::TGLuint64ConstPointer, token);
 }
 
 template <>
-void PackParameter<long *>(ParamBuffer &params, const Token &token, const TraceShaderMap &shaders)
+void PackParameter<long *>(ParamBuffer &params, const Token &token, const TraceStringMap &strings)
 {
     PackMutablePointerParameter<int64_t>(params, ParamType::TGLint64Pointer, token);
 }
 
 template <>
-void PackParameter<long>(ParamBuffer &params, const Token &token, const TraceShaderMap &shaders)
+void PackParameter<long>(ParamBuffer &params, const Token &token, const TraceStringMap &strings)
 {
     PackIntParameter<int64_t>(params, ParamType::TGLint64, token);
 }
@@ -876,9 +995,67 @@ void PackParameter<long>(ParamBuffer &params, const Token &token, const TraceSha
 template <>
 void PackParameter<unsigned long>(ParamBuffer &params,
                                   const Token &token,
-                                  const TraceShaderMap &shaders)
+                                  const TraceStringMap &strings)
 {
     PackIntParameter<uint64_t>(params, ParamType::TGLuint64, token);
 }
 #endif  // defined(ANGLE_PLATFORM_APPLE) || !defined(ANGLE_IS_64_BIT_CPU)
+
+GLuint GetResourceIDMapValue(ResourceIDType resourceIDType, GLuint key)
+{
+    switch (resourceIDType)
+    {
+        case ResourceIDType::Buffer:
+            return gBufferMap[key];
+        case ResourceIDType::FenceNV:
+            return gFenceNVMap[key];
+        case ResourceIDType::Framebuffer:
+            return gFramebufferMap[key];
+        case ResourceIDType::ProgramPipeline:
+            return gProgramPipelineMap[key];
+        case ResourceIDType::Query:
+            return gQueryMap[key];
+        case ResourceIDType::Renderbuffer:
+            return gRenderbufferMap[key];
+        case ResourceIDType::Sampler:
+            return gSamplerMap[key];
+        case ResourceIDType::Semaphore:
+            return gSemaphoreMap[key];
+        case ResourceIDType::ShaderProgram:
+            return gShaderProgramMap[key];
+        case ResourceIDType::Texture:
+            return gTextureMap[key];
+        case ResourceIDType::TransformFeedback:
+            return gTransformFeedbackMap[key];
+        case ResourceIDType::VertexArray:
+            return gVertexArrayMap[key];
+        default:
+            printf("Incompatible resource ID type: %d\n", static_cast<int>(resourceIDType));
+            UNREACHABLE();
+            return 0;
+    }
+}
+
 }  // namespace angle
+
+extern "C" {
+void SetupReplay()
+{
+    angle::GetInterpreter().setupReplay();
+}
+
+void ReplayFrame(uint32_t frameIndex)
+{
+    angle::GetInterpreter().replayFrame(frameIndex);
+}
+
+void ResetReplay()
+{
+    angle::GetInterpreter().resetReplay();
+}
+
+const char *GetSerializedContextState(uint32_t frameIndex)
+{
+    return angle::GetInterpreter().getSerializedContextState(frameIndex);
+}
+}  // extern "C"
