@@ -52,10 +52,10 @@ public:
     constexpr bool get(size_t, Dependency = Dependency()) const;
     constexpr void set(size_t);
     constexpr void set(size_t, bool);
-    constexpr bool testAndSet(size_t);
-    constexpr bool testAndClear(size_t);
-    constexpr bool concurrentTestAndSet(size_t, Dependency = Dependency());
-    constexpr bool concurrentTestAndClear(size_t, Dependency = Dependency());
+    constexpr bool testAndSet(size_t); // Returns the previous bit value.
+    constexpr bool testAndClear(size_t); // Returns the previous bit value.
+    constexpr bool concurrentTestAndSet(size_t, Dependency = Dependency()); // Returns the previous bit value.
+    constexpr bool concurrentTestAndClear(size_t, Dependency = Dependency()); // Returns the previous bit value.
     constexpr size_t nextPossiblyUnset(size_t) const;
     constexpr void clear(size_t);
     constexpr void clearAll();
@@ -109,11 +109,6 @@ public:
             return m_index == other.m_index;
         }
         
-        constexpr bool operator!=(const iterator& other) const
-        {
-            return !(*this == other);
-        }
-
     private:
         const Bitmap* m_bitmap;
         size_t m_index;
@@ -129,7 +124,6 @@ public:
     void setEachNthBit(size_t n, size_t start = 0, size_t end = bitmapSize);
 
     constexpr bool operator==(const Bitmap&) const;
-    constexpr bool operator!=(const Bitmap&) const;
 
     constexpr void operator|=(const Bitmap&);
     constexpr void operator&=(const Bitmap&);
@@ -186,9 +180,9 @@ inline constexpr bool Bitmap<bitmapSize, WordType>::testAndSet(size_t n)
 {
     WordType mask = one << (n % wordSize);
     size_t index = n / wordSize;
-    bool result = bits[index] & mask;
+    bool previousValue = bits[index] & mask;
     bits[index] |= mask;
-    return result;
+    return previousValue;
 }
 
 template<size_t bitmapSize, typename WordType>
@@ -196,9 +190,9 @@ inline constexpr bool Bitmap<bitmapSize, WordType>::testAndClear(size_t n)
 {
     WordType mask = one << (n % wordSize);
     size_t index = n / wordSize;
-    bool result = bits[index] & mask;
+    bool previousValue = bits[index] & mask;
     bits[index] &= ~mask;
-    return result;
+    return previousValue;
 }
 
 template<size_t bitmapSize, typename WordType>
@@ -207,6 +201,9 @@ ALWAYS_INLINE constexpr bool Bitmap<bitmapSize, WordType>::concurrentTestAndSet(
     WordType mask = one << (n % wordSize);
     size_t index = n / wordSize;
     WordType* data = dependency.consume(bits.data()) + index;
+    // transactionRelaxed() returns true if the bit was changed. If the bit was changed,
+    // then the previous bit must have been false since we're trying to set it. Hence,
+    // the result of transactionRelaxed() is the inverse of our expected result.
     return !bitwise_cast<Atomic<WordType>*>(data)->transactionRelaxed(
         [&] (WordType& value) -> bool {
             if (value & mask)
@@ -223,7 +220,10 @@ ALWAYS_INLINE constexpr bool Bitmap<bitmapSize, WordType>::concurrentTestAndClea
     WordType mask = one << (n % wordSize);
     size_t index = n / wordSize;
     WordType* data = dependency.consume(bits.data()) + index;
-    return !bitwise_cast<Atomic<WordType>*>(data)->transactionRelaxed(
+    // transactionRelaxed() returns true if the bit was changed. If the bit was changed,
+    // then the previous bit must have been true since we're trying to clear it. Hence,
+    // the result of transactionRelaxed() matches our expected result.
+    return bitwise_cast<Atomic<WordType>*>(data)->transactionRelaxed(
         [&] (WordType& value) -> bool {
             if (!(value & mask))
                 return false;
@@ -403,6 +403,21 @@ ALWAYS_INLINE constexpr void Bitmap<bitmapSize, WordType>::forEachSetBit(const F
         if (!word)
             continue;
         size_t base = i * wordSize;
+
+#if COMPILER(GCC_COMPATIBLE) && (CPU(X86_64) || CPU(ARM64))
+        // We should only use ctz() when we know that ctz() is implementated using
+        // a fast hardware instruction. Otherwise, this will actually result in
+        // worse performance.
+        while (word) {
+            size_t offset = ctz(word);
+            if constexpr (std::is_same_v<IterationStatus, decltype(func(base + offset))>) {
+                if (func(base + offset) == IterationStatus::Done)
+                    return;
+            } else
+                func(base + offset);
+            word &= ~(1ull << offset);
+        }
+#else
         for (size_t j = 0; j < wordSize; ++j) {
             if (word & 1) {
                 if constexpr (std::is_same_v<IterationStatus, decltype(func(base + j))>) {
@@ -413,6 +428,7 @@ ALWAYS_INLINE constexpr void Bitmap<bitmapSize, WordType>::forEachSetBit(const F
             }
             word >>= 1;
         }
+#endif
     }
 }
 
@@ -493,12 +509,6 @@ inline constexpr bool Bitmap<bitmapSize, WordType>::operator==(const Bitmap& oth
     return true;
 }
 
-template<size_t bitmapSize, typename WordType>
-inline constexpr bool Bitmap<bitmapSize, WordType>::operator!=(const Bitmap& other) const
-{
-    return !(*this == other);
-}
-
 template<size_t bitmapSize, typename WordType>
 inline constexpr void Bitmap<bitmapSize, WordType>::operator|=(const Bitmap& other)
 {
