@@ -73,32 +73,23 @@
 #include <wtf/DateMath.h>
 
 #include <algorithm>
-#include <limits.h>
 #include <limits>
 #include <stdint.h>
 #include <time.h>
+#include <unicode/ucal.h>
 #include <wtf/Assertions.h>
 #include <wtf/ASCIICType.h>
-#include <wtf/MathExtras.h>
-#include <wtf/StdLibExtras.h>
+#include <wtf/Language.h>
+#include <wtf/NeverDestroyed.h>
+#include <wtf/ThreadSpecific.h>
 #include <wtf/text/StringBuilder.h>
+#include <wtf/unicode/UTF8Conversion.h>
+#include <wtf/unicode/icu/ICUHelpers.h>
 
 #if OS(WINDOWS)
 #include <windows.h>
 #endif
 
-#if HAVE(ERRNO_H)
-#include <errno.h>
-#endif
-
-#if HAVE(SYS_TIME_H)
-#include <sys/time.h>
-#endif
-
-#if HAVE(SYS_TIMEB_H)
-#include <sys/timeb.h>
-#endif
-
 namespace WTF {
 
 // FIXME: Should this function go into StringCommon.h or some other header?
@@ -107,11 +98,18 @@ template<unsigned length> inline bool startsWithLettersIgnoringASCIICase(const c
     return equalLettersIgnoringASCIICase(string, lowercaseLetters, length - 1);
 }
 
+static Lock innerTimeZoneOverrideLock;
+static Vector<UChar>& innerTimeZoneOverride() WTF_REQUIRES_LOCK(innerTimeZoneOverrideLock)
+{
+    static NeverDestroyed<Vector<UChar>> timeZoneOverride;
+    return timeZoneOverride;
+}
+
 /* Constants */
 
-const char* const weekdayName[7] = { "Mon", "Tue", "Wed", "Thu", "Fri", "Sat", "Sun" };
-const char* const monthName[12] = { "Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec" };
-const char* const monthFullName[12] = { "January", "February", "March", "April", "May", "June", "July", "August", "September", "October", "November", "December" };
+const ASCIILiteral weekdayName[7] = { "Mon"_s, "Tue"_s, "Wed"_s, "Thu"_s, "Fri"_s, "Sat"_s, "Sun"_s };
+const ASCIILiteral monthName[12] = { "Jan"_s, "Feb"_s, "Mar"_s, "Apr"_s, "May"_s, "Jun"_s, "Jul"_s, "Aug"_s, "Sep"_s, "Oct"_s, "Nov"_s, "Dec"_s };
+const ASCIILiteral monthFullName[12] = { "January"_s, "February"_s, "March"_s, "April"_s, "May"_s, "June"_s, "July"_s, "August"_s, "September"_s, "October"_s, "November"_s, "December"_s };
 
 // Day of year for the first day of each month, where index 0 is January, and day 0 is January 1.
 // First for non-leap years, then for leap years.
@@ -281,15 +279,13 @@ static int32_t calculateUTCOffset()
 #if !HAVE(TM_GMTOFF)
 
 #if OS(WINDOWS)
-// Code taken from http://support.microsoft.com/kb/167296
+// Code taken from <https://learn.microsoft.com/en-us/windows/win32/sysinfo/converting-a-time-t-value-to-a-file-time>
 static void UnixTimeToFileTime(time_t t, LPFILETIME pft)
 {
-    // Note that LONGLONG is a 64-bit value
-    LONGLONG ll;
-
-    ll = Int32x32To64(t, 10000000) + 116444736000000000;
-    pft->dwLowDateTime = (DWORD)ll;
-    pft->dwHighDateTime = ll >> 32;
+    ULARGE_INTEGER timeValue;
+    timeValue.QuadPart = (t * 10000000LL) + 116444736000000000LL;
+    pft->dwLowDateTime = timeValue.LowPart;
+    pft->dwHighDateTime = timeValue.HighPart;
 }
 #endif
 
@@ -377,7 +373,7 @@ LocalTimeOffset calculateLocalTimeOffset(double ms, TimeType inputTimeType)
 
 void initializeDates()
 {
-#if !ASSERT_DISABLED
+#if ASSERT_ENABLED
     static bool alreadyInitialized;
     ASSERT(!alreadyInitialized);
     alreadyInitialized = true;
@@ -386,19 +382,19 @@ void initializeDates()
     equivalentYearForDST(2000); // Need to call once to initialize a static used in this function.
 }
 
-static inline double ymdhmsToSeconds(int year, long mon, long day, long hour, long minute, double second)
+static inline double ymdhmsToMilliseconds(int year, long mon, long day, long hour, long minute, long second, double milliseconds)
 {
     int mday = firstDayOfMonth[isLeapYear(year)][mon - 1];
     double ydays = daysFrom1970ToYear(year);
 
-    double dateSeconds = second + minute * secondsPerMinute + hour * secondsPerHour + (mday + day - 1 + ydays) * secondsPerDay;
+    double dateMilliseconds = milliseconds + second * msPerSecond + minute * (secondsPerMinute * msPerSecond) + hour * (secondsPerHour * msPerSecond) + (mday + day - 1 + ydays) * (secondsPerDay * msPerSecond);
 
     // Clamp to EcmaScript standard (ecma262/#sec-time-values-and-time-range) of
     //  +/- 100,000,000 days from 01 January, 1970.
-    if (dateSeconds < -8640000000000.0 || dateSeconds > 8640000000000.0)
+    if (dateMilliseconds < -8640000000000000.0 || dateMilliseconds > 8640000000000000.0)
         return std::numeric_limits<double>::quiet_NaN();
     
-    return dateSeconds;
+    return dateMilliseconds;
 }
 
 // We follow the recommendation of RFC 2822 to consider all
@@ -427,7 +423,7 @@ inline static void skipSpacesAndComments(const char*& s)
     int nesting = 0;
     char ch;
     while ((ch = *s)) {
-        if (!isASCIISpace(ch)) {
+        if (!isUnicodeCompatibleASCIIWhitespace(ch)) {
             if (ch == '(')
                 nesting++;
             else if (ch == ')' && nesting > 0)
@@ -489,8 +485,11 @@ static char* parseES5DatePortion(const char* currentPosition, int& year, long& m
     // This is a bit more lenient on the year string than ES5 specifies:
     // instead of restricting to 4 digits (or 6 digits with mandatory +/-),
     // it accepts any integer value. Consider this an implementation fallback.
+    bool hasNegativeYear = *currentPosition == '-';
     if (!parseInt(currentPosition, &postParsePosition, 10, &year))
-        return 0;
+        return nullptr;
+    if (!year && hasNegativeYear)
+        return nullptr;
 
     // Check for presence of -MM portion.
     if (*postParsePosition != '-')
@@ -498,11 +497,11 @@ static char* parseES5DatePortion(const char* currentPosition, int& year, long& m
     currentPosition = postParsePosition + 1;
     
     if (!isASCIIDigit(*currentPosition))
-        return 0;
+        return nullptr;
     if (!parseLong(currentPosition, &postParsePosition, 10, &month))
-        return 0;
+        return nullptr;
     if ((postParsePosition - currentPosition) != 2)
-        return 0;
+        return nullptr;
 
     // Check for presence of -DD portion.
     if (*postParsePosition != '-')
@@ -510,48 +509,48 @@ static char* parseES5DatePortion(const char* currentPosition, int& year, long& m
     currentPosition = postParsePosition + 1;
     
     if (!isASCIIDigit(*currentPosition))
-        return 0;
+        return nullptr;
     if (!parseLong(currentPosition, &postParsePosition, 10, &day))
-        return 0;
+        return nullptr;
     if ((postParsePosition - currentPosition) != 2)
-        return 0;
+        return nullptr;
     return postParsePosition;
 }
 
-// Parses a time with the format HH:mm[:ss[.sss]][Z|(+|-)00:00].
+// Parses a time with the format HH:mm[:ss[.sss]][Z|(+|-)(00:00|0000|00)].
 // Fractional seconds parsing is lenient, allows any number of digits.
 // Returns 0 if a parse error occurs, else returns the end of the parsed portion of the string.
-static char* parseES5TimePortion(char* currentPosition, long& hours, long& minutes, double& seconds, long& timeZoneSeconds)
+static char* parseES5TimePortion(char* currentPosition, long& hours, long& minutes, long& seconds, double& milliseconds, bool& isLocalTime, long& timeZoneSeconds)
 {
+    isLocalTime = false;
+
     char* postParsePosition;
     if (!isASCIIDigit(*currentPosition))
-        return 0;
+        return nullptr;
     if (!parseLong(currentPosition, &postParsePosition, 10, &hours))
-        return 0;
+        return nullptr;
     if (*postParsePosition != ':' || (postParsePosition - currentPosition) != 2)
-        return 0;
+        return nullptr;
     currentPosition = postParsePosition + 1;
     
     if (!isASCIIDigit(*currentPosition))
-        return 0;
+        return nullptr;
     if (!parseLong(currentPosition, &postParsePosition, 10, &minutes))
-        return 0;
+        return nullptr;
     if ((postParsePosition - currentPosition) != 2)
-        return 0;
+        return nullptr;
     currentPosition = postParsePosition;
 
     // Seconds are optional.
     if (*currentPosition == ':') {
         ++currentPosition;
     
-        long intSeconds;
         if (!isASCIIDigit(*currentPosition))
-            return 0;
-        if (!parseLong(currentPosition, &postParsePosition, 10, &intSeconds))
-            return 0;
+            return nullptr;
+        if (!parseLong(currentPosition, &postParsePosition, 10, &seconds))
+            return nullptr;
         if ((postParsePosition - currentPosition) != 2)
-            return 0;
-        seconds = intSeconds;
+            return nullptr;
         if (*postParsePosition == '.') {
             currentPosition = postParsePosition + 1;
             
@@ -559,15 +558,15 @@ static char* parseES5TimePortion(char* currentPosition, long& hours, long& minut
             // a reasonable interpretation guided by the given examples and RFC 3339 says "no".
             // We check the next character to avoid reading +/- timezone hours after an invalid decimal.
             if (!isASCIIDigit(*currentPosition))
-                return 0;
+                return nullptr;
             
             // We are more lenient than ES5 by accepting more or less than 3 fraction digits.
             long fracSeconds;
             if (!parseLong(currentPosition, &postParsePosition, 10, &fracSeconds))
-                return 0;
+                return nullptr;
             
             long numFracDigits = postParsePosition - currentPosition;
-            seconds += fracSeconds * pow(10.0, static_cast<double>(-numFracDigits));
+            milliseconds = fracSeconds * pow(10.0, static_cast<double>(-numFracDigits + 3));
         }
         currentPosition = postParsePosition;
     }
@@ -575,40 +574,57 @@ static char* parseES5TimePortion(char* currentPosition, long& hours, long& minut
     if (*currentPosition == 'Z')
         return currentPosition + 1;
 
+    // Parse (+|-)(00:00|0000|00).
     bool tzNegative;
     if (*currentPosition == '-')
         tzNegative = true;
     else if (*currentPosition == '+')
         tzNegative = false;
-    else
-        return currentPosition; // no timezone
+    else {
+        isLocalTime = true;
+        return currentPosition;
+    }
     ++currentPosition;
     
-    long tzHours;
-    long tzHoursAbs;
-    long tzMinutes;
+    long tzHours = 0;
+    long tzHoursAbs = 0;
+    long tzMinutes = 0;
     
     if (!isASCIIDigit(*currentPosition))
-        return 0;
+        return nullptr;
     if (!parseLong(currentPosition, &postParsePosition, 10, &tzHours))
-        return 0;
-    if (*postParsePosition != ':' || (postParsePosition - currentPosition) != 2)
-        return 0;
-    tzHoursAbs = labs(tzHours);
-    currentPosition = postParsePosition + 1;
+        return nullptr;
+    if (*postParsePosition != ':') {
+        if ((postParsePosition - currentPosition) == 2) {
+            // "00" case.
+            tzHoursAbs = labs(tzHours);
+        } else if ((postParsePosition - currentPosition) == 4) {
+            // "0000" case.
+            tzHoursAbs = labs(tzHours);
+            tzMinutes = tzHoursAbs % 100;
+            tzHoursAbs = tzHoursAbs / 100;
+        } else
+            return nullptr;
+    } else {
+        // "00:00" case.
+        if ((postParsePosition - currentPosition) != 2)
+            return nullptr;
+        tzHoursAbs = labs(tzHours);
+        currentPosition = postParsePosition + 1; // Skip ":".
     
-    if (!isASCIIDigit(*currentPosition))
-        return 0;
-    if (!parseLong(currentPosition, &postParsePosition, 10, &tzMinutes))
-        return 0;
-    if ((postParsePosition - currentPosition) != 2)
-        return 0;
+        if (!isASCIIDigit(*currentPosition))
+            return nullptr;
+        if (!parseLong(currentPosition, &postParsePosition, 10, &tzMinutes))
+            return nullptr;
+        if ((postParsePosition - currentPosition) != 2)
+            return nullptr;
+    }
     currentPosition = postParsePosition;
     
     if (tzHoursAbs > 24)
-        return 0;
+        return nullptr;
     if (tzMinutes < 0 || tzMinutes > 59)
-        return 0;
+        return nullptr;
     
     timeZoneSeconds = 60 * (tzMinutes + (60 * tzHoursAbs));
     if (tzNegative)
@@ -617,8 +633,10 @@ static char* parseES5TimePortion(char* currentPosition, long& hours, long& minut
     return currentPosition;
 }
 
-double parseES5DateFromNullTerminatedCharacters(const char* dateString)
+double parseES5DateFromNullTerminatedCharacters(const char* dateString, bool& isLocalTime)
 {
+    isLocalTime = false;
+
     // This parses a date of the form defined in ecma262/#sec-date-time-string-format
     // (similar to RFC 3339 / ISO 8601: YYYY-MM-DDTHH:mm:ss[.sss]Z).
     // In most cases it is intentionally strict (e.g. correct field widths, no stray whitespace).
@@ -631,7 +649,8 @@ double parseES5DateFromNullTerminatedCharacters(const char* dateString)
     long day = 1;
     long hours = 0;
     long minutes = 0;
-    double seconds = 0;
+    long seconds = 0;
+    double milliseconds = 0;
     long timeZoneSeconds = 0;
 
     // Parse the date YYYY[-MM[-DD]]
@@ -639,9 +658,10 @@ double parseES5DateFromNullTerminatedCharacters(const char* dateString)
     if (!currentPosition)
         return std::numeric_limits<double>::quiet_NaN();
     // Look for a time portion.
-    if (*currentPosition == 'T') {
-        // Parse the time HH:mm[:ss[.sss]][Z|(+|-)00:00]
-        currentPosition = parseES5TimePortion(currentPosition + 1, hours, minutes, seconds, timeZoneSeconds);
+    // Note: As of ES2016, when a UTC offset is missing, date-time forms are local time while date-only forms are UTC.
+    if (*currentPosition == 'T' || *currentPosition == 't' || *currentPosition == ' ') {
+        // Parse the time HH:mm[:ss[.sss]][Z|(+|-)(00:00|0000|00)]
+        currentPosition = parseES5TimePortion(currentPosition + 1, hours, minutes, seconds, milliseconds, isLocalTime, timeZoneSeconds);
         if (!currentPosition)
             return std::numeric_limits<double>::quiet_NaN();
     }
@@ -665,20 +685,19 @@ double parseES5DateFromNullTerminatedCharacters(const char* dateString)
         return std::numeric_limits<double>::quiet_NaN();
     if (seconds < 0 || seconds >= 61)
         return std::numeric_limits<double>::quiet_NaN();
-    if (seconds > 60) {
+    if (seconds == 60) {
         // Discard leap seconds by clamping to the end of a minute.
-        seconds = 60;
+        milliseconds = 0;
     }
         
-    double dateSeconds = ymdhmsToSeconds(year, month, day, hours, minutes, seconds) - timeZoneSeconds;
-    return dateSeconds * msPerSecond;
+    return ymdhmsToMilliseconds(year, month, day, hours, minutes, seconds, milliseconds) - (timeZoneSeconds * msPerSecond);
 }
 
 // Odd case where 'exec' is allowed to be 0, to accomodate a caller in WebCore.
-double parseDateFromNullTerminatedCharacters(const char* dateString, bool& haveTZ, int& offset)
+double parseDateFromNullTerminatedCharacters(const char* dateString, bool& isLocalTime)
 {
-    haveTZ = false;
-    offset = 0;
+    isLocalTime = true;
+    int offset = 0;
 
     // This parses a date in the form:
     //     Tuesday, 09-Nov-99 23:12:40 GMT
@@ -701,7 +720,7 @@ double parseDateFromNullTerminatedCharacters(const char* dateString, bool& haveT
     const char *wordStart = dateString;
     // Check contents of first words if not number
     while (*dateString && !isASCIIDigit(*dateString)) {
-        if (isASCIISpace(*dateString) || *dateString == '(') {
+        if (isUnicodeCompatibleASCIIWhitespace(*dateString) || *dateString == '(') {
             if (dateString - wordStart >= 3)
                 month = findMonth(wordStart);
             skipSpacesAndComments(dateString);
@@ -729,7 +748,7 @@ double parseDateFromNullTerminatedCharacters(const char* dateString, bool& haveT
     if (day < 0)
         return std::numeric_limits<double>::quiet_NaN();
 
-    Optional<int> year;
+    std::optional<int> year;
     if (day > 31) {
         // ### where is the boundary and what happens below?
         if (*dateString != '/')
@@ -776,14 +795,14 @@ double parseDateFromNullTerminatedCharacters(const char* dateString, bool& haveT
             if (month == -1)
                 return std::numeric_limits<double>::quiet_NaN();
 
-            while (*dateString && *dateString != '-' && *dateString != ',' && !isASCIISpace(*dateString))
+            while (*dateString && *dateString != '-' && *dateString != ',' && !isUnicodeCompatibleASCIIWhitespace(*dateString))
                 dateString++;
 
             if (!*dateString)
                 return std::numeric_limits<double>::quiet_NaN();
 
             // '-99 23:12:40 GMT'
-            if (*dateString != '-' && *dateString != '/' && *dateString != ',' && !isASCIISpace(*dateString))
+            if (*dateString != '-' && *dateString != '/' && *dateString != ',' && !isUnicodeCompatibleASCIIWhitespace(*dateString))
                 return std::numeric_limits<double>::quiet_NaN();
             dateString++;
         }
@@ -808,14 +827,19 @@ double parseDateFromNullTerminatedCharacters(const char* dateString, bool& haveT
         dateString = newPosStr;
     else {
         // ' 23:12:40 GMT'
-        if (!(isASCIISpace(*newPosStr) || *newPosStr == ',')) {
+        if (!(isUnicodeCompatibleASCIIWhitespace(*newPosStr) || *newPosStr == ',')) {
             if (*newPosStr != ':')
                 return std::numeric_limits<double>::quiet_NaN();
             // There was no year; the number was the hour.
-            year = WTF::nullopt;
+            year = std::nullopt;
         } else {
             // in the normal case (we parsed the year), advance to the next number
-            dateString = ++newPosStr;
+            // ' at 23:12:40 GMT'
+            if (isUnicodeCompatibleASCIIWhitespace(newPosStr[0]) && isASCIIAlphaCaselessEqual(newPosStr[1], 'a') && isASCIIAlphaCaselessEqual(newPosStr[2], 't'))
+                newPosStr += 3;
+            else
+                ++newPosStr; // space or comma
+            dateString = newPosStr;
             skipSpacesAndComments(dateString);
         }
 
@@ -846,7 +870,7 @@ double parseDateFromNullTerminatedCharacters(const char* dateString, bool& haveT
                 return std::numeric_limits<double>::quiet_NaN();
 
             // ':40 GMT'
-            if (*dateString && *dateString != ':' && !isASCIISpace(*dateString))
+            if (*dateString && *dateString != ':' && !isUnicodeCompatibleASCIIWhitespace(*dateString))
                 return std::numeric_limits<double>::quiet_NaN();
 
             // seconds are optional in rfc822 + rfc2822
@@ -863,14 +887,14 @@ double parseDateFromNullTerminatedCharacters(const char* dateString, bool& haveT
 
             skipSpacesAndComments(dateString);
 
-            if (startsWithLettersIgnoringASCIICase(dateString, "am")) {
+            if (startsWithLettersIgnoringASCIICase(StringView::fromLatin1(dateString), "am"_s)) {
                 if (hour > 12)
                     return std::numeric_limits<double>::quiet_NaN();
                 if (hour == 12)
                     hour = 0;
                 dateString += 2;
                 skipSpacesAndComments(dateString);
-            } else if (startsWithLettersIgnoringASCIICase(dateString, "pm")) {
+            } else if (startsWithLettersIgnoringASCIICase(StringView::fromLatin1(dateString), "pm"_s)) {
                 if (hour > 12)
                     return std::numeric_limits<double>::quiet_NaN();
                 if (hour != 12)
@@ -894,9 +918,9 @@ double parseDateFromNullTerminatedCharacters(const char* dateString, bool& haveT
     // Don't fail if the time zone is missing. 
     // Some websites omit the time zone (4275206).
     if (*dateString) {
-        if (startsWithLettersIgnoringASCIICase(dateString, "gmt") || startsWithLettersIgnoringASCIICase(dateString, "utc")) {
+        if (startsWithLettersIgnoringASCIICase(StringView ::fromLatin1(dateString), "gmt"_s) || startsWithLettersIgnoringASCIICase(StringView::fromLatin1(dateString), "utc"_s)) {
             dateString += 3;
-            haveTZ = true;
+            isLocalTime = false;
         }
 
         if (*dateString == '+' || *dateString == '-') {
@@ -909,7 +933,7 @@ double parseDateFromNullTerminatedCharacters(const char* dateString, bool& haveT
                 return std::numeric_limits<double>::quiet_NaN();
 
             int sgn = (o < 0) ? -1 : 1;
-            o = abs(o);
+            o = std::abs(o);
             if (*dateString != ':') {
                 if (o >= 24)
                     offset = ((o / 100) * 60 + (o % 100)) * sgn;
@@ -923,7 +947,7 @@ double parseDateFromNullTerminatedCharacters(const char* dateString, bool& haveT
                 dateString = newPosStr;
                 offset = (o * 60 + o2) * sgn;
             }
-            haveTZ = true;
+            isLocalTime = false;
         } else {
             for (auto& knownZone : knownZones) {
                 // Since the passed-in length is used for both strings, the following checks that
@@ -932,7 +956,7 @@ double parseDateFromNullTerminatedCharacters(const char* dateString, bool& haveT
                 if (equalLettersIgnoringASCIICase(dateString, knownZone.tzName, length)) {
                     offset = knownZone.tzOffset;
                     dateString += length;
-                    haveTZ = true;
+                    isLocalTime = false;
                     break;
                 }
             }
@@ -975,23 +999,19 @@ double parseDateFromNullTerminatedCharacters(const char* dateString, bool& haveT
         year = 2000;
     }
     ASSERT(year);
-    
-    return ymdhmsToSeconds(year.value(), month + 1, day, hour, minute, second) * msPerSecond;
+
+    return ymdhmsToMilliseconds(year.value(), month + 1, day, hour, minute, second, 0) - offset * (secondsPerMinute * msPerSecond);
 }
 
 double parseDateFromNullTerminatedCharacters(const char* dateString)
 {
-    bool haveTZ;
-    int offset;
-    double ms = parseDateFromNullTerminatedCharacters(dateString, haveTZ, offset);
-    if (std::isnan(ms))
-        return std::numeric_limits<double>::quiet_NaN();
+    bool isLocalTime;
+    double value = parseDateFromNullTerminatedCharacters(dateString, isLocalTime);
 
-    // fall back to local timezone
-    if (!haveTZ)
-        offset = calculateLocalTimeOffset(ms, LocalTime).offset / msPerMinute; // ms value is in local time milliseconds.
+    if (isLocalTime)
+        value -= calculateLocalTimeOffset(value, LocalTime).offset;
 
-    return ms - (offset * msPerMinute);
+    return value;
 }
 
 // See http://tools.ietf.org/html/rfc2822#section-3.3 for more information.
@@ -1008,11 +1028,52 @@ String makeRFC2822DateString(unsigned dayOfWeek, unsigned day, unsigned month, u
     stringBuilder.append(' ');
 
     stringBuilder.append(utcOffset > 0 ? '+' : '-');
-    int absoluteUTCOffset = abs(utcOffset);
+    int absoluteUTCOffset = std::abs(utcOffset);
     appendTwoDigitNumber(stringBuilder, absoluteUTCOffset / 60);
     appendTwoDigitNumber(stringBuilder, absoluteUTCOffset % 60);
 
     return stringBuilder.toString();
 }
 
+static std::optional<Vector<UChar, 32>> validateTimeZone(StringView timeZone)
+{
+    auto buffer = timeZone.upconvertedCharacters();
+    const UChar* characters = buffer;
+    Vector<UChar, 32> canonicalBuffer;
+    auto status = callBufferProducingFunction(ucal_getCanonicalTimeZoneID, characters, timeZone.length(), canonicalBuffer, nullptr);
+    if (!U_SUCCESS(status))
+        return std::nullopt;
+    return canonicalBuffer;
+}
+
+bool isTimeZoneValid(StringView timeZone)
+{
+    return validateTimeZone(timeZone).has_value();
+}
+
+bool setTimeZoneOverride(StringView timeZone)
+{
+    if (timeZone.isEmpty()) {
+        Locker locker { innerTimeZoneOverrideLock };
+        innerTimeZoneOverride().clear();
+        return true;
+    }
+
+    auto canonicalBuffer = validateTimeZone(timeZone);
+    if (!canonicalBuffer)
+        return false;
+
+    {
+        Locker locker { innerTimeZoneOverrideLock };
+        innerTimeZoneOverride() = WTFMove(*canonicalBuffer);
+    }
+    return true;
+}
+
+void getTimeZoneOverride(Vector<UChar, 32>& timeZoneID)
+{
+    Locker locker { innerTimeZoneOverrideLock };
+    timeZoneID = innerTimeZoneOverride();
+}
+
 } // namespace WTF
