@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 1999-2000 Harri Porten (porten@kde.org)
- * Copyright (C) 2006, 2007 Apple Inc. All rights reserved.
+ * Copyright (C) 2006-2020 Apple Inc. All rights reserved.
  * Copyright (C) 2009 Google Inc. All rights reserved.
  * Copyright (C) 2010 Research In Motion Limited. All rights reserved.
  *
@@ -59,41 +59,31 @@ enum TimeType {
 struct LocalTimeOffset {
     WTF_MAKE_STRUCT_FAST_ALLOCATED;
 
-    LocalTimeOffset()
-        : isDST(false)
-        , offset(0)
-    {
-    }
-
-    LocalTimeOffset(bool isDST, int offset)
+    LocalTimeOffset() = default;
+    constexpr LocalTimeOffset(bool isDST, int offset)
         : isDST(isDST)
         , offset(offset)
     {
     }
 
-    bool operator==(const LocalTimeOffset& other)
+    bool operator==(const LocalTimeOffset& other) const
     {
         return isDST == other.isDST && offset == other.offset;
     }
 
-    bool operator!=(const LocalTimeOffset& other)
-    {
-        return isDST != other.isDST || offset != other.offset;
-    }
-
-    bool isDST;
-    int offset;
+    bool isDST { false };
+    int offset { 0 };
 };
 
 void initializeDates();
 int equivalentYearForDST(int year);
 
 // Not really math related, but this is currently the only shared place to put these.
-WTF_EXPORT_PRIVATE double parseES5DateFromNullTerminatedCharacters(const char* dateString);
+WTF_EXPORT_PRIVATE double parseES5DateFromNullTerminatedCharacters(const char* dateString, bool& isLocalTime);
 WTF_EXPORT_PRIVATE double parseDateFromNullTerminatedCharacters(const char* dateString);
-WTF_EXPORT_PRIVATE double parseDateFromNullTerminatedCharacters(const char* dateString, bool& haveTZ, int& offset);
+WTF_EXPORT_PRIVATE double parseDateFromNullTerminatedCharacters(const char* dateString, bool& isLocalTime);
 // dayOfWeek: [0, 6] 0 being Monday, day: [1, 31], month: [0, 11], year: ex: 2011, hours: [0, 23], minutes: [0, 59], seconds: [0, 59], utcOffset: [-720,720]. 
-String makeRFC2822DateString(unsigned dayOfWeek, unsigned day, unsigned month, unsigned year, unsigned hours, unsigned minutes, unsigned seconds, int utcOffset);
+WTF_EXPORT_PRIVATE String makeRFC2822DateString(unsigned dayOfWeek, unsigned day, unsigned month, unsigned year, unsigned hours, unsigned minutes, unsigned seconds, int utcOffset);
 
 inline double jsCurrentTime()
 {
@@ -101,9 +91,9 @@ inline double jsCurrentTime()
     return floor(WallTime::now().secondsSinceEpoch().milliseconds());
 }
 
-extern WTF_EXPORT_PRIVATE const char* const weekdayName[7];
-extern WTF_EXPORT_PRIVATE const char* const monthName[12];
-extern WTF_EXPORT_PRIVATE const char* const monthFullName[12];
+extern WTF_EXPORT_PRIVATE const ASCIILiteral weekdayName[7];
+extern WTF_EXPORT_PRIVATE const ASCIILiteral monthName[12];
+extern WTF_EXPORT_PRIVATE const ASCIILiteral monthFullName[12];
 extern WTF_EXPORT_PRIVATE const int firstDayOfMonth[2][12];
 
 static constexpr double hoursPerDay = 24.0;
@@ -167,7 +157,7 @@ inline double daysFrom1970ToYear(int year)
     static constexpr int excludedLeapDaysBefore1971By100Rule = 1970 / 100;
     static constexpr int leapDaysBefore1971By400Rule = 1970 / 400;
 
-    const double yearMinusOne = year - 1;
+    const double yearMinusOne = static_cast<double>(year) - 1;
     const double yearsToAddBy4Rule = floor(yearMinusOne / 4.0) - leapDaysBefore1971By4Rule;
     const double yearsToExcludeBy100Rule = floor(yearMinusOne / 100.0) - excludedLeapDaysBefore1971By100Rule;
     const double yearsToAddBy400Rule = floor(yearMinusOne / 400.0) - leapDaysBefore1971By400Rule;
@@ -223,7 +213,8 @@ inline int dayInYear(int year, int month, int day)
 
 inline int dayInYear(double ms, int year)
 {
-    return static_cast<int>(msToDays(ms) - daysFrom1970ToYear(year));
+    double result = msToDays(ms) - daysFrom1970ToYear(year);
+    return std::isnan(result) ? 0 : static_cast<int>(result);
 }
 
 inline int dayInYear(TimeClippedPositiveMilliseconds ms, int year)
@@ -249,7 +240,10 @@ inline double dateToDaysFrom1970(int year, int month, int day)
 
 inline int msToYear(double ms)
 {
-    int approxYear = static_cast<int>(floor(ms / (msPerDay * 365.2425)) + 1970);
+    double msAsYears = std::floor(ms / (msPerDay * 365.2425));
+    if (std::isnan(msAsYears))
+        msAsYears = 0;
+    int approxYear = static_cast<int>(msAsYears + 1970);
     double msFromApproxYearTo1970 = msPerDay * daysFrom1970ToYear(approxYear);
     if (msFromApproxYearTo1970 > ms)
         return approxYear - 1;
@@ -345,7 +339,7 @@ inline int monthFromDayInYear(int dayInYear, bool leapYear)
         return 8;
     if (d < (step += 31))
         return 9;
-    if (d < (step += 30))
+    if (d < step + 30)
         return 10;
     return 11;
 }
@@ -389,30 +383,45 @@ inline int dayInMonthFromDayInYear(int dayInYear, bool leapYear)
     return d - step;
 }
 
+inline double timeToMS(double hour, double min, double sec, double ms)
+{
+    return (((hour * WTF::minutesPerHour + min) * WTF::secondsPerMinute + sec) * WTF::msPerSecond + ms);
+}
+
+WTF_EXPORT_PRIVATE bool isTimeZoneValid(StringView);
+WTF_EXPORT_PRIVATE bool setTimeZoneOverride(StringView);
+WTF_EXPORT_PRIVATE void getTimeZoneOverride(Vector<UChar, 32>& timeZoneID);
+
 // Returns combined offset in millisecond (UTC + DST).
 WTF_EXPORT_PRIVATE LocalTimeOffset calculateLocalTimeOffset(double utcInMilliseconds, TimeType = UTCTime);
 
 } // namespace WTF
 
-using WTF::isLeapYear;
+using WTF::calculateLocalTimeOffset;
 using WTF::dateToDaysFrom1970;
 using WTF::dayInMonthFromDayInYear;
 using WTF::dayInYear;
+using WTF::daysFrom1970ToYear;
+using WTF::daysInYear;
+using WTF::getTimeZoneOverride;
+using WTF::isLeapYear;
+using WTF::isTimeZoneValid;
+using WTF::jsCurrentTime;
+using WTF::LocalTimeOffset;
+using WTF::makeRFC2822DateString;
 using WTF::minutesPerHour;
 using WTF::monthFromDayInYear;
 using WTF::msPerDay;
 using WTF::msPerHour;
 using WTF::msPerMinute;
 using WTF::msPerSecond;
-using WTF::msToYear;
 using WTF::msToDays;
-using WTF::msToMinutes;
 using WTF::msToHours;
+using WTF::msToMinutes;
+using WTF::msToYear;
+using WTF::parseDateFromNullTerminatedCharacters;
 using WTF::secondsPerDay;
 using WTF::secondsPerMinute;
-using WTF::parseDateFromNullTerminatedCharacters;
-using WTF::makeRFC2822DateString;
-using WTF::LocalTimeOffset;
-using WTF::calculateLocalTimeOffset;
+using WTF::setTimeZoneOverride;
 using WTF::timeClip;
-using WTF::jsCurrentTime;
+using WTF::timeToMS;
