@@ -27,23 +27,16 @@
 
 #include <initializer_list>
 #include <iterator>
+#include <optional>
 #include <type_traits>
 #include <wtf/Assertions.h>
 #include <wtf/EnumTraits.h>
+#include <wtf/FastMalloc.h>
 #include <wtf/MathExtras.h>
-#include <wtf/Optional.h>
+#include <wtf/StdLibExtras.h>
 
 namespace WTF {
 
-// Detecting in C++ whether a type is defined, part 3: SFINAE and incomplete types
-// <https://devblogs.microsoft.com/oldnewthing/20190710-00/?p=102678>
-template<typename, typename = void>
-constexpr bool is_type_complete_v = false;
-
-template<typename T>
-constexpr bool is_type_complete_v<T, std::void_t<decltype(sizeof(T))>> = true;
-
-
 template<typename E> class OptionSet;
 
 
@@ -75,33 +68,27 @@ struct OptionSetValueChecker<T, EnumValues<E>> {
     }
 };
 
-
-template<typename E, std::enable_if_t<std::is_enum<E>::value && is_type_complete_v<EnumTraits<E>>>* = nullptr>
-constexpr bool isValidOptionSetEnum(E e)
+template<typename E>
+constexpr std::enable_if_t<std::is_enum_v<E>, bool> isValidOptionSetEnum(E e)
 {
-    return OptionSetValueChecker<std::underlying_type_t<E>, typename EnumTraits<E>::values>::isValidOptionSetEnum(static_cast<std::underlying_type_t<E>>(e));
+    if constexpr (IsTypeComplete<EnumTraits<E>>)
+        return OptionSetValueChecker<std::underlying_type_t<E>, typename EnumTraits<E>::values>::isValidOptionSetEnum(static_cast<std::underlying_type_t<E>>(e));
+    else {
+        // FIXME: Remove once all OptionSet<> enums have EnumTraits<> defined.
+        return hasOneBitSet(static_cast<typename OptionSet<E>::StorageType>(e));
+    }
 }
 
-template<typename E, std::enable_if_t<std::is_enum<E>::value && !is_type_complete_v<EnumTraits<E>>>* = nullptr>
-constexpr bool isValidOptionSetEnum(E e)
+template<typename E>
+constexpr std::enable_if_t<std::is_enum_v<E>, typename OptionSet<E>::StorageType> maskRawValue(typename OptionSet<E>::StorageType rawValue)
 {
-    // FIXME: Remove once all OptionSet<> enums have EnumTraits<> defined.
-    return hasOneBitSet(static_cast<typename OptionSet<E>::StorageType>(e));
-}
-
-
-template<typename E, std::enable_if_t<std::is_enum<E>::value && is_type_complete_v<EnumTraits<E>>>* = nullptr>
-constexpr typename OptionSet<E>::StorageType maskRawValue(typename OptionSet<E>::StorageType rawValue)
-{
-    auto allValidBitsValue = OptionSetValueChecker<std::underlying_type_t<E>, typename EnumTraits<E>::values>::allValidBits();
-    return rawValue & allValidBitsValue;
-}
-
-template<typename E, std::enable_if_t<std::is_enum<E>::value && !is_type_complete_v<EnumTraits<E>>>* = nullptr>
-constexpr typename OptionSet<E>::StorageType maskRawValue(typename OptionSet<E>::StorageType rawValue)
-{
-    // FIXME: Remove once all OptionSet<> enums have EnumTraits<> defined.
-    return rawValue;
+    if constexpr (IsTypeComplete<EnumTraits<E>>) {
+        auto allValidBitsValue = OptionSetValueChecker<std::underlying_type_t<E>, typename EnumTraits<E>::values>::allValidBits();
+        return rawValue & allValidBitsValue;
+    } else {
+        // FIXME: Remove once all OptionSet<> enums have EnumTraits<> defined.
+        return rawValue;
+    }
 }
 
 
@@ -131,7 +118,6 @@ public:
         Iterator& operator++(int) = delete;
 
         bool operator==(const Iterator& other) const { return m_value == other.m_value; }
-        bool operator!=(const Iterator& other) const { return m_value != other.m_value; }
 
     private:
         Iterator(StorageType value) : m_value(value) { }
@@ -162,7 +148,7 @@ public:
         }
     }
 
-    constexpr OptionSet(Optional<E> optional)
+    constexpr OptionSet(std::optional<E> optional)
         : m_storage(optional ? static_cast<StorageType>(*optional) : 0)
     {
     }
@@ -201,14 +187,22 @@ public:
         m_storage &= ~optionSet.m_storage;
     }
 
+    constexpr void set(OptionSet optionSet, bool value)
+    {
+        if (value)
+            add(optionSet);
+        else
+            remove(optionSet);
+    }
+
     constexpr bool hasExactlyOneBitSet() const
     {
         return m_storage && !(m_storage & (m_storage - 1));
     }
 
-    constexpr Optional<E> toSingleValue() const
+    constexpr std::optional<E> toSingleValue() const
     {
-        return hasExactlyOneBitSet() ? Optional<E>(static_cast<E>(m_storage)) : WTF::nullopt;
+        return hasExactlyOneBitSet() ? std::optional<E>(static_cast<E>(m_storage)) : std::nullopt;
     }
 
     constexpr friend bool operator==(OptionSet lhs, OptionSet rhs)
@@ -216,11 +210,6 @@ public:
         return lhs.m_storage == rhs.m_storage;
     }
 
-    constexpr friend bool operator!=(OptionSet lhs, OptionSet rhs)
-    {
-        return lhs.m_storage != rhs.m_storage;
-    }
-
     constexpr friend OptionSet operator|(OptionSet lhs, OptionSet rhs)
     {
         return fromRaw(lhs.m_storage | rhs.m_storage);
