@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2006-2020 Apple Inc. All rights reserved.
+ * Copyright (C) 2006-2023 Apple Inc. All rights reserved.
  * Copyright (C) 2007-2009 Torch Mobile, Inc.
  * Copyright (C) 2010, 2011 Research In Motion Limited. All rights reserved.
  * Copyright (C) 2013 Samsung Electronics. All rights reserved.
@@ -43,11 +43,11 @@
 #define ENABLE_3D_TRANSFORMS 1
 #endif
 
-#if !defined(ENABLE_ACCESSIBILITY_ISOLATED_TREE) && PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 110000
+#if !defined(ENABLE_ACCESSIBILITY_ISOLATED_TREE) && PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 130000
 #define ENABLE_ACCESSIBILITY_ISOLATED_TREE 1
 #endif
 
-#if !defined(ENABLE_AIRPLAY_PICKER) && !PLATFORM(MAC) && !PLATFORM(MACCATALYST)
+#if !defined(ENABLE_AIRPLAY_PICKER) && !PLATFORM(MAC) && !PLATFORM(MACCATALYST) && !PLATFORM(WATCHOS) && !PLATFORM(APPLETV)
 #define ENABLE_AIRPLAY_PICKER 1
 #endif
 
@@ -87,10 +87,14 @@
 #define ENABLE_APPLE_PAY_INSTALLMENTS 1
 #endif
 
-#if !defined(ENABLE_APPLE_PAY_LATER) && (PLATFORM(IOS) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 160000)
+#if !defined(ENABLE_APPLE_PAY_LATER) && ((PLATFORM(IOS) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 160000)  || PLATFORM(VISION))
 #define ENABLE_APPLE_PAY_LATER 1
 #endif
 
+#if !defined(ENABLE_APPLE_PAY_LATER_AVAILABILITY) && HAVE(PASSKIT_APPLE_PAY_LATER_AVAILABILITY)
+#define ENABLE_APPLE_PAY_LATER_AVAILABILITY 1
+#endif
+
 #if !defined(ENABLE_APPLE_PAY_MULTI_MERCHANT_PAYMENTS) && HAVE(PASSKIT_MULTI_MERCHANT_PAYMENTS)
 #define ENABLE_APPLE_PAY_MULTI_MERCHANT_PAYMENTS 1
 #endif
@@ -119,19 +123,23 @@
 #define ENABLE_APPLE_PAY_RECURRING_PAYMENTS 1
 #endif
 
+#if !defined(ENABLE_APPLE_PAY_DEFERRED_PAYMENTS) && HAVE(PASSKIT_DEFERRED_PAYMENTS)
+#define ENABLE_APPLE_PAY_DEFERRED_PAYMENTS 1
+#endif
+
 #if !PLATFORM(MAC) && !PLATFORM(MACCATALYST) && !PLATFORM(WATCHOS) && !PLATFORM(APPLETV)
 
 #if !defined(ENABLE_APPLE_PAY_REMOTE_UI)
 #define ENABLE_APPLE_PAY_REMOTE_UI 1
 #endif
 
-#if !defined(ENABLE_APPLE_PAY_REMOTE_UI_USES_SCENE) && HAVE(UIKIT_WEBKIT_INTERNALS)
+#if !defined(ENABLE_APPLE_PAY_REMOTE_UI_USES_SCENE) && PLATFORM(VISION)
 #define ENABLE_APPLE_PAY_REMOTE_UI_USES_SCENE 1
 #endif
 
 #endif // !PLATFORM(MAC) && !PLATFORM(MACCATALYST) && !PLATFORM(WATCHOS) && !PLATFORM(APPLETV)
 
-#if !defined(ENABLE_APPLE_PAY_SESSION_V11) && ((PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 110300) || (PLATFORM(IOS) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 140500))
+#if !defined(ENABLE_APPLE_PAY_SESSION_V11) && ((PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 110300) || (PLATFORM(IOS) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 140500) || PLATFORM(VISION))
 #define ENABLE_APPLE_PAY_SESSION_V11 1
 #endif
 
@@ -143,7 +151,7 @@
 #define ENABLE_APPLE_PAY_UPDATE_SHIPPING_METHODS_WHEN_CHANGING_LINE_ITEMS 1
 #endif
 
-#if !defined(ENABLE_APPLE_PAY_AMS_UI) && ((PLATFORM(MAC) && __MAC_OS_X_VERSION_MAX_ALLOWED >= 120300) || (PLATFORM(IOS) && __IPHONE_OS_VERSION_MAX_ALLOWED >= 150400))
+#if !defined(ENABLE_APPLE_PAY_AMS_UI) && ((PLATFORM(MAC) && __MAC_OS_X_VERSION_MAX_ALLOWED >= 120300) || (PLATFORM(IOS) && __IPHONE_OS_VERSION_MAX_ALLOWED >= 150400) || PLATFORM(VISION))
 #define ENABLE_APPLE_PAY_AMS_UI 1
 #endif
 
@@ -179,11 +187,11 @@
 #define ENABLE_AVF_CAPTIONS 1
 #endif
 
-#if !defined(ENABLE_BUILT_IN_NOTIFICATIONS) && (PLATFORM(MAC) || PLATFORM(IOS))
+#if !defined(ENABLE_BUILT_IN_NOTIFICATIONS) && (PLATFORM(MAC) || PLATFORM(IOS) || PLATFORM(VISION))
 #define ENABLE_BUILT_IN_NOTIFICATIONS 1
 #endif
 
-#if !defined(ENABLE_INSTALL_COORDINATION_BUNDLES) && PLATFORM(IOS)
+#if !defined(ENABLE_INSTALL_COORDINATION_BUNDLES) && (PLATFORM(IOS) || PLATFORM(VISION))
 #define ENABLE_INSTALL_COORDINATION_BUNDLES 1
 #endif
 
@@ -219,6 +227,10 @@
 #define ENABLE_CONTEXT_MENU_EVENT 0
 #endif
 
+#if !defined(ENABLE_CONTEXT_MENU_QR_CODE_DETECTION) && PLATFORM(MAC)
+#define ENABLE_CONTEXT_MENU_QR_CODE_DETECTION 1
+#endif
+
 #if !defined(ENABLE_CSS_COMPOSITING)
 #define ENABLE_CSS_COMPOSITING 1
 #endif
@@ -251,11 +263,11 @@
 #define ENABLE_DATALIST_ELEMENT 1
 #endif
 
-#if !defined(ENABLE_DATA_DETECTION) && (PLATFORM(MAC) || PLATFORM(IOS) || (PLATFORM(MACCATALYST) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 150000))
+#if !defined(ENABLE_DATA_DETECTION) && (PLATFORM(MAC) || PLATFORM(IOS) || (PLATFORM(MACCATALYST) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 150000) || PLATFORM(VISION))
 #define ENABLE_DATA_DETECTION 1
 #endif
 
-#if !defined(ENABLE_DEVICE_ORIENTATION) && !PLATFORM(MAC) && !PLATFORM(MACCATALYST)
+#if !defined(ENABLE_DEVICE_ORIENTATION)
 #define ENABLE_DEVICE_ORIENTATION 1
 #endif
 
@@ -263,7 +275,7 @@
 #define ENABLE_DRAG_SUPPORT 0
 #endif
 
-#if !defined(ENABLE_EDITABLE_REGION) && PLATFORM(IOS)
+#if !defined(ENABLE_EDITABLE_REGION) && (PLATFORM(IOS) || PLATFORM(VISION))
 #define ENABLE_EDITABLE_REGION 1
 #endif
 
@@ -311,7 +323,8 @@
 #if PLATFORM(MAC) \
     || ((PLATFORM(IOS) || PLATFORM(MACCATALYST)) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 150000) \
     || (PLATFORM(WATCHOS) && __WATCH_OS_VERSION_MIN_REQUIRED >= 80000) \
-    || (PLATFORM(APPLETV) && __TV_OS_VERSION_MIN_REQUIRED >= 150000)
+    || (PLATFORM(APPLETV) && __TV_OS_VERSION_MIN_REQUIRED >= 150000) \
+    || PLATFORM(VISION)
 #define ENABLE_GPU_PROCESS_BY_DEFAULT 1
 #endif
 #endif
@@ -324,10 +337,6 @@
 #define ENABLE_HARDWARE_JPEG 1
 #endif
 
-#if PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 110000
-#define ENABLE_UI_PROCESS_PDF_HUD 1
-#endif
-
 #if !defined(ENABLE_INPUT_TYPE_DATE)
 #define ENABLE_INPUT_TYPE_DATE 1
 #endif
@@ -400,7 +409,7 @@
 #define ENABLE_MEDIA_CONTROLS_SCRIPT 1
 #endif
 
-#if !defined(ENABLE_MEDIA_CONTROLS_CONTEXT_MENUS) && (PLATFORM(MAC) || ((PLATFORM(IOS) || PLATFORM(MACCATALYST)) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 150000))
+#if !defined(ENABLE_MEDIA_CONTROLS_CONTEXT_MENUS) && (PLATFORM(MAC) || ((PLATFORM(IOS) || PLATFORM(MACCATALYST) || PLATFORM(VISION)) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 150000))
 #define ENABLE_MEDIA_CONTROLS_CONTEXT_MENUS 1
 #endif
 
@@ -424,6 +433,10 @@
 #define ENABLE_MEDIA_SOURCE 1
 #endif
 
+#if !defined(MANAGED_MEDIA_SOURCE) && ENABLE(MEDIA_SOURCE)
+#define ENABLE_MANAGED_MEDIA_SOURCE 1
+#endif
+
 #if !defined(ENABLE_MEDIA_STREAM) && !PLATFORM(MACCATALYST) && !PLATFORM(WATCHOS) && !PLATFORM(APPLETV)
 #define ENABLE_MEDIA_STREAM 1
 #endif
@@ -461,7 +474,7 @@
 #define ENABLE_MOUSE_CURSOR_SCALE 1
 #endif
 
-#if !defined(ENABLE_NAVIGATOR_STANDALONE) && PLATFORM(IOS_FAMILY)
+#if !defined(ENABLE_NAVIGATOR_STANDALONE)
 #define ENABLE_NAVIGATOR_STANDALONE 1
 #endif
 
@@ -485,6 +498,7 @@
 #define ENABLE_OFFSCREEN_CANVAS 1
 #endif
 
+// ORIENTATION_EVENTS should never get enabled on Desktop, only Mobile.
 #if !defined(ENABLE_ORIENTATION_EVENTS) && PLATFORM(IOS_FAMILY)
 #define ENABLE_ORIENTATION_EVENTS 1
 #endif
@@ -526,11 +540,11 @@
 #endif
 
 
-#if !defined(ENABLE_POST_EDITING_GRAMMAR_CHECKING) && ((PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 130000) || (PLATFORM(IOS) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 160000))
+#if !defined(ENABLE_POST_EDITING_GRAMMAR_CHECKING) && ((PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 130000) || (PLATFORM(IOS) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 160000) || PLATFORM(VISION))
 #define ENABLE_POST_EDITING_GRAMMAR_CHECKING 1
 #endif
 
-#if !defined(ENABLE_PREVIEW_CONVERTER) && PLATFORM(IOS)
+#if !defined(ENABLE_PREVIEW_CONVERTER) && (PLATFORM(IOS) || PLATFORM(VISION))
 #define ENABLE_PREVIEW_CONVERTER 1
 #endif
 
@@ -552,7 +566,8 @@
 
 #if ((PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 130000) \
     || (PLATFORM(IOS) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 160000) \
-    || (PLATFORM(MACCATALYST) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 160000))
+    || (PLATFORM(MACCATALYST) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 160000) \
+    || PLATFORM(VISION))
 #define ENABLE_JS_COOKIE_CHECKING 1
 #endif
 
@@ -604,10 +619,6 @@
 #define ENABLE_SHAREABLE_RESOURCE 1
 #endif
 
-#if !defined(ENABLE_SIGILL_CRASH_ANALYZER) && !PLATFORM(MAC) && !PLATFORM(WATCHOS)
-#define ENABLE_SIGILL_CRASH_ANALYZER 1
-#endif
-
 // FIXME: Should this be enabled for watchOS and tvOS?
 #if !defined(ENABLE_TAKE_UNBOUNDED_NETWORKING_ASSERTION) && !PLATFORM(MAC) && !PLATFORM(WATCHOS) && !PLATFORM(APPLETV)
 #define ENABLE_TAKE_UNBOUNDED_NETWORKING_ASSERTION 1
@@ -638,11 +649,11 @@
 #endif
 
 // FIXME (rdar://49961589): Enable on macCatalyst.
-#if !defined(ENABLE_TRANSCODE_UIIMAGEPICKERCONTROLLER_VIDEO) && PLATFORM(IOS)
+#if !defined(ENABLE_TRANSCODE_UIIMAGEPICKERCONTROLLER_VIDEO) && (PLATFORM(IOS) || PLATFORM(VISION))
 #define ENABLE_TRANSCODE_UIIMAGEPICKERCONTROLLER_VIDEO 1
 #endif
 
-#if !defined(ENABLE_WHEEL_EVENT_REGIONS) && (PLATFORM(MAC) || PLATFORM(IOS) || PLATFORM(MACCATALYST))
+#if !defined(ENABLE_WHEEL_EVENT_REGIONS) && (PLATFORM(MAC) || PLATFORM(IOS) || PLATFORM(MACCATALYST) || PLATFORM(VISION))
 #define ENABLE_WHEEL_EVENT_REGIONS 1
 #endif
 
@@ -666,15 +677,15 @@
 #define ENABLE_VIDEO_PRESENTATION_MODE 1
 #endif
 
-#if !defined(ENABLE_VIDEO_USES_ELEMENT_FULLSCREEN) && (PLATFORM(MAC) || HAVE(UIKIT_WEBKIT_INTERNALS))
+#if !defined(ENABLE_VIDEO_USES_ELEMENT_FULLSCREEN) && (PLATFORM(MAC) || PLATFORM(VISION))
 #define ENABLE_VIDEO_USES_ELEMENT_FULLSCREEN 1
 #endif
 
-#if HAVE(UIKIT_WEBKIT_INTERNALS)
+#if PLATFORM(VISION)
 #define HAVE_PIP_CONTROLLER 1
 #endif
 
-#if !defined(ENABLE_VP9) && ((PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 110000) || PLATFORM(IOS) || PLATFORM(IOS_SIMULATOR))
+#if !defined(ENABLE_VP9) && ((PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 110000) || PLATFORM(IOS) || PLATFORM(IOS_SIMULATOR) || PLATFORM(VISION))
 #define ENABLE_VP9 1
 #endif
 
@@ -702,22 +713,10 @@
 #define ENABLE_WEBGL 1
 #endif
 
-#if !defined(ENABLE_WEBGL2)
-#define ENABLE_WEBGL2 1
-#endif
-
 #if !defined(ENABLE_WEBPROCESS_NSRUNLOOP) && PLATFORM(MAC)
 #define ENABLE_WEBPROCESS_NSRUNLOOP 1
 #endif
 
-#if !defined(ENABLE_WEBXR) && !PLATFORM(WATCHOS) && !PLATFORM(APPLETV)
-#define ENABLE_WEBXR 1
-#endif
-
-#if !defined(ENABLE_WEBXR_HANDS) && !PLATFORM(WATCHOS) && !PLATFORM(APPLETV)
-#define ENABLE_WEBXR_HANDS 1
-#endif
-
 #if !defined(ENABLE_WEB_API_STATISTICS)
 #define ENABLE_WEB_API_STATISTICS 1
 #endif
@@ -742,7 +741,7 @@
 #define ENABLE_WEB_CODECS 1
 #endif
 
-#if !defined(ENABLE_WK_WEB_EXTENSIONS) && (PLATFORM(MAC) || PLATFORM(MACCATALYST) || PLATFORM(IOS) || PLATFORM(IOS_SIMULATOR))
+#if !defined(ENABLE_WK_WEB_EXTENSIONS) && (PLATFORM(MAC) || PLATFORM(MACCATALYST) || PLATFORM(IOS) || PLATFORM(IOS_SIMULATOR) || PLATFORM(VISION))
 #define ENABLE_WK_WEB_EXTENSIONS 1
 #endif
 
@@ -758,11 +757,15 @@
 #define ENABLE_WIRELESS_PLAYBACK_TARGET 1
 #endif
 
-#if !defined(ENABLE_WKPDFVIEW) && (PLATFORM(IOS) || (PLATFORM(MACCATALYST) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 140000))
+#if !defined(ENABLE_WIRELESS_PLAYBACK_TARGET_AVAILABILITY_API) && ENABLE(WIRELESS_PLAYBACK_TARGET) && !PLATFORM(VISION)
+#define ENABLE_WIRELESS_PLAYBACK_TARGET_AVAILABILITY_API 1
+#endif
+
+#if !defined(ENABLE_WKPDFVIEW) && (PLATFORM(IOS) || (PLATFORM(MACCATALYST) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 140000) || PLATFORM(VISION))
 #define ENABLE_WKPDFVIEW 1
 #endif
 
-#if !defined(ENABLE_VORBIS) && (PLATFORM(MAC) && __MAC_OS_X_VERSION_MAX_ALLOWED >= 110300 || PLATFORM(IOS) || PLATFORM(IOS_SIMULATOR))
+#if !defined(ENABLE_VORBIS) && (PLATFORM(MAC) && __MAC_OS_X_VERSION_MAX_ALLOWED >= 110300 || PLATFORM(IOS) || PLATFORM(IOS_SIMULATOR) || PLATFORM(VISION))
 #define ENABLE_VORBIS 1
 #endif
 
@@ -774,11 +777,11 @@
 #define ENABLE_WEBM_FORMAT_READER 1
 #endif
 
-#if !defined(ENABLE_OPUS) && (PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 120000 || PLATFORM(IOS) || PLATFORM(IOS_SIMULATOR))
+#if !defined(ENABLE_OPUS) && (PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 120000 || PLATFORM(IOS) || PLATFORM(IOS_SIMULATOR) || PLATFORM(VISION))
 #define ENABLE_OPUS 1
 #endif
 
-#if !defined(ENABLE_SEPARATED_MODEL) && HAVE(UIKIT_WEBKIT_INTERNALS) && HAVE(CORE_ANIMATION_SEPARATED_LAYERS)
+#if !defined(ENABLE_SEPARATED_MODEL) && PLATFORM(VISION) && HAVE(CORE_ANIMATION_SEPARATED_LAYERS)
 #define ENABLE_SEPARATED_MODEL 1
 #endif
 
@@ -790,9 +793,10 @@
 #define ENABLE_APP_PRIVACY_REPORT 1
 #endif
 
-#if (!defined(ENABLE_APP_HIGHLIGHTS) && \
-    (PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 120000 \
-    || PLATFORM(IOS) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 150000))
+#if (!defined(ENABLE_APP_HIGHLIGHTS) \
+    && ((PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 120000) \
+        || (PLATFORM(IOS) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 150000) \
+        || PLATFORM(VISION)))
 #define ENABLE_APP_HIGHLIGHTS 1
 #endif
 
@@ -842,18 +846,19 @@
 #define ENABLE_FULLSCREEN_DISMISSAL_GESTURES 1
 #endif
 
-#if (!defined(ENABLE_LOCKDOWN_MODE_API) && (PLATFORM(MAC) || PLATFORM(IOS)))
+#if (!defined(ENABLE_LOCKDOWN_MODE_API) && (PLATFORM(MAC) || PLATFORM(IOS) || PLATFORM(VISION)))
 #define ENABLE_LOCKDOWN_MODE_API 1
 #endif
 
 #if !defined(ENABLE_WEBCONTENT_CRASH_TESTING) && USE(APPLE_INTERNAL_SDK) \
     && ((PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 130000) \
-        || (PLATFORM(IOS) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 160000)) \
+        || (PLATFORM(IOS) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 160000) \
+        || PLATFORM(VISION)) \
     && HAVE(CHIRP_SPI)
 #define ENABLE_WEBCONTENT_CRASH_TESTING 1
 #endif
 
-#if !defined(ENABLE_WEBCONTENT_GPU_SANDBOX_EXTENSIONS_BLOCKING) && PLATFORM(IOS) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 170000
+#if !defined(ENABLE_WEBCONTENT_GPU_SANDBOX_EXTENSIONS_BLOCKING) && ((PLATFORM(IOS) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 170000) || PLATFORM(VISION))
 #define ENABLE_WEBCONTENT_GPU_SANDBOX_EXTENSIONS_BLOCKING 1
 #endif
 
@@ -868,3 +873,39 @@
 #if !defined(ENABLE_THREADED_ANIMATION_RESOLUTION)
 #define ENABLE_THREADED_ANIMATION_RESOLUTION 1
 #endif
+
+#if !defined(ENABLE_WEBGL_IN_WORKERS) \
+    && ((PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 140000) \
+    || ((PLATFORM(IOS) || PLATFORM(MACCATALYST)) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 170000) \
+    || (PLATFORM(WATCHOS) && __WATCH_OS_VERSION_MIN_REQUIRED >= 100000) \
+    || (PLATFORM(APPLETV) && __TV_OS_VERSION_MIN_REQUIRED >= 170000) \
+    || PLATFORM(VISION))
+#define ENABLE_WEBGL_IN_WORKERS 1
+#endif
+
+#if !defined(ENABLE_CARENDERSERVER_BLOCKING_IN_WEBCONTENT) \
+    && (((PLATFORM(IOS) || PLATFORM(MACCATALYST)) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 170000) && !PLATFORM(VISION) \
+    || (PLATFORM(WATCHOS) && __WATCH_OS_VERSION_MIN_REQUIRED >= 100000) \
+    || (PLATFORM(APPLETV) && __TV_OS_VERSION_MIN_REQUIRED >= 170000) \
+    || PLATFORM(VISION))
+#define ENABLE_CARENDERSERVER_BLOCKING_IN_WEBCONTENT 1
+#endif
+
+#if !defined(ENABLE_LAUNCHD_BLOCKING_IN_WEBCONTENT) \
+    && ((PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 140000) \
+    || PLATFORM(IOS_FAMILY))
+#define ENABLE_LAUNCHD_BLOCKING_IN_WEBCONTENT 1
+#endif
+
+#if !defined(ENABLE_LOGD_BLOCKING_IN_WEBCONTENT) && PLATFORM(IOS) && !PLATFORM(VISION) && USE(APPLE_INTERNAL_SDK)
+#define ENABLE_LOGD_BLOCKING_IN_WEBCONTENT 1
+#endif
+
+#if !defined(ENABLE_SAMPLE_BUFFER_CONTENT_KEY_SESSION_SUPPORT) \
+    && ((PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 140000) \
+    || ((PLATFORM(IOS) || PLATFORM(MACCATALYST)) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 170000) \
+    || (PLATFORM(WATCHOS) && __WATCH_OS_VERSION_MIN_REQUIRED >= 100000) \
+    || (PLATFORM(APPLETV) && __TV_OS_VERSION_MIN_REQUIRED >= 170000) \
+    || PLATFORM(VISION))
+#define ENABLE_SAMPLE_BUFFER_CONTENT_KEY_SESSION_SUPPORT 1
+#endif
