@@ -85,10 +85,6 @@
 #define HAVE_VISIBILITY_PROPAGATION_VIEW 1
 #endif
 
-#if !defined(HAVE_AVSTREAMSESSION) && PLATFORM(MAC)
-#define HAVE_AVSTREAMSESSION 1
-#endif
-
 #if !defined(HAVE_TIMINGSAFE_BCMP) && (PLATFORM(COCOA) || OS(OPENBSD) || OS(FREEBSD))
 #define HAVE_TIMINGSAFE_BCMP 1
 #endif
@@ -173,7 +169,7 @@
 #define HAVE_PTHREAD_MAIN_NP 1
 #endif
 
-#if OS(DARWIN) && (CPU(X86_64) || CPU(ARM64))
+#if OS(DARWIN) && (CPU(X86_64) || CPU(ARM64)) && CPU(ADDRESS64)
 #define HAVE_INT128_T 1
 #endif
 
@@ -249,7 +245,7 @@
 #define HAVE_FAST_TLS 1
 
 #if (PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 120000) \
-    || (PLATFORM(IOS) && !PLATFORM(IOS_FAMILY_SIMULATOR) && !PLATFORM(MACCATALYST))
+    || ((PLATFORM(IOS) || PLATFORM(VISION)) && !PLATFORM(IOS_FAMILY_SIMULATOR) && !PLATFORM(MACCATALYST))
 #define HAVE_SIMPLIFIED_FAST_TLS_BASE 1
 #endif
 
@@ -346,21 +342,23 @@
 #endif
 
 #if (PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 120000) \
-    || (PLATFORM(IOS) || PLATFORM(MACCATALYST))
+    || (PLATFORM(IOS) || PLATFORM(MACCATALYST) || PLATFORM(VISION))
 #define HAVE_NETWORK_CONNECTION_PRIVACY_STANCE 1
 #endif
 
 #if (PLATFORM(MAC) && __MAC_OS_X_VERSION_MAX_ALLOWED >= 110300) \
     || (((PLATFORM(IOS) && !PLATFORM(IOS_FAMILY_SIMULATOR)) || PLATFORM(MACCATALYST)) && __IPHONE_OS_VERSION_MAX_ALLOWED >= 140500) \
     || (PLATFORM(WATCHOS) && !PLATFORM(IOS_FAMILY_SIMULATOR) && __WATCH_OS_VERSION_MAX_ALLOWED >= 70500) \
-    || (PLATFORM(APPLETV) && !PLATFORM(IOS_FAMILY_SIMULATOR) && __TV_OS_VERSION_MAX_ALLOWED >= 140500)
+    || (PLATFORM(APPLETV) && !PLATFORM(IOS_FAMILY_SIMULATOR) && __TV_OS_VERSION_MAX_ALLOWED >= 140500) \
+    || (PLATFORM(VISION) && !PLATFORM(IOS_FAMILY_SIMULATOR))
 #define HAVE_TASK_IDENTITY_TOKEN 1
 #endif
 
 #if (PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 130000) \
     || (((PLATFORM(IOS) && !PLATFORM(IOS_FAMILY_SIMULATOR)) || PLATFORM(MACCATALYST)) && __IPHONE_OS_VERSION_MAX_ALLOWED >= 160000) \
     || ((PLATFORM(WATCHOS) && !PLATFORM(IOS_FAMILY_SIMULATOR)) && __WATCH_OS_VERSION_MAX_ALLOWED >= 90000) \
-    || ((PLATFORM(APPLETV) && !PLATFORM(IOS_FAMILY_SIMULATOR)) && __TV_OS_VERSION_MAX_ALLOWED >= 160000)
+    || ((PLATFORM(APPLETV) && !PLATFORM(IOS_FAMILY_SIMULATOR)) && __TV_OS_VERSION_MAX_ALLOWED >= 160000) \
+    || (PLATFORM(VISION) && !PLATFORM(IOS_FAMILY_SIMULATOR))
 #define HAVE_CG_CONTEXT_SET_OWNER_IDENTITY 1
 #endif
 
@@ -381,7 +379,7 @@
 #define HAVE_HSTS_STORAGE_PATH 1
 #endif
 
-#if PLATFORM(MAC) || PLATFORM(IOS) || PLATFORM(MACCATALYST)
+#if PLATFORM(MAC) || PLATFORM(IOS) || PLATFORM(MACCATALYST) || PLATFORM(VISION)
 #define HAVE_URL_FORMATTING 1
 #endif
 
@@ -393,19 +391,20 @@
 #define HAVE_MEMMEM 1
 #endif
 
-#if PLATFORM(MAC) || PLATFORM(IOS) || PLATFORM(MACCATALYST)
+#if PLATFORM(MAC) || PLATFORM(IOS) || PLATFORM(MACCATALYST) || PLATFORM(VISION)
 #define HAVE_AVCONTENTKEYSESSION 1
 #endif
 
 #if (PLATFORM(MAC) && __MAC_OS_X_VERSION_MAX_ALLOWED >= 101504) \
     || (((PLATFORM(IOS) && !PLATFORM(IOS_FAMILY_SIMULATOR)) || PLATFORM(MACCATALYST)) && __IPHONE_OS_VERSION_MAX_ALLOWED >= 130400) \
     || (PLATFORM(WATCHOS) && !PLATFORM(IOS_FAMILY_SIMULATOR) && __WATCH_OS_VERSION_MAX_ALLOWED >= 60400) \
-    || (PLATFORM(APPLETV) && !PLATFORM(IOS_FAMILY_SIMULATOR) && __TV_OS_VERSION_MAX_ALLOWED >= 130400)
+    || (PLATFORM(APPLETV) && !PLATFORM(IOS_FAMILY_SIMULATOR) && __TV_OS_VERSION_MAX_ALLOWED >= 130400) \
+    || (PLATFORM(VISION) && !PLATFORM(IOS_FAMILY_SIMULATOR))
 #define HAVE_AVCONTENTKEYREPORTGROUP 1
 #endif
 
 #if PLATFORM(MAC) \
-    || (PLATFORM(IOS) && !PLATFORM(IOS_FAMILY_SIMULATOR)) \
+    || ((PLATFORM(IOS) || PLATFORM(VISION)) && !PLATFORM(IOS_FAMILY_SIMULATOR)) \
     || PLATFORM(MACCATALYST)
 #define HAVE_AVCONTENTKEYSESSIONWILLOUTPUTBEOBSCURED 1
 #endif
@@ -413,20 +412,21 @@
 #if (PLATFORM(MAC) && __MAC_OS_X_VERSION_MAX_ALLOWED >= 110300) \
     || (((PLATFORM(IOS) && !PLATFORM(IOS_FAMILY_SIMULATOR)) || PLATFORM(MACCATALYST)) && __IPHONE_OS_VERSION_MAX_ALLOWED >= 140500) \
     || (PLATFORM(WATCHOS) && !PLATFORM(IOS_FAMILY_SIMULATOR) && __WATCH_OS_VERSION_MAX_ALLOWED >= 70400) \
-    || (PLATFORM(APPLETV) && !PLATFORM(IOS_FAMILY_SIMULATOR) && __TV_OS_VERSION_MAX_ALLOWED >= 140500)
+    || (PLATFORM(APPLETV) && !PLATFORM(IOS_FAMILY_SIMULATOR) && __TV_OS_VERSION_MAX_ALLOWED >= 140500) \
+    || (PLATFORM(VISION) && !PLATFORM(IOS_FAMILY_SIMULATOR))
 #define HAVE_AVCONTENTKEYSPECIFIER 1
 #endif
 
-#if PLATFORM(MAC) || PLATFORM(IOS) || PLATFORM(MACCATALYST)
+#if PLATFORM(MAC) || PLATFORM(IOS) || PLATFORM(MACCATALYST) || PLATFORM(VISION)
 #define HAVE_SEC_KEY_PROXY 1
 #endif
 
 /* FIXME: Should this be enabled for IOS_FAMILY, not just IOS? */
-#if PLATFORM(MAC) || PLATFORM(IOS)
+#if PLATFORM(MAC) || PLATFORM(IOS) || PLATFORM(VISION)
 #define HAVE_FOUNDATION_WITH_SAME_SITE_COOKIE_SUPPORT 1
 #endif
 
-#if PLATFORM(MAC) || PLATFORM(IOS) || PLATFORM(MACCATALYST) || PLATFORM(GTK) || PLATFORM(WPE)
+#if PLATFORM(MAC) || PLATFORM(IOS) || PLATFORM(MACCATALYST) || PLATFORM(VISION) || PLATFORM(GTK) || PLATFORM(WPE)
 #define HAVE_OS_DARK_MODE_SUPPORT 1
 #endif
 
@@ -446,16 +446,16 @@
 #define HAVE_SEC_ACCESS_CONTROL 1
 #endif
 
-#if PLATFORM(IOS)
+#if PLATFORM(IOS) || PLATFORM(VISION)
 /* FIXME: SafariServices.framework exists on macOS. It is only used by WebKit on iOS, so the behavior is correct, but the name is misleading. */
 #define HAVE_SAFARI_SERVICES_FRAMEWORK 1
 #endif
 
-#if PLATFORM(MAC) || PLATFORM(IOS) || PLATFORM(WATCHOS) || PLATFORM(MACCATALYST)
+#if PLATFORM(MAC) || PLATFORM(IOS) || PLATFORM(WATCHOS) || PLATFORM(MACCATALYST) || PLATFORM(VISION)
 #define HAVE_SAFE_BROWSING 1
 #endif
 
-#if PLATFORM(IOS) || PLATFORM(MACCATALYST)
+#if PLATFORM(IOS) || PLATFORM(MACCATALYST) || PLATFORM(VISION)
 #define HAVE_LINK_PREVIEW 1
 #endif
 
@@ -463,7 +463,10 @@
 #define HAVE_CFNETWORK_OVERRIDE_SESSION_COOKIE_ACCEPT_POLICY 1
 #define HAVE_CFNETWORK_NSURLSESSION_CONNECTION_CACHE_LIMITS 1
 #define HAVE_CFNETWORK_METRICS_APIS_V4 1
-#define HAVE_CFNETWORK_ALTERNATIVE_SERVICE 1
+#endif
+
+#if PLATFORM(COCOA) || (defined(USE_CURL) && USE_CURL)
+#define HAVE_ALTERNATIVE_SERVICE 1
 #endif
 
 #if PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED < 120000
@@ -479,29 +482,29 @@
 #define HAVE_MDNS_FAST_REGISTRATION 1
 #endif
 
-#if PLATFORM(IOS)
+#if PLATFORM(IOS) || PLATFORM(VISION)
 #define HAVE_ARKIT_QUICK_LOOK_PREVIEW_ITEM 1
 #endif
 
-#if PLATFORM(IOS) || PLATFORM(MACCATALYST)
+#if PLATFORM(IOS) || PLATFORM(MACCATALYST) || PLATFORM(VISION)
 #define HAVE_UI_WK_DOCUMENT_CONTEXT 1
 #endif
 
-#if (PLATFORM(IOS) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 130400)
+#if (PLATFORM(IOS) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 130400) || PLATFORM(VISION)
 #define HAVE_UI_POINTER_INTERACTION 1
 #endif
 
-#if (PLATFORM(IOS) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 130400) || PLATFORM(MACCATALYST)
+#if (PLATFORM(IOS) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 130400) || PLATFORM(MACCATALYST) || PLATFORM(VISION)
 #define HAVE_UIKIT_WITH_MOUSE_SUPPORT 1
 #define HAVE_UI_PARALLAX_TRANSITION_GESTURE_RECOGNIZER 1
 #endif
 
-#if ((PLATFORM(IOS) || PLATFORM(MACCATALYST)) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 160000)
+#if ((PLATFORM(IOS) || PLATFORM(MACCATALYST)) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 160000) || PLATFORM(VISION)
 #define HAVE_UIKIT_HOVER_EVENT_PROTOCOL 1
 #define HAVE_UIKIT_BAR_BUTTON_LAYOUT_CUSTOMIZATION 1
 #endif
 
-#if PLATFORM(IOS) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 130400
+#if PLATFORM(IOS) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 130400 || PLATFORM(VISION)
 #define HAVE_MOUSE_DEVICE_OBSERVATION 1
 #endif
 
@@ -521,16 +524,15 @@
 #define HAVE_APP_LINKS_WITH_ISENABLED 1
 #endif
 
-#if PLATFORM(IOS)
+#if PLATFORM(IOS) || PLATFORM(VISION)
 #define HAVE_ROUTE_SHARING_POLICY_LONG_FORM_VIDEO 1
 #endif
 
-#if PLATFORM(IOS) && !PLATFORM(IOS_SIMULATOR)
+#if (PLATFORM(IOS) || PLATFORM(VISION)) && !PLATFORM(IOS_SIMULATOR)
 #define HAVE_DEVICE_MANAGEMENT 1
 #endif
 
-#if (PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 120000) || PLATFORM(IOS) || PLATFORM(MACCATALYST)
-// FIXME: This should be enabled on watchOS and tvOS, but there are some build failures.
+#if (PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 120000) || PLATFORM(IOS_FAMILY)
 #define HAVE_NSURLSESSION_WEBSOCKET 1
 #endif
 
@@ -550,11 +552,7 @@
 #define HAVE_VIDEO_PERFORMANCE_METRICS 1
 #endif
 
-#if PLATFORM(COCOA) && !PLATFORM(MACCATALYST)
-#define HAVE_CORETEXT_AUTO_OPTICAL_SIZING 1
-#endif
-
-#if PLATFORM(IOS) || PLATFORM(MAC)
+#if PLATFORM(IOS) || PLATFORM(MAC) || PLATFORM(VISION)
 #define HAVE_APP_SSO 1
 #endif
 
@@ -618,7 +616,7 @@
 #define HAVE_ALTERNATE_ICONS 1
 #endif
 
-#if (PLATFORM(IOS) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 140000)
+#if (PLATFORM(IOS) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 140000) || PLATFORM(VISION)
 #define HAVE_UICONTEXTMENU_LOCATION 1
 #endif
 
@@ -626,12 +624,12 @@
 #define HAVE_SYSTEM_FEATURE_FLAGS 1
 #endif
 
-#if PLATFORM(IOS)
+#if PLATFORM(IOS) || PLATFORM(VISION)
 #define HAVE_AVOBSERVATIONCONTROLLER 1
 #endif
 
 // FIXME: Should this be enabled on other iOS-family platforms?
-#if PLATFORM(IOS) || PLATFORM(MACCATALYST)
+#if PLATFORM(IOS) || PLATFORM(MACCATALYST) || PLATFORM(VISION)
 #define HAVE_CANCEL_WEB_TOUCH_EVENTS_GESTURE 1
 #endif
 
@@ -639,7 +637,7 @@
 #define HAVE_DD_HIGHLIGHT_CREATE_WITH_SCALE 1
 #endif
 
-#if PLATFORM(IOS) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 130400
+#if (PLATFORM(IOS) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 130400) || PLATFORM(VISION)
 #define HAVE_UISCENE_BASED_VIEW_SERVICE_STATE_NOTIFICATIONS 1
 #endif
 
@@ -647,6 +645,10 @@
 #define HAVE_IOS_JIT_RESTRICTIONS 1
 #endif
 
+#if (PLATFORM(MAC) || PLATFORM(MACCATALYST)) && CPU(ARM64)
+#define HAVE_MAC_JIT_RESTRICTIONS 1
+#endif
+
 #if PLATFORM(MAC)
 #define HAVE_AVAUDIO_ROUTING_ARBITER 1
 #endif
@@ -655,15 +657,15 @@
 #define HAVE_MEDIA_USAGE_FRAMEWORK 1
 #endif
 
-#if PLATFORM(IOS) || PLATFORM(MAC)
+#if PLATFORM(IOS) || PLATFORM(MAC) || PLATFORM(VISION)
 #define HAVE_PASSKIT_INSTALLMENTS 1
 #endif
 
-#if PLATFORM(MAC) || PLATFORM(IOS)
+#if PLATFORM(MAC) || PLATFORM(IOS) || PLATFORM(VISION)
 #define HAVE_PASSKIT_NEW_BUTTON_TYPES 1
 #endif
 
-#if (PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 120000) || PLATFORM(IOS)
+#if (PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 120000) || PLATFORM(IOS) || PLATFORM(VISION)
 #define HAVE_PASSKIT_RECURRING_SUMMARY_ITEM 1
 #define HAVE_PASSKIT_DEFERRED_SUMMARY_ITEM 1
 #define HAVE_PASSKIT_SHIPPING_METHOD_DATE_COMPONENTS_RANGE 1
@@ -672,7 +674,7 @@
 #define HAVE_PASSKIT_UPDATE_SHIPPING_METHODS_WHEN_CHANGING_SUMMARY_ITEMS 1
 #endif
 
-#if (PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 130000) || (PLATFORM(IOS) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 160000)
+#if (PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 130000) || (PLATFORM(IOS) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 160000) || PLATFORM(VISION)
 #define HAVE_PASSKIT_MODULARIZATION 1
 #define HAVE_PASSKIT_AUTOMATIC_RELOAD_SUMMARY_ITEM 1
 #define HAVE_PASSKIT_RECURRING_PAYMENTS 1
@@ -681,15 +683,24 @@
 #define HAVE_PASSKIT_PAYMENT_ORDER_DETAILS 1
 #endif
 
+#if (PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 130300) || (PLATFORM(IOS) && __IPHONE_OS_VERSION_MAX_ALLOWED >= 160400) || PLATFORM(VISION)
+#define HAVE_PASSKIT_DEFERRED_PAYMENTS 1
+#endif
+
 #if __has_include(<PassKitMacHelperTemp/PKPaymentAuthorizationViewController_Private.h>)
 #define HAVE_PASSKIT_MAC_HELPER_TEMP 1
 #endif
 
-#if (PLATFORM(MAC) && __MAC_OS_X_VERSION_MAX_ALLOWED >= 120300) || (PLATFORM(IOS) && __IPHONE_OS_VERSION_MAX_ALLOWED >= 150400)
+#if (PLATFORM(MAC) && __MAC_OS_X_VERSION_MAX_ALLOWED >= 120300) || (PLATFORM(IOS) && __IPHONE_OS_VERSION_MAX_ALLOWED >= 150400) || PLATFORM(VISION)
 #define HAVE_PASSKIT_DEFAULT_SHIPPING_METHOD 1
 #endif
 
-#if PLATFORM(IOS) || PLATFORM(MAC)
+#if (PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 140000) || (PLATFORM(IOS) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 170000) || PLATFORM(VISION)
+#define HAVE_PASSKIT_APPLE_PAY_LATER_AVAILABILITY 1
+#endif
+
+
+#if PLATFORM(IOS) || PLATFORM(MAC) || PLATFORM(VISION)
 #define HAVE_DISPLAY_LAYER_BIPLANAR_SUPPORT 1
 #endif
 
@@ -729,7 +740,6 @@
 #define HAVE_CF_PREFS_SET_READ_ONLY 1
 #define HAVE_CGIMAGESOURCE_WITH_ACCURATE_LOOP_COUNT 1
 #define HAVE_CGIMAGESOURCE_WITH_SET_ALLOWABLE_TYPES 1
-#define HAVE_HSTS_STORAGE 1
 #define HAVE_LSDATABASECONTEXT 1
 #define HAVE_PRECONNECT_PING 1
 #define HAVE_WEBP 1
@@ -738,15 +748,7 @@
 #define HAVE_CORE_ANIMATION_FIX_FOR_RADAR_93560567 0
 #endif
 
-#if PLATFORM(COCOA)
-#define HAVE_CTFONTISSYSTEMUIFONT 1
-#endif
-
-#if PLATFORM(COCOA)
-#define HAVE_CTFONTDESCRIPTOR_CREATE_WITH_TEXT_STYLE_AND_ATTRIBUTES 1
-#endif
-
-#if PLATFORM(IOS) || PLATFORM(MACCATALYST)
+#if PLATFORM(IOS) || PLATFORM(MACCATALYST) || PLATFORM(VISION)
 #define HAVE_UI_EVENT_ATTRIBUTION 1
 #endif
 
@@ -762,27 +764,28 @@
 #define HAVE_CATALYST_USER_INTERFACE_IDIOM_AND_SCALE_FACTOR 1
 #endif
 
-#if (PLATFORM(IOS) || PLATFORM(MACCATALYST)) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 140000
+#if ((PLATFORM(IOS) || PLATFORM(MACCATALYST)) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 140000) \
+    || PLATFORM(VISION)
 #define HAVE_PENCILKIT_TEXT_INPUT 1
 #endif
 
-#if PLATFORM(IOS) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 140000
+#if (PLATFORM(IOS) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 140000) || PLATFORM(VISION)
 #define HAVE_STYLUS_DEVICE_OBSERVATION 1
 #endif
 
-#if PLATFORM(IOS) || PLATFORM(MAC)
+#if PLATFORM(IOS) || PLATFORM(MAC) || PLATFORM(VISION)
 #if !defined(HAVE_APPLE_ATTESTATION)
 #define HAVE_APPLE_ATTESTATION 1
 #endif
 #endif
 
-#if ((PLATFORM(IOS) || PLATFORM(MACCATALYST)) && __IPHONE_OS_VERSION_MAX_ALLOWED >= 140300) || (PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 130000)
+#if ((PLATFORM(IOS) || PLATFORM(MACCATALYST)) && __IPHONE_OS_VERSION_MAX_ALLOWED >= 140300) || (PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 130000) || PLATFORM(VISION)
 #if !defined(HAVE_AVSPEECHSYNTHESIS_SYSTEMVOICE)
 #define HAVE_AVSPEECHSYNTHESIS_SYSTEMVOICE 1
 #endif
 #endif
 
-#if PLATFORM(IOS) || PLATFORM(MAC)
+#if PLATFORM(IOS) || PLATFORM(MAC) || PLATFORM(VISION)
 #if !defined(HAVE_TCC_IOS_14_BIG_SUR_SPI)
 #define HAVE_TCC_IOS_14_BIG_SUR_SPI 1
 #endif
@@ -818,7 +821,7 @@
 #define HAVE_SPEECHRECOGNIZER 1
 #endif
 
-#if PLATFORM(IOS) || PLATFORM(MACCATALYST) || PLATFORM(MAC)
+#if PLATFORM(IOS) || PLATFORM(MACCATALYST) || PLATFORM(MAC) || PLATFORM(VISION)
 #define HAVE_AVCAPTUREDEVICE 1
 #endif
 
@@ -826,7 +829,7 @@
 #define HAVE_SANDBOX_MESSAGE_FILTERING 1
 #endif
 
-#if (PLATFORM(IOS) && !PLATFORM(IOS_FAMILY_SIMULATOR)) || PLATFORM(MAC)
+#if ((PLATFORM(IOS) || PLATFORM(VISION)) && !PLATFORM(IOS_FAMILY_SIMULATOR)) || PLATFORM(MAC)
 #define HAVE_COREVIDEO_COMPRESSED_PIXEL_FORMAT_TYPES 1
 #endif
 
@@ -837,16 +840,18 @@
 #if (PLATFORM(MAC) && __MAC_OS_X_VERSION_MAX_ALLOWED >= 110200) \
     || (PLATFORM(IOS) && __IPHONE_OS_VERSION_MAX_ALLOWED >= 140400) \
     || (PLATFORM(WATCHOS) && __WATCH_OS_VERSION_MAX_ALLOWED >= 70300) \
-    || (PLATFORM(APPLETV) && __TV_OS_VERSION_MAX_ALLOWED >= 140400)
+    || (PLATFORM(APPLETV) && __TV_OS_VERSION_MAX_ALLOWED >= 140400) \
+    || PLATFORM(VISION)
 #define HAVE_LIBXSLT_FIX_FOR_RADAR_71864140 1
 #endif
 
-#if (PLATFORM(IOS) && __IPHONE_OS_VERSION_MAX_ALLOWED >= 140400)
+#if (PLATFORM(IOS) && __IPHONE_OS_VERSION_MAX_ALLOWED >= 140400) || PLATFORM(VISION)
 #define HAVE_ASC_AUTH_UI 1
 #endif
 
 #if PLATFORM(IOS) \
-    || (PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 120000)
+    || (PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 120000) \
+    || PLATFORM(VISION)
 #define HAVE_ASC_WEBKIT_SPI 1
 #endif
 
@@ -863,24 +868,19 @@
 #endif
 
 #if (PLATFORM(MAC) && __MAC_OS_X_VERSION_MAX_ALLOWED >= 110200) \
-    || (PLATFORM(IOS) && __IPHONE_OS_VERSION_MAX_ALLOWED >= 140400)
+    || (PLATFORM(IOS) && __IPHONE_OS_VERSION_MAX_ALLOWED >= 140400) \
+    || PLATFORM(VISION)
 #define HAVE_AVCONTENTKEYREQUEST_PENDING_PROTECTION_STATUS 1
 #endif
 
 #if (PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 130000) \
     || (PLATFORM(IOS) && __IPHONE_OS_VERSION_MAX_ALLOWED >= 160100) \
     || (PLATFORM(APPLETV) && __TV_OS_VERSION_MAX_ALLOWED >= 160100) \
-    || (PLATFORM(WATCHOS) && __WATCH_OS_VERSION_MAX_ALLOWED >= 90100)
+    || (PLATFORM(WATCHOS) && __WATCH_OS_VERSION_MAX_ALLOWED >= 90100) \
+    || PLATFORM(VISION)
 #define HAVE_AVCONTENTKEYREQUEST_COMPATABILITIY_MODE 1
 #endif
 
-#if PLATFORM(MAC) || PLATFORM(IOS) || PLATFORM(MACCATALYST)
-#if !defined(HAVE_CORE_ANIMATION_SEPARATED_LAYERS)
-#define HAVE_CORE_ANIMATION_SEPARATED_LAYERS 1
-#define HAVE_CORE_ANIMATION_SEPARATED_PORTALS 1
-#endif
-#endif
-
 #if (PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 120000) \
     || PLATFORM(IOS_FAMILY)
 #define HAVE_LEVEL_3_SYSTEM_FONT_WIDTH_VALUES 1
@@ -927,7 +927,8 @@
 #endif
 
 #if (PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 120000) \
-    || PLATFORM(IOS)
+    || PLATFORM(IOS) \
+    || PLATFORM(VISION)
 // FIXME: Does this intend to exclude macCatalyst?
 #define HAVE_RSA_BSSA 1
 #endif
@@ -944,10 +945,14 @@
 #define HAVE_NSURL_TITLE 1
 #endif
 
-#if PLATFORM(MAC) || PLATFORM(IOS) || PLATFORM(MACCATALYST)
+#if PLATFORM(MAC) || PLATFORM(IOS) || PLATFORM(MACCATALYST) || PLATFORM(VISION)
 #define HAVE_PDFKIT 1
 #endif
 
+#if PLATFORM(COCOA) && !PLATFORM(WATCHOS)
+#define HAVE_VISION 1
+#endif
+
 #if (PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 110000 && __MAC_OS_X_VERSION_MAX_ALLOWED >= 110300) \
     || (PLATFORM(IOS_FAMILY) && __IPHONE_OS_VERSION_MAX_ALLOWED >= 140500) \
     || (PLATFORM(APPLETV) && __TV_OS_VERSION_MAX_ALLOWED >= 140500) \
@@ -972,47 +977,57 @@
 #define HAVE_CFNETWORK_NSURLSESSION_HSTS_WITH_UNTRUSTED_ROOT 1
 #define HAVE_NSURLSESSION_TASK_DELEGATE 1
 #define HAVE_AUDIO_OBJECT_PROPERTY_ELEMENT_MAIN 1
+#define HAVE_IOKIT_MAIN_PORT 1
 #define HAVE_IMAGE_RESTRICTED_DECODING 1
 #define HAVE_XPC_CONNECTION_COPY_INVALIDATION_REASON 1
 #endif
 
-#if (PLATFORM(IOS) && !PLATFORM(IOS_SIMULATOR)) \
+#if ((PLATFORM(IOS) || PLATFORM(VISION)) && !PLATFORM(IOS_SIMULATOR)) \
     || PLATFORM(MACCATALYST) \
     || (PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 120000)
 #define HAVE_ASV_INLINE_PREVIEW 1
 #endif
 
-#if PLATFORM(IOS) || PLATFORM(MACCATALYST) || PLATFORM(MAC)
+#if PLATFORM(IOS) || PLATFORM(MACCATALYST) || PLATFORM(MAC) || PLATFORM(VISION)
 #define HAVE_NSURLPROTOCOL_WITH_SKIPAPPSSO 1
 #endif
 
-#if PLATFORM(IOS) || PLATFORM(MACCATALYST)
+#if PLATFORM(IOS) || PLATFORM(MACCATALYST) || PLATFORM(VISION)
 #define HAVE_UIDATEPICKER_OVERLAY_PRESENTATION 1
 #endif
 
 #if (PLATFORM(IOS) || PLATFORM(MACCATALYST)) \
-    || (PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 120000)
+    || (PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 120000) \
+    || PLATFORM(VISION)
 #define HAVE_UPDATE_WEB_ACCESSIBILITY_SETTINGS 1
 #endif
 
 #if PLATFORM(COCOA)
 #define HAVE_CORE_TEXT_SBIX_IMAGE_SIZE_FUNCTIONS 1
+#define HAVE_WOFF_SUPPORT 1
 #endif
 
 #if (PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 120000) \
-    || (PLATFORM(IOS) || PLATFORM(MACCATALYST))
+    || (PLATFORM(IOS) || PLATFORM(MACCATALYST) || PLATFORM(VISION))
 #if !defined(HAVE_VK_IMAGE_ANALYSIS)
 #define HAVE_VK_IMAGE_ANALYSIS 1
 #endif
 #endif
 
-#if PLATFORM(IOS)
+#if PLATFORM(IOS) || PLATFORM(VISION)
 #if !defined(HAVE_VK_IMAGE_ANALYSIS_FOR_MACHINE_READABLE_CODES)
 #define HAVE_VK_IMAGE_ANALYSIS_FOR_MACHINE_READABLE_CODES 1
 #endif
 #endif
 
-#if PLATFORM(IOS)
+#if !defined(HAVE_VK_IMAGE_ANALYSIS_TYPE_IMAGE_SEGMENTATION) \
+    && ((PLATFORM(MAC) && __MAC_OS_X_VERSION_MAX_ALLOWED >= 130000) \
+    || ((PLATFORM(IOS) || PLATFORM(MACCATALYST)) && __IPHONE_OS_VERSION_MAX_ALLOWED >= 160400)) \
+    || PLATFORM(VISION)
+#define HAVE_VK_IMAGE_ANALYSIS_TYPE_IMAGE_SEGMENTATION 1
+#endif
+
+#if PLATFORM(IOS) || PLATFORM(VISION)
 #define HAVE_PER_APP_ACCESSIBILITY_PREFERENCES 1
 #endif
 
@@ -1031,7 +1046,7 @@
 #define HAVE_QUICKLOOK_PREVIEW_ACTIVITY 1
 #endif
 
-#if PLATFORM(IOS) || PLATFORM(MACCATALYST)
+#if PLATFORM(IOS) || PLATFORM(MACCATALYST) || PLATFORM(VISION)
 #define HAVE_UITOOLBAR_STANDARD_APPEARANCE 1
 #endif
 
@@ -1055,11 +1070,12 @@
     || (PLATFORM(IOS) && __IPHONE_OS_VERSION_MAX_ALLOWED >= 140500) \
     || (PLATFORM(WATCHOS) && __WATCH_OS_VERSION_MAX_ALLOWED >= 70400) \
     || (PLATFORM(APPLETV) && __TV_OS_VERSION_MAX_ALLOWED >= 140500) \
-    || (PLATFORM(MACCATALYST) && __IPHONE_OS_VERSION_MAX_ALLOWED >= 140500))
+    || (PLATFORM(MACCATALYST) && __IPHONE_OS_VERSION_MAX_ALLOWED >= 140500)) \
+    || PLATFORM(VISION)
 #define HAVE_AVSAMPLEBUFFERRENDERSYNCHRONIZER_RATEATHOSTTIME 1
 #endif
 
-#if PLATFORM(IOS) && !PLATFORM(IOS_FAMILY_SIMULATOR)
+#if (PLATFORM(IOS) || PLATFORM(VISION)) && !PLATFORM(IOS_FAMILY_SIMULATOR)
 #define HAVE_SYSTEM_STATUS 1
 #endif
 
@@ -1074,11 +1090,11 @@
 #endif
 #endif
 
-#if PLATFORM(MAC) || PLATFORM(IOS) || PLATFORM(MACCATALYST)
+#if PLATFORM(MAC) || PLATFORM(IOS) || PLATFORM(MACCATALYST) || PLATFORM(VISION)
 #define HAVE_AVSAMPLEBUFFERVIDEOOUTPUT 1
 #endif
 
-#if PLATFORM(IOS)
+#if PLATFORM(IOS) || PLATFORM(VISION)
 #define HAVE_PHOTOS_UI 1
 // FIXME (102246762): Remove this have (and make it true everywhere) when possible.
 #if __has_include(<PhotosUICore/PXActivityProgressController.h>)
@@ -1100,18 +1116,20 @@
 #if (PLATFORM(MAC) \
     || ((PLATFORM(IOS) || PLATFORM(MACCATALYST)) && __IPHONE_OS_VERSION_MAX_ALLOWED >= 160000) \
     || (PLATFORM(WATCHOS) && __WATCH_OS_VERSION_MAX_ALLOWED >= 90000) \
-    || (PLATFORM(APPLETV) && __TV_OS_VERSION_MAX_ALLOWED >= 160000))
+    || (PLATFORM(APPLETV) && __TV_OS_VERSION_MAX_ALLOWED >= 160000) \
+    || PLATFORM(VISION))
 #define HAVE_NSTEXTLIST_MARKER_FORMATS 1
 #endif
 
 #if (PLATFORM(IOS) && __IPHONE_OS_VERSION_MAX_ALLOWED >= 160000) \
     || (PLATFORM(WATCHOS) && __WATCH_OS_VERSION_MAX_ALLOWED >= 90000) \
-    || (PLATFORM(APPLETV) && __TV_OS_VERSION_MAX_ALLOWED >= 160000)
+    || (PLATFORM(APPLETV) && __TV_OS_VERSION_MAX_ALLOWED >= 160000) \
+    || PLATFORM(VISION)
 #define HAVE_CORE_LOCATION_WEBSITE_IDENTIFIERS 1
 #define HAVE_UIKIT_BAR_BUTTON_LAYOUT_CUSTOMIZATION 1
 #endif
 
-#if (PLATFORM(IOS) || PLATFORM(MACCATALYST)) && __IPHONE_OS_VERSION_MAX_ALLOWED >= 160000
+#if (PLATFORM(IOS) || PLATFORM(MACCATALYST) || PLATFORM(VISION)) && __IPHONE_OS_VERSION_MAX_ALLOWED >= 160000
 #if !defined(HAVE_TEXT_INTERACTION_WITH_CONTEXT_MENU_INTERACTION)
 #define HAVE_TEXT_INTERACTION_WITH_CONTEXT_MENU_INTERACTION 1
 #endif
@@ -1136,8 +1154,12 @@
 #define HAVE_WEBGPU_IMPLEMENTATION 1
 #endif
 
-#if HAVE(WEBGPU_IMPLEMENTATION) && ((PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 130000) || (PLATFORM(IOS) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 160000))
-#define HAVE_METAL_BUFFER_BINDING_REFLECTION 1
+#if PLATFORM(COCOA) && !PLATFORM(WATCHOS)
+#define HAVE_SHAPE_DETECTION_API_IMPLEMENTATION 1
+#endif
+
+#if HAVE(WEBGPU_IMPLEMENTATION) && ((PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 130000) || (PLATFORM(IOS) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 160000) || PLATFORM(VISION))
+#define HAVE_COREVIDEO_METAL_SUPPORT COREVIDEO_SUPPORTS_METAL
 #endif
 
 #if (PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 130000)
@@ -1159,17 +1181,18 @@
 #define HAVE_SCREEN_CAPTURE_KIT 1
 #endif
 
-#if (PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 130000)
-#define HAVE_SC_CONTENT_SHARING_SESSION 1
+#if (PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 140000)
+#define HAVE_SC_CONTENT_SHARING_PICKER 1
 #endif
 
 #if ((PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 130000) \
     || (PLATFORM(IOS) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 160000) \
-    || (PLATFORM(MACCATALYST) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 160000))
+    || (PLATFORM(MACCATALYST) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 160000) \
+    || PLATFORM(VISION))
 #define HAVE_SANDBOX_STATE_FLAGS 1
 #endif
 
-#if ((PLATFORM(IOS) || PLATFORM(MACCATALYST)) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 160000)
+#if ((PLATFORM(IOS) || PLATFORM(MACCATALYST) || PLATFORM(VISION)) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 160000)
 #define HAVE_UIFINDINTERACTION 1
 #endif
 
@@ -1192,11 +1215,11 @@
 #define HAVE_CFNETWORK_DISABLE_CACHE_SPI 1
 #endif
 
-#if ((PLATFORM(IOS) || PLATFORM(MACCATALYST)) && __IPHONE_OS_VERSION_MAX_ALLOWED >= 150400)
+#if ((PLATFORM(IOS) || PLATFORM(MACCATALYST) || PLATFORM(VISION)) && __IPHONE_OS_VERSION_MAX_ALLOWED >= 150400)
 #define HAVE_UIACTIVITYTYPE_SHAREPLAY 1
 #endif
 
-#if (PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 130000) || (PLATFORM(IOS) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 160000) || (PLATFORM(WATCHOS) && __WATCH_OS_VERSION_MIN_REQUIRED >= 90000)
+#if (PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 130000) || (PLATFORM(IOS) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 160000) || (PLATFORM(WATCHOS) && __WATCH_OS_VERSION_MIN_REQUIRED >= 90000) || PLATFORM(VISION)
 #define HAVE_SAFE_BROWSING_RESULT_DETAILS 1
 #endif
 
@@ -1204,7 +1227,8 @@
     || (PLATFORM(MACCATALYST) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 160000) \
     || (PLATFORM(IOS) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 160000) \
     || (PLATFORM(APPLETV) && __TV_OS_VERSION_MAX_ALLOWED >= 160000) \
-    || (PLATFORM(WATCHOS) && __WATCH_OS_VERSION_MIN_REQUIRED >= 90000)
+    || (PLATFORM(WATCHOS) && __WATCH_OS_VERSION_MIN_REQUIRED >= 90000) \
+    || PLATFORM(VISION)
 #define HAVE_MEMORY_ATTRIBUTION_VM_SHARE_SUPPORT 1
 #define HAVE_PROHIBIT_PRIVACY_PROXY 1
 #endif
@@ -1213,24 +1237,27 @@
     || (PLATFORM(MACCATALYST) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 160000) \
     || (PLATFORM(IOS) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 160000) \
     || (PLATFORM(APPLETV) && __TV_OS_VERSION_MAX_ALLOWED >= 160000) \
-    || (PLATFORM(WATCHOS) && __WATCH_OS_VERSION_MIN_REQUIRED >= 90000)
+    || (PLATFORM(WATCHOS) && __WATCH_OS_VERSION_MIN_REQUIRED >= 90000) \
+    || PLATFORM(VISION)
 #define HAVE_MACH_MEMORY_ENTRY_OWNERSHIP_IDENTITY_TOKEN_SUPPORT 1
 #endif
 
-#if (PLATFORM(IOS) && __IPHONE_OS_VERSION_MAX_ALLOWED >= 160000)
+#if (PLATFORM(IOS) && __IPHONE_OS_VERSION_MAX_ALLOWED >= 160000) || PLATFORM(VISION)
 #define HAVE_SKADNETWORK_v4 1
 #endif
 
 #if (PLATFORM(IOS) && __IPHONE_OS_VERSION_MAX_ALLOWED >= 160000) \
     || (PLATFORM(WATCHOS) && __WATCH_OS_VERSION_MAX_ALLOWED >= 90000) \
-    || (PLATFORM(APPLETV) && __TV_OS_VERSION_MAX_ALLOWED >= 160000)
+    || (PLATFORM(APPLETV) && __TV_OS_VERSION_MAX_ALLOWED >= 160000) \
+    || PLATFORM(VISION)
 #define HAVE_IMAGEIO_CREATE_UNPREMULTIPLIED_PNG 1
 #endif
 
 #if !defined(HAVE_UI_CONTEXT_MENU_PREVIEW_ITEM_IDENTIFIER) \
     && (((PLATFORM(IOS) || PLATFORM(MACCATALYST)) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 160000) \
     || (PLATFORM(WATCHOS) && __WATCH_OS_VERSION_MIN_REQUIRED >= 90000) \
-    || (PLATFORM(APPLETV) && __TV_OS_VERSION_MIN_REQUIRED >= 160000))
+    || (PLATFORM(APPLETV) && __TV_OS_VERSION_MIN_REQUIRED >= 160000) \
+    || PLATFORM(VISION))
 #define HAVE_UI_CONTEXT_MENU_PREVIEW_ITEM_IDENTIFIER 1
 #endif
 
@@ -1242,20 +1269,22 @@
     || (PLATFORM(MACCATALYST) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 160000) \
     || (PLATFORM(IOS) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 160000) \
     || (PLATFORM(APPLETV) && __TV_OS_VERSION_MAX_ALLOWED >= 160000) \
-    || (PLATFORM(WATCHOS) && __WATCH_OS_VERSION_MIN_REQUIRED >= 90000)
+    || (PLATFORM(WATCHOS) && __WATCH_OS_VERSION_MIN_REQUIRED >= 90000) \
+    || PLATFORM(VISION)
 #define HAVE_CORE_TEXT_FIX_FOR_RADAR_93925620 1
 #endif
 
-#if (PLATFORM(IOS) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 160000)
+#if (PLATFORM(IOS) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 160000) || PLATFORM(VISION)
 #define HAVE_DDRESULT_DISABLE_URL_SCHEME_CHECKING 1
 #endif
 
 #if ((PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 130000) \
-    || (PLATFORM(IOS) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 160000))
+    || (PLATFORM(IOS) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 160000) \
+    || PLATFORM(VISION))
 #define HAVE_SYSTEM_CONTENT_LS_DATABASE 1
 #endif
 
-#if PLATFORM(IOS) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 160000
+#if (PLATFORM(IOS) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 160000) || PLATFORM(VISION)
 #if !defined(HAVE_UIKIT_RESIZABLE_WINDOWS)
 #define HAVE_UIKIT_RESIZABLE_WINDOWS 1
 #endif
@@ -1277,7 +1306,8 @@
 #endif
 
 #if ((PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 130000) \
-    || (PLATFORM(IOS) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 160000))
+    || (PLATFORM(IOS) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 160000) \
+    || PLATFORM(VISION))
 #define HAVE_APPLE_PUSH_SERVICE_URL_TOKEN_SUPPORT 1
 #endif
 
@@ -1285,10 +1315,15 @@
     || (PLATFORM(MACCATALYST) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 160000) \
     || (PLATFORM(IOS) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 160000) \
     || (PLATFORM(APPLETV) && __TV_OS_VERSION_MAX_ALLOWED >= 160000) \
-    || (PLATFORM(WATCHOS) && __WATCH_OS_VERSION_MIN_REQUIRED >= 90000)
+    || (PLATFORM(WATCHOS) && __WATCH_OS_VERSION_MIN_REQUIRED >= 90000) \
+    || PLATFORM(VISION)
 #define HAVE_AVIF 1
 #endif
 
+#if PLATFORM(IOS_SIMULATOR) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 160000
+#define HAVE_CG_IMAGE_SOURCE_AVIF_IMAGE_TYPES_BUG 1
+#endif
+
 #if !PLATFORM(IOS_FAMILY)
 #define HAVE_MEDIA_VOLUME_PER_ELEMENT 1
 #endif
@@ -1296,7 +1331,8 @@
 #if (PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 130000) \
     || ((PLATFORM(IOS) || PLATFORM(MACCATALYST)) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 160000) \
     || (PLATFORM(WATCHOS) && __WATCH_OS_VERSION_MIN_REQUIRED >= 90000) \
-    || (PLATFORM(APPLETV) && __TV_OS_VERSION_MIN_REQUIRED >= 160000)
+    || (PLATFORM(APPLETV) && __TV_OS_VERSION_MIN_REQUIRED >= 160000) \
+    || PLATFORM(VISION)
 #if !defined(HAVE_AVSAMPLEBUFFERDISPLAYLAYER_COPYDISPLAYEDPIXELBUFFER)
 #define HAVE_AVSAMPLEBUFFERDISPLAYLAYER_COPYDISPLAYEDPIXELBUFFER 1
 #endif
@@ -1312,7 +1348,8 @@
 
 #if (PLATFORM(IOS) && __IPHONE_OS_VERSION_MIN_REQUIRED < 160000) \
     || PLATFORM(WATCHOS) \
-    || PLATFORM(APPLETV)
+    || PLATFORM(APPLETV) \
+    || PLATFORM(VISION)
 #define HAVE_AGX_COMPILER_SERVICE 1
 #endif
 
@@ -1322,7 +1359,8 @@
 
 #if (!defined(HAVE_LOCKDOWN_MODE_PDF_ADDITIONS) && \
     (PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 130000 \
-    || PLATFORM(IOS) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 160000))
+    || PLATFORM(IOS) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 160000) \
+    || PLATFORM(VISION))
 #define HAVE_LOCKDOWN_MODE_PDF_ADDITIONS 1
 #endif
 
@@ -1333,7 +1371,8 @@
 #if (PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 130000) \
     || ((PLATFORM(IOS) || PLATFORM(MACCATALYST)) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 160000) \
     || (PLATFORM(WATCHOS) && __WATCH_OS_VERSION_MIN_REQUIRED >= 90000) \
-    || (PLATFORM(APPLETV) && __TV_OS_VERSION_MIN_REQUIRED >= 160000)
+    || (PLATFORM(APPLETV) && __TV_OS_VERSION_MIN_REQUIRED >= 160000) \
+    || PLATFORM(VISION)
 #define HAVE_DYLD_DLOPEN_IMAGE_HEADER_SPI 1
 #endif
 
@@ -1349,19 +1388,21 @@
 #define HAVE_MEDIAEXPERIENCE_AVSYSTEMCONTROLLER 1
 #endif
 
-#if PLATFORM(IOS) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 160100
+#if (PLATFORM(IOS) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 160100) || PLATFORM(VISION)
 #if !defined(HAVE_UI_WINDOW_SCENE_LIVE_RESIZE)
 #define HAVE_UI_WINDOW_SCENE_LIVE_RESIZE 1
 #endif
 #endif
 
 #if (PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 130000) \
-    || ((PLATFORM(IOS) || PLATFORM(MACCATALYST)) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 160000)
-#define HAVE_CONTINUITY_CAMEARA 1
+    || ((PLATFORM(IOS) || PLATFORM(MACCATALYST)) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 160000) \
+    || PLATFORM(VISION)
+#define HAVE_CONTINUITY_CAMERA 1
 #endif
 
 #if (PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 140000) \
-    || ((PLATFORM(IOS) || PLATFORM(MACCATALYST)) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 170000)
+    || ((PLATFORM(IOS) || PLATFORM(MACCATALYST)) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 170000) \
+    || PLATFORM(VISION)
 #define HAVE_VPIO_DUCKING_LEVEL_API 1
 #endif
 
@@ -1369,27 +1410,38 @@
     && ((PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 130000) \
     || ((PLATFORM(IOS) || PLATFORM(MACCATALYST)) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 160000) \
     || (PLATFORM(WATCHOS) && __WATCH_OS_VERSION_MIN_REQUIRED >= 90000) \
-    || (PLATFORM(APPLETV) && __TV_OS_VERSION_MIN_REQUIRED >= 160000))
+    || (PLATFORM(APPLETV) && __TV_OS_VERSION_MIN_REQUIRED >= 160000) \
+    || PLATFORM(VISION))
 #define HAVE_CGSTYLE_CREATE_SHADOW2 1
 #endif
 
+#if (PLATFORM(IOS) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 170000) || PLATFORM(VISION)
+#define HAVE_NON_HOSTING_VISIBILITY_PROPAGATION_VIEW 1
+#define HAVE_ASD_INSTALL_WEB_ATTRIBUTION_SERVICE 1
+#endif
+
 #if PLATFORM(COCOA) && !PLATFORM(IOS_SIMULATOR) && __has_include(<apfs/apfs_fsctl.h>)
 #define HAVE_APFS_CACHEDELETE_PURGEABLE 1
 #endif
 
 #if (PLATFORM(IOS) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 170000) \
     || PLATFORM(WATCHOS) \
-    || PLATFORM(APPLETV)
+    || PLATFORM(APPLETV) \
+    || PLATFORM(VISION)
 #define HAVE_MEDIAPLAYBACKD 1
 #endif
 
+#if (PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 120000) || PLATFORM(IOS_FAMILY)
+#define HAVE_AVCAPTUREDEVICE_MINFOCUSLENGTH 1
+#endif
+
 #if (PLATFORM(GTK) || PLATFORM(WPE)) && defined(__has_include)
 #if __has_include(<gio/gdesktopappinfo.h>)
 #define HAVE_GDESKTOPAPPINFO 1
 #endif
 #endif
 
-#if (PLATFORM(IOS) || PLATFORM(WATCHOS) || PLATFORM(APPLETV))
+#if (PLATFORM(IOS) || PLATFORM(WATCHOS) || PLATFORM(APPLETV) || PLATFORM(VISION)) || (PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 140000)
 #define HAVE_MACH_BOOTSTRAP_EXTENSION 1
 #endif
 
@@ -1397,6 +1449,112 @@
 #define HAVE_STAGE_MANAGER_NS_WINDOW_COLLECTION_BEHAVIORS 1
 #endif
 
-#if PLATFORM(IOS)
+#if PLATFORM(IOS) || PLATFORM(VISION)
 #define HAVE_PICTUREINPICTUREPLAYERLAYERVIEW 1
 #endif
+
+#if ((PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 140000) \
+    || ((PLATFORM(IOS) || PLATFORM(MACCATALYST)) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 170000) \
+    || (PLATFORM(WATCHOS) && __WATCH_OS_VERSION_MIN_REQUIRED >= 100000) \
+    || (PLATFORM(APPLETV) && __TV_OS_VERSION_MIN_REQUIRED >= 170000) \
+    || PLATFORM(VISION))
+#define HAVE_SETUSEIOSURFACEFORTILES 1
+#endif
+
+#if ((PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 140000) \
+    || ((PLATFORM(IOS) || PLATFORM(MACCATALYST)) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 170000) \
+    || (PLATFORM(WATCHOS) && __WATCH_OS_VERSION_MIN_REQUIRED >= 100000) \
+    || (PLATFORM(APPLETV) && __TV_OS_VERSION_MIN_REQUIRED >= 170000) \
+    || PLATFORM(VISION))
+#if __has_include(<Network/proxy_config.h>)
+#define HAVE_NW_PROXY_CONFIG 1
+#endif
+#endif
+
+#if (PLATFORM(COCOA) && !PLATFORM(WATCHOS))
+#define HAVE_CFNETWORK_HOSTOVERRIDE 1
+#endif
+
+#if (PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 140000) \
+    || ((PLATFORM(IOS) && !PLATFORM(VISION) && !PLATFORM(IOS_FAMILY_SIMULATOR)) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 170000) \
+    || (PLATFORM(MACCATALYST) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 170000) \
+    || (PLATFORM(WATCHOS) && !PLATFORM(IOS_FAMILY_SIMULATOR) && __WATCH_OS_VERSION_MIN_REQUIRED >= 100000)
+#if __has_include(<LockdownMode/LockdownMode.h>)
+#define HAVE_LOCKDOWN_MODE_FRAMEWORK 1
+#endif
+#endif
+
+#if ((PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 140000) \
+    || ((PLATFORM(IOS) || PLATFORM(MACCATALYST)) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 170000)) \
+    || PLATFORM(VISION)
+// Only used inside ENABLE(DATA_DETECTION)
+#define HAVE_SECURE_ACTION_CONTEXT 1
+#endif
+
+#if ((PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 140000) \
+    || ((PLATFORM(IOS) || PLATFORM(MACCATALYST)) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 170000) \
+    || (PLATFORM(WATCHOS) && __WATCH_OS_VERSION_MIN_REQUIRED >= 100000) \
+    || (PLATFORM(APPLETV) && __TV_OS_VERSION_MIN_REQUIRED >= 170000) \
+    || PLATFORM(VISION))
+#define HAVE_CFNETWORK_SEPARATE_CREDENTIAL_STORAGE 1
+#define HAVE_STRICT_DECODABLE_CNCONTACT 1
+#define HAVE_STRICT_DECODABLE_PKCONTACT 1
+#define HAVE_STRICT_DECODABLE_NSTEXTTABLE 1
+#define HAVE_GB_18030_2022 1
+#endif
+
+#if ((PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 140000) \
+    || (PLATFORM(IOS) || PLATFORM(MACCATALYST) || PLATFORM(WATCHOS) || PLATFORM(APPLETV) || PLATFORM(VISION)))
+#define HAVE_AVAUDIOSESSION 1
+#endif
+
+#if ((PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 140000) \
+    || (PLATFORM(MACCATALYST) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 170000) \
+    || (PLATFORM(IOS) || PLATFORM(WATCHOS) || PLATFORM(VISION)))
+#define HAVE_AVAUDIOSESSION_SMARTROUTING 1
+#endif
+
+#if (PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 140000)
+#define HAVE_NSCOLOR_FILL_COLOR_HIERARCHY 1
+#endif
+
+#if (PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 140000)
+#define HAVE_APPLE_CAMERA_USER_CLIENT 1
+#endif
+
+#if ((PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 140000) \
+    || (PLATFORM(IOS) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 170000) \
+    || (PLATFORM(MACCATALYST) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 170000) \
+    || PLATFORM(VISION))
+#define HAVE_MACH_RANGE_CREATE 1
+#endif
+
+#if !defined(HAVE_INLINE_PREDICTIONS) \
+    && ((PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 140000) \
+    || ((PLATFORM(IOS) || PLATFORM(MACCATALYST)) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 170000) \
+    || PLATFORM(VISION))
+#define HAVE_INLINE_PREDICTIONS 1
+#endif
+
+#if !defined(HAVE_REDESIGNED_TEXT_CURSOR) \
+    && ((PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 140000) \
+    || ((PLATFORM(IOS) || PLATFORM(MACCATALYST)) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 170000) \
+    || PLATFORM(VISION))
+#define HAVE_REDESIGNED_TEXT_CURSOR 1
+#endif
+
+#if !defined(HAVE_WEB_PRIVACY_FRAMEWORK) && __has_include(<WebPrivacy/WebPrivacy.h>)
+#define HAVE_WEB_PRIVACY_FRAMEWORK 1
+#endif
+
+#if (PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 140000) \
+    || (PLATFORM(MACCATALYST) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 170000) \
+    || (PLATFORM(IOS) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 170000) \
+    || (PLATFORM(APPLETV) && __TV_OS_VERSION_MAX_ALLOWED >= 170000) \
+    || (PLATFORM(WATCHOS) && __WATCH_OS_VERSION_MIN_REQUIRED >= 100000)
+#define HAVE_JPEGXL 1
+#endif
+
+#if PLATFORM(COCOA)
+#define HAVE_HEIC 1
+#endif
