@@ -113,7 +113,7 @@ bool URL::hasSpecialScheme() const
 {
     // https://url.spec.whatwg.org/#special-scheme
     return protocolIs("ftp"_s)
-        || protocolIs("file"_s)
+        || protocolIsFile()
         || protocolIs("http"_s)
         || protocolIs("https"_s)
         || protocolIs("ws"_s)
@@ -123,9 +123,9 @@ bool URL::hasSpecialScheme() const
 bool URL::hasLocalScheme() const
 {
     // https://fetch.spec.whatwg.org/#local-scheme
-    return protocolIs("about"_s)
-        || protocolIs("blob"_s)
-        || protocolIs("data"_s);
+    return protocolIsAbout()
+        || protocolIsBlob()
+        || protocolIsData();
 }
 
 unsigned URL::pathStart() const
@@ -191,7 +191,7 @@ static std::optional<LChar> decodeEscapeSequence(StringView input, unsigned inde
 
 static String decodeEscapeSequencesFromParsedURL(StringView input)
 {
-    ASSERT(input.isAllASCII());
+    ASSERT(input.containsOnlyASCII());
 
     auto length = input.length();
     if (length < 3 || !input.contains('%'))
@@ -273,12 +273,13 @@ URL URL::truncatedForUseAsBase() const
 
 String URL::fileSystemPath() const
 {
-    if (!isLocalFile())
+    if (!protocolIsFile())
         return { };
 
     auto result = decodeEscapeSequencesFromParsedURL(path());
 #if PLATFORM(WIN)
-    result = FileSystem::fileSystemRepresentation(result);
+    if (result.startsWith('/'))
+        result = result.substring(1);
 #endif
     return result;
 }
@@ -361,11 +362,6 @@ bool URL::protocolIsJavaScript() const
     return WTF::protocolIsJavaScript(string());
 }
 
-bool URL::protocolIsInFTPFamily() const
-{
-    return WTF::protocolIsInFTPFamily(string());
-}
-
 bool URL::protocolIs(StringView protocol) const
 {
     assertProtocolIsGood(protocol);
@@ -416,7 +412,7 @@ bool URL::setProtocol(StringView newProtocol)
     if ((m_passwordEnd != m_userStart || port()) && *newProtocolCanonicalized == "file"_s)
         return true;
 
-    if (isLocalFile() && host().isEmpty())
+    if (protocolIsFile() && host().isEmpty())
         return true;
 
     parse(makeString(*newProtocolCanonicalized, StringView(m_string).substring(m_schemeEnd)));
@@ -430,7 +426,7 @@ static bool appendEncodedHostname(Vector<UChar, 512>& buffer, StringView string)
 {
     // hostnameBuffer needs to be big enough to hold an IDN-encoded name.
     // For host names bigger than this, we won't do IDN encoding, which is almost certainly OK.
-    if (string.length() > URLParser::hostnameBufferLength || string.isAllASCII()) {
+    if (string.length() > URLParser::hostnameBufferLength || string.containsOnlyASCII()) {
         append(buffer, string);
         return true;
     }
@@ -492,7 +488,7 @@ void URL::setHost(StringView newHost)
     bool slashSlashNeeded = m_userStart == m_schemeEnd + 1U;
     parse(makeString(
         StringView(m_string).left(hostStart()),
-        slashSlashNeeded ? "//" : "",
+        slashSlashNeeded ? "//"_s : ""_s,
         hasSpecialScheme() ? StringView(encodedHostName.data(), encodedHostName.size()) : newHost,
         StringView(m_string).substring(m_hostEnd)
     ));
@@ -545,9 +541,9 @@ void URL::setHostAndPort(StringView hostAndPort)
     bool slashSlashNeeded = m_userStart == m_schemeEnd + 1U;
     parse(makeString(
         StringView(m_string).left(hostStart()),
-        slashSlashNeeded ? "//" : "",
+        slashSlashNeeded ? "//"_s : ""_s,
         hasSpecialScheme() ? StringView(encodedHostName.data(), encodedHostName.size()) : hostName,
-        portString.isEmpty() ? "" : ":",
+        portString.isEmpty() ? ""_s : ":"_s,
         portString,
         StringView(m_string).substring(pathStart())
     ));
@@ -557,7 +553,7 @@ template<typename StringType>
 static String percentEncodeCharacters(const StringType& input, bool(*shouldEncode)(UChar))
 {
     auto encode = [shouldEncode] (const StringType& input) {
-        auto result = input.tryGetUTF8([&](Span<const char> span) -> String {
+        auto result = input.tryGetUTF8([&](std::span<const char> span) -> String {
             StringBuilder builder;
             for (unsigned j = 0; j < span.size(); j++) {
                 auto c = span[j];
@@ -611,9 +607,9 @@ void URL::setUser(StringView newUser)
         bool needSeparator = end == m_hostEnd || (end == m_passwordEnd && m_string[end] != '@');
         parse(makeString(
             StringView(m_string).left(m_userStart),
-            slashSlashNeeded ? "//" : "",
+            slashSlashNeeded ? "//"_s : ""_s,
             percentEncodeCharacters(newUser, URLParser::isInUserInfoEncodeSet),
-            needSeparator ? "@" : "",
+            needSeparator ? "@"_s : ""_s,
             StringView(m_string).substring(end)
         ));
     } else {
@@ -633,7 +629,7 @@ void URL::setPassword(StringView newPassword)
         bool needLeadingSlashes = m_userEnd == m_schemeEnd + 1U;
         parse(makeString(
             StringView(m_string).left(m_userEnd),
-            needLeadingSlashes ? "//:" : ":",
+            needLeadingSlashes ? "//:"_s : ":"_s,
             percentEncodeCharacters(newPassword, URLParser::isInUserInfoEncodeSet),
             '@',
             StringView(m_string).substring(credentialsEnd())
@@ -687,7 +683,7 @@ void URL::setQuery(StringView newQuery)
 
     parse(makeString(
         StringView(m_string).left(m_pathEnd),
-        (!newQuery.startsWith('?') && !newQuery.isNull()) ? "?" : "",
+        (!newQuery.startsWith('?') && !newQuery.isNull()) ? "?"_s : ""_s,
         newQuery,
         StringView(m_string).substring(m_queryEnd)
     ));
@@ -708,7 +704,7 @@ void URL::setPath(StringView path)
 
     parse(makeString(
         StringView(m_string).left(pathStart()),
-        path.startsWith('/') || (path.startsWith('\\') && (hasSpecialScheme() || protocolIs("file"_s))) || (!hasSpecialScheme() && path.isEmpty() && m_schemeEnd + 1U < pathStart()) ? ""_s : "/"_s,
+        path.startsWith('/') || (path.startsWith('\\') && (hasSpecialScheme() || protocolIsFile())) || (!hasSpecialScheme() && path.isEmpty() && m_schemeEnd + 1U < pathStart()) ? ""_s : "/"_s,
         !hasSpecialScheme() && host().isEmpty() && path.startsWith("//"_s) && path.length() > 2 ? "/."_s : ""_s,
         escapePathWithoutCopying(path),
         StringView(m_string).substring(m_pathEnd)
@@ -890,14 +886,17 @@ String URL::strippedForUseAsReferrerWithExplicitPort() const
     return makeString(StringView(m_string).left(m_hostEnd), ':', static_cast<unsigned>(*port), StringView(m_string).substring(end, m_queryEnd - end));
 }
 
-
-bool URL::isLocalFile() const
+String URL::strippedForUseAsReport() const
 {
-    // Including feed here might be a bad idea since drag and drop uses this check
-    // and including feed would allow feeds to potentially let someone's blog
-    // read the contents of the clipboard on a drag, even without a drop.
-    // Likewise with using the FrameLoader::shouldTreatURLAsLocal() function.
-    return protocolIs("file"_s);
+    if (!m_isValid)
+        return m_string;
+
+    unsigned end = credentialsEnd();
+
+    if (m_userStart == end && m_pathEnd == m_string.length())
+        return m_string;
+
+    return makeString(StringView(m_string).left(m_userStart), StringView(m_string).substring(end, m_pathEnd - end));
 }
 
 bool protocolIsJavaScript(StringView string)
@@ -905,17 +904,6 @@ bool protocolIsJavaScript(StringView string)
     return protocolIsInternal(string, "javascript"_s);
 }
 
-bool protocolIsInFTPFamily(StringView url)
-{
-    auto length = url.length();
-    // Do the comparison without making a new string object.
-    return length >= 4
-        && isASCIIAlphaCaselessEqual(url[0], 'f')
-        && isASCIIAlphaCaselessEqual(url[1], 't')
-        && isASCIIAlphaCaselessEqual(url[2], 'p')
-        && (url[3] == ':' || (isASCIIAlphaCaselessEqual(url[3], 's') && length >= 5 && url[4] == ':'));
-}
-
 bool protocolIsInHTTPFamily(StringView url)
 {
     auto length = url.length();
@@ -951,11 +939,6 @@ const URL& aboutSrcDocURL()
     return staticSrcDocURL;
 }
 
-bool URL::protocolIsAbout() const
-{
-    return protocolIs("about"_s);
-}
-
 bool portAllowed(const URL& url)
 {
     std::optional<uint16_t> port = url.port();
@@ -1061,7 +1044,7 @@ bool portAllowed(const URL& url)
         return true;
 
     // Allow any port number in a file URL, since the port number is ignored.
-    if (url.protocolIs("file"_s))
+    if (url.protocolIsFile())
         return true;
 
     return false;
@@ -1092,7 +1075,7 @@ String URL::stringCenterEllipsizedToLength(unsigned length) const
     if (m_string.length() <= length)
         return m_string;
 
-    return makeString(StringView(m_string).left(length / 2 - 1), "...", StringView(m_string).right(length / 2 - 2));
+    return makeString(StringView(m_string).left(length / 2 - 1), "..."_s, StringView(m_string).right(length / 2 - 2));
 }
 
 URL URL::fakeURLWithRelativePart(StringView relativePart)
@@ -1104,7 +1087,7 @@ URL URL::fileURLWithFileSystemPath(StringView path)
 {
     return URL(makeString(
         "file://"_s,
-        path.startsWith('/') ? "" : "/",
+        path.startsWith('/') ? ""_s : "/"_s,
         escapePathWithoutCopying(path)
     ));
 }
