@@ -68,8 +68,8 @@ public:
     StringView(const UChar*, unsigned length);
     StringView(const char*, unsigned length);
     StringView(ASCIILiteral);
-    ALWAYS_INLINE StringView(Span<const LChar> characters) : StringView(characters.data(), characters.size()) { }
-    ALWAYS_INLINE StringView(Span<const UChar> characters) : StringView(characters.data(), characters.size()) { }
+    ALWAYS_INLINE StringView(std::span<const LChar> characters) : StringView(characters.data(), characters.size()) { }
+    ALWAYS_INLINE StringView(std::span<const UChar> characters) : StringView(characters.data(), characters.size()) { }
 
     ALWAYS_INLINE static StringView fromLatin1(const char* characters) { return StringView { characters }; }
 
@@ -96,15 +96,15 @@ public:
     bool is8Bit() const;
     const LChar* characters8() const;
     const UChar* characters16() const;
-    Span<const LChar> span8() const { return { characters8(), length() }; }
-    Span<const UChar> span16() const { return { characters16(), length() }; }
+    std::span<const LChar> span8() const { return { characters8(), length() }; }
+    std::span<const UChar> span16() const { return { characters16(), length() }; }
 
     unsigned hash() const;
 
     // Return characters8() or characters16() depending on CharacterType.
     template<typename CharacterType> const CharacterType* characters() const;
 
-    bool isAllASCII() const;
+    bool containsOnlyASCII() const;
 
     String toString() const;
     String toStringWithoutCopying() const;
@@ -127,7 +127,7 @@ public:
     WTF_EXPORT_PRIVATE CString utf8(ConversionMode = LenientConversion) const;
 
     template<typename Func>
-    Expected<std::invoke_result_t<Func, Span<const char>>, UTF8ConversionError> tryGetUTF8(const Func&, ConversionMode = LenientConversion) const;
+    Expected<std::invoke_result_t<Func, std::span<const char>>, UTF8ConversionError> tryGetUTF8(const Func&, ConversionMode = LenientConversion) const;
 
     class UpconvertedCharacters;
     UpconvertedCharacters upconvertedCharacters() const;
@@ -145,14 +145,15 @@ public:
     StringView right(unsigned length) const { return substring(this->length() - length, length); }
 
     template<typename MatchedCharacterPredicate>
-    StringView stripLeadingAndTrailingMatchedCharacters(const MatchedCharacterPredicate&) const;
-    WTF_EXPORT_PRIVATE StringView stripWhiteSpace() const;
+    StringView trim(const MatchedCharacterPredicate&) const;
 
     class SplitResult;
     SplitResult split(UChar) const;
     SplitResult splitAllowingEmptyEntries(UChar) const;
 
     size_t find(UChar, unsigned start = 0) const;
+    size_t find(LChar, unsigned start = 0) const;
+    ALWAYS_INLINE size_t find(char c, unsigned start = 0) const { return find(static_cast<LChar>(c), start); }
     template<typename CodeUnitMatchFunction, std::enable_if_t<std::is_invocable_r_v<bool, CodeUnitMatchFunction, UChar>>* = nullptr>
     size_t find(CodeUnitMatchFunction&&, unsigned start = 0) const;
     ALWAYS_INLINE size_t find(ASCIILiteral literal, unsigned start = 0) const { return find(literal.characters8(), literal.length(), start); }
@@ -178,7 +179,7 @@ public:
     WTF_EXPORT_PRIVATE bool containsIgnoringASCIICase(StringView) const;
     WTF_EXPORT_PRIVATE bool containsIgnoringASCIICase(StringView, unsigned start) const;
 
-    template<bool isSpecialCharacter(UChar)> bool isAllSpecialCharacters() const;
+    template<bool isSpecialCharacter(UChar)> bool containsOnly() const;
 
     WTF_EXPORT_PRIVATE bool startsWith(UChar) const;
     WTF_EXPORT_PRIVATE bool startsWith(StringView) const;
@@ -210,7 +211,7 @@ private:
     WTF_EXPORT_PRIVATE size_t reverseFind(const LChar* match, unsigned matchLength, unsigned start) const;
 
     template<typename CharacterType, typename MatchedCharacterPredicate>
-    StringView stripLeadingAndTrailingMatchedCharacters(const CharacterType*, const MatchedCharacterPredicate&) const;
+    StringView trim(const CharacterType*, const MatchedCharacterPredicate&) const;
 
     WTF_EXPORT_PRIVATE bool underlyingStringIsValidImpl() const;
     WTF_EXPORT_PRIVATE void setUnderlyingStringImpl(const StringImpl*);
@@ -256,10 +257,6 @@ inline bool operator==(StringView a, StringView b) { return equal(a, b); }
 inline bool operator==(StringView a, ASCIILiteral b) { return equal(a, b); }
 inline bool operator==(ASCIILiteral a, StringView b) { return equal(b, a); }
 
-inline bool operator!=(StringView a, StringView b) { return !equal(a, b); }
-inline bool operator!=(StringView a, ASCIILiteral b) { return !equal(a, b); }
-inline bool operator!=(ASCIILiteral a, StringView b) { return !equal(b, a); }
-
 struct StringViewWithUnderlyingString;
 
 // This returns a StringView of the normalized result, and a String that is either
@@ -279,14 +276,28 @@ namespace WTF {
 
 struct StringViewWithUnderlyingString {
     WTF_MAKE_STRUCT_FAST_ALLOCATED;
-    StringView view;
+    StringViewWithUnderlyingString(StringView passedView, String passedUnderlyingString)
+        : underlyingString(WTFMove(passedUnderlyingString))
+        , view(WTFMove(passedView))
+    { }
+
     String underlyingString;
+    StringView view;
+
+    String toString() const;
 };
 
 inline StringView::StringView()
 {
 }
 
+inline String StringViewWithUnderlyingString::toString() const
+{
+    if (LIKELY(view.length() == underlyingString.length()))
+        return underlyingString;
+    return view.toString();
+}
+
 #if CHECK_STRINGVIEW_LIFETIME
 
 inline StringView::~StringView()
@@ -470,7 +481,7 @@ template<> ALWAYS_INLINE const UChar* StringView::characters<UChar>() const
     return characters16();
 }
 
-inline bool StringView::isAllASCII() const
+inline bool StringView::containsOnlyASCII() const
 {
     if (is8Bit())
         return charactersAreAllASCII(characters8(), length());
@@ -564,11 +575,11 @@ inline bool StringView::contains(CodeUnitMatchFunction&& function) const
     return find(std::forward<CodeUnitMatchFunction>(function)) != notFound;
 }
 
-template<bool isSpecialCharacter(UChar)> inline bool StringView::isAllSpecialCharacters() const
+template<bool isSpecialCharacter(UChar)> inline bool StringView::containsOnly() const
 {
     if (is8Bit())
-        return WTF::isAllSpecialCharacters<isSpecialCharacter>(characters8(), length());
-    return WTF::isAllSpecialCharacters<isSpecialCharacter>(characters16(), length());
+        return WTF::containsOnly<isSpecialCharacter>(characters8(), length());
+    return WTF::containsOnly<isSpecialCharacter>(characters16(), length());
 }
 
 template<typename CharacterType> inline void StringView::getCharacters8(CharacterType* destination) const
@@ -651,6 +662,13 @@ inline size_t StringView::find(UChar character, unsigned start) const
     return WTF::find(characters16(), m_length, character, start);
 }
 
+inline size_t StringView::find(LChar character, unsigned start) const
+{
+    if (is8Bit())
+        return WTF::find(characters8(), m_length, character, start);
+    return WTF::find(characters16(), m_length, character, start);
+}
+
 template<typename CodeUnitMatchFunction, std::enable_if_t<std::is_invocable_r_v<bool, CodeUnitMatchFunction, UChar>>*>
 inline size_t StringView::find(CodeUnitMatchFunction&& matchFunction, unsigned start) const
 {
@@ -815,7 +833,6 @@ public:
     WTF_EXPORT_PRIVATE Iterator& operator++();
 
     bool operator==(const Iterator&) const;
-    bool operator!=(const Iterator&) const;
 
 private:
     enum PositionTag { AtEnd };
@@ -848,7 +865,6 @@ public:
     WTF_EXPORT_PRIVATE Iterator& operator++();
 
     WTF_EXPORT_PRIVATE bool operator==(const Iterator&) const;
-    WTF_EXPORT_PRIVATE bool operator!=(const Iterator&) const;
 
 private:
     class Impl;
@@ -865,7 +881,6 @@ public:
     Iterator& operator++();
 
     bool operator==(const Iterator&) const;
-    bool operator!=(const Iterator&) const;
 
 private:
     const void* m_current;
@@ -885,7 +900,6 @@ public:
     Iterator& operator++();
 
     bool operator==(const Iterator&) const;
-    bool operator!=(const Iterator&) const;
 
 private:
     StringView m_stringView;
@@ -985,11 +999,6 @@ inline bool StringView::CodePoints::Iterator::operator==(const Iterator& other)
     return m_current == other.m_current;
 }
 
-inline bool StringView::CodePoints::Iterator::operator!=(const Iterator& other) const
-{
-    return !(*this == other);
-}
-
 inline auto StringView::CodePoints::begin() const -> Iterator
 {
     return Iterator(m_stringView, 0);
@@ -1029,11 +1038,6 @@ inline bool StringView::CodeUnits::Iterator::operator==(const Iterator& other) c
     return m_index == other.m_index;
 }
 
-inline bool StringView::CodeUnits::Iterator::operator!=(const Iterator& other) const
-{
-    return !(*this == other);
-}
-
 inline auto StringView::CodeUnits::begin() const -> Iterator
 {
     return Iterator(m_stringView, 0);
@@ -1098,13 +1102,8 @@ inline bool StringView::SplitResult::Iterator::operator==(const Iterator& other)
     return m_position == other.m_position && m_isDone == other.m_isDone;
 }
 
-inline bool StringView::SplitResult::Iterator::operator!=(const Iterator& other) const
-{
-    return !(*this == other);
-}
-
 template<typename CharacterType, typename MatchedCharacterPredicate>
-inline StringView StringView::stripLeadingAndTrailingMatchedCharacters(const CharacterType* characters, const MatchedCharacterPredicate& predicate) const
+inline StringView StringView::trim(const CharacterType* characters, const MatchedCharacterPredicate& predicate) const
 {
     if (!m_length)
         return *this;
@@ -1130,11 +1129,11 @@ inline StringView StringView::stripLeadingAndTrailingMatchedCharacters(const Cha
 }
 
 template<typename MatchedCharacterPredicate>
-StringView StringView::stripLeadingAndTrailingMatchedCharacters(const MatchedCharacterPredicate& predicate) const
+StringView StringView::trim(const MatchedCharacterPredicate& predicate) const
 {
     if (is8Bit())
-        return stripLeadingAndTrailingMatchedCharacters<LChar>(characters8(), predicate);
-    return stripLeadingAndTrailingMatchedCharacters<UChar>(characters16(), predicate);
+        return trim<LChar>(characters8(), predicate);
+    return trim<UChar>(characters16(), predicate);
 }
 
 inline bool equalLettersIgnoringASCIICase(StringView string, ASCIILiteral literal)
@@ -1442,13 +1441,22 @@ inline bool AtomString::endsWithIgnoringASCIICase(StringView string) const
 }
 
 template<typename Func>
-inline Expected<std::invoke_result_t<Func, Span<const char>>, UTF8ConversionError> StringView::tryGetUTF8(const Func& function, ConversionMode mode) const
+inline Expected<std::invoke_result_t<Func, std::span<const char>>, UTF8ConversionError> StringView::tryGetUTF8(const Func& function, ConversionMode mode) const
 {
     if (is8Bit())
         return StringImpl::tryGetUTF8ForCharacters(function, characters8(), length());
     return StringImpl::tryGetUTF8ForCharacters(function, characters16(), length(), mode);
 }
 
+template<> struct VectorTraits<StringView> : VectorTraitsBase<false, void> {
+    static constexpr bool canMoveWithMemcpy = true;
+    static constexpr bool canCopyWithMemcpy = true;
+};
+
+template<> struct VectorTraits<StringViewWithUnderlyingString> : VectorTraitsBase<false, void> {
+    static constexpr bool canMoveWithMemcpy = true;
+};
+
 } // namespace WTF
 
 using WTF::append;
