@@ -39,6 +39,7 @@
 #include "GPUComputePipelineDescriptor.h"
 #include "GPUExternalTexture.h"
 #include "GPUExternalTextureDescriptor.h"
+#include "GPUPipelineError.h"
 #include "GPUPipelineLayout.h"
 #include "GPUPipelineLayoutDescriptor.h"
 #include "GPUPresentationContext.h"
@@ -58,7 +59,10 @@
 #include "GPUTextureDescriptor.h"
 #include "JSDOMPromiseDeferred.h"
 #include "JSGPUComputePipeline.h"
+#include "JSGPUDeviceLostInfo.h"
+#include "JSGPUInternalError.h"
 #include "JSGPUOutOfMemoryError.h"
+#include "JSGPUPipelineError.h"
 #include "JSGPURenderPipeline.h"
 #include "JSGPUValidationError.h"
 #include <wtf/IsoMallocInlines.h>
@@ -108,19 +112,29 @@ void GPUDevice::destroy()
     m_backing->destroy();
 }
 
+GPUDevice::LostPromise& GPUDevice::lost()
+{
+    if (m_waitingForDeviceLostPromise)
+        return m_lostPromise;
+
+    m_waitingForDeviceLostPromise = true;
+
+    // FIXME: https://bugs.webkit.org/show_bug.cgi?id=257656 Implement this.
+
+    return m_lostPromise;
+}
+
 Ref<GPUBuffer> GPUDevice::createBuffer(const GPUBufferDescriptor& bufferDescriptor)
 {
-    return GPUBuffer::create(m_backing->createBuffer(bufferDescriptor.convertToBacking()));
+    auto bufferSize = bufferDescriptor.size;
+    auto usage = bufferDescriptor.usage;
+    auto mappedAtCreation = bufferDescriptor.mappedAtCreation;
+    return GPUBuffer::create(m_backing->createBuffer(bufferDescriptor.convertToBacking()), bufferSize, usage, mappedAtCreation);
 }
 
 Ref<GPUTexture> GPUDevice::createTexture(const GPUTextureDescriptor& textureDescriptor)
 {
-    return GPUTexture::create(m_backing->createTexture(textureDescriptor.convertToBacking()));
-}
-
-Ref<GPUTexture> GPUDevice::createSurfaceTexture(const GPUTextureDescriptor& textureDescriptor, const GPUPresentationContext& presentationContext)
-{
-    return GPUTexture::create(m_backing->createSurfaceTexture(textureDescriptor.convertToBacking(), presentationContext.backing()));
+    return GPUTexture::create(m_backing->createTexture(textureDescriptor.convertToBacking()), textureDescriptor.format);
 }
 
 static PAL::WebGPU::SamplerDescriptor convertToBacking(const std::optional<GPUSamplerDescriptor>& samplerDescriptor)
@@ -186,15 +200,21 @@ Ref<GPURenderPipeline> GPUDevice::createRenderPipeline(const GPURenderPipelineDe
 
 void GPUDevice::createComputePipelineAsync(const GPUComputePipelineDescriptor& computePipelineDescriptor, CreateComputePipelineAsyncPromise&& promise)
 {
-    m_backing->createComputePipelineAsync(computePipelineDescriptor.convertToBacking(m_autoPipelineLayout), [promise = WTFMove(promise)] (Ref<PAL::WebGPU::ComputePipeline>&& computePipeline) mutable {
-        promise.resolve(GPUComputePipeline::create(WTFMove(computePipeline)));
+    m_backing->createComputePipelineAsync(computePipelineDescriptor.convertToBacking(m_autoPipelineLayout), [promise = WTFMove(promise)](RefPtr<PAL::WebGPU::ComputePipeline>&& computePipeline) mutable {
+        if (computePipeline.get())
+            promise.resolve(GPUComputePipeline::create(computePipeline.releaseNonNull()));
+        else
+            promise.rejectType<IDLInterface<GPUPipelineError>>(GPUPipelineError::create(""_s, { GPUPipelineErrorReason::Validation }));
     });
 }
 
 void GPUDevice::createRenderPipelineAsync(const GPURenderPipelineDescriptor& renderPipelineDescriptor, CreateRenderPipelineAsyncPromise&& promise)
 {
-    m_backing->createRenderPipelineAsync(renderPipelineDescriptor.convertToBacking(m_autoPipelineLayout), [promise = WTFMove(promise)] (Ref<PAL::WebGPU::RenderPipeline>&& renderPipeline) mutable {
-        promise.resolve(GPURenderPipeline::create(WTFMove(renderPipeline)));
+    m_backing->createRenderPipelineAsync(renderPipelineDescriptor.convertToBacking(m_autoPipelineLayout), [promise = WTFMove(promise)](RefPtr<PAL::WebGPU::RenderPipeline>&& renderPipeline) mutable {
+        if (renderPipeline.get())
+            promise.resolve(GPURenderPipeline::create(renderPipeline.releaseNonNull()));
+        else
+            promise.rejectType<IDLInterface<GPUPipelineError>>(GPUPipelineError::create(""_s, { GPUPipelineErrorReason::Validation }));
     });
 }
 
@@ -228,17 +248,20 @@ void GPUDevice::pushErrorScope(GPUErrorFilter errorFilter)
 
 void GPUDevice::popErrorScope(ErrorScopePromise&& errorScopePromise)
 {
-    m_backing->popErrorScope([promise = WTFMove(errorScopePromise)] (std::optional<PAL::WebGPU::Error>&& error) mutable {
+    m_backing->popErrorScope([promise = WTFMove(errorScopePromise)](std::optional<PAL::WebGPU::Error>&& error) mutable {
         if (!error) {
             promise.resolve(std::nullopt);
             return;
         }
-        WTF::switchOn(WTFMove(*error), [&] (Ref<PAL::WebGPU::OutOfMemoryError>&& outOfMemoryError) {
+        WTF::switchOn(WTFMove(*error), [&promise](Ref<PAL::WebGPU::OutOfMemoryError>&& outOfMemoryError) {
             GPUError error = RefPtr<GPUOutOfMemoryError>(GPUOutOfMemoryError::create(WTFMove(outOfMemoryError)));
             promise.resolve(error);
-        }, [&] (Ref<PAL::WebGPU::ValidationError>&& validationError) {
+        }, [&promise](Ref<PAL::WebGPU::ValidationError>&& validationError) {
             GPUError error = RefPtr<GPUValidationError>(GPUValidationError::create(WTFMove(validationError)));
             promise.resolve(error);
+        }, [&promise](Ref<PAL::WebGPU::InternalError>&& internalError) {
+            GPUError error = RefPtr<GPUInternalError>(GPUInternalError::create(WTFMove(internalError)));
+            promise.resolve(error);
         });
     });
 }
