@@ -29,6 +29,10 @@
 #if ENABLE(APPLE_PAY)
 
 #include "ApplePayCancelEvent.h"
+#include "ApplePayCouponCodeChangedEvent.h"
+#include "ApplePayCouponCodeUpdate.h"
+#include "ApplePayErrorCode.h"
+#include "ApplePayErrorContactField.h"
 #include "ApplePayLineItem.h"
 #include "ApplePayPaymentAuthorizationResult.h"
 #include "ApplePayPaymentAuthorizedEvent.h"
@@ -41,22 +45,21 @@
 #include "ApplePayShippingMethodSelectedEvent.h"
 #include "ApplePayShippingMethodUpdate.h"
 #include "ApplePayValidateMerchantEvent.h"
-#include "DOMWindow.h"
 #include "Document.h"
 #include "DocumentLoader.h"
 #include "EventNames.h"
-#include "Frame.h"
 #include "JSDOMPromiseDeferred.h"
 #include "LinkIconCollector.h"
 #include "LinkIconType.h"
+#include "LocalDOMWindow.h"
+#include "LocalFrame.h"
 #include "Page.h"
 #include "PageConsoleClient.h"
-#include "PaymentAuthorizationStatus.h"
 #include "PaymentContact.h"
 #include "PaymentCoordinator.h"
 #include "PaymentMerchantSession.h"
 #include "PaymentMethod.h"
-#include "PaymentMethodUpdate.h"
+#include "PaymentRequestUtilities.h"
 #include "PaymentRequestValidator.h"
 #include "SecurityOrigin.h"
 #include "Settings.h"
@@ -64,122 +67,42 @@
 #include <wtf/IsoMallocInlines.h>
 #include <wtf/RunLoop.h>
 
-namespace WebCore {
-
-static void finishConverting(PaymentMethodUpdate& convertedUpdate, ApplePayPaymentMethodUpdate&& update)
-{
-#if ENABLE(APPLE_PAY_INSTALLMENTS)
-    convertedUpdate.setInstallmentGroupIdentifier(update.installmentGroupIdentifier);
-#else
-    UNUSED_PARAM(convertedUpdate);
-    UNUSED_PARAM(update);
+#if ENABLE(APPLE_PAY_DEFERRED_PAYMENTS)
+#include <wtf/DateMath.h>
 #endif
-}
+
+namespace WebCore {
 
 WTF_MAKE_ISO_ALLOCATED_IMPL(ApplePaySession);
 
-// The amount follows the regular expression -?[0-9]+(\.[0-9][0-9])?.
-static bool validateAmount(const String& amountString)
+static ExceptionOr<ApplePayLineItem> convertAndValidateTotal(ApplePayLineItem&& lineItem)
 {
-    enum class State {
-        Start,
-        Sign,
-        Digit,
-        Dot,
-        DotDigit,
-        End,
-    };
-
-    State state = State::Start;
-
-    for (unsigned i = 0; i < amountString.length(); ++i) {
-        UChar c = amountString[i];
-
-        switch (state) {
-        case State::Start:
-            if (c == '-') {
-                state = State::Sign;
-                break;
-            }
-
-            if (!isASCIIDigit(c))
-                return false;
-            state = State::Digit;
-            break;
-
-        case State::Sign:
-            if (!isASCIIDigit(c))
-                return false;
-            state = State::Digit;
-            break;
-
-        case State::Digit:
-            if (c == '.') {
-                state = State::Dot;
-                break;
-            }
-
-            if (!isASCIIDigit(c))
-                return false;
-            break;
-
-        case State::Dot:
-            if (!isASCIIDigit(c))
-                return false;
-
-            state = State::DotDigit;
-            break;
-
-        case State::DotDigit:
-            if (!isASCIIDigit(c))
-                return false;
-
-            state = State::End;
-            break;
-
-        case State::End:
-            return false;
-        }
-    }
-
-    return state == State::Digit || state == State::DotDigit || state == State::End;
-}
-
-static ExceptionOr<ApplePaySessionPaymentRequest::LineItem> convertAndValidateTotal(ApplePayLineItem&& lineItem)
-{
-    if (!validateAmount(lineItem.amount))
+    if (!isValidDecimalMonetaryValue(lineItem.amount))
         return Exception { TypeError, makeString("\"" + lineItem.amount, "\" is not a valid amount.") };
 
-    ApplePaySessionPaymentRequest::LineItem total { lineItem.type, lineItem.amount, lineItem.label };
-
-    auto validatedTotal = PaymentRequestValidator::validateTotal(total);
+    auto validatedTotal = PaymentRequestValidator::validateTotal(lineItem);
     if (validatedTotal.hasException())
         return validatedTotal.releaseException();
 
-    return WTFMove(total);
+    return WTFMove(lineItem);
 }
 
-static ExceptionOr<ApplePaySessionPaymentRequest::LineItem> convertAndValidate(ApplePayLineItem&& lineItem)
+static ExceptionOr<ApplePayLineItem> convertAndValidate(ApplePayLineItem&& lineItem)
 {
-    ApplePaySessionPaymentRequest::LineItem result;
-
-    // It is OK for pending types to not have an amount.
-    if (lineItem.type != ApplePaySessionPaymentRequest::LineItem::Type::Pending) {
-        if (!validateAmount(lineItem.amount))
+    if (lineItem.type == ApplePayLineItem::Type::Pending) {
+        // It is OK for pending types to not have an amount.
+        lineItem.amount = nullString();
+    } else {
+        if (!isValidDecimalMonetaryValue(lineItem.amount))
             return Exception { TypeError, makeString("\"" + lineItem.amount, "\" is not a valid amount.") };
-
-        result.amount = lineItem.amount;
     }
 
-    result.type = lineItem.type;
-    result.label = lineItem.label;
-
-    return WTFMove(result);
+    return WTFMove(lineItem);
 }
 
-static ExceptionOr<Vector<ApplePaySessionPaymentRequest::LineItem>> convertAndValidate(Optional<Vector<ApplePayLineItem>>&& lineItems)
+static ExceptionOr<Vector<ApplePayLineItem>> convertAndValidate(std::optional<Vector<ApplePayLineItem>>&& lineItems)
 {
-    Vector<ApplePaySessionPaymentRequest::LineItem> result;
+    Vector<ApplePayLineItem> result;
     if (!lineItems)
         return WTFMove(result);
 
@@ -195,25 +118,19 @@ static ExceptionOr<Vector<ApplePaySessionPaymentRequest::LineItem>> convertAndVa
     return WTFMove(result);
 }
 
-static ExceptionOr<ApplePaySessionPaymentRequest::ShippingMethod> convertAndValidate(ApplePayShippingMethod&& shippingMethod)
+static ExceptionOr<ApplePayShippingMethod> convertAndValidate(ApplePayShippingMethod&& shippingMethod)
 {
-    if (!validateAmount(shippingMethod.amount))
+    if (!isValidDecimalMonetaryValue(shippingMethod.amount))
         return Exception { TypeError, makeString("\"" + shippingMethod.amount, "\" is not a valid amount.") };
 
-    ApplePaySessionPaymentRequest::ShippingMethod result;
-    result.amount = shippingMethod.amount;
-    result.label = shippingMethod.label;
-    result.detail = shippingMethod.detail;
-    result.identifier = shippingMethod.identifier;
-
-    return WTFMove(result);
+    return WTFMove(shippingMethod);
 }
 
-static ExceptionOr<Vector<ApplePaySessionPaymentRequest::ShippingMethod>> convertAndValidate(Vector<ApplePayShippingMethod>&& shippingMethods)
+static ExceptionOr<Vector<ApplePayShippingMethod>> convertAndValidate(Vector<ApplePayShippingMethod>&& shippingMethods)
 {
-    Vector<ApplePaySessionPaymentRequest::ShippingMethod> result;
+    Vector<ApplePayShippingMethod> result;
     result.reserveInitialCapacity(shippingMethods.size());
-    
+
     for (auto& shippingMethod : shippingMethods) {
         auto convertedShippingMethod = convertAndValidate(WTFMove(shippingMethod));
         if (convertedShippingMethod.hasException())
@@ -224,6 +141,132 @@ static ExceptionOr<Vector<ApplePaySessionPaymentRequest::ShippingMethod>> conver
     return WTFMove(result);
 }
 
+#if ENABLE(APPLE_PAY_RECURRING_PAYMENTS)
+
+static ExceptionOr<ApplePayRecurringPaymentRequest> convertAndValidate(ApplePayRecurringPaymentRequest&& recurringPaymentRequest)
+{
+    auto& regularBilling = recurringPaymentRequest.regularBilling;
+    if (regularBilling.paymentTiming != ApplePayPaymentTiming::Recurring)
+        return Exception(TypeError, "'regularBilling' must be a 'recurring' line item."_s);
+    if (!regularBilling.label)
+        return Exception(TypeError, "Missing label for 'regularBilling'."_s);
+    if (!isValidDecimalMonetaryValue(regularBilling.amount) && regularBilling.type != ApplePayLineItem::Type::Pending)
+        return Exception(TypeError, makeString('"', regularBilling.amount, "\" is not a valid amount."));
+
+    if (auto& trialBilling = recurringPaymentRequest.trialBilling) {
+        if (trialBilling->paymentTiming != ApplePayPaymentTiming::Recurring)
+            return Exception(TypeError, "'trialBilling' must be a 'recurring' line item."_s);
+        if (!trialBilling->label)
+            return Exception(TypeError, "Missing label for 'trialBilling'."_s);
+        if (!isValidDecimalMonetaryValue(trialBilling->amount) && trialBilling->type != ApplePayLineItem::Type::Pending)
+            return Exception(TypeError, makeString('"', trialBilling->amount, "\" is not a valid amount."));
+    }
+
+    if (auto& managementURL = recurringPaymentRequest.managementURL; !URL { managementURL }.isValid())
+        return Exception(TypeError, makeString('"', managementURL, "\" is not a valid URL."));
+
+    if (auto& tokenNotificationURL = recurringPaymentRequest.tokenNotificationURL; !tokenNotificationURL.isNull() && !URL { tokenNotificationURL }.isValid())
+        return Exception(TypeError, makeString('"', tokenNotificationURL, "\" is not a valid URL."));
+
+    return WTFMove(recurringPaymentRequest);
+}
+
+#endif // ENABLE(APPLE_PAY_RECURRING_PAYMENTS)
+
+#if ENABLE(APPLE_PAY_AUTOMATIC_RELOAD_PAYMENTS)
+
+static ExceptionOr<ApplePayAutomaticReloadPaymentRequest> convertAndValidate(ApplePayAutomaticReloadPaymentRequest&& automaticReloadPaymentRequest)
+{
+    auto& automaticReloadBilling = automaticReloadPaymentRequest.automaticReloadBilling;
+    if (automaticReloadBilling.paymentTiming != ApplePayPaymentTiming::AutomaticReload)
+        return Exception(TypeError, "'automaticReloadBilling' must be an 'automaticReload' line item."_s);
+    if (!automaticReloadBilling.label)
+        return Exception(TypeError, "Missing label for 'automaticReloadBilling'."_s);
+    if (!isValidDecimalMonetaryValue(automaticReloadBilling.amount) && automaticReloadBilling.type != ApplePayLineItem::Type::Pending)
+        return Exception(TypeError, makeString('"', automaticReloadBilling.amount, "\" is not a valid amount."));
+    if (!isValidDecimalMonetaryValue(automaticReloadBilling.automaticReloadPaymentThresholdAmount))
+        return Exception(TypeError, makeString('"', automaticReloadBilling.automaticReloadPaymentThresholdAmount, "\" is not a valid automaticReloadPaymentThresholdAmount."));
+
+    if (auto& managementURL = automaticReloadPaymentRequest.managementURL; !URL { managementURL }.isValid())
+        return Exception(TypeError, makeString('"', managementURL, "\" is not a valid URL."));
+
+    if (auto& tokenNotificationURL = automaticReloadPaymentRequest.tokenNotificationURL; !tokenNotificationURL.isNull() && !URL { tokenNotificationURL }.isValid())
+        return Exception(TypeError, makeString('"', tokenNotificationURL, "\" is not a valid URL."));
+
+    return WTFMove(automaticReloadPaymentRequest);
+}
+
+#endif // ENABLE(APPLE_PAY_AUTOMATIC_RELOAD_PAYMENTS)
+
+#if ENABLE(APPLE_PAY_MULTI_MERCHANT_PAYMENTS)
+
+static ExceptionOr<ApplePayPaymentTokenContext> convertAndValidate(ApplePayPaymentTokenContext&& tokenContext)
+{
+    if (!isValidDecimalMonetaryValue(tokenContext.amount))
+        return Exception { TypeError, makeString("\"" + tokenContext.amount, "\" is not a valid amount.") };
+
+    return WTFMove(tokenContext);
+}
+
+static ExceptionOr<Vector<ApplePayPaymentTokenContext>> convertAndValidate(Vector<ApplePayPaymentTokenContext>&& tokenContexts)
+{
+    Vector<ApplePayPaymentTokenContext> result;
+    result.reserveInitialCapacity(tokenContexts.size());
+
+    for (auto& tokenContext : tokenContexts) {
+        auto convertedTokenContext = convertAndValidate(WTFMove(tokenContext));
+        if (convertedTokenContext.hasException())
+            return convertedTokenContext.releaseException();
+        result.uncheckedAppend(convertedTokenContext.releaseReturnValue());
+    }
+
+    return WTFMove(result);
+}
+
+#endif // ENABLE(APPLE_PAY_MULTI_MERCHANT_PAYMENTS)
+
+#if ENABLE(APPLE_PAY_DEFERRED_PAYMENTS)
+
+ExceptionOr<void> ApplePayDeferredPaymentRequest::validate() const
+{
+    if (deferredBilling.paymentTiming != ApplePayPaymentTiming::Deferred)
+        return Exception(TypeError, "'deferredBilling' must be a 'deferred' line item."_s);
+    if (!deferredBilling.label)
+        return Exception(TypeError, "Missing label for 'deferredBilling'."_s);
+    if (!isValidDecimalMonetaryValue(deferredBilling.amount) && deferredBilling.type != ApplePayLineItem::Type::Pending)
+        return Exception(TypeError, makeString('"', deferredBilling.amount, "\" is not a valid amount."));
+
+    if (!URL { managementURL }.isValid())
+        return Exception(TypeError, makeString('"', managementURL, "\" is not a valid URL."));
+
+    if (std::isnan(freeCancellationDate)) {
+        if (!freeCancellationDateTimeZone.isEmpty())
+            return Exception(TypeError, "Unexpected 'freeCancellationDateTimeZone' when 'freeCancellationDate' is not set."_s);
+    } else if (freeCancellationDateTimeZone.isEmpty())
+        return Exception(TypeError, "Missing 'freeCancellationDateTimeZone' when 'freeCancellationDate' is set."_s);
+    else if (!isTimeZoneValid(freeCancellationDateTimeZone))
+        return Exception(TypeError, makeString('"', freeCancellationDateTimeZone, "\" is not a valid time zone."));
+
+    if (paymentDescription.isEmpty())
+        return Exception(TypeError, "Missing 'paymentDescription'."_s);
+
+    if (!tokenNotificationURL.isNull() && !URL { tokenNotificationURL }.isValid())
+        return Exception(TypeError, makeString('"', tokenNotificationURL, "\" is not a valid URL."));
+
+    return { };
+}
+
+ExceptionOr<ApplePayDeferredPaymentRequest> ApplePayDeferredPaymentRequest::convertAndValidate() &&
+{
+    auto validity = validate();
+    if (validity.hasException())
+        return validity.releaseException();
+
+    return WTFMove(*this);
+}
+
+#endif // ENABLE(APPLE_PAY_DEFERRED_PAYMENTS)
+
 static ExceptionOr<ApplePaySessionPaymentRequest> convertAndValidate(Document& document, unsigned version, ApplePayPaymentRequest&& paymentRequest, const PaymentCoordinator& paymentCoordinator)
 {
     auto convertedRequest = convertAndValidate(document, version, paymentRequest, paymentCoordinator);
@@ -253,140 +296,185 @@ static ExceptionOr<ApplePaySessionPaymentRequest> convertAndValidate(Document& d
         result.setShippingMethods(shippingMethods.releaseReturnValue());
     }
 
+#if ENABLE(APPLE_PAY_RECURRING_PAYMENTS)
+    if (paymentRequest.recurringPaymentRequest) {
+        auto recurringPaymentRequest = convertAndValidate(WTFMove(*paymentRequest.recurringPaymentRequest));
+        if (recurringPaymentRequest.hasException())
+            return recurringPaymentRequest.releaseException();
+        result.setRecurringPaymentRequest(recurringPaymentRequest.releaseReturnValue());
+    }
+#endif
+
+#if ENABLE(APPLE_PAY_AUTOMATIC_RELOAD_PAYMENTS)
+    if (paymentRequest.automaticReloadPaymentRequest) {
+        auto automaticReloadPaymentRequest = convertAndValidate(WTFMove(*paymentRequest.automaticReloadPaymentRequest));
+        if (automaticReloadPaymentRequest.hasException())
+            return automaticReloadPaymentRequest.releaseException();
+        result.setAutomaticReloadPaymentRequest(automaticReloadPaymentRequest.releaseReturnValue());
+    }
+#endif
+
+#if ENABLE(APPLE_PAY_MULTI_MERCHANT_PAYMENTS)
+    if (paymentRequest.multiTokenContexts) {
+        auto multiTokenContexts = convertAndValidate(WTFMove(*paymentRequest.multiTokenContexts));
+        if (multiTokenContexts.hasException())
+            return multiTokenContexts.releaseException();
+        result.setMultiTokenContexts(multiTokenContexts.releaseReturnValue());
+    }
+#endif
+
+#if ENABLE(APPLE_PAY_DEFERRED_PAYMENTS)
+    if (paymentRequest.deferredPaymentRequest) {
+        auto deferredPaymentRequest = WTFMove(*paymentRequest.deferredPaymentRequest).convertAndValidate();
+        if (deferredPaymentRequest.hasException())
+            return deferredPaymentRequest.releaseException();
+        result.setDeferredPaymentRequest(deferredPaymentRequest.releaseReturnValue());
+    }
+#endif
+
     // FIXME: Merge this validation into the validation we are doing above.
-    auto validatedPaymentRequest = PaymentRequestValidator::validate(result);
+    constexpr OptionSet fieldsToValidate = {
+        PaymentRequestValidator::Field::MerchantCapabilities,
+        PaymentRequestValidator::Field::SupportedNetworks,
+        PaymentRequestValidator::Field::CountryCode,
+        PaymentRequestValidator::Field::CurrencyCode,
+        PaymentRequestValidator::Field::Total,
+        PaymentRequestValidator::Field::ShippingMethods,
+    };
+    auto validatedPaymentRequest = PaymentRequestValidator::validate(result, fieldsToValidate);
     if (validatedPaymentRequest.hasException())
         return validatedPaymentRequest.releaseException();
 
     return WTFMove(result);
 }
 
+#if HAVE(APPLE_PAY_PAYMENT_ORDER_DETAILS)
 
-static Vector<PaymentError> convert(const Vector<RefPtr<ApplePayError>>& errors)
+static ExceptionOr<ApplePayPaymentOrderDetails> convertAndValidate(ApplePayPaymentOrderDetails&& orderDetails)
 {
-    Vector<PaymentError> convertedErrors;
+    if (auto& webServiceURL = orderDetails.webServiceURL; !URL { webServiceURL }.isValid())
+        return Exception(TypeError, makeString('"', webServiceURL, "\" is not a valid URL."));
 
-    for (auto& error : errors) {
-        PaymentError convertedError;
-
-        convertedError.code = error->code();
-        convertedError.message = error->message();
-        convertedError.contactField = error->contactField();
-
-        convertedErrors.append(convertedError);
-    }
-
-    return convertedErrors;
+    return WTFMove(orderDetails);
 }
 
-static ExceptionOr<PaymentAuthorizationResult> convertAndValidate(ApplePayPaymentAuthorizationResult&& result)
+#endif // HAVE(APPLE_PAY_PAYMENT_ORDER_DETAILS)
+
+static ExceptionOr<ApplePayPaymentAuthorizationResult> convertAndValidate(ApplePayPaymentAuthorizationResult&& result)
 {
-    PaymentAuthorizationResult convertedResult;
-
     switch (result.status) {
-    case ApplePaySession::STATUS_SUCCESS:
-        convertedResult.status = PaymentAuthorizationStatus::Success;
+    case ApplePayPaymentAuthorizationResult::Success:
+    case ApplePayPaymentAuthorizationResult::Failure:
+    case ApplePayPaymentAuthorizationResult::PINRequired:
+    case ApplePayPaymentAuthorizationResult::PINIncorrect:
+    case ApplePayPaymentAuthorizationResult::PINLockout:
         break;
 
-    case ApplePaySession::STATUS_FAILURE:
-        convertedResult.status = PaymentAuthorizationStatus::Failure;
+    case ApplePayPaymentAuthorizationResult::InvalidBillingPostalAddress:
+        result.status = ApplePayPaymentAuthorizationResult::Failure;
+        result.errors.insert(0, ApplePayError::create(ApplePayErrorCode::BillingContactInvalid, std::nullopt, nullString()));
         break;
 
-    case ApplePaySession::STATUS_INVALID_BILLING_POSTAL_ADDRESS:
-        convertedResult.status = PaymentAuthorizationStatus::Failure;
-        convertedResult.errors.append({ PaymentError::Code::BillingContactInvalid, { }, WTF::nullopt });
+    case ApplePayPaymentAuthorizationResult::InvalidShippingPostalAddress:
+        result.status = ApplePayPaymentAuthorizationResult::Failure;
+        result.errors.insert(0, ApplePayError::create(ApplePayErrorCode::ShippingContactInvalid, ApplePayErrorContactField::PostalAddress, nullString()));
         break;
 
-    case ApplePaySession::STATUS_INVALID_SHIPPING_POSTAL_ADDRESS:
-        convertedResult.status = PaymentAuthorizationStatus::Failure;
-        convertedResult.errors.append({ PaymentError::Code::ShippingContactInvalid, { }, PaymentError::ContactField::PostalAddress });
-        break;
-
-    case ApplePaySession::STATUS_INVALID_SHIPPING_CONTACT:
-        convertedResult.status = PaymentAuthorizationStatus::Failure;
-        convertedResult.errors.append({ PaymentError::Code::ShippingContactInvalid, { }, WTF::nullopt });
-        break;
-
-    case ApplePaySession::STATUS_PIN_REQUIRED:
-        convertedResult.status = PaymentAuthorizationStatus::PINRequired;
-        break;
-
-    case ApplePaySession::STATUS_PIN_INCORRECT:
-        convertedResult.status = PaymentAuthorizationStatus::PINIncorrect;
-        break;
-
-    case ApplePaySession::STATUS_PIN_LOCKOUT:
-        convertedResult.status = PaymentAuthorizationStatus::PINLockout;
+    case ApplePayPaymentAuthorizationResult::InvalidShippingContact:
+        result.status = ApplePayPaymentAuthorizationResult::Failure;
+        result.errors.insert(0, ApplePayError::create(ApplePayErrorCode::ShippingContactInvalid, std::nullopt, nullString()));
         break;
 
     default:
         return Exception { InvalidAccessError };
     }
 
-    convertedResult.errors.appendVector(convert(result.errors));
+#if HAVE(APPLE_PAY_PAYMENT_ORDER_DETAILS)
+    if (auto orderDetails = WTFMove(result.orderDetails)) {
+        auto convertedOrderDetails = convertAndValidate(WTFMove(*orderDetails));
+        if (convertedOrderDetails.hasException())
+            return convertedOrderDetails.releaseException();
+        result.orderDetails = convertedOrderDetails.releaseReturnValue();
+    }
+#endif
 
-    return WTFMove(convertedResult);
+    return WTFMove(result);
 }
 
-static ExceptionOr<PaymentMethodUpdate> convertAndValidate(ApplePayPaymentMethodUpdate&& update)
+static ExceptionOr<ApplePayPaymentMethodUpdate> convertAndValidate(ApplePayPaymentMethodUpdate&& update)
 {
     auto convertedNewTotal = convertAndValidateTotal(WTFMove(update.newTotal));
     if (convertedNewTotal.hasException())
         return convertedNewTotal.releaseException();
+    update.newTotal = convertedNewTotal.releaseReturnValue();
 
     auto convertedNewLineItems = convertAndValidate(WTFMove(update.newLineItems));
     if (convertedNewLineItems.hasException())
         return convertedNewLineItems.releaseException();
+    update.newLineItems = convertedNewLineItems.releaseReturnValue();
 
-    PaymentMethodUpdate convertedUpdate { convertedNewTotal.releaseReturnValue(), convertedNewLineItems.releaseReturnValue() };
-    finishConverting(convertedUpdate, WTFMove(update));
-
-    return WTFMove(convertedUpdate);
+    return WTFMove(update);
 }
 
-static ExceptionOr<ShippingContactUpdate> convertAndValidate(ApplePayShippingContactUpdate&& update)
+static ExceptionOr<ApplePayShippingContactUpdate> convertAndValidate(ApplePayShippingContactUpdate&& update)
 {
-    ShippingContactUpdate convertedUpdate;
-
-    convertedUpdate.errors = convert(update.errors);
-
     auto convertedNewShippingMethods = convertAndValidate(WTFMove(update.newShippingMethods));
     if (convertedNewShippingMethods.hasException())
         return convertedNewShippingMethods.releaseException();
-    convertedUpdate.newShippingMethods = convertedNewShippingMethods.releaseReturnValue();
+    update.newShippingMethods = convertedNewShippingMethods.releaseReturnValue();
 
     auto convertedNewTotal = convertAndValidateTotal(WTFMove(update.newTotal));
     if (convertedNewTotal.hasException())
         return convertedNewTotal.releaseException();
-    convertedUpdate.newTotalAndLineItems.total = convertedNewTotal.releaseReturnValue();
+    update.newTotal = convertedNewTotal.releaseReturnValue();
 
     auto convertedNewLineItems = convertAndValidate(WTFMove(update.newLineItems));
     if (convertedNewLineItems.hasException())
         return convertedNewLineItems.releaseException();
-    convertedUpdate.newTotalAndLineItems.lineItems = convertedNewLineItems.releaseReturnValue();
+    update.newLineItems = convertedNewLineItems.releaseReturnValue();
 
-    return WTFMove(convertedUpdate);
+    return WTFMove(update);
 }
 
-static ExceptionOr<ShippingMethodUpdate> convertAndValidate(ApplePayShippingMethodUpdate&& update)
+static ExceptionOr<ApplePayShippingMethodUpdate> convertAndValidate(ApplePayShippingMethodUpdate&& update)
 {
-    ShippingMethodUpdate convertedUpdate;
-
     auto convertedNewTotal = convertAndValidateTotal(WTFMove(update.newTotal));
     if (convertedNewTotal.hasException())
         return convertedNewTotal.releaseException();
-
-    convertedUpdate.newTotalAndLineItems.total = convertedNewTotal.releaseReturnValue();
+    update.newTotal = convertedNewTotal.releaseReturnValue();
 
     auto convertedNewLineItems = convertAndValidate(WTFMove(update.newLineItems));
     if (convertedNewLineItems.hasException())
         return convertedNewLineItems.releaseException();
+    update.newLineItems = convertedNewLineItems.releaseReturnValue();
 
-    convertedUpdate.newTotalAndLineItems.lineItems = convertedNewLineItems.releaseReturnValue();
-
-    return WTFMove(convertedUpdate);
+    return WTFMove(update);
 }
 
+#if ENABLE(APPLE_PAY_COUPON_CODE)
+
+static ExceptionOr<ApplePayCouponCodeUpdate> convertAndValidate(ApplePayCouponCodeUpdate&& update)
+{
+    auto convertedNewShippingMethods = convertAndValidate(WTFMove(update.newShippingMethods));
+    if (convertedNewShippingMethods.hasException())
+        return convertedNewShippingMethods.releaseException();
+    update.newShippingMethods = convertedNewShippingMethods.releaseReturnValue();
+
+    auto convertedNewTotal = convertAndValidateTotal(WTFMove(update.newTotal));
+    if (convertedNewTotal.hasException())
+        return convertedNewTotal.releaseException();
+    update.newTotal = convertedNewTotal.releaseReturnValue();
+
+    auto convertedNewLineItems = convertAndValidate(WTFMove(update.newLineItems));
+    if (convertedNewLineItems.hasException())
+        return convertedNewLineItems.releaseException();
+    update.newLineItems = convertedNewLineItems.releaseReturnValue();
+
+    return WTFMove(update);
+}
+
+#endif // ENABLE(APPLE_PAY_COUPON_CODE)
+
 ExceptionOr<Ref<ApplePaySession>> ApplePaySession::create(Document& document, unsigned version, ApplePayPaymentRequest&& paymentRequest)
 {
     auto canCall = canCreateSession(document);
@@ -394,16 +482,18 @@ ExceptionOr<Ref<ApplePaySession>> ApplePaySession::create(Document& document, un
         return canCall.releaseException();
 
     if (!UserGestureIndicator::processingUserGesture())
-        return Exception { InvalidAccessError, "Must create a new ApplePaySession from a user gesture handler." };
+        return Exception { InvalidAccessError, "Must create a new ApplePaySession from a user gesture handler."_s };
 
     if (!document.page())
-        return Exception { InvalidAccessError, "Frame is detached" };
+        return Exception { InvalidAccessError, "Frame is detached"_s };
 
     auto convertedPaymentRequest = convertAndValidate(document, version, WTFMove(paymentRequest), document.page()->paymentCoordinator());
     if (convertedPaymentRequest.hasException())
         return convertedPaymentRequest.releaseException();
 
-    return adoptRef(*new ApplePaySession(document, version, convertedPaymentRequest.releaseReturnValue()));
+    auto session = adoptRef(*new ApplePaySession(document, version, convertedPaymentRequest.releaseReturnValue()));
+    session->suspendIfNeeded();
+    return session;
 }
 
 ApplePaySession::ApplePaySession(Document& document, unsigned version, ApplePaySessionPaymentRequest&& paymentRequest)
@@ -412,7 +502,6 @@ ApplePaySession::ApplePaySession(Document& document, unsigned version, ApplePayS
     , m_version { version }
 {
     ASSERT(document.page()->paymentCoordinator().supportsVersion(document, version));
-    suspendIfNeeded();
 }
 
 ApplePaySession::~ApplePaySession() = default;
@@ -452,7 +541,7 @@ ExceptionOr<bool> ApplePaySession::canMakePayments(Document& document)
     if (!page)
         return Exception { InvalidAccessError };
 
-    return page->paymentCoordinator().canMakePayments(document);
+    return page->paymentCoordinator().canMakePayments();
 }
 
 ExceptionOr<void> ApplePaySession::canMakePaymentsWithActiveCard(Document& document, const String& merchantIdentifier, Ref<DeferredPromise>&& passedPromise)
@@ -468,7 +557,7 @@ ExceptionOr<void> ApplePaySession::canMakePaymentsWithActiveCard(Document& docum
             return Exception { InvalidAccessError };
 
         auto& paymentCoordinator = page->paymentCoordinator();
-        bool canMakePayments = paymentCoordinator.canMakePayments(document);
+        bool canMakePayments = paymentCoordinator.canMakePayments();
 
         RunLoop::main().dispatch([promise, canMakePayments]() mutable {
             promise->resolve<IDLBoolean>(canMakePayments);
@@ -495,7 +584,7 @@ ExceptionOr<void> ApplePaySession::openPaymentSetup(Document& document, const St
         return canCall.releaseException();
 
     if (!UserGestureIndicator::processingUserGesture())
-        return Exception { InvalidAccessError, "Must call ApplePaySession.openPaymentSetup from a user gesture handler." };
+        return Exception { InvalidAccessError, "Must call ApplePaySession.openPaymentSetup from a user gesture handler."_s };
 
     auto* page = document.page();
     if (!page)
@@ -514,18 +603,16 @@ ExceptionOr<void> ApplePaySession::openPaymentSetup(Document& document, const St
 ExceptionOr<void> ApplePaySession::begin(Document& document)
 {
     if (!canBegin())
-        return Exception { InvalidAccessError, "Payment session is already active." };
+        return Exception { InvalidAccessError, "Payment session is already active."_s };
 
     if (paymentCoordinator().hasActiveSession())
-        return Exception { InvalidAccessError, "Page already has an active payment session." };
+        return Exception { InvalidAccessError, "Page already has an active payment session."_s };
 
     if (!paymentCoordinator().beginPaymentSession(document, *this, m_paymentRequest))
-        return Exception { InvalidAccessError, "There is already has an active payment session." };
+        return Exception { InvalidAccessError, "There is already has an active payment session."_s };
 
     m_state = State::Active;
 
-    setPendingActivity(*this);
-
     return { };
 }
 
@@ -537,8 +624,6 @@ ExceptionOr<void> ApplePaySession::abort()
     m_state = State::Aborted;
     paymentCoordinator().abortPaymentSession();
 
-    didReachFinalState();
-
     return { };
 }
 
@@ -611,6 +696,25 @@ ExceptionOr<void> ApplePaySession::completePaymentMethodSelection(ApplePayPaymen
     return { };
 }
 
+#if ENABLE(APPLE_PAY_COUPON_CODE)
+
+ExceptionOr<void> ApplePaySession::completeCouponCodeChange(ApplePayCouponCodeUpdate&& update)
+{
+    if (!canCompleteCouponCodeChange())
+        return Exception { InvalidAccessError };
+
+    auto convertedUpdate = convertAndValidate(WTFMove(update));
+    if (convertedUpdate.hasException())
+        return convertedUpdate.releaseException();
+
+    m_state = State::Active;
+    paymentCoordinator().completeCouponCodeChange(convertedUpdate.releaseReturnValue());
+
+    return { };
+}
+
+#endif // ENABLE(APPLE_PAY_COUPON_CODE)
+
 ExceptionOr<void> ApplePaySession::completePayment(ApplePayPaymentAuthorizationResult&& result)
 {
     if (!canCompletePayment())
@@ -621,7 +725,7 @@ ExceptionOr<void> ApplePaySession::completePayment(ApplePayPaymentAuthorizationR
         return convertedResultOrException.releaseException();
 
     auto&& convertedResult = convertedResultOrException.releaseReturnValue();
-    bool isFinalState = isFinalStateResult(convertedResult);
+    bool isFinalState = convertedResult.isFinalState();
 
     paymentCoordinator().completePaymentSession(WTFMove(convertedResult));
 
@@ -631,7 +735,6 @@ ExceptionOr<void> ApplePaySession::completePayment(ApplePayPaymentAuthorizationR
     }
 
     m_state = State::Completed;
-    unsetPendingActivity(*this);
 
     return { };
 }
@@ -670,8 +773,8 @@ ExceptionOr<void> ApplePaySession::completeShippingContactSelection(unsigned sho
 {
     ApplePayShippingContactUpdate update;
 
-    Optional<ApplePayError::Code> errorCode;
-    Optional<ApplePayError::ContactField> contactField;
+    std::optional<ApplePayErrorCode> errorCode;
+    std::optional<ApplePayErrorContactField> contactField;
 
     switch (status) {
     case ApplePaySession::STATUS_SUCCESS:
@@ -681,20 +784,20 @@ ExceptionOr<void> ApplePaySession::completeShippingContactSelection(unsigned sho
     case ApplePaySession::STATUS_PIN_REQUIRED:
     case ApplePaySession::STATUS_PIN_INCORRECT:
     case ApplePaySession::STATUS_PIN_LOCKOUT:
-        errorCode = ApplePayError::Code::Unknown;
+        errorCode = ApplePayErrorCode::Unknown;
         break;
 
     case ApplePaySession::STATUS_INVALID_BILLING_POSTAL_ADDRESS:
-        errorCode = ApplePayError::Code::BillingContactInvalid;
+        errorCode = ApplePayErrorCode::BillingContactInvalid;
         break;
 
     case ApplePaySession::STATUS_INVALID_SHIPPING_POSTAL_ADDRESS:
-        errorCode = ApplePayError::Code::ShippingContactInvalid;
-        contactField = ApplePayError::ContactField::PostalAddress;
+        errorCode = ApplePayErrorCode::ShippingContactInvalid;
+        contactField = ApplePayErrorContactField::PostalAddress;
         break;
 
     case ApplePaySession::STATUS_INVALID_SHIPPING_CONTACT:
-        errorCode = ApplePayError::Code::ShippingContactInvalid;
+        errorCode = ApplePayErrorCode::ShippingContactInvalid;
         break;
 
     default:
@@ -766,7 +869,7 @@ void ApplePaySession::didAuthorizePayment(const Payment& payment)
     dispatchEvent(event.get());
 }
 
-void ApplePaySession::didSelectShippingMethod(const ApplePaySessionPaymentRequest::ShippingMethod& shippingMethod)
+void ApplePaySession::didSelectShippingMethod(const ApplePayShippingMethod& shippingMethod)
 {
     ASSERT(m_state == State::Active);
 
@@ -808,6 +911,24 @@ void ApplePaySession::didSelectPaymentMethod(const PaymentMethod& paymentMethod)
     dispatchEvent(event.get());
 }
 
+#if ENABLE(APPLE_PAY_COUPON_CODE)
+
+void ApplePaySession::didChangeCouponCode(String&& couponCode)
+{
+    ASSERT(m_state == State::Active);
+
+    if (!hasEventListeners(eventNames().couponcodechangedEvent)) {
+        paymentCoordinator().completeCouponCodeChange({ });
+        return;
+    }
+
+    m_state = State::CouponCodeChanged;
+    auto event = ApplePayCouponCodeChangedEvent::create(eventNames().couponcodechangedEvent, WTFMove(couponCode));
+    dispatchEvent(event.get());
+}
+
+#endif // ENABLE(APPLE_PAY_COUPON_CODE)
+
 void ApplePaySession::didCancelPaymentSession(PaymentSessionError&& error)
 {
     ASSERT(canCancel());
@@ -816,8 +937,6 @@ void ApplePaySession::didCancelPaymentSession(PaymentSessionError&& error)
 
     auto event = ApplePayCancelEvent::create(eventNames().cancelEvent, WTFMove(error));
     dispatchEvent(event.get());
-
-    didReachFinalState();
 }
 
 const char* ApplePaySession::activeDOMObjectName() const
@@ -839,6 +958,9 @@ bool ApplePaySession::canSuspendWithoutCanceling() const
     case State::ShippingMethodSelected:
     case State::ShippingContactSelected:
     case State::PaymentMethodSelected:
+#if ENABLE(APPLE_PAY_COUPON_CODE)
+    case State::CouponCodeChanged:
+#endif
     case State::CancelRequested:
         return false;
     }
@@ -851,8 +973,6 @@ void ApplePaySession::stop()
 
     m_state = State::Aborted;
     paymentCoordinator().abortPaymentSession();
-
-    didReachFinalState();
 }
 
 void ApplePaySession::suspend(ReasonForSuspension reason)
@@ -863,11 +983,10 @@ void ApplePaySession::suspend(ReasonForSuspension reason)
     if (canSuspendWithoutCanceling())
         return;
 
+    auto jsWrapperProtector = makePendingActivity(*this);
     m_state = State::Canceled;
     paymentCoordinator().abortPaymentSession();
     queueTaskToDispatchEvent(*this, TaskSource::UserInteraction, ApplePayCancelEvent::create(eventNames().cancelEvent, { }));
-
-    didReachFinalState();
 }
 
 PaymentCoordinator& ApplePaySession::paymentCoordinator() const
@@ -889,6 +1008,9 @@ bool ApplePaySession::canBegin() const
     case State::ShippingMethodSelected:
     case State::ShippingContactSelected:
     case State::PaymentMethodSelected:
+#if ENABLE(APPLE_PAY_COUPON_CODE)
+    case State::CouponCodeChanged:
+#endif
     case State::CancelRequested:
         return false;
     }
@@ -908,6 +1030,9 @@ bool ApplePaySession::canAbort() const
     case State::ShippingMethodSelected:
     case State::ShippingContactSelected:
     case State::PaymentMethodSelected:
+#if ENABLE(APPLE_PAY_COUPON_CODE)
+    case State::CouponCodeChanged:
+#endif
     case State::CancelRequested:
         return true;
     }
@@ -927,6 +1052,9 @@ bool ApplePaySession::canCancel() const
     case State::ShippingMethodSelected:
     case State::ShippingContactSelected:
     case State::PaymentMethodSelected:
+#if ENABLE(APPLE_PAY_COUPON_CODE)
+    case State::CouponCodeChanged:
+#endif
     case State::CancelRequested:
         return true;
     }
@@ -954,6 +1082,9 @@ bool ApplePaySession::canCompleteShippingMethodSelection() const
     case State::Authorized:
     case State::PaymentMethodSelected:
     case State::ShippingContactSelected:
+#if ENABLE(APPLE_PAY_COUPON_CODE)
+    case State::CouponCodeChanged:
+#endif
     case State::CancelRequested:
         return false;
 
@@ -973,6 +1104,9 @@ bool ApplePaySession::canCompleteShippingContactSelection() const
     case State::Authorized:
     case State::PaymentMethodSelected:
     case State::ShippingMethodSelected:
+#if ENABLE(APPLE_PAY_COUPON_CODE)
+    case State::CouponCodeChanged:
+#endif
     case State::CancelRequested:
         return false;
 
@@ -992,6 +1126,9 @@ bool ApplePaySession::canCompletePaymentMethodSelection() const
     case State::Authorized:
     case State::ShippingMethodSelected:
     case State::ShippingContactSelected:
+#if ENABLE(APPLE_PAY_COUPON_CODE)
+    case State::CouponCodeChanged:
+#endif
     case State::CancelRequested:
         return false;
 
@@ -1000,6 +1137,30 @@ bool ApplePaySession::canCompletePaymentMethodSelection() const
     }
 }
 
+#if ENABLE(APPLE_PAY_COUPON_CODE)
+
+bool ApplePaySession::canCompleteCouponCodeChange() const
+{
+    switch (m_state) {
+    case State::Idle:
+    case State::Aborted:
+    case State::Active:
+    case State::Completed:
+    case State::Canceled:
+    case State::Authorized:
+    case State::ShippingMethodSelected:
+    case State::ShippingContactSelected:
+    case State::CancelRequested:
+    case State::PaymentMethodSelected:
+        return false;
+
+    case State::CouponCodeChanged:
+        return true;
+    }
+}
+
+#endif // ENABLE(APPLE_PAY_COUPON_CODE)
+
 bool ApplePaySession::canCompletePayment() const
 {
     switch (m_state) {
@@ -1011,6 +1172,9 @@ bool ApplePaySession::canCompletePayment() const
     case State::ShippingMethodSelected:
     case State::ShippingContactSelected:
     case State::PaymentMethodSelected:
+#if ENABLE(APPLE_PAY_COUPON_CODE)
+    case State::CouponCodeChanged:
+#endif
     case State::CancelRequested:
         return false;
 
@@ -1027,6 +1191,9 @@ bool ApplePaySession::isFinalState() const
     case State::ShippingMethodSelected:
     case State::ShippingContactSelected:
     case State::PaymentMethodSelected:
+#if ENABLE(APPLE_PAY_COUPON_CODE)
+    case State::CouponCodeChanged:
+#endif
     case State::Authorized:
     case State::CancelRequested:
         return false;
@@ -1038,12 +1205,11 @@ bool ApplePaySession::isFinalState() const
     }
 }
 
-void ApplePaySession::didReachFinalState()
+bool ApplePaySession::virtualHasPendingActivity() const
 {
-    ASSERT(isFinalState());
-    unsetPendingActivity(*this);
+    return m_state != State::Idle && !isFinalState();
 }
 
-}
+} // namespace WebCore
 
-#endif
+#endif // ENABLE(APPLE_PAY)
