@@ -37,6 +37,7 @@
 #include "WebGPUShaderStage.h"
 #include "WebGPUTextureUsage.h"
 #include <WebGPU/WebGPU.h>
+#include <WebGPU/WebGPUExt.h>
 #include <cstdint>
 #include <wtf/RefCounted.h>
 
@@ -62,6 +63,7 @@ class ComputePipeline;
 enum class CullMode : uint8_t;
 class Device;
 enum class ErrorFilter : uint8_t;
+class ExternalTexture;
 enum class FeatureName : uint8_t;
 enum class FilterMode : uint8_t;
 enum class FrontFace : uint8_t;
@@ -150,9 +152,11 @@ public:
     virtual WGPUBuffer convertToBacking(const Buffer&) = 0;
     virtual WGPUCommandBuffer convertToBacking(const CommandBuffer&) = 0;
     virtual WGPUCommandEncoder convertToBacking(const CommandEncoder&) = 0;
+    virtual CompositorIntegrationImpl& convertToBacking(CompositorIntegration&) = 0;
     virtual WGPUComputePassEncoder convertToBacking(const ComputePassEncoder&) = 0;
     virtual WGPUComputePipeline convertToBacking(const ComputePipeline&) = 0;
     virtual WGPUDevice convertToBacking(const Device&) = 0;
+    virtual WGPUExternalTexture convertToBacking(const ExternalTexture&) = 0;
     virtual WGPUInstance convertToBacking(const GPU&) = 0;
     virtual WGPUPipelineLayout convertToBacking(const PipelineLayout&) = 0;
     virtual WGPUQuerySet convertToBacking(const QuerySet&) = 0;
@@ -166,7 +170,6 @@ public:
     virtual WGPUSurface convertToBacking(const PresentationContext&) = 0;
     virtual WGPUTexture convertToBacking(const Texture&) = 0;
     virtual WGPUTextureView convertToBacking(const TextureView&) = 0;
-    virtual CompositorIntegrationImpl& convertToBacking(CompositorIntegration&) = 0;
 };
 
 } // namespace PAL::WebGPU
