@@ -31,7 +31,6 @@
 
 #include "AccessibilityObject.h"
 #include "AccessibilityTable.h"
-#include "RenderObject.h"
 
 namespace WebCore {
     
@@ -40,13 +39,11 @@ AccessibilityARIAGridRow::AccessibilityARIAGridRow(RenderObject* renderer)
 {
 }
 
-AccessibilityARIAGridRow::~AccessibilityARIAGridRow()
-{
-}
+AccessibilityARIAGridRow::~AccessibilityARIAGridRow() = default;
 
-PassRefPtr<AccessibilityARIAGridRow> AccessibilityARIAGridRow::create(RenderObject* renderer)
+Ref<AccessibilityARIAGridRow> AccessibilityARIAGridRow::create(RenderObject* renderer)
 {
-    return adoptRef(new AccessibilityARIAGridRow(renderer));
+    return adoptRef(*new AccessibilityARIAGridRow(renderer));
 }
 
 bool AccessibilityARIAGridRow::isARIATreeGridRow() const
@@ -55,88 +52,98 @@ bool AccessibilityARIAGridRow::isARIATreeGridRow() const
     if (!parent)
         return false;
     
-    return parent->ariaRoleAttribute() == TreeGridRole;
+    return parent->isTreeGrid();
 }
     
-void AccessibilityARIAGridRow::disclosedRows(AccessibilityChildrenVector& disclosedRows)
+AXCoreObject::AccessibilityChildrenVector AccessibilityARIAGridRow::disclosedRows()
 {
+    AccessibilityChildrenVector disclosedRows;
     // The contiguous disclosed rows will be the rows in the table that 
     // have an aria-level of plus 1 from this row.
     AccessibilityObject* parent = parentObjectUnignored();
-    if (!is<AccessibilityTable>(*parent) || !downcast<AccessibilityTable>(*parent).isExposableThroughAccessibility())
-        return;
-    
+    if (!is<AccessibilityTable>(*parent) || !downcast<AccessibilityTable>(*parent).isExposable())
+        return disclosedRows;
+
     // Search for rows that match the correct level. 
     // Only take the subsequent rows from this one that are +1 from this row's level.
     int index = rowIndex();
     if (index < 0)
-        return;
-    
+        return disclosedRows;
+
     unsigned level = hierarchicalLevel();
-    auto& allRows = downcast<AccessibilityTable>(*parent).rows();
+    auto allRows = parent->rows();
     int rowCount = allRows.size();
     for (int k = index + 1; k < rowCount; ++k) {
-        AccessibilityObject* row = allRows[k].get();
+        auto* row = allRows[k].get();
         // Stop at the first row that doesn't match the correct level.
         if (row->hierarchicalLevel() != level + 1)
             break;
 
         disclosedRows.append(row);
     }
+
+    return disclosedRows;
 }
     
-AccessibilityObject* AccessibilityARIAGridRow::disclosedByRow() const
+AXCoreObject* AccessibilityARIAGridRow::disclosedByRow() const
 {
     // The row that discloses this one is the row in the table
     // that is aria-level subtract 1 from this row.
     AccessibilityObject* parent = parentObjectUnignored();
-    if (!is<AccessibilityTable>(*parent) || !downcast<AccessibilityTable>(*parent).isExposableThroughAccessibility())
+    if (!is<AccessibilityTable>(*parent) || !downcast<AccessibilityTable>(*parent).isExposable())
         return nullptr;
-    
+
     // If the level is 1 or less, than nothing discloses this row.
     unsigned level = hierarchicalLevel();
     if (level <= 1)
         return nullptr;
-    
+
     // Search for the previous row that matches the correct level.
     int index = rowIndex();
-    auto& allRows = downcast<AccessibilityTable>(parent)->rows();
+    auto allRows = parent->rows();
     int rowCount = allRows.size();
     if (index >= rowCount)
         return nullptr;
-    
+
     for (int k = index - 1; k >= 0; --k) {
-        AccessibilityObject* row = allRows[k].get();
+        auto* row = allRows[k].get();
         if (row->hierarchicalLevel() == level - 1)
             return row;
     }
-    
+
     return nullptr;
 }
-    
+
+AccessibilityObject* AccessibilityARIAGridRow::parentObjectUnignored() const
+{
+    if (auto* table = parentTable())
+        return table;
+    return AccessibilityTableRow::parentObjectUnignored();
+}
+
 AccessibilityTable* AccessibilityARIAGridRow::parentTable() const
 {
     // The parent table might not be the direct ancestor of the row unfortunately. ARIA states that role="grid" should
     // only have "row" elements, but if not, we still should handle it gracefully by finding the right table.
-    for (AccessibilityObject* parent = parentObject(); parent; parent = parent->parentObject()) {
+    return downcast<AccessibilityTable>(Accessibility::findAncestor<AccessibilityObject>(*this, false, [this] (const auto& ancestor) {
         // The parent table for an ARIA grid row should be an ARIA table.
-        if (is<AccessibilityTable>(*parent)) {
-            AccessibilityTable& tableParent = downcast<AccessibilityTable>(*parent);
-            if (tableParent.isExposableThroughAccessibility() && tableParent.isAriaTable())
-                return &tableParent;
+        // Unless the row is a native tr element.
+        if (is<AccessibilityTable>(ancestor)) {
+            auto& ancestorTable = downcast<AccessibilityTable>(ancestor);
+            return ancestorTable.isExposable() && (ancestorTable.isAriaTable() || node()->hasTagName(HTMLNames::trTag));
         }
-    }
-    
-    return nullptr;
+
+        return false;
+    }));
 }
 
-AccessibilityObject* AccessibilityARIAGridRow::headerObject()
+AXCoreObject* AccessibilityARIAGridRow::headerObject()
 {
     for (const auto& child : children()) {
-        if (child->ariaRoleAttribute() == RowHeaderRole)
+        if (child->roleValue() == AccessibilityRole::RowHeader)
             return child.get();
     }
-    
+
     return nullptr;
 }
 
