@@ -52,99 +52,87 @@ Ref<AccessibilityListBox> AccessibilityListBox::create(RenderObject* renderer)
 {
     return adoptRef(*new AccessibilityListBox(renderer));
 }
-    
-bool AccessibilityListBox::canSetSelectedChildrenAttribute() const
+
+bool AccessibilityListBox::canSetSelectedChildren() const
 {
-    Node* selectNode = m_renderer->node();
-    if (!selectNode)
-        return false;
-    
-    return !downcast<HTMLSelectElement>(*selectNode).isDisabledFormControl();
+    auto* selectElement = dynamicDowncast<HTMLSelectElement>(node());
+    return selectElement && !selectElement->isDisabledFormControl();
 }
 
 void AccessibilityListBox::addChildren()
 {
-    if (!m_renderer)
+    m_childrenInitialized = true;
+
+    auto* selectElement = dynamicDowncast<HTMLSelectElement>(node());
+    if (!selectElement)
         return;
 
-    Node* selectNode = m_renderer->node();
-    if (!selectNode)
-        return;
-
-    m_haveChildren = true;
-
-    for (const auto& listItem : downcast<HTMLSelectElement>(*selectNode).listItems()) {
-        // The cast to HTMLElement below is safe because the only other possible listItem type
-        // would be a WMLElement, but WML builds don't use accessibility features at all.
-        AccessibilityObject* listOption = listBoxOptionAccessibilityObject(listItem);
-        if (listOption && !listOption->accessibilityIsIgnored())
-            m_children.append(listOption);
-    }
+    for (const auto& listItem : selectElement->listItems())
+        addChild(listBoxOptionAccessibilityObject(listItem.get()), DescendIfIgnored::No);
 }
 
 void AccessibilityListBox::setSelectedChildren(const AccessibilityChildrenVector& children)
 {
-    if (!canSetSelectedChildrenAttribute())
+    if (!canSetSelectedChildren())
         return;
-    
-    Node* selectNode = m_renderer->node();
-    if (!selectNode)
+
+    WeakPtr node = this->node();
+    if (!node)
         return;
     
     // disable any selected options
     for (const auto& child : m_children) {
-        auto& listBoxOption = downcast<AccessibilityListBoxOption>(*child);
-        if (listBoxOption.isSelected())
-            listBoxOption.setSelected(false);
+        auto* listBoxOption = dynamicDowncast<AccessibilityListBoxOption>(child.get());
+        if (listBoxOption->isSelected())
+            listBoxOption->setSelected(false);
     }
     
-    for (const auto& obj : children) {
-        if (obj->roleValue() != AccessibilityRole::ListBoxOption)
+    for (const auto& object : children) {
+        if (object->roleValue() != AccessibilityRole::ListBoxOption)
             continue;
-
-        downcast<AccessibilityListBoxOption>(*obj).setSelected(true);
+        dynamicDowncast<AccessibilityListBoxOption>(object)->setSelected(true);
     }
 }
     
-void AccessibilityListBox::selectedChildren(AccessibilityChildrenVector& result)
+AXCoreObject::AccessibilityChildrenVector AccessibilityListBox::selectedChildren()
 {
-    ASSERT(result.isEmpty());
-
-    if (!hasChildren())
+    if (!childrenInitialized())
         addChildren();
-        
+
+    AccessibilityChildrenVector result;
     for (const auto& child : m_children) {
         if (downcast<AccessibilityListBoxOption>(*child).isSelected())
             result.append(child.get());
-    }    
+    }
+    return result;
 }
 
-void AccessibilityListBox::visibleChildren(AccessibilityChildrenVector& result)
+AXCoreObject::AccessibilityChildrenVector AccessibilityListBox::visibleChildren()
 {
-    ASSERT(result.isEmpty());
-    
-    if (!hasChildren())
+    ASSERT(!m_renderer || is<RenderListBox>(m_renderer.get()));
+    auto* listBox = dynamicDowncast<RenderListBox>(m_renderer.get());
+    if (!listBox)
+        return { };
+
+    if (!childrenInitialized())
         addChildren();
     
-    unsigned length = m_children.size();
-    for (unsigned i = 0; i < length; i++) {
-        if (downcast<RenderListBox>(*m_renderer).listIndexIsVisible(i))
+    AccessibilityChildrenVector result;
+    for (unsigned i = 0; i < m_children.size(); i++) {
+        if (listBox->listIndexIsVisible(i))
             result.append(m_children[i]);
     }
+    return result;
 }
 
 AccessibilityObject* AccessibilityListBox::listBoxOptionAccessibilityObject(HTMLElement* element) const
 {
-    // skip hr elements
-    if (!element || element->hasTagName(hrTag))
-        return nullptr;
-    
-    AccessibilityObject& listBoxObject = *m_renderer->document().axObjectCache()->getOrCreate(AccessibilityRole::ListBoxOption);
-    downcast<AccessibilityListBoxOption>(listBoxObject).setHTMLElement(element);
-    
-    return &listBoxObject;
+    // FIXME: Why does AccessibilityMenuListPopup::menuListOptionAccessibilityObject check inRenderedDocument, but this does not?
+    if (auto* document = this->document())
+        return document->axObjectCache()->getOrCreate(element);
+    return nullptr;
 }
-    
+
 AXCoreObject* AccessibilityListBox::elementAccessibilityHitTest(const IntPoint& point) const
 {
     // the internal HTMLSelectElement methods for returning a listbox option at a point
