@@ -29,12 +29,14 @@
 #include "config.h"
 #include "AccessibilityTable.h"
 
+#include "AXLogger.h"
 #include "AXObjectCache.h"
 #include "AccessibilityTableCell.h"
 #include "AccessibilityTableColumn.h"
 #include "AccessibilityTableHeaderContainer.h"
 #include "AccessibilityTableRow.h"
-#include "ElementIterator.h"
+#include "ElementAncestorIteratorInlines.h"
+#include "ElementChildIteratorInlines.h"
 #include "HTMLNames.h"
 #include "HTMLTableCaptionElement.h"
 #include "HTMLTableCellElement.h"
@@ -70,51 +72,56 @@ Ref<AccessibilityTable> AccessibilityTable::create(RenderObject* renderer)
     return adoptRef(*new AccessibilityTable(renderer));
 }
 
-bool AccessibilityTable::hasARIARole() const
+bool AccessibilityTable::hasNonTableARIARole() const
 {
-    if (!m_renderer)
+    switch (ariaRoleAttribute()) {
+    case AccessibilityRole::Unknown: // No role attribute specified.
+    case AccessibilityRole::Table:
+    case AccessibilityRole::Grid:
+    case AccessibilityRole::TreeGrid:
         return false;
-    
-    AccessibilityRole ariaRole = ariaRoleAttribute();
-    if (ariaRole != AccessibilityRole::Unknown)
+    default:
         return true;
-
-    return false;
+    }
 }
 
 bool AccessibilityTable::isExposable() const
 {
-    if (!m_renderer)
-        return false;
-    
-    return m_isExposable;
+    return m_isExposable && (m_renderer || is<HTMLTableElement>(node()));
 }
 
 HTMLTableElement* AccessibilityTable::tableElement() const
 {
-    if (!is<RenderTable>(*m_renderer))
+    if (auto* tableElement = dynamicDowncast<HTMLTableElement>(node()))
+        return tableElement;
+
+    auto* renderTable = dynamicDowncast<RenderTable>(renderer());
+    if (!renderTable)
         return nullptr;
-    
-    RenderTable& table = downcast<RenderTable>(*m_renderer);
-    if (is<HTMLTableElement>(table.element()))
-        return downcast<HTMLTableElement>(table.element());
+
+    if (auto* tableElement = dynamicDowncast<HTMLTableElement>(renderTable->element()))
+        return tableElement;
     // Try to find the table element, when the AccessibilityTable is mapped to an anonymous table renderer.
-    auto* firstChild = table.firstChild();
+    auto* firstChild = renderTable->firstChild();
     if (!firstChild || !firstChild->node())
         return nullptr;
-    if (is<HTMLTableElement>(*firstChild->node()))
-        return downcast<HTMLTableElement>(firstChild->node());
+    if (auto* childTable = dynamicDowncast<HTMLTableElement>(firstChild->node()))
+        return childTable;
     // FIXME: This might find an unrelated parent table element.
     return ancestorsOfType<HTMLTableElement>(*(firstChild->node())).first();
 }
-    
+
 bool AccessibilityTable::isDataTable() const
 {
     if (!m_renderer)
         return false;
 
-    // Do not consider it a data table is it has an ARIA role.
-    if (hasARIARole())
+    auto* objectCache = axObjectCache();
+    if (!objectCache)
+        return false;
+
+    // Do not consider it a data table if it has a non-table ARIA role.
+    if (hasNonTableARIARole())
         return false;
 
     // When a section of the document is contentEditable, all tables should be
@@ -132,7 +139,10 @@ bool AccessibilityTable::isDataTable() const
     // between a "layout" table and a "data" table.
     if (HTMLTableElement* tableElement = this->tableElement()) {
         // If there is a caption element, summary, THEAD, or TFOOT section, it's most certainly a data table.
-        if (!tableElement->summary().isEmpty() || tableElement->tHead() || tableElement->tFoot() || tableElement->caption())
+        if (!tableElement->summary().isEmpty()
+            || (tableElement->tHead() && tableElement->tHead()->renderer())
+            || (tableElement->tFoot() && tableElement->tFoot()->renderer())
+            || tableElement->caption())
             return true;
         
         // If someone used "rules" attribute than the table should appear.
@@ -167,7 +177,14 @@ bool AccessibilityTable::isDataTable() const
     RenderTableSection* firstBody = table.firstBody();
     if (!firstBody)
         return false;
-    
+
+    // If the tbody has any non-group role, then don't make this a data table. The author probably wants to use the role inside the <tbody>.
+    if (auto* topSection = objectCache->getOrCreate(table.topSection())) {
+        auto role = topSection->roleValue();
+        if (!topSection->isGroup() && role != AccessibilityRole::Unknown && role != AccessibilityRole::Ignored)
+            return false;
+    }
+
     int numCols = firstBody->numColumns();
     int numRows = firstBody->numRows();
     
@@ -343,25 +360,49 @@ bool AccessibilityTable::isDataTable() const
     
     return false;
 }
-    
+
 bool AccessibilityTable::computeIsTableExposableThroughAccessibility() const
 {
     // The following is a heuristic used to determine if a
     // <table> should be exposed as an AXTable. The goal
     // is to only show "data" tables.
 
-    if (!m_renderer)
+    if (!m_renderer && !is<HTMLTableElement>(node()))
         return false;
 
-    // If the developer assigned an aria role to this, then we
-    // shouldn't expose it as a table, unless, of course, the aria
-    // role is a table.
-    if (hasARIARole())
+    // If it has a non-table ARIA role, it shouldn't be exposed as a table.
+    if (hasNonTableARIARole())
         return false;
 
     return isDataTable();
 }
 
+
+void AccessibilityTable::recomputeIsExposable()
+{
+    bool previouslyExposable = m_isExposable;
+    m_isExposable = computeIsTableExposableThroughAccessibility();
+    if (previouslyExposable != m_isExposable) {
+        // A table's role value is dependent on whether it's exposed, so notify the cache this has changed.
+        if (auto* cache = axObjectCache())
+            cache->handleRoleChanged(this);
+
+        // Before resetting our existing children, possibly losing references to them, ensure we update their role (since a table cell's role is dependent on whether its parent table is exposable).
+        updateChildrenRoles();
+
+        m_childrenDirty = true;
+    }
+}
+
+void AccessibilityTable::updateChildrenRoles()
+{
+    for (const auto& row : m_rows) {
+        downcast<AccessibilityObject>(*row).updateRole();
+        for (const auto& cell : row->children())
+            downcast<AccessibilityObject>(*cell).updateRole();
+    }
+}
+
 void AccessibilityTable::clearChildren()
 {
     AccessibilityRenderObject::clearChildren();
@@ -381,9 +422,9 @@ void AccessibilityTable::addChildren()
         return;
     }
     
-    ASSERT(!m_haveChildren); 
+    ASSERT(!m_childrenInitialized); 
     
-    m_haveChildren = true;
+    m_childrenInitialized = true;
     if (!is<RenderTable>(renderer()))
         return;
     
@@ -394,8 +435,7 @@ void AccessibilityTable::addChildren()
     if (HTMLTableElement* tableElement = this->tableElement()) {
         if (auto caption = tableElement->caption()) {
             AccessibilityObject* axCaption = axObjectCache()->getOrCreate(caption.get());
-            if (axCaption && !axCaption->accessibilityIsIgnored())
-                m_children.append(axCaption);
+            addChild(axCaption, DescendIfIgnored::No);
         }
     }
 
@@ -412,7 +452,7 @@ void AccessibilityTable::addChildren()
     if (footer)
         addChildrenFromSection(footer, maxColumnCount);
     
-    AXObjectCache* axCache = m_renderer->document().axObjectCache();
+    auto* axCache = document()->axObjectCache();
     // make the columns based on the number of columns in the first body
     unsigned length = maxColumnCount;
     for (unsigned i = 0; i < length; ++i) {
@@ -420,22 +460,15 @@ void AccessibilityTable::addChildren()
         column.setColumnIndex(i);
         column.setParent(this);
         m_columns.append(&column);
-        if (!column.accessibilityIsIgnored())
-            m_children.append(&column);
+        addChild(&column, DescendIfIgnored::No);
     }
-
-    auto* headerContainerObject = headerContainer();
-    if (headerContainerObject && !headerContainerObject->accessibilityIsIgnored())
-        m_children.append(headerContainerObject);
+    addChild(headerContainer(), DescendIfIgnored::No);
 
     // Sometimes the cell gets the wrong role initially because it is created before the parent
     // determines whether it is an accessibility table. Iterate all the cells and allow them to
     // update their roles now that the table knows its status.
     // see bug: https://bugs.webkit.org/show_bug.cgi?id=147001
-    for (const auto& row : m_rows) {
-        for (const auto& cell : row->children())
-            cell->updateAccessibilityRole();
-    }
+    updateChildrenRoles();
 }
 
 void AccessibilityTable::addTableCellChild(AccessibilityObject* rowObject, HashSet<AccessibilityObject*>& appendedRows, unsigned& columnCount)
@@ -451,8 +484,7 @@ void AccessibilityTable::addTableCellChild(AccessibilityObject* rowObject, HashS
     
     row.setRowIndex(static_cast<int>(m_rows.size()));
     m_rows.append(&row);
-    if (!row.accessibilityIsIgnored())
-        m_children.append(&row);
+    addChild(&row, DescendIfIgnored::No);
     appendedRows.add(&row);
         
     // store the maximum number of columns
@@ -467,7 +499,7 @@ void AccessibilityTable::addChildrenFromSection(RenderTableSection* tableSection
     if (!tableSection)
         return;
     
-    AXObjectCache* axCache = m_renderer->document().axObjectCache();
+    auto* axCache = document()->axObjectCache();
     HashSet<AccessibilityObject*> appendedRows;
     unsigned numRows = tableSection->numRows();
     for (unsigned rowIndex = 0; rowIndex < numRows; ++rowIndex) {
@@ -503,11 +535,15 @@ AXCoreObject* AccessibilityTable::headerContainer()
 {
     if (m_headerContainer)
         return m_headerContainer.get();
-    
-    auto& tableHeader = downcast<AccessibilityMockObject>(*axObjectCache()->create(AccessibilityRole::TableHeaderContainer));
-    tableHeader.setParent(this);
 
-    m_headerContainer = &tableHeader;
+    auto* cache = axObjectCache();
+    if (!cache)
+        return nullptr;
+
+    auto* tableHeader = downcast<AccessibilityMockObject>(cache->create(AccessibilityRole::TableHeaderContainer));
+    tableHeader->setParent(this);
+
+    m_headerContainer = tableHeader;
     return m_headerContainer.get();
 }
 
@@ -527,15 +563,11 @@ AXCoreObject::AccessibilityChildrenVector AccessibilityTable::rows()
 
 AXCoreObject::AccessibilityChildrenVector AccessibilityTable::columnHeaders()
 {
-    AccessibilityChildrenVector headers;
-    if (!m_renderer)
-        return headers;
-
     updateChildrenIfNecessary();
 
+    AccessibilityChildrenVector headers;
     // Sometimes m_columns can be reset during the iteration, we cache it here to be safe.
     AccessibilityChildrenVector columnsCopy = m_columns;
-
     for (const auto& column : columnsCopy) {
         if (auto* header = column->columnHeader())
             headers.append(header);
@@ -546,17 +578,13 @@ AXCoreObject::AccessibilityChildrenVector AccessibilityTable::columnHeaders()
 
 AXCoreObject::AccessibilityChildrenVector AccessibilityTable::rowHeaders()
 {
-    AccessibilityChildrenVector headers;
-    if (!m_renderer)
-        return headers;
-
     updateChildrenIfNecessary();
 
+    AccessibilityChildrenVector headers;
     // Sometimes m_rows can be reset during the iteration, we cache it here to be safe.
     AccessibilityChildrenVector rowsCopy = m_rows;
-
     for (const auto& row : rowsCopy) {
-        if (AXCoreObject* header = downcast<AccessibilityTableRow>(*row).headerObject())
+        if (auto* header = downcast<AccessibilityTableRow>(*row).headerObject())
             headers.append(header);
     }
 
@@ -565,12 +593,9 @@ AXCoreObject::AccessibilityChildrenVector AccessibilityTable::rowHeaders()
 
 AXCoreObject::AccessibilityChildrenVector AccessibilityTable::visibleRows()
 {
-    AccessibilityChildrenVector rows;
-    if (!m_renderer)
-        return rows;
-
     updateChildrenIfNecessary();
 
+    AccessibilityChildrenVector rows;
     for (const auto& row : m_rows) {
         if (row && !row->isOffScreen())
             rows.append(row);
@@ -581,15 +606,11 @@ AXCoreObject::AccessibilityChildrenVector AccessibilityTable::visibleRows()
 
 AXCoreObject::AccessibilityChildrenVector AccessibilityTable::cells()
 {
-    AccessibilityChildrenVector cells;
-    if (!m_renderer)
-        return cells;
-
     updateChildrenIfNecessary();
 
+    AccessibilityChildrenVector cells;
     for (const auto& row : m_rows)
         cells.appendVector(row->children());
-
     return cells;
 }
     
@@ -652,7 +673,7 @@ AccessibilityRole AccessibilityTable::roleValue() const
 {
     if (!isExposable())
         return AccessibilityRenderObject::roleValue();
-    
+
     AccessibilityRole ariaRole = ariaRoleAttribute();
     if (ariaRole == AccessibilityRole::Grid || ariaRole == AccessibilityRole::TreeGrid)
         return ariaRole;
@@ -687,20 +708,15 @@ String AccessibilityTable::title() const
         return AccessibilityRenderObject::title();
     
     String title;
-    if (!m_renderer)
-        return title;
-    
-    // see if there is a caption
-    Node* tableElement = m_renderer->node();
-    if (is<HTMLTableElement>(tableElement)) {
-        if (auto caption = downcast<HTMLTableElement>(*tableElement).caption())
+    // Prefer the table caption if present.
+    if (auto* tableElement = dynamicDowncast<HTMLTableElement>(node())) {
+        if (RefPtr caption = tableElement->caption())
             title = caption->innerText();
     }
     
-    // try the standard 
+    // Fall back to standard title computation.
     if (title.isEmpty())
         title = AccessibilityRenderObject::title();
-    
     return title;
 }
 
