@@ -41,17 +41,13 @@ namespace WebCore {
     
 using namespace HTMLNames;
 
-AccessibilityTableColumn::AccessibilityTableColumn()
-{
-}
+AccessibilityTableColumn::AccessibilityTableColumn() = default;
 
-AccessibilityTableColumn::~AccessibilityTableColumn()
-{
-}    
+AccessibilityTableColumn::~AccessibilityTableColumn() = default;
 
-PassRefPtr<AccessibilityTableColumn> AccessibilityTableColumn::create()
+Ref<AccessibilityTableColumn> AccessibilityTableColumn::create()
 {
-    return adoptRef(new AccessibilityTableColumn());
+    return adoptRef(*new AccessibilityTableColumn());
 }
 
 void AccessibilityTableColumn::setParent(AccessibilityObject* parent)
@@ -63,31 +59,34 @@ void AccessibilityTableColumn::setParent(AccessibilityObject* parent)
     
 LayoutRect AccessibilityTableColumn::elementRect() const
 {
-    // this will be filled in when addChildren is called
-    return m_columnRect;
+    // This used to be cached during the call to addChildren(), but calling elementRect()
+    // can invalidate elements, so its better to ask for this on demand.
+    LayoutRect columnRect;
+    AccessibilityChildrenVector childrenCopy = m_children;
+    for (const auto& cell : childrenCopy)
+        columnRect.unite(cell->elementRect());
+
+    return columnRect;
 }
 
-AccessibilityObject* AccessibilityTableColumn::headerObject()
+AXCoreObject* AccessibilityTableColumn::columnHeader()
 {
-    if (!m_parent)
+    if (!m_parent || !is<AccessibilityTable>(*m_parent)
+        || !m_parent->isExposable())
         return nullptr;
-    
+
     RenderObject* renderer = m_parent->renderer();
     if (!renderer)
         return nullptr;
-    if (!is<AccessibilityTable>(*m_parent))
-        return nullptr;
 
     auto& parentTable = downcast<AccessibilityTable>(*m_parent);
-    if (!parentTable.isExposableThroughAccessibility())
-        return nullptr;
-    
+
     if (parentTable.isAriaTable()) {
         for (const auto& cell : children()) {
-            if (cell->ariaRoleAttribute() == ColumnHeaderRole)
+            if (cell->roleValue() == AccessibilityRole::ColumnHeader)
                 return cell.get();
         }
-        
+
         return nullptr;
     }
 
@@ -136,12 +135,21 @@ AccessibilityObject* AccessibilityTableColumn::headerObjectForSection(RenderTabl
             if ((testCell->col() + (testCell->colSpan()-1)) < m_columnIndex)
                 break;
             
-            // If this does not have an element (like a <caption>) then check the next row
-            if (!testCell->element())
+            Node* testCellNode = testCell->element();
+            // If the RenderTableCell doesn't have an element because its anonymous,
+            // try to see if we can find the original cell element to check if it has a <th> tag.
+            if (!testCellNode && testCell->isAnonymous()) {
+                if (Element* parentElement = testCell->parent() ? testCell->parent()->element() : nullptr) {
+                    if (parentElement->hasTagName(trTag) && testCol < static_cast<int>(parentElement->countChildNodes()))
+                        testCellNode = parentElement->traverseToChildAt(testCol);
+                }
+            }
+
+            if (!testCellNode)
                 continue;
             
             // If th is required, but we found an element that doesn't have a th tag, we can stop looking.
-            if (thTagRequired && !testCell->element()->hasTagName(thTag))
+            if (thTagRequired && !testCellNode->hasTagName(thTag))
                 break;
             
             cell = testCell;
@@ -152,7 +160,11 @@ AccessibilityObject* AccessibilityTableColumn::headerObjectForSection(RenderTabl
     if (!cell)
         return nullptr;
 
-    return axObjectCache()->getOrCreate(cell);
+    auto* cellObject = axObjectCache()->getOrCreate(cell);
+    if (!cellObject || cellObject->accessibilityIsIgnored())
+        return nullptr;
+        
+    return cellObject;
 }
     
 bool AccessibilityTableColumn::computeAccessibilityIsIgnored() const
@@ -160,7 +172,7 @@ bool AccessibilityTableColumn::computeAccessibilityIsIgnored() const
     if (!m_parent)
         return true;
     
-#if PLATFORM(IOS) || PLATFORM(GTK) || PLATFORM(EFL)
+#if PLATFORM(IOS_FAMILY) || USE(ATSPI)
     return true;
 #endif
     
@@ -169,20 +181,20 @@ bool AccessibilityTableColumn::computeAccessibilityIsIgnored() const
     
 void AccessibilityTableColumn::addChildren()
 {
-    ASSERT(!m_haveChildren); 
+    ASSERT(!m_childrenInitialized); 
     
-    m_haveChildren = true;
+    m_childrenInitialized = true;
     if (!is<AccessibilityTable>(m_parent))
         return;
 
     auto& parentTable = downcast<AccessibilityTable>(*m_parent);
-    if (!parentTable.isExposableThroughAccessibility())
+    if (!parentTable.isExposable())
         return;
     
     int numRows = parentTable.rowCount();
     
     for (int i = 0; i < numRows; ++i) {
-        AccessibilityTableCell* cell = parentTable.cellForColumnAndRow(m_columnIndex, i);
+        auto* cell = parentTable.cellForColumnAndRow(m_columnIndex, i);
         if (!cell)
             continue;
         
@@ -190,8 +202,7 @@ void AccessibilityTableColumn::addChildren()
         if (m_children.size() > 0 && m_children.last() == cell)
             continue;
             
-        m_children.append(cell);
-        m_columnRect.unite(cell->elementRect());
+        addChild(cell);
     }
 }
     
