@@ -21,7 +21,6 @@
 #include "config.h"
 #include "CSSPrimitiveValue.h"
 
-#include "CSSBasicShapes.h"
 #include "CSSCalcValue.h"
 #include "CSSHelper.h"
 #include "CSSMarkup.h"
@@ -37,13 +36,10 @@
 #include "Color.h"
 #include "ColorSerialization.h"
 #include "ContainerQueryEvaluator.h"
-#include "Counter.h"
-#include "DeprecatedCSSOMPrimitiveValue.h"
 #include "FontCascade.h"
 #include "Length.h"
 #include "NodeRenderStyle.h"
-#include "Pair.h"
-#include "Rect.h"
+#include "RenderBoxInlines.h"
 #include "RenderStyle.h"
 #include "RenderView.h"
 #include <wtf/NeverDestroyed.h>
@@ -123,18 +119,12 @@ static inline bool isValidCSSUnitTypeForDoubleConversion(CSSUnitType unitType)
     case CSSUnitType::CSS_CQMAX:
         return true;
     case CSSUnitType::CSS_ATTR:
-    case CSSUnitType::CSS_COUNTER:
     case CSSUnitType::CSS_COUNTER_NAME:
     case CSSUnitType::CSS_FONT_FAMILY:
     case CSSUnitType::CustomIdent:
-    case CSSUnitType::CSS_PAIR:
     case CSSUnitType::CSS_PROPERTY_ID:
-    case CSSUnitType::CSS_QUAD:
-    case CSSUnitType::CSS_RECT:
     case CSSUnitType::CSS_RGBCOLOR:
-    case CSSUnitType::CSS_SHAPE:
     case CSSUnitType::CSS_STRING:
-    case CSSUnitType::CSS_UNICODE_RANGE:
     case CSSUnitType::CSS_UNKNOWN:
     case CSSUnitType::CSS_UNRESOLVED_COLOR:
     case CSSUnitType::CSS_URI:
@@ -166,7 +156,6 @@ static inline bool isStringType(CSSUnitType type)
     case CSSUnitType::CSS_CHS:
     case CSSUnitType::CSS_IC:
     case CSSUnitType::CSS_CM:
-    case CSSUnitType::CSS_COUNTER:
     case CSSUnitType::CSS_DEG:
     case CSSUnitType::CSS_DIMENSION:
     case CSSUnitType::CSS_DPCM:
@@ -197,7 +186,6 @@ static inline bool isStringType(CSSUnitType type)
     case CSSUnitType::CSS_MS:
     case CSSUnitType::CSS_NUMBER:
     case CSSUnitType::CSS_INTEGER:
-    case CSSUnitType::CSS_PAIR:
     case CSSUnitType::CSS_PC:
     case CSSUnitType::CSS_PERCENTAGE:
     case CSSUnitType::CSS_PROPERTY_ID:
@@ -206,10 +194,8 @@ static inline bool isStringType(CSSUnitType type)
     case CSSUnitType::CSS_Q:
     case CSSUnitType::CSS_LHS:
     case CSSUnitType::CSS_RLHS:
-    case CSSUnitType::CSS_QUAD:
     case CSSUnitType::CSS_QUIRKY_EMS:
     case CSSUnitType::CSS_RAD:
-    case CSSUnitType::CSS_RECT:
     case CSSUnitType::CSS_REMS:
     case CSSUnitType::CSS_RGBCOLOR:
     case CSSUnitType::CSS_S:
@@ -219,9 +205,7 @@ static inline bool isStringType(CSSUnitType type)
     case CSSUnitType::CSS_SVMAX:
     case CSSUnitType::CSS_SVMIN:
     case CSSUnitType::CSS_SVW:
-    case CSSUnitType::CSS_SHAPE:
     case CSSUnitType::CSS_TURN:
-    case CSSUnitType::CSS_UNICODE_RANGE:
     case CSSUnitType::CSS_UNKNOWN:
     case CSSUnitType::CSS_UNRESOLVED_COLOR:
     case CSSUnitType::CSS_VALUE_ID:
@@ -264,7 +248,7 @@ CSSUnitType CSSPrimitiveValue::primitiveType() const
     if (primitiveUnitType() == CSSUnitType::CSS_FONT_FAMILY)
         return CSSUnitType::CSS_STRING;
 
-    if (primitiveUnitType() != CSSUnitType::CSS_CALC)
+    if (!isCalculated())
         return primitiveUnitType();
 
     switch (m_value.calc->category()) {
@@ -311,12 +295,12 @@ CSSPrimitiveValue::CSSPrimitiveValue(const String& string, CSSUnitType type)
         m_value.string->ref();
 }
 
-CSSPrimitiveValue::CSSPrimitiveValue(const Color& color)
+CSSPrimitiveValue::CSSPrimitiveValue(Color color)
     : CSSValue(PrimitiveClass)
 {
     setPrimitiveUnitType(CSSUnitType::CSS_RGBCOLOR);
     static_assert(sizeof(m_value.colorAsInteger) == sizeof(color));
-    new (reinterpret_cast<Color*>(&m_value.colorAsInteger)) Color(color);
+    new (reinterpret_cast<Color*>(&m_value.colorAsInteger)) Color(WTFMove(color));
 }
 
 CSSPrimitiveValue::CSSPrimitiveValue(StaticCSSValueTag, CSSValueID valueID)
@@ -327,8 +311,8 @@ CSSPrimitiveValue::CSSPrimitiveValue(StaticCSSValueTag, CSSValueID valueID)
     makeStatic();
 }
 
-CSSPrimitiveValue::CSSPrimitiveValue(StaticCSSValueTag, const Color& color)
-    : CSSPrimitiveValue(color)
+CSSPrimitiveValue::CSSPrimitiveValue(StaticCSSValueTag, Color color)
+    : CSSPrimitiveValue(WTFMove(color))
 {
     makeStatic();
 }
@@ -345,53 +329,18 @@ CSSPrimitiveValue::CSSPrimitiveValue(StaticCSSValueTag, ImplicitInitialValueTag)
     m_isImplicitInitialValue = true;
 }
 
-CSSPrimitiveValue::CSSPrimitiveValue(Ref<Counter>&& counter)
-    : CSSValue(PrimitiveClass)
-{
-    setPrimitiveUnitType(CSSUnitType::CSS_COUNTER);
-    m_value.counter = &counter.leakRef();
-}
-
-CSSPrimitiveValue::CSSPrimitiveValue(Ref<Rect>&& rect)
-    : CSSValue(PrimitiveClass)
-{
-    setPrimitiveUnitType(CSSUnitType::CSS_RECT);
-    m_value.rect = &rect.leakRef();
-}
-
-CSSPrimitiveValue::CSSPrimitiveValue(Ref<Quad>&& quad)
-    : CSSValue(PrimitiveClass)
-{
-    setPrimitiveUnitType(CSSUnitType::CSS_QUAD);
-    m_value.quad = &quad.leakRef();
-}
-
-CSSPrimitiveValue::CSSPrimitiveValue(Ref<Pair>&& pair)
-    : CSSValue(PrimitiveClass)
-{
-    setPrimitiveUnitType(CSSUnitType::CSS_PAIR);
-    m_value.pair = &pair.leakRef();
-}
-
-CSSPrimitiveValue::CSSPrimitiveValue(Ref<CSSBasicShape>&& shape)
-    : CSSValue(PrimitiveClass)
-{
-    setPrimitiveUnitType(CSSUnitType::CSS_SHAPE);
-    m_value.shape = &shape.leakRef();
-}
-
-CSSPrimitiveValue::CSSPrimitiveValue(Ref<CSSCalcValue>&& value)
+CSSPrimitiveValue::CSSPrimitiveValue(Ref<CSSCalcValue> value)
     : CSSValue(PrimitiveClass)
 {
     setPrimitiveUnitType(CSSUnitType::CSS_CALC);
     m_value.calc = &value.leakRef();
 }
 
-CSSPrimitiveValue::CSSPrimitiveValue(Ref<CSSUnresolvedColor>&& unresolvedColor)
+CSSPrimitiveValue::CSSPrimitiveValue(CSSUnresolvedColor unresolvedColor)
     : CSSValue(PrimitiveClass)
 {
     setPrimitiveUnitType(CSSUnitType::CSS_UNRESOLVED_COLOR);
-    m_value.unresolvedColor = &unresolvedColor.leakRef();
+    m_value.unresolvedColor = new CSSUnresolvedColor(WTFMove(unresolvedColor));
 }
 
 CSSPrimitiveValue::~CSSPrimitiveValue()
@@ -407,18 +356,6 @@ CSSPrimitiveValue::~CSSPrimitiveValue()
         if (m_value.string)
             m_value.string->deref();
         break;
-    case CSSUnitType::CSS_COUNTER:
-        m_value.counter->deref();
-        break;
-    case CSSUnitType::CSS_RECT:
-        m_value.rect->deref();
-        break;
-    case CSSUnitType::CSS_QUAD:
-        m_value.quad->deref();
-        break;
-    case CSSUnitType::CSS_PAIR:
-        m_value.pair->deref();
-        break;
     case CSSUnitType::CSS_CALC:
         m_value.calc->deref();
         break;
@@ -426,14 +363,11 @@ CSSPrimitiveValue::~CSSPrimitiveValue()
     case CSSUnitType::CSS_CALC_PERCENTAGE_WITH_LENGTH:
         ASSERT_NOT_REACHED();
         break;
-    case CSSUnitType::CSS_SHAPE:
-        m_value.shape->deref();
-        break;
     case CSSUnitType::CSS_RGBCOLOR:
         std::destroy_at(reinterpret_cast<Color*>(&m_value.colorAsInteger));
         break;
     case CSSUnitType::CSS_UNRESOLVED_COLOR:
-        m_value.unresolvedColor->deref();
+        delete m_value.unresolvedColor;
         break;
     case CSSUnitType::CSS_DIMENSION:
     case CSSUnitType::CSS_NUMBER:
@@ -493,7 +427,6 @@ CSSPrimitiveValue::~CSSPrimitiveValue()
     case CSSUnitType::CSS_RLHS:
     case CSSUnitType::CSS_IDENT:
     case CSSUnitType::CSS_UNKNOWN:
-    case CSSUnitType::CSS_UNICODE_RANGE:
     case CSSUnitType::CSS_PROPERTY_ID:
     case CSSUnitType::CSS_VALUE_ID:
     case CSSUnitType::CSS_CQW:
@@ -516,31 +449,48 @@ Ref<CSSPrimitiveValue> CSSPrimitiveValue::create(CSSPropertyID propertyID)
     return adoptRef(*new CSSPrimitiveValue(propertyID));
 }
 
+static CSSPrimitiveValue* valueFromPool(std::span<LazyNeverDestroyed<CSSPrimitiveValue>> pool, double value)
+{
+    // Casting to a signed integer first since casting a negative floating point value to an unsigned
+    // integer is undefined behavior.
+    unsigned poolIndex = static_cast<unsigned>(static_cast<int>(value));
+    double roundTripValue = poolIndex;
+    if (!memcmp(&value, &roundTripValue, sizeof(double)) && poolIndex < pool.size())
+        return &pool[poolIndex].get();
+    return nullptr;
+}
+
+Ref<CSSPrimitiveValue> CSSPrimitiveValue::create(double value)
+{
+    if (auto* result = valueFromPool(staticCSSValuePool->m_numberValues, value))
+        return *result;
+    return adoptRef(*new CSSPrimitiveValue(value, CSSUnitType::CSS_NUMBER));
+}
+
 Ref<CSSPrimitiveValue> CSSPrimitiveValue::create(double value, CSSUnitType type)
 {
-    if (auto values = [type]() -> LazyNeverDestroyed<CSSPrimitiveValue>* {
-        switch (type) {
-        case CSSUnitType::CSS_NUMBER:
-            return staticCSSValuePool->m_numberValues;
-        case CSSUnitType::CSS_PERCENTAGE:
-            return staticCSSValuePool->m_percentValues;
-        case CSSUnitType::CSS_PX:
-            return staticCSSValuePool->m_pixelValues;
-        default:
-            return nullptr;
-        }
-    }()) {
-        int intValue = value;
-        double roundTripValue = intValue;
-        if (!memcmp(&value, &roundTripValue, sizeof(double)) && intValue >= 0 && intValue <= StaticCSSValuePool::maximumCacheableIntegerValue)
-            return values[intValue].get();
+    switch (type) {
+    case CSSUnitType::CSS_NUMBER:
+        if (auto* result = valueFromPool(staticCSSValuePool->m_numberValues, value))
+            return *result;
+        break;
+    case CSSUnitType::CSS_PERCENTAGE:
+        if (auto* result = valueFromPool(staticCSSValuePool->m_percentValues, value))
+            return *result;
+        break;
+    case CSSUnitType::CSS_PX:
+        if (auto* result = valueFromPool(staticCSSValuePool->m_pixelValues, value))
+            return *result;
+        break;
+    default:
+        break;
     }
     return adoptRef(*new CSSPrimitiveValue(value, type));
 }
 
-Ref<CSSPrimitiveValue> CSSPrimitiveValue::create(const String& value, CSSUnitType type)
+Ref<CSSPrimitiveValue> CSSPrimitiveValue::create(String value)
 {
-    return adoptRef(*new CSSPrimitiveValue(value, type));
+    return adoptRef(*new CSSPrimitiveValue(WTFMove(value), CSSUnitType::CSS_STRING));
 }
 
 Ref<CSSPrimitiveValue> CSSPrimitiveValue::create(const Length& length)
@@ -600,44 +550,44 @@ Ref<CSSPrimitiveValue> CSSPrimitiveValue::create(const Length& length, const Ren
     RELEASE_ASSERT_NOT_REACHED();
 }
 
-Ref<CSSPrimitiveValue> CSSPrimitiveValue::create(const LengthSize& lengthSize, const RenderStyle& style)
-{
-    return create(Pair::create(create(lengthSize.width, style), create(lengthSize.height, style)));
-}
-
-Ref<CSSPrimitiveValue> CSSPrimitiveValue::create(Ref<CSSBasicShape>&& value)
+Ref<CSSPrimitiveValue> CSSPrimitiveValue::create(Ref<CSSCalcValue> value)
 {
     return adoptRef(*new CSSPrimitiveValue(WTFMove(value)));
 }
 
-Ref<CSSPrimitiveValue> CSSPrimitiveValue::create(Ref<CSSCalcValue>&& value)
+Ref<CSSPrimitiveValue> CSSPrimitiveValue::create(CSSUnresolvedColor value)
 {
     return adoptRef(*new CSSPrimitiveValue(WTFMove(value)));
 }
 
-Ref<CSSPrimitiveValue> CSSPrimitiveValue::create(Ref<Counter>&& value)
+Ref<CSSPrimitiveValue> CSSPrimitiveValue::createAttr(String value)
 {
-    return adoptRef(*new CSSPrimitiveValue(WTFMove(value)));
+    return adoptRef(*new CSSPrimitiveValue(WTFMove(value), CSSUnitType::CSS_ATTR));
 }
 
-Ref<CSSPrimitiveValue> CSSPrimitiveValue::create(Ref<Pair>&& value)
+Ref<CSSPrimitiveValue> CSSPrimitiveValue::createCounterName(String value)
 {
-    return adoptRef(*new CSSPrimitiveValue(WTFMove(value)));
+    return adoptRef(*new CSSPrimitiveValue(WTFMove(value), CSSUnitType::CSS_COUNTER_NAME));
 }
 
-Ref<CSSPrimitiveValue> CSSPrimitiveValue::create(Ref<Quad>&& value)
+Ref<CSSPrimitiveValue> CSSPrimitiveValue::createCustomIdent(String value)
 {
-    return adoptRef(*new CSSPrimitiveValue(WTFMove(value)));
+    return adoptRef(*new CSSPrimitiveValue(WTFMove(value), CSSUnitType::CustomIdent));
 }
 
-Ref<CSSPrimitiveValue> CSSPrimitiveValue::create(Ref<Rect>&& value)
+Ref<CSSPrimitiveValue> CSSPrimitiveValue::createFontFamily(String value)
 {
-    return adoptRef(*new CSSPrimitiveValue(WTFMove(value)));
+    return adoptRef(*new CSSPrimitiveValue(WTFMove(value), CSSUnitType::CSS_FONT_FAMILY));
 }
 
-Ref<CSSPrimitiveValue> CSSPrimitiveValue::create(Ref<CSSUnresolvedColor>&& value)
+Ref<CSSPrimitiveValue> CSSPrimitiveValue::createInteger(double value)
 {
-    return adoptRef(*new CSSPrimitiveValue(WTFMove(value)));
+    return adoptRef(*new CSSPrimitiveValue(value, CSSUnitType::CSS_INTEGER));
+}
+
+Ref<CSSPrimitiveValue> CSSPrimitiveValue::createURI(String value)
+{
+    return adoptRef(*new CSSPrimitiveValue(WTFMove(value), CSSUnitType::CSS_URI));
 }
 
 double CSSPrimitiveValue::computeDegrees() const
@@ -687,7 +637,7 @@ template<> LayoutUnit CSSPrimitiveValue::computeLength(const CSSToLengthConversi
 
 double CSSPrimitiveValue::computeLengthDouble(const CSSToLengthConversionData& conversionData) const
 {
-    if (primitiveUnitType() == CSSUnitType::CSS_CALC) {
+    if (isCalculated()) {
         // The multiplier and factor is applied to each value in the calc expression individually
         return m_value.calc->computeLengthPx(conversionData);
     }
@@ -955,25 +905,25 @@ double CSSPrimitiveValue::computeNonCalcLengthDouble(const CSSToLengthConversion
 
     case CSSUnitType::CSS_CQW: {
         if (auto* containerRenderer = selectContainerRenderer(CQ::Axis::Width))
-            return containerRenderer->width() * value / 100;
+            return containerRenderer->contentWidth() * value / 100;
         return value * conversionData.smallViewportFactor().width();
     }
 
     case CSSUnitType::CSS_CQH: {
         if (auto* containerRenderer = selectContainerRenderer(CQ::Axis::Height))
-            return containerRenderer->height() * value / 100;
+            return containerRenderer->contentHeight() * value / 100;
         return value * conversionData.smallViewportFactor().height();
     }
 
     case CSSUnitType::CSS_CQI: {
         if (auto* containerRenderer = selectContainerRenderer(CQ::Axis::Inline))
-            return containerRenderer->logicalWidth() * value / 100;
+            return containerRenderer->contentLogicalWidth() * value / 100;
         return value * lengthOfViewportPhysicalAxisForLogicalAxis(LogicalBoxAxis::Inline, conversionData.smallViewportFactor(), conversionData.rootStyle());
     }
 
     case CSSUnitType::CSS_CQB: {
         if (auto* containerRenderer = selectContainerRenderer(CQ::Axis::Block))
-            return containerRenderer->logicalHeight() * value / 100;
+            return containerRenderer->contentLogicalHeight() * value / 100;
         return value * lengthOfViewportPhysicalAxisForLogicalAxis(LogicalBoxAxis::Block, conversionData.smallViewportFactor(), conversionData.rootStyle());
     }
 
@@ -1099,49 +1049,39 @@ double CSSPrimitiveValue::doubleValue(CSSUnitType unitType) const
 
 double CSSPrimitiveValue::doubleValue() const
 {
-    return primitiveUnitType() != CSSUnitType::CSS_CALC ? m_value.number : m_value.calc->doubleValue();
+    return isCalculated() ? m_value.calc->doubleValue() : m_value.number;
 }
 
 double CSSPrimitiveValue::doubleValueDividingBy100IfPercentage() const
 {
-    switch (primitiveUnitType()) {
-    case CSSUnitType::CSS_CALC:
+    if (isCalculated())
         return m_value.calc->primitiveType() == CSSUnitType::CSS_PERCENTAGE ? m_value.calc->doubleValue() / 100.0 : m_value.calc->doubleValue();
-    
-    case CSSUnitType::CSS_PERCENTAGE:
+    if (isPercentage())
         return m_value.number / 100.0;
-        
-    default:
-        return m_value.number;
-    }
+    return m_value.number;
 }
 
 std::optional<bool> CSSPrimitiveValue::isZero() const
 {
-    if (primitiveUnitType() == CSSUnitType::CSS_CALC)
+    if (isCalculated())
         return std::nullopt;
     return !m_value.number;
 }
 
 std::optional<bool> CSSPrimitiveValue::isPositive() const
 {
-    if (primitiveUnitType() == CSSUnitType::CSS_CALC)
+    if (isCalculated())
         return std::nullopt;
     return m_value.number > 0;
 }
 
 std::optional<bool> CSSPrimitiveValue::isNegative() const
 {
-    if (primitiveUnitType() == CSSUnitType::CSS_CALC)
+    if (isCalculated())
         return std::nullopt;
     return m_value.number < 0;
 }
 
-bool CSSPrimitiveValue::isCenterPosition() const
-{
-    return valueID() == CSSValueCenter || doubleValue(CSSUnitType::CSS_PERCENTAGE) == 50;
-}
-
 std::optional<double> CSSPrimitiveValue::doubleValueInternal(CSSUnitType requestedUnitType) const
 {
     if (!isValidCSSUnitTypeForDoubleConversion(primitiveUnitType()) || !isValidCSSUnitTypeForDoubleConversion(requestedUnitType))
@@ -1314,22 +1254,16 @@ ASCIILiteral CSSPrimitiveValue::unitTypeString(CSSUnitType unitType)
     case CSSUnitType::CSS_CALC:
     case CSSUnitType::CSS_CALC_PERCENTAGE_WITH_LENGTH:
     case CSSUnitType::CSS_CALC_PERCENTAGE_WITH_NUMBER:
-    case CSSUnitType::CSS_COUNTER:
     case CSSUnitType::CSS_COUNTER_NAME:
     case CSSUnitType::CSS_DIMENSION:
     case CSSUnitType::CSS_FONT_FAMILY:
     case CSSUnitType::CSS_IDENT:
     case CSSUnitType::CSS_INTEGER:
     case CSSUnitType::CSS_NUMBER:
-    case CSSUnitType::CSS_PAIR:
     case CSSUnitType::CSS_PROPERTY_ID:
-    case CSSUnitType::CSS_QUAD:
     case CSSUnitType::CSS_QUIRKY_EMS:
-    case CSSUnitType::CSS_RECT:
     case CSSUnitType::CSS_RGBCOLOR:
-    case CSSUnitType::CSS_SHAPE:
     case CSSUnitType::CSS_STRING:
-    case CSSUnitType::CSS_UNICODE_RANGE:
     case CSSUnitType::CSS_UNKNOWN:
     case CSSUnitType::CSS_UNRESOLVED_COLOR:
     case CSSUnitType::CSS_URI:
@@ -1341,19 +1275,6 @@ ASCIILiteral CSSPrimitiveValue::unitTypeString(CSSUnitType unitType)
     return ""_s;
 }
 
-static String serialize(const Counter& counter)
-{
-    auto listStyleSeparator = counter.listStyle == CSSValueDecimal ? ""_s : ", "_s;
-    auto listStyleLiteral = counter.listStyle == CSSValueDecimal ? ""_s : nameLiteral(counter.listStyle);
-    if (counter.separator.isEmpty())
-        return makeString("counter("_s, counter.identifier, listStyleSeparator, listStyleLiteral, ')');
-    StringBuilder result;
-    result.append("counters("_s, counter.identifier, ", "_s);
-    serializeString(counter.separator, result);
-    result.append(listStyleSeparator, listStyleLiteral, ')');
-    return result.toString();
-}
-
 ALWAYS_INLINE String CSSPrimitiveValue::serializeInternal() const
 {
     auto type = primitiveUnitType();
@@ -1422,11 +1343,7 @@ ALWAYS_INLINE String CSSPrimitiveValue::serializeInternal() const
     case CSSUnitType::CSS_ATTR:
         return makeString("attr("_s, m_value.string, ')');
     case CSSUnitType::CSS_CALC:
-        if (!m_value.calc)
-            break;
         return m_value.calc->cssText();
-    case CSSUnitType::CSS_COUNTER:
-        return serialize(*m_value.counter);
     case CSSUnitType::CSS_COUNTER_NAME:
         return makeString("counter(", m_value.string, ')');
     case CSSUnitType::CSS_DIMENSION:
@@ -1436,18 +1353,10 @@ ALWAYS_INLINE String CSSPrimitiveValue::serializeInternal() const
         return serializeFontFamily(m_value.string);
     case CSSUnitType::CSS_INTEGER:
         return formatIntegerValue(""_s);
-    case CSSUnitType::CSS_PAIR:
-        return m_value.pair->cssText();
-    case CSSUnitType::CSS_QUAD:
-        return m_value.quad->cssText();
     case CSSUnitType::CSS_QUIRKY_EMS:
         return formatNumberValue("em"_s);
-    case CSSUnitType::CSS_RECT:
-        return m_value.rect->cssText();
     case CSSUnitType::CSS_RGBCOLOR:
         return serializationForCSS(color());
-    case CSSUnitType::CSS_SHAPE:
-        return m_value.shape->cssText();
     case CSSUnitType::CSS_STRING:
         return serializeString(m_value.string);
     case CSSUnitType::CSS_UNRESOLVED_COLOR:
@@ -1464,7 +1373,6 @@ ALWAYS_INLINE String CSSPrimitiveValue::serializeInternal() const
     case CSSUnitType::CSS_CALC_PERCENTAGE_WITH_NUMBER:
     case CSSUnitType::CSS_IDENT:
     case CSSUnitType::CSS_PROPERTY_ID:
-    case CSSUnitType::CSS_UNICODE_RANGE:
     case CSSUnitType::CSS_UNKNOWN:
     case CSSUnitType::CSS_VALUE_ID:
         break;
@@ -1576,24 +1484,13 @@ bool CSSPrimitiveValue::equals(const CSSPrimitiveValue& other) const
     case CSSUnitType::CSS_COUNTER_NAME:
     case CSSUnitType::CSS_FONT_FAMILY:
         return equal(m_value.string, other.m_value.string);
-    case CSSUnitType::CSS_COUNTER:
-        return m_value.counter->equals(*other.m_value.counter);
-    case CSSUnitType::CSS_RECT:
-        return m_value.rect->equals(*other.m_value.rect);
-    case CSSUnitType::CSS_QUAD:
-        return m_value.quad->equals(*other.m_value.quad);
     case CSSUnitType::CSS_RGBCOLOR:
         return color() == other.color();
-    case CSSUnitType::CSS_PAIR:
-        return m_value.pair->equals(*other.m_value.pair);
     case CSSUnitType::CSS_CALC:
-        return m_value.calc && other.m_value.calc && m_value.calc->equals(*other.m_value.calc);
-    case CSSUnitType::CSS_SHAPE:
-        return m_value.shape->equals(*other.m_value.shape);
+        return m_value.calc->equals(*other.m_value.calc);
     case CSSUnitType::CSS_UNRESOLVED_COLOR:
         return m_value.unresolvedColor->equals(*other.m_value.unresolvedColor);
     case CSSUnitType::CSS_IDENT:
-    case CSSUnitType::CSS_UNICODE_RANGE:
     case CSSUnitType::CSS_CALC_PERCENTAGE_WITH_NUMBER:
     case CSSUnitType::CSS_CALC_PERCENTAGE_WITH_LENGTH:
         // FIXME: seems like these should be handled.
@@ -1603,11 +1500,6 @@ bool CSSPrimitiveValue::equals(const CSSPrimitiveValue& other) const
     return false;
 }
 
-Ref<DeprecatedCSSOMPrimitiveValue> CSSPrimitiveValue::createDeprecatedCSSOMPrimitiveWrapper(CSSStyleDeclaration& styleDeclaration) const
-{
-    return DeprecatedCSSOMPrimitiveValue::create(*this, styleDeclaration);
-}
-
 // https://drafts.css-houdini.org/css-properties-values-api/#dependency-cycles
 void CSSPrimitiveValue::collectComputedStyleDependencies(ComputedStyleDependencies& dependencies) const
 {
@@ -1696,14 +1588,8 @@ void CSSPrimitiveValue::collectComputedStyleDependencies(ComputedStyleDependenci
     case CSSUnitType::CSS_IDENT:
     case CSSUnitType::CustomIdent:
     case CSSUnitType::CSS_ATTR:
-    case CSSUnitType::CSS_COUNTER:
-    case CSSUnitType::CSS_RECT:
     case CSSUnitType::CSS_RGBCOLOR:
-    case CSSUnitType::CSS_PAIR:
-    case CSSUnitType::CSS_UNICODE_RANGE:
     case CSSUnitType::CSS_COUNTER_NAME:
-    case CSSUnitType::CSS_SHAPE:
-    case CSSUnitType::CSS_QUAD:
     case CSSUnitType::CSS_CALC_PERCENTAGE_WITH_NUMBER:
     case CSSUnitType::CSS_CALC_PERCENTAGE_WITH_LENGTH:
     case CSSUnitType::CSS_UNRESOLVED_COLOR:
