@@ -60,22 +60,13 @@ CSSSelector::CSSSelector(const QualifiedName& tagQName, bool tagIsForNamespaceRu
     , m_match(Tag)
     , m_tagIsForNamespaceRule(tagIsForNamespaceRule)
 {
-    const AtomString& tagLocalName = tagQName.localName();
-    const AtomString tagLocalNameASCIILowercase = tagLocalName.convertToASCIILowercase();
-
-    if (tagLocalName == tagLocalNameASCIILowercase) {
-        m_data.tagQName = tagQName.impl();
-        m_data.tagQName->ref();
-    } else {
-        m_data.nameWithCase = adoptRef(new NameWithCase(tagQName, tagLocalNameASCIILowercase)).leakRef();
-        m_hasNameWithCase = true;
-    }
+    m_data.tagQName = tagQName.impl();
+    m_data.tagQName->ref();
 }
 
 void CSSSelector::createRareData()
 {
     ASSERT(match() != Tag);
-    ASSERT(!m_hasNameWithCase);
     if (m_hasRareData)
         return;
     // Move the value to the rare data stucture.
@@ -319,34 +310,6 @@ const CSSSelector* CSSSelector::firstInCompound() const
     return selector;
 }
 
-bool CSSSelector::operator==(const CSSSelector& other) const
-{
-    const CSSSelector* sel1 = this;
-    const CSSSelector* sel2 = &other;
-
-    while (sel1 && sel2) {
-        if (sel1->attribute() != sel2->attribute()
-            || sel1->relation() != sel2->relation()
-            || sel1->match() != sel2->match()
-            || sel1->value() != sel2->value()
-            || sel1->m_pseudoType != sel2->m_pseudoType
-            || sel1->argument() != sel2->argument()) {
-            return false;
-        }
-        if (sel1->match() == Tag) {
-            if (sel1->tagQName() != sel2->tagQName())
-                return false;
-        }
-        sel1 = sel1->tagHistory();
-        sel2 = sel2->tagHistory();
-    }
-
-    if (sel1 || sel2)
-        return false;
-
-    return true;
-}
-
 static void appendPseudoClassFunctionTail(StringBuilder& builder, const CSSSelector* selector)
 {
     switch (selector->pseudoClassType()) {
@@ -416,11 +379,19 @@ String CSSSelector::selectorText(StringView separator, StringView rightSide) con
 {
     StringBuilder builder;
 
-    if (match() == CSSSelector::Tag && !m_tagIsForNamespaceRule) {
-        if (tagQName().prefix().isNull())
-            builder.append(tagQName().localName());
+    auto serializeIdentifierOrStar = [&] (const AtomString& identifier) {
+        if (identifier == starAtom())
+            builder.append('*');
         else
-            builder.append(tagQName().prefix().string(), '|', tagQName().localName());
+            serializeIdentifier(identifier, builder);
+    };
+
+    if (match() == CSSSelector::Tag && !m_tagIsForNamespaceRule) {
+        if (auto& prefix = tagQName().prefix(); !prefix.isNull()) {
+            serializeIdentifierOrStar(prefix);
+            builder.append('|');
+        }
+        serializeIdentifierOrStar(tagQName().localName());
     }
 
     const CSSSelector* cs = this;
@@ -651,6 +622,9 @@ String CSSSelector::selectorText(StringView separator, StringView rightSide) con
             case CSSSelector::PseudoClassOnlyOfType:
                 builder.append(":only-of-type");
                 break;
+            case CSSSelector::PseudoClassPopoverOpen:
+                builder.append(":popover-open");
+                break;
             case CSSSelector::PseudoClassOptional:
                 builder.append(":optional");
                 break;
@@ -786,9 +760,11 @@ String CSSSelector::selectorText(StringView separator, StringView rightSide) con
             }
         } else if (cs->isAttributeSelector()) {
             builder.append('[');
-            if (auto& prefix = cs->attribute().prefix(); !prefix.isEmpty())
-                builder.append(prefix, '|');
-            builder.append(cs->attribute().localName());
+            if (auto& prefix = cs->attribute().prefix(); !prefix.isEmpty()) {
+                serializeIdentifierOrStar(prefix);
+                builder.append('|');
+            }
+            serializeIdentifierOrStar(cs->attribute().localName());
             switch (cs->match()) {
                 case CSSSelector::Exact:
                     builder.append('=');
@@ -843,6 +819,19 @@ String CSSSelector::selectorText(StringView separator, StringView rightSide) con
 
     builder.append(separator, rightSide);
 
+    auto separatorTextForNestingRelative = [&] () -> String {
+        switch (cs->relation()) {
+        case CSSSelector::Child:
+            return "> "_s;
+        case CSSSelector::DirectAdjacent:
+            return "+ "_s;
+        case CSSSelector::IndirectAdjacent:
+            return "~ "_s;
+        default:
+            return { };
+        }
+    };
+
     if (auto* previousSelector = cs->tagHistory()) {
         ASCIILiteral separator = ""_s;
         switch (cs->relation()) {
@@ -867,16 +856,18 @@ String CSSSelector::selectorText(StringView separator, StringView rightSide) con
             break;
         }
         return previousSelector->selectorText(separator, builder);
+    } else if (auto separatorText = separatorTextForNestingRelative(); !separatorText.isNull()) {
+        // We have a separator but no tag history which can happen with implicit relative nesting selector
+        return separatorText + builder.toString();
     }
 
     return builder.toString();
 }
 
-void CSSSelector::setAttribute(const QualifiedName& value, bool convertToLowercase, AttributeMatchType matchType)
+void CSSSelector::setAttribute(const QualifiedName& value, AttributeMatchType matchType)
 {
     createRareData();
     m_data.rareData->attribute = value;
-    m_data.rareData->attributeCanonicalLocalName = convertToLowercase ? value.localName().convertToASCIILowercase() : value.localName();
     m_caseInsensitiveAttributeValueMatching = matchType == CaseInsensitive;
 }
     
@@ -936,7 +927,6 @@ CSSSelector::RareData::RareData(const RareData& other)
     , a(other.a)
     , b(other.b)
     , attribute(other.attribute)
-    , attributeCanonicalLocalName(other.attributeCanonicalLocalName)
     , argument(other.argument)
     , argumentList(other.argumentList)
 {
@@ -973,19 +963,14 @@ CSSSelector::CSSSelector(const CSSSelector& other)
     , m_isFirstInTagHistory(other.m_isFirstInTagHistory)
     , m_isLastInTagHistory(other.m_isLastInTagHistory)
     , m_hasRareData(other.m_hasRareData)
-    , m_hasNameWithCase(other.m_hasNameWithCase)
     , m_isForPage(other.m_isForPage)
     , m_tagIsForNamespaceRule(other.m_tagIsForNamespaceRule)
     , m_caseInsensitiveAttributeValueMatching(other.m_caseInsensitiveAttributeValueMatching)
 {
-    if (other.m_hasRareData) {
-        auto copied = other.m_data.rareData->deepCopy(); 
-        m_data.rareData = &copied.leakRef();
-        m_data.rareData->ref();
-    } else if (other.m_hasNameWithCase) {
-        m_data.nameWithCase = other.m_data.nameWithCase;
-        m_data.nameWithCase->ref();
-    } else if (other.match() == Tag) {
+    // Manually ref count the m_data union because they are stored as raw ptr, not as Ref.
+    if (other.m_hasRareData)
+        m_data.rareData = &other.m_data.rareData->deepCopy().leakRef();
+    else if (other.match() == Tag) {
         m_data.tagQName = other.m_data.tagQName;
         m_data.tagQName->ref();
     } else if (other.m_data.value) {
@@ -1027,18 +1012,12 @@ void CSSSelector::resolveNestingParentSelectors(const CSSSelectorList& parent)
     visitAllSimpleSelectors(replaceParentSelector);
 }
 
-void CSSSelector::replaceNestingParentByNotAll()
+void CSSSelector::replaceNestingParentByPseudoClassScope()
 {
     auto replaceParentSelector = [] (CSSSelector& selector) {
         if (selector.match() == CSSSelector::PseudoClass && selector.pseudoClassType() == CSSSelector::PseudoClassNestingParent) {
-            // We replace by :not(*)
-            auto allSelector = makeUnique<CSSParserSelector>(CSSSelector(anyQName()));
-            Vector<std::unique_ptr<CSSParserSelector>> vector;
-            vector.append(WTFMove(allSelector));
-            auto selectorList = makeUnique<CSSSelectorList>(WTFMove(vector));
-            selector.setMatch(Match::PseudoClass);
-            selector.setPseudoClassType(PseudoClassType::PseudoClassNot);
-            selector.setSelectorList(WTFMove(selectorList));
+            // Replace by :scope
+            selector.setPseudoClassType(PseudoClassType::PseudoClassScope);
         }
     };
 
