@@ -1,7 +1,7 @@
 /*
  * Copyright (C) 1999-2003 Lars Knoll (knoll@kde.org)
  *               1999 Waldo Bastian (bastian@kde.org)
- * Copyright (C) 2004, 2006, 2007, 2008, 2009, 2010, 2013, 2014 Apple Inc. All rights reserved.
+ * Copyright (C) 2004-2022 Apple Inc. All rights reserved.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -23,9 +23,18 @@
 
 #include "QualifiedName.h"
 #include "RenderStyleConstants.h"
+#include <wtf/FixedVector.h>
 
 namespace WebCore {
-    class CSSSelectorList;
+
+class CSSSelectorList;
+
+struct PossiblyQuotedIdentifier {
+    AtomString identifier;
+    bool wasQuoted { false };
+
+    bool isNull() const { return identifier.isNull(); }
+};
 
     enum class SelectorSpecificityIncrement {
         ClassA = 0x10000,
@@ -33,35 +42,31 @@ namespace WebCore {
         ClassC = 1
     };
 
-    // this class represents a selector for a StyleRule
+    // Selector for a StyleRule.
+    DECLARE_ALLOCATOR_WITH_HEAP_IDENTIFIER(CSSSelectorRareData);
     class CSSSelector {
         WTF_MAKE_FAST_ALLOCATED;
     public:
-        CSSSelector();
+        CSSSelector() = default;
         CSSSelector(const CSSSelector&);
         explicit CSSSelector(const QualifiedName&, bool tagIsForNamespaceRule = false);
 
         ~CSSSelector();
 
-        /**
-         * Re-create selector text from selector's data
-         */
-        String selectorText(const String& = emptyString()) const;
+        // Re-create selector text from selector's data.
+        String selectorText(StringView separator = { }, StringView rightSide = { }) const;
 
-        // checks if the 2 selectors (including sub selectors) agree.
-        bool operator==(const CSSSelector&) const;
-
-        static const unsigned maxValueMask = 0xffffff;
-        static const unsigned idMask = 0xff0000;
-        static const unsigned classMask = 0xff00;
-        static const unsigned elementMask = 0xff;
-
-        unsigned staticSpecificity(bool& ok) const;
+        unsigned computeSpecificity() const;
+        std::array<uint8_t, 3> computeSpecificityTuple() const;
         unsigned specificityForPage() const;
-        unsigned simpleSelectorSpecificity() const;
-        static unsigned addSpecificities(unsigned, unsigned);
 
-        /* how the attribute value has to match.... Default is Exact */
+        void visitAllSimpleSelectors(auto& apply) const;
+
+        bool hasExplicitNestingParent() const;
+        void resolveNestingParentSelectors(const CSSSelectorList& parent);
+        void replaceNestingParentByPseudoClassScope();
+
+        // How the attribute value has to match. Default is Exact.
         enum Match {
             Unknown = 0,
             Tag,
@@ -85,7 +90,9 @@ namespace WebCore {
             Child,
             DirectAdjacent,
             IndirectAdjacent,
-            ShadowDescendant
+            ShadowDescendant,
+            ShadowPartDescendant,
+            ShadowSlotted
         };
 
         enum PseudoClassType {
@@ -107,12 +114,13 @@ namespace WebCore {
             PseudoClassAnyLink,
             PseudoClassAnyLinkDeprecated,
             PseudoClassAutofill,
+            PseudoClassAutofillAndObscured,
             PseudoClassAutofillStrongPassword,
             PseudoClassAutofillStrongPasswordViewable,
             PseudoClassHover,
-            PseudoClassDirectFocus,
             PseudoClassDrag,
             PseudoClassFocus,
+            PseudoClassFocusVisible,
             PseudoClassFocusWithin,
             PseudoClassActive,
             PseudoClassChecked,
@@ -120,7 +128,9 @@ namespace WebCore {
             PseudoClassFullPageMedia,
             PseudoClassDefault,
             PseudoClassDisabled,
-            PseudoClassMatches,
+            PseudoClassIs,
+            PseudoClassMatches, // obsolete synonym for PseudoClassIs
+            PseudoClassWhere,
             PseudoClassOptional,
             PseudoClassPlaceholderShown,
             PseudoClassRequired,
@@ -134,10 +144,12 @@ namespace WebCore {
             PseudoClassNot,
             PseudoClassRoot,
             PseudoClassScope,
+            PseudoClassRelativeScope, // Like :scope but for internal use with relative selectors like :has(> foo).
             PseudoClassWindowInactive,
             PseudoClassCornerPresent,
             PseudoClassDecrement,
             PseudoClassIncrement,
+            PseudoClassHas,
             PseudoClassHorizontal,
             PseudoClassVertical,
             PseudoClassStart,
@@ -146,7 +158,8 @@ namespace WebCore {
             PseudoClassSingleButton,
             PseudoClassNoButton,
 #if ENABLE(FULLSCREEN_API)
-            PseudoClassFullScreen,
+            PseudoClassFullscreen,
+            PseudoClassWebkitFullScreen,
             PseudoClassFullScreenDocument,
             PseudoClassFullScreenAncestor,
             PseudoClassAnimatingFullScreenTransition,
@@ -157,30 +170,41 @@ namespace WebCore {
 #endif
             PseudoClassInRange,
             PseudoClassOutOfRange,
-#if ENABLE(VIDEO_TRACK)
+#if ENABLE(VIDEO)
             PseudoClassFuture,
             PseudoClassPast,
+            PseudoClassPlaying,
+            PseudoClassPaused,
+            PseudoClassSeeking,
+            PseudoClassBuffering,
+            PseudoClassStalled,
+            PseudoClassMuted,
+            PseudoClassVolumeLocked,
 #endif
-#if ENABLE(CSS_SELECTORS_LEVEL4)
             PseudoClassDir,
-            PseudoClassRole,
-#endif
             PseudoClassHost,
             PseudoClassDefined,
 #if ENABLE(ATTACHMENT_ELEMENT)
             PseudoClassHasAttachment,
 #endif
+            PseudoClassModal,
+            PseudoClassPopoverOpen,
+            PseudoClassUserInvalid,
+            PseudoClassUserValid,
+            PseudoClassNestingParent,
         };
 
         enum PseudoElementType {
             PseudoElementUnknown = 0,
             PseudoElementAfter,
+            PseudoElementBackdrop,
             PseudoElementBefore,
-#if ENABLE(VIDEO_TRACK)
+#if ENABLE(VIDEO)
             PseudoElementCue,
 #endif
             PseudoElementFirstLetter,
             PseudoElementFirstLine,
+            PseudoElementHighlight,
             PseudoElementMarker,
             PseudoElementPart,
             PseudoElementResizer,
@@ -224,17 +248,13 @@ namespace WebCore {
             RightBottomMarginBox,
         };
 
-        enum AttributeMatchType {
-            CaseSensitive,
-            CaseInsensitive,
-        };
-
         static PseudoElementType parsePseudoElementType(StringView);
         static PseudoId pseudoId(PseudoElementType);
 
-        // Selectors are kept in an array by CSSSelectorList. The next component of the selector is
-        // the next item in the array.
-        const CSSSelector* tagHistory() const { return m_isLastInTagHistory ? 0 : const_cast<CSSSelector*>(this + 1); }
+        // Selectors are kept in an array by CSSSelectorList.
+        // The next component of the selector is the next item in the array.
+        const CSSSelector* tagHistory() const { return m_isLastInTagHistory ? nullptr : this + 1; }
+        const CSSSelector* firstInCompound() const;
 
         const QualifiedName& tagQName() const;
         const AtomString& tagLowercaseLocalName() const;
@@ -242,18 +262,19 @@ namespace WebCore {
         const AtomString& value() const;
         const AtomString& serializingValue() const;
         const QualifiedName& attribute() const;
-        const AtomString& attributeCanonicalLocalName() const;
-        const AtomString& argument() const { return m_hasRareData ? m_data.m_rareData->m_argument : nullAtom(); }
+        const AtomString& argument() const { return m_hasRareData ? m_data.rareData->argument : nullAtom(); }
         bool attributeValueMatchingIsCaseInsensitive() const;
-        const Vector<AtomString>* argumentList() const { return m_hasRareData ? m_data.m_rareData->m_argumentList.get() : nullptr; }
-        const CSSSelectorList* selectorList() const { return m_hasRareData ? m_data.m_rareData->m_selectorList.get() : nullptr; }
+        const FixedVector<PossiblyQuotedIdentifier>* argumentList() const { return m_hasRareData ? &m_data.rareData->argumentList : nullptr; }
+        const CSSSelectorList* selectorList() const { return m_hasRareData ? m_data.rareData->selectorList.get() : nullptr; }
+        CSSSelectorList* selectorList() { return m_hasRareData ? m_data.rareData->selectorList.get() : nullptr; }
 
         void setValue(const AtomString&, bool matchLowerCase = false);
-        
-        void setAttribute(const QualifiedName&, bool convertToLowercase, AttributeMatchType);
+
+        enum AttributeMatchType { CaseSensitive, CaseInsensitive };
+        void setAttribute(const QualifiedName&, AttributeMatchType);
         void setNth(int a, int b);
         void setArgument(const AtomString&);
-        void setArgumentList(std::unique_ptr<Vector<AtomString>>);
+        void setArgumentList(FixedVector<PossiblyQuotedIdentifier>);
         void setSelectorList(std::unique_ptr<CSSSelectorList>);
 
         bool matchNth(int count) const;
@@ -264,38 +285,14 @@ namespace WebCore {
 
         bool hasDescendantOrChildRelation() const { return relation() == Child || hasDescendantRelation(); }
 
-        PseudoClassType pseudoClassType() const
-        {
-            ASSERT(match() == PseudoClass);
-            return static_cast<PseudoClassType>(m_pseudoType);
-        }
-        void setPseudoClassType(PseudoClassType pseudoType)
-        {
-            m_pseudoType = pseudoType;
-            ASSERT(m_pseudoType == pseudoType);
-        }
+        PseudoClassType pseudoClassType() const;
+        void setPseudoClassType(PseudoClassType);
 
-        PseudoElementType pseudoElementType() const
-        {
-            ASSERT(match() == PseudoElement);
-            return static_cast<PseudoElementType>(m_pseudoType);
-        }
-        void setPseudoElementType(PseudoElementType pseudoElementType)
-        {
-            m_pseudoType = pseudoElementType;
-            ASSERT(m_pseudoType == pseudoElementType);
-        }
+        PseudoElementType pseudoElementType() const;
+        void setPseudoElementType(PseudoElementType);
 
-        PagePseudoClassType pagePseudoClassType() const
-        {
-            ASSERT(match() == PagePseudoClass);
-            return static_cast<PagePseudoClassType>(m_pseudoType);
-        }
-        void setPagePseudoType(PagePseudoClassType pagePseudoType)
-        {
-            m_pseudoType = pagePseudoType;
-            ASSERT(m_pseudoType == pagePseudoType);
-        }
+        PagePseudoClassType pagePseudoClassType() const;
+        void setPagePseudoType(PagePseudoClassType);
 
         bool matchesPseudoElement() const;
         bool isUnknownPseudoElement() const;
@@ -305,105 +302,89 @@ namespace WebCore {
         bool isAttributeSelector() const;
 
         RelationType relation() const { return static_cast<RelationType>(m_relation); }
-        void setRelation(RelationType relation)
-        {
-            m_relation = relation;
-            ASSERT(m_relation == relation);
-        }
+        void setRelation(RelationType);
 
         Match match() const { return static_cast<Match>(m_match); }
-        void setMatch(Match match)
-        {
-            m_match = match;
-            ASSERT(m_match == match);
-        }
+        void setMatch(Match);
 
         bool isLastInSelectorList() const { return m_isLastInSelectorList; }
         void setLastInSelectorList() { m_isLastInSelectorList = true; }
+        void setNotLastInSelectorList() { m_isLastInSelectorList = false; }
+
+        bool isFirstInTagHistory() const { return m_isFirstInTagHistory; }
+        void setNotFirstInTagHistory() { m_isFirstInTagHistory = false; }
+
         bool isLastInTagHistory() const { return m_isLastInTagHistory; }
         void setNotLastInTagHistory() { m_isLastInTagHistory = false; }
+        void setLastInTagHistory() { m_isLastInTagHistory = true; }
 
         bool isForPage() const { return m_isForPage; }
         void setForPage() { m_isForPage = true; }
 
     private:
-        unsigned m_relation              : 4; // enum RelationType.
-        mutable unsigned m_match         : 4; // enum Match.
-        mutable unsigned m_pseudoType    : 8; // PseudoType.
-        unsigned m_isLastInSelectorList  : 1;
-        unsigned m_isLastInTagHistory    : 1;
-        unsigned m_hasRareData           : 1;
-        unsigned m_hasNameWithCase       : 1;
-        unsigned m_isForPage             : 1;
-        unsigned m_tagIsForNamespaceRule : 1;
-        unsigned m_caseInsensitiveAttributeValueMatching : 1;
+        unsigned m_relation : 4 { DescendantSpace }; // enum RelationType.
+        mutable unsigned m_match : 4 { Unknown }; // enum Match.
+        mutable unsigned m_pseudoType : 8 { 0 }; // PseudoType.
+        unsigned m_isLastInSelectorList : 1 { false };
+        unsigned m_isFirstInTagHistory : 1 { true };
+        unsigned m_isLastInTagHistory : 1 { true };
+        unsigned m_hasRareData : 1 { false };
+        unsigned m_isForPage : 1 { false };
+        unsigned m_tagIsForNamespaceRule : 1 { false };
+        unsigned m_caseInsensitiveAttributeValueMatching : 1 { false };
 #if !ASSERT_WITH_SECURITY_IMPLICATION_DISABLED
-        unsigned m_destructorHasBeenCalled : 1;
+        unsigned m_destructorHasBeenCalled : 1 { false };
 #endif
 
         unsigned simpleSelectorSpecificityForPage() const;
+        CSSSelector* tagHistory() { return m_isLastInTagHistory ? nullptr : this + 1; }
 
-        // Hide.
-        CSSSelector& operator=(const CSSSelector&);
+        CSSSelector& operator=(const CSSSelector&) = delete;
+        CSSSelector(CSSSelector&&) = delete;
 
         struct RareData : public RefCounted<RareData> {
-            static Ref<RareData> create(AtomString&& value) { return adoptRef(*new RareData(WTFMove(value))); }
-            ~RareData();
+            WTF_MAKE_STRUCT_FAST_ALLOCATED_WITH_HEAP_IDENTIFIER(CSSSelectorRareData);
+            static Ref<RareData> create(AtomString);
+            WEBCORE_EXPORT ~RareData();
 
             bool matchNth(int count);
 
             // For quirks mode, class and id are case-insensitive. In the case where uppercase
             // letters are used in quirks mode, |m_matchingValue| holds the lowercase class/id
             // and |m_serializingValue| holds the original string.
-            AtomString m_matchingValue;
-            AtomString m_serializingValue;
-            
-            int m_a; // Used for :nth-*
-            int m_b; // Used for :nth-*
-            QualifiedName m_attribute; // used for attribute selector
-            AtomString m_attributeCanonicalLocalName;
-            AtomString m_argument; // Used for :contains and :nth-*
-            std::unique_ptr<Vector<AtomString>> m_argumentList; // Used for :lang and ::part arguments.
-            std::unique_ptr<CSSSelectorList> m_selectorList; // Used for :matches() and :not().
-        
+            AtomString matchingValue;
+            AtomString serializingValue;
+
+            int a { 0 }; // Used for :nth-*
+            int b { 0 }; // Used for :nth-*
+            QualifiedName attribute; // used for attribute selector
+            AtomString argument; // Used for :contains and :nth-*
+            FixedVector<PossiblyQuotedIdentifier> argumentList; // Used for :lang and ::part arguments.
+            std::unique_ptr<CSSSelectorList> selectorList; // Used for :is(), :matches(), and :not().
+
+            Ref<RareData> deepCopy() const;
+
         private:
             RareData(AtomString&& value);
+            RareData(const RareData& other);
         };
         void createRareData();
 
-        struct NameWithCase : public RefCounted<NameWithCase> {
-            NameWithCase(const QualifiedName& originalName, const AtomString& lowercaseName)
-                : m_originalName(originalName)
-                , m_lowercaseLocalName(lowercaseName)
-            {
-                ASSERT(originalName.localName() != lowercaseName);
-            }
-
-            const QualifiedName m_originalName;
-            const AtomString m_lowercaseLocalName;
-        };
-
         union DataUnion {
-            DataUnion() : m_value(0) { }
-            AtomStringImpl* m_value;
-            QualifiedName::QualifiedNameImpl* m_tagQName;
-            RareData* m_rareData;
-            NameWithCase* m_nameWithCase;
+            AtomStringImpl* value { nullptr };
+            QualifiedName::QualifiedNameImpl* tagQName;
+            RareData* rareData;
         } m_data;
     };
 
+inline bool operator==(const AtomString& a, const PossiblyQuotedIdentifier& b) { return a == b.identifier; }
+inline bool operator==(const PossiblyQuotedIdentifier& a, const AtomString& b) { return a.identifier == b; }
+
 inline const QualifiedName& CSSSelector::attribute() const
 {
     ASSERT(isAttributeSelector());
     ASSERT(m_hasRareData);
-    return m_data.m_rareData->m_attribute;
-}
-
-inline const AtomString& CSSSelector::attributeCanonicalLocalName() const
-{
-    ASSERT(isAttributeSelector());
-    ASSERT(m_hasRareData);
-    return m_data.m_rareData->m_attributeCanonicalLocalName;
+    return m_data.rareData->attribute;
 }
 
 inline bool CSSSelector::matchesPseudoElement() const
@@ -448,6 +429,21 @@ static inline bool isTreeStructuralPseudoClass(CSSSelector::PseudoClassType type
     return pseudoClassIsRelativeToSiblings(type) || type == CSSSelector::PseudoClassRoot;
 }
 
+inline bool isLogicalCombinationPseudoClass(CSSSelector::PseudoClassType pseudoClassType)
+{
+    switch (pseudoClassType) {
+    case CSSSelector::PseudoClassIs:
+    case CSSSelector::PseudoClassWhere:
+    case CSSSelector::PseudoClassNot:
+    case CSSSelector::PseudoClassAny:
+    case CSSSelector::PseudoClassMatches:
+    case CSSSelector::PseudoClassHas:
+        return true;
+    default:
+        return false;
+    }
+}
+
 inline bool CSSSelector::isSiblingSelector() const
 {
     return relation() == DirectAdjacent
@@ -458,7 +454,7 @@ inline bool CSSSelector::isSiblingSelector() const
 inline bool CSSSelector::isAttributeSelector() const
 {
     return match() == CSSSelector::Exact
-        || match() ==  CSSSelector::Set
+        || match() == CSSSelector::Set
         || match() == CSSSelector::List
         || match() == CSSSelector::Hyphen
         || match() == CSSSelector::Contain
@@ -475,62 +471,15 @@ inline void CSSSelector::setValue(const AtomString& value, bool matchLowerCase)
     
     // Need to do ref counting manually for the union.
     if (!m_hasRareData) {
-        if (m_data.m_value)
-            m_data.m_value->deref();
-        m_data.m_value = value.impl();
-        m_data.m_value->ref();
+        if (m_data.value)
+            m_data.value->deref();
+        m_data.value = value.impl();
+        m_data.value->ref();
         return;
     }
 
-    m_data.m_rareData->m_matchingValue = WTFMove(matchingValue);
-    m_data.m_rareData->m_serializingValue = value;
-}
-
-inline CSSSelector::CSSSelector()
-    : m_relation(DescendantSpace)
-    , m_match(Unknown)
-    , m_pseudoType(0)
-    , m_isLastInSelectorList(false)
-    , m_isLastInTagHistory(true)
-    , m_hasRareData(false)
-    , m_hasNameWithCase(false)
-    , m_isForPage(false)
-    , m_tagIsForNamespaceRule(false)
-    , m_caseInsensitiveAttributeValueMatching(false)
-#if !ASSERT_WITH_SECURITY_IMPLICATION_DISABLED
-    , m_destructorHasBeenCalled(false)
-#endif
-{
-}
-
-inline CSSSelector::CSSSelector(const CSSSelector& o)
-    : m_relation(o.m_relation)
-    , m_match(o.m_match)
-    , m_pseudoType(o.m_pseudoType)
-    , m_isLastInSelectorList(o.m_isLastInSelectorList)
-    , m_isLastInTagHistory(o.m_isLastInTagHistory)
-    , m_hasRareData(o.m_hasRareData)
-    , m_hasNameWithCase(o.m_hasNameWithCase)
-    , m_isForPage(o.m_isForPage)
-    , m_tagIsForNamespaceRule(o.m_tagIsForNamespaceRule)
-    , m_caseInsensitiveAttributeValueMatching(o.m_caseInsensitiveAttributeValueMatching)
-#if !ASSERT_WITH_SECURITY_IMPLICATION_DISABLED
-    , m_destructorHasBeenCalled(false)
-#endif
-{
-    if (o.m_hasRareData) {
-        m_data.m_rareData = o.m_data.m_rareData;
-        m_data.m_rareData->ref();
-    } else if (o.m_hasNameWithCase) {
-        m_data.m_nameWithCase = o.m_data.m_nameWithCase;
-        m_data.m_nameWithCase->ref();
-    } if (o.match() == Tag) {
-        m_data.m_tagQName = o.m_data.m_tagQName;
-        m_data.m_tagQName->ref();
-    } else if (o.m_data.m_value) {
-        m_data.m_value = o.m_data.m_value;
-        m_data.m_value->ref();
-    }
+    m_data.rareData->matchingValue = WTFMove(matchingValue);
+    m_data.rareData->serializingValue = value;
 }
 
 inline CSSSelector::~CSSSelector()
@@ -540,61 +489,100 @@ inline CSSSelector::~CSSSelector()
     m_destructorHasBeenCalled = true;
 #endif
     if (m_hasRareData) {
-        m_data.m_rareData->deref();
-        m_data.m_rareData = nullptr;
+        m_data.rareData->deref();
+        m_data.rareData = nullptr;
         m_hasRareData = false;
-    } else if (m_hasNameWithCase) {
-        m_data.m_nameWithCase->deref();
-        m_data.m_nameWithCase = nullptr;
-        m_hasNameWithCase = false;
     } else if (match() == Tag) {
-        m_data.m_tagQName->deref();
-        m_data.m_tagQName = nullptr;
+        m_data.tagQName->deref();
+        m_data.tagQName = nullptr;
         m_match = Unknown;
-    } else if (m_data.m_value) {
-        m_data.m_value->deref();
-        m_data.m_value = nullptr;
+    } else if (m_data.value) {
+        m_data.value->deref();
+        m_data.value = nullptr;
     }
 }
 
 inline const QualifiedName& CSSSelector::tagQName() const
 {
-    ASSERT(match() == Tag);
-    if (m_hasNameWithCase)
-        return m_data.m_nameWithCase->m_originalName;
-    return *reinterpret_cast<const QualifiedName*>(&m_data.m_tagQName);
+    return *reinterpret_cast<const QualifiedName*>(&m_data.tagQName);
 }
 
 inline const AtomString& CSSSelector::tagLowercaseLocalName() const
 {
-    if (m_hasNameWithCase)
-        return m_data.m_nameWithCase->m_lowercaseLocalName;
-    return m_data.m_tagQName->m_localName;
+    return tagQName().localNameLowercase();
 }
 
 inline const AtomString& CSSSelector::value() const
 {
     ASSERT(match() != Tag);
     if (m_hasRareData)
-        return m_data.m_rareData->m_matchingValue;
+        return m_data.rareData->matchingValue;
 
     // AtomString is really just an AtomStringImpl* so the cast below is safe.
-    return *reinterpret_cast<const AtomString*>(&m_data.m_value);
+    return *reinterpret_cast<const AtomString*>(&m_data.value);
 }
 
 inline const AtomString& CSSSelector::serializingValue() const
 {
     ASSERT(match() != Tag);
     if (m_hasRareData)
-        return m_data.m_rareData->m_serializingValue;
+        return m_data.rareData->serializingValue;
     
     // AtomString is really just an AtomStringImpl* so the cast below is safe.
-    return *reinterpret_cast<const AtomString*>(&m_data.m_value);
+    return *reinterpret_cast<const AtomString*>(&m_data.value);
 }
-    
+
 inline bool CSSSelector::attributeValueMatchingIsCaseInsensitive() const
 {
     return m_caseInsensitiveAttributeValueMatching;
 }
 
+inline auto CSSSelector::pseudoClassType() const -> PseudoClassType
+{
+    ASSERT(match() == PseudoClass);
+    return static_cast<PseudoClassType>(m_pseudoType);
+}
+
+inline void CSSSelector::setPseudoClassType(PseudoClassType pseudoType)
+{
+    m_pseudoType = pseudoType;
+    ASSERT(m_pseudoType == pseudoType);
+}
+
+inline auto CSSSelector::pseudoElementType() const -> PseudoElementType
+{
+    ASSERT(match() == PseudoElement);
+    return static_cast<PseudoElementType>(m_pseudoType);
+}
+
+inline void CSSSelector::setPseudoElementType(PseudoElementType pseudoElementType)
+{
+    m_pseudoType = pseudoElementType;
+    ASSERT(m_pseudoType == pseudoElementType);
+}
+
+inline auto CSSSelector::pagePseudoClassType() const -> PagePseudoClassType
+{
+    ASSERT(match() == PagePseudoClass);
+    return static_cast<PagePseudoClassType>(m_pseudoType);
+}
+
+inline void CSSSelector::setPagePseudoType(PagePseudoClassType pagePseudoType)
+{
+    m_pseudoType = pagePseudoType;
+    ASSERT(m_pseudoType == pagePseudoType);
+}
+
+inline void CSSSelector::setRelation(RelationType relation)
+{
+    m_relation = relation;
+    ASSERT(m_relation == relation);
+}
+
+inline void CSSSelector::setMatch(Match match)
+{
+    m_match = match;
+    ASSERT(m_match == match);
+}
+
 } // namespace WebCore
