@@ -26,11 +26,12 @@
 #include "CSSRule.h"
 #include "CSSStyleSheet.h"
 #include "CustomElementReactionQueue.h"
-#include "DOMWindow.h"
 #include "HTMLNames.h"
 #include "InspectorInstrumentation.h"
 #include "JSDOMGlobalObject.h"
 #include "JSDOMWindowBase.h"
+#include "LocalDOMWindow.h"
+#include "MutableStyleProperties.h"
 #include "MutationObserverInterestGroup.h"
 #include "MutationRecord.h"
 #include "Quirks.h"
