@@ -33,16 +33,15 @@
 #include "CSSSelectorList.h"
 #include "CommonAtomStrings.h"
 #include "Document.h"
-#include "ElementInlines.h"
+#include "ElementChildIteratorInlines.h"
 #include "ElementRareData.h"
 #include "ElementTraversal.h"
-#include "Frame.h"
 #include "FrameSelection.h"
 #include "HTMLDocument.h"
 #include "HTMLNames.h"
-#include "HTMLParserIdioms.h"
 #include "HTMLSlotElement.h"
 #include "InspectorInstrumentation.h"
+#include "LocalFrame.h"
 #include "Page.h"
 #include "RenderElement.h"
 #include "RuleFeature.h"
@@ -51,7 +50,7 @@
 #include "StyleRule.h"
 #include "StyleScope.h"
 #include "Text.h"
-#include "TypedElementDescendantIterator.h"
+#include "TypedElementDescendantIteratorInlines.h"
 
 namespace WebCore {
 
@@ -478,7 +477,7 @@ static bool attributeValueMatches(const Attribute& attribute, CSSSelector::Match
     case CSSSelector::List:
         {
             // Ignore empty selectors or selectors containing spaces.
-            if (selectorValue.isEmpty() || selectorValue.find(isHTMLSpace<UChar>) != notFound)
+            if (selectorValue.isEmpty() || selectorValue.find(isASCIIWhitespace<UChar>) != notFound)
                 return false;
 
             unsigned startSearchAt = 0;
@@ -490,9 +489,9 @@ static bool attributeValueMatches(const Attribute& attribute, CSSSelector::Match
                     foundPos = value.findIgnoringASCIICase(selectorValue, startSearchAt);
                 if (foundPos == notFound)
                     return false;
-                if (!foundPos || isHTMLSpace(value[foundPos - 1])) {
+                if (!foundPos || isASCIIWhitespace(value[foundPos - 1])) {
                     unsigned endStr = foundPos + selectorValue.length();
-                    if (endStr == value.length() || isHTMLSpace(value[endStr]))
+                    if (endStr == value.length() || isASCIIWhitespace(value[endStr]))
                         break; // We found a match.
                 }
 
@@ -560,8 +559,9 @@ static bool attributeValueMatches(const Attribute& attribute, CSSSelector::Match
 static bool anyAttributeMatches(const Element& element, const CSSSelector& selector, const QualifiedName& selectorAttr, bool caseSensitive)
 {
     ASSERT(element.hasAttributesWithoutUpdate());
+    bool isHTML = element.isHTMLElement() && element.document().isHTMLDocument();
     for (const Attribute& attribute : element.attributesIterator()) {
-        if (!attribute.matches(selectorAttr.prefix(), element.isHTMLElement() ? selector.attributeCanonicalLocalName() : selectorAttr.localName(), selectorAttr.namespaceURI()))
+        if (!attribute.matches(selectorAttr.prefix(), isHTML ? selectorAttr.localNameLowercase() : selectorAttr.localName(), selectorAttr.namespaceURI()))
             continue;
 
         if (attributeValueMatches(attribute, selector.match(), selector.value(), caseSensitive))
@@ -575,7 +575,8 @@ bool SelectorChecker::attributeSelectorMatches(const Element& element, const Qua
 {
     ASSERT(selector.isAttributeSelector());
     auto& selectorAttribute = selector.attribute();
-    auto& selectorName = element.isHTMLElement() ? selector.attributeCanonicalLocalName() : selectorAttribute.localName();
+    bool isHTML = element.isHTMLElement() && element.document().isHTMLDocument();
+    auto& selectorName = isHTML ? selectorAttribute.localNameLowercase() : selectorAttribute.localName();
     if (!Attribute::nameMatchesFilter(attributeName, selectorAttribute.prefix(), selectorName, selectorAttribute.namespaceURI()))
         return false;
     bool caseSensitive = true;
@@ -754,8 +755,6 @@ bool SelectorChecker::checkOne(CheckingContext& checkingContext, const LocalCont
             auto* parent = element.parentNode();
             if (is<Element>(parent))
                 addStyleRelation(checkingContext, downcast<Element>(*parent), Style::Relation::ChildrenAffectedByFirstChildRules);
-            else if (!is<ShadowRoot>(parent))
-                break; // FIXME: Add the support for specifying relations on ShadowRoot.
             if (!isFirstChild)
                 break;
             addStyleRelation(checkingContext, element, Style::Relation::FirstChild);
@@ -767,8 +766,7 @@ bool SelectorChecker::checkOne(CheckingContext& checkingContext, const LocalCont
             if (is<Element>(parent)) {
                 auto relation = context.isSubjectOrAdjacentElement ? Style::Relation::ChildrenAffectedByForwardPositionalRules : Style::Relation::DescendantsAffectedByForwardPositionalRules;
                 addStyleRelation(checkingContext, downcast<Element>(*parent), relation);
-            } else if (!is<ShadowRoot>(parent))
-                break; // FIXME: Add the support for specifying relations on ShadowRoot.
+            }
             return isFirstOfType(element, element.tagQName());
         }
         case CSSSelector::PseudoClassLastChild: {
@@ -780,8 +778,7 @@ bool SelectorChecker::checkOne(CheckingContext& checkingContext, const LocalCont
                 if (!parentElement.isFinishedParsingChildren())
                     isLastChild = false;
                 addStyleRelation(checkingContext, parentElement, Style::Relation::ChildrenAffectedByLastChildRules);
-            } else if (!is<ShadowRoot>(parent))
-                break; // FIXME: Add the support for specifying relations on ShadowRoot.
+            }
             if (!isLastChild)
                 break;
             addStyleRelation(checkingContext, element, Style::Relation::LastChild);
@@ -796,8 +793,7 @@ bool SelectorChecker::checkOne(CheckingContext& checkingContext, const LocalCont
                 addStyleRelation(checkingContext, parentElement, relation);
                 if (!parentElement.isFinishedParsingChildren())
                     return false;
-            } else if (!is<ShadowRoot>(parent))
-                break; // FIXME: Add the support for specifying relations on ShadowRoot.
+            }
             return isLastOfType(element, element.tagQName());
         }
         case CSSSelector::PseudoClassOnlyChild: {
@@ -810,8 +806,7 @@ bool SelectorChecker::checkOne(CheckingContext& checkingContext, const LocalCont
                 addStyleRelation(checkingContext, parentElement, Style::Relation::ChildrenAffectedByLastChildRules);
                 if (!parentElement.isFinishedParsingChildren())
                     onlyChild = false;
-            } else if (!is<ShadowRoot>(parent))
-                break; // FIXME: Add the support for specifying relations on ShadowRoot.
+            }
             if (firstChild)
                 addStyleRelation(checkingContext, element, Style::Relation::FirstChild);
             if (onlyChild)
@@ -830,8 +825,7 @@ bool SelectorChecker::checkOne(CheckingContext& checkingContext, const LocalCont
 
                 if (!parentElement.isFinishedParsingChildren())
                     return false;
-            } else if (!is<ShadowRoot>(parent))
-                break; // FIXME: Add the support for specifying relations on ShadowRoot.
+            }
             return isFirstOfType(element, element.tagQName()) && isLastOfType(element, element.tagQName());
         }
         case CSSSelector::PseudoClassIs:
@@ -885,8 +879,7 @@ bool SelectorChecker::checkOne(CheckingContext& checkingContext, const LocalCont
             if (is<Element>(parent)) {
                 auto relation = context.isSubjectOrAdjacentElement ? Style::Relation::ChildrenAffectedByForwardPositionalRules : Style::Relation::DescendantsAffectedByForwardPositionalRules;
                 addStyleRelation(checkingContext, downcast<Element>(*parent), relation);
-            } else if (!is<ShadowRoot>(parent))
-                break; // FIXME: Add the support for specifying relations on ShadowRoot.
+            }
 
             if (const CSSSelectorList* selectorList = selector.selectorList()) {
                 if (!matchSelectorList(checkingContext, context, element, *selectorList))
@@ -913,8 +906,7 @@ bool SelectorChecker::checkOne(CheckingContext& checkingContext, const LocalCont
             if (is<Element>(parent)) {
                 auto relation = context.isSubjectOrAdjacentElement ? Style::Relation::ChildrenAffectedByForwardPositionalRules : Style::Relation::DescendantsAffectedByForwardPositionalRules;
                 addStyleRelation(checkingContext, downcast<Element>(*parent), relation);
-            } else if (!is<ShadowRoot>(parent))
-                break; // FIXME: Add the support for specifying relations on ShadowRoot.
+            }
 
             int count = 1 + countElementsOfTypeBefore(element, element.tagQName());
             if (selector.matchNth(count))
@@ -929,15 +921,14 @@ bool SelectorChecker::checkOne(CheckingContext& checkingContext, const LocalCont
                     if (!matchSelectorList(checkingContext, context, element, *selectorList))
                         return false;
 
-                    addStyleRelation(checkingContext, parentElement, Style::Relation::ChildrenAffectedByPropertyBasedBackwardPositionalRules);
+                    addStyleRelation(checkingContext, parentElement, Style::Relation::ChildrenAffectedByBackwardPositionalRules);
                 } else {
                     auto relation = context.isSubjectOrAdjacentElement ? Style::Relation::ChildrenAffectedByBackwardPositionalRules : Style::Relation::DescendantsAffectedByBackwardPositionalRules;
                     addStyleRelation(checkingContext, parentElement, relation);
                 }
                 if (!parentElement.isFinishedParsingChildren())
                     return false;
-            } else if (!is<ShadowRoot>(parent))
-                break; // FIXME: Add the support for specifying relations on ShadowRoot.
+            }
 
             int count = 1;
             if (const CSSSelectorList* selectorList = selector.selectorList()) {
@@ -959,8 +950,7 @@ bool SelectorChecker::checkOne(CheckingContext& checkingContext, const LocalCont
 
                 if (!parentElement.isFinishedParsingChildren())
                     return false;
-            } else if (!is<ShadowRoot>(parent))
-                break; // FIXME: Add the support for specifying relations on ShadowRoot.
+            }
             int count = 1 + countElementsOfTypeAfter(element, element.tagQName());
             return selector.matchNth(count);
         }
@@ -1127,6 +1117,8 @@ bool SelectorChecker::checkOne(CheckingContext& checkingContext, const LocalCont
         case CSSSelector::PseudoClassHasAttachment:
             return hasAttachment(element);
 #endif
+        case CSSSelector::PseudoClassPopoverOpen:
+            return matchesPopoverOpenPseudoClass(element);
 
         case CSSSelector::PseudoClassModal:
             return matchesModalPseudoClass(element);
