@@ -28,10 +28,11 @@
 #include "CachedCSSStyleSheet.h"
 #include "CommonAtomStrings.h"
 #include "Document.h"
-#include "Frame.h"
 #include "FrameLoader.h"
+#include "LocalFrame.h"
 #include "MediaList.h"
 #include "Node.h"
+#include "OriginAccessPatterns.h"
 #include "Page.h"
 #include "PageConsoleClient.h"
 #include "ResourceLoadInfo.h"
@@ -132,6 +133,8 @@ bool StyleSheetContents::isCacheable() const
     // FIXME: Valid mime type avoids the check too.
     if (!m_hasSyntacticallyValidCSSHeader)
         return false;
+    if (m_hasNestingRules)
+        return false;
     return true;
 }
 
@@ -167,8 +170,17 @@ void StyleSheetContents::parserAppendRule(Ref<StyleRuleBase>&& rule)
     }
 
     // NOTE: The selector list has to fit into RuleData. <http://webkit.org/b/118369>
-    // If we're adding a rule with a huge number of selectors, split it up into multiple rules
-    if (is<StyleRule>(rule) && downcast<StyleRule>(rule.get()).selectorList().componentCount() > Style::RuleData::maximumSelectorComponentCount) {
+    auto ruleHasTooManySelectors = [&] {
+        return downcast<StyleRule>(rule.get()).selectorList().componentCount() > Style::RuleData::maximumSelectorComponentCount;
+    };
+
+    if (is<StyleRuleWithNesting>(rule) && ruleHasTooManySelectors()) {
+        // We don't support nested rules with too many selectors
+        return;
+    }
+
+    if (is<StyleRule>(rule) && ruleHasTooManySelectors()) {
+        // If we're adding a rule with a huge number of selectors, split it up into multiple rules
         m_childRules.appendVector(downcast<StyleRule>(rule.get()).splitIntoMultipleRulesWithMaximumSelectorComponentCount(Style::RuleData::maximumSelectorComponentCount));
         return;
     }
@@ -369,7 +381,7 @@ const AtomString& StyleSheetContents::namespaceURIFromPrefix(const AtomString& p
 
 bool StyleSheetContents::parseAuthorStyleSheet(const CachedCSSStyleSheet* cachedStyleSheet, const SecurityOrigin* securityOrigin)
 {
-    bool isSameOriginRequest = securityOrigin && securityOrigin->canRequest(baseURL());
+    bool isSameOriginRequest = securityOrigin && securityOrigin->canRequest(baseURL(), OriginAccessPatternsForWebProcess::singleton());
     CachedCSSStyleSheet::MIMETypeCheckHint mimeTypeCheckHint = isStrictParserMode(m_parserContext.mode) || !isSameOriginRequest ? CachedCSSStyleSheet::MIMETypeCheckHint::Strict : CachedCSSStyleSheet::MIMETypeCheckHint::Lax;
     bool hasValidMIMEType = true;
     String sheetText = cachedStyleSheet->sheetText(mimeTypeCheckHint, &hasValidMIMEType);
@@ -472,6 +484,14 @@ static bool traverseRulesInVector(const Vector<RefPtr<StyleRuleBase>>& rules, co
     for (auto& rule : rules) {
         if (handler(*rule))
             return true;
+        if (auto styleRuleWithNesting = dynamicDowncast<StyleRuleWithNesting>(*rule)) {
+            auto nestedRules = styleRuleWithNesting->nestedRules().map(
+                [](auto& ref) -> RefPtr<StyleRuleBase> {
+                    return ref.ptr();
+                });
+            if (traverseRulesInVector(nestedRules, handler))
+                return true;
+        }
         if (!rule->isGroupRule())
             continue;
         if (traverseRulesInVector(downcast<StyleRuleGroup>(*rule).childRules(), handler))
@@ -498,6 +518,8 @@ bool StyleSheetContents::traverseSubresources(const Function<bool(const CachedRe
         switch (rule.type()) {
         case StyleRuleType::Style:
             return downcast<StyleRule>(rule).properties().traverseSubresources(handler);
+        case StyleRuleType::StyleWithNesting:
+            return downcast<StyleRuleWithNesting>(rule).properties().traverseSubresources(handler);
         case StyleRuleType::FontFace:
             return downcast<StyleRuleFontFace>(rule).properties().traverseSubresources(handler);
         case StyleRuleType::Import:
