@@ -27,11 +27,12 @@
 
 #include "Chrome.h"
 #include "DocumentLoader.h"
-#include "Frame.h"
-#include "FrameView.h"
+#include "LocalFrame.h"
+#include "LocalFrameView.h"
 #include "MediaQueryEvaluator.h"
 #include "Page.h"
 #include "Quirks.h"
+#include "RenderElementInlines.h"
 #include "RenderLayerCompositor.h"
 #include "RenderView.h"
 #include "ScreenProperties.h"
@@ -248,7 +249,9 @@ const FeatureSchema& color()
     static MainThreadNeverDestroyed<IntegerSchema> schema {
         "color"_s,
         [](auto& context) {
-            return screenDepthPerComponent(context.document.frame()->mainFrame().view());
+            if (auto* localFrame = dynamicDowncast<LocalFrame>(context.document.frame()->mainFrame()))
+                return screenDepthPerComponent(localFrame->view()); 
+            return 8;
         }
     };
     return schema;
@@ -264,8 +267,10 @@ const FeatureSchema& colorGamut()
 
             // FIXME: At some point we should start detecting displays that support more colors.
             MatchingIdentifiers identifiers { CSSValueSRGB };
-            if (screenSupportsExtendedColor(frame.mainFrame().view()))
-                identifiers.append(CSSValueP3);
+            if (auto* localFrame = dynamicDowncast<LocalFrame>(frame.mainFrame())) {
+                if (screenSupportsExtendedColor(localFrame->view()))
+                    identifiers.append(CSSValueP3);
+            }
             return identifiers;
         }
     };
@@ -286,8 +291,11 @@ const FeatureSchema& deviceAspectRatio()
     static MainThreadNeverDestroyed<RatioSchema> schema {
         "device-aspect-ratio"_s,
         [](auto& context) {
-            auto screenSize = context.document.frame()->mainFrame().screenSize();
-            return FloatSize { screenSize.width(), screenSize.height() };
+            if (auto* localFrame = dynamicDowncast<LocalFrame>(context.document.frame()->mainFrame())) {
+                auto screenSize = localFrame->screenSize();
+                return FloatSize { screenSize.width(), screenSize.height() };
+            }
+            return FloatSize { 0.0f, 0.0f };
         }
     };
     return schema;
@@ -298,7 +306,9 @@ const FeatureSchema& deviceHeight()
     static MainThreadNeverDestroyed<LengthSchema> schema {
         "device-height"_s,
         [](auto& context) {
-            return LayoutUnit { context.document.frame()->mainFrame().screenSize().height() };
+            if (auto* localFrame = dynamicDowncast<LocalFrame>(context.document.frame()->mainFrame()))
+                return LayoutUnit { localFrame->screenSize().height() };
+            return LayoutUnit { 0.0f };
         }
     };
     return schema;
@@ -320,7 +330,9 @@ const FeatureSchema& deviceWidth()
     static MainThreadNeverDestroyed<LengthSchema> schema {
         "device-width"_s,
         [](auto& context) {
-            return LayoutUnit { context.document.frame()->mainFrame().screenSize().width() };
+            if (auto* localFrame = dynamicDowncast<LocalFrame>(context.document.frame()->mainFrame()))
+                return LayoutUnit { localFrame->screenSize().width() };
+            return LayoutUnit { 0.0f };
         }
     };
     return schema;
@@ -338,7 +350,9 @@ const FeatureSchema& dynamicRange()
                     return true;
                 if (frame.settings().forcedSupportsHighDynamicRangeValue() == ForcedAccessibilityValue::Off)
                     return false;
-                return screenSupportsHighDynamicRange(frame.mainFrame().view());
+                if (auto* localFrame = dynamicDowncast<LocalFrame>(frame.mainFrame()))
+                    return screenSupportsHighDynamicRange(localFrame->view());
+                return false;
             }();
 
             MatchingIdentifiers identifiers { CSSValueStandard };
@@ -425,16 +439,19 @@ const FeatureSchema& monochrome()
     static MainThreadNeverDestroyed<IntegerSchema> schema {
         "monochrome"_s,
         [](auto& context) {
+            auto& frame = *context.document.frame();
+            auto* localFrame = dynamicDowncast<LocalFrame>(frame.mainFrame());
             bool isMonochrome = [&] {
-                auto& frame = *context.document.frame();
                 if (frame.settings().forcedDisplayIsMonochromeAccessibilityValue() == ForcedAccessibilityValue::On)
                     return true;
                 if (frame.settings().forcedDisplayIsMonochromeAccessibilityValue() == ForcedAccessibilityValue::Off)
                     return false;
-                return screenIsMonochrome(frame.mainFrame().view());
+                if (localFrame)
+                    return screenIsMonochrome(localFrame->view());
+                return false;
             }();
 
-            return isMonochrome ? screenDepthPerComponent(context.document.frame()->mainFrame().view()) : 0;
+            return isMonochrome && localFrame ? screenDepthPerComponent(localFrame->view()) : 0;
         }
     };
     return schema;
@@ -497,7 +514,7 @@ const FeatureSchema& prefersContrast()
                 case ForcedAccessibilityValue::Off:
                     return false;
                 case ForcedAccessibilityValue::System:
-#if PLATFORM(MAC) || PLATFORM(IOS_FAMILY)
+#if PLATFORM(MAC) || PLATFORM(IOS_FAMILY) || PLATFORM(GTK)
                     return Theme::singleton().userPrefersContrast();
 #else
                     return false;
@@ -673,6 +690,37 @@ const FeatureSchema& displayMode()
 }
 #endif
 
+const FeatureSchema& overflowBlock()
+{
+    static MainThreadNeverDestroyed<IdentifierSchema> schema {
+        "overflow-block"_s,
+        Vector { CSSValueNone, CSSValueScroll, CSSValuePaged },
+        [](auto& context) {
+            // FIXME: Match none when scrollEnabled is set to false by UIKit.
+            bool usesPaginatedMode = [&] {
+                auto& frame = *context.document.frame();
+                auto* frameView = frame.view();
+                return frameView && frameView->pagination().mode != PaginationMode::Unpaginated;
+            }();
+            return MatchingIdentifiers { usesPaginatedMode ? CSSValuePaged : CSSValueScroll };
+        }
+    };
+    return schema;
+}
+
+const FeatureSchema& overflowInline()
+{
+    static MainThreadNeverDestroyed<IdentifierSchema> schema {
+        "overflow-inline"_s,
+        Vector { CSSValueNone, CSSValueScroll },
+        [](auto&) {
+            // FIXME: Match none when scrollEnabled is set to false by UIKit.
+            return MatchingIdentifiers { CSSValueScroll };
+        }
+    };
+    return schema;
+}
+
 #if ENABLE(DARK_MODE_CSS)
 const FeatureSchema& prefersColorScheme()
 {
@@ -680,16 +728,8 @@ const FeatureSchema& prefersColorScheme()
         "prefers-color-scheme"_s,
         Vector { CSSValueLight, CSSValueDark },
         [](auto& context) {
-            bool useDarkAppearance = [&] {
-                auto& frame = *context.document.frame();
-                if (frame.document()->loader()) {
-                    auto colorSchemePreference = frame.document()->loader()->colorSchemePreference();
-                    if (colorSchemePreference != ColorSchemePreference::NoPreference)
-                        return colorSchemePreference == ColorSchemePreference::Dark;
-                }
-
-                return frame.page()->useDarkAppearance();
-            }();
+            auto& frame = *context.document.frame();
+            bool useDarkAppearance = frame.page()->useDarkAppearance();
 
             return MatchingIdentifiers { useDarkAppearance ? CSSValueDark : CSSValueLight };
         }
@@ -719,6 +759,8 @@ Vector<const FeatureSchema*> allSchemas()
         &hover(),
         &invertedColors(),
         &monochrome(),
+        &overflowBlock(),
+        &overflowInline(),
         &orientation(),
         &pointer(),
         &prefersContrast(),
