@@ -26,60 +26,70 @@
 #include "config.h"
 #include "CustomElementRegistry.h"
 
-#if ENABLE(CUSTOM_ELEMENTS)
-
 #include "CustomElementReactionQueue.h"
-#include "DOMWindow.h"
 #include "Document.h"
-#include "Element.h"
+#include "ElementRareData.h"
 #include "ElementTraversal.h"
 #include "JSCustomElementInterface.h"
+#include "JSDOMPromiseDeferred.h"
+#include "LocalDOMWindow.h"
 #include "MathMLNames.h"
 #include "QualifiedName.h"
-#include "SVGNames.h"
 #include "ShadowRoot.h"
-#include <runtime/JSCJSValueInlines.h>
-#include <wtf/text/AtomicString.h>
+#include "TypedElementDescendantIteratorInlines.h"
+#include <JavaScriptCore/JSCJSValueInlines.h>
+#include <wtf/text/AtomString.h>
 
 namespace WebCore {
 
-Ref<CustomElementRegistry> CustomElementRegistry::create(DOMWindow& window)
+Ref<CustomElementRegistry> CustomElementRegistry::create(LocalDOMWindow& window, ScriptExecutionContext* scriptExecutionContext)
 {
-    return adoptRef(*new CustomElementRegistry(window));
+    return adoptRef(*new CustomElementRegistry(window, scriptExecutionContext));
 }
 
-CustomElementRegistry::CustomElementRegistry(DOMWindow& window)
-    : m_window(window)
-{ }
+CustomElementRegistry::CustomElementRegistry(LocalDOMWindow& window, ScriptExecutionContext* scriptExecutionContext)
+    : ContextDestructionObserver(scriptExecutionContext)
+    , m_window(window)
+{
+}
 
-CustomElementRegistry::~CustomElementRegistry()
-{ }
+CustomElementRegistry::~CustomElementRegistry() = default;
+
+Document* CustomElementRegistry::document() const
+{
+    return m_window.document();
+}
 
 // https://dom.spec.whatwg.org/#concept-shadow-including-tree-order
 static void enqueueUpgradeInShadowIncludingTreeOrder(ContainerNode& node, JSCustomElementInterface& elementInterface)
 {
     for (Element* element = ElementTraversal::firstWithin(node); element; element = ElementTraversal::next(*element)) {
-        if (element->isCustomElementUpgradeCandidate() && element->tagQName() == elementInterface.name())
+        if (element->isCustomElementUpgradeCandidate() && element->tagQName().matches(elementInterface.name()))
             element->enqueueToUpgrade(elementInterface);
         if (auto* shadowRoot = element->shadowRoot()) {
-            if (shadowRoot->mode() != ShadowRoot::Mode::UserAgent)
+            if (shadowRoot->mode() != ShadowRootMode::UserAgent)
                 enqueueUpgradeInShadowIncludingTreeOrder(*shadowRoot, elementInterface);
         }
     }
 }
 
-void CustomElementRegistry::addElementDefinition(Ref<JSCustomElementInterface>&& elementInterface)
+RefPtr<DeferredPromise> CustomElementRegistry::addElementDefinition(Ref<JSCustomElementInterface>&& elementInterface)
 {
-    AtomicString localName = elementInterface->name().localName();
+    AtomString localName = elementInterface->name().localName();
     ASSERT(!m_nameMap.contains(localName));
-    m_constructorMap.add(elementInterface->constructor(), elementInterface.ptr());
     m_nameMap.add(localName, elementInterface.copyRef());
+    {
+        Locker locker { m_constructorMapLock };
+        m_constructorMap.add(elementInterface->constructor(), elementInterface.ptr());
+    }
+
+    if (elementInterface->isShadowDisabled())
+        m_disabledShadowSet.add(localName);
 
     if (auto* document = m_window.document())
         enqueueUpgradeInShadowIncludingTreeOrder(*document, elementInterface.get());
 
-    if (auto promise = m_promiseMap.take(localName))
-        promise.value()->resolve(nullptr);
+    return m_promiseMap.take(localName);
 }
 
 JSCustomElementInterface* CustomElementRegistry::findInterface(const Element& element) const
@@ -89,35 +99,76 @@ JSCustomElementInterface* CustomElementRegistry::findInterface(const Element& el
 
 JSCustomElementInterface* CustomElementRegistry::findInterface(const QualifiedName& name) const
 {
-    ASSERT(!name.hasPrefix());
     if (name.namespaceURI() != HTMLNames::xhtmlNamespaceURI)
         return nullptr;
-    auto it = m_nameMap.find(name.localName());
-    return it == m_nameMap.end() || it->value->name() != name ? nullptr : const_cast<JSCustomElementInterface*>(it->value.ptr());
+    return m_nameMap.get(name.localName());
 }
 
-JSCustomElementInterface* CustomElementRegistry::findInterface(const AtomicString& name) const
+JSCustomElementInterface* CustomElementRegistry::findInterface(const AtomString& name) const
 {
     return m_nameMap.get(name);
 }
 
 JSCustomElementInterface* CustomElementRegistry::findInterface(const JSC::JSObject* constructor) const
 {
+    Locker locker { m_constructorMapLock };
     return m_constructorMap.get(constructor);
 }
 
 bool CustomElementRegistry::containsConstructor(const JSC::JSObject* constructor) const
 {
+    Locker locker { m_constructorMapLock };
     return m_constructorMap.contains(constructor);
 }
 
-JSC::JSValue CustomElementRegistry::get(const AtomicString& name)
+JSC::JSValue CustomElementRegistry::get(const AtomString& name)
 {
     if (auto* elementInterface = m_nameMap.get(name))
         return elementInterface->constructor();
     return JSC::jsUndefined();
 }
 
+String CustomElementRegistry::getName(JSC::JSValue constructorValue)
+{
+    auto* constructor = constructorValue.getObject();
+    if (!constructor)
+        return String { };
+    auto* elementInterface = findInterface(constructor);
+    if (!elementInterface)
+        return String { };
+    return elementInterface->name().localName();
 }
 
-#endif
+static void upgradeElementsInShadowIncludingDescendants(ContainerNode& root)
+{
+    for (auto& element : descendantsOfType<Element>(root)) {
+        if (element.isCustomElementUpgradeCandidate())
+            CustomElementReactionQueue::tryToUpgradeElement(element);
+        if (auto* shadowRoot = element.shadowRoot())
+            upgradeElementsInShadowIncludingDescendants(*shadowRoot);
+    }
+}
+
+void CustomElementRegistry::upgrade(Node& root)
+{
+    if (!is<ContainerNode>(root))
+        return;
+
+    if (is<Element>(root) && downcast<Element>(root).isCustomElementUpgradeCandidate())
+        CustomElementReactionQueue::tryToUpgradeElement(downcast<Element>(root));
+
+    upgradeElementsInShadowIncludingDescendants(downcast<ContainerNode>(root));
+}
+
+template<typename Visitor>
+void CustomElementRegistry::visitJSCustomElementInterfaces(Visitor& visitor) const
+{
+    Locker locker { m_constructorMapLock };
+    for (const auto& iterator : m_constructorMap)
+        iterator.value->visitJSFunctions(visitor);
+}
+
+template void CustomElementRegistry::visitJSCustomElementInterfaces(JSC::AbstractSlotVisitor&) const;
+template void CustomElementRegistry::visitJSCustomElementInterfaces(JSC::SlotVisitor&) const;
+
+}
