@@ -23,39 +23,44 @@
 #include "config.h"
 #include "MouseRelatedEvent.h"
 
-#include "DOMWindow.h"
 #include "Document.h"
-#include "Frame.h"
-#include "FrameView.h"
 #include "LayoutPoint.h"
+#include "LocalDOMWindow.h"
+#include "LocalFrame.h"
+#include "LocalFrameView.h"
 #include "RenderLayer.h"
 #include "RenderObject.h"
+#include <wtf/IsoMallocInlines.h>
 
 namespace WebCore {
 
-MouseRelatedEvent::MouseRelatedEvent(const AtomicString& eventType, bool canBubble, bool cancelable, MonotonicTime timestamp, RefPtr<WindowProxy>&& view,
-                                     int detail, const IntPoint& screenLocation, const IntPoint& windowLocation,
-#if ENABLE(POINTER_LOCK)
-                                     const IntPoint& movementDelta,
-#endif
-                                     bool ctrlKey, bool altKey, bool shiftKey, bool metaKey, bool isSimulated)
-    : UIEventWithKeyState(eventType, canBubble, cancelable, timestamp, WTFMove(view), detail, ctrlKey, altKey, shiftKey, metaKey, false, false)
+WTF_MAKE_ISO_ALLOCATED_IMPL(MouseRelatedEvent);
+
+MouseRelatedEvent::MouseRelatedEvent(const AtomString& eventType, CanBubble canBubble, IsCancelable isCancelable, IsComposed isComposed,
+    MonotonicTime timestamp, RefPtr<WindowProxy>&& view, int detail,
+    const IntPoint& screenLocation, const IntPoint& windowLocation, double movementX, double movementY, OptionSet<Modifier> modifiers, IsSimulated isSimulated, IsTrusted isTrusted)
+    : UIEventWithKeyState(eventType, canBubble, isCancelable, isComposed, timestamp, WTFMove(view), detail, modifiers, isTrusted)
     , m_screenLocation(screenLocation)
-#if ENABLE(POINTER_LOCK)
-    , m_movementDelta(movementDelta)
-#endif
-    , m_isSimulated(isSimulated)
+    , m_movementX(movementX)
+    , m_movementY(movementY)
+    , m_isSimulated(isSimulated == IsSimulated::Yes)
 {
-    init(isSimulated, windowLocation);
+    init(m_isSimulated, windowLocation);
 }
 
-MouseRelatedEvent::MouseRelatedEvent(const AtomicString& eventType, const MouseRelatedEventInit& initializer, IsTrusted isTrusted)
-    : UIEventWithKeyState(eventType, initializer, isTrusted)
-    , m_screenLocation(IntPoint(initializer.screenX, initializer.screenY))
-#if ENABLE(POINTER_LOCK)
-    , m_movementDelta(IntPoint(0, 0))
-#endif
+MouseRelatedEvent::MouseRelatedEvent(const AtomString& type, IsCancelable isCancelable, MonotonicTime timestamp, RefPtr<WindowProxy>&& view, const IntPoint& globalLocation, OptionSet<Modifier> modifiers)
+    : MouseRelatedEvent(type, CanBubble::Yes, isCancelable, IsComposed::Yes, timestamp,
+        WTFMove(view), 0, globalLocation, globalLocation /* Converted in init */, 0, 0, modifiers, IsSimulated::No)
 {
+}
+
+MouseRelatedEvent::MouseRelatedEvent(const AtomString& eventType, const MouseRelatedEventInit& initializer, IsTrusted isTrusted)
+    : UIEventWithKeyState(eventType, initializer)
+    , m_screenLocation(IntPoint(initializer.screenX, initializer.screenY))
+    , m_movementX(initializer.movementX)
+    , m_movementY(initializer.movementY)
+{
+    ASSERT_UNUSED(isTrusted, isTrusted == IsTrusted::No);
     init(false, IntPoint(0, 0));
 }
 
@@ -84,16 +89,16 @@ void MouseRelatedEvent::initCoordinates()
     m_hasCachedRelativePosition = false;
 }
 
-FrameView* MouseRelatedEvent::frameViewFromWindowProxy(WindowProxy* windowProxy)
+LocalFrameView* MouseRelatedEvent::frameViewFromWindowProxy(WindowProxy* windowProxy)
 {
-    if (!windowProxy || !is<DOMWindow>(windowProxy->window()))
+    if (!windowProxy || !is<LocalDOMWindow>(windowProxy->window()))
         return nullptr;
 
-    auto* frame = downcast<DOMWindow>(*windowProxy->window()).frame();
+    auto* frame = downcast<LocalDOMWindow>(*windowProxy->window()).frame();
     return frame ? frame->view() : nullptr;
 }
 
-LayoutPoint MouseRelatedEvent::pagePointToClientPoint(LayoutPoint pagePoint, FrameView* frameView)
+LayoutPoint MouseRelatedEvent::pagePointToClientPoint(LayoutPoint pagePoint, LocalFrameView* frameView)
 {
     if (!frameView)
         return pagePoint;
@@ -101,7 +106,7 @@ LayoutPoint MouseRelatedEvent::pagePointToClientPoint(LayoutPoint pagePoint, Fra
     return flooredLayoutPoint(frameView->documentToClientPoint(pagePoint));
 }
 
-LayoutPoint MouseRelatedEvent::pagePointToAbsolutePoint(LayoutPoint pagePoint, FrameView* frameView)
+LayoutPoint MouseRelatedEvent::pagePointToAbsolutePoint(LayoutPoint pagePoint, LocalFrameView* frameView)
 {
     if (!frameView)
         return pagePoint;
