@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2011, 2013, 2014, 2016 Apple Inc. All rights reserved.
+ * Copyright (C) 2011-2020 Apple Inc. All rights reserved.
  * Copyright (C) 2014 Yusuke Suzuki <utatane.tea@gmail.com>
  *
  * Redistribution and use in source and binary forms, with or without
@@ -27,17 +27,19 @@
 #include "config.h"
 #include "SelectorQuery.h"
 
-#include "CSSParser.h"
-#include "ElementDescendantIterator.h"
-#include "ExceptionCode.h"
+#include "CSSSelectorParser.h"
+#include "CSSTokenizer.h"
+#include "CommonAtomStrings.h"
+#include "ElementAncestorIteratorInlines.h"
 #include "HTMLNames.h"
 #include "SelectorChecker.h"
 #include "StaticNodeList.h"
 #include "StyledElement.h"
+#include "TypedElementDescendantIteratorInlines.h"
 
 namespace WebCore {
 
-#if !ASSERT_DISABLED
+#if ASSERT_ENABLED
 static bool isSingleTagNameSelector(const CSSSelector& selector)
 {
     return selector.isLastInTagHistory() && selector.match() == CSSSelector::Tag;
@@ -47,7 +49,7 @@ static bool isSingleClassNameSelector(const CSSSelector& selector)
 {
     return selector.isLastInTagHistory() && selector.match() == CSSSelector::Class;
 }
-#endif
+#endif // ASSERT_ENABLED
 
 enum class IdMatchingType : uint8_t {
     None,
@@ -55,6 +57,15 @@ enum class IdMatchingType : uint8_t {
     Filter
 };
 
+template<typename Output> static ALWAYS_INLINE void appendOutputForElement(Output& output, Element& element)
+{
+    if constexpr (std::is_same_v<Output, Element*>) {
+        ASSERT(!output);
+        output = &element;
+    } else
+        output.append(element);
+}
+
 static bool canBeUsedForIdFastPath(const CSSSelector& selector)
 {
     return selector.match() == CSSSelector::Id
@@ -84,7 +95,7 @@ SelectorDataList::SelectorDataList(const CSSSelectorList& selectorList)
 
     m_selectors.reserveInitialCapacity(selectorCount);
     for (const CSSSelector* selector = selectorList.first(); selector; selector = CSSSelectorList::next(selector))
-        m_selectors.uncheckedAppend(SelectorData(selector));
+        m_selectors.uncheckedAppend({ selector });
 
     if (selectorCount == 1) {
         const CSSSelector& selector = *m_selectors.first().selector;
@@ -125,8 +136,7 @@ inline bool SelectorDataList::selectorMatches(const SelectorData& selectorData,
     SelectorChecker selectorChecker(element.document());
     SelectorChecker::CheckingContext selectorCheckingContext(SelectorChecker::Mode::QueryingRules);
     selectorCheckingContext.scope = rootNode.isDocumentNode() ? nullptr : &rootNode;
-    unsigned ignoredSpecificity;
-    return selectorChecker.match(*selectorData.selector, element, selectorCheckingContext, ignoredSpecificity);
+    return selectorChecker.match(*selectorData.selector, element, selectorCheckingContext);
 }
 
 inline Element* SelectorDataList::selectorClosest(const SelectorData& selectorData, Element& element, const ContainerNode& rootNode) const
@@ -134,8 +144,7 @@ inline Element* SelectorDataList::selectorClosest(const SelectorData& selectorDa
     SelectorChecker selectorChecker(element.document());
     SelectorChecker::CheckingContext selectorCheckingContext(SelectorChecker::Mode::QueryingRules);
     selectorCheckingContext.scope = rootNode.isDocumentNode() ? nullptr : &rootNode;
-    unsigned ignoredSpecificity;
-    if (!selectorChecker.match(*selectorData.selector, element, selectorCheckingContext, ignoredSpecificity))
+    if (!selectorChecker.match(*selectorData.selector, element, selectorCheckingContext))
         return nullptr;
     return &element;
 }
@@ -151,52 +160,32 @@ bool SelectorDataList::matches(Element& targetElement) const
 
 Element* SelectorDataList::closest(Element& targetElement) const
 {
-    Element* currentNode = &targetElement;
-    do {
+    for (auto& currentElement : lineageOfType<Element>(targetElement)) {
         for (auto& selector : m_selectors) {
-            Element* candidateElement = selectorClosest(selector, *currentNode, targetElement);
-            if (candidateElement)
+            if (auto* candidateElement = selectorClosest(selector, currentElement, targetElement))
                 return candidateElement;
         }
-        currentNode = currentNode->parentElement();
-    } while (currentNode);
+    }
     return nullptr;
 }
 
-struct AllElementExtractorSelectorQueryTrait {
-    typedef Vector<Ref<Element>> OutputType;
-    static const bool shouldOnlyMatchFirstElement = false;
-    ALWAYS_INLINE static void appendOutputForElement(OutputType& output, Element* element) { ASSERT(element); output.append(*element); }
-};
-
-RefPtr<NodeList> SelectorDataList::queryAll(ContainerNode& rootNode) const
+Ref<NodeList> SelectorDataList::queryAll(ContainerNode& rootNode) const
 {
     Vector<Ref<Element>> result;
-    execute<AllElementExtractorSelectorQueryTrait>(rootNode, result);
+    execute(rootNode, result);
     return StaticElementList::create(WTFMove(result));
 }
 
-struct SingleElementExtractorSelectorQueryTrait {
-    typedef Element* OutputType;
-    static const bool shouldOnlyMatchFirstElement = true;
-    ALWAYS_INLINE static void appendOutputForElement(OutputType& output, Element* element)
-    {
-        ASSERT(element);
-        ASSERT(!output);
-        output = element;
-    }
-};
-
 Element* SelectorDataList::queryFirst(ContainerNode& rootNode) const
 {
     Element* result = nullptr;
-    execute<SingleElementExtractorSelectorQueryTrait>(rootNode, result);
+    execute(rootNode, result);
     return result;
 }
 
 static const CSSSelector* selectorForIdLookup(const ContainerNode& rootNode, const CSSSelector& firstSelector)
 {
-    if (!rootNode.inDocument())
+    if (!rootNode.isConnected())
         return nullptr;
     if (rootNode.document().inQuirksMode())
         return nullptr;
@@ -211,26 +200,21 @@ static const CSSSelector* selectorForIdLookup(const ContainerNode& rootNode, con
     return nullptr;
 }
 
-static inline bool isTreeScopeRoot(const ContainerNode& node)
-{
-    return node.isDocumentNode() || node.isShadowRoot();
-}
-
-template <typename SelectorQueryTrait>
-ALWAYS_INLINE void SelectorDataList::executeFastPathForIdSelector(const ContainerNode& rootNode, const SelectorData& selectorData, const CSSSelector* idSelector, typename SelectorQueryTrait::OutputType& output) const
+template<typename OutputType>
+ALWAYS_INLINE void SelectorDataList::executeFastPathForIdSelector(const ContainerNode& rootNode, const SelectorData& selectorData, const CSSSelector* idSelector, OutputType& output) const
 {
     ASSERT(m_selectors.size() == 1);
     ASSERT(idSelector);
 
-    const AtomicString& idToMatch = idSelector->value();
+    const AtomString& idToMatch = idSelector->value();
     if (UNLIKELY(rootNode.treeScope().containsMultipleElementsWithId(idToMatch))) {
         const Vector<Element*>* elements = rootNode.treeScope().getAllElementsById(idToMatch);
         ASSERT(elements);
-        bool rootNodeIsTreeScopeRoot = isTreeScopeRoot(rootNode);
+        bool rootNodeIsTreeScopeRoot = rootNode.isTreeScope();
         for (auto& element : *elements) {
-            if ((rootNodeIsTreeScopeRoot || element->isDescendantOf(&rootNode)) && selectorMatches(selectorData, *element, rootNode)) {
-                SelectorQueryTrait::appendOutputForElement(output, element);
-                if (SelectorQueryTrait::shouldOnlyMatchFirstElement)
+            if ((rootNodeIsTreeScopeRoot || element->isDescendantOf(rootNode)) && selectorMatches(selectorData, *element, rootNode)) {
+                appendOutputForElement(output, *element);
+                if constexpr (std::is_same_v<OutputType, Element*>)
                     return;
             }
         }
@@ -238,15 +222,15 @@ ALWAYS_INLINE void SelectorDataList::executeFastPathForIdSelector(const Containe
     }
 
     Element* element = rootNode.treeScope().getElementById(idToMatch);
-    if (!element || !(isTreeScopeRoot(rootNode) || element->isDescendantOf(&rootNode)))
+    if (!element || !(rootNode.isTreeScope() || element->isDescendantOf(rootNode)))
         return;
     if (selectorMatches(selectorData, *element, rootNode))
-        SelectorQueryTrait::appendOutputForElement(output, element);
+        appendOutputForElement(output, *element);
 }
 
 static ContainerNode& filterRootById(ContainerNode& rootNode, const CSSSelector& firstSelector)
 {
-    if (!rootNode.inDocument())
+    if (!rootNode.isConnected())
         return rootNode;
     if (rootNode.document().inQuirksMode())
         return rootNode;
@@ -264,27 +248,24 @@ static ContainerNode& filterRootById(ContainerNode& rootNode, const CSSSelector&
     bool inAdjacentChain = false;
     for (; selector; selector = selector->tagHistory()) {
         if (canBeUsedForIdFastPath(*selector)) {
-            const AtomicString& idToMatch = selector->value();
+            const AtomString& idToMatch = selector->value();
             if (ContainerNode* searchRoot = rootNode.treeScope().getElementById(idToMatch)) {
                 if (LIKELY(!rootNode.treeScope().containsMultipleElementsWithId(idToMatch))) {
                     if (inAdjacentChain)
                         searchRoot = searchRoot->parentNode();
-                    if (searchRoot && (isTreeScopeRoot(rootNode) || searchRoot == &rootNode || searchRoot->isDescendantOf(&rootNode)))
+                    if (searchRoot && (rootNode.isTreeScope() || searchRoot == &rootNode || searchRoot->isDescendantOf(rootNode)))
                         return *searchRoot;
                 }
             }
         }
         if (selector->relation() == CSSSelector::Subselector)
             continue;
-        if (selector->relation() == CSSSelector::DirectAdjacent || selector->relation() == CSSSelector::IndirectAdjacent)
-            inAdjacentChain = true;
-        else
-            inAdjacentChain = false;
+        inAdjacentChain = selector->relation() == CSSSelector::DirectAdjacent || selector->relation() == CSSSelector::IndirectAdjacent;
     }
     return rootNode;
 }
 
-static ALWAYS_INLINE bool localNameMatches(const Element& element, const AtomicString& localName, const AtomicString& lowercaseLocalName)
+static ALWAYS_INLINE bool localNameMatches(const Element& element, const AtomString& localName, const AtomString& lowercaseLocalName)
 {
     if (element.isHTMLElement() && element.document().isHTMLDocument())
         return element.localName() == lowercaseLocalName;
@@ -292,108 +273,108 @@ static ALWAYS_INLINE bool localNameMatches(const Element& element, const AtomicS
 
 }
 
-template <typename SelectorQueryTrait>
-static inline void elementsForLocalName(const ContainerNode& rootNode, const AtomicString& localName, const AtomicString& lowercaseLocalName, typename SelectorQueryTrait::OutputType& output)
+template<typename OutputType>
+static inline void elementsForLocalName(const ContainerNode& rootNode, const AtomString& localName, const AtomString& lowercaseLocalName, OutputType& output)
 {
     if (localName == lowercaseLocalName) {
-        for (auto& element : elementDescendants(const_cast<ContainerNode&>(rootNode))) {
+        for (auto& element : descendantsOfType<Element>(const_cast<ContainerNode&>(rootNode))) {
             if (element.tagQName().localName() == localName) {
-                SelectorQueryTrait::appendOutputForElement(output, &element);
-                if (SelectorQueryTrait::shouldOnlyMatchFirstElement)
+                appendOutputForElement(output, element);
+                if constexpr (std::is_same_v<OutputType, Element*>)
                 return;
             }
         }
     } else {
-        for (auto& element : elementDescendants(const_cast<ContainerNode&>(rootNode))) {
+        for (auto& element : descendantsOfType<Element>(const_cast<ContainerNode&>(rootNode))) {
             if (localNameMatches(element, localName, lowercaseLocalName)) {
-                SelectorQueryTrait::appendOutputForElement(output, &element);
-                if (SelectorQueryTrait::shouldOnlyMatchFirstElement)
+                appendOutputForElement(output, element);
+                if constexpr (std::is_same_v<OutputType, Element*>)
                 return;
             }
         }
     }
 }
 
-template <typename SelectorQueryTrait>
-static inline void anyElement(const ContainerNode& rootNode, typename SelectorQueryTrait::OutputType& output)
+template<typename OutputType>
+static inline void anyElement(const ContainerNode& rootNode, OutputType& output)
 {
-    for (auto& element : elementDescendants(const_cast<ContainerNode&>(rootNode))) {
-        SelectorQueryTrait::appendOutputForElement(output, &element);
-        if (SelectorQueryTrait::shouldOnlyMatchFirstElement)
+    for (auto& element : descendantsOfType<Element>(const_cast<ContainerNode&>(rootNode))) {
+        appendOutputForElement(output, element);
+        if constexpr (std::is_same_v<OutputType, Element*>)
             return;
     }
 }
 
 
-template <typename SelectorQueryTrait>
-ALWAYS_INLINE void SelectorDataList::executeSingleTagNameSelectorData(const ContainerNode& rootNode, const SelectorData& selectorData, typename SelectorQueryTrait::OutputType& output) const
+template<typename OutputType>
+ALWAYS_INLINE void SelectorDataList::executeSingleTagNameSelectorData(const ContainerNode& rootNode, const SelectorData& selectorData, OutputType& output) const
 {
     ASSERT(m_selectors.size() == 1);
     ASSERT(isSingleTagNameSelector(*selectorData.selector));
 
     const QualifiedName& tagQualifiedName = selectorData.selector->tagQName();
-    const AtomicString& selectorLocalName = tagQualifiedName.localName();
-    const AtomicString& selectorLowercaseLocalName = selectorData.selector->tagLowercaseLocalName();
-    const AtomicString& selectorNamespaceURI = tagQualifiedName.namespaceURI();
+    const AtomString& selectorLocalName = tagQualifiedName.localName();
+    const AtomString& selectorLowercaseLocalName = selectorData.selector->tagLowercaseLocalName();
+    const AtomString& selectorNamespaceURI = tagQualifiedName.namespaceURI();
 
-    if (selectorNamespaceURI == starAtom) {
-        if (selectorLocalName != starAtom) {
+    if (selectorNamespaceURI == starAtom()) {
+        if (selectorLocalName != starAtom()) {
             // Common case: name defined, selectorNamespaceURI is a wildcard.
-            elementsForLocalName<SelectorQueryTrait>(rootNode, selectorLocalName, selectorLowercaseLocalName, output);
+            elementsForLocalName(rootNode, selectorLocalName, selectorLowercaseLocalName, output);
         } else {
             // Other fairly common case: both are wildcards.
-            anyElement<SelectorQueryTrait>(rootNode, output);
+            anyElement(rootNode, output);
         }
     } else {
-        // Fallback: NamespaceURI is set, selectorLocalName may be starAtom.
-        for (auto& element : elementDescendants(const_cast<ContainerNode&>(rootNode))) {
+        // Fallback: NamespaceURI is set, selectorLocalName may be starAtom().
+        for (auto& element : descendantsOfType<Element>(const_cast<ContainerNode&>(rootNode))) {
             if (element.namespaceURI() == selectorNamespaceURI && localNameMatches(element, selectorLocalName, selectorLowercaseLocalName)) {
-                SelectorQueryTrait::appendOutputForElement(output, &element);
-                if (SelectorQueryTrait::shouldOnlyMatchFirstElement)
+                appendOutputForElement(output, element);
+                if constexpr (std::is_same_v<OutputType, Element*>)
                     return;
             }
         }
     }
 }
 
-template <typename SelectorQueryTrait>
-ALWAYS_INLINE void SelectorDataList::executeSingleClassNameSelectorData(const ContainerNode& rootNode, const SelectorData& selectorData, typename SelectorQueryTrait::OutputType& output) const
+template<typename OutputType>
+ALWAYS_INLINE void SelectorDataList::executeSingleClassNameSelectorData(const ContainerNode& rootNode, const SelectorData& selectorData, OutputType& output) const
 {
     ASSERT(m_selectors.size() == 1);
     ASSERT(isSingleClassNameSelector(*selectorData.selector));
 
-    const AtomicString& className = selectorData.selector->value();
-    for (auto& element : elementDescendants(const_cast<ContainerNode&>(rootNode))) {
+    const AtomString& className = selectorData.selector->value();
+    for (auto& element : descendantsOfType<Element>(const_cast<ContainerNode&>(rootNode))) {
         if (element.hasClass() && element.classNames().contains(className)) {
-            SelectorQueryTrait::appendOutputForElement(output, &element);
-            if (SelectorQueryTrait::shouldOnlyMatchFirstElement)
+            appendOutputForElement(output, element);
+            if constexpr (std::is_same_v<OutputType, Element*>)
                 return;
         }
     }
 }
 
-template <typename SelectorQueryTrait>
-ALWAYS_INLINE void SelectorDataList::executeSingleSelectorData(const ContainerNode& rootNode, const ContainerNode& searchRootNode, const SelectorData& selectorData, typename SelectorQueryTrait::OutputType& output) const
+template<typename OutputType>
+ALWAYS_INLINE void SelectorDataList::executeSingleSelectorData(const ContainerNode& rootNode, const ContainerNode& searchRootNode, const SelectorData& selectorData, OutputType& output) const
 {
     ASSERT(m_selectors.size() == 1);
 
-    for (auto& element : elementDescendants(const_cast<ContainerNode&>(searchRootNode))) {
+    for (auto& element : descendantsOfType<Element>(const_cast<ContainerNode&>(searchRootNode))) {
         if (selectorMatches(selectorData, element, rootNode)) {
-            SelectorQueryTrait::appendOutputForElement(output, &element);
-            if (SelectorQueryTrait::shouldOnlyMatchFirstElement)
+            appendOutputForElement(output, element);
+            if constexpr (std::is_same_v<OutputType, Element*>)
                 return;
         }
     }
 }
 
-template <typename SelectorQueryTrait>
-ALWAYS_INLINE void SelectorDataList::executeSingleMultiSelectorData(const ContainerNode& rootNode, typename SelectorQueryTrait::OutputType& output) const
+template<typename OutputType>
+ALWAYS_INLINE void SelectorDataList::executeSingleMultiSelectorData(const ContainerNode& rootNode, OutputType& output) const
 {
-    for (auto& element : elementDescendants(const_cast<ContainerNode&>(rootNode))) {
+    for (auto& element : descendantsOfType<Element>(const_cast<ContainerNode&>(rootNode))) {
         for (auto& selector : m_selectors) {
             if (selectorMatches(selector, element, rootNode)) {
-                SelectorQueryTrait::appendOutputForElement(output, &element);
-                if (SelectorQueryTrait::shouldOnlyMatchFirstElement)
+                appendOutputForElement(output, element);
+                if constexpr (std::is_same_v<OutputType, Element*>)
                     return;
                 break;
             }
@@ -402,66 +383,56 @@ ALWAYS_INLINE void SelectorDataList::executeSingleMultiSelectorData(const Contai
 }
 
 #if ENABLE(CSS_SELECTOR_JIT)
-template <typename SelectorQueryTrait>
-ALWAYS_INLINE void SelectorDataList::executeCompiledSimpleSelectorChecker(const ContainerNode& searchRootNode, SelectorCompiler::QuerySelectorSimpleSelectorChecker selectorChecker, typename SelectorQueryTrait::OutputType& output, const SelectorData& selectorData) const
+template<typename Checker, typename OutputType>
+ALWAYS_INLINE void SelectorDataList::executeCompiledSimpleSelectorChecker(const ContainerNode& searchRootNode, Checker selectorChecker, OutputType& output, const SelectorData& selectorData) const
 {
-    for (auto& element : elementDescendants(const_cast<ContainerNode&>(searchRootNode))) {
-#if CSS_SELECTOR_JIT_PROFILING
-        selectorData.compiledSelectorUsed();
-#else
-        UNUSED_PARAM(selectorData);
-#endif
+    for (auto& element : descendantsOfType<Element>(const_cast<ContainerNode&>(searchRootNode))) {
+        selectorData.compiledSelector.wasUsed();
+
         if (selectorChecker(&element)) {
-            SelectorQueryTrait::appendOutputForElement(output, &element);
-            if (SelectorQueryTrait::shouldOnlyMatchFirstElement)
+            appendOutputForElement(output, element);
+            if constexpr (std::is_same_v<OutputType, Element*>)
                 return;
         }
     }
 }
 
-template <typename SelectorQueryTrait>
-ALWAYS_INLINE void SelectorDataList::executeCompiledSelectorCheckerWithCheckingContext(const ContainerNode& rootNode, const ContainerNode& searchRootNode, SelectorCompiler::QuerySelectorSelectorCheckerWithCheckingContext selectorChecker, typename SelectorQueryTrait::OutputType& output, const SelectorData& selectorData) const
+template<typename Checker, typename OutputType>
+ALWAYS_INLINE void SelectorDataList::executeCompiledSelectorCheckerWithCheckingContext(const ContainerNode& rootNode, const ContainerNode& searchRootNode, Checker selectorChecker, OutputType& output, const SelectorData& selectorData) const
 {
     SelectorChecker::CheckingContext checkingContext(SelectorChecker::Mode::QueryingRules);
     checkingContext.scope = rootNode.isDocumentNode() ? nullptr : &rootNode;
 
-    for (auto& element : elementDescendants(const_cast<ContainerNode&>(searchRootNode))) {
-#if CSS_SELECTOR_JIT_PROFILING
-        selectorData.compiledSelectorUsed();
-#else
-        UNUSED_PARAM(selectorData);
-#endif
+    for (auto& element : descendantsOfType<Element>(const_cast<ContainerNode&>(searchRootNode))) {
+        selectorData.compiledSelector.wasUsed();
+
         if (selectorChecker(&element, &checkingContext)) {
-            SelectorQueryTrait::appendOutputForElement(output, &element);
-            if (SelectorQueryTrait::shouldOnlyMatchFirstElement)
+            appendOutputForElement(output, element);
+            if constexpr (std::is_same_v<OutputType, Element*>)
                 return;
         }
     }
 }
 
-template <typename SelectorQueryTrait>
-ALWAYS_INLINE void SelectorDataList::executeCompiledSingleMultiSelectorData(const ContainerNode& rootNode, typename SelectorQueryTrait::OutputType& output) const
+template<typename OutputType>
+ALWAYS_INLINE void SelectorDataList::executeCompiledSingleMultiSelectorData(const ContainerNode& rootNode, OutputType& output) const
 {
     SelectorChecker::CheckingContext checkingContext(SelectorChecker::Mode::QueryingRules);
     checkingContext.scope = rootNode.isDocumentNode() ? nullptr : &rootNode;
-    for (auto& element : elementDescendants(const_cast<ContainerNode&>(rootNode))) {
+    for (auto& element : descendantsOfType<Element>(const_cast<ContainerNode&>(rootNode))) {
         for (auto& selector : m_selectors) {
-#if CSS_SELECTOR_JIT_PROFILING
-            selector.compiledSelectorUsed();
-#endif
+            selector.compiledSelector.wasUsed();
+
             bool matched = false;
-            void* compiledSelectorChecker = selector.compiledSelectorCodeRef.code().executableAddress();
-            if (selector.compilationStatus == SelectorCompilationStatus::SimpleSelectorChecker) {
-                auto selectorChecker = SelectorCompiler::querySelectorSimpleSelectorCheckerFunction(compiledSelectorChecker, selector.compilationStatus);
-                matched = selectorChecker(&element);
-            } else {
-                ASSERT(selector.compilationStatus == SelectorCompilationStatus::SelectorCheckerWithCheckingContext);
-                auto selectorChecker = SelectorCompiler::querySelectorSelectorCheckerFunctionWithCheckingContext(compiledSelectorChecker, selector.compilationStatus);
-                matched = selectorChecker(&element, &checkingContext);
+            if (selector.compiledSelector.status == SelectorCompilationStatus::SimpleSelectorChecker)
+                matched = SelectorCompiler::querySelectorSimpleSelectorChecker(selector.compiledSelector, &element);
+            else {
+                ASSERT(selector.compiledSelector.status == SelectorCompilationStatus::SelectorCheckerWithCheckingContext);
+                matched = SelectorCompiler::querySelectorSelectorCheckerWithCheckingContext(selector.compiledSelector, &element, &checkingContext);
             }
             if (matched) {
-                SelectorQueryTrait::appendOutputForElement(output, &element);
-                if (SelectorQueryTrait::shouldOnlyMatchFirstElement)
+                appendOutputForElement(output, element);
+                if constexpr (std::is_same_v<OutputType, Element*>)
                     return;
                 break;
             }
@@ -469,26 +440,20 @@ ALWAYS_INLINE void SelectorDataList::executeCompiledSingleMultiSelectorData(cons
     }
 }
 
-static bool isCompiledSelector(SelectorCompilationStatus compilationStatus)
+bool SelectorDataList::compileSelector(const SelectorData& selectorData)
 {
-    return compilationStatus == SelectorCompilationStatus::SimpleSelectorChecker || compilationStatus == SelectorCompilationStatus::SelectorCheckerWithCheckingContext;
+    auto& compiledSelector = selectorData.compiledSelector;
+    
+    if (compiledSelector.status == SelectorCompilationStatus::NotCompiled)
+        SelectorCompiler::compileSelector(compiledSelector, selectorData.selector, SelectorCompiler::SelectorContext::QuerySelector);
+
+    return compiledSelector.status != SelectorCompilationStatus::CannotCompile;
 }
 
-bool SelectorDataList::compileSelector(const SelectorData& selectorData, const ContainerNode& rootNode)
-{
-    if (selectorData.compilationStatus != SelectorCompilationStatus::NotCompiled)
-        return isCompiledSelector(selectorData.compilationStatus);
-
-    JSC::VM& vm = rootNode.document().scriptExecutionContext()->vm();
-    selectorData.compilationStatus = SelectorCompiler::compileSelector(selectorData.selector, &vm, SelectorCompiler::SelectorContext::QuerySelector, selectorData.compiledSelectorCodeRef);
-    return isCompiledSelector(selectorData.compilationStatus);
-}
-
-
 #endif // ENABLE(CSS_SELECTOR_JIT)
 
-template <typename SelectorQueryTrait>
-ALWAYS_INLINE void SelectorDataList::execute(ContainerNode& rootNode, typename SelectorQueryTrait::OutputType& output) const
+template<typename OutputType>
+ALWAYS_INLINE void SelectorDataList::execute(ContainerNode& rootNode, OutputType& output) const
 {
     ContainerNode* searchRootNode = &rootNode;
     switch (m_matchType) {
@@ -496,11 +461,11 @@ ALWAYS_INLINE void SelectorDataList::execute(ContainerNode& rootNode, typename S
         {
         const SelectorData& selectorData = m_selectors.first();
         if (const CSSSelector* idSelector = selectorForIdLookup(*searchRootNode, *selectorData.selector)) {
-            executeFastPathForIdSelector<SelectorQueryTrait>(*searchRootNode, m_selectors.first(), idSelector, output);
+            executeFastPathForIdSelector(*searchRootNode, m_selectors.first(), idSelector, output);
             break;
         }
 #if ENABLE(CSS_SELECTOR_JIT)
-        if (compileSelector(selectorData, *searchRootNode))
+        if (compileSelector(selectorData))
             goto CompiledSingleCase;
 #endif // ENABLE(CSS_SELECTOR_JIT)
         goto SingleSelectorCase;
@@ -512,9 +477,9 @@ ALWAYS_INLINE void SelectorDataList::execute(ContainerNode& rootNode, typename S
         {
 #if ENABLE(CSS_SELECTOR_JIT)
         const SelectorData& selectorData = m_selectors.first();
-        ASSERT(selectorData.compilationStatus == SelectorCompilationStatus::NotCompiled);
+        ASSERT(selectorData.compiledSelector.status == SelectorCompilationStatus::NotCompiled);
         ASSERT(m_matchType == CompilableSingle || m_matchType == CompilableSingleWithRootFilter);
-        if (compileSelector(selectorData, *searchRootNode)) {
+        if (compileSelector(selectorData)) {
             if (m_matchType == CompilableSingle) {
                 m_matchType = CompiledSingle;
                 goto CompiledSingleCase;
@@ -543,14 +508,15 @@ ALWAYS_INLINE void SelectorDataList::execute(ContainerNode& rootNode, typename S
         {
         CompiledSingleCase:
         const SelectorData& selectorData = m_selectors.first();
-        void* compiledSelectorChecker = selectorData.compiledSelectorCodeRef.code().executableAddress();
-        if (selectorData.compilationStatus == SelectorCompilationStatus::SimpleSelectorChecker) {
-            SelectorCompiler::QuerySelectorSimpleSelectorChecker selectorChecker = SelectorCompiler::querySelectorSimpleSelectorCheckerFunction(compiledSelectorChecker, selectorData.compilationStatus);
-            executeCompiledSimpleSelectorChecker<SelectorQueryTrait>(*searchRootNode, selectorChecker, output, selectorData);
+        if (selectorData.compiledSelector.status == SelectorCompilationStatus::SimpleSelectorChecker) {
+            executeCompiledSimpleSelectorChecker(*searchRootNode, [&] (const Element* element) {
+                return SelectorCompiler::querySelectorSimpleSelectorChecker(selectorData.compiledSelector, element);
+            }, output, selectorData);
         } else {
-            ASSERT(selectorData.compilationStatus == SelectorCompilationStatus::SelectorCheckerWithCheckingContext);
-            SelectorCompiler::QuerySelectorSelectorCheckerWithCheckingContext selectorChecker = SelectorCompiler::querySelectorSelectorCheckerFunctionWithCheckingContext(compiledSelectorChecker, selectorData.compilationStatus);
-            executeCompiledSelectorCheckerWithCheckingContext<SelectorQueryTrait>(rootNode, *searchRootNode, selectorChecker, output, selectorData);
+            ASSERT(selectorData.compiledSelector.status == SelectorCompilationStatus::SelectorCheckerWithCheckingContext);
+            executeCompiledSelectorCheckerWithCheckingContext(rootNode, *searchRootNode, [&] (const Element* element, const SelectorChecker::CheckingContext* context) {
+                return SelectorCompiler::querySelectorSelectorCheckerWithCheckingContext(selectorData.compiledSelector, element, context);
+            }, output, selectorData);
         }
         break;
         }
@@ -558,7 +524,7 @@ ALWAYS_INLINE void SelectorDataList::execute(ContainerNode& rootNode, typename S
     case CompiledSingleWithRootFilter:
     case CompiledSingle:
         ASSERT_NOT_REACHED();
-#if ASSERT_DISABLED
+#if !ASSERT_ENABLED
         FALLTHROUGH;
 #endif
 #endif // ENABLE(CSS_SELECTOR_JIT)
@@ -569,20 +535,20 @@ ALWAYS_INLINE void SelectorDataList::execute(ContainerNode& rootNode, typename S
         FALLTHROUGH;
     case SingleSelector:
         SingleSelectorCase:
-        executeSingleSelectorData<SelectorQueryTrait>(rootNode, *searchRootNode, m_selectors.first(), output);
+        executeSingleSelectorData(rootNode, *searchRootNode, m_selectors.first(), output);
         break;
 
     case TagNameMatch:
-        executeSingleTagNameSelectorData<SelectorQueryTrait>(*searchRootNode, m_selectors.first(), output);
+        executeSingleTagNameSelectorData(*searchRootNode, m_selectors.first(), output);
         break;
     case ClassNameMatch:
-        executeSingleClassNameSelectorData<SelectorQueryTrait>(*searchRootNode, m_selectors.first(), output);
+        executeSingleClassNameSelectorData(*searchRootNode, m_selectors.first(), output);
         break;
     case CompilableMultipleSelectorMatch:
 #if ENABLE(CSS_SELECTOR_JIT)
         {
         for (auto& selector : m_selectors) {
-            if (!compileSelector(selector, *searchRootNode)) {
+            if (!compileSelector(selector)) {
                 m_matchType = MultipleSelectorMatch;
                 goto MultipleSelectorMatch;
             }
@@ -596,7 +562,7 @@ ALWAYS_INLINE void SelectorDataList::execute(ContainerNode& rootNode, typename S
     case CompiledMultipleSelectorMatch:
 #if ENABLE(CSS_SELECTOR_JIT)
         CompiledMultipleSelectorMatch:
-        executeCompiledSingleMultiSelectorData<SelectorQueryTrait>(*searchRootNode, output);
+        executeCompiledSingleMultiSelectorData(*searchRootNode, output);
         break;
 #else
         FALLTHROUGH;
@@ -605,7 +571,7 @@ ALWAYS_INLINE void SelectorDataList::execute(ContainerNode& rootNode, typename S
 #if ENABLE(CSS_SELECTOR_JIT)
         MultipleSelectorMatch:
 #endif
-        executeSingleMultiSelectorData<SelectorQueryTrait>(*searchRootNode, output);
+        executeSingleMultiSelectorData(*searchRootNode, output);
         break;
     }
 }
@@ -616,31 +582,43 @@ SelectorQuery::SelectorQuery(CSSSelectorList&& selectorList)
 {
 }
 
-SelectorQuery* SelectorQueryCache::add(const String& selectors, Document& document, ExceptionCode& ec)
+SelectorQueryCache& SelectorQueryCache::singleton()
 {
-    auto it = m_entries.find(selectors);
-    if (it != m_entries.end())
-        return it->value.get();
+    static NeverDestroyed<SelectorQueryCache> cache;
+    return cache.get();
+}
 
-    CSSParser parser(document);
-    CSSSelectorList selectorList;
-    parser.parseSelector(selectors, selectorList);
+SelectorQuery* SelectorQueryCache::add(const String& selectors, const Document& document)
+{
+    ASSERT(!selectors.isEmpty());
 
-    if (!selectorList.first() || selectorList.hasInvalidSelector()) {
-        ec = SYNTAX_ERR;
-        return nullptr;
-    }
-
-    if (selectorList.selectorsNeedNamespaceResolution()) {
-        ec = SYNTAX_ERR;
-        return nullptr;
-    }
-
-    const int maximumSelectorQueryCacheSize = 256;
+    constexpr auto maximumSelectorQueryCacheSize = 512;
     if (m_entries.size() == maximumSelectorQueryCacheSize)
-        m_entries.remove(m_entries.begin());
+        m_entries.remove(m_entries.random());
 
-    return m_entries.add(selectors, std::make_unique<SelectorQuery>(WTFMove(selectorList))).iterator->value.get();
+    auto context = CSSSelectorParserContext { document };
+    auto key = Key { selectors, context, document.securityOrigin().data() };
+
+    return m_entries.ensure(key, [&]() -> std::unique_ptr<SelectorQuery> {
+        auto tokenizer = CSSTokenizer { selectors };
+        auto selectorList = parseCSSSelector(tokenizer.tokenRange(), context, nullptr, CSSParserEnum::IsNestedContext::No);
+
+        if (!selectorList || selectorList->hasInvalidSelector())
+            return nullptr;
+
+        if (selectorList->selectorsNeedNamespaceResolution())
+            return nullptr;
+
+        if (selectorList->hasExplicitNestingParent())
+            selectorList = CSSSelectorParser::resolveNestingParent(WTFMove(*selectorList), nullptr);
+
+        return makeUnique<SelectorQuery>(WTFMove(*selectorList));
+    }).iterator->value.get();
+}
+
+void SelectorQueryCache::clear()
+{
+    m_entries.clear();
 }
 
 }
