@@ -28,11 +28,9 @@
 #include "TreeScope.h"
 
 #include "Attr.h"
-#include "DOMWindow.h"
-#include "ElementIterator.h"
+#include "CSSStyleSheet.h"
+#include "CSSStyleSheetObservableArray.h"
 #include "FocusController.h"
-#include "Frame.h"
-#include "FrameView.h"
 #include "HTMLAnchorElement.h"
 #include "HTMLFrameOwnerElement.h"
 #include "HTMLImageElement.h"
@@ -40,6 +38,10 @@
 #include "HTMLMapElement.h"
 #include "HitTestResult.h"
 #include "IdTargetObserverRegistry.h"
+#include "JSObservableArray.h"
+#include "LocalDOMWindow.h"
+#include "LocalFrame.h"
+#include "LocalFrameView.h"
 #include "NodeRareData.h"
 #include "Page.h"
 #include "PointerLockController.h"
@@ -48,15 +50,16 @@
 #include "RenderView.h"
 #include "Settings.h"
 #include "ShadowRoot.h"
+#include "TypedElementDescendantIteratorInlines.h"
 #include <wtf/text/CString.h>
 
 namespace WebCore {
 
 struct SameSizeAsTreeScope {
-    void* pointers[10];
+    void* pointers[11];
 };
 
-COMPILE_ASSERT(sizeof(TreeScope) == sizeof(SameSizeAsTreeScope), treescope_should_stay_small);
+static_assert(sizeof(TreeScope) == sizeof(SameSizeAsTreeScope), "treescope should stay small");
 
 using namespace HTMLNames;
 
@@ -64,7 +67,7 @@ TreeScope::TreeScope(ShadowRoot& shadowRoot, Document& document)
     : m_rootNode(shadowRoot)
     , m_documentScope(document)
     , m_parentTreeScope(&document)
-    , m_idTargetObserverRegistry(makeUnique<IdTargetObserverRegistry>())
+    , m_idTargetObserverRegistry(makeUniqueRef<IdTargetObserverRegistry>())
 {
     shadowRoot.setTreeScope(*this);
 }
@@ -73,7 +76,7 @@ TreeScope::TreeScope(Document& document)
     : m_rootNode(document)
     , m_documentScope(document)
     , m_parentTreeScope(nullptr)
-    , m_idTargetObserverRegistry(makeUnique<IdTargetObserverRegistry>())
+    , m_idTargetObserverRegistry(makeUniqueRef<IdTargetObserverRegistry>())
 {
     document.setTreeScope(*this);
 }
@@ -123,8 +126,8 @@ Element* TreeScope::getElementById(StringView elementId) const
     if (!m_elementsById)
         return nullptr;
 
-    if (auto atomElementId = elementId.toExistingAtomString())
-        return m_elementsById->getElementById(*atomElementId, *this);
+    if (auto atomElementId = elementId.toExistingAtomString(); !atomElementId.isNull())
+        return m_elementsById->getElementById(*atomElementId.impl(), *this);
 
     return nullptr;
 }
@@ -312,21 +315,21 @@ HTMLLabelElement* TreeScope::labelElementForId(const AtomString& forAttributeVal
     return m_labelsByForAttribute->getElementByLabelForAttribute(*forAttributeValue.impl(), *this);
 }
 
-static Optional<LayoutPoint> absolutePointIfNotClipped(Document& document, const LayoutPoint& clientPoint)
+static std::optional<LayoutPoint> absolutePointIfNotClipped(Document& document, const LayoutPoint& clientPoint)
 {
     if (!document.frame() || !document.view())
-        return WTF::nullopt;
+        return std::nullopt;
 
     const auto& settings = document.frame()->settings();
     if (settings.visualViewportEnabled() && settings.clientCoordinatesRelativeToLayoutViewport()) {
         document.updateLayout();
         if (!document.view() || !document.hasLivingRenderTree())
-            return WTF::nullopt;
+            return std::nullopt;
         auto* view = document.view();
         FloatPoint layoutViewportPoint = view->clientToLayoutViewportPoint(clientPoint);
         FloatRect layoutViewportBounds({ }, view->layoutViewportRect().size());
         if (!layoutViewportBounds.contains(layoutViewportPoint))
-            return WTF::nullopt;
+            return std::nullopt;
         return LayoutPoint(view->layoutViewportToAbsolutePoint(layoutViewportPoint));
     }
 
@@ -346,7 +349,7 @@ static Optional<LayoutPoint> absolutePointIfNotClipped(Document& document, const
 #endif
     if (visibleRect.contains(absolutePoint))
         return absolutePoint;
-    return WTF::nullopt;
+    return std::nullopt;
 }
 
 RefPtr<Node> TreeScope::nodeFromPoint(const LayoutPoint& clientPoint, LayoutPoint* localPoint)
@@ -380,7 +383,7 @@ RefPtr<Element> TreeScope::elementFromPoint(double clientX, double clientY)
         node = retargetToScope(*node);
     }
 
-    return static_pointer_cast<Element>(node);
+    return static_pointer_cast<Element>(WTFMove(node));
 }
 
 Vector<RefPtr<Element>> TreeScope::elementsFromPoint(double clientX, double clientY)
@@ -395,7 +398,7 @@ Vector<RefPtr<Element>> TreeScope::elementsFromPoint(double clientX, double clie
     if (!absolutePoint)
         return elements;
 
-    constexpr OptionSet<HitTestRequest::RequestType> hitType { HitTestRequest::ReadOnly, HitTestRequest::Active, HitTestRequest::DisallowUserAgentShadowContent, HitTestRequest::CollectMultipleElements, HitTestRequest::IncludeAllElementsUnderPoint };
+    constexpr OptionSet<HitTestRequest::Type> hitType { HitTestRequest::Type::ReadOnly, HitTestRequest::Type::Active, HitTestRequest::Type::DisallowUserAgentShadowContent, HitTestRequest::Type::CollectMultipleElements, HitTestRequest::Type::IncludeAllElementsUnderPoint };
     HitTestResult result { absolutePoint.value() };
     documentScope().hitTest(hitType, result);
 
@@ -414,8 +417,8 @@ Vector<RefPtr<Element>> TreeScope::elementsFromPoint(double clientX, double clie
         if (!node)
             continue;
 
-        if (is<PseudoElement>(node))
-            node = downcast<PseudoElement>(*node).hostElement();
+        if (auto pseudoElement = dynamicDowncast<PseudoElement>(*node))
+            node = pseudoElement->hostElement();
 
         // Prune duplicate entries. A pseudo ::before content above its parent
         // node should only result in one entry.
@@ -443,7 +446,7 @@ Vector<RefPtr<Element>> TreeScope::elementsFromPoint(const FloatPoint& p)
 
 // FIXME: Would be nice to change this to take a StringView, since that's what callers have
 // and there is no particular advantage to already having a String.
-Element* TreeScope::findAnchor(const String& name)
+Element* TreeScope::findAnchor(StringView name)
 {
     if (name.isEmpty())
         return nullptr;
@@ -465,11 +468,11 @@ Element* TreeScope::findAnchor(const String& name)
     return nullptr;
 }
 
-static Element* focusedFrameOwnerElement(Frame* focusedFrame, Frame* currentFrame)
+static Element* focusedFrameOwnerElement(Frame* focusedFrame, LocalFrame* currentFrame)
 {
     for (; focusedFrame; focusedFrame = focusedFrame->tree().parent()) {
         if (focusedFrame->tree().parent() == currentFrame)
-            return focusedFrame->ownerElement();
+            return is<LocalFrame>(focusedFrame) ? downcast<LocalFrame>(focusedFrame)->ownerElement() : nullptr;
     }
     return nullptr;
 }
@@ -545,4 +548,28 @@ RadioButtonGroups& TreeScope::radioButtonGroups()
     return *m_radioButtonGroups;
 }
 
+CSSStyleSheetObservableArray& TreeScope::ensureAdoptedStyleSheets()
+{
+    if (UNLIKELY(!m_adoptedStyleSheets))
+        m_adoptedStyleSheets = CSSStyleSheetObservableArray::create(m_rootNode);
+    return *m_adoptedStyleSheets;
+}
+
+std::span<const RefPtr<CSSStyleSheet>> TreeScope::adoptedStyleSheets() const
+{
+    return m_adoptedStyleSheets ? m_adoptedStyleSheets->sheets().span() : std::span<const RefPtr<CSSStyleSheet>> { };
+}
+
+JSC::JSValue TreeScope::adoptedStyleSheetWrapper(JSDOMGlobalObject& lexicalGlobalObject)
+{
+    return JSC::JSObservableArray::create(&lexicalGlobalObject, ensureAdoptedStyleSheets());
+}
+
+ExceptionOr<void> TreeScope::setAdoptedStyleSheets(Vector<RefPtr<CSSStyleSheet>>&& sheets)
+{
+    if (!m_adoptedStyleSheets && sheets.isEmpty())
+        return { };
+    return ensureAdoptedStyleSheets().setSheets(WTFMove(sheets));
+}
+
 } // namespace WebCore
