@@ -31,11 +31,13 @@
 #include "DocumentMarkerController.h"
 #include "Editing.h"
 #include "Editor.h"
+#include "EditorClient.h"
 #include "Element.h"
 #include "FloatQuad.h"
-#include "Frame.h"
-#include "FrameView.h"
+#include "LocalFrame.h"
+#include "LocalFrameView.h"
 #include "Page.h"
+#include "Range.h"
 #include "RenderedDocumentMarker.h"
 #include "SpellingCorrectionCommand.h"
 #include "TextCheckerClient.h"
@@ -90,18 +92,20 @@ AlternativeTextController::AlternativeTextController(Document& document)
 
 AlternativeTextController::~AlternativeTextController()
 {
-    dismiss(ReasonForDismissingAlternativeTextIgnored);
+    dismiss(ReasonForDismissingAlternativeText::Ignored);
 }
 
 void AlternativeTextController::startAlternativeTextUITimer(AlternativeTextType type)
 {
     const Seconds correctionPanelTimerInterval { 300_ms };
+#if ENABLE(ALTERNATIVE_TEXT_REQUIRES_AUTOMATIC_SPELLING_CORRECTION)
     if (!isAutomaticSpellingCorrectionEnabled())
         return;
+#endif
 
     // If type is PanelTypeReversion, then the new range has been set. So we shouldn't clear it.
-    if (type == AlternativeTextTypeCorrection)
-        m_rangeWithAlternative = nullptr;
+    if (type == AlternativeTextType::Correction)
+        m_rangeWithAlternative = std::nullopt;
     m_type = type;
     m_timer.startOneShot(correctionPanelTimerInterval);
 }
@@ -109,7 +113,7 @@ void AlternativeTextController::startAlternativeTextUITimer(AlternativeTextType
 void AlternativeTextController::stopAlternativeTextUITimer()
 {
     m_timer.stop();
-    m_rangeWithAlternative = nullptr;
+    m_rangeWithAlternative = std::nullopt;
 }
 
 void AlternativeTextController::stopPendingCorrection(const VisibleSelection& oldSelection)
@@ -120,7 +124,7 @@ void AlternativeTextController::stopPendingCorrection(const VisibleSelection& ol
         return;
 
     stopAlternativeTextUITimer();
-    dismiss(ReasonForDismissingAlternativeTextIgnored);
+    dismiss(ReasonForDismissingAlternativeText::Ignored);
 }
 
 void AlternativeTextController::applyPendingCorrection(const VisibleSelection& selectionAfterTyping)
@@ -137,28 +141,28 @@ void AlternativeTextController::applyPendingCorrection(const VisibleSelection& s
         }
     }
     if (doApplyCorrection)
-        handleAlternativeTextUIResult(dismissSoon(ReasonForDismissingAlternativeTextAccepted)); 
+        handleAlternativeTextUIResult(dismissSoon(ReasonForDismissingAlternativeText::Accepted));
     else
-        m_rangeWithAlternative = nullptr;
+        m_rangeWithAlternative = std::nullopt;
 }
 
 bool AlternativeTextController::hasPendingCorrection() const
 {
-    return m_rangeWithAlternative;
+    return m_rangeWithAlternative.has_value();
 }
 
-bool AlternativeTextController::isSpellingMarkerAllowed(Range& misspellingRange) const
+bool AlternativeTextController::isSpellingMarkerAllowed(const SimpleRange& misspellingRange) const
 {
     return !m_document.markers().hasMarkers(misspellingRange, DocumentMarker::SpellCheckingExemption);
 }
 
-void AlternativeTextController::show(Range& rangeToReplace, const String& replacement)
+void AlternativeTextController::show(const SimpleRange& rangeToReplace, const String& replacement)
 {
     auto boundingBox = rootViewRectForRange(rangeToReplace);
     if (boundingBox.isEmpty())
         return;
     m_originalText = plainText(rangeToReplace);
-    m_rangeWithAlternative = &rangeToReplace;
+    m_rangeWithAlternative = rangeToReplace;
     m_details = replacement;
     m_isActive = true;
     if (AlternativeTextClient* client = alternativeTextClient())
@@ -170,7 +174,7 @@ void AlternativeTextController::handleCancelOperation()
     if (!m_isActive)
         return;
     m_isActive = false;
-    dismiss(ReasonForDismissingAlternativeTextCancelled);
+    dismiss(ReasonForDismissingAlternativeText::Cancelled);
 }
 
 void AlternativeTextController::dismiss(ReasonForDismissingAlternativeText reasonForDismissing)
@@ -199,19 +203,19 @@ bool AlternativeTextController::applyAutocorrectionBeforeTypingIfAppropriate()
     if (!m_rangeWithAlternative || !m_isActive)
         return false;
 
-    if (m_type != AlternativeTextTypeCorrection)
+    if (m_type != AlternativeTextType::Correction)
         return false;
 
     Position caretPosition = m_document.selection().selection().start();
 
-    if (m_rangeWithAlternative->endPosition() == caretPosition) {
-        handleAlternativeTextUIResult(dismissSoon(ReasonForDismissingAlternativeTextAccepted));
+    if (makeDeprecatedLegacyPosition(m_rangeWithAlternative->end) == caretPosition) {
+        handleAlternativeTextUIResult(dismissSoon(ReasonForDismissingAlternativeText::Accepted));
         return true;
     } 
     
     // Pending correction should always be where caret is. But in case this is not always true, we still want to dismiss the panel without accepting the correction.
-    ASSERT(m_rangeWithAlternative->endPosition() == caretPosition);
-    dismiss(ReasonForDismissingAlternativeTextIgnored);
+    ASSERT(makeDeprecatedLegacyPosition(m_rangeWithAlternative->end) == caretPosition);
+    dismiss(ReasonForDismissingAlternativeText::Ignored);
     return false;
 }
 
@@ -220,36 +224,35 @@ void AlternativeTextController::respondToUnappliedSpellCorrection(const VisibleS
     if (auto client = alternativeTextClient())
         client->recordAutocorrectionResponse(AutocorrectionResponse::Reverted, corrected, correction);
 
-    RefPtr<Frame> protector(m_document.frame());
+    RefPtr protector(m_document.frame());
     m_document.updateLayout();
 
-    m_document.selection().setSelection(selectionOfCorrected, FrameSelection::defaultSetSelectionOptions() | FrameSelection::SpellCorrectionTriggered);
+    m_document.selection().setSelection(selectionOfCorrected, FrameSelection::defaultSetSelectionOptions() | FrameSelection::SetSelectionOption::SpellCorrectionTriggered);
     auto range = m_document.selection().selection().firstRange();
     if (!range)
         return;
-    auto& markers = m_document.markers();
-    markers.removeMarkers(*range, OptionSet<DocumentMarker::MarkerType> { DocumentMarker::Spelling, DocumentMarker::Autocorrected }, DocumentMarkerController::RemovePartiallyOverlappingMarker);
-    markers.addMarker(*range, DocumentMarker::Replacement);
-    markers.addMarker(*range, DocumentMarker::SpellCheckingExemption);
+    removeMarkers(*range, OptionSet<DocumentMarker::MarkerType> { DocumentMarker::Spelling, DocumentMarker::Autocorrected }, RemovePartiallyOverlappingMarker::Yes);
+    addMarker(*range, DocumentMarker::Replacement);
+    addMarker(*range, DocumentMarker::SpellCheckingExemption);
 }
 
 void AlternativeTextController::timerFired()
 {
     m_isDismissedByEditing = false;
     switch (m_type) {
-    case AlternativeTextTypeCorrection: {
+    case AlternativeTextType::Correction: {
         VisibleSelection selection(m_document.selection().selection());
         VisiblePosition start(selection.start(), selection.affinity());
         VisiblePosition p = startOfWord(start, LeftWordIfOnBoundary);
         VisibleSelection adjacentWords = VisibleSelection(p, start);
         auto adjacentWordRange = adjacentWords.toNormalizedRange();
-        m_document.editor().markAllMisspellingsAndBadGrammarInRanges({ TextCheckingType::Spelling, TextCheckingType::Replacement, TextCheckingType::ShowCorrectionPanel }, createLiveRange(adjacentWordRange), createLiveRange(adjacentWordRange), nullptr);
+        m_document.editor().markAllMisspellingsAndBadGrammarInRanges({ TextCheckingType::Spelling, TextCheckingType::Replacement, TextCheckingType::ShowCorrectionPanel }, adjacentWordRange, adjacentWordRange, std::nullopt);
     }
         break;
-    case AlternativeTextTypeReversion: {
+    case AlternativeTextType::Reversion: {
         if (!m_rangeWithAlternative)
             break;
-        String replacementString = WTF::get<AutocorrectionReplacement>(m_details);
+        String replacementString = std::get<AutocorrectionReplacement>(m_details);
         if (replacementString.isEmpty())
             break;
         m_isActive = true;
@@ -261,14 +264,24 @@ void AlternativeTextController::timerFired()
         }
     }
         break;
-    case AlternativeTextTypeSpellingSuggestions: {
+    case AlternativeTextType::SpellingSuggestions:
+    case AlternativeTextType::GrammarSuggestions: {
         if (!m_rangeWithAlternative || plainText(*m_rangeWithAlternative) != m_originalText)
             break;
-        String paragraphText = plainText(TextCheckingParagraph(*m_rangeWithAlternative).paragraphRange());
+
         Vector<String> suggestions;
-        textChecker()->getGuessesForWord(m_originalText, paragraphText, m_document.selection().selection(), suggestions);
+        if (m_type == AlternativeTextType::GrammarSuggestions) {
+            if (auto* editorClient = this->editorClient()) {
+                TextCheckingHelper checker(*editorClient, *m_rangeWithAlternative);
+                suggestions = checker.guessesForMisspelledWordOrUngrammaticalPhrase(true).guesses;
+            }
+        } else {
+            auto paragraphText = plainText(TextCheckingParagraph(*m_rangeWithAlternative).paragraphRange());
+            textChecker()->getGuessesForWord(m_originalText, paragraphText, m_document.selection().selection(), suggestions);
+        }
+
         if (suggestions.isEmpty()) {
-            m_rangeWithAlternative = nullptr;
+            m_rangeWithAlternative = std::nullopt;
             break;
         }
         String topSuggestion = suggestions.first();
@@ -281,12 +294,12 @@ void AlternativeTextController::timerFired()
         }
     }
         break;
-    case AlternativeTextTypeDictationAlternatives:
+    case AlternativeTextType::DictationAlternatives:
     {
 #if USE(DICTATION_ALTERNATIVES)
         if (!m_rangeWithAlternative)
             return;
-        auto dictationContext = WTF::get<DictationContext>(m_details);
+        auto dictationContext = std::get<DictationContext>(m_details);
         if (!dictationContext)
             return;
         auto boundingBox = rootViewRectForRange(*m_rangeWithAlternative);
@@ -303,11 +316,10 @@ void AlternativeTextController::timerFired()
 
 void AlternativeTextController::handleAlternativeTextUIResult(const String& result)
 {
-    Range* rangeWithAlternative = m_rangeWithAlternative.get();
-    if (!rangeWithAlternative || m_document != rangeWithAlternative->ownerDocument())
+    if (!m_rangeWithAlternative || &m_document != &m_rangeWithAlternative->start.document())
         return;
 
-    String currentWord = plainText(*rangeWithAlternative);
+    String currentWord = plainText(*m_rangeWithAlternative);
     // Check to see if the word we are about to correct has been changed between timer firing and callback being triggered.
     if (currentWord != m_originalText)
         return;
@@ -315,29 +327,47 @@ void AlternativeTextController::handleAlternativeTextUIResult(const String& resu
     m_isActive = false;
 
     switch (m_type) {
-    case AlternativeTextTypeCorrection:
+    case AlternativeTextType::Correction:
         if (result.length())
-            applyAlternativeTextToRange(*rangeWithAlternative, result, m_type, markerTypesForAutocorrection());
+            applyAlternativeTextToRange(*m_rangeWithAlternative, result, m_type, markerTypesForAutocorrection());
         else if (!m_isDismissedByEditing)
-            rangeWithAlternative->startContainer().document().markers().addMarker(*rangeWithAlternative, DocumentMarker::RejectedCorrection, m_originalText);
+            addMarker(*m_rangeWithAlternative, DocumentMarker::RejectedCorrection, m_originalText);
         break;
-    case AlternativeTextTypeReversion:
-    case AlternativeTextTypeSpellingSuggestions:
+    case AlternativeTextType::Reversion:
+    case AlternativeTextType::SpellingSuggestions:
+    case AlternativeTextType::GrammarSuggestions:
         if (result.length())
-            applyAlternativeTextToRange(*rangeWithAlternative, result, m_type, markerTypesForReplacement());
+            applyAlternativeTextToRange(*m_rangeWithAlternative, result, m_type, markerTypesForReplacement());
         break;
-    case AlternativeTextTypeDictationAlternatives:
+    case AlternativeTextType::DictationAlternatives:
         if (result.length())
-            applyAlternativeTextToRange(*rangeWithAlternative, result, m_type, markerTypesForAppliedDictationAlternative());
+            applyAlternativeTextToRange(*m_rangeWithAlternative, result, m_type, markerTypesForAppliedDictationAlternative());
         break;
     }
 
-    m_rangeWithAlternative = nullptr;
+    m_rangeWithAlternative = std::nullopt;
+}
+
+bool AlternativeTextController::canEnableAutomaticSpellingCorrection() const
+{
+#if ENABLE(AUTOCORRECT)
+    auto position = m_document.selection().selection().start();
+    if (RefPtr control = enclosingTextFormControl(position)) {
+        if (!control->shouldAutocorrect())
+            return false;
+    } else if (RefPtr editableRoot = position.rootEditableElement(); is<HTMLElement>(editableRoot) && !downcast<HTMLElement>(*editableRoot).shouldAutocorrect())
+        return false;
+#endif
+
+    return true;
 }
 
 bool AlternativeTextController::isAutomaticSpellingCorrectionEnabled()
 {
-    return editorClient() && editorClient()->isAutomaticSpellingCorrectionEnabled();
+    if (!editorClient() || !editorClient()->isAutomaticSpellingCorrectionEnabled())
+        return false;
+
+    return canEnableAutomaticSpellingCorrection();
 }
 
 FloatRect AlternativeTextController::rootViewRectForRange(const SimpleRange& range) const
@@ -382,15 +412,24 @@ void AlternativeTextController::respondToChangedSelection(const VisibleSelection
     }
 }
 
+#if USE(APPLE_INTERNAL_SDK)
+#include <WebKitAdditions/AlternativeTextControllerAdditions.cpp>
+#else
+static inline void removeCorrectionIndicatorMarkers(Document& document)
+{
+    document.markers().removeMarkers(DocumentMarker::CorrectionIndicator);
+}
+#endif
+
 void AlternativeTextController::respondToAppliedEditing(CompositeEditCommand* command)
 {
     if (command->isTopLevelCommand() && !command->shouldRetainAutocorrectionIndicator())
-        m_document.markers().removeMarkers(DocumentMarker::CorrectionIndicator);
+        removeCorrectionIndicatorMarkers(m_document);
 
     markPrecedingWhitespaceForDeletedAutocorrectionAfterCommand(command);
     m_originalStringForLastDeletedAutocorrection = String();
 
-    dismiss(ReasonForDismissingAlternativeTextIgnored);
+    dismiss(ReasonForDismissingAlternativeText::Ignored);
 }
 
 void AlternativeTextController::respondToUnappliedEditing(EditCommandComposition* command)
@@ -400,9 +439,8 @@ void AlternativeTextController::respondToUnappliedEditing(EditCommandComposition
     auto range = command->startingSelection().firstRange();
     if (!range)
         return;
-    auto& markers = m_document.markers();
-    markers.addMarker(*range, DocumentMarker::Replacement);
-    markers.addMarker(*range, DocumentMarker::SpellCheckingExemption);
+    addMarker(*range, DocumentMarker::Replacement);
+    addMarker(*range, DocumentMarker::SpellCheckingExemption);
 }
 
 EditorClient* AlternativeTextController::editorClient()
@@ -423,24 +461,23 @@ void AlternativeTextController::recordAutocorrectionResponse(AutocorrectionRespo
         client->recordAutocorrectionResponse(response, replacedString, plainText(replacementRange));
 }
 
-void AlternativeTextController::markReversed(Range& changedRange)
+void AlternativeTextController::markReversed(const SimpleRange& changedRange)
 {
-    changedRange.startContainer().document().markers().removeMarkers(changedRange, DocumentMarker::Autocorrected, DocumentMarkerController::RemovePartiallyOverlappingMarker);
-    changedRange.startContainer().document().markers().addMarker(changedRange, DocumentMarker::SpellCheckingExemption);
+    removeMarkers(changedRange, DocumentMarker::Autocorrected, RemovePartiallyOverlappingMarker::Yes);
+    addMarker(changedRange, DocumentMarker::SpellCheckingExemption);
 }
 
-void AlternativeTextController::markCorrection(Range& replacedRange, const String& replacedString)
+void AlternativeTextController::markCorrection(const SimpleRange& replacedRange, const String& replacedString)
 {
-    DocumentMarkerController& markers = replacedRange.startContainer().document().markers();
     for (auto markerType : markerTypesForAutocorrection()) {
         if (markerType == DocumentMarker::Replacement || markerType == DocumentMarker::Autocorrected)
-            markers.addMarker(replacedRange, markerType, replacedString);
+            addMarker(replacedRange, markerType, replacedString);
         else
-            markers.addMarker(replacedRange, markerType);
+            addMarker(replacedRange, markerType);
     }
 }
 
-void AlternativeTextController::recordSpellcheckerResponseForModifiedCorrection(Range& rangeOfCorrection, const String& corrected, const String& correction)
+void AlternativeTextController::recordSpellcheckerResponseForModifiedCorrection(const SimpleRange& rangeOfCorrection, const String& corrected, const String& correction)
 {
     DocumentMarkerController& markers = rangeOfCorrection.startContainer().document().markers();
     Vector<RenderedDocumentMarker*> correctedOnceMarkers = markers.markersInRange(rangeOfCorrection, DocumentMarker::Autocorrected);
@@ -456,7 +493,7 @@ void AlternativeTextController::recordSpellcheckerResponseForModifiedCorrection(
             client->recordAutocorrectionResponse(AutocorrectionResponse::Edited, corrected, correction);
     }
 
-    markers.removeMarkers(rangeOfCorrection, DocumentMarker::Autocorrected, DocumentMarkerController::RemovePartiallyOverlappingMarker);
+    removeMarkers(rangeOfCorrection, DocumentMarker::Autocorrected, RemovePartiallyOverlappingMarker::Yes);
 }
 
 void AlternativeTextController::deletedAutocorrectionAtPosition(const Position& position, const String& originalString)
@@ -475,18 +512,20 @@ void AlternativeTextController::markPrecedingWhitespaceForDeletedAutocorrectionA
     if (endOfSelection == precedingCharacterPosition)
         return;
 
-    auto precedingCharacterRange = SimpleRange { *makeBoundaryPoint(precedingCharacterPosition), *makeBoundaryPoint(endOfSelection) };
-    String string = plainText(precedingCharacterRange);
+    auto precedingCharacterRange = makeSimpleRange(precedingCharacterPosition, endOfSelection);
+    if (!precedingCharacterRange)
+        return;
+    String string = plainText(*precedingCharacterRange);
     if (string.isEmpty() || !deprecatedIsEditingWhitespace(string[string.length() - 1]))
         return;
 
     // Mark this whitespace to indicate we have deleted an autocorrection following this
     // whitespace. So if the user types the same original word again at this position, we
     // won't autocorrect it again.
-    m_document.markers().addMarker(precedingCharacterRange, DocumentMarker::DeletedAutocorrection, m_originalStringForLastDeletedAutocorrection);
+    addMarker(*precedingCharacterRange, DocumentMarker::DeletedAutocorrection, m_originalStringForLastDeletedAutocorrection);
 }
 
-bool AlternativeTextController::processMarkersOnTextToBeReplacedByResult(const TextCheckingResult& result, Range& rangeWithAlternative, const String& stringToBeReplaced)
+bool AlternativeTextController::processMarkersOnTextToBeReplacedByResult(const TextCheckingResult& result, const SimpleRange& rangeWithAlternative, const String& stringToBeReplaced)
 {
     auto& markers = m_document.markers();
     if (markers.hasMarkers(rangeWithAlternative, DocumentMarker::Replacement)) {
@@ -501,12 +540,11 @@ bool AlternativeTextController::processMarkersOnTextToBeReplacedByResult(const T
     if (markers.hasMarkers(rangeWithAlternative, DocumentMarker::AcceptedCandidate))
         return false;
 
-    Position beginningOfRange = rangeWithAlternative.startPosition();
-    Position precedingCharacterPosition = beginningOfRange.previous();
+    auto precedingCharacterRange = makeSimpleRange(makeDeprecatedLegacyPosition(rangeWithAlternative.start).previous(), rangeWithAlternative.start);
+    if (!precedingCharacterRange)
+        return false;
 
-    auto precedingCharacterRange = SimpleRange { *makeBoundaryPoint(precedingCharacterPosition), *makeBoundaryPoint(beginningOfRange) };
-
-    for (auto& marker : markers.markersInRange(precedingCharacterRange, DocumentMarker::DeletedAutocorrection)) {
+    for (auto& marker : markers.markersInRange(*precedingCharacterRange, DocumentMarker::DeletedAutocorrection)) {
         if (marker->description() == stringToBeReplaced)
             return false;
     }
@@ -516,7 +554,19 @@ bool AlternativeTextController::processMarkersOnTextToBeReplacedByResult(const T
 
 bool AlternativeTextController::shouldStartTimerFor(const WebCore::DocumentMarker &marker, int endOffset) const
 {
-    return (((marker.type() == DocumentMarker::Replacement && !marker.description().isNull()) || marker.type() == DocumentMarker::Spelling || marker.type() == DocumentMarker::DictationAlternatives) && static_cast<int>(marker.endOffset()) == endOffset);
+    if (static_cast<int>(marker.endOffset()) != endOffset)
+        return false;
+
+    switch (marker.type()) {
+    case DocumentMarker::Spelling:
+    case DocumentMarker::Grammar:
+    case DocumentMarker::DictationAlternatives:
+        return true;
+    case DocumentMarker::Replacement:
+        return !marker.description().isNull();
+    default:
+        return false;
+    }
 }
 
 bool AlternativeTextController::respondToMarkerAtEndOfWord(const DocumentMarker& marker, const Position& endOfWordPosition)
@@ -524,29 +574,30 @@ bool AlternativeTextController::respondToMarkerAtEndOfWord(const DocumentMarker&
     if (!shouldStartTimerFor(marker, endOfWordPosition.offsetInContainerNode()))
         return false;
     Node* node = endOfWordPosition.containerNode();
-    auto wordRange = Range::create(m_document, node, marker.startOffset(), node, marker.endOffset());
+    auto wordRange = makeSimpleRange(*node, marker);
     String currentWord = plainText(wordRange);
     if (!currentWord.length())
         return false;
     m_originalText = currentWord;
     switch (marker.type()) {
     case DocumentMarker::Spelling:
+    case DocumentMarker::Grammar:
         m_rangeWithAlternative = WTFMove(wordRange);
         m_details = emptyString();
-        startAlternativeTextUITimer(AlternativeTextTypeSpellingSuggestions);
+        startAlternativeTextUITimer((marker.type() == DocumentMarker::Spelling) ? AlternativeTextType::SpellingSuggestions : AlternativeTextType::GrammarSuggestions);
         break;
     case DocumentMarker::Replacement:
         m_rangeWithAlternative = WTFMove(wordRange);
         m_details = marker.description();
-        startAlternativeTextUITimer(AlternativeTextTypeReversion);
+        startAlternativeTextUITimer(AlternativeTextType::Reversion);
         break;
     case DocumentMarker::DictationAlternatives: {
-        auto& markerData = WTF::get<DocumentMarker::DictationData>(marker.data());
+        auto& markerData = std::get<DocumentMarker::DictationData>(marker.data());
         if (currentWord != markerData.originalText)
             return false;
         m_rangeWithAlternative = WTFMove(wordRange);
         m_details = markerData.context;
-        startAlternativeTextUITimer(AlternativeTextTypeDictationAlternatives);
+        startAlternativeTextUITimer(AlternativeTextType::DictationAlternatives);
     }
         break;
     default:
@@ -568,7 +619,7 @@ AlternativeTextClient* AlternativeTextController::alternativeTextClient()
 String AlternativeTextController::markerDescriptionForAppliedAlternativeText(AlternativeTextType alternativeTextType, DocumentMarker::MarkerType markerType)
 {
 #if USE(AUTOCORRECTION_PANEL)
-    if (alternativeTextType != AlternativeTextTypeReversion && alternativeTextType != AlternativeTextTypeDictationAlternatives && (markerType == DocumentMarker::Replacement || markerType == DocumentMarker::Autocorrected))
+    if (alternativeTextType != AlternativeTextType::Reversion && alternativeTextType != AlternativeTextType::DictationAlternatives && (markerType == DocumentMarker::Replacement || markerType == DocumentMarker::Autocorrected))
         return m_originalText;
 #else
     UNUSED_PARAM(alternativeTextType);
@@ -577,25 +628,22 @@ String AlternativeTextController::markerDescriptionForAppliedAlternativeText(Alt
     return emptyString();
 }
 
-void AlternativeTextController::applyAlternativeTextToRange(const Range& range, const String& alternative, AlternativeTextType alternativeType, OptionSet<DocumentMarker::MarkerType> markerTypesToAdd)
+void AlternativeTextController::applyAlternativeTextToRange(const SimpleRange& range, const String& alternative, AlternativeTextType alternativeType, OptionSet<DocumentMarker::MarkerType> markerTypesToAdd)
 {
     // After we replace the word at range rangeWithAlternative, we need to add markers to that range.
     // So before we carry out the replacement,store the start position relative to the start position
     // of the containing paragraph.
 
     // Take note of the location of autocorrection so that we can add marker after the replacement took place.
-    auto correctionStartPosition = range.startPosition();
-    auto correctionStart { makeBoundaryPoint(correctionStartPosition) };
-    auto paragraphStart { makeBoundaryPoint(startOfParagraph(correctionStartPosition)) };
-    if (!correctionStart || !paragraphStart)
+    auto paragraphStart = makeBoundaryPoint(startOfParagraph(makeDeprecatedLegacyPosition(range.start)));
+    if (!paragraphStart)
         return;
-    auto treeScopeRoot = makeRef(correctionStart->container->treeScope().rootNode());
+    Ref treeScopeRoot = range.start.container->treeScope().rootNode();
     auto treeScopeStart = BoundaryPoint { treeScopeRoot.get(), 0 };
-    auto correctionOffsetInParagraph = characterCount({ *paragraphStart, *correctionStart });
+    auto correctionOffsetInParagraph = characterCount({ *paragraphStart, range.start });
     auto paragraphOffsetInTreeScope = characterCount({ treeScopeStart, *paragraphStart });
 
-    // Clone the range, since the caller of may keep a refernece to the original range and modify it.
-    SpellingCorrectionCommand::create(range.cloneRange(), alternative)->apply();
+    SpellingCorrectionCommand::create(range, alternative)->apply();
 
     // Recalculate pragraphRangeContainingCorrection, since SpellingCorrectionCommand modified the DOM, such that the original paragraphRangeContainingCorrection is no longer valid. Radar: 10305315 Bugzilla: 89526
     auto updatedParagraphStartContainingCorrection = resolveCharacterLocation(makeRangeSelectingNodeContents(treeScopeRoot), paragraphOffsetInTreeScope);
@@ -608,9 +656,8 @@ void AlternativeTextController::applyAlternativeTextToRange(const Range& range,
     if (plainText(replacementRange) != alternative)
         return;
 
-    auto& markers = replacementRange.start.document().markers();
     for (auto markerType : markerTypesToAdd)
-        markers.addMarker(replacementRange, markerType, markerDescriptionForAppliedAlternativeText(alternativeType, markerType));
+        addMarker(replacementRange, markerType, markerDescriptionForAppliedAlternativeText(alternativeType, markerType));
 }
 
 #endif
@@ -636,7 +683,7 @@ void AlternativeTextController::removeDictationAlternativesForMarker(const Docum
 {
 #if USE(DICTATION_ALTERNATIVES)
     if (auto* client = alternativeTextClient())
-        client->removeDictationAlternatives(WTF::get<DocumentMarker::DictationData>(marker.data()).context);
+        client->removeDictationAlternatives(std::get<DocumentMarker::DictationData>(marker.data()).context);
 #else
     UNUSED_PARAM(marker);
 #endif
@@ -646,7 +693,7 @@ Vector<String> AlternativeTextController::dictationAlternativesForMarker(const D
 {
 #if USE(DICTATION_ALTERNATIVES)
     if (auto* client = alternativeTextClient())
-        return client->dictationAlternatives(WTF::get<DocumentMarker::DictationData>(marker.data()).context);
+        return client->dictationAlternatives(std::get<DocumentMarker::DictationData>(marker.data()).context);
     return Vector<String>();
 #else
     UNUSED_PARAM(marker);
@@ -659,11 +706,11 @@ void AlternativeTextController::applyDictationAlternative(const String& alternat
 #if USE(DICTATION_ALTERNATIVES)
     auto& editor = m_document.editor();
     auto selection = editor.selectedRange();
-    if (!selection || !editor.shouldInsertText(alternativeString, selection.get(), EditorInsertAction::Pasted))
+    if (!selection || !editor.shouldInsertText(alternativeString, *selection, EditorInsertAction::Pasted))
         return;
     for (auto* marker : selection->startContainer().document().markers().markersInRange(*selection, DocumentMarker::DictationAlternatives))
         removeDictationAlternativesForMarker(*marker);
-    applyAlternativeTextToRange(*selection, alternativeString, AlternativeTextTypeDictationAlternatives, markerTypesForAppliedDictationAlternative());
+    applyAlternativeTextToRange(*selection, alternativeString, AlternativeTextType::DictationAlternatives, markerTypesForAppliedDictationAlternative());
 #else
     UNUSED_PARAM(alternativeString);
 #endif
