@@ -26,18 +26,20 @@
 #include "config.h"
 #include "CanvasBase.h"
 
+#include "ByteArrayPixelBuffer.h"
 #include "CanvasRenderingContext.h"
 #include "Chrome.h"
 #include "Document.h"
 #include "Element.h"
-#include "FloatRect.h"
 #include "GraphicsClient.h"
 #include "GraphicsContext.h"
 #include "HTMLCanvasElement.h"
 #include "HostWindow.h"
 #include "ImageBuffer.h"
 #include "InspectorInstrumentation.h"
+#include "IntRect.h"
 #include "StyleCanvasImage.h"
+#include "RenderElement.h"
 #include "WebCoreOpaqueRoot.h"
 #include "WorkerClient.h"
 #include "WorkerGlobalScope.h"
@@ -105,8 +107,11 @@ AffineTransform CanvasBase::baseTransform() const
 
 void CanvasBase::makeRenderingResultsAvailable()
 {
-    if (auto* context = renderingContext())
+    if (auto* context = renderingContext()) {
         context->paintRenderingResultsToCanvas();
+        if (auto noiseInjectionHashSalt = shouldInjectNoiseBeforeReadback() ? scriptExecutionContext()->noiseInjectionHashSalt() : std::nullopt)
+            m_canvasNoiseInjection.postProcessDirtyCanvasBuffer(buffer(), *noiseInjectionHashSalt);
+    }
 }
 
 size_t CanvasBase::memoryCost() const
@@ -190,12 +195,28 @@ void CanvasBase::removeObserver(CanvasObserver& observer)
         InspectorInstrumentation::didChangeCSSCanvasClientNodes(*this);
 }
 
+bool CanvasBase::hasObserver(CanvasObserver& observer) const
+{
+    return m_observers.contains(observer);
+}
+
 void CanvasBase::notifyObserversCanvasChanged(const std::optional<FloatRect>& rect)
 {
     for (auto& observer : m_observers)
         observer.canvasChanged(*this, rect);
 }
 
+void CanvasBase::didDraw(const std::optional<FloatRect>& rect, ShouldApplyPostProcessingToDirtyRect shouldApplyPostProcessingToDirtyRect)
+{
+    // FIXME: We should exclude rects with ShouldApplyPostProcessingToDirtyRect::No
+    if (shouldInjectNoiseBeforeReadback() && shouldApplyPostProcessingToDirtyRect == ShouldApplyPostProcessingToDirtyRect::Yes) {
+        if (rect)
+            m_canvasNoiseInjection.updateDirtyRect(intersection(enclosingIntRect(*rect), { { }, size() }));
+        else
+            m_canvasNoiseInjection.updateDirtyRect({ { }, size() });
+    }
+}
+
 void CanvasBase::notifyObserversCanvasResized()
 {
     for (auto& observer : m_observers)
@@ -237,8 +258,9 @@ HashSet<Element*> CanvasBase::cssCanvasClients() const
         if (!is<StyleCanvasImage>(observer))
             continue;
 
-        for (auto& client : downcast<StyleCanvasImage>(observer).clients().values()) {
-            if (auto element = client->element())
+        for (auto entry : downcast<StyleCanvasImage>(observer).clients()) {
+            auto& client = entry.key;
+            if (auto element = client.element())
                 cssCanvasClients.add(element);
         }
     }
@@ -315,14 +337,14 @@ bool CanvasBase::shouldAccelerate(unsigned area) const
 #endif
 }
 
-void CanvasBase::createImageBuffer(bool usesDisplayListDrawing, bool avoidBackendSizeCheckForTesting) const
+RefPtr<ImageBuffer> CanvasBase::allocateImageBuffer(bool usesDisplayListDrawing, bool avoidBackendSizeCheckForTesting) const
 {
     auto checkedArea = size().area<RecordOverflow>();
 
     if (checkedArea.hasOverflowed() || checkedArea > maxCanvasArea()) {
         auto message = makeString("Canvas area exceeds the maximum limit (width * height > ", maxCanvasArea(), ").");
         scriptExecutionContext()->addConsoleMessage(MessageSource::JS, MessageLevel::Warning, message);
-        return;
+        return nullptr;
     }
 
     // Make sure we don't use more pixel memory than the system can support.
@@ -330,12 +352,12 @@ void CanvasBase::createImageBuffer(bool usesDisplayListDrawing, bool avoidBacken
     if (checkedRequestedPixelMemory.hasOverflowed() || checkedRequestedPixelMemory > maxActivePixelMemory()) {
         auto message = makeString("Total canvas memory use exceeds the maximum limit (", maxActivePixelMemory() / 1024 / 1024, " MB).");
         scriptExecutionContext()->addConsoleMessage(MessageSource::JS, MessageLevel::Warning, message);
-        return;
+        return nullptr;
     }
 
     unsigned area = checkedArea.value();
     if (!area)
-        return;
+        return nullptr;
 
     OptionSet<ImageBufferOptions> bufferOptions;
     if (shouldAccelerate(area))
@@ -352,7 +374,20 @@ void CanvasBase::createImageBuffer(bool usesDisplayListDrawing, bool avoidBacken
     ImageBufferCreationContext context = { };
     context.graphicsClient = graphicsClient();
     context.avoidIOSurfaceSizeCheckInWebProcessForTesting = avoidBackendSizeCheckForTesting;
-    setImageBuffer(ImageBuffer::create(size(), RenderingPurpose::Canvas, 1, colorSpace, pixelFormat, bufferOptions, context));
+    return ImageBuffer::create(size(), RenderingPurpose::Canvas, 1, colorSpace, pixelFormat, bufferOptions, context);
+}
+
+bool CanvasBase::shouldInjectNoiseBeforeReadback() const
+{
+    // Note, every early-return resulting from this check potentially leaks this state. This is a risk that we're accepting right now.
+    return m_imageBuffer && scriptExecutionContext() && scriptExecutionContext()->noiseInjectionHashSalt();
+}
+
+bool CanvasBase::postProcessPixelBufferResults(Ref<PixelBuffer>&& pixelBuffer) const
+{
+    if (auto noiseInjectionHashSalt = shouldInjectNoiseBeforeReadback() ? scriptExecutionContext()->noiseInjectionHashSalt() : std::nullopt)
+        return m_canvasNoiseInjection.postProcessPixelBufferResults(std::forward<Ref<PixelBuffer>>(pixelBuffer), *noiseInjectionHashSalt);
+    return false;
 }
 
 size_t CanvasBase::activePixelMemory()
