@@ -23,11 +23,13 @@
 #include "HTMLAreaElement.h"
 
 #include "AffineTransform.h"
-#include "Frame.h"
+#include "ElementInlines.h"
 #include "HTMLImageElement.h"
 #include "HTMLMapElement.h"
 #include "HTMLParserIdioms.h"
 #include "HitTestResult.h"
+#include "LocalFrame.h"
+#include "NodeName.h"
 #include "Path.h"
 #include "RenderImage.h"
 #include "RenderView.h"
@@ -52,27 +54,33 @@ Ref<HTMLAreaElement> HTMLAreaElement::create(const QualifiedName& tagName, Docum
     return adoptRef(*new HTMLAreaElement(tagName, document));
 }
 
-void HTMLAreaElement::parseAttribute(const QualifiedName& name, const AtomicString& value)
+void HTMLAreaElement::attributeChanged(const QualifiedName& name, const AtomString& oldValue, const AtomString& newValue, AttributeModificationReason attributeModificationReason)
 {
-    if (name == shapeAttr) {
-        if (equalLettersIgnoringASCIICase(value, "default"))
+    switch (name.nodeName()) {
+    case AttributeNames::shapeAttr:
+        if (equalLettersIgnoringASCIICase(newValue, "default"_s))
             m_shape = Default;
-        else if (equalLettersIgnoringASCIICase(value, "circle") || equalLettersIgnoringASCIICase(value, "circ"))
+        else if (equalLettersIgnoringASCIICase(newValue, "circle"_s) || equalLettersIgnoringASCIICase(newValue, "circ"_s))
             m_shape = Circle;
-        else if (equalLettersIgnoringASCIICase(value, "poly") || equalLettersIgnoringASCIICase(value, "polygon"))
+        else if (equalLettersIgnoringASCIICase(newValue, "poly"_s) || equalLettersIgnoringASCIICase(newValue, "polygon"_s))
             m_shape = Poly;
         else {
             // The missing value default is the rectangle state.
             m_shape = Rect;
         }
         invalidateCachedRegion();
-    } else if (name == coordsAttr) {
-        m_coords = parseHTMLListOfOfFloatingPointNumberValues(value.string());
+        break;
+    case AttributeNames::coordsAttr:
+        m_coords = parseHTMLListOfOfFloatingPointNumberValues(newValue.string());
         invalidateCachedRegion();
-    } else if (name == altAttr || name == accesskeyAttr) {
+        break;
+    case AttributeNames::altAttr:
         // Do nothing.
-    } else
-        HTMLAnchorElement::parseAttribute(name, value);
+        break;
+    default:
+        HTMLAnchorElement::attributeChanged(name, oldValue, newValue, attributeModificationReason);
+        break;
+    }
 }
 
 void HTMLAreaElement::invalidateCachedRegion()
@@ -83,7 +91,7 @@ void HTMLAreaElement::invalidateCachedRegion()
 bool HTMLAreaElement::mapMouseEvent(LayoutPoint location, const LayoutSize& size, HitTestResult& result)
 {
     if (m_lastSize != size) {
-        m_region = std::make_unique<Path>(getRegion(size));
+        m_region = makeUnique<Path>(getRegion(size));
         m_lastSize = size;
     }
 
@@ -210,18 +218,18 @@ bool HTMLAreaElement::isMouseFocusable() const
 bool HTMLAreaElement::isFocusable() const
 {
     RefPtr<HTMLImageElement> image = imageElement();
-    if (!image || !image->renderer() || image->renderer()->style().visibility() != Visibility::Visible)
+    if (!image || !image->hasFocusableStyle())
         return false;
 
-    return supportsFocus() && Element::tabIndex() >= 0;
+    return supportsFocus() && tabIndexSetExplicitly().value_or(0) >= 0;
 }
     
-void HTMLAreaElement::setFocus(bool shouldBeFocused)
+void HTMLAreaElement::setFocus(bool shouldBeFocused, FocusVisibility visibility)
 {
     if (focused() == shouldBeFocused)
         return;
 
-    HTMLAnchorElement::setFocus(shouldBeFocused);
+    HTMLAnchorElement::setFocus(shouldBeFocused, visibility);
 
     RefPtr<HTMLImageElement> imageElement = this->imageElement();
     if (!imageElement)
@@ -249,7 +257,7 @@ bool HTMLAreaElement::supportsFocus() const
     return isLink();
 }
 
-String HTMLAreaElement::target() const
+AtomString HTMLAreaElement::target() const
 {
     return attributeWithoutSynchronization(targetAttr);
 }
