@@ -25,7 +25,7 @@
 #include "config.h"
 #include "HTMLFieldSetElement.h"
 
-#include "ElementIterator.h"
+#include "ElementChildIteratorInlines.h"
 #include "GenericCachedHTMLCollection.h"
 #include "HTMLFormControlsCollection.h"
 #include "HTMLLegendElement.h"
@@ -35,6 +35,7 @@
 #include "PseudoClassChangeInvalidation.h"
 #include "RenderElement.h"
 #include "ScriptDisallowedScope.h"
+#include "TypedElementDescendantIteratorInlines.h"
 #include <wtf/IsoMallocInlines.h>
 #include <wtf/StdLibExtras.h>
 
@@ -61,6 +62,17 @@ Ref<HTMLFieldSetElement> HTMLFieldSetElement::create(const QualifiedName& tagNam
     return adoptRef(*new HTMLFieldSetElement(tagName, document, form));
 }
 
+bool HTMLFieldSetElement::isDisabledFormControl() const
+{
+    if (document().settings().sendMouseEventsToDisabledFormControlsEnabled()) {
+        // The fieldset element itself should never be considered disabled, it is
+        // only supposed to affect its descendants:
+        // https://html.spec.whatwg.org/multipage/form-control-infrastructure.html#concept-fe-disabled
+        return false;
+    }
+    return HTMLFormControlElement::isDisabledFormControl();
+}
+
 static void updateFromControlElementsAncestorDisabledStateUnder(HTMLElement& startNode, bool isDisabled)
 {
     auto range = inclusiveDescendantsOfType<Element>(startNode);
@@ -82,11 +94,11 @@ static void updateFromControlElementsAncestorDisabledStateUnder(HTMLElement& sta
     }
 }
 
-void HTMLFieldSetElement::parseAttribute(const QualifiedName& name, const AtomString& value)
+void HTMLFieldSetElement::attributeChanged(const QualifiedName& name, const AtomString& oldValue, const AtomString& newValue, AttributeModificationReason attributeModificationReason)
 {
     bool oldHasDisabledAttribute = hasDisabledAttribute();
 
-    HTMLFormControlElement::parseAttribute(name, value);
+    HTMLFormControlElement::attributeChanged(name, oldValue, newValue, attributeModificationReason);
 
     if (hasDisabledAttribute() != oldHasDisabledAttribute) {
         if (hasDisabledAttribute())
@@ -176,7 +188,7 @@ HTMLLegendElement* HTMLFieldSetElement::legend() const
 
 Ref<HTMLCollection> HTMLFieldSetElement::elements()
 {
-    return ensureRareData().ensureNodeLists().addCachedCollection<GenericCachedHTMLCollection<CollectionTypeTraits<FieldSetElements>::traversalType>>(*this, FieldSetElements);
+    return ensureRareData().ensureNodeLists().addCachedCollection<GenericCachedHTMLCollection<CollectionTypeTraits<CollectionType::FieldSetElements>::traversalType>>(*this, CollectionType::FieldSetElements);
 }
 
 void HTMLFieldSetElement::addInvalidDescendant(const HTMLElement& invalidFormControlElement)
