@@ -28,18 +28,15 @@
 #include "CommonAtomStrings.h"
 #include "DOMFormData.h"
 #include "DOMTokenList.h"
-#include "DOMWindow.h"
 #include "DiagnosticLoggingClient.h"
 #include "Document.h"
-#include "ElementIterator.h"
+#include "ElementAncestorIteratorInlines.h"
 #include "Event.h"
 #include "EventNames.h"
 #include "FormController.h"
 #include "FormData.h"
 #include "FormDataEvent.h"
-#include "Frame.h"
 #include "FrameLoader.h"
-#include "FrameLoaderClient.h"
 #include "HTMLDialogElement.h"
 #include "HTMLFieldSetElement.h"
 #include "HTMLFormControlsCollection.h"
@@ -47,10 +44,13 @@
 #include "HTMLInputElement.h"
 #include "HTMLNames.h"
 #include "HTMLObjectElement.h"
-#include "HTMLParserIdioms.h"
 #include "HTMLTableElement.h"
 #include "InputTypeNames.h"
+#include "LocalDOMWindow.h"
+#include "LocalFrame.h"
+#include "LocalFrameLoaderClient.h"
 #include "MixedContentChecker.h"
+#include "NodeName.h"
 #include "NodeRareData.h"
 #include "Page.h"
 #include "PseudoClassChangeInvalidation.h"
@@ -59,6 +59,7 @@
 #include "ScriptDisallowedScope.h"
 #include "Settings.h"
 #include "SubmitEvent.h"
+#include "TypedElementDescendantIteratorInlines.h"
 #include "UserGestureIndicator.h"
 #include <limits>
 #include <wtf/IsoMallocInlines.h>
@@ -282,7 +283,7 @@ void HTMLFormElement::submitIfPossible(Event* event, HTMLFormControlElement* sub
     if (!isConnected())
         return;
 
-    RefPtr<Frame> frame = document().frame();
+    RefPtr frame { document().frame() };
     if (m_isSubmittingOrPreparingForSubmission || !frame)
         return;
 
@@ -354,13 +355,11 @@ ExceptionOr<void> HTMLFormElement::requestSubmit(HTMLElement* submitter)
     RefPtr<HTMLFormControlElement> control;
     if (submitter) {
         // https://html.spec.whatwg.org/multipage/forms.html#dom-form-requestsubmit
-        if (!is<HTMLFormControlElement>(submitter))
-            return Exception { TypeError };
-        control = downcast<HTMLFormControlElement>(submitter);
-        if (!control->isSubmitButton())
-            return Exception { TypeError };
+        control = dynamicDowncast<HTMLFormControlElement>(*submitter);
+        if (!control || !control->isSubmitButton())
+            return Exception { TypeError, "The specified element is not a submit button."_s };
         if (control->form() != this)
-            return Exception { NotFoundError };
+            return Exception { NotFoundError, "The specified element is not owned by this form element."_s };
     }
 
     submitIfPossible(nullptr, control.get(), SubmittedByJavaScript);
@@ -412,8 +411,8 @@ void HTMLFormElement::submit(Event* event, bool processingUserGesture, FormSubmi
     if (m_isConstructingEntryList)
         return;
 
-    RefPtr<FrameView> view = document().view();
-    RefPtr<Frame> frame = document().frame();
+    RefPtr view { document().view() };
+    RefPtr frame { document().frame() };
     if (!view || !frame)
         return;
 
@@ -480,7 +479,7 @@ void HTMLFormElement::reset()
     if (m_isInResetFunction)
         return;
 
-    RefPtr<Frame> protectedFrame = document().frame();
+    RefPtr protectedFrame { document().frame() };
     if (!protectedFrame)
         return;
 
@@ -503,35 +502,44 @@ void HTMLFormElement::resetListedFormControlElements()
         control->reset();
 }
 
-void HTMLFormElement::parseAttribute(const QualifiedName& name, const AtomString& value)
+void HTMLFormElement::attributeChanged(const QualifiedName& name, const AtomString& oldValue, const AtomString& newValue, AttributeModificationReason attributeModificationReason)
 {
-    if (name == actionAttr) {
-        m_attributes.parseAction(value);
-        
+    switch (name.nodeName()) {
+    case AttributeNames::actionAttr:
+        m_attributes.parseAction(newValue);
         if (!m_attributes.action().isEmpty()) {
-            if (RefPtr<Frame> f = document().frame()) {
+            if (RefPtr f = document().frame()) {
                 if (auto* topFrame = dynamicDowncast<LocalFrame>(f->tree().top()))
                     MixedContentChecker::checkFormForMixedContent(*topFrame, topFrame->document()->securityOrigin(), document().completeURL(m_attributes.action()));
             }
         }
-    } else if (name == targetAttr)
-        m_attributes.setTarget(value);
-    else if (name == methodAttr)
-        m_attributes.updateMethodType(value, document().settings().dialogElementEnabled());
-    else if (name == enctypeAttr)
-        m_attributes.updateEncodingType(value);
-    else if (name == accept_charsetAttr)
-        m_attributes.setAcceptCharset(value);
-    else if (name == autocompleteAttr) {
+        break;
+    case AttributeNames::targetAttr:
+        m_attributes.setTarget(newValue);
+        break;
+    case AttributeNames::methodAttr:
+        m_attributes.updateMethodType(newValue, document().settings().dialogElementEnabled());
+        break;
+    case AttributeNames::enctypeAttr:
+        m_attributes.updateEncodingType(newValue);
+        break;
+    case AttributeNames::accept_charsetAttr:
+        m_attributes.setAcceptCharset(newValue);
+        break;
+    case AttributeNames::autocompleteAttr:
         if (!shouldAutocomplete())
             document().registerForDocumentSuspensionCallbacks(*this);
         else
             document().unregisterForDocumentSuspensionCallbacks(*this);
-    } else if (name == relAttr) {
+        break;
+    case AttributeNames::relAttr:
         if (m_relList)
-            m_relList->associatedAttributeValueChanged(value);
-    } else
-        HTMLElement::parseAttribute(name, value);
+            m_relList->associatedAttributeValueChanged(newValue);
+        break;
+    default:
+        HTMLElement::attributeChanged(name, oldValue, newValue, attributeModificationReason);
+        break;
+    }
 }
 
 unsigned HTMLFormElement::formElementIndexWithFormAttribute(Element* element, unsigned rangeStart, unsigned rangeEnd)
@@ -653,7 +661,7 @@ void HTMLFormElement::addInvalidFormControl(const HTMLElement& formControlElemen
     ASSERT(static_cast<const Element&>(formControlElement).matchesInvalidPseudoClass());
 
     std::optional<Style::PseudoClassChangeInvalidation> styleInvalidation;
-    if (m_invalidFormControls.computesEmpty())
+    if (m_invalidFormControls.isEmptyIgnoringNullReferences())
         emplace(styleInvalidation, *this, { { CSSSelector::PseudoClassValid, false }, { CSSSelector::PseudoClassInvalid, true } });
 
     m_invalidFormControls.add(formControlElement);
@@ -691,7 +699,7 @@ void HTMLFormElement::unregisterImgElement(HTMLImageElement& element)
 
 Ref<HTMLFormControlsCollection> HTMLFormElement::elements()
 {
-    return ensureRareData().ensureNodeLists().addCachedCollection<HTMLFormControlsCollection>(*this, FormControls);
+    return ensureRareData().ensureNodeLists().addCachedCollection<HTMLFormControlsCollection>(*this, CollectionType::FormControls);
 }
 
 Ref<HTMLCollection> HTMLFormElement::elementsForNativeBindings()
@@ -714,7 +722,7 @@ String HTMLFormElement::action() const
     auto& value = attributeWithoutSynchronization(actionAttr);
     if (value.isEmpty())
         return document().url().string();
-    return document().completeURL(stripLeadingAndTrailingHTMLSpaces(value)).string();
+    return document().completeURL(value).string();
 }
 
 void HTMLFormElement::setAction(const AtomString& value)
@@ -899,7 +907,7 @@ void HTMLFormElement::removeFromPastNamesMap(FormAssociatedElement& item)
 
 bool HTMLFormElement::matchesValidPseudoClass() const
 {
-    return m_invalidFormControls.computesEmpty();
+    return m_invalidFormControls.isEmptyIgnoringNullReferences();
 }
 
 bool HTMLFormElement::matchesInvalidPseudoClass() const
@@ -925,6 +933,11 @@ Vector<Ref<Element>> HTMLFormElement::namedElements(const AtomString& name)
     return namedItems;
 }
 
+bool HTMLFormElement::isSupportedPropertyName(const AtomString& name)
+{
+    return !name.isEmpty() && elements()->isSupportedPropertyName(name);
+}
+
 void HTMLFormElement::resumeFromDocumentSuspension()
 {
     ASSERT(!shouldAutocomplete());
