@@ -2,7 +2,8 @@
  * Copyright (C) 1999 Lars Knoll (knoll@kde.org)
  *           (C) 1999 Antti Koivisto (koivisto@kde.org)
  *           (C) 2001 Dirk Mueller (mueller@kde.org)
- * Copyright (C) 2003, 2010 Apple Inc. All rights reserved.
+ * Copyright (C) 2003-2023 Apple Inc. All rights reserved.
+ * Copyright (C) 2013 Google Inc. All rights reserved.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -24,17 +25,19 @@
 #include "HTMLMetaElement.h"
 
 #include "Attribute.h"
+#include "CSSParser.h"
 #include "Color.h"
 #include "Document.h"
 #include "ElementInlines.h"
-#include "Frame.h"
-#include "FrameView.h"
 #include "HTMLHeadElement.h"
 #include "HTMLNames.h"
 #include "HTMLParserIdioms.h"
+#include "LocalFrame.h"
+#include "LocalFrameView.h"
 #include "MediaQueryEvaluator.h"
 #include "MediaQueryParser.h"
 #include "MediaQueryParserContext.h"
+#include "NodeName.h"
 #include "RenderStyle.h"
 #include "Settings.h"
 #include "StyleResolveForDocument.h"
@@ -92,45 +95,32 @@ const Color& HTMLMetaElement::contentColor()
     return *m_contentColor;
 }
 
-void HTMLMetaElement::attributeChanged(const QualifiedName& name, const AtomString& oldValue, const AtomString& newValue, AttributeModificationReason reason)
+void HTMLMetaElement::attributeChanged(const QualifiedName& name, const AtomString& oldValue, const AtomString& newValue, AttributeModificationReason attributeModificationReason)
 {
-    HTMLElement::attributeChanged(name, oldValue, newValue, reason);
+    HTMLElement::attributeChanged(name, oldValue, newValue, attributeModificationReason);
 
-    if (!isInDocumentTree())
-        return;
-
-    if (name == nameAttr) {
-        if (equalLettersIgnoringASCIICase(oldValue, "theme-color"_s) && !equalLettersIgnoringASCIICase(newValue, "theme-color"_s))
-            document().metaElementThemeColorChanged(*this);
-        return;
-    }
-}
-
-void HTMLMetaElement::parseAttribute(const QualifiedName& name, const AtomString& value)
-{
-    if (name == nameAttr) {
+    switch (name.nodeName()) {
+    case AttributeNames::nameAttr:
         process();
-        return;
-    }
-
-    if (name == contentAttr) {
+        if (isInDocumentTree()) {
+            if (equalLettersIgnoringASCIICase(oldValue, "theme-color"_s) && !equalLettersIgnoringASCIICase(newValue, "theme-color"_s))
+                document().metaElementThemeColorChanged(*this);
+        }
+        break;
+    case AttributeNames::contentAttr:
         m_contentColor = std::nullopt;
         process();
-        return;
-    }
-
-    if (name == http_equivAttr) {
+        break;
+    case AttributeNames::http_equivAttr:
         process();
-        return;
-    }
-
-    if (name == mediaAttr) {
+        break;
+    case AttributeNames::mediaAttr:
         m_mediaQueryList = { };
         process();
-        return;
+        break;
+    default:
+        break;
     }
-
-    HTMLElement::parseAttribute(name, value);
 }
 
 Node::InsertedIntoAncestorResult HTMLMetaElement::insertedIntoAncestor(InsertionType insertionType, ContainerNode& parentOfInsertedTree)
@@ -160,32 +150,41 @@ void HTMLMetaElement::process()
     if (!isInDocumentTree())
         return;
 
+    // https://html.spec.whatwg.org/multipage/semantics.html#the-meta-element
+    // All below situations require a content attribute (which can be the empty string).
     const AtomString& contentValue = attributeWithoutSynchronization(contentAttr);
     if (contentValue.isNull())
         return;
-
-    if (equalLettersIgnoringASCIICase(name(), "viewport"_s))
-        document().processViewport(contentValue, ViewportArguments::ViewportMeta);
-    else if (document().settings().disabledAdaptationsMetaTagEnabled() && equalLettersIgnoringASCIICase(name(), "disabled-adaptations"_s))
-        document().processDisabledAdaptations(contentValue);
-#if ENABLE(DARK_MODE_CSS)
-    else if (equalLettersIgnoringASCIICase(name(), "color-scheme"_s) || equalLettersIgnoringASCIICase(name(), "supported-color-schemes"_s))
-        document().processColorScheme(contentValue);
-#endif
-    else if (equalLettersIgnoringASCIICase(name(), "theme-color"_s))
-        document().metaElementThemeColorChanged(*this);
-#if PLATFORM(IOS_FAMILY)
-    else if (equalLettersIgnoringASCIICase(name(), "format-detection"_s))
-        document().processFormatDetection(contentValue);
-    else if (equalLettersIgnoringASCIICase(name(), "apple-mobile-web-app-orientations"_s))
-        document().processWebAppOrientations();
-#endif
-    else if (equalLettersIgnoringASCIICase(name(), "referrer"_s))
-        document().processReferrerPolicy(contentValue, ReferrerPolicySource::MetaTag);
-
+    
     const AtomString& httpEquivValue = attributeWithoutSynchronization(http_equivAttr);
+    // Get the document to process the tag, but only if we're actually part of DOM
+    // tree (changing a meta tag while it's not in the tree shouldn't have any effect
+    // on the document)
     if (!httpEquivValue.isNull())
         document().processMetaHttpEquiv(httpEquivValue, contentValue, isDescendantOf(document().head()));
+    
+    const AtomString& nameValue = attributeWithoutSynchronization(nameAttr);
+    if (nameValue.isNull())
+        return;
+
+    if (equalLettersIgnoringASCIICase(nameValue, "viewport"_s))
+        document().processViewport(contentValue, ViewportArguments::ViewportMeta);
+    else if (document().settings().disabledAdaptationsMetaTagEnabled() && equalLettersIgnoringASCIICase(nameValue, "disabled-adaptations"_s))
+        document().processDisabledAdaptations(contentValue);
+#if ENABLE(DARK_MODE_CSS)
+    else if (equalLettersIgnoringASCIICase(nameValue, "color-scheme"_s) || equalLettersIgnoringASCIICase(nameValue, "supported-color-schemes"_s))
+        document().processColorScheme(contentValue);
+#endif
+    else if (equalLettersIgnoringASCIICase(nameValue, "theme-color"_s))
+        document().metaElementThemeColorChanged(*this);
+#if PLATFORM(IOS_FAMILY)
+    else if (equalLettersIgnoringASCIICase(nameValue, "format-detection"_s))
+        document().processFormatDetection(contentValue);
+    else if (equalLettersIgnoringASCIICase(nameValue, "apple-mobile-web-app-orientations"_s))
+        document().processWebAppOrientations();
+#endif
+    else if (equalLettersIgnoringASCIICase(nameValue, "referrer"_s))
+        document().processReferrerPolicy(contentValue, ReferrerPolicySource::MetaTag);
 }
 
 const AtomString& HTMLMetaElement::content() const
