@@ -28,19 +28,21 @@
 #include "Document.h"
 #include "Event.h"
 #include "EventHandler.h"
-#include "Frame.h"
 #include "FrameLoader.h"
 #include "FrameTree.h"
 #include "HTMLNames.h"
 #include "HitTestResult.h"
+#include "LocalFrame.h"
 #include "Logging.h"
 #include "MIMETypeRegistry.h"
+#include "NodeName.h"
 #include "Page.h"
 #include "PluginData.h"
 #include "PluginReplacement.h"
 #include "PluginViewBase.h"
 #include "RenderEmbeddedObject.h"
 #include "RenderLayer.h"
+#include "RenderStyleInlines.h"
 #include "RenderView.h"
 #include "RenderWidget.h"
 #include "ScriptController.h"
@@ -50,12 +52,7 @@
 #include "Widget.h"
 #include <wtf/IsoMallocInlines.h>
 
-#if ENABLE(NETSCAPE_PLUGIN_API)
-#include "npruntime_impl.h"
-#endif
-
 #if PLATFORM(COCOA)
-#include "QuickTimePluginReplacement.h"
 #include "YouTubePluginReplacement.h"
 #endif
 
@@ -66,10 +63,9 @@ WTF_MAKE_ISO_ALLOCATED_IMPL(HTMLPlugInElement);
 using namespace HTMLNames;
 
 HTMLPlugInElement::HTMLPlugInElement(const QualifiedName& tagName, Document& document)
-    : HTMLFrameOwnerElement(tagName, document)
+    : HTMLFrameOwnerElement(tagName, document, CreateHTMLPlugInElement)
     , m_swapRendererTimer(*this, &HTMLPlugInElement::swapRendererTimerFired)
 {
-    setHasCustomStyleResolveCallbacks();
 }
 
 HTMLPlugInElement::~HTMLPlugInElement()
@@ -77,7 +73,7 @@ HTMLPlugInElement::~HTMLPlugInElement()
     ASSERT(!m_instance); // cleared in detach()
 }
 
-bool HTMLPlugInElement::willRespondToMouseClickEvents()
+bool HTMLPlugInElement::willRespondToMouseClickEventsWithEditability(Editability) const
 {
     if (isDisabledFormControl())
         return false;
@@ -90,7 +86,7 @@ void HTMLPlugInElement::willDetachRenderers()
     m_instance = nullptr;
 
     if (m_isCapturingMouseEvents) {
-        if (RefPtr<Frame> frame = document().frame())
+        if (RefPtr frame = document().frame())
             frame->eventHandler().setCapturingMouseEventsElement(nullptr);
         m_isCapturingMouseEvents = false;
     }
@@ -103,7 +99,7 @@ void HTMLPlugInElement::resetInstance()
 
 JSC::Bindings::Instance* HTMLPlugInElement::bindingsInstance()
 {
-    auto frame = makeRefPtr(document().frame());
+    RefPtr frame = document().frame();
     if (!frame)
         return nullptr;
 
@@ -111,45 +107,24 @@ JSC::Bindings::Instance* HTMLPlugInElement::bindingsInstance()
     // the cached allocated Bindings::Instance.  Not supporting this edge-case is OK.
 
     if (!m_instance) {
-        if (auto widget = makeRefPtr(pluginWidget()))
+        if (RefPtr widget = pluginWidget())
             m_instance = frame->script().createScriptInstanceForWidget(widget.get());
     }
     return m_instance.get();
 }
 
-bool HTMLPlugInElement::guardedDispatchBeforeLoadEvent(const String& sourceURL)
+PluginViewBase* HTMLPlugInElement::pluginWidget(PluginLoadingPolicy loadPolicy) const
 {
-    // FIXME: Our current plug-in loading design can't guarantee the following
-    // assertion is true, since plug-in loading can be initiated during layout,
-    // and synchronous layout can be initiated in a beforeload event handler!
-    // See <http://webkit.org/b/71264>.
-    // ASSERT(!m_inBeforeLoadEventHandler);
-    m_inBeforeLoadEventHandler = true;
-    // static_cast is used to avoid a compile error since dispatchBeforeLoadEvent
-    // is intentionally undefined on this class.
-    bool beforeLoadAllowedLoad = static_cast<HTMLFrameOwnerElement*>(this)->dispatchBeforeLoadEvent(sourceURL);
-    m_inBeforeLoadEventHandler = false;
-    return beforeLoadAllowedLoad;
-}
-
-Widget* HTMLPlugInElement::pluginWidget(PluginLoadingPolicy loadPolicy) const
-{
-    if (m_inBeforeLoadEventHandler) {
-        // The plug-in hasn't loaded yet, and it makes no sense to try to load if beforeload handler happened to touch the plug-in element.
-        // That would recursively call beforeload for the same element.
-        return nullptr;
-    }
-
     RenderWidget* renderWidget = loadPolicy == PluginLoadingPolicy::Load ? renderWidgetLoadingPlugin() : this->renderWidget();
     if (!renderWidget)
         return nullptr;
 
-    return renderWidget->widget();
+    return dynamicDowncast<PluginViewBase>(renderWidget->widget());
 }
 
 RenderWidget* HTMLPlugInElement::renderWidgetLoadingPlugin() const
 {
-    RefPtr<FrameView> view = document().view();
+    RefPtr view = document().view();
     if (!view || (!view->inUpdateEmbeddedObjects() && !view->layoutContext().isInLayout() && !view->isPainting())) {
         // Needs to load the plugin immediatedly because this function is called
         // when JavaScript code accesses the plugin.
@@ -159,29 +134,45 @@ RenderWidget* HTMLPlugInElement::renderWidgetLoadingPlugin() const
     return renderWidget(); // This will return nullptr if the renderer is not a RenderWidget.
 }
 
-bool HTMLPlugInElement::isPresentationAttribute(const QualifiedName& name) const
+bool HTMLPlugInElement::hasPresentationalHintsForAttribute(const QualifiedName& name) const
 {
-    if (name == widthAttr || name == heightAttr || name == vspaceAttr || name == hspaceAttr || name == alignAttr)
+    switch (name.nodeName()) {
+    case AttributeNames::widthAttr:
+    case AttributeNames::heightAttr:
+    case AttributeNames::vspaceAttr:
+    case AttributeNames::hspaceAttr:
+    case AttributeNames::alignAttr:
         return true;
-    return HTMLFrameOwnerElement::isPresentationAttribute(name);
+    default:
+        break;
+    }
+    return HTMLFrameOwnerElement::hasPresentationalHintsForAttribute(name);
 }
 
-void HTMLPlugInElement::collectStyleForPresentationAttribute(const QualifiedName& name, const AtomString& value, MutableStyleProperties& style)
+void HTMLPlugInElement::collectPresentationalHintsForAttribute(const QualifiedName& name, const AtomString& value, MutableStyleProperties& style)
 {
-    if (name == widthAttr)
+    switch (name.nodeName()) {
+    case AttributeNames::widthAttr:
         addHTMLLengthToStyle(style, CSSPropertyWidth, value);
-    else if (name == heightAttr)
+        break;
+    case AttributeNames::heightAttr:
         addHTMLLengthToStyle(style, CSSPropertyHeight, value);
-    else if (name == vspaceAttr) {
+        break;
+    case AttributeNames::vspaceAttr:
         addHTMLLengthToStyle(style, CSSPropertyMarginTop, value);
         addHTMLLengthToStyle(style, CSSPropertyMarginBottom, value);
-    } else if (name == hspaceAttr) {
+        break;
+    case AttributeNames::hspaceAttr:
         addHTMLLengthToStyle(style, CSSPropertyMarginLeft, value);
         addHTMLLengthToStyle(style, CSSPropertyMarginRight, value);
-    } else if (name == alignAttr)
+        break;
+    case AttributeNames::alignAttr:
         applyAlignmentAttributeToStyle(value, style);
-    else
-        HTMLFrameOwnerElement::collectStyleForPresentationAttribute(name, value, style);
+        break;
+    default:
+        HTMLFrameOwnerElement::collectPresentationalHintsForAttribute(name, value, style);
+        break;
+    }
 }
 
 void HTMLPlugInElement::defaultEventHandler(Event& event)
@@ -202,9 +193,8 @@ void HTMLPlugInElement::defaultEventHandler(Event& event)
     // Don't keep the widget alive over the defaultEventHandler call, since that can do things like navigate.
     {
         RefPtr<Widget> widget = downcast<RenderWidget>(*renderer).widget();
-        if (!widget)
-            return;
-        widget->handleEvent(event);
+        if (widget)
+            widget->handleEvent(event);
         if (event.defaultHandled())
             return;
     }
@@ -213,15 +203,7 @@ void HTMLPlugInElement::defaultEventHandler(Event& event)
 
 bool HTMLPlugInElement::isKeyboardFocusable(KeyboardEvent*) const
 {
-    // FIXME: Why is this check needed?
-    if (!document().page())
-        return false;
-
-    RefPtr<Widget> widget = pluginWidget();
-    if (!is<PluginViewBase>(widget))
-        return false;
-
-    return downcast<PluginViewBase>(*widget).supportsKeyboardFocus();
+    return false;
 }
 
 bool HTMLPlugInElement::isPluginElement() const
@@ -229,23 +211,6 @@ bool HTMLPlugInElement::isPluginElement() const
     return true;
 }
 
-bool HTMLPlugInElement::isUserObservable() const
-{
-    // No widget - can't be anything to see or hear here.
-    RefPtr<Widget> widget = pluginWidget(PluginLoadingPolicy::DoNotLoad);
-    if (!is<PluginViewBase>(widget))
-        return false;
-
-    PluginViewBase& pluginView = downcast<PluginViewBase>(*widget);
-
-    // If audio is playing (or might be) then the plugin is detectable.
-    if (pluginView.audioHardwareActivity() != AudioHardwareActivityType::IsInactive)
-        return true;
-
-    // If the plugin is visible and not vanishingly small in either dimension it is detectable.
-    return pluginView.isVisible() && pluginView.width() > 2 && pluginView.height() > 2;
-}
-
 bool HTMLPlugInElement::supportsFocus() const
 {
     if (HTMLFrameOwnerElement::supportsFocus())
@@ -272,6 +237,7 @@ void HTMLPlugInElement::swapRendererTimerFired()
     
     // Create a shadow root, which will trigger the code to add a snapshot container
     // and reattach, thus making a new Renderer.
+    Ref<HTMLPlugInElement> protectedThis(*this);
     ensureUserAgentShadowRoot();
 }
 
@@ -293,10 +259,11 @@ void HTMLPlugInElement::didAddUserAgentShadowRoot(ShadowRoot& root)
         return;
     
     root.setResetStyleInheritance(true);
-    if (m_pluginReplacement->installReplacement(root)) {
-        setDisplayState(DisplayingPluginReplacement);
-        invalidateStyleAndRenderersForSubtree();
-    }
+
+    m_pluginReplacement->installReplacement(root);
+
+    setDisplayState(DisplayingPluginReplacement);
+    invalidateStyleAndRenderersForSubtree();
 }
 
 #if PLATFORM(COCOA)
@@ -313,7 +280,6 @@ static Vector<ReplacementPlugin*>& registeredPluginReplacements()
     enginesQueried = true;
 
 #if PLATFORM(COCOA)
-    QuickTimePluginReplacement::registerPluginReplacement(registrar);
     YouTubePluginReplacement::registerPluginReplacement(registrar);
 #endif
     
@@ -333,11 +299,11 @@ static ReplacementPlugin* pluginReplacementForType(const URL& url, const String&
     if (replacements.isEmpty())
         return nullptr;
 
-    String extension;
+    StringView extension;
     auto lastPathComponent = url.lastPathComponent();
     size_t dotOffset = lastPathComponent.reverseFind('.');
     if (dotOffset != notFound)
-        extension = lastPathComponent.substring(dotOffset + 1).toString();
+        extension = lastPathComponent.substring(dotOffset + 1);
 
     String type = mimeType;
     if (type.isEmpty() && url.protocolIsData())
@@ -367,7 +333,7 @@ static ReplacementPlugin* pluginReplacementForType(const URL& url, const String&
     return nullptr;
 }
 
-bool HTMLPlugInElement::requestObject(const String& relativeURL, const String& mimeType, const Vector<String>& paramNames, const Vector<String>& paramValues)
+bool HTMLPlugInElement::requestObject(const String& relativeURL, const String& mimeType, const Vector<AtomString>& paramNames, const Vector<AtomString>& paramValues)
 {
     if (m_pluginReplacement)
         return true;
@@ -387,22 +353,6 @@ bool HTMLPlugInElement::requestObject(const String& relativeURL, const String& m
     return true;
 }
 
-JSC::JSObject* HTMLPlugInElement::scriptObjectForPluginReplacement()
-{
-    if (m_pluginReplacement)
-        return m_pluginReplacement->scriptObject();
-    return nullptr;
-}
-
-bool HTMLPlugInElement::isBelowSizeThreshold() const
-{
-    auto* renderObject = renderer();
-    if (!is<RenderEmbeddedObject>(renderObject))
-        return true;
-    auto& renderEmbeddedObject = downcast<RenderEmbeddedObject>(*renderObject);
-    return renderEmbeddedObject.isPluginUnavailable() && renderEmbeddedObject.pluginUnavailabilityReason() == RenderEmbeddedObject::PluginTooSmall;
-}
-
 bool HTMLPlugInElement::setReplacement(RenderEmbeddedObject::PluginUnavailabilityReason reason, const String& unavailabilityDescription)
 {
     if (!is<RenderEmbeddedObject>(renderer()))
@@ -422,8 +372,8 @@ bool HTMLPlugInElement::setReplacement(RenderEmbeddedObject::PluginUnavailabilit
 
 bool HTMLPlugInElement::isReplacementObscured()
 {
-    auto topDocument = makeRef(document().topDocument());
-    auto topFrameView = makeRefPtr(topDocument->view());
+    Ref topDocument = document().topDocument();
+    RefPtr topFrameView = topDocument->view();
     if (!topFrameView)
         return false;
 
@@ -456,7 +406,7 @@ bool HTMLPlugInElement::isReplacementObscured()
     auto height = viewRect.height();
     // Hit test the center and near the corners of the replacement text to ensure
     // it is visible and is not masked by other elements.
-    constexpr OptionSet<HitTestRequest::RequestType> hitType { HitTestRequest::ReadOnly, HitTestRequest::Active, HitTestRequest::IgnoreClipping, HitTestRequest::DisallowUserAgentShadowContent, HitTestRequest::AllowChildFrameContent };
+    constexpr OptionSet<HitTestRequest::Type> hitType { HitTestRequest::Type::ReadOnly, HitTestRequest::Type::Active, HitTestRequest::Type::IgnoreClipping, HitTestRequest::Type::DisallowUserAgentShadowContent, HitTestRequest::Type::AllowChildFrameContent };
     HitTestResult result;
     HitTestLocation location { LayoutPoint { viewRect.center() } };
     ASSERT(!renderView->needsLayout());
