@@ -31,6 +31,7 @@
 #include "HTMLNames.h"
 #include "HTMLParserIdioms.h"
 #include "HTMLTableElement.h"
+#include "NodeName.h"
 #include "RenderTableCell.h"
 #include <wtf/IsoMallocInlines.h>
 
@@ -93,38 +94,48 @@ int HTMLTableCellElement::cellIndex() const
 
 bool HTMLTableCellElement::hasPresentationalHintsForAttribute(const QualifiedName& name) const
 {
-    if (name == nowrapAttr || name == widthAttr || name == heightAttr)
+    switch (name.nodeName()) {
+    case AttributeNames::nowrapAttr:
+    case AttributeNames::widthAttr:
+    case AttributeNames::heightAttr:
         return true;
+    default:
+        break;
+    }
     return HTMLTablePartElement::hasPresentationalHintsForAttribute(name);
 }
 
 void HTMLTableCellElement::collectPresentationalHintsForAttribute(const QualifiedName& name, const AtomString& value, MutableStyleProperties& style)
 {
-    if (name == nowrapAttr)
+    switch (name.nodeName()) {
+    case AttributeNames::nowrapAttr:
         addPropertyToPresentationalHintStyle(style, CSSPropertyWhiteSpace, CSSValueNowrap);
-    else if (name == widthAttr) {
+        break;
+    case AttributeNames::widthAttr:
         // width="0" is not allowed for compatibility with WinIE.
         addHTMLLengthToStyle(style, CSSPropertyWidth, value, AllowZeroValue::No);
-    } else if (name == heightAttr) {
+        break;
+    case AttributeNames::heightAttr:
         // width="0" is not allowed for compatibility with WinIE.
         addHTMLLengthToStyle(style, CSSPropertyHeight, value, AllowZeroValue::No);
-    } else
+        break;
+    default:
         HTMLTablePartElement::collectPresentationalHintsForAttribute(name, value, style);
+        break;
+    }
 }
 
-void HTMLTableCellElement::parseAttribute(const QualifiedName& name, const AtomString& value)
+void HTMLTableCellElement::attributeChanged(const QualifiedName& name, const AtomString& oldValue, const AtomString& newValue, AttributeModificationReason attributeModificationReason)
 {
-    if (name == rowspanAttr) {
+    HTMLTablePartElement::attributeChanged(name, oldValue, newValue, attributeModificationReason);
+
+    if (name == rowspanAttr || name == colspanAttr) {
         if (is<RenderTableCell>(renderer()))
             downcast<RenderTableCell>(*renderer()).colSpanOrRowSpanChanged();
-    } else if (name == colspanAttr) {
-        if (is<RenderTableCell>(renderer()))
-            downcast<RenderTableCell>(*renderer()).colSpanOrRowSpanChanged();
-    } else
-        HTMLTablePartElement::parseAttribute(name, value);
+    }
 }
 
-const StyleProperties* HTMLTableCellElement::additionalPresentationalHintStyle() const
+const MutableStyleProperties* HTMLTableCellElement::additionalPresentationalHintStyle() const
 {
     if (auto table = findParentTable())
         return table->additionalCellStyle();
