@@ -1,7 +1,7 @@
 /*
- * Copyright (C) 2008, 2014 Apple Inc. All Rights Reserved.
+ * Copyright (C) 2008-2023 Apple Inc. All Rights Reserved.
  * Copyright (C) 2009 Torch Mobile, Inc. http://www.torchmobile.com/
- * Copyright (C) 2010 Google Inc. All Rights Reserved.
+ * Copyright (C) 2010-2021 Google Inc. All Rights Reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -30,10 +30,10 @@
 
 #include "HTMLImageElement.h"
 #include "HTMLNames.h"
-#include "HTMLParserIdioms.h"
 #include "HTMLSrcsetParser.h"
 #include "HTMLTokenizer.h"
 #include "InputTypeNames.h"
+#include "JSRequestPriority.h"
 #include "LinkLoader.h"
 #include "LinkRelAttribute.h"
 #include "Logging.h"
@@ -42,11 +42,11 @@
 #include "MediaQueryEvaluator.h"
 #include "MediaQueryParser.h"
 #include "RenderView.h"
-#include "RuntimeEnabledFeatures.h"
 #include "SecurityPolicy.h"
 #include "Settings.h"
 #include "SizesAttributeParser.h"
 #include <wtf/MainThread.h>
+#include <wtf/SortedArrayMap.h>
 
 namespace WebCore {
 
@@ -54,31 +54,23 @@ using namespace HTMLNames;
 
 TokenPreloadScanner::TagId TokenPreloadScanner::tagIdFor(const HTMLToken::DataVector& data)
 {
-    AtomString tagName(data);
-    if (tagName == imgTag)
-        return TagId::Img;
-    if (tagName == inputTag)
-        return TagId::Input;
-    if (tagName == linkTag)
-        return TagId::Link;
-    if (tagName == scriptTag)
-        return TagId::Script;
-    if (tagName == styleTag)
-        return TagId::Style;
-    if (tagName == baseTag)
-        return TagId::Base;
-    if (tagName == templateTag)
-        return TagId::Template;
-    if (tagName == metaTag)
-        return TagId::Meta;
-    if (tagName == pictureTag)
-        return TagId::Picture;
-    if (tagName == sourceTag)
-        return TagId::Source;
-    return TagId::Unknown;
+    static constexpr std::pair<PackedASCIILiteral<uint64_t>, TokenPreloadScanner::TagId> mappings[] = {
+        { "base", TagId::Base },
+        { "img", TagId::Img },
+        { "input", TagId::Input },
+        { "link", TagId::Link },
+        { "meta", TagId::Meta },
+        { "picture", TagId::Picture },
+        { "script", TagId::Script },
+        { "source", TagId::Source },
+        { "style", TagId::Style },
+        { "template", TagId::Template },
+    };
+    static constexpr SortedArrayMap map { mappings };
+    return map.get(data.span(), TagId::Unknown);
 }
 
-String TokenPreloadScanner::initiatorFor(TagId tagId)
+ASCIILiteral TokenPreloadScanner::initiatorFor(TagId tagId)
 {
     switch (tagId) {
     case TagId::Source:
@@ -105,8 +97,9 @@ String TokenPreloadScanner::initiatorFor(TagId tagId)
 
 class TokenPreloadScanner::StartTagScanner {
 public:
-    explicit StartTagScanner(TagId tagId, float deviceScaleFactor = 1.0)
-        : m_tagId(tagId)
+    explicit StartTagScanner(Document& document, TagId tagId, float deviceScaleFactor = 1.0)
+        : m_document(document)
+        , m_tagId(tagId)
         , m_linkIsStyleSheet(false)
         , m_linkIsPreload(false)
         , m_metaIsViewport(false)
@@ -116,40 +109,39 @@ public:
     {
     }
 
-    void processAttributes(const HTMLToken::AttributeList& attributes, Document& document, Vector<bool>& pictureState)
+    void processAttributes(const HTMLToken::AttributeList& attributes, Vector<bool>& pictureState)
     {
         ASSERT(isMainThread());
         if (m_tagId >= TagId::Unknown)
             return;
-        
+
         for (auto& attribute : attributes) {
-            AtomString attributeName(attribute.name);
-            String attributeValue = StringImpl::create8BitIfPossible(attribute.value);
-            processAttribute(attributeName, attributeValue, document, pictureState);
+            auto knownAttributeName = AtomString::lookUp(attribute.name.data(), attribute.name.size());
+            StringView attributeValue { attribute.value.data(), static_cast<unsigned>(attribute.value.size()) };
+            processAttribute(knownAttributeName, attributeValue, pictureState);
         }
-        
+
         if (m_tagId == TagId::Source && !pictureState.isEmpty() && !pictureState.last() && m_mediaMatched && m_typeMatched && !m_srcSetAttribute.isEmpty()) {
-            
-            auto sourceSize = SizesAttributeParser(m_sizesAttribute, document).length();
-            ImageCandidate imageCandidate = bestFitSourceForImageAttributes(m_deviceScaleFactor, m_urlToLoad, m_srcSetAttribute, sourceSize);
+            auto sourceSize = SizesAttributeParser(m_sizesAttribute, m_document).length();
+            ImageCandidate imageCandidate = bestFitSourceForImageAttributes(m_deviceScaleFactor, AtomString { m_urlToLoad }, m_srcSetAttribute, sourceSize);
             if (!imageCandidate.isEmpty()) {
                 pictureState.last() = true;
-                setUrlToLoad(imageCandidate.string.toString(), true);
+                setURLToLoadAllowingReplacement(imageCandidate.string);
             }
         }
-        
+
         // Resolve between src and srcSet if we have them and the tag is img.
         if (m_tagId == TagId::Img && !m_srcSetAttribute.isEmpty()) {
-            auto sourceSize = SizesAttributeParser(m_sizesAttribute, document).length();
-            ImageCandidate imageCandidate = bestFitSourceForImageAttributes(m_deviceScaleFactor, m_urlToLoad, m_srcSetAttribute, sourceSize);
-            setUrlToLoad(imageCandidate.string.toString(), true);
+            auto sourceSize = SizesAttributeParser(m_sizesAttribute, m_document).length();
+            ImageCandidate imageCandidate = bestFitSourceForImageAttributes(m_deviceScaleFactor, AtomString { m_urlToLoad }, m_srcSetAttribute, sourceSize);
+            setURLToLoadAllowingReplacement(imageCandidate.string);
         }
 
         if (m_metaIsViewport && !m_metaContent.isNull())
-            document.processViewport(m_metaContent, ViewportArguments::ViewportMeta);
+            m_document.processViewport(m_metaContent, ViewportArguments::ViewportMeta);
 
         if (m_metaIsDisabledAdaptations && !m_metaContent.isNull())
-            document.processDisabledAdaptations(m_metaContent);
+            m_document.processDisabledAdaptations(m_metaContent);
     }
 
     std::unique_ptr<PreloadRequest> createPreloadRequest(const URL& predictedBaseURL)
@@ -161,14 +153,14 @@ public:
         if (!type)
             return nullptr;
 
-        if (!LinkLoader::isSupportedType(type.value(), m_typeAttribute))
+        if (!LinkLoader::isSupportedType(type.value(), m_typeAttribute, m_document))
             return nullptr;
 
         // Do not preload if lazyload is possible but metadata fetch is disabled.
         if (HTMLImageElement::hasLazyLoadableAttributeValue(m_lazyloadAttribute))
             return nullptr;
 
-        auto request = makeUnique<PreloadRequest>(initiatorFor(m_tagId), m_urlToLoad, predictedBaseURL, type.value(), m_mediaAttribute, m_moduleScript, m_referrerPolicy);
+        auto request = makeUnique<PreloadRequest>(initiatorFor(m_tagId), m_urlToLoad, predictedBaseURL, type.value(), m_mediaAttribute, m_scriptType, m_referrerPolicy, m_fetchPriorityHint);
         request->setCrossOriginMode(m_crossOriginMode);
         request->setNonce(m_nonceAttribute);
         request->setScriptIsAsync(m_scriptIsAsync);
@@ -187,17 +179,17 @@ public:
     }
 
 private:
-    void processImageAndScriptAttribute(const AtomString& attributeName, const String& attributeValue)
+    void processImageAndScriptAttribute(const AtomString& attributeName, StringView attributeValue)
     {
         if (match(attributeName, srcAttr))
-            setUrlToLoad(attributeValue);
+            setURLToLoad(attributeValue);
         else if (match(attributeName, crossoriginAttr))
-            m_crossOriginMode = stripLeadingAndTrailingHTMLSpaces(attributeValue);
+            m_crossOriginMode = attributeValue.trim(isASCIIWhitespace<UChar>).toString();
         else if (match(attributeName, charsetAttr))
-            m_charset = attributeValue;
+            m_charset = attributeValue.toString();
     }
 
-    void processAttribute(const AtomString& attributeName, const String& attributeValue, Document& document, const Vector<bool>& pictureState)
+    void processAttribute(const AtomString& attributeName, StringView attributeValue, const Vector<bool>& pictureState)
     {
         bool inPicture = !pictureState.isEmpty();
         bool alreadyMatchedSource = inPicture && pictureState.last();
@@ -207,16 +199,20 @@ private:
             if (inPicture && alreadyMatchedSource)
                 break;
             if (match(attributeName, srcsetAttr) && m_srcSetAttribute.isNull()) {
-                m_srcSetAttribute = attributeValue;
+                m_srcSetAttribute = attributeValue.toString();
                 break;
             }
             if (match(attributeName, sizesAttr) && m_sizesAttribute.isNull()) {
-                m_sizesAttribute = attributeValue;
+                m_sizesAttribute = attributeValue.toString();
                 break;
             }
-            if (RuntimeEnabledFeatures::sharedFeatures().lazyImageLoadingEnabled()) {
+            if (match(attributeName, fetchpriorityAttr) && m_document.settings().fetchPriorityEnabled()) {
+                m_fetchPriorityHint = parseEnumerationFromString<RequestPriority>(attributeValue.toString()).value_or(RequestPriority::Auto);
+                break;
+            }
+            if (m_document.settings().lazyImageLoadingEnabled()) {
                 if (match(attributeName, loadingAttr) && m_lazyloadAttribute.isNull()) {
-                    m_lazyloadAttribute = attributeValue;
+                    m_lazyloadAttribute = attributeValue.toString();
                     break;
                 }
             }
@@ -226,35 +222,38 @@ private:
             if (inPicture && alreadyMatchedSource)
                 break;
             if (match(attributeName, srcsetAttr) && m_srcSetAttribute.isNull()) {
-                m_srcSetAttribute = attributeValue;
+                m_srcSetAttribute = attributeValue.toString();
                 break;
             }
             if (match(attributeName, sizesAttr) && m_sizesAttribute.isNull()) {
-                m_sizesAttribute = attributeValue;
+                m_sizesAttribute = attributeValue.toString();
                 break;
             }
             if (match(attributeName, mediaAttr) && m_mediaAttribute.isNull()) {
-                m_mediaAttribute = attributeValue;
-                auto mediaSet = MediaQuerySet::create(attributeValue, MediaQueryParserContext(document));
-                auto documentElement = makeRefPtr(document.documentElement());
+                m_mediaAttribute = attributeValue.toString();
+                auto mediaQueries = MQ::MediaQueryParser::parse(m_mediaAttribute, { m_document });
+                RefPtr documentElement = m_document.documentElement();
                 LOG(MediaQueries, "HTMLPreloadScanner %p processAttribute evaluating media queries", this);
-                m_mediaMatched = MediaQueryEvaluator { document.printing() ? "print" : "screen", document, documentElement ? documentElement->computedStyle() : nullptr }.evaluate(mediaSet.get());
+                m_mediaMatched = MQ::MediaQueryEvaluator { m_document.printing() ? printAtom() : screenAtom(), m_document, documentElement ? documentElement->computedStyle() : nullptr }.evaluate(mediaQueries);
             }
             if (match(attributeName, typeAttr) && m_typeAttribute.isNull()) {
                 // when multiple type attributes present: first value wins, ignore subsequent (to match ImageElement parser and Blink behaviours)
-                m_typeAttribute = attributeValue;
+                m_typeAttribute = attributeValue.toString();
                 m_typeMatched &= MIMETypeRegistry::isSupportedImageVideoOrSVGMIMEType(m_typeAttribute);
             }
             break;
         case TagId::Script:
             if (match(attributeName, typeAttr)) {
-                m_moduleScript = equalLettersIgnoringASCIICase(attributeValue, "module") ? PreloadRequest::ModuleScript::Yes : PreloadRequest::ModuleScript::No;
+                if (equalLettersIgnoringASCIICase(attributeValue, "module"_s))
+                    m_scriptType = ScriptType::Module;
+                else if (equalLettersIgnoringASCIICase(attributeValue, "importmap"_s))
+                    m_scriptType = ScriptType::ImportMap;
                 break;
             } else if (match(attributeName, nonceAttr)) {
-                m_nonceAttribute = attributeValue;
+                m_nonceAttribute = attributeValue.toString();
                 break;
             } else if (match(attributeName, referrerpolicyAttr)) {
-                m_referrerPolicy = parseReferrerPolicy(attributeValue, ReferrerPolicySource::ReferrerPolicyAttribute).valueOr(ReferrerPolicy::EmptyString);
+                m_referrerPolicy = parseReferrerPolicy(attributeValue, ReferrerPolicySource::ReferrerPolicyAttribute).value_or(ReferrerPolicy::EmptyString);
                 break;
             } else if (match(attributeName, nomoduleAttr)) {
                 m_scriptIsNomodule = true;
@@ -262,44 +261,49 @@ private:
             } else if (match(attributeName, asyncAttr)) {
                 m_scriptIsAsync = true;
                 break;
+            } else if (match(attributeName, fetchpriorityAttr) && m_document.settings().fetchPriorityEnabled()) {
+                m_fetchPriorityHint = parseEnumerationFromString<RequestPriority>(attributeValue.toString()).value_or(RequestPriority::Auto);
+                break;
             }
             processImageAndScriptAttribute(attributeName, attributeValue);
             break;
         case TagId::Link:
             if (match(attributeName, hrefAttr))
-                setUrlToLoad(attributeValue);
+                setURLToLoad(attributeValue);
             else if (match(attributeName, relAttr)) {
-                LinkRelAttribute parsedAttribute { document, attributeValue };
+                LinkRelAttribute parsedAttribute { m_document, attributeValue };
                 m_linkIsStyleSheet = relAttributeIsStyleSheet(parsedAttribute);
                 m_linkIsPreload = parsedAttribute.isLinkPreload;
             } else if (match(attributeName, mediaAttr))
-                m_mediaAttribute = attributeValue;
+                m_mediaAttribute = attributeValue.toString();
             else if (match(attributeName, charsetAttr))
-                m_charset = attributeValue;
+                m_charset = attributeValue.toString();
             else if (match(attributeName, crossoriginAttr))
-                m_crossOriginMode = stripLeadingAndTrailingHTMLSpaces(attributeValue);
+                m_crossOriginMode = attributeValue.trim(isASCIIWhitespace<UChar>).toString();
             else if (match(attributeName, nonceAttr))
-                m_nonceAttribute = attributeValue;
+                m_nonceAttribute = attributeValue.toString();
             else if (match(attributeName, asAttr))
-                m_asAttribute = attributeValue;
+                m_asAttribute = attributeValue.toString();
             else if (match(attributeName, typeAttr))
-                m_typeAttribute = attributeValue;
+                m_typeAttribute = attributeValue.toString();
             else if (match(attributeName, referrerpolicyAttr))
-                m_referrerPolicy = parseReferrerPolicy(attributeValue, ReferrerPolicySource::ReferrerPolicyAttribute).valueOr(ReferrerPolicy::EmptyString);
+                m_referrerPolicy = parseReferrerPolicy(attributeValue, ReferrerPolicySource::ReferrerPolicyAttribute).value_or(ReferrerPolicy::EmptyString);
+            else if (match(attributeName, fetchpriorityAttr) && m_document.settings().fetchPriorityEnabled())
+                m_fetchPriorityHint = parseEnumerationFromString<RequestPriority>(attributeValue.toString()).value_or(RequestPriority::Auto);
             break;
         case TagId::Input:
             if (match(attributeName, srcAttr))
-                setUrlToLoad(attributeValue);
+                setURLToLoad(attributeValue);
             else if (match(attributeName, typeAttr))
-                m_inputIsImage = equalLettersIgnoringASCIICase(attributeValue, "image");
+                m_inputIsImage = equalLettersIgnoringASCIICase(attributeValue, "image"_s);
             break;
         case TagId::Meta:
             if (match(attributeName, contentAttr))
-                m_metaContent = attributeValue;
+                m_metaContent = attributeValue.toString();
             else if (match(attributeName, nameAttr))
-                m_metaIsViewport = equalLettersIgnoringASCIICase(attributeValue, "viewport");
-            else if (RuntimeEnabledFeatures::sharedFeatures().disabledAdaptationsMetaTagEnabled() && match(attributeName, nameAttr))
-                m_metaIsDisabledAdaptations = equalLettersIgnoringASCIICase(attributeValue, "disabled-adaptations");
+                m_metaIsViewport = equalLettersIgnoringASCIICase(attributeValue, "viewport"_s);
+            else if (m_document.settings().disabledAdaptationsMetaTagEnabled() && match(attributeName, nameAttr))
+                m_metaIsDisabledAdaptations = equalLettersIgnoringASCIICase(attributeValue, "disabled-adaptations"_s);
             break;
         case TagId::Base:
         case TagId::Style:
@@ -315,16 +319,21 @@ private:
         return parsedAttribute.isStyleSheet && !parsedAttribute.isAlternate && !parsedAttribute.iconType && !parsedAttribute.isDNSPrefetch;
     }
 
-    void setUrlToLoad(const String& value, bool allowReplacement = false)
+    void setURLToLoad(StringView value)
     {
         // We only respect the first src/href, per HTML5:
         // http://www.whatwg.org/specs/web-apps/current-work/multipage/tokenization.html#attribute-name-state
-        if (!allowReplacement && !m_urlToLoad.isEmpty())
+        if (!m_urlToLoad.isEmpty())
             return;
-        String url = stripLeadingAndTrailingHTMLSpaces(value);
-        if (url.isEmpty())
+        setURLToLoadAllowingReplacement(value);
+    }
+
+    void setURLToLoadAllowingReplacement(StringView value)
+    {
+        auto trimmedURL = value.trim(isASCIIWhitespace<UChar>);
+        if (trimmedURL.isEmpty())
             return;
-        m_urlToLoad = url;
+        m_urlToLoad = trimmedURL.toString();
     }
 
     const String& charset() const
@@ -332,7 +341,7 @@ private:
         return m_charset;
     }
 
-    Optional<CachedResource::Type> resourceType() const
+    std::optional<CachedResource::Type> resourceType() const
     {
         switch (m_tagId) {
         case TagId::Script:
@@ -346,7 +355,7 @@ private:
             if (m_linkIsStyleSheet)
                 return CachedResource::Type::CSSStyleSheet;
             if (m_linkIsPreload)
-                return LinkLoader::resourceTypeFromAsAttribute(m_asAttribute);
+                return LinkLoader::resourceTypeFromAsAttribute(m_asAttribute, m_document);
             break;
         case TagId::Meta:
         case TagId::Unknown:
@@ -365,7 +374,7 @@ private:
         if (m_urlToLoad.isEmpty())
             return false;
 
-        if (protocolIs(m_urlToLoad, "data") || protocolIs(m_urlToLoad, "about"))
+        if (protocolIs(m_urlToLoad, "data"_s) || protocolIs(m_urlToLoad, "about"_s))
             return false;
 
         if (m_tagId == TagId::Link && !m_linkIsStyleSheet && !m_linkIsPreload)
@@ -374,12 +383,13 @@ private:
         if (m_tagId == TagId::Input && !m_inputIsImage)
             return false;
 
-        if (m_tagId == TagId::Script && m_moduleScript == PreloadRequest::ModuleScript::No && m_scriptIsNomodule)
+        if (m_tagId == TagId::Script && m_scriptType != ScriptType::Module && m_scriptIsNomodule)
             return false;
 
         return true;
     }
 
+    Document& m_document;
     TagId m_tagId;
     String m_urlToLoad;
     String m_srcSetAttribute;
@@ -402,8 +412,9 @@ private:
     bool m_scriptIsNomodule { false };
     bool m_scriptIsAsync { false };
     float m_deviceScaleFactor;
-    PreloadRequest::ModuleScript m_moduleScript { PreloadRequest::ModuleScript::No };
+    ScriptType m_scriptType { ScriptType::Classic };
     ReferrerPolicy m_referrerPolicy { ReferrerPolicy::EmptyString };
+    RequestPriority m_fetchPriorityHint { RequestPriority::Auto };
 };
 
 TokenPreloadScanner::TokenPreloadScanner(const URL& documentURL, float deviceScaleFactor)
@@ -415,13 +426,13 @@ TokenPreloadScanner::TokenPreloadScanner(const URL& documentURL, float deviceSca
 void TokenPreloadScanner::scan(const HTMLToken& token, Vector<std::unique_ptr<PreloadRequest>>& requests, Document& document)
 {
     switch (token.type()) {
-    case HTMLToken::Character:
+    case HTMLToken::Type::Character:
         if (!m_inStyle)
             return;
         m_cssScanner.scan(token.characters(), requests);
         return;
 
-    case HTMLToken::EndTag: {
+    case HTMLToken::Type::EndTag: {
         TagId tagId = tagIdFor(token.name());
         if (tagId == TagId::Template) {
             if (m_templateCount)
@@ -438,14 +449,14 @@ void TokenPreloadScanner::scan(const HTMLToken& token, Vector<std::unique_ptr<Pr
         return;
     }
 
-    case HTMLToken::StartTag: {
-        if (m_templateCount)
-            return;
+    case HTMLToken::Type::StartTag: {
         TagId tagId = tagIdFor(token.name());
         if (tagId == TagId::Template) {
             ++m_templateCount;
             return;
         }
+        if (m_templateCount)
+            return;
         if (tagId == TagId::Style) {
             m_inStyle = true;
             return;
@@ -462,8 +473,8 @@ void TokenPreloadScanner::scan(const HTMLToken& token, Vector<std::unique_ptr<Pr
             return;
         }
 
-        StartTagScanner scanner(tagId, m_deviceScaleFactor);
-        scanner.processAttributes(token.attributes(), document, m_pictureSourceState);
+        StartTagScanner scanner(document, tagId, m_deviceScaleFactor);
+        scanner.processAttributes(token.attributes(), m_pictureSourceState);
         if (auto request = scanner.createPreloadRequest(m_predictedBaseElementURL))
             requests.append(WTFMove(request));
         return;
@@ -477,12 +488,13 @@ void TokenPreloadScanner::scan(const HTMLToken& token, Vector<std::unique_ptr<Pr
 void TokenPreloadScanner::updatePredictedBaseURL(const HTMLToken& token, bool shouldRestrictBaseURLSchemes)
 {
     ASSERT(m_predictedBaseElementURL.isEmpty());
-    auto* hrefAttribute = findAttribute(token.attributes(), hrefAttr->localName().string());
+    static constexpr UChar hrefAsUChar[] = { 'h', 'r', 'e', 'f' };
+    auto* hrefAttribute = findAttribute(token.attributes(), hrefAsUChar);
     if (!hrefAttribute)
         return;
-    URL temp { m_documentURL, stripLeadingAndTrailingHTMLSpaces(StringImpl::create8BitIfPossible(hrefAttribute->value)) };
+    URL temp { m_documentURL, StringImpl::create8BitIfPossible(hrefAttribute->value) };
     if (!shouldRestrictBaseURLSchemes || SecurityPolicy::isBaseURLSchemeAllowed(temp))
-        m_predictedBaseElementURL = temp.isolatedCopy();
+        m_predictedBaseElementURL = WTFMove(temp).isolatedCopy();
 }
 
 HTMLPreloadScanner::HTMLPreloadScanner(const HTMLParserOptions& options, const URL& documentURL, float deviceScaleFactor)
@@ -509,8 +521,8 @@ void HTMLPreloadScanner::scan(HTMLResourcePreloader& preloader, Document& docume
     PreloadRequestStream requests;
 
     while (auto token = m_tokenizer.nextToken(m_source)) {
-        if (token->type() == HTMLToken::StartTag)
-            m_tokenizer.updateStateFor(AtomString(token->name()));
+        if (token->type() == HTMLToken::Type::StartTag)
+            m_tokenizer.updateStateFor(AtomString::lookUp(token->name().data(), token->name().size()));
         m_scanner.scan(*token, requests, document);
     }
 
@@ -523,7 +535,7 @@ bool testPreloadScannerViewportSupport(Document* document)
     HTMLParserOptions options(*document);
     HTMLPreloadScanner scanner(options, document->url());
     HTMLResourcePreloader preloader(*document);
-    scanner.appendToEnd(String("<meta name=viewport content='width=400'>"));
+    scanner.appendToEnd(String("<meta name=viewport content='width=400'>"_s));
     scanner.scan(preloader, *document);
     return (document->viewportArguments().width == 400);
 }
