@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2010 Google Inc. All rights reserved.
- * Copyright (C) 2015 Apple Inc. All rights reserved.
+ * Copyright (C) 2015-2020 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are
@@ -35,18 +35,19 @@
 #include "Chrome.h"
 #include "DOMWrapperWorld.h"
 #include "Document.h"
+#include "ExceptionDetails.h"
 #include "FloatRect.h"
-#include "Frame.h"
 #include "FrameLoadRequest.h"
 #include "FrameLoader.h"
-#include "FrameView.h"
 #include "InspectorController.h"
 #include "InspectorFrontendHost.h"
 #include "InspectorPageAgent.h"
+#include "LocalFrame.h"
+#include "LocalFrameView.h"
+#include "Logging.h"
 #include "Page.h"
 #include "ScriptController.h"
 #include "ScriptSourceCode.h"
-#include "ScriptState.h"
 #include "Settings.h"
 #include "Timer.h"
 #include "UserGestureIndicator.h"
@@ -54,14 +55,15 @@
 #include <JavaScriptCore/FrameTracers.h>
 #include <JavaScriptCore/InspectorBackendDispatchers.h>
 #include <wtf/Deque.h>
+#include <wtf/RunLoop.h>
 #include <wtf/text/CString.h>
-
+#include <wtf/text/StringToIntegerConversion.h>
 
 namespace WebCore {
 
 using namespace Inspector;
 
-static const char* inspectorAttachedHeightSetting = "inspectorAttachedHeight";
+static constexpr ASCIILiteral inspectorAttachedHeightSetting = "inspectorAttachedHeight"_s;
 static const unsigned defaultAttachedHeight = 300;
 static const float minimumAttachedHeight = 250.0f;
 static const float maximumAttachedHeightRatio = 0.75f;
@@ -105,7 +107,7 @@ private:
 
         // The frontend can be closed and destroy the owning frontend client before or in the
         // process of dispatching the task, so keep a protector reference here.
-        RunLoop::current().dispatch([this, protectedThis = makeRef(*this)] {
+        RunLoop::current().dispatch([this, protectedThis = Ref { *this }] {
             m_hasScheduledTask = false;
             dispatchOneMessage();
         });
@@ -150,6 +152,7 @@ InspectorFrontendClientLocal::InspectorFrontendClientLocal(InspectorController*
     , m_settings(WTFMove(settings))
     , m_dockSide(DockSide::Undocked)
     , m_dispatchTask(InspectorBackendDispatchTask::create(inspectedPageController))
+    , m_frontendAPIDispatcher(InspectorFrontendAPIDispatcher::create(*frontendPage))
 {
     m_frontendPage->settings().setAllowFileAccessFromFileURLs(true);
 }
@@ -185,10 +188,20 @@ void InspectorFrontendClientLocal::frontendLoaded()
     // Thus if we call canAttachWindow first we can avoid this problem. This change does not cause any regressions on Mac.
     setDockingUnavailable(!canAttachWindow());
     bringToFront();
-    m_frontendLoaded = true;
-    for (auto& evaluate : m_evaluateOnLoad)
-        evaluateOnLoad(evaluate);
-    m_evaluateOnLoad.clear();
+
+    m_frontendAPIDispatcher->frontendLoaded();
+}
+
+void InspectorFrontendClientLocal::pagePaused()
+{
+    // NOTE: pagePaused() and pageUnpaused() do not suspend/unsuspend the frontend API dispatcher
+    // for this subclass of InspectorFrontendClient. The inspected page and the frontend page
+    // exist in the same web process, so messages need to be sent even while the debugger is paused.
+    // Suspending here would stall out later commands that resume the debugger, causing the test to time out.
+}
+
+void InspectorFrontendClientLocal::pageUnpaused()
+{
 }
 
 UserInterfaceLayoutDirection InspectorFrontendClientLocal::userInterfaceLayoutDirection() const
@@ -219,20 +232,25 @@ bool InspectorFrontendClientLocal::canAttachWindow()
         return true;
 
     // Don't allow the attach if the window would be too small to accommodate the minimum inspector size.
-    unsigned inspectedPageHeight = m_inspectedPageController->inspectedPage().mainFrame().view()->visibleHeight();
-    unsigned inspectedPageWidth = m_inspectedPageController->inspectedPage().mainFrame().view()->visibleWidth();
+    auto* localMainFrame = dynamicDowncast<LocalFrame>(m_inspectedPageController->inspectedPage().mainFrame());
+    unsigned inspectedPageHeight = localMainFrame ? localMainFrame->view()->visibleHeight() : 0;
+    unsigned inspectedPageWidth = localMainFrame ? localMainFrame->view()->visibleWidth() : 0;
     unsigned maximumAttachedHeight = inspectedPageHeight * maximumAttachedHeightRatio;
     return minimumAttachedHeight <= maximumAttachedHeight && minimumAttachedWidth <= inspectedPageWidth;
 }
 
 void InspectorFrontendClientLocal::setDockingUnavailable(bool unavailable)
 {
-    dispatch(makeString("[\"setDockingUnavailable\", ", unavailable ? "true" : "false", ']'));
+    m_frontendAPIDispatcher->dispatchCommandWithResultAsync("setDockingUnavailable"_s, { JSON::Value::create(unavailable) });
 }
 
 void InspectorFrontendClientLocal::changeAttachedWindowHeight(unsigned height)
 {
-    unsigned totalHeight = m_frontendPage->mainFrame().view()->visibleHeight() + m_inspectedPageController->inspectedPage().mainFrame().view()->visibleHeight();
+    auto* localMainFrame = dynamicDowncast<LocalFrame>(m_frontendPage->mainFrame());
+    if (!localMainFrame)
+        return;
+    auto* otherMainFrame = dynamicDowncast<LocalFrame>(m_inspectedPageController->inspectedPage().mainFrame());
+    unsigned totalHeight = localMainFrame->view()->visibleHeight() + (otherMainFrame ? otherMainFrame->view()->visibleHeight() : 0);
     unsigned attachedHeight = constrainedAttachedWindowHeight(height, totalHeight);
     m_settings->setProperty(inspectorAttachedHeightSetting, String::number(attachedHeight));
     setAttachedWindowHeight(attachedHeight);
@@ -240,7 +258,11 @@ void InspectorFrontendClientLocal::changeAttachedWindowHeight(unsigned height)
 
 void InspectorFrontendClientLocal::changeAttachedWindowWidth(unsigned width)
 {
-    unsigned totalWidth = m_frontendPage->mainFrame().view()->visibleWidth() + m_inspectedPageController->inspectedPage().mainFrame().view()->visibleWidth();
+    auto* localMainFrame = dynamicDowncast<LocalFrame>(m_frontendPage->mainFrame());
+    if (!localMainFrame)
+        return;
+    auto* otherMainFrame = dynamicDowncast<LocalFrame>(m_inspectedPageController->inspectedPage().mainFrame());
+    unsigned totalWidth = localMainFrame->view()->visibleWidth() + (otherMainFrame ? localMainFrame->view()->visibleWidth() : 0);
     unsigned attachedWidth = constrainedAttachedWindowWidth(width, totalWidth);
     setAttachedWindowWidth(attachedWidth);
 }
@@ -250,14 +272,20 @@ void InspectorFrontendClientLocal::changeSheetRect(const FloatRect& rect)
     setSheetRect(rect);
 }
 
-void InspectorFrontendClientLocal::openInNewTab(const String& url)
+void InspectorFrontendClientLocal::openURLExternally(const String& url)
 {
-    UserGestureIndicator indicator { ProcessingUserGesture };
-    Frame& mainFrame = m_inspectedPageController->inspectedPage().mainFrame();
-    FrameLoadRequest frameLoadRequest { *mainFrame.document(), mainFrame.document()->securityOrigin(), { }, "_blank"_s, LockHistory::No, LockBackForwardList::No, MaybeSendReferrer, AllowNavigationToInvalidURL::Yes, NewFrameOpenerPolicy::Allow, ShouldOpenExternalURLsPolicy::ShouldNotAllow, InitiatedByMainFrame::Unknown };
+    auto* localMainFrame = dynamicDowncast<LocalFrame>(m_inspectedPageController->inspectedPage().mainFrame());
+    if (!localMainFrame)
+        return;
+    LocalFrame& mainFrame = *localMainFrame;
+
+    UserGestureIndicator indicator { ProcessingUserGesture, mainFrame.document() };
+
+    FrameLoadRequest frameLoadRequest { *mainFrame.document(), mainFrame.document()->securityOrigin(), { }, blankTargetFrameName(), InitiatedByMainFrame::Unknown };
 
     bool created;
-    auto frame = WebCore::createWindow(mainFrame, mainFrame, WTFMove(frameLoadRequest), { }, created);
+    WindowFeatures features;
+    auto frame = WebCore::createWindow(mainFrame, mainFrame, WTFMove(frameLoadRequest), features, created);
     if (!frame)
         return;
 
@@ -266,7 +294,7 @@ void InspectorFrontendClientLocal::openInNewTab(const String& url)
 
     // FIXME: Why do we compute the absolute URL with respect to |frame| instead of |mainFrame|?
     ResourceRequest resourceRequest { frame->document()->completeURL(url) };
-    FrameLoadRequest frameLoadRequest2 { *mainFrame.document(), mainFrame.document()->securityOrigin(), resourceRequest, "_self"_s, LockHistory::No, LockBackForwardList::No, MaybeSendReferrer, AllowNavigationToInvalidURL::Yes, NewFrameOpenerPolicy::Allow, ShouldOpenExternalURLsPolicy::ShouldNotAllow, InitiatedByMainFrame::Unknown };
+    FrameLoadRequest frameLoadRequest2 { *mainFrame.document(), mainFrame.document()->securityOrigin(), WTFMove(resourceRequest), selfTargetFrameName(), InitiatedByMainFrame::Unknown };
     frame->loader().changeLocation(WTFMove(frameLoadRequest2));
 }
 
@@ -297,76 +325,85 @@ void InspectorFrontendClientLocal::setAttachedWindow(DockSide dockSide)
 
     m_dockSide = dockSide;
 
-    dispatch(makeString("[\"setDockSide\", \"", side, "\"]"));
+    m_frontendAPIDispatcher->dispatchCommandWithResultAsync("setDockSide"_s, { JSON::Value::create(makeString(side)) });
 }
 
 void InspectorFrontendClientLocal::restoreAttachedWindowHeight()
 {
-    unsigned inspectedPageHeight = m_inspectedPageController->inspectedPage().mainFrame().view()->visibleHeight();
+    auto* localMainFrame = dynamicDowncast<LocalFrame>(m_inspectedPageController->inspectedPage().mainFrame());
+    unsigned inspectedPageHeight = localMainFrame ? localMainFrame->view()->visibleHeight() : 0;
     String value = m_settings->getProperty(inspectorAttachedHeightSetting);
-    unsigned preferredHeight = value.isEmpty() ? defaultAttachedHeight : value.toUInt();
-    
+    unsigned preferredHeight = value.isEmpty() ? defaultAttachedHeight : parseIntegerAllowingTrailingJunk<unsigned>(value).value_or(0);
+
     // This call might not go through (if the window starts out detached), but if the window is initially created attached,
     // InspectorController::attachWindow is never called, so we need to make sure to set the attachedWindowHeight.
     // FIXME: Clean up code so we only have to call setAttachedWindowHeight in InspectorController::attachWindow
     setAttachedWindowHeight(constrainedAttachedWindowHeight(preferredHeight, inspectedPageHeight));
 }
 
+std::optional<bool> InspectorFrontendClientLocal::evaluationResultToBoolean(InspectorFrontendAPIDispatcher::EvaluationResult result)
+{
+    if (!result)
+        return std::nullopt;
+
+    auto valueOrException = result.value();
+    if (!valueOrException) {
+        LOG(Inspector, "Encountered exception while evaluating upon the frontend: %s", valueOrException.error().message.utf8().data());
+        return std::nullopt;
+    }
+
+    return valueOrException.value().toBoolean(m_frontendAPIDispatcher->frontendGlobalObject());
+}
+
 bool InspectorFrontendClientLocal::isDebuggingEnabled()
 {
-    if (m_frontendLoaded)
-        return evaluateAsBoolean("[\"isDebuggingEnabled\"]");
-    return false;
+    return evaluationResultToBoolean(m_frontendAPIDispatcher->dispatchCommandWithResultSync("isDebuggingEnabled"_s)).value_or(false);
 }
 
 void InspectorFrontendClientLocal::setDebuggingEnabled(bool enabled)
 {
-    dispatch(makeString("[\"setDebuggingEnabled\", ", enabled ? "true" : "false", ']'));
+    m_frontendAPIDispatcher->dispatchCommandWithResultAsync("setDebuggingEnabled"_s, { JSON::Value::create(enabled) });
 }
 
 bool InspectorFrontendClientLocal::isTimelineProfilingEnabled()
 {
-    if (m_frontendLoaded)
-        return evaluateAsBoolean("[\"isTimelineProfilingEnabled\"]");
-    return false;
+    return evaluationResultToBoolean(m_frontendAPIDispatcher->dispatchCommandWithResultSync("isTimelineProfilingEnabled"_s)).value_or(false);
 }
 
 void InspectorFrontendClientLocal::setTimelineProfilingEnabled(bool enabled)
 {
-    dispatch(makeString("[\"setTimelineProfilingEnabled\", ", enabled ? "true" : "false", ']'));
+    m_frontendAPIDispatcher->dispatchCommandWithResultAsync("setTimelineProfilingEnabled"_s, { JSON::Value::create(enabled) });
 }
 
 bool InspectorFrontendClientLocal::isProfilingJavaScript()
 {
-    if (m_frontendLoaded)
-        return evaluateAsBoolean("[\"isProfilingJavaScript\"]");
-    return false;
+    return evaluationResultToBoolean(m_frontendAPIDispatcher->dispatchCommandWithResultSync("isProfilingJavaScript"_s)).value_or(false);
 }
 
 void InspectorFrontendClientLocal::startProfilingJavaScript()
 {
-    dispatch("[\"startProfilingJavaScript\"]");
+    m_frontendAPIDispatcher->dispatchCommandWithResultAsync("startProfilingJavaScript"_s);
 }
 
 void InspectorFrontendClientLocal::stopProfilingJavaScript()
 {
-    dispatch("[\"stopProfilingJavaScript\"]");
+    m_frontendAPIDispatcher->dispatchCommandWithResultAsync("stopProfilingJavaScript"_s);
 }
 
 void InspectorFrontendClientLocal::showConsole()
 {
-    dispatch("[\"showConsole\"]");
+    m_frontendAPIDispatcher->dispatchCommandWithResultAsync("showConsole"_s);
 }
 
 void InspectorFrontendClientLocal::showResources()
 {
-    dispatch("[\"showResources\"]");
+    m_frontendAPIDispatcher->dispatchCommandWithResultAsync("showResources"_s);
 }
 
-void InspectorFrontendClientLocal::showMainResourceForFrame(Frame* frame)
+void InspectorFrontendClientLocal::showMainResourceForFrame(LocalFrame* frame)
 {
     String frameId = m_inspectedPageController->ensurePageAgent().frameId(frame);
-    dispatch(makeString("[\"showMainResourceForFrame\", \"", frameId, "\"]"));
+    m_frontendAPIDispatcher->dispatchCommandWithResultAsync("showMainResourceForFrame"_s, { JSON::Value::create(frameId) });
 }
 
 unsigned InspectorFrontendClientLocal::constrainedAttachedWindowHeight(unsigned preferredHeight, unsigned totalWindowHeight)
@@ -394,46 +431,6 @@ unsigned InspectorFrontendClientLocal::inspectionLevel() const
     return m_inspectedPageController->inspectionLevel() + 1;
 }
 
-void InspectorFrontendClientLocal::dispatch(const String& signature)
-{
-    ASSERT(!signature.isEmpty());
-    ASSERT(signature.startsWith('['));
-    ASSERT(signature.endsWith(']'));
-
-    evaluateOnLoad("InspectorFrontendAPI.dispatch(" + signature + ")");
-}
-
-void InspectorFrontendClientLocal::dispatchMessage(const String& messageObject)
-{
-    ASSERT(!messageObject.isEmpty());
-
-    evaluateOnLoad("InspectorFrontendAPI.dispatchMessage(" + messageObject + ")");
-}
-
-void InspectorFrontendClientLocal::dispatchMessageAsync(const String& messageObject)
-{
-    ASSERT(!messageObject.isEmpty());
-
-    evaluateOnLoad("InspectorFrontendAPI.dispatchMessageAsync(" + messageObject + ")");
-}
-
-bool InspectorFrontendClientLocal::evaluateAsBoolean(const String& expression)
-{
-    auto& state = *mainWorldExecState(&m_frontendPage->mainFrame());
-    return m_frontendPage->mainFrame().script().executeScript(expression).toWTFString(&state) == "true";
-}
-
-void InspectorFrontendClientLocal::evaluateOnLoad(const String& expression)
-{
-    if (!m_frontendLoaded) {
-        m_evaluateOnLoad.append(expression);
-        return;
-    }
-
-    JSC::SuspendExceptionScope scope(&m_frontendPage->inspectorController().vm());
-    m_frontendPage->mainFrame().script().evaluate(ScriptSourceCode(expression));
-}
-
 Page* InspectorFrontendClientLocal::inspectedPage() const
 {
     if (!m_inspectedPageController)
