@@ -32,17 +32,13 @@
 #include "config.h"
 #include "InspectorIndexedDBAgent.h"
 
-#if ENABLE(INDEXED_DATABASE)
-
+#include "AddEventListenerOptions.h"
 #include "DOMStringList.h"
-#include "DOMWindow.h"
-#include "DOMWindowIndexedDatabase.h"
 #include "Document.h"
 #include "Event.h"
 #include "EventListener.h"
 #include "EventNames.h"
 #include "EventTarget.h"
-#include "Frame.h"
 #include "IDBBindingUtilities.h"
 #include "IDBCursor.h"
 #include "IDBCursorWithValue.h"
@@ -58,8 +54,11 @@
 #include "IDBTransaction.h"
 #include "InspectorPageAgent.h"
 #include "InstrumentingAgents.h"
-#include "ScriptState.h"
+#include "JSLocalDOMWindowCustom.h"
+#include "LocalDOMWindow.h"
+#include "LocalFrame.h"
 #include "SecurityOrigin.h"
+#include "WindowOrWorkerGlobalScopeIndexedDatabase.h"
 #include <JavaScriptCore/HeapInlines.h>
 #include <JavaScriptCore/InjectedScript.h>
 #include <JavaScriptCore/InjectedScriptManager.h>
@@ -70,26 +69,6 @@
 #include <wtf/Vector.h>
 #include <wtf/text/StringConcatenateNumbers.h>
 
-using JSON::ArrayOf;
-using Inspector::Protocol::IndexedDB::DatabaseWithObjectStores;
-using Inspector::Protocol::IndexedDB::DataEntry;
-using Inspector::Protocol::IndexedDB::Key;
-using Inspector::Protocol::IndexedDB::KeyPath;
-using Inspector::Protocol::IndexedDB::KeyRange;
-using Inspector::Protocol::IndexedDB::ObjectStore;
-using Inspector::Protocol::IndexedDB::ObjectStoreIndex;
-
-typedef Inspector::BackendDispatcher::CallbackBase RequestCallback;
-typedef Inspector::IndexedDBBackendDispatcherHandler::RequestDatabaseNamesCallback RequestDatabaseNamesCallback;
-typedef Inspector::IndexedDBBackendDispatcherHandler::RequestDatabaseCallback RequestDatabaseCallback;
-typedef Inspector::IndexedDBBackendDispatcherHandler::RequestDataCallback RequestDataCallback;
-typedef Inspector::IndexedDBBackendDispatcherHandler::ClearObjectStoreCallback ClearObjectStoreCallback;
-
-typedef String ErrorString;
-
-template <typename T>
-using ErrorStringOr = Expected<T, ErrorString>;
-
 namespace WebCore {
 
 using namespace Inspector;
@@ -103,7 +82,7 @@ public:
     virtual ~ExecutableWithDatabase() = default;
     void start(IDBFactory*, SecurityOrigin*, const String& databaseName);
     virtual void execute(IDBDatabase&) = 0;
-    virtual RequestCallback& requestCallback() = 0;
+    virtual BackendDispatcher::CallbackBase& requestCallback() = 0;
     ScriptExecutionContext* context() const { return m_context; }
 private:
     ScriptExecutionContext* m_context;
@@ -124,7 +103,7 @@ public:
     void handleEvent(ScriptExecutionContext&, Event& event) final
     {
         if (event.type() != eventNames().successEvent) {
-            m_executableWithDatabase->requestCallback().sendFailure("Unexpected event type.");
+            m_executableWithDatabase->requestCallback().sendFailure("Unexpected event type."_s);
             return;
         }
 
@@ -132,17 +111,17 @@ public:
 
         auto result = request.result();
         if (result.hasException()) {
-            m_executableWithDatabase->requestCallback().sendFailure("Could not get result in callback.");
+            m_executableWithDatabase->requestCallback().sendFailure("Could not get result in callback."_s);
             return;
         }
 
         auto resultValue = result.releaseReturnValue();
-        if (!WTF::holds_alternative<RefPtr<IDBDatabase>>(resultValue)) {
-            m_executableWithDatabase->requestCallback().sendFailure("Unexpected result type.");
+        if (!std::holds_alternative<RefPtr<IDBDatabase>>(resultValue)) {
+            m_executableWithDatabase->requestCallback().sendFailure("Unexpected result type."_s);
             return;
         }
 
-        auto databaseResult = WTF::get<RefPtr<IDBDatabase>>(resultValue);
+        auto databaseResult = std::get<RefPtr<IDBDatabase>>(resultValue);
         m_executableWithDatabase->execute(*databaseResult);
         databaseResult->close();
     }
@@ -157,13 +136,13 @@ private:
 void ExecutableWithDatabase::start(IDBFactory* idbFactory, SecurityOrigin*, const String& databaseName)
 {
     if (!context()) {
-        requestCallback().sendFailure("Could not open database.");
+        requestCallback().sendFailure("Could not open database."_s);
         return;
     }
 
-    auto result = idbFactory->open(*context(), databaseName, WTF::nullopt);
+    auto result = idbFactory->open(*context(), databaseName, std::nullopt);
     if (result.hasException()) {
-        requestCallback().sendFailure("Could not open database.");
+        requestCallback().sendFailure("Could not open database."_s);
         return;
     }
 
@@ -171,24 +150,24 @@ void ExecutableWithDatabase::start(IDBFactory* idbFactory, SecurityOrigin*, cons
 }
 
 
-static RefPtr<KeyPath> keyPathFromIDBKeyPath(const Optional<IDBKeyPath>& idbKeyPath)
+static Ref<Protocol::IndexedDB::KeyPath> keyPathFromIDBKeyPath(const std::optional<IDBKeyPath>& idbKeyPath)
 {
     if (!idbKeyPath)
-        return KeyPath::create().setType(KeyPath::Type::Null).release();
+        return Protocol::IndexedDB::KeyPath::create().setType(Protocol::IndexedDB::KeyPath::Type::Null).release();
 
     auto visitor = WTF::makeVisitor([](const String& string) {
-        auto keyPath = KeyPath::create().setType(KeyPath::Type::String).release();
+        auto keyPath = Protocol::IndexedDB::KeyPath::create().setType(Protocol::IndexedDB::KeyPath::Type::String).release();
         keyPath->setString(string);
         return keyPath;
     }, [](const Vector<String>& vector) {
         auto array = JSON::ArrayOf<String>::create();
         for (auto& string : vector)
             array->addItem(string);
-        auto keyPath = KeyPath::create().setType(KeyPath::Type::Array).release();
+        auto keyPath = Protocol::IndexedDB::KeyPath::create().setType(Protocol::IndexedDB::KeyPath::Type::Array).release();
         keyPath->setArray(WTFMove(array));
         return keyPath;
     });
-    return WTF::visit(visitor, idbKeyPath.value());
+    return std::visit(visitor, idbKeyPath.value());
 }
 
 static RefPtr<IDBTransaction> transactionForDatabase(IDBDatabase* idbDatabase, const String& objectStoreName, IDBTransactionMode mode = IDBTransactionMode::Readonly)
@@ -217,7 +196,7 @@ static RefPtr<IDBIndex> indexForObjectStore(IDBObjectStore* idbObjectStore, cons
 
 class DatabaseLoader final : public ExecutableWithDatabase {
 public:
-    static Ref<DatabaseLoader> create(ScriptExecutionContext* context, Ref<RequestDatabaseCallback>&& requestCallback)
+    static Ref<DatabaseLoader> create(ScriptExecutionContext* context, Ref<IndexedDBBackendDispatcherHandler::RequestDatabaseCallback>&& requestCallback)
     {
         return adoptRef(*new DatabaseLoader(context, WTFMove(requestCallback)));
     }
@@ -230,17 +209,17 @@ public:
             return;
     
         auto& databaseInfo = database.info();
-        auto objectStores = JSON::ArrayOf<Inspector::Protocol::IndexedDB::ObjectStore>::create();
+        auto objectStores = JSON::ArrayOf<Protocol::IndexedDB::ObjectStore>::create();
         auto objectStoreNames = databaseInfo.objectStoreNames();
         for (auto& name : objectStoreNames) {
             auto* objectStoreInfo = databaseInfo.infoForExistingObjectStore(name);
             if (!objectStoreInfo)
                 continue;
 
-            auto indexes = JSON::ArrayOf<Inspector::Protocol::IndexedDB::ObjectStoreIndex>::create();
+            auto indexes = JSON::ArrayOf<Protocol::IndexedDB::ObjectStoreIndex>::create();
     
             for (auto& indexInfo : objectStoreInfo->indexMap().values()) {
-                auto objectStoreIndex = ObjectStoreIndex::create()
+                auto objectStoreIndex = Protocol::IndexedDB::ObjectStoreIndex::create()
                     .setName(indexInfo.name())
                     .setKeyPath(keyPathFromIDBKeyPath(indexInfo.keyPath()))
                     .setUnique(indexInfo.unique())
@@ -249,7 +228,7 @@ public:
                 indexes->addItem(WTFMove(objectStoreIndex));
             }
     
-            auto objectStore = ObjectStore::create()
+            auto objectStore = Protocol::IndexedDB::ObjectStore::create()
                 .setName(objectStoreInfo->name())
                 .setKeyPath(keyPathFromIDBKeyPath(objectStoreInfo->keyPath()))
                 .setAutoIncrement(objectStoreInfo->autoIncrement())
@@ -258,7 +237,7 @@ public:
             objectStores->addItem(WTFMove(objectStore));
         }
     
-        auto result = DatabaseWithObjectStores::create()
+        auto result = Protocol::IndexedDB::DatabaseWithObjectStores::create()
             .setName(databaseInfo.name())
             .setVersion(databaseInfo.version())
             .setObjectStores(WTFMove(objectStores))
@@ -266,92 +245,96 @@ public:
         m_requestCallback->sendSuccess(WTFMove(result));
     }
 
-    RequestCallback& requestCallback() override { return m_requestCallback.get(); }
+    BackendDispatcher::CallbackBase& requestCallback() override { return m_requestCallback.get(); }
 private:
-    DatabaseLoader(ScriptExecutionContext* context, Ref<RequestDatabaseCallback>&& requestCallback)
+    DatabaseLoader(ScriptExecutionContext* context, Ref<IndexedDBBackendDispatcherHandler::RequestDatabaseCallback>&& requestCallback)
         : ExecutableWithDatabase(context)
         , m_requestCallback(WTFMove(requestCallback)) { }
-    Ref<RequestDatabaseCallback> m_requestCallback;
+    Ref<IndexedDBBackendDispatcherHandler::RequestDatabaseCallback> m_requestCallback;
 };
 
-static RefPtr<IDBKey> idbKeyFromInspectorObject(JSON::Object* key)
+static RefPtr<IDBKey> idbKeyFromInspectorObject(Ref<JSON::Object>&& key)
 {
-    String type;
-    if (!key->getString("type", type))
+    auto typeString = key->getString(Protocol::IndexedDB::Key::typeKey);
+    if (!typeString)
         return nullptr;
 
-    static NeverDestroyed<const String> numberType(MAKE_STATIC_STRING_IMPL("number"));
-    static NeverDestroyed<const String> stringType(MAKE_STATIC_STRING_IMPL("string"));
-    static NeverDestroyed<const String> dateType(MAKE_STATIC_STRING_IMPL("date"));
-    static NeverDestroyed<const String> arrayType(MAKE_STATIC_STRING_IMPL("array"));
+    auto type = Protocol::Helpers::parseEnumValueFromString<Protocol::IndexedDB::Key::Type>(typeString);
+    if (!type)
+        return nullptr;
 
-    RefPtr<IDBKey> idbKey;
-    if (type == numberType) {
-        double number;
-        if (!key->getDouble("number", number))
+    switch (*type) {
+    case Protocol::IndexedDB::Key::Type::Number: {
+        auto number = key->getDouble(Protocol::IndexedDB::Key::numberKey);
+        if (!number)
             return nullptr;
-        idbKey = IDBKey::createNumber(number);
-    } else if (type == stringType) {
-        String string;
-        if (!key->getString("string", string))
+        return IDBKey::createNumber(*number);
+    }
+
+    case Protocol::IndexedDB::Key::Type::String: {
+        auto string = key->getString(Protocol::IndexedDB::Key::stringKey);
+        if (!string)
             return nullptr;
-        idbKey = IDBKey::createString(string);
-    } else if (type == dateType) {
-        double date;
-        if (!key->getDouble("date", date))
+        return IDBKey::createString(string);
+    }
+
+    case Protocol::IndexedDB::Key::Type::Date: {
+        auto date = key->getDouble(Protocol::IndexedDB::Key::dateKey);
+        if (!date)
             return nullptr;
-        idbKey = IDBKey::createDate(date);
-    } else if (type == arrayType) {
+        return IDBKey::createDate(*date);
+    }
+
+    case Protocol::IndexedDB::Key::Type::Array: {
+        auto array = key->getArray(Protocol::IndexedDB::Key::arrayKey);
+        if (!array)
+            return nullptr;
+
         Vector<RefPtr<IDBKey>> keyArray;
-        RefPtr<JSON::Array> array;
-        if (!key->getArray("array", array))
-            return nullptr;
         for (size_t i = 0; i < array->length(); ++i) {
-            RefPtr<JSON::Value> value = array->get(i);
-            RefPtr<JSON::Object> object;
-            if (!value->asObject(object))
+            auto object = array->get(i)->asObject();
+            if (!object)
                 return nullptr;
-            keyArray.append(idbKeyFromInspectorObject(object.get()));
+            keyArray.append(idbKeyFromInspectorObject(object.releaseNonNull()));
         }
-        idbKey = IDBKey::createArray(keyArray);
-    } else
-        return nullptr;
+        return IDBKey::createArray(keyArray);
+    }
+    }
 
-    return idbKey;
+    ASSERT_NOT_REACHED();
+    return nullptr;
 }
 
-static RefPtr<IDBKeyRange> idbKeyRangeFromKeyRange(const JSON::Object* keyRange)
+static RefPtr<IDBKeyRange> idbKeyRangeFromKeyRange(JSON::Object& keyRange)
 {
     RefPtr<IDBKey> idbLower;
-    RefPtr<JSON::Object> lower;
-    if (keyRange->getObject("lower"_s, lower)) {
-        idbLower = idbKeyFromInspectorObject(lower.get());
+    if (auto lower = keyRange.getObject(Protocol::IndexedDB::KeyRange::lowerKey)) {
+        idbLower = idbKeyFromInspectorObject(lower.releaseNonNull());
         if (!idbLower)
             return nullptr;
     }
 
     RefPtr<IDBKey> idbUpper;
-    RefPtr<JSON::Object> upper;
-    if (keyRange->getObject("upper"_s, upper)) {
-        idbUpper = idbKeyFromInspectorObject(upper.get());
+    if (auto upper = keyRange.getObject(Protocol::IndexedDB::KeyRange::upperKey)) {
+        idbUpper = idbKeyFromInspectorObject(upper.releaseNonNull());
         if (!idbUpper)
             return nullptr;
     }
 
-    bool lowerOpen;
-    if (!keyRange->getBoolean("lowerOpen"_s, lowerOpen))
+    auto lowerOpen = keyRange.getBoolean(Protocol::IndexedDB::KeyRange::lowerOpenKey);
+    if (!lowerOpen)
         return nullptr;
 
-    bool upperOpen;
-    if (!keyRange->getBoolean("upperOpen"_s, upperOpen))
+    auto upperOpen = keyRange.getBoolean(Protocol::IndexedDB::KeyRange::upperOpenKey);
+    if (!upperOpen)
         return nullptr;
 
-    return IDBKeyRange::create(WTFMove(idbLower), WTFMove(idbUpper), lowerOpen, upperOpen);
+    return IDBKeyRange::create(WTFMove(idbLower), WTFMove(idbUpper), *lowerOpen, *upperOpen);
 }
 
 class OpenCursorCallback final : public EventListener {
 public:
-    static Ref<OpenCursorCallback> create(InjectedScript injectedScript, Ref<RequestDataCallback>&& requestCallback, int skipCount, unsigned pageSize)
+    static Ref<OpenCursorCallback> create(InjectedScript injectedScript, Ref<IndexedDBBackendDispatcherHandler::RequestDataCallback>&& requestCallback, int skipCount, unsigned pageSize)
     {
         return adoptRef(*new OpenCursorCallback(injectedScript, WTFMove(requestCallback), skipCount, pageSize));
     }
@@ -366,7 +349,7 @@ public:
     void handleEvent(ScriptExecutionContext& context, Event& event) override
     {
         if (event.type() != eventNames().successEvent) {
-            m_requestCallback->sendFailure("Unexpected event type.");
+            m_requestCallback->sendFailure("Unexpected event type."_s);
             return;
         }
 
@@ -374,21 +357,21 @@ public:
 
         auto result = request.result();
         if (result.hasException()) {
-            m_requestCallback->sendFailure("Could not get result in callback.");
+            m_requestCallback->sendFailure("Could not get result in callback."_s);
             return;
         }
         
         auto resultValue = result.releaseReturnValue();
-        if (!WTF::holds_alternative<RefPtr<IDBCursor>>(resultValue)) {
+        if (!std::holds_alternative<RefPtr<IDBCursor>>(resultValue)) {
             end(false);
             return;
         }
 
-        auto cursor = WTF::get<RefPtr<IDBCursor>>(resultValue);
+        auto cursor = std::get<RefPtr<IDBCursor>>(resultValue);
 
         if (m_skipCount) {
             if (cursor->advance(m_skipCount).hasException())
-                m_requestCallback->sendFailure("Could not advance cursor.");
+                m_requestCallback->sendFailure("Could not advance cursor."_s);
             m_skipCount = 0;
             return;
         }
@@ -400,18 +383,28 @@ public:
 
         // Continue cursor before making injected script calls, otherwise transaction might be finished.
         if (cursor->continueFunction(nullptr).hasException()) {
-            m_requestCallback->sendFailure("Could not continue cursor.");
+            m_requestCallback->sendFailure("Could not continue cursor."_s);
             return;
         }
 
-        auto* state = context.execState();
-        auto key =  toJS(*state, *state->lexicalGlobalObject(), cursor->key());
-        auto primaryKey = toJS(*state, *state->lexicalGlobalObject(), cursor->primaryKey());
-        auto value = deserializeIDBValueToJSValue(*state, cursor->value());
-        auto dataEntry = DataEntry::create()
-            .setKey(m_injectedScript.wrapObject(key, String(), true))
-            .setPrimaryKey(m_injectedScript.wrapObject(primaryKey, String(), true))
-            .setValue(m_injectedScript.wrapObject(value, String(), true))
+        auto* lexicalGlobalObject = context.globalObject();
+
+        auto key = m_injectedScript.wrapObject(toJS(*lexicalGlobalObject, *lexicalGlobalObject, cursor->key()), String(), true);
+        if (!key)
+            return;
+
+        auto primaryKey = m_injectedScript.wrapObject(toJS(*lexicalGlobalObject, *lexicalGlobalObject, cursor->primaryKey()), String(), true);
+        if (!primaryKey)
+            return;
+
+        auto value = m_injectedScript.wrapObject(deserializeIDBValueToJSValue(*lexicalGlobalObject, cursor->value()), String(), true);
+        if (!value)
+            return;
+
+        auto dataEntry = Protocol::IndexedDB::DataEntry::create()
+            .setKey(key.releaseNonNull())
+            .setPrimaryKey(primaryKey.releaseNonNull())
+            .setValue(value.releaseNonNull())
             .release();
         m_result->addItem(WTFMove(dataEntry));
     }
@@ -424,25 +417,25 @@ public:
     }
 
 private:
-    OpenCursorCallback(InjectedScript injectedScript, Ref<RequestDataCallback>&& requestCallback, int skipCount, unsigned pageSize)
+    OpenCursorCallback(InjectedScript injectedScript, Ref<IndexedDBBackendDispatcherHandler::RequestDataCallback>&& requestCallback, int skipCount, unsigned pageSize)
         : EventListener(EventListener::CPPEventListenerType)
         , m_injectedScript(injectedScript)
         , m_requestCallback(WTFMove(requestCallback))
-        , m_result(JSON::ArrayOf<DataEntry>::create())
+        , m_result(JSON::ArrayOf<Protocol::IndexedDB::DataEntry>::create())
         , m_skipCount(skipCount)
         , m_pageSize(pageSize)
     {
     }
     InjectedScript m_injectedScript;
-    Ref<RequestDataCallback> m_requestCallback;
-    Ref<JSON::ArrayOf<DataEntry>> m_result;
+    Ref<IndexedDBBackendDispatcherHandler::RequestDataCallback> m_requestCallback;
+    Ref<JSON::ArrayOf<Protocol::IndexedDB::DataEntry>> m_result;
     int m_skipCount;
     unsigned m_pageSize;
 };
 
 class DataLoader final : public ExecutableWithDatabase {
 public:
-    static Ref<DataLoader> create(ScriptExecutionContext* context, Ref<RequestDataCallback>&& requestCallback, const InjectedScript& injectedScript, const String& objectStoreName, const String& indexName, RefPtr<IDBKeyRange>&& idbKeyRange, int skipCount, unsigned pageSize)
+    static Ref<DataLoader> create(ScriptExecutionContext* context, Ref<IndexedDBBackendDispatcherHandler::RequestDataCallback>&& requestCallback, const InjectedScript& injectedScript, const String& objectStoreName, const String& indexName, RefPtr<IDBKeyRange>&& idbKeyRange, int skipCount, unsigned pageSize)
     {
         return adoptRef(*new DataLoader(context, WTFMove(requestCallback), injectedScript, objectStoreName, indexName, WTFMove(idbKeyRange), skipCount, pageSize));
     }
@@ -456,41 +449,36 @@ public:
 
         auto idbTransaction = transactionForDatabase(&database, m_objectStoreName);
         if (!idbTransaction) {
-            m_requestCallback->sendFailure("Could not get transaction");
+            m_requestCallback->sendFailure("Could not get transaction"_s);
             return;
         }
 
         auto idbObjectStore = objectStoreForTransaction(idbTransaction.get(), m_objectStoreName);
         if (!idbObjectStore) {
-            m_requestCallback->sendFailure("Could not get object store");
+            m_requestCallback->sendFailure("Could not get object store"_s);
             return;
         }
 
         TransactionActivator activator(idbTransaction.get());
         RefPtr<IDBRequest> idbRequest;
-        auto* exec = context() ? context()->execState() : nullptr;
         if (!m_indexName.isEmpty()) {
             auto idbIndex = indexForObjectStore(idbObjectStore.get(), m_indexName);
             if (!idbIndex) {
-                m_requestCallback->sendFailure("Could not get index");
+                m_requestCallback->sendFailure("Could not get index"_s);
                 return;
             }
 
-            if (exec) {
-                auto result = idbIndex->openCursor(*exec, m_idbKeyRange.get(), IDBCursorDirection::Next);
-                if (!result.hasException())
-                    idbRequest = result.releaseReturnValue();
-            }
+            auto result = idbIndex->openCursor(m_idbKeyRange.get(), IDBCursorDirection::Next);
+            if (!result.hasException())
+                idbRequest = result.releaseReturnValue();
         } else {
-            if (exec) {
-                auto result = idbObjectStore->openCursor(*exec, m_idbKeyRange.get(), IDBCursorDirection::Next);
-                if (!result.hasException())
-                    idbRequest = result.releaseReturnValue();
-            }
+            auto result = idbObjectStore->openCursor(m_idbKeyRange.get(), IDBCursorDirection::Next);
+            if (!result.hasException())
+                idbRequest = result.releaseReturnValue();
         }
 
         if (!idbRequest) {
-            m_requestCallback->sendFailure("Could not open cursor to populate database data");
+            m_requestCallback->sendFailure("Could not open cursor to populate database data"_s);
             return;
         }
 
@@ -498,8 +486,8 @@ public:
         idbRequest->addEventListener(eventNames().successEvent, WTFMove(openCursorCallback), false);
     }
 
-    RequestCallback& requestCallback() override { return m_requestCallback.get(); }
-    DataLoader(ScriptExecutionContext* scriptExecutionContext, Ref<RequestDataCallback>&& requestCallback, const InjectedScript& injectedScript, const String& objectStoreName, const String& indexName, RefPtr<IDBKeyRange> idbKeyRange, int skipCount, unsigned pageSize)
+    BackendDispatcher::CallbackBase& requestCallback() override { return m_requestCallback.get(); }
+    DataLoader(ScriptExecutionContext* scriptExecutionContext, Ref<IndexedDBBackendDispatcherHandler::RequestDataCallback>&& requestCallback, const InjectedScript& injectedScript, const String& objectStoreName, const String& indexName, RefPtr<IDBKeyRange> idbKeyRange, int skipCount, unsigned pageSize)
         : ExecutableWithDatabase(scriptExecutionContext)
         , m_requestCallback(WTFMove(requestCallback))
         , m_injectedScript(injectedScript)
@@ -508,7 +496,7 @@ public:
         , m_idbKeyRange(WTFMove(idbKeyRange))
         , m_skipCount(skipCount)
         , m_pageSize(pageSize) { }
-    Ref<RequestDataCallback> m_requestCallback;
+    Ref<IndexedDBBackendDispatcherHandler::RequestDataCallback> m_requestCallback;
     InjectedScript m_injectedScript;
     String m_objectStoreName;
     String m_indexName;
@@ -535,19 +523,20 @@ void InspectorIndexedDBAgent::didCreateFrontendAndBackend(Inspector::FrontendRou
 
 void InspectorIndexedDBAgent::willDestroyFrontendAndBackend(Inspector::DisconnectReason)
 {
-    ErrorString ignored;
-    disable(ignored);
+    disable();
 }
 
-void InspectorIndexedDBAgent::enable(ErrorString&)
+Protocol::ErrorStringOr<void> InspectorIndexedDBAgent::enable()
 {
+    return { };
 }
 
-void InspectorIndexedDBAgent::disable(ErrorString&)
+Protocol::ErrorStringOr<void> InspectorIndexedDBAgent::disable()
 {
+    return { };
 }
 
-static ErrorStringOr<Document*> documentFromFrame(Frame* frame)
+static Protocol::ErrorStringOr<Document*> documentFromFrame(LocalFrame* frame)
 {
     Document* document = frame ? frame->document() : nullptr;
     if (!document)
@@ -556,35 +545,35 @@ static ErrorStringOr<Document*> documentFromFrame(Frame* frame)
     return document;
 }
 
-static ErrorStringOr<IDBFactory*> IDBFactoryFromDocument(Document* document)
+static Protocol::ErrorStringOr<IDBFactory*> IDBFactoryFromDocument(Document* document)
 {
-    DOMWindow* domWindow = document->domWindow();
+    auto* domWindow = document->domWindow();
     if (!domWindow)
         return makeUnexpected("Missing window for given document"_s);
 
-    IDBFactory* idbFactory = DOMWindowIndexedDatabase::indexedDB(*domWindow);
+    IDBFactory* idbFactory = WindowOrWorkerGlobalScopeIndexedDatabase::indexedDB(*domWindow);
     if (!idbFactory)
         makeUnexpected("Missing IndexedDB factory of window for given document"_s);
     
     return idbFactory;
 }
 
-static bool getDocumentAndIDBFactoryFromFrameOrSendFailure(Frame* frame, Document*& out_document, IDBFactory*& out_idbFactory, BackendDispatcher::CallbackBase& callback)
+static bool getDocumentAndIDBFactoryFromFrameOrSendFailure(LocalFrame* frame, Document*& outDocument, IDBFactory*& outIDBFactory, BackendDispatcher::CallbackBase& callback)
 {
-    ErrorStringOr<Document*> document = documentFromFrame(frame);
+    Protocol::ErrorStringOr<Document*> document = documentFromFrame(frame);
     if (!document.has_value()) {
         callback.sendFailure(document.error());
         return false;
     }
 
-    ErrorStringOr<IDBFactory*> idbFactory = IDBFactoryFromDocument(document.value());
+    Protocol::ErrorStringOr<IDBFactory*> idbFactory = IDBFactoryFromDocument(document.value());
     if (!idbFactory.has_value()) {
         callback.sendFailure(idbFactory.error());
         return false;
     }
     
-    out_document = document.value();
-    out_idbFactory = idbFactory.value();
+    outDocument = document.value();
+    outIDBFactory = idbFactory.value();
     return true;
 }
     
@@ -596,9 +585,7 @@ void InspectorIndexedDBAgent::requestDatabaseNames(const String& securityOrigin,
     if (!getDocumentAndIDBFactoryFromFrameOrSendFailure(frame, document, idbFactory, callback))
         return;
 
-    auto& openingOrigin = document->securityOrigin();
-    auto& topOrigin = document->topOrigin();
-    idbFactory->getAllDatabaseNames(topOrigin, openingOrigin, [callback = WTFMove(callback)](auto& databaseNames) {
+    idbFactory->getAllDatabaseNames(*document, [callback = WTFMove(callback)](auto& databaseNames) {
         if (!callback->isActive())
             return;
 
@@ -622,7 +609,7 @@ void InspectorIndexedDBAgent::requestDatabase(const String& securityOrigin, cons
     databaseLoader->start(idbFactory, &document->securityOrigin(), databaseName);
 }
 
-void InspectorIndexedDBAgent::requestData(const String& securityOrigin, const String& databaseName, const String& objectStoreName, const String& indexName, int skipCount, int pageSize, const JSON::Object* keyRange, Ref<RequestDataCallback>&& callback)
+void InspectorIndexedDBAgent::requestData(const String& securityOrigin, const String& databaseName, const String& objectStoreName, const String& indexName, int skipCount, int pageSize, RefPtr<JSON::Object>&& keyRange, Ref<RequestDataCallback>&& callback)
 {
     auto* frame = InspectorPageAgent::findFrameWithSecurityOrigin(m_inspectedPage, securityOrigin);
     Document* document;
@@ -630,14 +617,17 @@ void InspectorIndexedDBAgent::requestData(const String& securityOrigin, const St
     if (!getDocumentAndIDBFactoryFromFrameOrSendFailure(frame, document, idbFactory, callback))
         return;
 
-    InjectedScript injectedScript = m_injectedScriptManager.injectedScriptFor(mainWorldExecState(frame));
-    RefPtr<IDBKeyRange> idbKeyRange = keyRange ? idbKeyRangeFromKeyRange(keyRange) : nullptr;
-    if (keyRange && !idbKeyRange) {
-        callback->sendFailure("Could not parse key range."_s);
-        return;
+    RefPtr<IDBKeyRange> idbKeyRange;
+    if (keyRange) {
+        idbKeyRange = idbKeyRangeFromKeyRange(*keyRange);
+        if (!idbKeyRange) {
+            callback->sendFailure("Could not parse key range."_s);
+            return;
+        }
     }
 
-    Ref<DataLoader> dataLoader = DataLoader::create(document, WTFMove(callback), injectedScript, objectStoreName, indexName, WTFMove(idbKeyRange), skipCount, pageSize);
+    auto injectedScript = m_injectedScriptManager.injectedScriptFor(&mainWorldGlobalObject(*frame));
+    auto dataLoader = DataLoader::create(document, WTFMove(callback), injectedScript, objectStoreName, indexName, WTFMove(idbKeyRange), skipCount, pageSize);
     dataLoader->start(idbFactory, &document->securityOrigin(), databaseName);
 }
 
@@ -646,7 +636,7 @@ namespace {
 class ClearObjectStoreListener final : public EventListener {
     WTF_MAKE_NONCOPYABLE(ClearObjectStoreListener);
 public:
-    static Ref<ClearObjectStoreListener> create(Ref<ClearObjectStoreCallback> requestCallback)
+    static Ref<ClearObjectStoreListener> create(Ref<IndexedDBBackendDispatcherHandler::ClearObjectStoreCallback> requestCallback)
     {
         return adoptRef(*new ClearObjectStoreListener(WTFMove(requestCallback)));
     }
@@ -663,30 +653,30 @@ public:
         if (!m_requestCallback->isActive())
             return;
         if (event.type() != eventNames().completeEvent) {
-            m_requestCallback->sendFailure("Unexpected event type.");
+            m_requestCallback->sendFailure("Unexpected event type."_s);
             return;
         }
 
         m_requestCallback->sendSuccess();
     }
 private:
-    ClearObjectStoreListener(Ref<ClearObjectStoreCallback>&& requestCallback)
+    ClearObjectStoreListener(Ref<IndexedDBBackendDispatcherHandler::ClearObjectStoreCallback>&& requestCallback)
         : EventListener(EventListener::CPPEventListenerType)
         , m_requestCallback(WTFMove(requestCallback))
     {
     }
 
-    Ref<ClearObjectStoreCallback> m_requestCallback;
+    Ref<IndexedDBBackendDispatcherHandler::ClearObjectStoreCallback> m_requestCallback;
 };
 
 class ClearObjectStore final : public ExecutableWithDatabase {
 public:
-    static Ref<ClearObjectStore> create(ScriptExecutionContext* context, const String& objectStoreName, Ref<ClearObjectStoreCallback>&& requestCallback)
+    static Ref<ClearObjectStore> create(ScriptExecutionContext* context, const String& objectStoreName, Ref<IndexedDBBackendDispatcherHandler::ClearObjectStoreCallback>&& requestCallback)
     {
         return adoptRef(*new ClearObjectStore(context, objectStoreName, WTFMove(requestCallback)));
     }
 
-    ClearObjectStore(ScriptExecutionContext* context, const String& objectStoreName, Ref<ClearObjectStoreCallback>&& requestCallback)
+    ClearObjectStore(ScriptExecutionContext* context, const String& objectStoreName, Ref<IndexedDBBackendDispatcherHandler::ClearObjectStoreCallback>&& requestCallback)
         : ExecutableWithDatabase(context)
         , m_objectStoreName(objectStoreName)
         , m_requestCallback(WTFMove(requestCallback))
@@ -700,35 +690,31 @@ public:
 
         auto idbTransaction = transactionForDatabase(&database, m_objectStoreName, IDBTransactionMode::Readwrite);
         if (!idbTransaction) {
-            m_requestCallback->sendFailure("Could not get transaction");
+            m_requestCallback->sendFailure("Could not get transaction"_s);
             return;
         }
 
         auto idbObjectStore = objectStoreForTransaction(idbTransaction.get(), m_objectStoreName);
         if (!idbObjectStore) {
-            m_requestCallback->sendFailure("Could not get object store");
+            m_requestCallback->sendFailure("Could not get object store"_s);
             return;
         }
 
         TransactionActivator activator(idbTransaction.get());
-        RefPtr<IDBRequest> idbRequest;
-        if (auto* exec = context() ? context()->execState() : nullptr) {
-            auto result = idbObjectStore->clear(*exec);
-            ASSERT(!result.hasException());
-            if (result.hasException()) {
-                m_requestCallback->sendFailure(makeString("Could not clear object store '", m_objectStoreName, "': ", static_cast<int>(result.releaseException().code())));
-                return;
-            }
-            idbRequest = result.releaseReturnValue();
+        auto result = idbObjectStore->clear();
+        ASSERT(!result.hasException());
+        if (result.hasException()) {
+            m_requestCallback->sendFailure(makeString("Could not clear object store '", m_objectStoreName, "': ", static_cast<int>(result.releaseException().code())));
+            return;
         }
 
         idbTransaction->addEventListener(eventNames().completeEvent, ClearObjectStoreListener::create(m_requestCallback.copyRef()), false);
     }
 
-    RequestCallback& requestCallback() override { return m_requestCallback.get(); }
+    BackendDispatcher::CallbackBase& requestCallback() override { return m_requestCallback.get(); }
 private:
     const String m_objectStoreName;
-    Ref<ClearObjectStoreCallback> m_requestCallback;
+    Ref<IndexedDBBackendDispatcherHandler::ClearObjectStoreCallback> m_requestCallback;
 };
 
 } // anonymous namespace
@@ -746,4 +732,3 @@ void InspectorIndexedDBAgent::clearObjectStore(const String& securityOrigin, con
 }
 
 } // namespace WebCore
-#endif // ENABLE(INDEXED_DATABASE)
