@@ -30,6 +30,7 @@
 #include "InlineFormattingContext.h"
 #include "InlineSoftLineBreakItem.h"
 #include "LayoutBoxGeometry.h"
+#include "RenderStyleInlines.h"
 #include "TextFlags.h"
 #include "TextUtil.h"
 #include <wtf/IsoMallocInlines.h>
@@ -37,6 +38,11 @@
 namespace WebCore {
 namespace Layout {
 
+inline bool Line::Run::hasTextCombine() const
+{
+    return m_style.hasTextCombine();
+}
+
 Line::Line(const InlineFormattingContext& inlineFormattingContext)
     : m_inlineFormattingContext(inlineFormattingContext)
     , m_trimmableTrailingContent(m_runs)
@@ -50,7 +56,6 @@ Line::~Line()
 void Line::initialize(const Vector<InlineItem>& lineSpanningInlineBoxes, bool isFirstFormattedLine)
 {
     m_isFirstFormattedLine = isFirstFormattedLine;
-    m_contentIsTruncated = false;
     m_inlineBoxListWithClonedDecorationEnd.clear();
     m_clonedEndDecorationWidthForInlineBoxRuns = { };
     m_nonSpanningInlineLevelBoxCount = 0;
@@ -181,27 +186,6 @@ void Line::applyRunExpansion(InlineLayoutUnit horizontalAvailableSpace)
     m_contentLogicalWidth += accumulatedExpansion;
 }
 
-void Line::truncate(InlineLayoutUnit logicalRight)
-{
-    ASSERT(!m_contentIsTruncated);
-    ASSERT(m_contentLogicalWidth > logicalRight);
-    auto isFirstRun = true;
-    for (auto& run : m_runs) {
-        if (run.isInlineBox())
-            continue;
-        if (run.logicalRight() > logicalRight) {
-            // The first character or atomic inline-level element on a line must be clipped rather than ellipsed.
-            if (isFirstRun)
-                m_contentIsTruncated = run.truncate(logicalRight - run.logicalLeft(), Run::CanFullyTruncate::No);
-            else {
-                run.truncate(logicalRight - run.logicalLeft());
-                m_contentIsTruncated = true;
-            }
-        }
-        isFirstRun = false;
-    }
-}
-
 void Line::handleTrailingTrimmableContent(TrailingContentAction trailingTrimmableContentAction)
 {
     if (m_trimmableTrailingContent.isEmpty() || m_runs.isEmpty())
@@ -479,6 +463,7 @@ void Line::appendTextContent(const InlineTextItem& inlineTextItem, const RenderS
     }();
     auto oldContentLogicalWidth = contentLogicalWidth();
     auto runHasHangablePunctuationStart = isFirstFormattedLine() && TextUtil::hasHangablePunctuationStart(inlineTextItem, style) && !lineHasVisuallyNonEmptyContent();
+    auto contentLogicalRight = InlineLayoutUnit { };
     if (needsNewRun) {
         // Note, negative word spacing may cause glyph overlap.
         auto runLogicalLeft = [&] {
@@ -488,31 +473,32 @@ void Line::appendTextContent(const InlineTextItem& inlineTextItem, const RenderS
         }();
         m_runs.append({ inlineTextItem, style, runLogicalLeft, logicalWidth });
         // Note that the _content_ logical right may be larger than the _run_ logical right.
-        auto contentLogicalRight = runLogicalLeft + logicalWidth + m_clonedEndDecorationWidthForInlineBoxRuns;
-        m_contentLogicalWidth = std::max(oldContentLogicalWidth, contentLogicalRight);
-    } else if (style.letterSpacing() >= 0) {
-        auto& lastRun = m_runs.last();
-        lastRun.expand(inlineTextItem, logicalWidth);
-        // Ensure that property values that act like negative margin are not making the line wider.
-        m_contentLogicalWidth = std::max(oldContentLogicalWidth, lastRun.logicalRight());
+        contentLogicalRight = runLogicalLeft + logicalWidth;
     } else {
         auto& lastRun = m_runs.last();
         ASSERT(lastRun.isText());
-        // Negative letter spacing should only shorten the content to the boundary of the previous run.
-        // FIXME: We may need to traverse all the way to the previous non-text run (or even across inline boxes).
-        auto contentWidthWithoutLastTextRun = [&] {
-            if (style.fontCascade().wordSpacing() >= 0)
-                return m_contentLogicalWidth - std::max(0.f, lastRun.logicalWidth());
-            // FIXME: Let's see if we need to optimize for this is the rare case of both letter and word spacing being negative.
-            auto rightMostPosition = InlineLayoutUnit { };
-            for (auto& run : makeReversedRange(m_runs))
-                rightMostPosition = std::max(rightMostPosition, run.logicalRight());
-            return std::max(0.f, rightMostPosition);
-        }();
-        auto lastRunLogicalRight = lastRun.logicalRight();
-        lastRun.expand(inlineTextItem, logicalWidth);
-        m_contentLogicalWidth = std::max(contentWidthWithoutLastTextRun, lastRunLogicalRight + logicalWidth);
+        if (style.letterSpacing() >= 0) {
+            lastRun.expand(inlineTextItem, logicalWidth);
+            contentLogicalRight = lastRun.logicalRight();
+        } else {
+            auto contentWidthWithoutLastTextRun = [&] {
+                // FIXME: We may need to traverse all the way to the previous non-text run (or even across inline boxes).
+                if (style.fontCascade().wordSpacing() >= 0)
+                    return m_contentLogicalWidth - std::max(0.f, lastRun.logicalWidth());
+                // FIXME: Let's see if we need to optimize for this is the rare case of both letter and word spacing being negative.
+                auto rightMostPosition = InlineLayoutUnit { };
+                for (auto& run : makeReversedRange(m_runs))
+                    rightMostPosition = std::max(rightMostPosition, run.logicalRight());
+                return std::max(0.f, rightMostPosition);
+            }();
+            auto lastRunLogicalRight = lastRun.logicalRight();
+            lastRun.expand(inlineTextItem, logicalWidth);
+            // Negative letter spacing should only shorten the content to the boundary of the previous run.
+            contentLogicalRight = std::max(contentWidthWithoutLastTextRun, lastRunLogicalRight + logicalWidth);
+        }
     }
+    // Ensure that property values that act like negative margin are not making the line wider.
+    m_contentLogicalWidth = std::max(oldContentLogicalWidth, contentLogicalRight + m_clonedEndDecorationWidthForInlineBoxRuns);
 
     auto lastRunIndex = m_runs.size() - 1;
     m_trailingSoftHyphenWidth = { };
@@ -524,12 +510,6 @@ void Line::appendTextContent(const InlineTextItem& inlineTextItem, const RenderS
             m_trimmableTrailingContent.addFullyTrimmableContent(lastRunIndex, trimmableContentOffset, trimmableWidth);
             return true;
         }
-        // FIXME: Move it to InlineTextItem after removing the integration codepath check.
-        auto isPartiallyTrimmable = !inlineTextItem.isWhitespace() && style.letterSpacing() > 0 && !formattingContext().layoutState().shouldIgnoreTrailingLetterSpacing();
-        if (isPartiallyTrimmable) {
-            m_trimmableTrailingContent.addPartiallyTrimmableContent(lastRunIndex, style.letterSpacing());
-            return true;
-        }
         m_trimmableTrailingContent.reset();
         return false;
     };
@@ -740,7 +720,9 @@ inline static Line::Run::Type toLineRunType(const InlineItem& inlineItem)
     case InlineItem::Type::WordBreakOpportunity:
         return Line::Run::Type::WordBreakOpportunity;
     case InlineItem::Type::Box:
-        return inlineItem.layoutBox().isListMarkerBox() ? Line::Run::Type::ListMarker : Line::Run::Type::GenericInlineLevelBox;
+        if (inlineItem.layoutBox().isListMarkerBox())
+            return downcast<ElementBox>(inlineItem.layoutBox()).isListMarkerOutside() ? Line::Run::Type::ListMarkerOutside : Line::Run::Type::ListMarkerInside;
+        return Line::Run::Type::GenericInlineLevelBox;
     case InlineItem::Type::InlineBoxStart:
         return Line::Run::Type::InlineBoxStart;
     case InlineItem::Type::InlineBoxEnd:
@@ -851,7 +833,7 @@ std::optional<Line::Run> Line::Run::detachTrailingWhitespace()
     auto trailingWhitespaceRun = *this;
 
     auto leadingNonWhitespaceContentLength = m_textContent->length - m_trailingWhitespace->length;
-    trailingWhitespaceRun.m_textContent = { m_textContent->start + leadingNonWhitespaceContentLength, m_trailingWhitespace->length, { }, false };
+    trailingWhitespaceRun.m_textContent = { m_textContent->start + leadingNonWhitespaceContentLength, m_trailingWhitespace->length, false };
 
     trailingWhitespaceRun.m_logicalWidth = m_trailingWhitespace->width;
     trailingWhitespaceRun.m_logicalLeft = logicalRight() - m_trailingWhitespace->width;
@@ -913,30 +895,6 @@ InlineLayoutUnit Line::Run::removeTrailingWhitespace()
     return trimmedWidth;
 }
 
-bool Line::Run::truncate(InlineLayoutUnit visibleWidth, CanFullyTruncate canFullyTruncate)
-{
-    ASSERT(!visibleWidth || visibleWidth < m_logicalWidth);
-
-    if (isText()) {
-        m_isTruncated = true;
-        auto& inlineTextBox = downcast<InlineTextBox>(*m_layoutBox);
-        auto leftSide = TextUtil::WordBreakLeft { };
-        if (visibleWidth > 0.f)
-            leftSide = TextUtil::breakWord(inlineTextBox, m_textContent->start, m_textContent->length, m_logicalWidth, visibleWidth, m_logicalLeft, m_style.fontCascade());
-
-        if (leftSide.length)
-            m_textContent->partiallyVisibleContent = { leftSide.length, leftSide.logicalWidth };
-        else if (canFullyTruncate == CanFullyTruncate::No) {
-            auto firstCharacterLength = TextUtil::firstUserPerceivedCharacterLength(inlineTextBox, m_textContent->start, m_textContent->length);
-            auto firstCharacterWidth = TextUtil::width(inlineTextBox, inlineTextBox.style().fontCascade(), m_textContent->start, m_textContent->start + firstCharacterLength, { }, TextUtil::UseTrailingWhitespaceMeasuringOptimization::No);
-            m_textContent->partiallyVisibleContent = { firstCharacterLength, firstCharacterWidth };
-        }
-    } else
-        m_isTruncated = canFullyTruncate == CanFullyTruncate::Yes;
-
-    return m_isTruncated;
-}
-
 }
 }
 
