@@ -33,23 +33,24 @@
 #include "DocumentLoader.h"
 #include "DOMApplicationCache.h"
 #include "EventNames.h"
-#include "FileSystem.h"
-#include "Frame.h"
+#include "FrameDestructionObserverInlines.h"
 #include "FrameLoader.h"
-#include "FrameLoaderClient.h"
 #include "InspectorInstrumentation.h"
+#include "LocalFrame.h"
+#include "LocalFrameLoaderClient.h"
 #include "Page.h"
 #include "ProgressEvent.h"
 #include "ResourceHandle.h"
 #include "ResourceRequest.h"
 #include "Settings.h"
 #include "SubresourceLoader.h"
+#include <wtf/FileSystem.h>
 #include <wtf/UUID.h>
 
 namespace WebCore {
 
 ApplicationCacheHost::ApplicationCacheHost(DocumentLoader& documentLoader)
-    : m_documentLoader(documentLoader)
+    : m_documentLoader(CheckedRef { documentLoader })
 {
 }
 
@@ -62,16 +63,28 @@ ApplicationCacheHost::~ApplicationCacheHost()
         m_candidateApplicationCacheGroup->disassociateDocumentLoader(m_documentLoader);
 }
 
+ApplicationCacheGroup* ApplicationCacheHost::candidateApplicationCacheGroup() const
+{
+    return m_candidateApplicationCacheGroup.get();
+}
+
 void ApplicationCacheHost::selectCacheWithoutManifest()
 {
-    ASSERT(m_documentLoader.frame());
-    ApplicationCacheGroup::selectCacheWithoutManifestURL(*m_documentLoader.frame());
+    ASSERT(m_documentLoader->frame());
+    ApplicationCacheGroup::selectCacheWithoutManifestURL(*m_documentLoader->frame());
 }
 
 void ApplicationCacheHost::selectCacheWithManifest(const URL& manifestURL)
 {
-    ASSERT(m_documentLoader.frame());
-    ApplicationCacheGroup::selectCache(*m_documentLoader.frame(), manifestURL);
+    ASSERT(m_documentLoader->frame());
+    ApplicationCacheGroup::selectCache(*m_documentLoader->frame(), manifestURL);
+}
+
+bool ApplicationCacheHost::canLoadMainResource(const ResourceRequest& request)
+{
+    if (!isApplicationCacheEnabled() || isApplicationCacheBlockedForRequest(request))
+        return false;
+    return !!ApplicationCacheGroup::cacheForMainRequest(request, m_documentLoader.ptr());
 }
 
 void ApplicationCacheHost::maybeLoadMainResource(const ResourceRequest& request, SubstituteData& substituteData)
@@ -80,7 +93,7 @@ void ApplicationCacheHost::maybeLoadMainResource(const ResourceRequest& request,
     if (!substituteData.isValid() && isApplicationCacheEnabled() && !isApplicationCacheBlockedForRequest(request)) {
         ASSERT(!m_mainResourceApplicationCache);
 
-        m_mainResourceApplicationCache = ApplicationCacheGroup::cacheForMainRequest(request, &m_documentLoader);
+        m_mainResourceApplicationCache = ApplicationCacheGroup::cacheForMainRequest(request, m_documentLoader.ptr());
 
         if (m_mainResourceApplicationCache) {
             // Get the resource from the application cache. By definition, cacheForMainRequest() returns a cache that contains the resource.
@@ -114,9 +127,9 @@ bool ApplicationCacheHost::maybeLoadFallbackForMainResponse(const ResourceReques
     if (r.httpStatusCode() / 100 == 4 || r.httpStatusCode() / 100 == 5) {
         ASSERT(!m_mainResourceApplicationCache);
         if (isApplicationCacheEnabled() && !isApplicationCacheBlockedForRequest(request)) {
-            m_mainResourceApplicationCache = ApplicationCacheGroup::fallbackCacheForMainRequest(request, &m_documentLoader);
+            m_mainResourceApplicationCache = ApplicationCacheGroup::fallbackCacheForMainRequest(request, m_documentLoader.ptr());
 
-            if (scheduleLoadFallbackResourceFromApplicationCache(m_documentLoader.mainResourceLoader(), m_mainResourceApplicationCache.get()))
+            if (scheduleLoadFallbackResourceFromApplicationCache(m_documentLoader->mainResourceLoader(), m_mainResourceApplicationCache.get()))
                 return true;
         }
     }
@@ -128,22 +141,22 @@ bool ApplicationCacheHost::maybeLoadFallbackForMainError(const ResourceRequest&
     if (!error.isCancellation()) {
         ASSERT(!m_mainResourceApplicationCache);
         if (isApplicationCacheEnabled() && !isApplicationCacheBlockedForRequest(request)) {
-            m_mainResourceApplicationCache = ApplicationCacheGroup::fallbackCacheForMainRequest(request, &m_documentLoader);
+            m_mainResourceApplicationCache = ApplicationCacheGroup::fallbackCacheForMainRequest(request, m_documentLoader.ptr());
 
-            if (scheduleLoadFallbackResourceFromApplicationCache(m_documentLoader.mainResourceLoader(), m_mainResourceApplicationCache.get()))
+            if (scheduleLoadFallbackResourceFromApplicationCache(m_documentLoader->mainResourceLoader(), m_mainResourceApplicationCache.get()))
                 return true;
         }
     }
     return false;
 }
 
-void ApplicationCacheHost::mainResourceDataReceived(const char*, int, long long, bool)
+void ApplicationCacheHost::mainResourceDataReceived(const SharedBuffer&, long long, bool)
 {
 }
 
 void ApplicationCacheHost::failedLoadingMainResource()
 {
-    auto* group = m_candidateApplicationCacheGroup;
+    auto* group = m_candidateApplicationCacheGroup.get();
     if (!group && m_applicationCache) {
         if (mainResourceApplicationCache()) {
             // Even when the main resource is being loaded from an application cache, loading can fail if aborted.
@@ -187,9 +200,9 @@ bool ApplicationCacheHost::maybeLoadResource(ResourceLoader& loader, const Resou
         return false;
 
     if (resource)
-        m_documentLoader.scheduleSubstituteResourceLoad(loader, *resource);
+        m_documentLoader->scheduleSubstituteResourceLoad(loader, *resource);
     else
-        m_documentLoader.scheduleCannotShowURLError(loader);
+        m_documentLoader->scheduleCannotShowURLError(loader);
     return true;
 }
 
@@ -223,7 +236,7 @@ bool ApplicationCacheHost::maybeLoadFallbackForError(ResourceLoader* resourceLoa
         return false;
 
     if (!error.isCancellation()) {
-        if (resourceLoader == m_documentLoader.mainResourceLoader())
+        if (resourceLoader == m_documentLoader->mainResourceLoader())
             return maybeLoadFallbackForMainError(resourceLoader->request(), error);
         if (scheduleLoadFallbackResourceFromApplicationCache(resourceLoader))
             return true;
@@ -233,23 +246,13 @@ bool ApplicationCacheHost::maybeLoadFallbackForError(ResourceLoader* resourceLoa
 
 URL ApplicationCacheHost::createFileURL(const String& path)
 {
-    // FIXME: Can we just use fileURLWithFileSystemPath instead?
-#if USE(CF) && PLATFORM(WIN)
-    URL url(adoptCF(CFURLCreateWithFileSystemPath(0, path.createCFString().get(), kCFURLWindowsPathStyle, false)).get());
-#else
-    URL url;
-    url.setProtocol("file"_s);
-    url.setPath(path);
-#endif
-    return url;
+    return URL::fileURLWithFileSystemPath(path);
 }
 
 static inline RefPtr<SharedBuffer> bufferFromResource(ApplicationCacheResource& resource)
 {
-    // FIXME: Clients probably do not need a copy of the SharedBuffer.
-    // Remove the call to copy() once we ensure SharedBuffer will not be modified.
     if (resource.path().isEmpty())
-        return resource.data().copy();
+        return resource.data().makeContiguous();
     return SharedBuffer::createWithContentsOfFile(resource.path());
 }
 
@@ -261,7 +264,7 @@ bool ApplicationCacheHost::maybeLoadSynchronously(ResourceRequest& request, Reso
 
     auto responseData = resource ? bufferFromResource(*resource) : nullptr;
     if (!responseData) {
-        error = m_documentLoader.frameLoader()->client().cannotShowURLError(request);
+        error = m_documentLoader->frameLoader()->client().cannotShowURLError(request);
         return true;
     }
 
@@ -281,14 +284,12 @@ void ApplicationCacheHost::maybeLoadFallbackSynchronously(const ResourceRequest&
         ApplicationCacheResource* resource;
         if (getApplicationCacheFallbackResource(request, resource)) {
             response = resource->response();
-            // FIXME: Clients proably do not need a copy of the SharedBuffer.
-            // Remove the call to copy() once we ensure SharedBuffer will not be modified.
-            data = resource->data().copy();
+            data = resource->data().makeContiguous();
         }
     }
 }
 
-bool ApplicationCacheHost::canCacheInPageCache()
+bool ApplicationCacheHost::canCacheInBackForwardCache()
 {
     return !applicationCache() && !candidateApplicationCacheGroup();
 }
@@ -299,10 +300,10 @@ void ApplicationCacheHost::setDOMApplicationCache(DOMApplicationCache* domApplic
     m_domApplicationCache = domApplicationCache;
 }
 
-void ApplicationCacheHost::notifyDOMApplicationCache(const AtomicString& eventType, int total, int done)
+void ApplicationCacheHost::notifyDOMApplicationCache(const AtomString& eventType, int total, int done)
 {
     if (eventType != eventNames().progressEvent)
-        InspectorInstrumentation::updateApplicationCacheStatus(m_documentLoader.frame());
+        InspectorInstrumentation::updateApplicationCacheStatus(m_documentLoader->frame());
 
     if (m_defersEvents) {
         // Event dispatching is deferred until document.onload has fired.
@@ -313,7 +314,7 @@ void ApplicationCacheHost::notifyDOMApplicationCache(const AtomicString& eventTy
     dispatchDOMEvent(eventType, total, done);
 }
 
-void ApplicationCacheHost::stopLoadingInFrame(Frame& frame)
+void ApplicationCacheHost::stopLoadingInFrame(LocalFrame& frame)
 {
     ASSERT(!m_applicationCache || !m_candidateApplicationCacheGroup || m_applicationCache->group() == m_candidateApplicationCacheGroup);
 
@@ -370,17 +371,18 @@ ApplicationCacheHost::CacheInfo ApplicationCacheHost::applicationCacheInfo()
     return { cache->manifestResource()->url(), 0, 0, cache->estimatedSizeInStorage() };
 }
 
-static Ref<Event> createApplicationCacheEvent(const AtomicString& eventType, int total, int done)
+static Ref<Event> createApplicationCacheEvent(const AtomString& eventType, int total, int done)
 {
     if (eventType == eventNames().progressEvent)
         return ProgressEvent::create(eventType, true, done, total);
-    return Event::create(eventType, false, false);
+    return Event::create(eventType, Event::CanBubble::No, Event::IsCancelable::No);
 }
 
-void ApplicationCacheHost::dispatchDOMEvent(const AtomicString& eventType, int total, int done)
+void ApplicationCacheHost::dispatchDOMEvent(const AtomString& eventType, int total, int done)
 {
-    if (!m_domApplicationCache)
+    if (!m_domApplicationCache || !m_domApplicationCache->frame())
         return;
+
     m_domApplicationCache->dispatchEvent(createApplicationCacheEvent(eventType, total, done));
 }
 
@@ -406,7 +408,7 @@ bool ApplicationCacheHost::shouldLoadResourceFromApplicationCache(const Resource
         return false;
 
     ResourceRequest request(originalRequest);
-    if (auto* loaderFrame = m_documentLoader.frame()) {
+    if (auto* loaderFrame = m_documentLoader->frame()) {
         if (auto* document = loaderFrame->document())
             document->contentSecurityPolicy()->upgradeInsecureRequestIfNeeded(request, ContentSecurityPolicy::InsecureRequestType::Load);
     }
@@ -418,11 +420,11 @@ bool ApplicationCacheHost::shouldLoadResourceFromApplicationCache(const Resource
 
     // If the resource's URL is an master entry, the manifest, an explicit entry, or a fallback entry
     // in the application cache, then get the resource from the cache (instead of fetching it).
-    resource = cache->resourceForURL(request.url());
+    resource = cache->resourceForURL(request.url().string());
 
-    // Resources that match fallback namespaces or online whitelist entries are fetched from the network,
+    // Resources that match fallback namespaces or online allowlist entries are fetched from the network,
     // unless they are also cached.
-    if (!resource && (cache->allowsAllNetworkRequests() || cache->urlMatchesFallbackNamespace(request.url()) || cache->isURLInOnlineWhitelist(request.url())))
+    if (!resource && (cache->allowsAllNetworkRequests() || cache->urlMatchesFallbackNamespace(request.url()) || cache->isURLInOnlineAllowlist(request.url())))
         return false;
 
     // Resources that are not present in the manifest will always fail to load (at least, after the
@@ -445,12 +447,12 @@ bool ApplicationCacheHost::getApplicationCacheFallbackResource(const ResourceReq
         return false;
 
     URL fallbackURL;
-    if (cache->isURLInOnlineWhitelist(request.url()))
+    if (cache->isURLInOnlineAllowlist(request.url()))
         return false;
     if (!cache->urlMatchesFallbackNamespace(request.url(), &fallbackURL))
         return false;
 
-    resource = cache->resourceForURL(fallbackURL);
+    resource = cache->resourceForURL(fallbackURL.string());
     ASSERT(resource);
     return true;
 }
@@ -473,7 +475,7 @@ bool ApplicationCacheHost::scheduleLoadFallbackResourceFromApplicationCache(Reso
         return false;
 
     loader->willSwitchToSubstituteResource();
-    m_documentLoader.scheduleSubstituteResourceLoad(*loader, *resource);
+    m_documentLoader->scheduleSubstituteResourceLoad(*loader, *resource);
     return true;
 }
 
@@ -505,7 +507,7 @@ bool ApplicationCacheHost::update()
     auto* cache = applicationCache();
     if (!cache)
         return false;
-    auto* frame = m_documentLoader.frame();
+    auto* frame = m_documentLoader->frame();
     if (!frame)
         return false;
     cache->group()->update(*frame, ApplicationCacheUpdateWithoutBrowsingContext);
@@ -517,27 +519,31 @@ bool ApplicationCacheHost::swapCache()
     auto* cache = applicationCache();
     if (!cache)
         return false;
+    
+    auto* group = cache->group();
+    if (!group)
+        return false;
 
     // If the group of application caches to which cache belongs has the lifecycle status obsolete, unassociate document from cache.
-    if (cache->group()->isObsolete()) {
-        cache->group()->disassociateDocumentLoader(m_documentLoader);
+    if (group->isObsolete()) {
+        group->disassociateDocumentLoader(m_documentLoader);
         return true;
     }
 
     // If there is no newer cache, raise an InvalidStateError exception.
-    auto* newestCache = cache->group()->newestCache();
-    if (cache == newestCache)
+    auto* newestCache = group->newestCache();
+    if (!newestCache || cache == newestCache)
         return false;
     
-    ASSERT(cache->group() == newestCache->group());
+    ASSERT(group == newestCache->group());
     setApplicationCache(newestCache);
-    InspectorInstrumentation::updateApplicationCacheStatus(m_documentLoader.frame());
+    InspectorInstrumentation::updateApplicationCacheStatus(m_documentLoader->frame());
     return true;
 }
 
 void ApplicationCacheHost::abort()
 {
-    auto* frame = m_documentLoader.frame();
+    auto* frame = m_documentLoader->frame();
     if (!frame)
         return;
     if (auto* cacheGroup = candidateApplicationCacheGroup())
@@ -548,17 +554,17 @@ void ApplicationCacheHost::abort()
 
 bool ApplicationCacheHost::isApplicationCacheEnabled()
 {
-    return m_documentLoader.frame() && m_documentLoader.frame()->settings().offlineWebApplicationCacheEnabled() && !m_documentLoader.frame()->page()->usesEphemeralSession();
+    return m_documentLoader->frame() && m_documentLoader->frame()->settings().offlineWebApplicationCacheEnabled() && m_documentLoader->frame()->page() && !m_documentLoader->frame()->page()->usesEphemeralSession();
 }
 
-bool ApplicationCacheHost::isApplicationCacheBlockedForRequest(const ResourceRequest& request)
+bool ApplicationCacheHost::isApplicationCacheBlockedForRequest(const ResourceRequest&)
 {
-    auto* frame = m_documentLoader.frame();
+    auto* frame = m_documentLoader->frame();
     if (!frame)
         return false;
     if (frame->isMainFrame())
         return false;
-    return !SecurityOrigin::create(request.url())->canAccessApplicationCache(frame->document()->topOrigin());
+    return frame->document()->canAccessResource(ScriptExecutionContext::ResourceType::ApplicationCache) != ScriptExecutionContext::HasResourceAccess::Yes;
 }
 
 }  // namespace WebCore
