@@ -32,8 +32,8 @@
 #include "Document.h"
 #include "FrameLoader.h"
 #include "FrameLoaderTypes.h"
-#include "FrameView.h"
 #include "Image.h"
+#include "LocalFrameView.h"
 #include "Logging.h"
 #include "PublicSuffix.h"
 #include "SharedBuffer.h"
@@ -222,8 +222,11 @@ void MemoryCache::pruneLiveResources(bool shouldDestroyDecodedDataForAllLiveReso
 void MemoryCache::forEachResource(const Function<void(CachedResource&)>& function)
 {
     for (auto& unprotectedLRUList : m_allResources) {
-        for (auto& resource : copyToVector(*unprotectedLRUList))
+        for (auto& resource : copyToVector(*unprotectedLRUList)) {
+            if (!resource)
+                continue;
             function(*resource);
+        }
     }
 }
 
@@ -257,7 +260,7 @@ void MemoryCache::pruneLiveResourcesToSize(unsigned targetSize, bool shouldDestr
 
     SetForScope reentrancyProtector(m_inPruneResources, true);
 
-    MonotonicTime currentTime = FrameView::currentPaintTimeStamp();
+    MonotonicTime currentTime = LocalFrameView::currentPaintTimeStamp();
     if (!currentTime) // In case prune is called directly, outside of a Frame paint.
         currentTime = MonotonicTime::now();
     
@@ -271,9 +274,9 @@ void MemoryCache::pruneLiveResourcesToSize(unsigned targetSize, bool shouldDestr
     // For more details see: https://bugs.webkit.org/show_bug.cgi?id=30209
     auto it = m_liveDecodedResources.begin();
     while (it != m_liveDecodedResources.end()) {
-        auto* current = *it;
+        auto& current = *it;
 
-        LOG(ResourceLoading, " live resource %p %.255s - loaded %d, decodedSize %u", current, current->url().string().utf8().data(), current->isLoaded(), current->decodedSize());
+        LOG(ResourceLoading, " live resource %p %.255s - loaded %d, decodedSize %u", &current, current.url().string().utf8().data(), current.isLoaded(), current.decodedSize());
 
         // Increment the iterator now because the call to destroyDecodedData() below
         // may cause a call to ListHashSet::remove() and invalidate the current
@@ -283,10 +286,10 @@ void MemoryCache::pruneLiveResourcesToSize(unsigned targetSize, bool shouldDestr
         // by a given iterator.
         ++it;
 
-        ASSERT(current->hasClients());
-        if (current->isLoaded() && current->decodedSize()) {
+        ASSERT(current.hasClients());
+        if (current.isLoaded() && current.decodedSize()) {
             // Check to see if the remaining resources are too new to prune.
-            Seconds elapsedTime = currentTime - current->m_lastDecodedAccessTime;
+            Seconds elapsedTime = currentTime - current.m_lastDecodedAccessTime;
             if (!shouldDestroyDecodedDataForAllLiveResources && elapsedTime < cMinDelayBeforeLiveDecodedPrune) {
                 LOG(ResourceLoading, " current time is less than min delay before pruning (%.3fms)", elapsedTime.milliseconds());
                 return;
@@ -294,7 +297,7 @@ void MemoryCache::pruneLiveResourcesToSize(unsigned targetSize, bool shouldDestr
 
             // Destroy our decoded data. This will remove us from m_liveDecodedResources, and possibly move us
             // to a different LRU list in m_allResources.
-            current->destroyDecodedData();
+            current.destroyDecodedData();
 
             if (targetSize && m_liveSize <= targetSize)
                 return;
@@ -337,7 +340,10 @@ void MemoryCache::pruneDeadResourcesToSize(unsigned targetSize)
         // First flush all the decoded data in this queue.
         // Remove from the head, since this is the least frequently accessed of the objects.
         for (auto& resource : lruList) {
-            LOG(ResourceLoading, " lru resource %p - in cache %d, has clients %d, preloaded %d, loaded %d", resource, resource->inCache(), resource->hasClients(), resource->isPreloaded(), resource->isLoaded());
+            if (!resource)
+                continue;
+
+            LOG(ResourceLoading, " lru resource %p - in cache %d, has clients %d, preloaded %d, loaded %d", resource.get(), resource->inCache(), resource->hasClients(), resource->isPreloaded(), resource->isLoaded());
             if (!resource->inCache())
                 continue;
 
@@ -346,7 +352,7 @@ void MemoryCache::pruneDeadResourcesToSize(unsigned targetSize)
                 // m_liveDecodedResources, and possibly move us to a different 
                 // LRU list in m_allResources.
 
-                LOG(ResourceLoading, " lru resource %p destroyDecodedData", resource);
+                LOG(ResourceLoading, " lru resource %p destroyDecodedData", resource.get());
 
                 resource->destroyDecodedData();
 
@@ -360,7 +366,10 @@ void MemoryCache::pruneDeadResourcesToSize(unsigned targetSize)
         // Now evict objects from this list.
         // Remove from the head, since this is the least frequently accessed of the objects.
         for (auto& resource : lruList) {
-            LOG(ResourceLoading, " lru resource %p - in cache %d, has clients %d, preloaded %d, loaded %d", resource, resource->inCache(), resource->hasClients(), resource->isPreloaded(), resource->isLoaded());
+            if (!resource)
+                continue;
+
+            LOG(ResourceLoading, " lru resource %p - in cache %d, has clients %d, preloaded %d, loaded %d", resource.get(), resource->inCache(), resource->hasClients(), resource->isPreloaded(), resource->isLoaded());
             if (!resource->inCache())
                 continue;
 
@@ -373,7 +382,7 @@ void MemoryCache::pruneDeadResourcesToSize(unsigned targetSize)
             
         // Shrink the vector back down so we don't waste time inspecting
         // empty LRU lists on future prunes.
-        if (!m_allResources[i]->isEmpty())
+        if (!m_allResources[i]->isEmptyIgnoringNullReferences())
             canShrinkLRULists = false;
         else if (canShrinkLRULists)
             m_allResources.shrink(i);
@@ -453,7 +462,7 @@ void MemoryCache::removeFromLRUList(CachedResource& resource)
     // Verify that the list we got is the list we want.
     ASSERT(resource.m_lruIndex == oldListIndex);
 
-    bool removed = list.remove(&resource);
+    bool removed = list.remove(resource);
     ASSERT_UNUSED(removed, removed);
 }
 
@@ -462,7 +471,7 @@ void MemoryCache::insertInLRUList(CachedResource& resource)
     ASSERT(resource.inCache());
     ASSERT(resource.accessCount() > 0);
     
-    auto addResult = lruListFor(resource).add(&resource);
+    auto addResult = lruListFor(resource).add(resource);
     ASSERT_UNUSED(addResult, addResult.isNewEntry);
 }
 
@@ -485,6 +494,11 @@ void MemoryCache::resourceAccessed(CachedResource& resource)
     insertInLRUList(resource);
 }
 
+bool MemoryCache::inLiveDecodedResourcesList(CachedResource& resource) const
+{
+    return m_liveDecodedResources.contains(resource);
+}
+
 void MemoryCache::removeResourcesWithOrigin(const SecurityOrigin& origin, const String& cachePartition)
 {
     Vector<CachedResource*> resourcesWithOrigin;
@@ -514,7 +528,7 @@ void MemoryCache::removeResourcesWithOrigin(const SecurityOrigin& origin)
 
 void MemoryCache::removeResourcesWithOrigin(const ClientOrigin& origin)
 {
-    auto cachePartition = origin.topOrigin == origin.clientOrigin ? emptyString() : ResourceRequest::partitionName(origin.topOrigin.host);
+    auto cachePartition = origin.topOrigin == origin.clientOrigin ? emptyString() : ResourceRequest::partitionName(origin.topOrigin.host());
     removeResourcesWithOrigin(origin.clientOrigin.securityOrigin(), cachePartition);
 }
 
@@ -580,14 +594,14 @@ HashSet<RefPtr<SecurityOrigin>> MemoryCache::originsWithCache(PAL::SessionID ses
 
 void MemoryCache::removeFromLiveDecodedResourcesList(CachedResource& resource)
 {
-    m_liveDecodedResources.remove(&resource);
+    m_liveDecodedResources.remove(resource);
 }
 
 void MemoryCache::insertInLiveDecodedResourcesList(CachedResource& resource)
 {
     // Make sure we aren't in the list already.
-    ASSERT(!m_liveDecodedResources.contains(&resource));
-    m_liveDecodedResources.add(&resource);
+    ASSERT(!m_liveDecodedResources.contains(resource));
+    m_liveDecodedResources.add(resource);
 }
 
 void MemoryCache::addToLiveResourcesSize(CachedResource& resource)
@@ -761,9 +775,9 @@ void MemoryCache::dumpLRULists(bool includeLive) const
     int size = m_allResources.size();
     for (int i = size - 1; i >= 0; i--) {
         WTFLogAlways("\nList %d:\n", i);
-        for (auto* resource : *m_allResources[i]) {
-            if (includeLive || !resource->hasClients())
-                WTFLogAlways("  %p %.255s %.1fK, %.1fK, accesses: %u, clients: %d\n", resource, resource->url().string().utf8().data(), resource->decodedSize() / 1024.0f, (resource->encodedSize() + resource->overheadSize()) / 1024.0f, resource->accessCount(), resource->numberOfClients());
+        for (auto& resource : *m_allResources[i]) {
+            if (includeLive || !resource.hasClients())
+                WTFLogAlways("  %p %.255s %.1fK, %.1fK, accesses: %u, clients: %d\n", &resource, resource.url().string().utf8().data(), resource.decodedSize() / 1024.0f, (resource.encodedSize() + resource.overheadSize()) / 1024.0f, resource.accessCount(), resource.numberOfClients());
         }
     }
 }
