@@ -40,20 +40,26 @@
 #include "DocumentLoader.h"
 #include "Editor.h"
 #include "EditorClient.h"
+#include "ElementAncestorIteratorInlines.h"
 #include "Event.h"
 #include "EventHandler.h"
 #include "FormState.h"
-#include "Frame.h"
 #include "FrameLoadRequest.h"
 #include "FrameLoader.h"
-#include "FrameLoaderClient.h"
 #include "FrameSelection.h"
+#include "FrameSnapshotting.h"
+#include "HTMLCanvasElement.h"
 #include "HTMLFormControlElement.h"
 #include "HTMLFormElement.h"
 #include "HTMLFrameOwnerElement.h"
+#include "HTMLImageElement.h"
+#include "HTMLTableElement.h"
 #include "HitTestResult.h"
+#include "ImageBuffer.h"
 #include "ImageOverlay.h"
 #include "InspectorController.h"
+#include "LocalFrame.h"
+#include "LocalFrameLoaderClient.h"
 #include "LocalizedStrings.h"
 #include "MouseEvent.h"
 #include "NavigationAction.h"
@@ -63,6 +69,8 @@
 #include "RenderImage.h"
 #include "ReplaceSelectionCommand.h"
 #include "ResourceRequest.h"
+#include "SVGElementTypeHelpers.h"
+#include "SVGSVGElement.h"
 #include "Settings.h"
 #include "TextIterator.h"
 #include "TranslationContextMenuInfo.h"
@@ -86,19 +94,24 @@
 #include <wtf/cocoa/RuntimeApplicationChecksCocoa.h>
 #endif
 
+#if ENABLE(ACCESSIBILITY_ANIMATION_CONTROL)
+#include <wtf/SoftLinking.h>
+SOFT_LINK_LIBRARY_OPTIONAL(libAccessibility)
+SOFT_LINK_OPTIONAL(libAccessibility, _AXSReduceMotionAutoplayAnimatedImagesEnabled, Boolean, (), ());
+#endif
+
 namespace WebCore {
 
 using namespace WTF::Unicode;
 
-ContextMenuController::ContextMenuController(Page& page, ContextMenuClient& client)
+ContextMenuController::ContextMenuController(Page& page, UniqueRef<ContextMenuClient>&& client)
     : m_page(page)
-    , m_client(client)
+    , m_client(WTFMove(client))
 {
 }
 
 ContextMenuController::~ContextMenuController()
 {
-    m_client.contextMenuDestroyed();
 }
 
 void ContextMenuController::clearContextMenu()
@@ -155,6 +168,55 @@ void ContextMenuController::showContextMenu(Event& event, ContextMenuProvider& p
     showContextMenu(event);
 }
 
+#if ENABLE(CONTEXT_MENU_QR_CODE_DETECTION)
+
+static void prepareContextForQRCode(ContextMenuContext& context)
+{
+    auto& result = context.hitTestResult();
+
+    RefPtr node = result.innerNonSharedNode();
+    if (!node || !node->document().settings().contextMenuQRCodeDetectionEnabled())
+        return;
+
+    if (result.image() || !result.absoluteLinkURL().isEmpty())
+        return;
+
+    RefPtr<Element> element;
+    for (auto& lineage : lineageOfType<Element>(is<Element>(*node) ? downcast<Element>(*node) : *node->parentElement())) {
+        if (is<HTMLTableElement>(lineage) || is<HTMLCanvasElement>(lineage) || is<HTMLImageElement>(lineage) || is<SVGSVGElement>(lineage)) {
+            element = &lineage;
+            break;
+        }
+    }
+
+    if (!element || !element->renderer())
+        return;
+
+    auto elementRect = element->renderer()->absoluteBoundingBoxRect();
+
+    // Constant chosen to be larger than we think any QR code would be, matching the original Safari implementation.
+    constexpr auto maxQRCodeContainerDimension = 800;
+    if (elementRect.width() > maxQRCodeContainerDimension || elementRect.height() > maxQRCodeContainerDimension)
+        return;
+
+    RefPtr frame = element->document().frame();
+    if (!frame)
+        return;
+
+    auto nodeSnapshotImageBuffer = snapshotNode(*frame, *element, { { }, PixelFormat::BGRA8, DestinationColorSpace::SRGB() });
+    auto nodeSnapshotImage = ImageBuffer::sinkIntoImage(WTFMove(nodeSnapshotImageBuffer), PreserveResolution::Yes);
+    context.setPotentialQRCodeNodeSnapshotImage(nodeSnapshotImage.get());
+
+    // FIXME: Node snapshotting does not take transforms into account, making it unreliable for QR code detection.
+    // As a fallback, also take a viewport-level snapshot. A node snapshot is still required to capture partially
+    // obscured elements. This workaround can be removed once rdar://87204215 is fixed.
+    auto viewportSnapshotImageBuffer = snapshotFrameRect(*frame, elementRect, { { }, PixelFormat::BGRA8, DestinationColorSpace::SRGB() });
+    auto viewportSnapshotImage = ImageBuffer::sinkIntoImage(WTFMove(viewportSnapshotImageBuffer), PreserveResolution::Yes);
+    context.setPotentialQRCodeViewportSnapshotImage(viewportSnapshotImage.get());
+}
+
+#endif
+
 std::unique_ptr<ContextMenu> ContextMenuController::maybeCreateContextMenu(Event& event, OptionSet<HitTestRequest::Type> hitType, ContextMenuContext::Type contextType)
 {
     if (!is<MouseEvent>(event))
@@ -173,6 +235,9 @@ std::unique_ptr<ContextMenu> ContextMenuController::maybeCreateContextMenu(Event
         return nullptr;
 
     m_context = { contextType, result, &event };
+#if ENABLE(CONTEXT_MENU_QR_CODE_DETECTION)
+    prepareContextForQRCode(m_context);
+#endif
     
     return makeUnique<ContextMenu>();
 }
@@ -191,7 +256,7 @@ void ContextMenuController::didDismissContextMenu()
         m_menuProvider->didDismissContextMenu();
 }
 
-static void openNewWindow(const URL& urlToLoad, Frame& frame, Event* event, ShouldOpenExternalURLsPolicy shouldOpenExternalURLsPolicy)
+static void openNewWindow(const URL& urlToLoad, LocalFrame& frame, Event* event, ShouldOpenExternalURLsPolicy shouldOpenExternalURLsPolicy)
 {
     Page* oldPage = frame.page();
     if (!oldPage)
@@ -205,19 +270,20 @@ static void openNewWindow(const URL& urlToLoad, Frame& frame, Event* event, Shou
     if (!newPage)
         return;
     newPage->chrome().show();
-    newPage->mainFrame().loader().loadFrameRequest(WTFMove(frameLoadRequest), event, { });
+    if (auto* localMainFrame = dynamicDowncast<LocalFrame>(newPage->mainFrame()))
+        localMainFrame->loader().loadFrameRequest(WTFMove(frameLoadRequest), event, { });
 }
 
 #if PLATFORM(GTK)
 
-static void insertUnicodeCharacter(UChar character, Frame& frame)
+static void insertUnicodeCharacter(UChar character, LocalFrame& frame)
 {
     String text(&character, 1);
     if (!frame.editor().shouldInsertText(text, frame.selection().selection().toNormalizedRange(), EditorInsertAction::Typed))
         return;
 
     ASSERT(frame.document());
-    TypingCommand::insertText(*frame.document(), text, 0, TypingCommand::TextCompositionNone);
+    TypingCommand::insertText(*frame.document(), text, { }, TypingCommand::TextCompositionType::None);
 }
 
 #endif
@@ -236,7 +302,7 @@ void ContextMenuController::contextMenuItemSelected(ContextMenuAction action, co
     if (!frame)
         return;
 
-    Ref<Frame> protector(*frame);
+    Ref protector(*frame);
     RefPtr<Event> eventForLoadRequests = [&]() -> Event* {
 #if PLATFORM(COCOA)
         if (!linkedOnOrAfterSDKWithBehavior(SDKAlignedBehavior::ContextMenuTriggersLinkActivationNavigationType))
@@ -251,7 +317,7 @@ void ContextMenuController::contextMenuItemSelected(ContextMenuAction action, co
         break;
     case ContextMenuItemTagDownloadLinkToDisk:
         // FIXME: Some day we should be able to do this from within WebCore. (Bug 117709)
-        m_client.downloadURL(m_context.hitTestResult().absoluteLinkURL());
+        m_client->downloadURL(m_context.hitTestResult().absoluteLinkURL());
         break;
     case ContextMenuItemTagCopyLinkToClipboard:
         frame->editor().copyURL(m_context.hitTestResult().absoluteLinkURL(), m_context.hitTestResult().textContent());
@@ -261,7 +327,7 @@ void ContextMenuController::contextMenuItemSelected(ContextMenuAction action, co
         break;
     case ContextMenuItemTagDownloadImageToDisk:
         // FIXME: Some day we should be able to do this from within WebCore. (Bug 117709)
-        m_client.downloadURL(m_context.hitTestResult().absoluteImageURL());
+        m_client->downloadURL(m_context.hitTestResult().absoluteImageURL());
         break;
     case ContextMenuItemTagCopyImageToClipboard:
         // FIXME: The Pasteboard class is not written yet
@@ -278,7 +344,7 @@ void ContextMenuController::contextMenuItemSelected(ContextMenuAction action, co
         break;
     case ContextMenuItemTagDownloadMediaToDisk:
         // FIXME: Some day we should be able to do this from within WebCore. (Bug 117709)
-        m_client.downloadURL(m_context.hitTestResult().absoluteMediaURL());
+        m_client->downloadURL(m_context.hitTestResult().absoluteMediaURL());
         break;
     case ContextMenuItemTagCopyMediaLinkToClipboard:
         frame->editor().copyURL(m_context.hitTestResult().absoluteMediaURL(), m_context.hitTestResult().textContent());
@@ -326,6 +392,12 @@ void ContextMenuController::contextMenuItemSelected(ContextMenuAction action, co
             page->setImageAnimationEnabled(false);
         break;
     }
+    case ContextMenuItemTagPlayAnimation:
+        m_context.hitTestResult().playAnimation();
+        break;
+    case ContextMenuItemTagPauseAnimation:
+        m_context.hitTestResult().pauseAnimation();
+        break;
 #endif // ENABLE(ACCESSIBILITY_ANIMATION_CONTROL)
     case ContextMenuItemTagCopy:
         frame->editor().copy();
@@ -391,7 +463,7 @@ void ContextMenuController::contextMenuItemSelected(ContextMenuAction action, co
         frame->editor().command("SelectAll"_s).execute();
         break;
     case ContextMenuItemTagInsertEmoji:
-        m_client.insertEmoji(*frame);
+        m_client->insertEmoji(*frame);
         break;
 #endif
     case ContextMenuItemTagSpellingGuess: {
@@ -424,14 +496,14 @@ void ContextMenuController::contextMenuItemSelected(ContextMenuAction action, co
         frame->editor().learnSpelling();
         break;
     case ContextMenuItemTagSearchWeb:
-        m_client.searchWithGoogle(frame);
+        m_client->searchWithGoogle(frame);
         break;
     case ContextMenuItemTagLookUpInDictionary:
         // FIXME: Some day we may be able to do this from within WebCore.
-        m_client.lookUpInDictionary(frame);
+        m_client->lookUpInDictionary(frame);
         break;
     case ContextMenuItemTagOpenLink:
-        if (Frame* targetFrame = m_context.hitTestResult().targetFrame()) {
+        if (auto* targetFrame = m_context.hitTestResult().targetFrame()) {
             ResourceRequest resourceRequest { m_context.hitTestResult().absoluteLinkURL(), frame->loader().outgoingReferrer() };
             FrameLoadRequest frameLoadRequest { *frame->document(), frame->document()->securityOrigin(), WTFMove(resourceRequest), { }, InitiatedByMainFrame::Unknown };
             frameLoadRequest.setNewFrameOpenerPolicy(NewFrameOpenerPolicy::Suppress);
@@ -458,11 +530,11 @@ void ContextMenuController::contextMenuItemSelected(ContextMenuAction action, co
         auto selectedRange = frame->selection().selection().toNormalizedRange();
         if (!selectedRange || selectedRange->collapsed())
             selectedRange = makeRangeSelectingNodeContents(document);
-        m_client.speak(plainText(*selectedRange));
+        m_client->speak(plainText(*selectedRange));
         break;
     }
     case ContextMenuItemTagStopSpeaking:
-        m_client.stopSpeaking();
+        m_client->stopSpeaking();
         break;
     case ContextMenuItemTagDefaultDirection:
         frame->editor().setBaseWritingDirection(WritingDirection::Natural);
@@ -484,7 +556,7 @@ void ContextMenuController::contextMenuItemSelected(ContextMenuAction action, co
         break;
 #if PLATFORM(COCOA)
     case ContextMenuItemTagSearchInSpotlight:
-        m_client.searchWithSpotlight();
+        m_client->searchWithSpotlight();
         break;
 #endif
     case ContextMenuItemTagShowSpellingPanel:
@@ -558,7 +630,7 @@ void ContextMenuController::contextMenuItemSelected(ContextMenuAction action, co
     case ContextMenuItemTagTranslate:
 #if HAVE(TRANSLATION_UI_SERVICES)
         if (RefPtr view = frame->view()) {
-            m_client.handleTranslation({
+            m_client->handleTranslation({
                 m_context.hitTestResult().selectedText(),
                 view->contentsToRootView(enclosingIntRect(frame->selection().selectionBounds())),
                 view->contentsToRootView(m_context.hitTestResult().roundedPointInInnerNodeFrame()),
@@ -843,8 +915,12 @@ void ContextMenuController::populate()
         contextMenuItemTagMediaPlay());
     ContextMenuItem MediaMute(ActionType, ContextMenuItemTagMediaMute, 
         contextMenuItemTagMediaMute());
+#if ENABLE(ACCESSIBILITY_ANIMATION_CONTROL)
     ContextMenuItem PlayAllAnimations(ActionType, ContextMenuItemTagPlayAllAnimations, contextMenuItemTagPlayAllAnimations());
     ContextMenuItem PauseAllAnimations(ActionType, ContextMenuItemTagPauseAllAnimations, contextMenuItemTagPauseAllAnimations());
+    ContextMenuItem PlayAnimation(ActionType, ContextMenuItemTagPlayAnimation, contextMenuItemTagPlayAnimation());
+    ContextMenuItem PauseAnimation(ActionType, ContextMenuItemTagPauseAnimation, contextMenuItemTagPauseAnimation());
+#endif // ENABLE(ACCESSIBILITY_ANIMATION_CONTROL)
 #if SUPPORTS_TOGGLE_SHOW_HIDE_MEDIA_CONTROLS
     ContextMenuItem ToggleMediaControls(ActionType, ContextMenuItemTagToggleMediaControls,
         contextMenuItemTagHideMediaControls());
@@ -962,6 +1038,18 @@ void ContextMenuController::populate()
 #endif
     };
 
+#if ENABLE(ACCESSIBILITY_ANIMATION_CONTROL)
+    auto canAddAnimationControls = [&] () -> bool {
+        if (!frame->page() || !frame->page()->settings().imageAnimationControlEnabled())
+            return false;
+
+        auto* autoplayAnimatedImagesFunction = _AXSReduceMotionAutoplayAnimatedImagesEnabledPtr();
+        // Only show these controls if autoplay of animated images has been disabled.
+        bool systemAllowsAnimationControls = autoplayAnimatedImagesFunction && !autoplayAnimatedImagesFunction();
+        return systemAllowsAnimationControls || frame->page()->settings().allowAnimationControlsOverride();
+    };
+#endif
+
     auto selectedText = m_context.hitTestResult().selectedText();
     m_context.setSelectedText(selectedText);
 
@@ -986,19 +1074,29 @@ void ContextMenuController::populate()
             appendItem(DownloadImageItem, m_contextMenu.get());
 
             auto image = m_context.hitTestResult().image();
-            if (imageURL.isLocalFile() || image) {
+            if (imageURL.protocolIsFile() || image) {
                 appendItem(CopyImageItem, m_contextMenu.get());
 
                 if (image && !image->isAnimated()) {
 #if ENABLE(IMAGE_ANALYSIS_ENHANCEMENTS)
-                    if (m_client.supportsCopySubject())
+                    if (m_client->supportsCopySubject())
                         appendItem(copySubjectItem, m_contextMenu.get());
 #endif
 #if ENABLE(IMAGE_ANALYSIS)
-                    if (m_client.supportsLookUpInImages())
+                    if (m_client->supportsLookUpInImages())
                         appendItem(LookUpImageItem, m_contextMenu.get());
 #endif
                 }
+
+#if ENABLE(ACCESSIBILITY_ANIMATION_CONTROL)
+                if (image && image->isAnimated() && canAddAnimationControls()) {
+                    appendItem(*separatorItem(), m_contextMenu.get());
+                    if (m_context.hitTestResult().isAnimating())
+                        appendItem(PauseAnimation, m_contextMenu.get());
+                    else
+                        appendItem(PlayAnimation, m_contextMenu.get());
+                }
+#endif // ENABLE(ACCESSIBILITY_ANIMATION_CONTROL)
             }
 #if PLATFORM(GTK)
             appendItem(CopyImageUrlItem, m_contextMenu.get());
@@ -1006,7 +1104,7 @@ void ContextMenuController::populate()
         }
 
 #if ENABLE(ACCESSIBILITY_ANIMATION_CONTROL)
-        if (frame->page() && frame->page()->settings().imageAnimationControlEnabled()) {
+        if (canAddAnimationControls()) {
             if (frame->page()->imageAnimationEnabled())
                 appendItem(PauseAllAnimations, m_contextMenu.get());
             else
@@ -1290,7 +1388,7 @@ void ContextMenuController::addDebuggingItems()
     if (!node)
         return;
 
-    Frame* frame = node->document().frame();
+    auto* frame = node->document().frame();
     if (!frame)
         return;
 
@@ -1323,7 +1421,7 @@ void ContextMenuController::checkOrEnableIfNeeded(ContextMenuItem& item) const
     if (item.type() == SeparatorType)
         return;
     
-    Frame* frame = m_context.hitTestResult().innerNonSharedNode()->document().frame();
+    auto* frame = m_context.hitTestResult().innerNonSharedNode()->document().frame();
     if (!frame)
         return;
 
@@ -1488,7 +1586,7 @@ void ContextMenuController::checkOrEnableIfNeeded(ContextMenuItem& item) const
             shouldCheck = frame->editor().isAutomaticTextReplacementEnabled();
             break;
         case ContextMenuItemTagStopSpeaking:
-            shouldEnable = m_client.isSpeaking();
+            shouldEnable = m_client->isSpeaking();
             break;
 #else // PLATFORM(COCOA) ends here
         case ContextMenuItemTagStopSpeaking:
@@ -1530,7 +1628,7 @@ void ContextMenuController::checkOrEnableIfNeeded(ContextMenuItem& item) const
             break;
         case ContextMenuItemTagDownloadImageToDisk:
 #if PLATFORM(MAC)
-            if (m_context.hitTestResult().absoluteImageURL().protocolIs("file"_s))
+            if (m_context.hitTestResult().absoluteImageURL().protocolIsFile())
                 shouldEnable = false;
 #endif
             break;
@@ -1545,7 +1643,7 @@ void ContextMenuController::checkOrEnableIfNeeded(ContextMenuItem& item) const
                 item.setTitle(contextMenuItemTagDownloadVideoToDisk());
             else
                 item.setTitle(contextMenuItemTagDownloadAudioToDisk());
-            if (m_context.hitTestResult().absoluteImageURL().protocolIs("file"_s))
+            if (m_context.hitTestResult().absoluteImageURL().protocolIsFile())
                 shouldEnable = false;
             break;
         case ContextMenuItemTagCopyMediaLinkToClipboard:
@@ -1555,10 +1653,24 @@ void ContextMenuController::checkOrEnableIfNeeded(ContextMenuItem& item) const
                 item.setTitle(contextMenuItemTagCopyAudioLinkToClipboard());
             break;
         case ContextMenuItemTagPlayAllAnimations:
+#if ENABLE(ACCESSIBILITY_ANIMATION_CONTROL)
             item.setTitle(contextMenuItemTagPlayAllAnimations());
+#endif
             break;
         case ContextMenuItemTagPauseAllAnimations:
+#if ENABLE(ACCESSIBILITY_ANIMATION_CONTROL)
             item.setTitle(contextMenuItemTagPauseAllAnimations());
+#endif
+            break;
+        case ContextMenuItemTagPlayAnimation:
+#if ENABLE(ACCESSIBILITY_ANIMATION_CONTROL)
+            item.setTitle(contextMenuItemTagPlayAnimation());
+#endif
+            break;
+        case ContextMenuItemTagPauseAnimation:
+#if ENABLE(ACCESSIBILITY_ANIMATION_CONTROL)
+            item.setTitle(contextMenuItemTagPauseAnimation());
+#endif
             break;
         case ContextMenuItemTagToggleMediaControls:
 #if SUPPORTS_TOGGLE_SHOW_HIDE_MEDIA_CONTROLS
@@ -1645,7 +1757,7 @@ void ContextMenuController::checkOrEnableIfNeeded(ContextMenuItem& item) const
 
 #if USE(ACCESSIBILITY_CONTEXT_MENUS)
 
-void ContextMenuController::showContextMenuAt(Frame& frame, const IntPoint& clickPoint)
+void ContextMenuController::showContextMenuAt(LocalFrame& frame, const IntPoint& clickPoint)
 {
     clearContextMenu();
     
@@ -1654,7 +1766,7 @@ void ContextMenuController::showContextMenuAt(Frame& frame, const IntPoint& clic
     frame.eventHandler().handleMousePressEvent(mouseEvent);
     bool handled = frame.eventHandler().sendContextMenuEvent(mouseEvent);
     if (handled)
-        m_client.showContextMenu();
+        m_client->showContextMenu();
 }
 
 #endif
@@ -1665,14 +1777,14 @@ void ContextMenuController::showImageControlsMenu(Event& event)
 {
     clearContextMenu();
     handleContextMenuEvent(event);
-    m_client.showContextMenu();
+    m_client->showContextMenu();
 }
 
 #endif
 
 #if ENABLE(PDFJS)
 
-void ContextMenuController::performPDFJSAction(Frame& frame, const String& action)
+void ContextMenuController::performPDFJSAction(LocalFrame& frame, const String& action)
 {
     auto* pdfDocument = dynamicDowncast<PDFDocument>(frame.ownerElement()->document());
     if (pdfDocument)
