@@ -26,18 +26,18 @@
 #include "config.h"
 #include "PageOverlay.h"
 
-#include "FrameView.h"
 #include "GraphicsContext.h"
-#include "MainFrame.h"
+#include "LocalFrame.h"
+#include "LocalFrameView.h"
+#include "Logging.h"
 #include "Page.h"
 #include "PageOverlayController.h"
 #include "PlatformMouseEvent.h"
 #include "ScrollbarTheme.h"
-#include <wtf/CurrentTime.h>
 
 namespace WebCore {
 
-static const double fadeAnimationDuration = 0.2;
+static const Seconds fadeAnimationDuration { 200_ms };
 static const double fadeAnimationFrameRate = 30;
 
 static PageOverlay::PageOverlayID generatePageOverlayID()
@@ -46,30 +46,29 @@ static PageOverlay::PageOverlayID generatePageOverlayID()
     return ++pageOverlayID;
 }
 
-Ref<PageOverlay> PageOverlay::create(Client& client, OverlayType overlayType)
+Ref<PageOverlay> PageOverlay::create(Client& client, OverlayType overlayType, AlwaysTileOverlayLayer alwaysTileOverlayLayer)
 {
-    return adoptRef(*new PageOverlay(client, overlayType));
+    return adoptRef(*new PageOverlay(client, overlayType, alwaysTileOverlayLayer));
 }
 
-PageOverlay::PageOverlay(Client& client, OverlayType overlayType)
+PageOverlay::PageOverlay(Client& client, OverlayType overlayType, AlwaysTileOverlayLayer alwaysTileOverlayLayer)
     : m_client(client)
     , m_fadeAnimationTimer(*this, &PageOverlay::fadeAnimationTimerFired)
     , m_fadeAnimationDuration(fadeAnimationDuration)
     , m_needsSynchronousScrolling(overlayType == OverlayType::View)
     , m_overlayType(overlayType)
+    , m_alwaysTileOverlayLayer(alwaysTileOverlayLayer)
     , m_pageOverlayID(generatePageOverlayID())
 {
 }
 
-PageOverlay::~PageOverlay()
-{
-}
+PageOverlay::~PageOverlay() = default;
 
 PageOverlayController* PageOverlay::controller() const
 {
     if (!m_page)
         return nullptr;
-    return &m_page->mainFrame().pageOverlayController();
+    return &m_page->pageOverlayController();
 }
 
 IntRect PageOverlay::bounds() const
@@ -77,7 +76,11 @@ IntRect PageOverlay::bounds() const
     if (!m_overrideFrame.isEmpty())
         return { { }, m_overrideFrame.size() };
 
-    FrameView* frameView = m_page->mainFrame().view();
+    auto* localMainFrame = dynamicDowncast<LocalFrame>(m_page->mainFrame());
+    if (!localMainFrame)
+        return IntRect();
+
+    auto* frameView = localMainFrame->view();
 
     if (!frameView)
         return IntRect();
@@ -129,7 +132,8 @@ IntSize PageOverlay::viewToOverlayOffset() const
         return IntSize();
 
     case OverlayType::Document: {
-        FrameView* frameView = m_page->mainFrame().view();
+        auto* localMainFrame = dynamicDowncast<LocalFrame>(m_page->mainFrame());
+        auto* frameView = localMainFrame ? localMainFrame->view() : nullptr;
         return frameView ? toIntSize(frameView->viewToContents(IntPoint())) : IntSize();
     }
     }
@@ -180,7 +184,8 @@ void PageOverlay::drawRect(GraphicsContext& graphicsContext, const IntRect& dirt
     GraphicsContextStateSaver stateSaver(graphicsContext);
 
     if (m_overlayType == PageOverlay::OverlayType::Document) {
-        if (FrameView* frameView = m_page->mainFrame().view()) {
+        auto* localMainFrame = dynamicDowncast<LocalFrame>(m_page->mainFrame());
+        if (auto* frameView = localMainFrame ? localMainFrame->view() : nullptr) {
             auto offset = frameView->scrollOrigin();
             graphicsContext.translate(toFloatSize(offset));
             paintRect.moveBy(-offset);
@@ -194,8 +199,9 @@ bool PageOverlay::mouseEvent(const PlatformMouseEvent& mouseEvent)
 {
     IntPoint mousePositionInOverlayCoordinates(mouseEvent.position());
 
-    if (m_overlayType == PageOverlay::OverlayType::Document)
-        mousePositionInOverlayCoordinates = m_page->mainFrame().view()->windowToContents(mousePositionInOverlayCoordinates);
+    auto* localMainFrame = dynamicDowncast<LocalFrame>(m_page->mainFrame());
+    if (m_overlayType == PageOverlay::OverlayType::Document && localMainFrame)
+        mousePositionInOverlayCoordinates = localMainFrame->view()->windowToContents(mousePositionInOverlayCoordinates);
     mousePositionInOverlayCoordinates.moveBy(-frame().location());
 
     // Ignore events outside the bounds.
@@ -205,7 +211,7 @@ bool PageOverlay::mouseEvent(const PlatformMouseEvent& mouseEvent)
     return m_client.mouseEvent(*this, mouseEvent);
 }
 
-void PageOverlay::didScrollFrame(Frame& frame)
+void PageOverlay::didScrollFrame(LocalFrame& frame)
 {
     m_client.didScrollFrame(*this, frame);
 }
@@ -227,6 +233,9 @@ Vector<String> PageOverlay::copyAccessibilityAttributeNames(bool parameterizedNa
 
 void PageOverlay::startFadeInAnimation()
 {
+    if (m_fadeAnimationType == FadeInAnimation && m_fadeAnimationTimer.isActive())
+        return;
+
     m_fractionFadedIn = 0;
     m_fadeAnimationType = FadeInAnimation;
 
@@ -235,6 +244,9 @@ void PageOverlay::startFadeInAnimation()
 
 void PageOverlay::startFadeOutAnimation()
 {
+    if (m_fadeAnimationType == FadeOutAnimation && m_fadeAnimationTimer.isActive())
+        return;
+
     m_fractionFadedIn = 1;
     m_fadeAnimationType = FadeOutAnimation;
 
@@ -249,13 +261,19 @@ void PageOverlay::stopFadeOutAnimation()
 
 void PageOverlay::startFadeAnimation()
 {
-    m_fadeAnimationStartTime = currentTime();
-    m_fadeAnimationTimer.startRepeating(1 / fadeAnimationFrameRate);
+    ASSERT(m_page);
+    if (!m_page)
+        RELEASE_LOG_FAULT(Animations, "PageOverlay::startFadeAnimation() was called on a PageOverlay without a page");
+    m_fadeAnimationStartTime = WallTime::now();
+    m_fadeAnimationTimer.startRepeating(1_s / fadeAnimationFrameRate);
 }
 
 void PageOverlay::fadeAnimationTimerFired()
 {
-    float animationProgress = (currentTime() - m_fadeAnimationStartTime) / m_fadeAnimationDuration;
+    auto controller = this->controller();
+    ASSERT(controller);
+
+    float animationProgress = (WallTime::now() - m_fadeAnimationStartTime) / m_fadeAnimationDuration;
 
     if (animationProgress >= 1.0)
         animationProgress = 1.0;
@@ -264,7 +282,9 @@ void PageOverlay::fadeAnimationTimerFired()
     float fadeAnimationValue = sine * sine;
 
     m_fractionFadedIn = (m_fadeAnimationType == FadeInAnimation) ? fadeAnimationValue : 1 - fadeAnimationValue;
-    controller()->setPageOverlayOpacity(*this, m_fractionFadedIn);
+
+    if (controller)
+        controller->setPageOverlayOpacity(*this, m_fractionFadedIn);
 
     if (animationProgress == 1.0) {
         m_fadeAnimationTimer.stop();
@@ -273,8 +293,8 @@ void PageOverlay::fadeAnimationTimerFired()
         m_fadeAnimationType = NoAnimation;
 
         // If this was a fade out, uninstall the page overlay.
-        if (wasFadingOut)
-            controller()->uninstallPageOverlay(this, PageOverlay::FadeMode::DoNotFade);
+        if (wasFadingOut && controller)
+            controller->uninstallPageOverlay(*this, PageOverlay::FadeMode::DoNotFade);
     }
 }
 
