@@ -26,6 +26,8 @@
 #include "config.h"
 #include "ThemeAdwaita.h"
 
+#if USE(THEME_ADWAITA)
+
 #include "Color.h"
 #include "ColorBlending.h"
 #include "ControlStates.h"
@@ -34,6 +36,11 @@
 #include "LengthSize.h"
 #include <wtf/NeverDestroyed.h>
 
+#if PLATFORM(GTK)
+#include <gtk/gtk.h>
+#include <wtf/glib/GUniquePtr.h>
+#endif
+
 namespace WebCore {
 
 static const double focusRingOpacity = 0.8; // Keep in sync with focusRingOpacity in RenderThemeAdwaita.
@@ -80,6 +87,49 @@ Theme& Theme::singleton()
     return theme;
 }
 
+ThemeAdwaita::ThemeAdwaita()
+{
+#if PLATFORM(GTK)
+    if (auto* settings = gtk_settings_get_default()) {
+        refreshGtkSettings();
+
+        // Note that Theme is NeverDestroy'd so the destructor will never be called to disconnect this.
+        g_signal_connect_swapped(G_OBJECT(settings), "notify::gtk-enable-animations", G_CALLBACK(+[](ThemeAdwaita* theme, GParamSpec*, GObject*) {
+            theme->refreshGtkSettings();
+        }), this);
+#if !USE(GTK4)
+        g_signal_connect_swapped(G_OBJECT(settings), "notify::gtk-theme-name", G_CALLBACK(+[](ThemeAdwaita* theme, GParamSpec*, GObject*) {
+            theme->refreshGtkSettings();
+        }), this);
+#endif // !USE(GTK4)
+    }
+
+#endif // PLATFORM(GTK)
+}
+
+#if PLATFORM(GTK)
+
+void ThemeAdwaita::refreshGtkSettings()
+{
+    if (auto* settings = gtk_settings_get_default()) {
+        gboolean enableAnimations;
+        g_object_get(settings, "gtk-enable-animations", &enableAnimations, nullptr);
+        m_prefersReducedMotion = !enableAnimations;
+
+        // For high contrast in GTK3 we can rely on the theme name and be accurate most of the time.
+        // However whether or not high-contrast is enabled is also stored in GSettings/xdg-desktop-portal.
+        // We could rely on libadwaita, dynamically, to re-use its logic but, no matter how we do it, the setting
+        // has to be proxied over from the UI process different than how GtkSettings is.
+#if !USE(GTK4)
+        GUniqueOutPtr<char> gtkThemeName;
+        g_object_get(settings, "gtk-theme-name", &gtkThemeName.outPtr(), nullptr);
+        m_prefersContrast = !g_strcmp0(gtkThemeName.get(), "HighContrast") || !g_strcmp0(gtkThemeName.get(), "HighContrastInverse");
+#endif // !USE(GTK4)
+    }
+}
+
+#endif // PLATFORM(GTK)
+
 Color ThemeAdwaita::focusColor(const Color& accentColor)
 {
     return accentColor.colorWithAlphaMultipliedBy(focusRingOpacity);
@@ -545,4 +595,20 @@ Color ThemeAdwaita::accentColor()
     return m_accentColor;
 }
 
+bool ThemeAdwaita::userPrefersReducedMotion() const
+{
+    return m_prefersReducedMotion;
+}
+
+bool ThemeAdwaita::userPrefersContrast() const
+{
+#if !USE(GTK4)
+    return m_prefersContrast;
+#else
+    return false;
+#endif
+}
+
 } // namespace WebCore
+
+#endif // USE(THEME_ADWAITA)
