@@ -26,9 +26,12 @@
 #include "config.h"
 #include "TimingFunction.h"
 
+#include "CSSParserContext.h"
+#include "CSSPrimitiveValue.h"
 #include "CSSTimingFunctionValue.h"
+#include "CSSValueKeywords.h"
+#include "MutableStyleProperties.h"
 #include "SpringSolver.h"
-#include "StyleProperties.h"
 #include "UnitBezier.h"
 #include <wtf/text/StringConcatenateNumbers.h>
 #include <wtf/text/TextStream.h>
@@ -157,7 +160,7 @@ ExceptionOr<RefPtr<TimingFunction>> TimingFunction::createFromCSSText(const Stri
 RefPtr<TimingFunction> TimingFunction::createFromCSSValue(const CSSValue& value)
 {
     if (is<CSSPrimitiveValue>(value)) {
-        switch (downcast<CSSPrimitiveValue>(value).valueID()) {
+        switch (value.valueID()) {
         case CSSValueLinear:
             return LinearTimingFunction::create();
         case CSSValueEase:
