@@ -11,10 +11,10 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  *
- * THIS SOFTWARE IS PROVIDED BY APPLE COMPUTER, INC. ``AS IS'' AND ANY
+ * THIS SOFTWARE IS PROVIDED BY APPLE INC. ``AS IS'' AND ANY
  * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
- * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL APPLE COMPUTER, INC. OR
+ * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL APPLE INC. OR
  * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
  * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
@@ -32,7 +32,7 @@
 #include <algorithm>
 #include <math.h>
 #include <wtf/MathExtras.h>
-#include <wtf/PrintStream.h>
+#include <wtf/text/TextStream.h>
 
 namespace WebCore {
 
@@ -54,9 +54,15 @@ bool FloatRect::isExpressibleAsIntRect() const
         && isWithinIntRange(maxX()) && isWithinIntRange(maxY());
 }
 
+bool FloatRect::inclusivelyIntersects(const FloatRect& other) const
+{
+    return width() >= 0 && height() >= 0 && other.width() >= 0 && other.height() >= 0
+        && x() <= other.maxX() && other.x() <= maxX() && y() <= other.maxY() && other.y() <= maxY();
+}
+
 bool FloatRect::intersects(const FloatRect& other) const
 {
-    // Checking emptiness handles negative widths as well as zero.
+    // Checking emptiness handles negative widths and heights as well as zero.
     return !isEmpty() && !other.isEmpty()
         && x() < other.maxX() && other.x() < maxX()
         && y() < other.maxY() && other.y() < maxY();
@@ -93,6 +99,25 @@ void FloatRect::intersect(const FloatRect& other)
     setLocationAndSizeFromEdges(l, t, r, b);
 }
 
+bool FloatRect::edgeInclusiveIntersect(const FloatRect& other)
+{
+    FloatPoint newLocation(std::max(x(), other.x()), std::max(y(), other.y()));
+    FloatPoint newMaxPoint(std::min(maxX(), other.maxX()), std::min(maxY(), other.maxY()));
+
+    bool intersects = true;
+
+    // Return a clean empty rectangle for non-intersecting cases.
+    if (newLocation.x() > newMaxPoint.x() || newLocation.y() > newMaxPoint.y()) {
+        newLocation = { };
+        newMaxPoint = { };
+        intersects = false;
+    }
+
+    m_location = newLocation;
+    m_size = newMaxPoint - newLocation;
+    return intersects;
+}
+
 void FloatRect::unite(const FloatRect& other)
 {
     // Handle empty special cases first.
@@ -147,17 +172,6 @@ void FloatRect::scale(float sx, float sy)
     m_size.setHeight(height() * sy);
 }
 
-FloatRect unionRect(const Vector<FloatRect>& rects)
-{
-    FloatRect result;
-
-    size_t count = rects.size();
-    for (size_t i = 0; i < count; ++i)
-        result.unite(rects[i]);
-
-    return result;
-}
-
 void FloatRect::fitToPoints(const FloatPoint& p0, const FloatPoint& p1)
 {
     float left = std::min(p0.x(), p1.x());
@@ -217,22 +231,38 @@ void FloatRect::fitToPoints(const FloatPoint& p0, const FloatPoint& p1, const Fl
     setLocationAndSizeFromEdges(left, top, right, bottom);
 }
 
-IntRect enclosingIntRect(const FloatRect& rect)
+FloatRect normalizeRect(const FloatRect& rect)
 {
-    IntPoint location = flooredIntPoint(rect.minXMinYCorner());
-    IntPoint maxPoint = ceiledIntPoint(rect.maxXMaxYCorner());
-
-    return IntRect(location, maxPoint - location);
+    return FloatRect(std::min(rect.x(), rect.maxX()),
+        std::min(rect.y(), rect.maxY()),
+        std::max(rect.width(), -rect.width()),
+        std::max(rect.height(), -rect.height()));
 }
 
-IntRect enclosedIntRect(const FloatRect& rect)
+FloatRect encloseRectToDevicePixels(const FloatRect& rect, float deviceScaleFactor)
 {
-    IntPoint location = ceiledIntPoint(rect.minXMinYCorner());
-    IntPoint maxPoint = flooredIntPoint(rect.maxXMaxYCorner());
-    IntSize size = maxPoint - location;
-    size.clampNegativeToZero();
+    FloatPoint location = floorPointToDevicePixels(rect.minXMinYCorner(), deviceScaleFactor);
+    FloatPoint maxPoint = ceilPointToDevicePixels(rect.maxXMaxYCorner(), deviceScaleFactor);
+    return FloatRect(location, maxPoint - location);
+}
 
-    return IntRect(location, size);
+IntRect enclosingIntRect(const FloatRect& rect)
+{
+    FloatPoint location = flooredIntPoint(rect.minXMinYCorner());
+    FloatPoint maxPoint = ceiledIntPoint(rect.maxXMaxYCorner());
+    return IntRect(IntPoint(location), IntSize(maxPoint - location));
+}
+
+IntRect enclosingIntRectPreservingEmptyRects(const FloatRect& rect)
+{
+    // Empty rects with fractional x, y values turn into non-empty rects when converting to enclosing.
+    // We want to ensure that empty rects stay empty after the conversion, since some callers
+    // prefer this behavior.
+    FloatPoint location = flooredIntPoint(rect.minXMinYCorner());
+    if (rect.isEmpty())
+        return IntRect(IntPoint(location), { });
+    FloatPoint maxPoint = ceiledIntPoint(rect.maxXMaxYCorner());
+    return IntRect(IntPoint(location), IntSize(maxPoint - location));
 }
 
 IntRect roundedIntRect(const FloatRect& rect)
@@ -240,21 +270,30 @@ IntRect roundedIntRect(const FloatRect& rect)
     return IntRect(roundedIntPoint(rect.location()), roundedIntSize(rect.size()));
 }
 
-FloatRect mapRect(const FloatRect& r, const FloatRect& srcRect, const FloatRect& destRect)
+TextStream& operator<<(TextStream& ts, const FloatRect &r)
 {
-    if (srcRect.width() == 0 || srcRect.height() == 0)
-        return FloatRect();
+    if (ts.hasFormattingFlag(TextStream::Formatting::SVGStyleRect)) {
+        // FIXME: callers should use the NumberRespectingIntegers flag.
+        return ts << "at (" << TextStream::FormatNumberRespectingIntegers(r.x()) << "," << TextStream::FormatNumberRespectingIntegers(r.y())
+            << ") size " << TextStream::FormatNumberRespectingIntegers(r.width()) << "x" << TextStream::FormatNumberRespectingIntegers(r.height());
+    }
 
-    float widthScale = destRect.width() / srcRect.width();
-    float heightScale = destRect.height() / srcRect.height();
-    return FloatRect(destRect.x() + (r.x() - srcRect.x()) * widthScale,
-                     destRect.y() + (r.y() - srcRect.y()) * heightScale,
-                     r.width() * widthScale, r.height() * heightScale);
+    return ts << r.location() << " " << r.size();
 }
 
-void FloatRect::dump(PrintStream& out) const
+Ref<JSON::Object> FloatRect::toJSONObject() const
 {
-    out.print(location(), " ", size());
+    auto object = JSON::Object::create();
+
+    object->setObject("location"_s, m_location.toJSONObject());
+    object->setObject("size"_s, m_size.toJSONObject());
+
+    return object;
+}
+
+String FloatRect::toJSONString() const
+{
+    return toJSONObject()->toJSONString();
 }
 
 }
