@@ -74,12 +74,20 @@
 #include "EpoxyEGL.h"
 #else
 #include <EGL/egl.h>
+#include <EGL/eglext.h>
 #endif
-#include "GLContextEGL.h"
 #include <wtf/HashSet.h>
 #include <wtf/NeverDestroyed.h>
 #endif
 
+#if USE(EGL) && USE(GBM)
+#include "GBMDevice.h"
+#include <xf86drm.h>
+#ifndef EGL_DRM_RENDER_NODE_FILE_EXT
+#define EGL_DRM_RENDER_NODE_FILE_EXT 0x3377
+#endif
+#endif
+
 #if USE(ATSPI)
 #include <wtf/glib/GUniquePtr.h>
 #endif
@@ -88,6 +96,21 @@
 #include <wtf/glib/GRefPtr.h>
 #endif
 
+#if USE(EGL) && !USE(LIBEPOXY)
+#if !defined(PFNEGLCREATEIMAGEPROC)
+typedef EGLImage (*PFNEGLCREATEIMAGEPROC) (EGLDisplay, EGLContext, EGLenum, EGLClientBuffer, const EGLAttrib*);
+#endif
+#if !defined(PFNEGLDESTROYIMAGEPROC)
+typedef EGLBoolean (*PFNEGLDESTROYIMAGEPROC) (EGLDisplay, EGLImage);
+#endif
+#if !defined(PFNEGLCREATEIMAGEKHRPROC)
+typedef EGLImageKHR (*PFNEGLCREATEIMAGEKHRPROC) (EGLDisplay, EGLContext, EGLenum target, EGLClientBuffer, const EGLint* attribList);
+#endif
+#if !defined(PFNEGLDESTROYIMAGEKHRPROC)
+typedef EGLBoolean (*PFNEGLDESTROYIMAGEKHRPROC) (EGLDisplay, EGLImageKHR);
+#endif
+#endif
+
 namespace WebCore {
 
 std::unique_ptr<PlatformDisplay> PlatformDisplay::createPlatformDisplay()
@@ -192,46 +215,53 @@ PlatformDisplay::PlatformDisplay(GdkDisplay* display)
 
 void PlatformDisplay::sharedDisplayDidClose()
 {
-#if USE(EGL) || USE(GLX)
-    clearSharingGLContext();
+#if USE(EGL)
+    terminateEGLDisplay();
 #endif
 }
 #endif
 
-PlatformDisplay::~PlatformDisplay()
-{
-#if USE(EGL) && !PLATFORM(WIN)
-    ASSERT(m_eglDisplay == EGL_NO_DISPLAY);
-#endif
-#if PLATFORM(GTK)
-    if (m_sharedDisplay)
-        g_signal_handlers_disconnect_by_data(m_sharedDisplay.get(), this);
-#endif
-    if (s_sharedDisplayForCompositing == this)
-        s_sharedDisplayForCompositing = nullptr;
-}
-
-#if USE(EGL) || USE(GLX)
-GLContext* PlatformDisplay::sharingGLContext()
-{
-    if (!m_sharingGLContext)
-        m_sharingGLContext = GLContext::createSharingContext(*this);
-    return m_sharingGLContext.get();
-}
-
-void PlatformDisplay::clearSharingGLContext()
-{
-    m_sharingGLContext = nullptr;
-}
-#endif
-
 #if USE(EGL)
 static HashSet<PlatformDisplay*>& eglDisplays()
 {
     static NeverDestroyed<HashSet<PlatformDisplay*>> displays;
     return displays;
 }
+#endif
 
+PlatformDisplay::~PlatformDisplay()
+{
+#if USE(EGL)
+    if (m_eglDisplay != EGL_NO_DISPLAY && eglDisplays().remove(this))
+        terminateEGLDisplay();
+#endif
+
+#if PLATFORM(GTK)
+    if (m_sharedDisplay)
+        g_signal_handlers_disconnect_by_data(m_sharedDisplay.get(), this);
+#endif
+    if (s_sharedDisplayForCompositing == this)
+        s_sharedDisplayForCompositing = nullptr;
+}
+
+#if USE(EGL)
+GLContext* PlatformDisplay::sharingGLContext()
+{
+    if (!m_sharingGLContext)
+        m_sharingGLContext = GLContext::createSharing(*this);
+    return m_sharingGLContext.get();
+}
+
+void PlatformDisplay::clearSharingGLContext()
+{
+#if ENABLE(VIDEO) && USE(GSTREAMER_GL)
+    m_gstGLContext = nullptr;
+#endif
+    m_sharingGLContext = nullptr;
+}
+#endif
+
+#if USE(EGL)
 EGLDisplay PlatformDisplay::eglDisplay() const
 {
     if (!m_eglDisplayInitialized)
@@ -247,6 +277,13 @@ bool PlatformDisplay::eglCheckVersion(int major, int minor) const
     return (m_eglMajorVersion > major) || ((m_eglMajorVersion == major) && (m_eglMinorVersion >= minor));
 }
 
+const PlatformDisplay::EGLExtensions& PlatformDisplay::eglExtensions() const
+{
+    if (!m_eglDisplayInitialized)
+        const_cast<PlatformDisplay*>(this)->initializeEGLDisplay();
+    return m_eglExtensions;
+}
+
 void PlatformDisplay::initializeEGLDisplay()
 {
     m_eglDisplayInitialized = true;
@@ -254,14 +291,14 @@ void PlatformDisplay::initializeEGLDisplay()
     if (m_eglDisplay == EGL_NO_DISPLAY) {
         m_eglDisplay = eglGetDisplay(EGL_DEFAULT_DISPLAY);
         if (m_eglDisplay == EGL_NO_DISPLAY) {
-            WTFLogAlways("Cannot get default EGL display: %s\n", GLContextEGL::lastErrorString());
+            WTFLogAlways("Cannot get default EGL display: %s\n", GLContext::lastErrorString());
             return;
         }
     }
 
     EGLint majorVersion, minorVersion;
     if (eglInitialize(m_eglDisplay, &majorVersion, &minorVersion) == EGL_FALSE) {
-        WTFLogAlways("EGLDisplay Initialization failed: %s\n", GLContextEGL::lastErrorString());
+        WTFLogAlways("EGLDisplay Initialization failed: %s\n", GLContext::lastErrorString());
         terminateEGLDisplay();
         return;
     }
@@ -281,10 +318,14 @@ void PlatformDisplay::initializeEGLDisplay()
             };
 
         m_eglExtensions.KHR_image_base = findExtension("EGL_KHR_image_base"_s);
+        m_eglExtensions.KHR_surfaceless_context = findExtension("EGL_KHR_surfaceless_context"_s);
         m_eglExtensions.EXT_image_dma_buf_import = findExtension("EGL_EXT_image_dma_buf_import"_s);
         m_eglExtensions.EXT_image_dma_buf_import_modifiers = findExtension("EGL_EXT_image_dma_buf_import_modifiers"_s);
     }
 
+    if (!m_eglDisplayOwned)
+        return;
+
     eglDisplays().add(this);
 
 #if !PLATFORM(WIN)
@@ -313,17 +354,162 @@ void PlatformDisplay::terminateEGLDisplay()
 {
 #if ENABLE(VIDEO) && USE(GSTREAMER_GL)
     m_gstGLDisplay = nullptr;
-    m_gstGLContext = nullptr;
 #endif
     clearSharingGLContext();
     ASSERT(m_eglDisplayInitialized);
     if (m_eglDisplay == EGL_NO_DISPLAY)
         return;
-    eglMakeCurrent(m_eglDisplay, EGL_NO_SURFACE, EGL_NO_SURFACE, EGL_NO_CONTEXT);
-    eglTerminate(m_eglDisplay);
+
+    if (m_eglDisplayOwned) {
+        eglMakeCurrent(m_eglDisplay, EGL_NO_SURFACE, EGL_NO_SURFACE, EGL_NO_CONTEXT);
+        eglTerminate(m_eglDisplay);
+    }
     m_eglDisplay = EGL_NO_DISPLAY;
 }
 
+EGLImage PlatformDisplay::createEGLImage(EGLContext context, EGLenum target, EGLClientBuffer clientBuffer, const Vector<EGLAttrib>& attributes) const
+{
+    if (eglCheckVersion(1, 5)) {
+#if USE(LIBEPOXY)
+        return eglCreateImage(m_eglDisplay, context, target, clientBuffer, attributes.isEmpty() ? nullptr : attributes.data());
+#else
+        static PFNEGLCREATEIMAGEPROC s_eglCreateImage = reinterpret_cast<PFNEGLCREATEIMAGEPROC>(eglGetProcAddress("eglCreateImage"));
+        if (s_eglCreateImage)
+            return s_eglCreateImage(m_eglDisplay, context, target, clientBuffer, attributes.isEmpty() ? nullptr : attributes.data());
+        return EGL_NO_IMAGE;
+#endif
+    }
+
+    if (!m_eglExtensions.KHR_image_base)
+        return EGL_NO_IMAGE;
+
+    Vector<EGLint> intAttributes = attributes.map<Vector<EGLint>>([] (EGLAttrib value) {
+        return value;
+    });
+#if USE(LIBEPOXY)
+    return eglCreateImageKHR(m_eglDisplay, context, target, clientBuffer, intAttributes.isEmpty() ? nullptr : intAttributes.data());
+#else
+    static PFNEGLCREATEIMAGEKHRPROC s_eglCreateImageKHR = reinterpret_cast<PFNEGLCREATEIMAGEKHRPROC>(eglGetProcAddress("eglCreateImageKHR"));
+    if (s_eglCreateImageKHR)
+        return s_eglCreateImageKHR(m_eglDisplay, context, target, clientBuffer, intAttributes.isEmpty() ? nullptr : intAttributes.data());
+    return EGL_NO_IMAGE_KHR;
+#endif
+}
+
+bool PlatformDisplay::destroyEGLImage(EGLImage image) const
+{
+    if (eglCheckVersion(1, 5)) {
+#if USE(LIBEPOXY)
+        return eglDestroyImage(m_eglDisplay, image);
+#else
+        static PFNEGLDESTROYIMAGEPROC s_eglDestroyImage = reinterpret_cast<PFNEGLDESTROYIMAGEPROC>(eglGetProcAddress("eglDestroyImage"));
+        if (s_eglDestroyImage)
+            return s_eglDestroyImage(m_eglDisplay, image);
+        return false;
+#endif
+    }
+
+    if (!m_eglExtensions.KHR_image_base)
+        return false;
+
+#if USE(LIBEPOXY)
+    return eglDestroyImageKHR(m_eglDisplay, image);
+#else
+    static PFNEGLDESTROYIMAGEKHRPROC s_eglDestroyImageKHR = reinterpret_cast<PFNEGLDESTROYIMAGEKHRPROC>(eglGetProcAddress("eglDestroyImageKHR"));
+    if (s_eglDestroyImageKHR)
+        return s_eglDestroyImageKHR(m_eglDisplay, image);
+    return false;
+#endif
+}
+
+#if USE(GBM)
+EGLDeviceEXT PlatformDisplay::eglDevice()
+{
+    if (!GLContext::isExtensionSupported(eglQueryString(nullptr, EGL_EXTENSIONS), "EGL_EXT_device_query"))
+        return nullptr;
+
+    if (!m_eglDisplayInitialized)
+        const_cast<PlatformDisplay*>(this)->initializeEGLDisplay();
+
+    EGLDeviceEXT eglDevice;
+    if (eglQueryDisplayAttribEXT(m_eglDisplay, EGL_DEVICE_EXT, reinterpret_cast<EGLAttrib*>(&eglDevice)))
+        return eglDevice;
+
+    return nullptr;
+}
+
+const String& PlatformDisplay::drmDeviceFile()
+{
+    if (!m_drmDeviceFile.has_value()) {
+        if (EGLDeviceEXT device = eglDevice()) {
+            if (GLContext::isExtensionSupported(eglQueryDeviceStringEXT(device, EGL_EXTENSIONS), "EGL_EXT_device_drm")) {
+                m_drmDeviceFile = String::fromUTF8(eglQueryDeviceStringEXT(device, EGL_DRM_DEVICE_FILE_EXT));
+                return m_drmDeviceFile.value();
+            }
+        }
+        m_drmDeviceFile = String();
+    }
+
+    return m_drmDeviceFile.value();
+}
+
+static String drmRenderNodeFromPrimaryDeviceFile(const String& primaryDeviceFile)
+{
+    if (primaryDeviceFile.isEmpty())
+        return { };
+
+    drmDevicePtr devices[64];
+    memset(devices, 0, sizeof(devices));
+
+    int numDevices = drmGetDevices2(0, devices, std::size(devices));
+    if (numDevices <= 0)
+        return { };
+
+    String renderNodeDeviceFile;
+    for (int i = 0; i < numDevices; ++i) {
+        drmDevice* device = devices[i];
+        if (!(device->available_nodes & (1 << DRM_NODE_PRIMARY | 1 << DRM_NODE_RENDER)))
+            continue;
+
+        if (String::fromUTF8(device->nodes[DRM_NODE_PRIMARY]) == primaryDeviceFile) {
+            renderNodeDeviceFile = String::fromUTF8(device->nodes[DRM_NODE_RENDER]);
+            break;
+        }
+    }
+    drmFreeDevices(devices, numDevices);
+
+    return renderNodeDeviceFile;
+}
+
+const String& PlatformDisplay::drmRenderNodeFile()
+{
+    if (!m_drmRenderNodeFile.has_value()) {
+        if (EGLDeviceEXT device = eglDevice()) {
+            if (GLContext::isExtensionSupported(eglQueryDeviceStringEXT(device, EGL_EXTENSIONS), "EGL_EXT_device_drm_render_node")) {
+                m_drmRenderNodeFile = String::fromUTF8(eglQueryDeviceStringEXT(device, EGL_DRM_RENDER_NODE_FILE_EXT));
+                return m_drmRenderNodeFile.value();
+            }
+
+            // If EGL_EXT_device_drm_render_node is not present, try to get the render node using DRM API.
+            m_drmRenderNodeFile = drmRenderNodeFromPrimaryDeviceFile(drmDeviceFile());
+        } else
+            m_drmRenderNodeFile = String();
+    }
+
+    return m_drmRenderNodeFile.value();
+}
+
+struct gbm_device* PlatformDisplay::gbmDevice()
+{
+    auto& device = GBMDevice::singleton();
+    if (!device.isInitialized()) {
+        const char* envDeviceFile = getenv("WEBKIT_WEB_RENDER_DEVICE_FILE");
+        device.initialize(envDeviceFile && *envDeviceFile ? String::fromUTF8(envDeviceFile) : drmRenderNodeFile());
+    }
+    return device.device();
+}
+#endif
+
 #endif // USE(EGL)
 
 #if USE(LCMS)
