@@ -30,7 +30,16 @@
 #include <wtf/text/WTFString.h>
 
 #if USE(EGL)
+typedef intptr_t EGLAttrib;
+typedef void *EGLClientBuffer;
+typedef void *EGLContext;
 typedef void *EGLDisplay;
+typedef void *EGLImage;
+typedef unsigned EGLenum;
+#if USE(GBM)
+typedef void *EGLDeviceEXT;
+struct gbm_device;
+#endif
 #endif
 
 #if PLATFORM(GTK)
@@ -73,12 +82,18 @@ public:
 #endif
 #if USE(WPE_RENDERER)
         WPE,
+#endif
+#if USE(EGL)
+        Surfaceless,
+#if USE(GBM)
+        GBM,
+#endif
 #endif
     };
 
     virtual Type type() const = 0;
 
-#if USE(EGL) || USE(GLX)
+#if USE(EGL)
     WEBCORE_EXPORT GLContext* sharingGLContext();
     void clearSharingGLContext();
 #endif
@@ -89,10 +104,23 @@ public:
 
     struct EGLExtensions {
         bool KHR_image_base { false };
+        bool KHR_surfaceless_context { false };
         bool EXT_image_dma_buf_import { false };
         bool EXT_image_dma_buf_import_modifiers { false };
     };
-    const EGLExtensions& eglExtensions() const { return m_eglExtensions; }
+    const EGLExtensions& eglExtensions() const;
+
+    EGLImage createEGLImage(EGLContext, EGLenum target, EGLClientBuffer, const Vector<EGLAttrib>&) const;
+    bool destroyEGLImage(EGLImage) const;
+#if USE(GBM)
+    const String& drmDeviceFile();
+    const String& drmRenderNodeFile();
+    struct gbm_device* gbmDevice();
+#endif
+
+#if PLATFORM(GTK)
+    virtual EGLDisplay gtkEGLDisplay() { return nullptr; }
+#endif
 #endif
 
 #if ENABLE(VIDEO) && USE(GSTREAMER_GL)
@@ -126,10 +154,13 @@ protected:
     virtual void initializeEGLDisplay();
 
     EGLDisplay m_eglDisplay;
-#endif
-
-#if USE(EGL) || USE(GLX)
+    bool m_eglDisplayOwned { true };
     std::unique_ptr<GLContext> m_sharingGLContext;
+
+#if USE(GBM)
+    std::optional<String> m_drmDeviceFile;
+    std::optional<String> m_drmRenderNodeFile;
+#endif
 #endif
 
 #if USE(LCMS)
@@ -147,6 +178,9 @@ private:
 
 #if USE(EGL)
     void terminateEGLDisplay();
+#if USE(GBM)
+    EGLDeviceEXT eglDevice();
+#endif
 
     bool m_eglDisplayInitialized { false };
     int m_eglMajorVersion { 0 };
