@@ -35,16 +35,9 @@
 #include "SharedBuffer.h"
 #include <wtf/text/TextStream.h>
 
-#if USE(CG)
-#include "GraphicsContextPlatformPrivateCG.h"
-#endif
-
 namespace WebCore {
 namespace DisplayList {
 
-// Should match RenderTheme::platformFocusRingWidth()
-static const float platformFocusRingWidth = 3;
-
 void Save::apply(GraphicsContext& context) const
 {
     context.save();
@@ -135,11 +128,21 @@ void Clip::apply(GraphicsContext& context) const
     context.clip(m_rect);
 }
 
+void ClipRoundedRect::apply(GraphicsContext& context) const
+{
+    context.clipRoundedRect(m_rect);
+}
+
 void ClipOut::apply(GraphicsContext& context) const
 {
     context.clipOut(m_rect);
 }
 
+void ClipOutRoundedRect::apply(GraphicsContext& context) const
+{
+    context.clipOutRoundedRect(m_rect);
+}
+
 void ClipToImageBuffer::apply(GraphicsContext& context, WebCore::ImageBuffer& imageBuffer) const
 {
     context.clipToImageBuffer(imageBuffer, m_destinationRect);
@@ -155,6 +158,11 @@ void ClipPath::apply(GraphicsContext& context) const
     context.clipPath(m_path, m_windRule);
 }
 
+void ResetClip::apply(GraphicsContext& context) const
+{
+    context.resetClip();
+}
+
 DrawFilteredImageBuffer::DrawFilteredImageBuffer(std::optional<RenderingResourceIdentifier> sourceImageIdentifier, const FloatRect& sourceImageRect, Filter& filter)
     : m_sourceImageIdentifier(sourceImageIdentifier)
     , m_sourceImageRect(sourceImageRect)
@@ -196,7 +204,7 @@ void DrawImageBuffer::apply(GraphicsContext& context, WebCore::ImageBuffer& imag
 
 void DrawNativeImage::apply(GraphicsContext& context, NativeImage& image) const
 {
-    context.drawNativeImage(image, m_imageSize, m_destinationRect, m_srcRect, m_options);
+    context.drawNativeImageInternal(image, m_imageSize, m_destinationRect, m_srcRect, m_options);
 }
 
 void DrawSystemImage::apply(GraphicsContext& context) const
@@ -240,7 +248,7 @@ void DrawLine::apply(GraphicsContext& context) const
     context.drawLine(m_point1, m_point2);
 }
 
-DrawLinesForText::DrawLinesForText(const FloatPoint& blockLocation, const FloatSize& localAnchor, float thickness, const DashArray& widths, bool printing, bool doubleLines, StrokeStyle style)
+DrawLinesForText::DrawLinesForText(const FloatPoint& blockLocation, const FloatSize& localAnchor, const DashArray& widths, float thickness, bool printing, bool doubleLines, StrokeStyle style)
     : m_blockLocation(blockLocation)
     , m_localAnchor(localAnchor)
     , m_widths(widths)
@@ -258,10 +266,7 @@ void DrawLinesForText::apply(GraphicsContext& context) const
 
 void DrawDotsForDocumentMarker::apply(GraphicsContext& context) const
 {
-    context.drawDotsForDocumentMarker(m_rect, {
-        static_cast<DocumentMarkerLineStyle::Mode>(m_styleMode),
-        m_styleShouldUseDarkAppearance,
-    });
+    context.drawDotsForDocumentMarker(m_rect, m_style);
 }
 
 void DrawEllipse::apply(GraphicsContext& context) const
@@ -300,6 +305,12 @@ FillRectWithGradient::FillRectWithGradient(const FloatRect& rect, Gradient& grad
 {
 }
 
+FillRectWithGradient::FillRectWithGradient(FloatRect&& rect, Ref<Gradient>&& gradient)
+    : m_rect(WTFMove(rect))
+    , m_gradient(WTFMove(gradient))
+{
+}
+
 void FillRectWithGradient::apply(GraphicsContext& context) const
 {
     context.fillRect(m_rect, m_gradient.get());
@@ -454,8 +465,6 @@ void ApplyDeviceScaleFactor::apply(GraphicsContext& context) const
 {
     context.applyDeviceScaleFactor(m_scaleFactor);
 }
-
-#if !LOG_DISABLED
 TextStream& operator<<(TextStream& ts, ItemType type)
 {
     switch (type) {
@@ -475,10 +484,13 @@ TextStream& operator<<(TextStream& ts, ItemType type)
     case ItemType::SetLineJoin: ts << "set-line-join"; break;
     case ItemType::SetMiterLimit: ts << "set-miter-limit"; break;
     case ItemType::Clip: ts << "clip"; break;
+    case ItemType::ClipRoundedRect: ts << "clip-rounded-rect"; break;
     case ItemType::ClipOut: ts << "clip-out"; break;
+    case ItemType::ClipOutRoundedRect: ts << "clip-out-rounded-rect"; break;
     case ItemType::ClipToImageBuffer: ts << "clip-to-image-buffer"; break;
     case ItemType::ClipOutToPath: ts << "clip-out-to-path"; break;
     case ItemType::ClipPath: ts << "clip-path"; break;
+    case ItemType::ResetClip: ts << "reset-clip"; break;
     case ItemType::DrawFilteredImageBuffer: ts << "draw-filtered-image-buffer"; break;
     case ItemType::DrawGlyphs: ts << "draw-glyphs"; break;
     case ItemType::DrawDecomposedGlyphs: ts << "draw-decomposed-glyphs"; break;
@@ -606,11 +618,21 @@ void dumpItem(TextStream& ts, const Clip& item, OptionSet<AsTextFlag>)
     ts.dumpProperty("rect", item.rect());
 }
 
+void dumpItem(TextStream& ts, const ClipRoundedRect& item, OptionSet<AsTextFlag>)
+{
+    ts.dumpProperty("rect", item.rect());
+}
+
 void dumpItem(TextStream& ts, const ClipOut& item, OptionSet<AsTextFlag>)
 {
     ts.dumpProperty("rect", item.rect());
 }
 
+void dumpItem(TextStream& ts, const ClipOutRoundedRect& item, OptionSet<AsTextFlag>)
+{
+    ts.dumpProperty("rect", item.rect());
+}
+
 void dumpItem(TextStream& ts, const ClipToImageBuffer& item, OptionSet<AsTextFlag> flags)
 {
     if (flags.contains(AsTextFlag::IncludeResourceIdentifiers))
@@ -930,9 +952,15 @@ void dumpItemHandle(TextStream& ts, const ItemHandle& item, OptionSet<AsTextFlag
     case ItemType::Clip:
         dumpItem(ts, item.get<Clip>(), flags);
         break;
+    case ItemType::ClipRoundedRect:
+        dumpItem(ts, item.get<ClipRoundedRect>(), flags);
+        break;
     case ItemType::ClipOut:
         dumpItem(ts, item.get<ClipOut>(), flags);
         break;
+    case ItemType::ClipOutRoundedRect:
+        dumpItem(ts, item.get<ClipOutRoundedRect>(), flags);
+        break;
     case ItemType::ClipToImageBuffer:
         dumpItem(ts, item.get<ClipToImageBuffer>(), flags);
         break;
@@ -1074,10 +1102,10 @@ void dumpItemHandle(TextStream& ts, const ItemHandle& item, OptionSet<AsTextFlag
     case ItemType::ApplyFillPattern:
 #endif
     case ItemType::ClearShadow:
+    case ItemType::ResetClip:
         break;
     }
 }
-#endif
 
 } // namespace DisplayList
 } // namespace WebCore
