@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2021-2022 Apple Inc. All rights reserved.
+ * Copyright (C) 2021-2023 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -42,8 +42,8 @@
 namespace WebCore {
 namespace DisplayList {
 
-RecorderImpl::RecorderImpl(DisplayList& displayList, const GraphicsContextState& state, const FloatRect& initialClip, const AffineTransform& initialCTM, DrawGlyphsMode drawGlyphsMode)
-    : Recorder(state, initialClip, initialCTM, drawGlyphsMode)
+RecorderImpl::RecorderImpl(DisplayList& displayList, const GraphicsContextState& state, const FloatRect& initialClip, const AffineTransform& initialCTM, const DestinationColorSpace& colorSpace, DrawGlyphsMode drawGlyphsMode)
+    : Recorder(state, initialClip, initialCTM, colorSpace, drawGlyphsMode)
     , m_displayList(displayList)
 {
     LOG_WITH_STREAM(DisplayLists, stream << "\nRecording with clip " << initialClip);
@@ -134,16 +134,31 @@ void RecorderImpl::recordClearShadow()
     append<ClearShadow>();
 }
 
+void RecorderImpl::recordResetClip()
+{
+    append<ResetClip>();
+}
+
 void RecorderImpl::recordClip(const FloatRect& clipRect)
 {
     append<Clip>(clipRect);
 }
 
+void RecorderImpl::recordClipRoundedRect(const FloatRoundedRect& clipRect)
+{
+    append<ClipRoundedRect>(clipRect);
+}
+
 void RecorderImpl::recordClipOut(const FloatRect& clipRect)
 {
     append<ClipOut>(clipRect);
 }
 
+void RecorderImpl::recordClipOutRoundedRect(const FloatRoundedRect& clipRect)
+{
+    append<ClipOutRoundedRect>(clipRect);
+}
+
 void RecorderImpl::recordClipToImageBuffer(ImageBuffer& imageBuffer, const FloatRect& destinationRect)
 {
     append<ClipToImageBuffer>(imageBuffer.renderingResourceIdentifier(), destinationRect);
@@ -219,7 +234,7 @@ void RecorderImpl::recordDrawLine(const FloatPoint& point1, const FloatPoint& po
 
 void RecorderImpl::recordDrawLinesForText(const FloatPoint& blockLocation, const FloatSize& localAnchor, float thickness, const DashArray& widths, bool printing, bool doubleLines, StrokeStyle style)
 {
-    append<DrawLinesForText>(blockLocation, localAnchor, thickness, widths, printing, doubleLines, style);
+    append<DrawLinesForText>(blockLocation, localAnchor, widths, thickness, printing, doubleLines, style);
 }
 
 void RecorderImpl::recordDrawDotsForDocumentMarker(const FloatRect& rect, const DocumentMarkerLineStyle& style)
@@ -433,14 +448,16 @@ bool RecorderImpl::recordResourceUse(DecomposedGlyphs& decomposedGlyphs)
     return true;
 }
 
-// FIXME: share with ShadowData
-static inline float shadowPaintingExtent(float blurRadius)
+bool RecorderImpl::recordResourceUse(Gradient& gradient)
 {
-    // Blurring uses a Gaussian function whose std. deviation is m_radius/2, and which in theory
-    // extends to infinity. In 8-bit contexts, however, rounding causes the effect to become
-    // undetectable at around 1.4x the radius.
-    const float radiusExtentMultiplier = 1.4;
-    return ceilf(blurRadius * radiusExtentMultiplier);
+    m_displayList.cacheGradient(gradient);
+    return true;
+}
+
+bool RecorderImpl::recordResourceUse(Filter& filter)
+{
+    m_displayList.cacheFilter(filter);
+    return true;
 }
 
 } // namespace DisplayList
