@@ -27,8 +27,34 @@
 #include <gst/gl/egl/gsteglimage.h>
 #endif
 
+#if USE(GSTREAMER_WEBRTC)
+#include <gst/rtp/rtp.h>
+#define GST_USE_UNSTABLE_API
+#include <gst/webrtc/webrtc.h>
+#undef GST_USE_UNSTABLE_API
+#endif
+
 namespace WTF {
 
+template<> GRefPtr<GstMiniObject> adoptGRef(GstMiniObject* ptr)
+{
+    return GRefPtr<GstMiniObject>(ptr, GRefPtrAdopt);
+}
+
+template<> GstMiniObject* refGPtr<GstMiniObject>(GstMiniObject* ptr)
+{
+    if (ptr)
+        gst_mini_object_ref(ptr);
+
+    return ptr;
+}
+
+template<> void derefGPtr<GstMiniObject>(GstMiniObject* ptr)
+{
+    if (ptr)
+        gst_mini_object_unref(ptr);
+}
+
 template <> GRefPtr<GstElement> adoptGRef(GstElement* ptr)
 {
     ASSERT(!ptr || !g_object_is_floating(ptr));
@@ -438,6 +464,28 @@ void derefGPtr<GstClock>(GstClock* ptr)
         gst_object_unref(ptr);
 }
 
+template <>
+GRefPtr<GstDeviceMonitor> adoptGRef(GstDeviceMonitor* ptr)
+{
+    return GRefPtr<GstDeviceMonitor>(ptr, GRefPtrAdopt);
+}
+
+template <>
+GstDeviceMonitor* refGPtr<GstDeviceMonitor>(GstDeviceMonitor* ptr)
+{
+    if (ptr)
+        gst_object_ref(GST_OBJECT_CAST(ptr));
+
+    return ptr;
+}
+
+template <>
+void derefGPtr<GstDeviceMonitor>(GstDeviceMonitor* ptr)
+{
+    if (ptr)
+        gst_object_unref(ptr);
+}
+
 template <> GRefPtr<WebKitVideoSink> adoptGRef(WebKitVideoSink* ptr)
 {
     ASSERT(!ptr || !g_object_is_floating(ptr));
@@ -458,6 +506,26 @@ template <> void derefGPtr<WebKitVideoSink>(WebKitVideoSink* ptr)
         gst_object_unref(GST_OBJECT(ptr));
 }
 
+template <> GRefPtr<GstBaseSink> adoptGRef(GstBaseSink* ptr)
+{
+    ASSERT(!ptr || !g_object_is_floating(ptr));
+    return GRefPtr<GstBaseSink>(ptr, GRefPtrAdopt);
+}
+
+template <> GstBaseSink* refGPtr<GstBaseSink>(GstBaseSink* ptr)
+{
+    if (ptr)
+        gst_object_ref_sink(GST_OBJECT(ptr));
+
+    return ptr;
+}
+
+template <> void derefGPtr<GstBaseSink>(GstBaseSink* ptr)
+{
+    if (ptr)
+        gst_object_unref(GST_OBJECT(ptr));
+}
+
 template <> GRefPtr<WebKitWebSrc> adoptGRef(WebKitWebSrc* ptr)
 {
     ASSERT(!ptr || !g_object_is_floating(ptr));
@@ -548,8 +616,206 @@ template <> void derefGPtr<GstEGLImage>(GstEGLImage* ptr)
         gst_egl_image_unref(ptr);
 }
 
+template<> GRefPtr<GstGLColorConvert> adoptGRef(GstGLColorConvert* ptr)
+{
+    ASSERT(!ptr || !g_object_is_floating(ptr));
+    return GRefPtr<GstGLColorConvert>(ptr, GRefPtrAdopt);
+}
+
+template<> GstGLColorConvert* refGPtr<GstGLColorConvert>(GstGLColorConvert* ptr)
+{
+    if (ptr)
+        gst_object_ref_sink(GST_OBJECT(ptr));
+
+    return ptr;
+}
+
+template<> void derefGPtr<GstGLColorConvert>(GstGLColorConvert* ptr)
+{
+    if (ptr)
+        gst_object_unref(GST_OBJECT(ptr));
+}
+
 #endif // USE(GSTREAMER_GL)
 
+template <>
+GRefPtr<GstEncodingProfile> adoptGRef(GstEncodingProfile* ptr)
+{
+    return GRefPtr<GstEncodingProfile>(ptr, GRefPtrAdopt);
+}
+
+template <>
+GstEncodingProfile* refGPtr<GstEncodingProfile>(GstEncodingProfile* ptr)
+{
+    if (ptr)
+        gst_encoding_profile_ref(ptr);
+
+    return ptr;
+}
+
+template<>
+void derefGPtr<GstEncodingProfile>(GstEncodingProfile* ptr)
+{
+    if (ptr)
+        gst_encoding_profile_unref(ptr);
+}
+
+#if USE(GSTREAMER_WEBRTC)
+
+template <> GRefPtr<GstWebRTCRTPReceiver> adoptGRef(GstWebRTCRTPReceiver* ptr)
+{
+    return GRefPtr<GstWebRTCRTPReceiver>(ptr, GRefPtrAdopt);
+}
+
+template <> GstWebRTCRTPReceiver* refGPtr<GstWebRTCRTPReceiver>(GstWebRTCRTPReceiver* ptr)
+{
+    if (ptr)
+        gst_object_ref(GST_OBJECT(ptr));
+
+    return ptr;
+}
+
+template <> void derefGPtr<GstWebRTCRTPReceiver>(GstWebRTCRTPReceiver* ptr)
+{
+    if (ptr)
+        gst_object_unref(ptr);
+}
+
+template <> GRefPtr<GstWebRTCRTPSender> adoptGRef(GstWebRTCRTPSender* ptr)
+{
+    return GRefPtr<GstWebRTCRTPSender>(ptr, GRefPtrAdopt);
+}
+
+template <> GstWebRTCRTPSender* refGPtr<GstWebRTCRTPSender>(GstWebRTCRTPSender* ptr)
+{
+    if (ptr)
+        gst_object_ref(GST_OBJECT(ptr));
+
+    return ptr;
+}
+
+template <> void derefGPtr<GstWebRTCRTPSender>(GstWebRTCRTPSender* ptr)
+{
+    if (ptr)
+        gst_object_unref(ptr);
+}
+
+template <> GRefPtr<GstWebRTCRTPTransceiver> adoptGRef(GstWebRTCRTPTransceiver* ptr)
+{
+    return GRefPtr<GstWebRTCRTPTransceiver>(ptr, GRefPtrAdopt);
+}
+
+template <> GstWebRTCRTPTransceiver* refGPtr<GstWebRTCRTPTransceiver>(GstWebRTCRTPTransceiver* ptr)
+{
+    if (ptr)
+        gst_object_ref(GST_OBJECT(ptr));
+
+    return ptr;
+}
+
+template <> void derefGPtr<GstWebRTCRTPTransceiver>(GstWebRTCRTPTransceiver* ptr)
+{
+    if (ptr)
+        gst_object_unref(ptr);
+}
+
+template <> GRefPtr<GstWebRTCDataChannel> adoptGRef(GstWebRTCDataChannel* ptr)
+{
+    return GRefPtr<GstWebRTCDataChannel>(ptr, GRefPtrAdopt);
+}
+
+template <> GstWebRTCDataChannel* refGPtr<GstWebRTCDataChannel>(GstWebRTCDataChannel* ptr)
+{
+    if (ptr)
+        g_object_ref(G_OBJECT(ptr));
+
+    return ptr;
+}
+
+template <> void derefGPtr<GstWebRTCDataChannel>(GstWebRTCDataChannel* ptr)
+{
+    if (ptr)
+        g_object_unref(ptr);
+}
+
+template <> GRefPtr<GstWebRTCDTLSTransport> adoptGRef(GstWebRTCDTLSTransport* ptr)
+{
+    return GRefPtr<GstWebRTCDTLSTransport>(ptr, GRefPtrAdopt);
+}
+
+template <> GstWebRTCDTLSTransport* refGPtr<GstWebRTCDTLSTransport>(GstWebRTCDTLSTransport* ptr)
+{
+    if (ptr)
+        gst_object_ref(GST_OBJECT(ptr));
+
+    return ptr;
+}
+
+template <> void derefGPtr<GstWebRTCDTLSTransport>(GstWebRTCDTLSTransport* ptr)
+{
+    if (ptr)
+        gst_object_unref(ptr);
+}
+
+template <> GRefPtr<GstWebRTCICETransport> adoptGRef(GstWebRTCICETransport* ptr)
+{
+    return GRefPtr<GstWebRTCICETransport>(ptr, GRefPtrAdopt);
+}
+
+template <> GstWebRTCICETransport* refGPtr<GstWebRTCICETransport>(GstWebRTCICETransport* ptr)
+{
+    if (ptr)
+        gst_object_ref(GST_OBJECT(ptr));
+
+    return ptr;
+}
+
+template <> void derefGPtr<GstWebRTCICETransport>(GstWebRTCICETransport* ptr)
+{
+    if (ptr)
+        gst_object_unref(ptr);
+}
+
+template<> GRefPtr<GstPromise> adoptGRef(GstPromise* ptr)
+{
+    return GRefPtr<GstPromise>(ptr, GRefPtrAdopt);
+}
+
+template<> GstPromise* refGPtr<GstPromise>(GstPromise* ptr)
+{
+    if (ptr)
+        gst_promise_ref(ptr);
+
+    return ptr;
+}
+
+template<> void derefGPtr<GstPromise>(GstPromise* ptr)
+{
+    if (ptr)
+        gst_promise_unref(ptr);
+}
+
+template<> GRefPtr<GstRTPHeaderExtension> adoptGRef(GstRTPHeaderExtension* ptr)
+{
+    return GRefPtr<GstRTPHeaderExtension>(ptr, GRefPtrAdopt);
+}
+
+template<> GstRTPHeaderExtension* refGPtr<GstRTPHeaderExtension>(GstRTPHeaderExtension* ptr)
+{
+    if (ptr)
+        gst_object_ref(GST_OBJECT(ptr));
+
+    return ptr;
+}
+
+template<> void derefGPtr<GstRTPHeaderExtension>(GstRTPHeaderExtension* ptr)
+{
+    if (ptr)
+        gst_object_unref(ptr);
+}
+
+#endif // USE(GSTREAMER_WEBRTC)
+
 } // namespace WTF
 
 #endif // USE(GSTREAMER)
