@@ -42,7 +42,7 @@ GST_DEBUG_CATEGORY(video_encoder_debug);
 #define NUMBER_OF_THREADS 4
 
 static GstStaticPadTemplate sinkTemplate = GST_STATIC_PAD_TEMPLATE("sink", GST_PAD_SINK, GST_PAD_ALWAYS, GST_STATIC_CAPS("video/x-raw(ANY)"));
-static GstStaticPadTemplate srcTemplate = GST_STATIC_PAD_TEMPLATE("src", GST_PAD_SRC, GST_PAD_ALWAYS, GST_STATIC_CAPS("video/x-h264;video/x-vp8;video/x-vp9;video/x-h265"));
+static GstStaticPadTemplate srcTemplate = GST_STATIC_PAD_TEMPLATE("src", GST_PAD_SRC, GST_PAD_ALWAYS, GST_STATIC_CAPS("video/x-h264;video/x-vp8;video/x-vp9;video/x-h265;video/x-av1"));
 
 // https://www.w3.org/TR/mediastream-recording/#bitratemode
 typedef enum {
@@ -113,7 +113,8 @@ enum EncoderId {
     VaapiH264,
     VaapiH265,
     Vp8,
-    Vp9
+    Vp9,
+    Av1
 };
 
 class Encoders {
@@ -124,7 +125,7 @@ public:
         return encoders;
     }
 
-    static void registerEncoder(EncoderId id, const char* name, const char* parserName, const char* caps, const char* encodedFormat,
+    static void registerEncoder(EncoderId id, const char* name, const char* parserName, const char* capsString, const char* encodedFormatString,
         SetupFunc&& setupEncoder, const char* bitratePropertyName, SetBitrateFunc&& setBitrate, const char* keyframeIntervalPropertyName, SetBitrateModeFunc&& setBitrateMode, SetLatencyModeFunc&& setLatency)
     {
         auto encoderFactory = adoptGRef(gst_element_factory_find(name));
@@ -146,12 +147,21 @@ public:
             }
         }
 
+        auto caps = adoptGRef(gst_caps_from_string(capsString));
+        GST_MINI_OBJECT_FLAG_SET(caps.get(), GST_MINI_OBJECT_FLAG_MAY_BE_LEAKED);
+
+        GRefPtr<GstCaps> encodedFormat;
+        if (encodedFormatString) {
+            encodedFormat = adoptGRef(gst_caps_from_string(encodedFormatString));
+            GST_MINI_OBJECT_FLAG_SET(encodedFormat.get(), GST_MINI_OBJECT_FLAG_MAY_BE_LEAKED);
+        }
+
         singleton().emplace(std::make_pair(id, EncoderDefinition {
-            .caps = adoptGRef(gst_caps_from_string(caps)),
+            .caps = WTFMove(caps),
             .name = name,
             .parserName = parserName,
             .factory = WTFMove(encoderFactory),
-            .encodedFormat = encodedFormat ? adoptGRef(gst_caps_from_string(encodedFormat)) : nullptr,
+            .encodedFormat = WTFMove(encodedFormat),
             .setBitrate = WTFMove(setBitrate),
             .setupEncoder = WTFMove(setupEncoder),
             .setBitrateMode = WTFMove(setBitrateMode),
@@ -189,7 +199,7 @@ struct _WebKitVideoEncoderPrivate {
 
 #define webkit_video_encoder_parent_class parent_class
 WEBKIT_DEFINE_TYPE_WITH_CODE(WebKitVideoEncoder, webkit_video_encoder, GST_TYPE_BIN,
-    GST_DEBUG_CATEGORY_INIT(video_encoder_debug, "webkitvideoencoder", 0, "WebKit Video encoder"))
+    GST_DEBUG_CATEGORY_INIT(video_encoder_debug, "webkitvideoencoderprivate", 0, "WebKit Video Encoder Private"))
 
 enum {
     PROP_FORMAT = 1,
@@ -485,6 +495,8 @@ static GRefPtr<GstCaps> createSrcPadTemplateCaps()
 
 static void videoEncoderConstructed(GObject* encoder)
 {
+    GST_CALL_PARENT(G_OBJECT_CLASS, constructed, (encoder));
+
     auto* self = WEBKIT_VIDEO_ENCODER(encoder);
     self->priv->encoderId = None;
 
@@ -630,9 +642,11 @@ static void webkit_video_encoder_class_init(WebKitVideoEncoderClass* klass)
             };
         }, [](GstElement* encoder, LatencyMode mode) {
             switch (mode) {
-            case REALTIME_LATENCY_MODE:
-                g_object_set(encoder, "threads", NUMBER_OF_THREADS, "cpu-used", NUMBER_OF_THREADS, "deadline", 1, "lag-in-frames", 0, nullptr);
+            case REALTIME_LATENCY_MODE: {
+                int64_t deadline = 1000000;
+                g_object_set(encoder, "threads", NUMBER_OF_THREADS, "cpu-used", NUMBER_OF_THREADS, "deadline", deadline, "lag-in-frames", 0, nullptr);
                 break;
+            }
             case QUALITY_LATENCY_MODE:
                 g_object_set(encoder, "threads", NUMBER_OF_THREADS, "cpu-used", NUMBER_OF_THREADS, "deadline", 0, "lag-in-frames", 25, nullptr);
                 gst_util_set_object_arg(G_OBJECT(encoder), "end-usage", "cq");
@@ -705,6 +719,35 @@ static void webkit_video_encoder_class_init(WebKitVideoEncoderClass* klass)
             };
         });
 
+    if (webkitGstCheckVersion(1, 22, 0)) {
+        Encoders::registerEncoder(Av1, "av1enc", "av1parse", "video/x-av1", nullptr,
+            [](WebKitVideoEncoder* self) {
+                g_object_set(self->priv->encoder.get(), "threads", NUMBER_OF_THREADS, nullptr);
+                gst_util_set_object_arg(G_OBJECT(self->priv->encoder.get()), "keyframe-mode", "disabled");
+            }, "target-bitrate", setBitrateKbitPerSec, "keyframe-max-dist", [](GstElement* encoder, BitrateMode mode) {
+                switch (mode) {
+                case CONSTANT_BITRATE_MODE:
+                    gst_util_set_object_arg(G_OBJECT(encoder), "end-usage", "cbr");
+                    break;
+                case VARIABLE_BITRATE_MODE:
+                    gst_util_set_object_arg(G_OBJECT(encoder), "end-usage", "vbr");
+                    break;
+                }
+            }, [](GstElement* encoder, LatencyMode mode) {
+                if (!gstObjectHasProperty(encoder, "usage-profile"))
+                    return;
+                switch (mode) {
+                case REALTIME_LATENCY_MODE:
+                    gst_util_set_object_arg(G_OBJECT(encoder), "usage-profile", "realtime");
+                    break;
+                case QUALITY_LATENCY_MODE:
+                    gst_util_set_object_arg(G_OBJECT(encoder), "usage-profile", "good-quality");
+                    gst_util_set_object_arg(G_OBJECT(encoder), "end-usage", "q");
+                    break;
+                }
+            });
+    }
+
     auto srcPadTemplateCaps = createSrcPadTemplateCaps();
     gst_element_class_add_pad_template(elementClass, gst_pad_template_new("src", GST_PAD_SRC, GST_PAD_ALWAYS, srcPadTemplateCaps.get()));
 
@@ -724,5 +767,6 @@ static void webkit_video_encoder_class_init(WebKitVideoEncoderClass* klass)
 }
 
 #undef NUMBER_OF_THREADS
+#undef GST_CAT_DEFAULT
 
 #endif // ENABLE(VIDEO) && USE(GSTREAMER)
