@@ -67,11 +67,11 @@ static inline Vector<MockMediaDevice> defaultDevices()
         MockMediaDevice { "239c24b2-2b15-11e3-8224-0800200c9a66"_s, "Mock video device 1"_s, false,
             MockCameraProperties {
                 30,
-                RealtimeMediaSourceSettings::VideoFacingMode::User, {
-                    { { 2560, 1440 }, { { 10, 10 }, { 7.5, 7.5 }, { 5, 5 } } },
-                    { { 1280, 720 }, { { 30, 30}, { 27.5, 27.5}, { 25, 25}, { 22.5, 22.5}, { 20, 20}, { 17.5, 17.5}, { 15, 15}, { 12.5, 12.5}, { 10, 10}, { 7.5, 7.5}, { 5, 5} } },
-                    { { 640, 480 },  { { 30, 30}, { 27.5, 27.5}, { 25, 25}, { 22.5, 22.5}, { 20, 20}, { 17.5, 17.5}, { 15, 15}, { 12.5, 12.5}, { 10, 10}, { 7.5, 7.5}, { 5, 5} } },
-                    { { 112, 112 },  { { 30, 30}, { 27.5, 27.5}, { 25, 25}, { 22.5, 22.5}, { 20, 20}, { 17.5, 17.5}, { 15, 15}, { 12.5, 12.5}, { 10, 10}, { 7.5, 7.5}, { 5, 5} } },
+                VideoFacingMode::User, {
+                    { { 2560, 1440 }, { { 10, 10 }, { 7.5, 7.5 }, { 5, 5 } }, 1, 1 },
+                    { { 1280, 720 }, { { 30, 30}, { 27.5, 27.5}, { 25, 25}, { 22.5, 22.5}, { 20, 20}, { 17.5, 17.5}, { 15, 15}, { 12.5, 12.5}, { 10, 10}, { 7.5, 7.5}, { 5, 5} }, 1, 1 },
+                    { { 640, 480 },  { { 30, 30}, { 27.5, 27.5}, { 25, 25}, { 22.5, 22.5}, { 20, 20}, { 17.5, 17.5}, { 15, 15}, { 12.5, 12.5}, { 10, 10}, { 7.5, 7.5}, { 5, 5} }, 1, 1 },
+                    { { 112, 112 },  { { 30, 30}, { 27.5, 27.5}, { 25, 25}, { 22.5, 22.5}, { 20, 20}, { 17.5, 17.5}, { 15, 15}, { 12.5, 12.5}, { 10, 10}, { 7.5, 7.5}, { 5, 5} }, 1, 1 },
                 },
                 Color::black,
             } },
@@ -79,15 +79,15 @@ static inline Vector<MockMediaDevice> defaultDevices()
         MockMediaDevice { "239c24b3-2b15-11e3-8224-0800200c9a66"_s, "Mock video device 2"_s, false,
             MockCameraProperties {
                 15,
-                RealtimeMediaSourceSettings::VideoFacingMode::Environment, {
-                    { { 3840, 2160 }, { { 2, 30 } } },
-                    { { 1920, 1080 }, { { 2, 30 } } },
-                    { { 1280, 720 },  { { 3, 120 } } },
-                    { { 960, 540 },   { { 3, 60 } } },
-                    { { 640, 480 },   { { 2, 30 } } },
-                    { { 352, 288 },   { { 2, 30 } } },
-                    { { 320, 240 },   { { 2, 30 } } },
-                    { { 160, 120 },   { { 2, 30 } } },
+                VideoFacingMode::Environment, {
+                    { { 3840, 2160 }, { { 2, 30 } }, 1, 4 },
+                    { { 1920, 1080 }, { { 2, 30 } }, 1, 4 },
+                    { { 1280, 720 },  { { 3, 120 } }, 1, 4 },
+                    { { 960, 540 },   { { 3, 60 } }, 1, 4 },
+                    { { 640, 480 },   { { 2, 30 } }, 1, 4 },
+                    { { 352, 288 },   { { 2, 30 } }, 1, 4 },
+                    { { 320, 240 },   { { 2, 30 } }, 1, 4 },
+                    { { 160, 120 },   { { 2, 30 } }, 1, 4 },
                 },
                 Color::darkGray,
             } },
@@ -115,7 +115,7 @@ private:
     CaptureDeviceManager& videoCaptureDeviceManager() final { return MockRealtimeMediaSourceCenter::singleton().videoCaptureDeviceManager(); }
 };
 
-#if PLATFORM(MAC)
+#if PLATFORM(COCOA)
 class MockDisplayCapturer final : public DisplayCaptureSourceCocoa::Capturer {
 public:
     MockDisplayCapturer(const CaptureDevice&, PageIdentifier);
@@ -124,7 +124,7 @@ private:
     bool start() final;
     void stop() final  { m_source->stop(); }
     DisplayCaptureSourceCocoa::DisplayFrameType generateFrame() final;
-    RealtimeMediaSourceSettings::DisplaySurfaceType surfaceType() const final { return RealtimeMediaSourceSettings::DisplaySurfaceType::Monitor; }
+    DisplaySurfaceType surfaceType() const final { return DisplaySurfaceType::Monitor; }
     void commitConfiguration(const RealtimeMediaSourceSettings&) final;
     CaptureDevice::DeviceType deviceType() const final { return CaptureDevice::DeviceType::Screen; }
     IntSize intrinsicSize() const final;
@@ -174,7 +174,7 @@ IntSize MockDisplayCapturer::intrinsicSize() const
     auto& properties = std::get<MockDisplayProperties>(device->properties);
     return properties.defaultSize;
 }
-#endif // PLATFORM(MAC)
+#endif // PLATFORM(COCOA)
 
 class MockRealtimeDisplaySourceFactory : public DisplayCaptureFactory {
 public:
@@ -186,11 +186,10 @@ public:
         switch (device.type()) {
         case CaptureDevice::DeviceType::Screen:
         case CaptureDevice::DeviceType::Window:
-#if PLATFORM(MAC)
+#if PLATFORM(COCOA)
             return DisplayCaptureSourceCocoa::create(UniqueRef<DisplayCaptureSourceCocoa::Capturer>(makeUniqueRef<MockDisplayCapturer>(device, pageIdentifier)), device, WTFMove(hashSalts), constraints, pageIdentifier);
 #elif USE(GSTREAMER)
-            UNUSED_PARAM(pageIdentifier);
-            return MockDisplayCaptureSourceGStreamer::create(device, WTFMove(hashSalts), constraints);
+            return MockDisplayCaptureSourceGStreamer::create(device, WTFMove(hashSalts), constraints, pageIdentifier);
 #else
             return MockRealtimeVideoSource::create(String { device.persistentId() }, AtomString { device.label() }, WTFMove(hashSalts), constraints, pageIdentifier);
 #endif
@@ -304,9 +303,12 @@ static CaptureDevice toCaptureDevice(const MockMediaDevice& device)
     return captureDevice;
 }
 
-static void createMockDevice(const MockMediaDevice& device)
+static void createMockDevice(const MockMediaDevice& device, bool isDefault)
 {
-    deviceListForDevice(device).append(toCaptureDevice(device));
+    if (isDefault)
+        deviceListForDevice(device).insert(0, toCaptureDevice(device));
+    else
+        deviceListForDevice(device).append(toCaptureDevice(device));
 }
 
 void MockRealtimeMediaSourceCenter::resetDevices()
@@ -347,16 +349,26 @@ void MockRealtimeMediaSourceCenter::setDevices(Vector<MockMediaDevice>&& newMock
 
     for (const auto& device : mockDevices) {
         map.add(device.persistentId, device);
-        createMockDevice(device);
+        createMockDevice(device, false);
     }
     RealtimeMediaSourceCenter::singleton().captureDevicesChanged();
 }
 
+static bool shouldBeDefaultDevice(const MockMediaDevice& device)
+{
+    auto* cameraProperties = device.cameraProperties();
+    return cameraProperties && cameraProperties->facingMode == VideoFacingMode::Unknown;
+}
+
 void MockRealtimeMediaSourceCenter::addDevice(const MockMediaDevice& device)
 {
-    devices().append(device);
+    bool isDefault = shouldBeDefaultDevice(device);
+    if (isDefault)
+        devices().insert(0, device);
+    else
+        devices().append(device);
     deviceMap().set(device.persistentId, device);
-    createMockDevice(device);
+    createMockDevice(device, isDefault);
     RealtimeMediaSourceCenter::singleton().captureDevicesChanged();
 }
 
@@ -489,17 +501,6 @@ DisplayCaptureFactory& MockRealtimeMediaSourceCenter::displayCaptureFactory()
     return factory.get();
 }
 
-void MockRealtimeMediaSourceCenter::MockDisplayCaptureDeviceManager::windowDevices(Vector<DisplayCaptureManager::WindowCaptureDevice>& windowDevices)
-{
-    auto devices = MockRealtimeMediaSourceCenter::displayDevices();
-    for (auto device : devices) {
-        if (!device.enabled() || device.type() != CaptureDevice::DeviceType::Window)
-            continue;
-
-        windowDevices.append({ WTFMove(device), "Mock Application"_s });
-    }
-}
-
 } // namespace WebCore
 
 #endif // ENABLE(MEDIA_STREAM)
