@@ -28,14 +28,16 @@
 #include "Document.h"
 #include "EventHandler.h"
 #include "EventNames.h"
-#include "Frame.h"
-#include "FrameView.h"
 #include "GraphicsContext.h"
 #include "HTMLFrameSetElement.h"
 #include "HitTestRequest.h"
 #include "HitTestResult.h"
+#include "LocalFrame.h"
+#include "LocalFrameView.h"
 #include "MouseEvent.h"
 #include "PaintInfo.h"
+#include "RenderBoxInlines.h"
+#include "RenderBoxModelObjectInlines.h"
 #include "RenderFrame.h"
 #include "RenderIterator.h"
 #include "RenderLayer.h"
@@ -46,6 +48,10 @@
 
 namespace WebCore {
 
+static constexpr auto borderStartEdgeColor = SRGBA<uint8_t> { 170, 170, 170 };
+static constexpr auto borderEndEdgeColor = Color::black;
+static constexpr auto borderFillColor = SRGBA<uint8_t> { 208, 208, 208 };
+
 WTF_MAKE_ISO_ALLOCATED_IMPL(RenderFrameSet);
 
 RenderFrameSet::RenderFrameSet(HTMLFrameSetElement& frameSet, RenderStyle&& style)
@@ -68,10 +74,6 @@ RenderFrameSet::GridAxis::GridAxis()
 {
 }
 
-constexpr auto borderStartEdgeColor = makeSimpleColor(170, 170, 170);
-constexpr auto borderEndEdgeColor = Color::black;
-constexpr auto borderFillColor = makeSimpleColor(208, 208, 208);
-
 void RenderFrameSet::paintColumnBorder(const PaintInfo& paintInfo, const IntRect& borderRect)
 {
     if (!paintInfo.rect.intersects(borderRect))
@@ -436,9 +438,9 @@ void RenderFrameSet::layout()
 
     bool doFullRepaint = selfNeedsLayout() && checkForRepaintDuringLayout();
     LayoutRect oldBounds;
-    RenderLayerModelObject* repaintContainer = 0;
+    const RenderLayerModelObject* repaintContainer = nullptr;
     if (doFullRepaint) {
-        repaintContainer = containerForRepaint();
+        repaintContainer = containerForRepaint().renderer;
         oldBounds = clippedOverflowRectForRepaint(repaintContainer);
     }
 
@@ -459,10 +461,7 @@ void RenderFrameSet::layout()
     layOutAxis(m_rows, frameSetElement().rowLengths(), height() - (rows - 1) * borderThickness);
     layOutAxis(m_cols, frameSetElement().colLengths(), width() - (cols - 1) * borderThickness);
 
-    if (flattenFrameSet())
-        positionFramesWithFlattening();
-    else
-        positionFrames();
+    positionFrames();
 
     RenderBox::layout();
 
@@ -533,114 +532,6 @@ void RenderFrameSet::positionFrames()
     resetFrameRendererAndDescendants(child, *this);
 }
 
-void RenderFrameSet::positionFramesWithFlattening()
-{
-    RenderBox* child = firstChildBox();
-    if (!child)
-        return;
-
-    int rows = frameSetElement().totalRows();
-    int cols = frameSetElement().totalCols();
-
-    int borderThickness = frameSetElement().border();
-    bool repaintNeeded = false;
-
-    // calculate frameset height based on actual content height to eliminate scrolling
-    bool out = false;
-    for (int r = 0; r < rows && !out; ++r) {
-        int extra = 0;
-        int height = m_rows.m_sizes[r];
-
-        for (int c = 0; c < cols; ++c) {
-            IntRect oldFrameRect = snappedIntRect(child->frameRect());
-
-            int width = m_cols.m_sizes[c];
-
-            bool fixedWidth = frameSetElement().colLengths() && frameSetElement().colLengths()[c].isFixed();
-            bool fixedHeight = frameSetElement().rowLengths() && frameSetElement().rowLengths()[r].isFixed();
-
-            // has to be resized and itself resize its contents
-            if (!fixedWidth)
-                child->setWidth(width ? width + extra / (cols - c) : 0);
-            else
-                child->setWidth(width);
-            child->setHeight(height);
-
-            child->setNeedsLayout();
-
-            if (is<RenderFrameSet>(*child))
-                downcast<RenderFrameSet>(*child).layout();
-            else
-                downcast<RenderFrame>(*child).layoutWithFlattening(fixedWidth, fixedHeight);
-
-            if (child->height() > m_rows.m_sizes[r])
-                m_rows.m_sizes[r] = child->height();
-            if (child->width() > m_cols.m_sizes[c])
-                m_cols.m_sizes[c] = child->width();
-
-            if (child->frameRect() != oldFrameRect)
-                repaintNeeded = true;
-
-            // difference between calculated frame width and the width it actually decides to have
-            extra += width - m_cols.m_sizes[c];
-
-            child = child->nextSiblingBox();
-            if (!child) {
-                out = true;
-                break;
-            }
-        }
-    }
-
-    int xPos = 0;
-    int yPos = 0;
-    out = false;
-    child = firstChildBox();
-    for (int r = 0; r < rows && !out; ++r) {
-        xPos = 0;
-        for (int c = 0; c < cols; ++c) {
-            // ensure the rows and columns are filled
-            IntRect oldRect = snappedIntRect(child->frameRect());
-
-            child->setLocation(IntPoint(xPos, yPos));
-            child->setHeight(m_rows.m_sizes[r]);
-            child->setWidth(m_cols.m_sizes[c]);
-
-            if (child->frameRect() != oldRect) {
-                repaintNeeded = true;
-
-                // update to final size
-                child->setNeedsLayout();
-                if (is<RenderFrameSet>(*child))
-                    downcast<RenderFrameSet>(*child).layout();
-                else
-                    downcast<RenderFrame>(*child).layoutWithFlattening(true, true);
-            }
-
-            xPos += m_cols.m_sizes[c] + borderThickness;
-            child = child->nextSiblingBox();
-            if (!child) {
-                out = true;
-                break;
-            }
-        }
-        yPos += m_rows.m_sizes[r] + borderThickness;
-    }
-
-    setWidth(xPos - borderThickness);
-    setHeight(yPos - borderThickness);
-
-    if (repaintNeeded)
-        repaint();
-
-    resetFrameRendererAndDescendants(child, *this);
-}
-
-bool RenderFrameSet::flattenFrameSet() const
-{
-    return view().frameView().effectiveFrameFlattening() != FrameFlattening::Disabled;
-}
-
 void RenderFrameSet::startResizing(GridAxis& axis, int position)
 {
     int split = hitTestSplit(axis, position);
@@ -669,9 +560,6 @@ void RenderFrameSet::continueResizing(GridAxis& axis, int position)
 
 bool RenderFrameSet::userResize(MouseEvent& event)
 {
-    if (flattenFrameSet())
-        return false;
-
     if (!m_isResizing) {
         if (needsLayout())
             return false;
