@@ -26,16 +26,22 @@
 #include "config.h"
 #include "RenderIFrame.h"
 
-#include "Frame.h"
-#include "FrameView.h"
 #include "HTMLIFrameElement.h"
 #include "HTMLNames.h"
+#include "LocalFrame.h"
+#include "LocalFrameView.h"
+#include "RenderBoxInlines.h"
+#include "RenderBoxModelObjectInlines.h"
+#include "RenderStyleInlines.h"
 #include "RenderView.h"
 #include "Settings.h"
+#include <wtf/IsoMallocInlines.h>
 #include <wtf/StackStats.h>
 
 namespace WebCore {
 
+WTF_MAKE_ISO_ALLOCATED_IMPL(RenderIFrame);
+
 using namespace HTMLNames;
     
 RenderIFrame::RenderIFrame(HTMLIFrameElement& element, RenderStyle&& style)
@@ -60,31 +66,21 @@ bool RenderIFrame::isInlineBlockOrInlineTable() const
 
 bool RenderIFrame::requiresLayer() const
 {
-    return RenderFrameBase::requiresLayer() || style().resize() != RESIZE_NONE;
+    return RenderFrameBase::requiresLayer() || style().resize() != Resize::None;
 }
 
 RenderView* RenderIFrame::contentRootRenderer() const
 {
-    FrameView* childFrameView = childView();
-    return childFrameView ? childFrameView->frame().contentRenderer() : 0;
+    auto* childFrameView = childView();
+    auto* localFrame = childFrameView ? dynamicDowncast<LocalFrame>(childFrameView->frame()) : nullptr;
+    return localFrame ? localFrame->contentRenderer() : nullptr;
 }
 
-bool RenderIFrame::flattenFrame() const
+bool RenderIFrame::isFullScreenIFrame() const
 {
-    if (!settings().frameFlatteningEnabled())
-        return false;
-
-    if (style().width().isFixed() && style().height().isFixed()) {
-        // Do not flatten iframes with scrolling="no".
-        if (iframeElement().scrollingMode() == ScrollbarAlwaysOff)
-            return false;
-        if (style().width().value() <= 0 || style().height().value() <= 0)
-            return false;
-    }
-
-    // Do not flatten offscreen inner frames during frame flattening, as flattening might make them visible.
-    IntRect boundingRect = absoluteBoundingBoxRectIgnoringTransforms();
-    return boundingRect.maxX() > 0 && boundingRect.maxY() > 0;
+    // Some authors implement fullscreen popups as out-of-flow iframes with size set to full viewport (using vw/vh units).
+    // The size used may not perfectly match the viewport size so the following heuristic uses a relaxed constraint.
+    return style().hasOutOfFlowPosition() && style().usesViewportUnits();
 }
 
 void RenderIFrame::layout()
@@ -96,9 +92,6 @@ void RenderIFrame::layout()
     // No kids to layout as a replaced element.
     updateLogicalHeight();
 
-    if (flattenFrame())
-        layoutWithFlattening(style().width().isFixed(), style().height().isFixed());
-
     clearOverflow();
     addVisualEffectOverflow();
     updateLayerTransform();
