@@ -35,7 +35,6 @@
 #include "FocusController.h"
 #include "FontCache.h"
 #include "FontCascade.h"
-#include "Frame.h"
 #include "FrameSelection.h"
 #include "GeometryUtilities.h"
 #include "GraphicsContext.h"
@@ -46,20 +45,27 @@
 #include "HTMLNames.h"
 #include "HitTestResult.h"
 #include "ImageOverlay.h"
+#include "InlineIteratorBoxInlines.h"
 #include "InlineIteratorInlineBox.h"
 #include "InlineIteratorLineBox.h"
 #include "LineSelection.h"
+#include "LocalFrame.h"
 #include "Page.h"
 #include "PaintInfo.h"
+#include "RenderBoxInlines.h"
+#include "RenderBoxModelObjectInlines.h"
 #include "RenderChildIterator.h"
+#include "RenderElementInlines.h"
 #include "RenderFragmentedFlow.h"
 #include "RenderImageResourceStyleImage.h"
 #include "RenderLayoutState.h"
+#include "RenderStyleSetters.h"
 #include "RenderTheme.h"
 #include "RenderView.h"
 #include "SVGElementTypeHelpers.h"
 #include "SVGImage.h"
 #include "Settings.h"
+#include "TextPainter.h"
 #include <wtf/IsoMallocInlines.h>
 #include <wtf/StackStats.h>
 
@@ -239,14 +245,6 @@ void RenderImage::styleDidChange(StyleDifference diff, const RenderStyle* oldSty
     }
     if (diff == StyleDifference::Layout && oldStyle && oldStyle->imageOrientation() != style().imageOrientation())
         return repaintOrMarkForLayout(ImageSizeChangeNone);
-
-#if ENABLE(CSS_IMAGE_RESOLUTION)
-    if (diff == StyleDifference::Layout && oldStyle
-        && (oldStyle->imageResolution() != style().imageResolution()
-            || oldStyle->imageResolutionSnap() != style().imageResolutionSnap()
-            || oldStyle->imageResolutionSource() != style().imageResolutionSource()))
-        repaintOrMarkForLayout(ImageSizeChangeNone);
-#endif
 }
 
 bool RenderImage::shouldCollapseToEmpty() const
@@ -340,16 +338,7 @@ void RenderImage::updateInnerContentRect()
 
 void RenderImage::repaintOrMarkForLayout(ImageSizeChangeType imageSizeChange, const IntRect* rect)
 {
-#if ENABLE(CSS_IMAGE_RESOLUTION)
-    double scale = style().imageResolution();
-    if (style().imageResolutionSnap() == ImageResolutionSnap::Pixels)
-        scale = roundForImpreciseConversion<int>(scale);
-    if (scale <= 0)
-        scale = 1;
-    LayoutSize newIntrinsicSize = imageResource().intrinsicSize(style().effectiveZoom() / scale);
-#else
     LayoutSize newIntrinsicSize = imageResource().intrinsicSize(style().effectiveZoom());
-#endif
     LayoutSize oldIntrinsicSize = intrinsicSize();
 
     updateIntrinsicSizeIfNeeded(newIntrinsicSize);
@@ -462,7 +451,7 @@ void RenderImage::paintIncompleteImageOutline(PaintInfo& paintInfo, LayoutPoint
 
     // Draw an outline rect where the image should be.
     GraphicsContext& context = paintInfo.context();
-    context.setStrokeStyle(SolidStroke);
+    context.setStrokeStyle(StrokeStyle::SolidStroke);
     context.setStrokeColor(Color::lightGray);
     context.setFillColor(Color::transparentBlack);
     context.drawRect(snapRectToDevicePixels(LayoutRect({ paintOffset.x() + leftBorder + leftPadding, paintOffset.y() + topBorder + topPadding }, contentSize), document().deviceScaleFactor()), borderWidth);
@@ -540,29 +529,51 @@ void RenderImage::paintReplaced(PaintInfo& paintInfo, const LayoutPoint& paintOf
             }
 
             if (!m_altText.isEmpty()) {
-                auto& font = style().fontCascade();
-                auto& fontMetrics = font.metricsOfPrimaryFont();
+                auto& style = this->style();
+                auto& fontCascade = style.fontCascade();
+                auto& fontMetrics = fontCascade.metricsOfPrimaryFont();
+                auto isHorizontal = style.isHorizontalWritingMode();
                 auto encodedDisplayString = document().displayStringModifiedByEncoding(m_altText);
-                auto textRun = RenderBlock::constructTextRun(encodedDisplayString, style(), ExpansionBehavior::defaultBehavior(), RespectDirection | RespectDirectionOverride);
-                auto textWidth = LayoutUnit { font.width(textRun) };
+                auto textRun = RenderBlock::constructTextRun(encodedDisplayString, style, ExpansionBehavior::defaultBehavior(), RespectDirection | RespectDirectionOverride);
+                auto textWidth = LayoutUnit { fontCascade.width(textRun) };
 
                 auto hasRoomForAltText = [&] {
-                    // Only draw the alt text if it'll fit within the content box,
-                    // and only if it fits above the error image.
-                    if (usableSize.width() < textWidth)
+                    // Only draw the alt text if it fits within the content box (content width) and above the error image (content height).
+                    // Error picture is always visually below the text regardless of writing direction.
+                    auto availableLogicalWidth = isHorizontal ? usableSize.width() : (errorPictureDrawn ? imageOffset.height() : usableSize.height());
+                    if (availableLogicalWidth < textWidth)
                         return false;
-                    return errorPictureDrawn ? fontMetrics.height() <= imageOffset.height() : usableSize.height() >= fontMetrics.height();
+                    auto availableLogicalHeight = isHorizontal ? (errorPictureDrawn ? imageOffset.height() : usableSize.height()) : usableSize.width();
+                    return availableLogicalHeight >= fontMetrics.height();
                 };
                 if (hasRoomForAltText()) {
-                    auto altTextLocation = [&]() -> LayoutPoint {
-                        auto contentHorizontalOffset = LayoutUnit { leftBorder + leftPadding + (paddingWidth / 2) - missingImageBorderWidth };
-                        auto contentVerticalOffset = LayoutUnit { topBorder + topPadding + fontMetrics.ascent() + (paddingHeight / 2) - missingImageBorderWidth };
-                        if (!style().isLeftToRightDirection())
-                            contentHorizontalOffset += contentSize.width() - textWidth;
-                        return paintOffset + LayoutPoint { contentHorizontalOffset, contentVerticalOffset };
-                    };
-                    context.setFillColor(style().visitedDependentColorWithColorFilter(CSSPropertyColor));
-                    context.drawBidiText(font, textRun, altTextLocation());
+                    context.setFillColor(style.visitedDependentColorWithColorFilter(CSSPropertyColor));
+                    if (isHorizontal) {
+                        auto altTextLocation = [&]() -> LayoutPoint {
+                            auto contentHorizontalOffset = LayoutUnit { leftBorder + leftPadding + (paddingWidth / 2) - missingImageBorderWidth };
+                            auto contentVerticalOffset = LayoutUnit { topBorder + topPadding + fontMetrics.ascent() + (paddingHeight / 2) - missingImageBorderWidth };
+                            if (!style.isLeftToRightDirection())
+                                contentHorizontalOffset += contentSize.width() - textWidth;
+                            return paintOffset + LayoutPoint { contentHorizontalOffset, contentVerticalOffset };
+                        };
+                        context.drawBidiText(fontCascade, textRun, altTextLocation());
+                    } else {
+                        // FIXME: TextBoxPainter has this logic already, maybe we should transition to some painter class.
+                        auto contentLogicalHeight = fontMetrics.height();
+                        auto adjustedPaintOffset = LayoutPoint { paintOffset.x(), paintOffset.y() - contentLogicalHeight };
+
+                        auto visualLeft = size().width() / 2 - contentLogicalHeight / 2;
+                        auto visualRight = visualLeft + contentLogicalHeight;
+                        if (style.isFlippedBlocksWritingMode())
+                            visualLeft = size().width() - visualRight;
+                        visualLeft += adjustedPaintOffset.x();
+
+                        auto rotationRect = LayoutRect { visualLeft, adjustedPaintOffset.y(), textWidth, contentLogicalHeight };
+                        context.concatCTM(rotation(rotationRect, Clockwise));
+                        auto textOrigin = LayoutPoint { visualLeft, adjustedPaintOffset.y() + fontCascade.metricsOfPrimaryFont().ascent() };
+                        context.drawBidiText(fontCascade, textRun, textOrigin);
+                        context.concatCTM(rotation(rotationRect, Counterclockwise));
+                    }
                 }
             }
         }
@@ -686,11 +697,6 @@ ImageDrawResult RenderImage::paintIntoRect(PaintInfo& paintInfo, const FloatRect
     // FIXME: Document when image != img.get().
     Image* image = imageResource().image().get();
 
-#if USE(CG)
-    if (is<PDFDocumentImage>(image))
-        downcast<PDFDocumentImage>(*image).setPdfImageCachingPolicy(settings().pdfImageCachingPolicy());
-#endif
-
     if (is<BitmapImage>(image))
         downcast<BitmapImage>(*image).updateFromSettings(settings());
 
@@ -710,6 +716,9 @@ ImageDrawResult RenderImage::paintIntoRect(PaintInfo& paintInfo, const FloatRect
         theme().paintSystemPreviewBadge(*img, paintInfo, rect);
 #endif
 
+    if (element() && !paintInfo.context().paintingDisabled())
+        element()->setHasEverPaintedImages(true);
+
     return drawResult;
 }
 
@@ -897,11 +906,4 @@ RenderBox* RenderImage::embeddedContentBox() const
     return nullptr;
 }
 
-bool RenderImage::allowsAnimation() const
-{
-    if (auto* imageElement = dynamicDowncast<HTMLImageElement>(element()))
-        return imageElement->allowsAnimation();
-    return RenderReplaced::allowsAnimation();
-}
-
 } // namespace WebCore
