@@ -27,22 +27,26 @@
 #include "CSSFontSelector.h"
 #include "CSSValueKeywords.h"
 #include "Chrome.h"
+#include "ElementInlines.h"
 #include "Font.h"
-#include "Frame.h"
 #include "FrameSelection.h"
-#include "FrameView.h"
 #include "HTMLInputElement.h"
 #include "HTMLNames.h"
 #include "HitTestResult.h"
+#include "LocalFrame.h"
+#include "LocalFrameView.h"
 #include "LocalizedStrings.h"
 #include "Page.h"
 #include "PopupMenu.h"
+#include "RenderBoxInlines.h"
+#include "RenderBoxModelObjectInlines.h"
 #include "RenderLayer.h"
 #include "RenderScrollbar.h"
 #include "RenderTheme.h"
 #include "RenderView.h"
 #include "StyleResolver.h"
 #include "TextControlInnerElements.h"
+#include "UnicodeBidi.h"
 #include <wtf/IsoMallocInlines.h>
 
 namespace WebCore {
@@ -54,7 +58,7 @@ WTF_MAKE_ISO_ALLOCATED_IMPL(RenderSearchField);
 RenderSearchField::RenderSearchField(HTMLInputElement& element, RenderStyle&& style)
     : RenderTextControlSingleLine(element, WTFMove(style))
     , m_searchPopupIsVisible(false)
-    , m_searchPopup(0)
+    , m_searchPopup(nullptr)
 {
     ASSERT(element.isSearchField());
 }
@@ -105,7 +109,7 @@ void RenderSearchField::addSearchResult()
     while (static_cast<int>(m_recentSearches.size()) > inputElement().maxResults())
         m_recentSearches.removeLast();
 
-    const AtomicString& name = autosaveName();
+    const AtomString& name = autosaveName();
     if (!m_searchPopup)
         m_searchPopup = page().chrome().createSearchPopupMenu(*this);
 
@@ -125,7 +129,7 @@ void RenderSearchField::showPopup()
 
     m_searchPopupIsVisible = true;
 
-    const AtomicString& name = autosaveName();
+    const AtomString& name = autosaveName();
     m_searchPopup->loadRecentSearches(name, m_recentSearches);
 
     // Trim the recent searches list if the maximum size has changed since we last saved.
@@ -166,6 +170,17 @@ LayoutUnit RenderSearchField::computeControlLogicalHeight(LayoutUnit lineHeight,
 
     return lineHeight + nonContentHeight;
 }
+    
+std::span<const RecentSearch> RenderSearchField::recentSearches()
+{
+    if (!m_searchPopup)
+        m_searchPopup = page().chrome().createSearchPopupMenu(*this);
+
+    const AtomString& name = autosaveName();
+    m_searchPopup->loadRecentSearches(name, m_recentSearches);
+
+    return m_recentSearches.span();
+}
 
 void RenderSearchField::updateFromElement()
 {
@@ -199,9 +214,9 @@ Visibility RenderSearchField::visibilityForCancelButton() const
     return (style().visibility() == Visibility::Hidden || inputElement().value().isEmpty()) ? Visibility::Hidden : Visibility::Visible;
 }
 
-const AtomicString& RenderSearchField::autosaveName() const
+const AtomString& RenderSearchField::autosaveName() const
 {
-    return inputElement().attributeWithoutSynchronization(autosaveAttr);
+    return inputElement().attributeWithoutSynchronization(nameAttr);
 }
 
 // PopupMenuClient methods
@@ -211,7 +226,7 @@ void RenderSearchField::valueChanged(unsigned listIndex, bool fireEvents)
     if (static_cast<int>(listIndex) == (listSize() - 1)) {
         if (fireEvents) {
             m_recentSearches.clear();
-            const AtomicString& name = autosaveName();
+            const AtomString& name = autosaveName();
             if (!name.isEmpty()) {
                 if (!m_searchPopup)
                     m_searchPopup = page().chrome().createSearchPopupMenu(*this);
@@ -354,15 +369,15 @@ FontSelector* RenderSearchField::fontSelector() const
 
 HostWindow* RenderSearchField::hostWindow() const
 {
-    return view().frameView().hostWindow();
+    return RenderTextControlSingleLine::hostWindow();
 }
 
-Ref<Scrollbar> RenderSearchField::createScrollbar(ScrollableArea& scrollableArea, ScrollbarOrientation orientation, ScrollbarControlSize controlSize)
+Ref<Scrollbar> RenderSearchField::createScrollbar(ScrollableArea& scrollableArea, ScrollbarOrientation orientation, ScrollbarWidth widthStyle)
 {
     bool hasCustomScrollbarStyle = style().hasPseudoStyle(PseudoId::Scrollbar);
     if (hasCustomScrollbarStyle)
         return RenderScrollbar::createCustomScrollbar(scrollableArea, orientation, &inputElement());
-    return Scrollbar::createNativeScrollbar(scrollableArea, orientation, controlSize);
+    return Scrollbar::createNativeScrollbar(scrollableArea, orientation, widthStyle);
 }
 
 }
