@@ -34,9 +34,9 @@
 #include "FloatRoundedRect.h"
 #include "FocusController.h"
 #include "FontSelector.h"
-#include "Frame.h"
 #include "FrameSelection.h"
 #include "GraphicsContext.h"
+#include "GraphicsTypes.h"
 #include "HTMLAttachmentElement.h"
 #include "HTMLButtonElement.h"
 #include "HTMLInputElement.h"
@@ -47,6 +47,7 @@
 #include "HTMLTextAreaElement.h"
 #include "ImageControlsButtonPart.h"
 #include "InnerSpinButtonPart.h"
+#include "LocalFrame.h"
 #include "LocalizedStrings.h"
 #include "MenuListButtonPart.h"
 #include "MenuListPart.h"
@@ -56,10 +57,11 @@
 #include "ProgressBarPart.h"
 #include "RenderMeter.h"
 #include "RenderProgress.h"
-#include "RenderStyle.h"
+#include "RenderStyleSetters.h"
 #include "RenderView.h"
 #include "SearchFieldCancelButtonPart.h"
 #include "SearchFieldPart.h"
+#include "SearchFieldResultsPart.h"
 #include "ShadowPseudoIds.h"
 #include "SliderThumbElement.h"
 #include "SliderThumbPart.h"
@@ -70,7 +72,9 @@
 #include "TextControlInnerElements.h"
 #include "TextFieldPart.h"
 #include "ToggleButtonPart.h"
+#include "TypedElementDescendantIteratorInlines.h"
 #include <wtf/FileSystem.h>
+#include <wtf/Language.h>
 #include <wtf/NeverDestroyed.h>
 #include <wtf/text/StringConcatenateNumbers.h>
 
@@ -484,11 +488,23 @@ StyleAppearance RenderTheme::autoAppearanceForElement(RenderStyle& style, const
     return StyleAppearance::None;
 }
 
-static RefPtr<ControlPart> createMeterPartForRenderer(const RenderObject& renderer)
+#if ENABLE(APPLE_PAY)
+static void updateApplePayButtonPartForRenderer(ApplePayButtonPart& applePayButtonPart, const RenderObject& renderer)
 {
-    ASSERT(is<RenderMeter>(renderer));
-    const auto& renderMeter = downcast<RenderMeter>(renderer);
+    auto& style = renderer.style();
 
+    String locale = style.computedLocale();
+    if (locale.isEmpty())
+        locale = defaultLanguage(ShouldMinimizeLanguages::No);
+
+    applePayButtonPart.setButtonType(style.applePayButtonType());
+    applePayButtonPart.setButtonStyle(style.applePayButtonStyle());
+    applePayButtonPart.setLocale(locale);
+}
+#endif
+
+static void updateMeterPartForRenderer(MeterPart& meterPart, const RenderMeter& renderMeter)
+{
     auto element = renderMeter.meterElement();
     MeterPart::GaugeRegion gaugeRegion;
 
@@ -504,21 +520,20 @@ static RefPtr<ControlPart> createMeterPartForRenderer(const RenderObject& render
         break;
     }
 
-    return MeterPart::create(gaugeRegion, element->value(), element->min(), element->max());
+    meterPart.setGaugeRegion(gaugeRegion);
+    meterPart.setValue(element->value());
+    meterPart.setMinimum(element->min());
+    meterPart.setMaximum(element->max());
 }
 
-static RefPtr<ControlPart> createProgressBarPartForRenderer(const RenderObject& renderer)
+static void updateProgressBarPartForRenderer(ProgressBarPart& progressBarPart, const RenderProgress& renderProgress)
 {
-    ASSERT(is<RenderProgress>(renderer));
-    const auto& renderProgress = downcast<RenderProgress>(renderer);
-    return ProgressBarPart::create(renderProgress.position(), renderProgress.animationStartTime().secondsSinceEpoch());
+    progressBarPart.setPosition(renderProgress.position());
+    progressBarPart.setAnimationStartTime(renderProgress.animationStartTime().secondsSinceEpoch());
 }
 
-static RefPtr<ControlPart> createSliderTrackPartForRenderer(const RenderObject& renderer)
+static void updateSliderTrackPartForRenderer(SliderTrackPart& sliderTrackPart, const RenderObject& renderer)
 {
-    auto type = renderer.style().effectiveAppearance();
-    ASSERT(type == StyleAppearance::SliderHorizontal || type == StyleAppearance::SliderVertical);
-
     ASSERT(is<HTMLInputElement>(renderer.node()));
     auto& input = downcast<HTMLInputElement>(*renderer.node());
     ASSERT(input.isRangeControl());
@@ -555,7 +570,10 @@ static RefPtr<ControlPart> createSliderTrackPartForRenderer(const RenderObject&
         }
     }
 #endif
-    return SliderTrackPart::create(type, thumbSize, trackBounds, WTFMove(tickRatios));
+
+    sliderTrackPart.setThumbSize(thumbSize);
+    sliderTrackPart.setTrackBounds(trackBounds);
+    sliderTrackPart.setTickRatios(WTFMove(tickRatios));
 }
 
 RefPtr<ControlPart> RenderTheme::createControlPart(const RenderObject& renderer) const
@@ -584,21 +602,21 @@ RefPtr<ControlPart> RenderTheme::createControlPart(const RenderObject& renderer)
         return MenuListButtonPart::create();
 
     case StyleAppearance::Meter:
-        return createMeterPartForRenderer(renderer);
+        return MeterPart::create();
 
     case StyleAppearance::ProgressBar:
-        return createProgressBarPartForRenderer(renderer);
+        return ProgressBarPart::create();
 
     case StyleAppearance::SliderHorizontal:
     case StyleAppearance::SliderVertical:
-        return createSliderTrackPartForRenderer(renderer);
+        return SliderTrackPart::create(appearance);
 
     case StyleAppearance::SearchField:
         return SearchFieldPart::create();
 
 #if ENABLE(APPLE_PAY)
     case StyleAppearance::ApplePayButton:
-        break;
+        return ApplePayButtonPart::create();
 #endif
 #if ENABLE(ATTACHMENT_ELEMENT)
     case StyleAppearance::Attachment:
@@ -634,9 +652,11 @@ RefPtr<ControlPart> RenderTheme::createControlPart(const RenderObject& renderer)
 #endif
 
     case StyleAppearance::SearchFieldDecoration:
+        break;
+
     case StyleAppearance::SearchFieldResultsDecoration:
     case StyleAppearance::SearchFieldResultsButton:
-        break;
+        return SearchFieldResultsPart::create(appearance);
 
     case StyleAppearance::SearchFieldCancelButton:
         return SearchFieldCancelButtonPart::create();
@@ -650,6 +670,33 @@ RefPtr<ControlPart> RenderTheme::createControlPart(const RenderObject& renderer)
     return nullptr;
 }
 
+void RenderTheme::updateControlPartForRenderer(ControlPart& part, const RenderObject& renderer) const
+{
+    if (auto* meterPart = dynamicDowncast<MeterPart>(part)) {
+        ASSERT(is<RenderMeter>(renderer));
+        updateMeterPartForRenderer(*meterPart, downcast<RenderMeter>(renderer));
+        return;
+    }
+
+    if (auto* progressBarPart = dynamicDowncast<ProgressBarPart>(part)) {
+        ASSERT(is<RenderProgress>(renderer));
+        updateProgressBarPartForRenderer(*progressBarPart, downcast<RenderProgress>(renderer));
+        return;
+    }
+
+    if (auto* sliderTrackPart = dynamicDowncast<SliderTrackPart>(part)) {
+        updateSliderTrackPartForRenderer(*sliderTrackPart, renderer);
+        return;
+    }
+
+#if ENABLE(APPLE_PAY)
+    if (auto* applePayButtonPart = dynamicDowncast<ApplePayButtonPart>(part)) {
+        updateApplePayButtonPartForRenderer(*applePayButtonPart, renderer);
+        return;
+    }
+#endif
+}
+
 OptionSet<ControlStyle::State> RenderTheme::extractControlStyleStatesForRenderer(const RenderObject& renderer) const
 {
     OptionSet<ControlStyle::State> states;
@@ -739,6 +786,8 @@ bool RenderTheme::paint(const RenderBox& box, ControlPart& part, const PaintInfo
     if (paintInfo.context().paintingDisabled())
         return false;
 
+    updateControlPartForRenderer(part, box);
+
     float deviceScaleFactor = box.document().deviceScaleFactor();
     auto zoomedRect = snapRectToDevicePixels(rect, deviceScaleFactor);
     auto borderRect = FloatRoundedRect(box.style().getRoundedBorderFor(LayoutRect(zoomedRect)));
@@ -1248,7 +1297,7 @@ bool RenderTheme::isFocused(const RenderObject& renderer) const
         delegate = downcast<SliderThumbElement>(*element).hostInput();
 
     Document& document = delegate->document();
-    Frame* frame = document.frame();
+    auto* frame = document.frame();
     return delegate == document.focusedElement() && frame && frame->selection().isFocusedAndActive();
 }
 
@@ -1945,31 +1994,6 @@ Color RenderTheme::tapHighlightColor()
 
 #endif
 
-// Value chosen by observation. This can be tweaked.
-constexpr double minColorContrastValue = 1.195;
-
-// For transparent or translucent background color, use lightening.
-constexpr float minDisabledColorAlphaValue = 0.5f;
-
-Color RenderTheme::disabledTextColor(const Color& textColor, const Color& backgroundColor) const
-{
-    // The explicit check for black is an optimization for the 99% case (black on white).
-    // This also means that black on black will turn into grey on black when disabled.
-    Color disabledColor;
-    if (equalIgnoringSemanticColor(textColor, Color::black) || backgroundColor.alphaAsFloat() < minDisabledColorAlphaValue || textColor.luminance() < backgroundColor.luminance())
-        disabledColor = textColor.lightened();
-    else
-        disabledColor = textColor.darkened();
-
-    // If there's not very much contrast between the disabled color and the background color,
-    // just leave the text color alone. We don't want to change a good contrast color scheme so that it has really bad contrast.
-    // If the contrast was already poor, then it doesn't do any good to change it to a different poor contrast color scheme.
-    if (contrastRatio(disabledColor, backgroundColor) < minColorContrastValue)
-        return textColor;
-
-    return disabledColor;
-}
-
 // Value chosen to return dark gray for both white on black and black on white.
 constexpr float datePlaceholderColorLightnessAdjustmentFactor = 0.66f;
 
@@ -1986,6 +2010,80 @@ Color RenderTheme::datePlaceholderTextColor(const Color& textColor, const Color&
     return convertColor<SRGBA<float>>(hsla);
 }
 
+
+Color RenderTheme::spellingMarkerColor(OptionSet<StyleColorOptions> options) const
+{
+    auto& cache = colorCache(options);
+    if (!cache.spellingMarkerColor.isValid())
+        cache.spellingMarkerColor = platformSpellingMarkerColor(options);
+    return cache.spellingMarkerColor;
+}
+
+Color RenderTheme::platformSpellingMarkerColor(OptionSet<StyleColorOptions>) const
+{
+    return Color::red;
+}
+
+Color RenderTheme::dictationAlternativesMarkerColor(OptionSet<StyleColorOptions> options) const
+{
+    auto& cache = colorCache(options);
+    if (!cache.dictationAlternativesMarkerColor.isValid())
+        cache.dictationAlternativesMarkerColor = platformDictationAlternativesMarkerColor(options);
+    return cache.dictationAlternativesMarkerColor;
+}
+
+Color RenderTheme::platformDictationAlternativesMarkerColor(OptionSet<StyleColorOptions>) const
+{
+    return Color::green;
+}
+
+Color RenderTheme::autocorrectionReplacementMarkerColor(const RenderText& renderer) const
+{
+    auto options = renderer.styleColorOptions();
+    auto& cache = colorCache(options);
+    if (!cache.autocorrectionReplacementMarkerColor.isValid())
+        cache.autocorrectionReplacementMarkerColor = platformAutocorrectionReplacementMarkerColor(options);
+    return cache.autocorrectionReplacementMarkerColor;
+}
+
+Color RenderTheme::platformAutocorrectionReplacementMarkerColor(OptionSet<StyleColorOptions>) const
+{
+    return Color::green;
+}
+
+Color RenderTheme::grammarMarkerColor(OptionSet<StyleColorOptions> options) const
+{
+    auto& cache = colorCache(options);
+    if (!cache.grammarMarkerColor.isValid())
+        cache.grammarMarkerColor = platformGrammarMarkerColor(options);
+    return cache.grammarMarkerColor;
+}
+
+Color RenderTheme::platformGrammarMarkerColor(OptionSet<StyleColorOptions>) const
+{
+    return Color::green;
+}
+
+Color RenderTheme::documentMarkerLineColor(const RenderText& renderer, DocumentMarkerLineStyleMode mode) const
+{
+    auto options = renderer.styleColorOptions();
+
+    switch (mode) {
+    case DocumentMarkerLineStyleMode::Spelling:
+        return spellingMarkerColor(options);
+    case DocumentMarkerLineStyleMode::DictationAlternatives:
+    case DocumentMarkerLineStyleMode::TextCheckingDictationPhraseWithAlternatives:
+        return dictationAlternativesMarkerColor(options);
+    case DocumentMarkerLineStyleMode::AutocorrectionReplacement:
+        return autocorrectionReplacementMarkerColor(renderer);
+    case DocumentMarkerLineStyleMode::Grammar:
+        return grammarMarkerColor(options);
+    }
+
+    ASSERT_NOT_REACHED();
+    return Color::transparentBlack;
+}
+
 void RenderTheme::setCustomFocusRingColor(const Color& color)
 {
     customFocusRingColor() = color;
