@@ -28,9 +28,9 @@
  */
 
 #include "config.h"
-
 #include "BasicShapes.h"
 
+#include "AnimationUtilities.h"
 #include "BasicShapeFunctions.h"
 #include "CalculationValue.h"
 #include "FloatRect.h"
@@ -40,7 +40,7 @@
 #include "RenderBox.h"
 #include "SVGPathByteStream.h"
 #include "SVGPathUtilities.h"
-
+#include <wtf/MathExtras.h>
 #include <wtf/NeverDestroyed.h>
 #include <wtf/TinyLRUCache.h>
 #include <wtf/text/TextStream.h>
@@ -49,38 +49,38 @@ namespace WebCore {
 
 void BasicShapeCenterCoordinate::updateComputedLength()
 {
-    if (m_direction == TopLeft) {
-        m_computedLength = m_length.isUndefined() ? Length(0, Fixed) : m_length;
+    if (m_direction == BasicShapeCenterCoordinate::Direction::TopLeft) {
+        m_computedLength = m_length.isUndefined() ? Length(0, LengthType::Fixed) : m_length;
         return;
     }
 
     if (m_length.isUndefined()) {
-        m_computedLength = Length(100, Percent);
+        m_computedLength = Length(100, LengthType::Percent);
         return;
     }
     
     m_computedLength = convertTo100PercentMinusLength(m_length);
 }
 
-struct SVGPathTranslatedByteStream {
-    SVGPathTranslatedByteStream(const FloatPoint& offset, const SVGPathByteStream& rawStream)
-        : m_offset(offset)
-        , m_rawStream(rawStream)
-    { }
-
-    bool operator==(const SVGPathTranslatedByteStream& other) const { return other.m_offset == m_offset && other.m_rawStream == m_rawStream; }
-    bool operator!=(const SVGPathTranslatedByteStream& other) const { return !(*this == other); }
-    bool isEmpty() const { return m_rawStream.isEmpty(); }
+struct SVGPathTransformedByteStream {
+    bool operator==(const SVGPathTransformedByteStream& other) const 
+    { 
+        return other.offset == offset && other.zoom == zoom && other.rawStream == rawStream;
+    }
+    bool isEmpty() const { return rawStream.isEmpty(); }
 
     Path path() const
     {
-        Path path = buildPathFromByteStream(m_rawStream);
-        path.translate(toFloatSize(m_offset));
+        Path path = buildPathFromByteStream(rawStream);
+        if (zoom != 1)
+            path.transform(AffineTransform().scale(zoom));
+        path.translate(toFloatSize(offset));
         return path;
     }
     
-    FloatPoint m_offset;
-    SVGPathByteStream m_rawStream;
+    SVGPathByteStream rawStream;
+    float zoom;
+    FloatPoint offset;
 };
 
 struct EllipsePathPolicy : public TinyLRUCachePolicy<FloatRect, Path> {
@@ -107,18 +107,17 @@ public:
     }
 };
 
-struct PolygonPathPolicy : public TinyLRUCachePolicy<Vector<FloatPoint>, Path> {
+struct PolygonPathPolicy : TinyLRUCachePolicy<Vector<FloatPoint>, Path> {
 public:
     static bool isKeyNull(const Vector<FloatPoint>& points) { return !points.size(); }
 
     static Path createValueForKey(const Vector<FloatPoint>& points) { return Path::polygonPathFromPoints(points); }
 };
 
-struct TranslatedByteStreamPathPolicy : public TinyLRUCachePolicy<SVGPathTranslatedByteStream, Path> {
-public:
-    static bool isKeyNull(const SVGPathTranslatedByteStream& stream) { return stream.isEmpty(); }
+struct TransformedByteStreamPathPolicy : TinyLRUCachePolicy<SVGPathTransformedByteStream, Path> {
+    static bool isKeyNull(const SVGPathTransformedByteStream& stream) { return stream.isEmpty(); }
 
-    static Path createValueForKey(const SVGPathTranslatedByteStream& stream) { return stream.path(); }
+    static Path createValueForKey(const SVGPathTransformedByteStream& stream) { return stream.path(); }
 };
 
 static const Path& cachedEllipsePath(const FloatRect& rect)
@@ -139,10 +138,30 @@ static const Path& cachedPolygonPath(const Vector<FloatPoint>& points)
     return cache.get().get(points);
 }
 
-static const Path& cachedTranslatedByteStreamPath(const SVGPathByteStream& stream, const FloatPoint& offset)
+static const Path& cachedTransformedByteStreamPath(const SVGPathByteStream& stream, float zoom, const FloatPoint& offset)
 {
-    static NeverDestroyed<TinyLRUCache<SVGPathTranslatedByteStream, Path, 4, TranslatedByteStreamPathPolicy>> cache;
-    return cache.get().get(SVGPathTranslatedByteStream(offset, stream));
+    static NeverDestroyed<TinyLRUCache<SVGPathTransformedByteStream, Path, 4, TransformedByteStreamPathPolicy>> cache;
+    return cache.get().get(SVGPathTransformedByteStream { stream, zoom, offset });
+}
+
+Ref<BasicShapeCircle> BasicShapeCircle::create(BasicShapeCenterCoordinate&& centerX, BasicShapeCenterCoordinate&& centerY, BasicShapeRadius&& radius)
+{
+    return adoptRef(*new BasicShapeCircle(WTFMove(centerX), WTFMove(centerY), WTFMove(radius)));
+}
+
+BasicShapeCircle::BasicShapeCircle(BasicShapeCenterCoordinate&& centerX, BasicShapeCenterCoordinate&& centerY, BasicShapeRadius&& radius)
+    : m_centerX(WTFMove(centerX))
+    , m_centerY(WTFMove(centerY))
+    , m_radius(WTFMove(radius))
+{
+}
+
+Ref<BasicShape> BasicShapeCircle::clone() const
+{
+    auto centerX = m_centerX;
+    auto centerY = m_centerY;
+    auto radius = m_radius;
+    return adoptRef(*new BasicShapeCircle(WTFMove(centerX), WTFMove(centerY), WTFMove(radius)));
 }
 
 bool BasicShapeCircle::operator==(const BasicShape& other) const
@@ -158,18 +177,18 @@ bool BasicShapeCircle::operator==(const BasicShape& other) const
 
 float BasicShapeCircle::floatValueForRadiusInBox(float boxWidth, float boxHeight) const
 {
-    if (m_radius.type() == BasicShapeRadius::Value)
-        return floatValueForLength(m_radius.value(), sqrtf((boxWidth * boxWidth + boxHeight * boxHeight) / 2));
+    if (m_radius.type() == BasicShapeRadius::Type::Value)
+        return floatValueForLength(m_radius.value(), std::hypot(boxWidth, boxHeight) / sqrtOfTwoFloat);
 
     float centerX = floatValueForCenterCoordinate(m_centerX, boxWidth);
     float centerY = floatValueForCenterCoordinate(m_centerY, boxHeight);
 
     float widthDelta = std::abs(boxWidth - centerX);
     float heightDelta = std::abs(boxHeight - centerY);
-    if (m_radius.type() == BasicShapeRadius::ClosestSide)
+    if (m_radius.type() == BasicShapeRadius::Type::ClosestSide)
         return std::min(std::min(std::abs(centerX), widthDelta), std::min(std::abs(centerY), heightDelta));
 
-    // If radius.type() == BasicShapeRadius::FarthestSide.
+    // If radius.type() == BasicShapeRadius::Type::FarthestSide.
     return std::max(std::max(std::abs(centerX), widthDelta), std::max(std::abs(centerY), heightDelta));
 }
 
@@ -190,15 +209,15 @@ bool BasicShapeCircle::canBlend(const BasicShape& other) const
     return radius().canBlend(downcast<BasicShapeCircle>(other).radius());
 }
 
-Ref<BasicShape> BasicShapeCircle::blend(const BasicShape& other, double progress) const
+Ref<BasicShape> BasicShapeCircle::blend(const BasicShape& other, const BlendingContext& context) const
 {
     ASSERT(type() == other.type());
     auto& otherCircle = downcast<BasicShapeCircle>(other);
     auto result =  BasicShapeCircle::create();
 
-    result->setCenterX(m_centerX.blend(otherCircle.centerX(), progress));
-    result->setCenterY(m_centerY.blend(otherCircle.centerY(), progress));
-    result->setRadius(m_radius.blend(otherCircle.radius(), progress));
+    result->setCenterX(m_centerX.blend(otherCircle.centerX(), context));
+    result->setCenterY(m_centerY.blend(otherCircle.centerY(), context));
+    result->setRadius(m_radius.blend(otherCircle.radius(), context));
     return result;
 }
 
@@ -209,6 +228,28 @@ void BasicShapeCircle::dump(TextStream& ts) const
     ts.dumpProperty("radius", radius());
 }
 
+Ref<BasicShapeEllipse> BasicShapeEllipse::create(BasicShapeCenterCoordinate&& centerX, BasicShapeCenterCoordinate&& centerY, BasicShapeRadius&& radiusX, BasicShapeRadius&& radiusY)
+{
+    return adoptRef(*new BasicShapeEllipse(WTFMove(centerX), WTFMove(centerY), WTFMove(radiusX), WTFMove(radiusY)));
+}
+
+BasicShapeEllipse::BasicShapeEllipse(BasicShapeCenterCoordinate&& centerX, BasicShapeCenterCoordinate&& centerY, BasicShapeRadius&& radiusX, BasicShapeRadius&& radiusY)
+    : m_centerX(WTFMove(centerX))
+    , m_centerY(WTFMove(centerY))
+    , m_radiusX(WTFMove(radiusX))
+    , m_radiusY(WTFMove(radiusY))
+{
+}
+
+Ref<BasicShape> BasicShapeEllipse::clone() const
+{
+    auto centerX = m_centerX;
+    auto centerY = m_centerY;
+    auto radiusX = m_radiusX;
+    auto radiusY = m_radiusY;
+    return adoptRef(*new BasicShapeEllipse(WTFMove(centerX), WTFMove(centerY), WTFMove(radiusX), WTFMove(radiusY)));
+}
+
 bool BasicShapeEllipse::operator==(const BasicShape& other) const
 {
     if (type() != other.type())
@@ -223,14 +264,14 @@ bool BasicShapeEllipse::operator==(const BasicShape& other) const
 
 float BasicShapeEllipse::floatValueForRadiusInBox(const BasicShapeRadius& radius, float center, float boxWidthOrHeight) const
 {
-    if (radius.type() == BasicShapeRadius::Value)
+    if (radius.type() == BasicShapeRadius::Type::Value)
         return floatValueForLength(radius.value(), std::abs(boxWidthOrHeight));
 
     float widthOrHeightDelta = std::abs(boxWidthOrHeight - center);
-    if (radius.type() == BasicShapeRadius::ClosestSide)
+    if (radius.type() == BasicShapeRadius::Type::ClosestSide)
         return std::min(std::abs(center), widthOrHeightDelta);
 
-    ASSERT(radius.type() == BasicShapeRadius::FarthestSide);
+    ASSERT(radius.type() == BasicShapeRadius::Type::FarthestSide);
     return std::max(std::abs(center), widthOrHeightDelta);
 }
 
@@ -253,14 +294,14 @@ bool BasicShapeEllipse::canBlend(const BasicShape& other) const
     return radiusX().canBlend(otherEllipse.radiusX()) && radiusY().canBlend(otherEllipse.radiusY());
 }
 
-Ref<BasicShape> BasicShapeEllipse::blend(const BasicShape& other, double progress) const
+Ref<BasicShape> BasicShapeEllipse::blend(const BasicShape& other, const BlendingContext& context) const
 {
     ASSERT(type() == other.type());
     auto& otherEllipse = downcast<BasicShapeEllipse>(other);
     auto result = BasicShapeEllipse::create();
 
-    if (m_radiusX.type() != BasicShapeRadius::Value || otherEllipse.radiusX().type() != BasicShapeRadius::Value
-        || m_radiusY.type() != BasicShapeRadius::Value || otherEllipse.radiusY().type() != BasicShapeRadius::Value) {
+    if (m_radiusX.type() != BasicShapeRadius::Type::Value || otherEllipse.radiusX().type() != BasicShapeRadius::Type::Value
+        || m_radiusY.type() != BasicShapeRadius::Type::Value || otherEllipse.radiusY().type() != BasicShapeRadius::Type::Value) {
         result->setCenterX(otherEllipse.centerX());
         result->setCenterY(otherEllipse.centerY());
         result->setRadiusX(otherEllipse.radiusX());
@@ -268,10 +309,10 @@ Ref<BasicShape> BasicShapeEllipse::blend(const BasicShape& other, double progres
         return result;
     }
 
-    result->setCenterX(m_centerX.blend(otherEllipse.centerX(), progress));
-    result->setCenterY(m_centerY.blend(otherEllipse.centerY(), progress));
-    result->setRadiusX(m_radiusX.blend(otherEllipse.radiusX(), progress));
-    result->setRadiusY(m_radiusY.blend(otherEllipse.radiusY(), progress));
+    result->setCenterX(m_centerX.blend(otherEllipse.centerX(), context));
+    result->setCenterY(m_centerY.blend(otherEllipse.centerY(), context));
+    result->setRadiusX(m_radiusX.blend(otherEllipse.radiusX(), context));
+    result->setRadiusY(m_radiusY.blend(otherEllipse.radiusY(), context));
     return result;
 }
 
@@ -283,6 +324,23 @@ void BasicShapeEllipse::dump(TextStream& ts) const
     ts.dumpProperty("radius-y", radiusY());
 }
 
+Ref<BasicShapePolygon> BasicShapePolygon::create(WindRule windRule, Vector<Length>&& values)
+{
+    return adoptRef(*new BasicShapePolygon(windRule, WTFMove(values)));
+}
+
+BasicShapePolygon::BasicShapePolygon(WindRule windRule, Vector<Length>&& values)
+    : m_windRule(windRule)
+    , m_values(WTFMove(values))
+{
+}
+
+Ref<BasicShape> BasicShapePolygon::clone() const
+{
+    auto values = m_values;
+    return adoptRef(*new BasicShapePolygon(m_windRule, WTFMove(values)));
+}
+
 bool BasicShapePolygon::operator==(const BasicShape& other) const
 {
     if (type() != other.type())
@@ -316,7 +374,7 @@ bool BasicShapePolygon::canBlend(const BasicShape& other) const
     return values().size() == otherPolygon.values().size() && windRule() == otherPolygon.windRule();
 }
 
-Ref<BasicShape> BasicShapePolygon::blend(const BasicShape& other, double progress) const
+Ref<BasicShape> BasicShapePolygon::blend(const BasicShape& other, const BlendingContext& context) const
 {
     ASSERT(type() == other.type());
 
@@ -333,8 +391,8 @@ Ref<BasicShape> BasicShapePolygon::blend(const BasicShape& other, double progres
 
     for (size_t i = 0; i < length; i = i + 2) {
         result->appendPoint(
-            WebCore::blend(otherPolygon.values().at(i), m_values.at(i), progress),
-            WebCore::blend(otherPolygon.values().at(i + 1), m_values.at(i + 1), progress));
+            WebCore::blend(otherPolygon.values().at(i), m_values.at(i), context),
+            WebCore::blend(otherPolygon.values().at(i + 1), m_values.at(i + 1), context));
     }
 
     return result;
@@ -346,14 +404,34 @@ void BasicShapePolygon::dump(TextStream& ts) const
     ts.dumpProperty("path", values());
 }
 
+Ref<BasicShapePath> BasicShapePath::create(std::unique_ptr<SVGPathByteStream>&& byteStream, float zoom, WindRule windRule)
+{
+    return adoptRef(*new BasicShapePath(WTFMove(byteStream), zoom, windRule));
+}
+
 BasicShapePath::BasicShapePath(std::unique_ptr<SVGPathByteStream>&& byteStream)
     : m_byteStream(WTFMove(byteStream))
 {
 }
 
+BasicShapePath::BasicShapePath(std::unique_ptr<SVGPathByteStream>&& byteStream, float zoom, WindRule windRule)
+    : m_byteStream(WTFMove(byteStream))
+    , m_zoom(zoom)
+    , m_windRule(windRule)
+{
+}
+
+Ref<BasicShape> BasicShapePath::clone() const
+{
+    std::unique_ptr<SVGPathByteStream> byteStream;
+    if (m_byteStream)
+        byteStream = m_byteStream->copy();
+    return adoptRef(*new BasicShapePath(WTFMove(byteStream), m_zoom, m_windRule));
+}
+
 const Path& BasicShapePath::path(const FloatRect& boundingBox)
 {
-    return cachedTranslatedByteStreamPath(*m_byteStream, boundingBox.location());
+    return cachedTransformedByteStreamPath(*m_byteStream, m_zoom, boundingBox.location());
 }
 
 bool BasicShapePath::operator==(const BasicShape& other) const
@@ -362,7 +440,7 @@ bool BasicShapePath::operator==(const BasicShape& other) const
         return false;
 
     auto& otherPath = downcast<BasicShapePath>(other);
-    return m_windRule == otherPath.m_windRule && *m_byteStream == *otherPath.m_byteStream;
+    return m_zoom == otherPath.m_zoom && m_windRule == otherPath.m_windRule && *m_byteStream == *otherPath.m_byteStream;
 }
 
 bool BasicShapePath::canBlend(const BasicShape& other) const
@@ -374,17 +452,17 @@ bool BasicShapePath::canBlend(const BasicShape& other) const
     return windRule() == otherPath.windRule() && canBlendSVGPathByteStreams(*m_byteStream, *otherPath.pathData());
 }
 
-Ref<BasicShape> BasicShapePath::blend(const BasicShape& from, double progress) const
+Ref<BasicShape> BasicShapePath::blend(const BasicShape& from, const BlendingContext& context) const
 {
     ASSERT(type() == from.type());
 
     auto& fromPath = downcast<BasicShapePath>(from);
 
     auto resultingPathBytes = makeUnique<SVGPathByteStream>();
-    buildAnimatedSVGPathByteStream(*fromPath.m_byteStream, *m_byteStream, *resultingPathBytes, progress);
-
+    buildAnimatedSVGPathByteStream(*fromPath.m_byteStream, *m_byteStream, *resultingPathBytes, context.progress);
     auto result = BasicShapePath::create(WTFMove(resultingPathBytes));
     result->setWindRule(windRule());
+    result->setZoom(m_zoom);
     return result;
 }
 
@@ -394,6 +472,36 @@ void BasicShapePath::dump(TextStream& ts) const
     // FIXME: print the byte stream?
 }
 
+Ref<BasicShapeInset> BasicShapeInset::create(Length&& right, Length&& top, Length&& bottom, Length&& left, LengthSize&& topLeftRadius, LengthSize&& topRightRadius, LengthSize&& bottomRightRadius, LengthSize&& bottomLeftRadius)
+{
+    return adoptRef(*new BasicShapeInset(WTFMove(right), WTFMove(top), WTFMove(bottom), WTFMove(left), WTFMove(topLeftRadius), WTFMove(topRightRadius), WTFMove(bottomRightRadius), WTFMove(bottomLeftRadius)));
+}
+
+BasicShapeInset::BasicShapeInset(Length&& right, Length&& top, Length&& bottom, Length&& left, LengthSize&& topLeftRadius, LengthSize&& topRightRadius, LengthSize&& bottomRightRadius, LengthSize&& bottomLeftRadius)
+    : m_right(WTFMove(right))
+    , m_top(WTFMove(top))
+    , m_bottom(WTFMove(bottom))
+    , m_left(WTFMove(left))
+    , m_topLeftRadius(WTFMove(topLeftRadius))
+    , m_topRightRadius(WTFMove(topRightRadius))
+    , m_bottomRightRadius(WTFMove(bottomRightRadius))
+    , m_bottomLeftRadius(WTFMove(bottomLeftRadius))
+{
+}
+
+Ref<BasicShape> BasicShapeInset::clone() const
+{
+    auto right = m_right;
+    auto top = m_top;
+    auto bottom = m_bottom;
+    auto left = m_left;
+    auto topLeftRadius = m_topLeftRadius;
+    auto topRightRadius = m_topRightRadius;
+    auto bottomRightRadius = m_bottomRightRadius;
+    auto bottomLeftRadius = m_bottomLeftRadius;
+    return adoptRef(*new BasicShapeInset(WTFMove(right), WTFMove(top), WTFMove(bottom), WTFMove(left), WTFMove(topLeftRadius), WTFMove(topRightRadius), WTFMove(bottomRightRadius), WTFMove(bottomLeftRadius)));
+}
+
 bool BasicShapeInset::operator==(const BasicShape& other) const
 {
     if (type() != other.type())
@@ -410,12 +518,6 @@ bool BasicShapeInset::operator==(const BasicShape& other) const
         && m_bottomLeftRadius == otherInset.m_bottomLeftRadius;
 }
 
-static FloatSize floatSizeForLengthSize(const LengthSize& lengthSize, const FloatRect& boundingBox)
-{
-    return { floatValueForLength(lengthSize.width, boundingBox.width()),
-        floatValueForLength(lengthSize.height, boundingBox.height()) };
-}
-
 const Path& BasicShapeInset::path(const FloatRect& boundingBox)
 {
     float left = floatValueForLength(m_left, boundingBox.width());
@@ -423,10 +525,10 @@ const Path& BasicShapeInset::path(const FloatRect& boundingBox)
     auto rect = FloatRect(left + boundingBox.x(), top + boundingBox.y(),
         std::max<float>(boundingBox.width() - left - floatValueForLength(m_right, boundingBox.width()), 0),
         std::max<float>(boundingBox.height() - top - floatValueForLength(m_bottom, boundingBox.height()), 0));
-    auto radii = FloatRoundedRect::Radii(floatSizeForLengthSize(m_topLeftRadius, boundingBox),
-        floatSizeForLengthSize(m_topRightRadius, boundingBox),
-        floatSizeForLengthSize(m_bottomLeftRadius, boundingBox),
-        floatSizeForLengthSize(m_bottomRightRadius, boundingBox));
+    auto radii = FloatRoundedRect::Radii(floatSizeForLengthSize(m_topLeftRadius, boundingBox.size()),
+        floatSizeForLengthSize(m_topRightRadius, boundingBox.size()),
+        floatSizeForLengthSize(m_bottomLeftRadius, boundingBox.size()),
+        floatSizeForLengthSize(m_bottomRightRadius, boundingBox.size()));
     radii.scale(calcBorderRadiiConstraintScaleFor(rect, radii));
 
     return cachedRoundedRectPath(FloatRoundedRect(rect, radii));
@@ -437,21 +539,21 @@ bool BasicShapeInset::canBlend(const BasicShape& other) const
     return type() == other.type();
 }
 
-Ref<BasicShape> BasicShapeInset::blend(const BasicShape& from, double progress) const
+Ref<BasicShape> BasicShapeInset::blend(const BasicShape& from, const BlendingContext& context) const
 {
     ASSERT(type() == from.type());
 
     auto& fromInset = downcast<BasicShapeInset>(from);
     auto result =  BasicShapeInset::create();
-    result->setTop(WebCore::blend(fromInset.top(), top(), progress));
-    result->setRight(WebCore::blend(fromInset.right(), right(), progress));
-    result->setBottom(WebCore::blend(fromInset.bottom(), bottom(), progress));
-    result->setLeft(WebCore::blend(fromInset.left(), left(), progress));
+    result->setTop(WebCore::blend(fromInset.top(), top(), context));
+    result->setRight(WebCore::blend(fromInset.right(), right(), context));
+    result->setBottom(WebCore::blend(fromInset.bottom(), bottom(), context));
+    result->setLeft(WebCore::blend(fromInset.left(), left(), context));
 
-    result->setTopLeftRadius(WebCore::blend(fromInset.topLeftRadius(), topLeftRadius(), progress));
-    result->setTopRightRadius(WebCore::blend(fromInset.topRightRadius(), topRightRadius(), progress));
-    result->setBottomRightRadius(WebCore::blend(fromInset.bottomRightRadius(), bottomRightRadius(), progress));
-    result->setBottomLeftRadius(WebCore::blend(fromInset.bottomLeftRadius(), bottomLeftRadius(), progress));
+    result->setTopLeftRadius(WebCore::blend(fromInset.topLeftRadius(), topLeftRadius(), context, ValueRange::NonNegative));
+    result->setTopRightRadius(WebCore::blend(fromInset.topRightRadius(), topRightRadius(), context, ValueRange::NonNegative));
+    result->setBottomRightRadius(WebCore::blend(fromInset.bottomRightRadius(), bottomRightRadius(), context, ValueRange::NonNegative));
+    result->setBottomLeftRadius(WebCore::blend(fromInset.bottomLeftRadius(), bottomLeftRadius(), context, ValueRange::NonNegative));
 
     return result;
 }
@@ -472,9 +574,9 @@ void BasicShapeInset::dump(TextStream& ts) const
 static TextStream& operator<<(TextStream& ts, BasicShapeRadius::Type radiusType)
 {
     switch (radiusType) {
-    case BasicShapeRadius::Value: ts << "value"; break;
-    case BasicShapeRadius::ClosestSide: ts << "closest-side"; break;
-    case BasicShapeRadius::FarthestSide: ts << "farthest-side"; break;
+    case BasicShapeRadius::Type::Value: ts << "value"; break;
+    case BasicShapeRadius::Type::ClosestSide: ts << "closest-side"; break;
+    case BasicShapeRadius::Type::FarthestSide: ts << "farthest-side"; break;
     }
     return ts;
 }
@@ -488,7 +590,7 @@ TextStream& operator<<(TextStream& ts, const BasicShapeRadius& radius)
 
 TextStream& operator<<(TextStream& ts, const BasicShapeCenterCoordinate& coordinate)
 {
-    ts.dumpProperty("direction", coordinate.direction() == BasicShapeCenterCoordinate::TopLeft ? "top left" : "bottom right");
+    ts.dumpProperty("direction", coordinate.direction() == BasicShapeCenterCoordinate::Direction::TopLeft ? "top left" : "bottom right");
     ts.dumpProperty("length", coordinate.length());
     return ts;
 }
@@ -500,4 +602,3 @@ TextStream& operator<<(TextStream& ts, const BasicShape& shape)
 }
 
 } // namespace WebCore
-
