@@ -224,8 +224,6 @@ TextStream& operator<<(TextStream& ts, CaptionSide side)
     switch (side) {
     case CaptionSide::Top: ts << "top"; break;
     case CaptionSide::Bottom: ts << "bottom"; break;
-    case CaptionSide::Left: ts << "left"; break;
-    case CaptionSide::Right: ts << "right"; break;
     }
     return ts;
 }
@@ -548,7 +546,6 @@ TextStream& operator<<(TextStream& ts, GridAutoFlow gridAutoFlow)
 TextStream& operator<<(TextStream& ts, HangingPunctuation punctuation)
 {
     switch (punctuation) {
-    case HangingPunctuation::None: ts << "none"; break;
     case HangingPunctuation::First: ts << "first"; break;
     case HangingPunctuation::Last: ts << "last"; break;
     case HangingPunctuation::AllowEnd: ts << "allow-end"; break;
@@ -670,11 +667,6 @@ TextStream& operator<<(TextStream& ts, ListStylePosition position)
     return ts;
 }
 
-TextStream& operator<<(TextStream& ts, ListStyleType styleType)
-{
-    return ts << nameLiteral(toCSSValueID(styleType)).characters();
-}
-
 TextStream& operator<<(TextStream& ts, MarginTrimType marginTrimType)
 {
     switch (marginTrimType) {
@@ -951,7 +943,6 @@ TextStream& operator<<(TextStream& ts, ScrollSnapStop stop)
     }
     return ts;
 }
-
 TextStream& operator<<(TextStream& ts, SpeakAs speakAs)
 {
     switch (speakAs) {
@@ -1158,7 +1149,8 @@ TextStream& operator<<(TextStream& ts, TextTransform textTransform)
     case TextTransform::Capitalize: ts << "capitalize"; break;
     case TextTransform::Uppercase: ts << "uppercase"; break;
     case TextTransform::Lowercase: ts << "lowercase"; break;
-    case TextTransform::None: ts << "none"; break;
+    case TextTransform::FullSizeKana: ts << "full-size-kana"; break;
+    case TextTransform::FullWidth: ts << "full-width"; break;
     }
     return ts;
 }
@@ -1169,6 +1161,8 @@ TextStream& operator<<(TextStream& ts, TextUnderlinePosition underlinePosition)
     case TextUnderlinePosition::Auto: ts << "Auto"; break;
     case TextUnderlinePosition::Under: ts << "Under"; break;
     case TextUnderlinePosition::FromFont: ts << "FromFont"; break;
+    case TextUnderlinePosition::Left: ts << "Left"; break;
+    case TextUnderlinePosition::Right: ts << "Right"; break;
     }
     return ts;
 }
@@ -1185,27 +1179,27 @@ TextStream& operator<<(TextStream& ts, TextWrap wrap)
     return ts;
 }
 
-TextStream& operator<<(TextStream& ts, LeadingTrim leadingTrim)
+TextStream& operator<<(TextStream& ts, TextBoxTrim textBoxTrim)
 {
-    switch (leadingTrim) {
-    case LeadingTrim::Normal: ts << "Normal"; break;
-    case LeadingTrim::Start: ts << "Start"; break;
-    case LeadingTrim::End: ts << "End"; break;
-    case LeadingTrim::Both: ts << "Both"; break;
+    switch (textBoxTrim) {
+    case TextBoxTrim::None: ts << "None"; break;
+    case TextBoxTrim::Start: ts << "Start"; break;
+    case TextBoxTrim::End: ts << "End"; break;
+    case TextBoxTrim::Both: ts << "Both"; break;
     }
     return ts;
 }
 
-TextStream& operator<<(TextStream& ts, TextEdgeType textEdgeType)
+TextStream& operator<<(TextStream& ts, TextBoxEdgeType textBoxEdgeType)
 {
-    switch (textEdgeType) {
-    case TextEdgeType::Leading: ts << "half-leading"; break;
-    case TextEdgeType::Text: ts << "text-over/under baseline"; break;
-    case TextEdgeType::CapHeight: ts << "cap-height baseline"; break;
-    case TextEdgeType::ExHeight: ts << "x-height baseline"; break;
-    case TextEdgeType::Alphabetic: ts << "alphabetic baseline"; break;
-    case TextEdgeType::CJKIdeographic: ts << "ideographic-over baseline"; break;
-    case TextEdgeType::CJKIdeographicInk: ts << "ideographic-ink-over/ink-under baseline"; break;
+    switch (textBoxEdgeType) {
+    case TextBoxEdgeType::Leading: ts << "half-leading"; break;
+    case TextBoxEdgeType::Text: ts << "text-over/under baseline"; break;
+    case TextBoxEdgeType::CapHeight: ts << "cap-height baseline"; break;
+    case TextBoxEdgeType::ExHeight: ts << "x-height baseline"; break;
+    case TextBoxEdgeType::Alphabetic: ts << "alphabetic baseline"; break;
+    case TextBoxEdgeType::CJKIdeographic: ts << "ideographic-over baseline"; break;
+    case TextBoxEdgeType::CJKIdeographicInk: ts << "ideographic-ink-over/ink-under baseline"; break;
     }
     return ts;
 }
@@ -1313,6 +1307,18 @@ TextStream& operator<<(TextStream& ts, WhiteSpace whiteSpace)
     }
     return ts;
 }
+TextStream& operator<<(TextStream& ts, WhiteSpaceCollapse whiteSpaceCollapse)
+{
+    switch (whiteSpaceCollapse) {
+    case WhiteSpaceCollapse::Collapse: ts << "collapse"; break;
+    case WhiteSpaceCollapse::Discard: ts << "discard"; break;
+    case WhiteSpaceCollapse::Preserve: ts << "preserve"; break;
+    case WhiteSpaceCollapse::PreserveBreaks: ts << "preserve-breaks"; break;
+    case WhiteSpaceCollapse::PreserveSpaces: ts << "preserve-spaces"; break;
+    case WhiteSpaceCollapse::BreakSpaces: ts << "break-spaces"; break;
+    }
+    return ts;
+}
 
 TextStream& operator<<(TextStream& ts, WordBreak wordBreak)
 {
@@ -1374,6 +1380,4 @@ CSSBoxType transformBoxToCSSBoxType(TransformBox transformBox)
     }
 }
 
-const float defaultMiterLimit = 4;
-
 } // namespace WebCore
