@@ -24,6 +24,7 @@
 #include "RenderElement.h"
 #include "RenderStyle.h"
 #include "SVGCircleElement.h"
+#include "SVGElementTypeHelpers.h"
 #include "SVGEllipseElement.h"
 #include "SVGLengthContext.h"
 #include "SVGLineElement.h"
@@ -35,6 +36,7 @@
 #include "SVGPolygonElement.h"
 #include "SVGPolylineElement.h"
 #include "SVGRectElement.h"
+#include "SVGRenderStyle.h"
 #include <wtf/HashMap.h>
 
 namespace WebCore {
@@ -161,7 +163,7 @@ static Path pathFromRectElement(const SVGElement& element)
         // FIXME: We currently enforce using beziers here, as at least on CoreGraphics/Lion, as
         // the native method uses a different line dash origin, causing svg/custom/dashOrigin.svg to fail.
         // See bug https://bugs.webkit.org/show_bug.cgi?id=79932 which tracks this issue.
-        path.addRoundedRect(FloatRect(x, y, width, height), FloatSize(rx, ry), Path::PreferBezierRoundedRect);
+        path.addRoundedRect(FloatRect(x, y, width, height), FloatSize(rx, ry), Path::RoundedRectStrategy::PreferBezier);
         return path;
     }
 
