@@ -47,11 +47,12 @@
 namespace WebCore {
 namespace Style {
 
-RuleSetBuilder::RuleSetBuilder(RuleSet& ruleSet, const MQ::MediaQueryEvaluator& evaluator, Resolver* resolver, ShrinkToFit shrinkToFit)
+RuleSetBuilder::RuleSetBuilder(RuleSet& ruleSet, const MQ::MediaQueryEvaluator& evaluator, Resolver* resolver, ShrinkToFit shrinkToFit, ShouldResolveNesting shouldResolveNesting)
     : m_ruleSet(&ruleSet)
     , m_mediaQueryCollector({ evaluator })
     , m_resolver(resolver)
     , m_shrinkToFit(shrinkToFit)
+    , m_shouldResolveNesting(shouldResolveNesting)
 {
 }
 
@@ -106,6 +107,11 @@ void RuleSetBuilder::addChildRules(const Vector<RefPtr<StyleRuleBase>>& rules)
 void RuleSetBuilder::addChildRule(RefPtr<StyleRuleBase> rule)
 {
     switch (rule->type()) {
+    case StyleRuleType::StyleWithNesting:
+        if (m_ruleSet)
+            addStyleRule(downcast<StyleRuleWithNesting>(*rule));
+        return;
+
     case StyleRuleType::Style:
         if (m_ruleSet)
             addStyleRule(downcast<StyleRule>(*rule));
@@ -210,21 +216,23 @@ void RuleSetBuilder::addRulesFromSheetContents(const StyleSheetContents& sheet)
     addChildRules(sheet.childRules());
 }
 
-void RuleSetBuilder::populateStyleRuleResolvedSelectorList(const StyleRule& rule)
+void RuleSetBuilder::resolveSelectorListWithNesting(StyleRuleWithNesting& rule)
 {
-    // FIXME: handle top-level nesting selector
-    if (!m_styleRuleStack.size()) 
+    const CSSSelectorList* parentResolvedSelectorList = nullptr;
+    if (m_styleRuleStack.size()) 
+        parentResolvedSelectorList =  m_styleRuleStack.last();
+
+    // If it's a top-level rule without a nesting parent selector, keep the selector list as is.
+    if (!rule.originalSelectorList().hasExplicitNestingParent() && !parentResolvedSelectorList)
         return;
 
-    auto resolvedSelectorList = CSSSelectorParser::resolveNestingParent(rule.selectorList(), m_styleRuleStack.last());
-    rule.setResolvedSelectorList(WTFMove(resolvedSelectorList));    
+    auto resolvedSelectorList = CSSSelectorParser::resolveNestingParent(rule.originalSelectorList(), parentResolvedSelectorList);
+    ASSERT(!resolvedSelectorList.hasExplicitNestingParent());
+    rule.wrapperAdoptSelectorList(WTFMove(resolvedSelectorList));    
 }
 
-void RuleSetBuilder::addStyleRule(const StyleRule& rule)
+void RuleSetBuilder::addStyleRuleWithSelectorList(const CSSSelectorList& selectorList, const StyleRule& rule)
 {
-    populateStyleRuleResolvedSelectorList(rule);
-    
-    auto& selectorList = rule.resolvedSelectorList();
     if (!selectorList.isEmpty()) {
         unsigned selectorListIndex = 0;
         for (size_t selectorIndex = 0; selectorIndex != notFound; selectorIndex = selectorList.indexOfNextSelectorAfter(selectorIndex)) {
@@ -234,7 +242,16 @@ void RuleSetBuilder::addStyleRule(const StyleRule& rule)
             ++selectorListIndex;
         }
     }
+}
 
+void RuleSetBuilder::addStyleRule(StyleRuleWithNesting& rule)
+{
+    if (m_shouldResolveNesting == ShouldResolveNesting::Yes)
+        resolveSelectorListWithNesting(rule);
+
+    auto& selectorList = rule.selectorList();
+    addStyleRuleWithSelectorList(selectorList, rule);
+    
     // Process nested rules
     m_styleRuleStack.append(&selectorList);
     for (auto& nestedRule : rule.nestedRules())
@@ -242,6 +259,11 @@ void RuleSetBuilder::addStyleRule(const StyleRule& rule)
     m_styleRuleStack.removeLast();
 }
 
+void RuleSetBuilder::addStyleRule(const StyleRule& rule)
+{
+    addStyleRuleWithSelectorList(rule.selectorList(), rule);
+}
+
 void RuleSetBuilder::disallowDynamicMediaQueryEvaluationIfNeeded()
 {
     bool isScanningForDynamicEvaluation = !m_ruleSet;
@@ -396,8 +418,6 @@ void RuleSetBuilder::addMutatingRulesToResolver()
                 continue;
             auto& registry = m_resolver->document().styleScope().counterStyleRegistry();
             registry.addCounterStyle(downcast<StyleRuleCounterStyle>(rule.get()).descriptors());
-            // FIXME: we probably need a cache solultion like fontSelector (invalidateMatchedDeclarations)
-            // KeyFrame does it differently, search for keyframesRuleDidChange. (rdar://103018993)
             continue;
         }
         if (is<StyleRuleProperty>(rule)) {
