@@ -29,8 +29,9 @@ namespace Style {
 
 class RuleSetBuilder {
 public:
-    enum class ShrinkToFit { Enable, Disable };
-    RuleSetBuilder(RuleSet&, const MQ::MediaQueryEvaluator&, Resolver* = nullptr, ShrinkToFit = ShrinkToFit::Enable);
+    enum class ShrinkToFit : bool { Enable, Disable };
+    enum class ShouldResolveNesting : bool { No, Yes };
+    RuleSetBuilder(RuleSet&, const MQ::MediaQueryEvaluator&, Resolver* = nullptr, ShrinkToFit = ShrinkToFit::Enable, ShouldResolveNesting = ShouldResolveNesting::No);
     ~RuleSetBuilder();
 
     void addRulesFromSheet(const StyleSheetContents&, const MQ::MediaQueryList& sheetQuery = { });
@@ -39,10 +40,12 @@ public:
 private:
     RuleSetBuilder(const MQ::MediaQueryEvaluator&);
 
+    void addStyleRule(StyleRuleWithNesting&);
     void addRulesFromSheetContents(const StyleSheetContents&);
     void addChildRules(const Vector<RefPtr<StyleRuleBase>>&);
     void addChildRule(RefPtr<StyleRuleBase>);
     void disallowDynamicMediaQueryEvaluationIfNeeded();
+    void addStyleRuleWithSelectorList(const CSSSelectorList&, const StyleRule&);
 
     void registerLayers(const Vector<CascadeLayerName>&);
     void pushCascadeLayer(const CascadeLayerName&);
@@ -51,7 +54,7 @@ private:
     
     void addMutatingRulesToResolver();
     void updateDynamicMediaQueries();
-    void populateStyleRuleResolvedSelectorList(const StyleRule&);
+    void resolveSelectorListWithNesting(StyleRuleWithNesting&);
 
     struct MediaQueryCollector {
         ~MediaQueryCollector();
@@ -83,6 +86,7 @@ private:
     HashMap<CascadeLayerName, RuleSet::CascadeLayerIdentifier> m_cascadeLayerIdentifierMap;
     RuleSet::CascadeLayerIdentifier m_currentCascadeLayerIdentifier { 0 };
     Vector<const CSSSelectorList*> m_styleRuleStack;
+    const ShouldResolveNesting m_shouldResolveNesting { ShouldResolveNesting::No };
 
     RuleSet::ContainerQueryIdentifier m_currentContainerQueryIdentifier { 0 };
 
