@@ -30,14 +30,18 @@
 #include "config.h"
 #include "StyleBuilder.h"
 
+#include "CSSCustomPropertyValue.h"
 #include "CSSFontSelector.h"
 #include "CSSPaintImageValue.h"
+#include "CSSPendingSubstitutionValue.h"
+#include "CSSPropertyParser.h"
 #include "CSSRegisteredCustomProperty.h"
 #include "CSSValuePool.h"
 #include "CustomPropertyRegistry.h"
 #include "Document.h"
 #include "HTMLElement.h"
 #include "PaintWorkletGlobalScope.h"
+#include "RenderStyle.h"
 #include "Settings.h"
 #include "StyleBuilderGenerated.h"
 #include "StyleFontSizeFunctions.h"
@@ -172,11 +176,20 @@ void Builder::applyCustomProperty(const AtomString& name)
     if (!property.cssValue[SelectorChecker::MatchDefault])
         return;
 
-    SetForScope levelScope(m_state.m_currentProperty, &property);
-
     Ref customPropertyValue = downcast<CSSCustomPropertyValue>(*property.cssValue[SelectorChecker::MatchDefault]);
 
     bool inCycle = !m_state.m_inProgressCustomProperties.add(name).isNewEntry;
+    if (inCycle) {
+        auto isNewCycle = m_state.m_inCycleCustomProperties.add(name).isNewEntry;
+        if (isNewCycle) {
+            // Continue resolving dependencies so we detect cycles for them as well.
+            resolveCustomPropertyValueWithVariableReferences(customPropertyValue.get());
+        }
+        return;
+    }
+    
+    // There may be multiple cycles through the same property. Avoid interference from any previously detected cycles.
+    auto savedInCycleProperties = std::exchange(m_state.m_inCycleCustomProperties, { });
 
     auto createInvalidOrUnset = [&] {
         // https://drafts.csswg.org/css-variables-2/#invalid-variables
@@ -191,35 +204,18 @@ void Builder::applyCustomProperty(const AtomString& name)
         return CSSCustomPropertyValue::createWithID(name, CSSValueUnset);
     };
 
-    auto valueToApply = [&]() -> RefPtr<CSSCustomPropertyValue> {
-        if (inCycle)
-            return createInvalidOrUnset();
+    auto resolvedValue = resolveCustomPropertyValueWithVariableReferences(customPropertyValue.get());
 
-        auto resolvedValue = resolveCustomPropertyValueWithVariableReferences(customPropertyValue.get());
-        if (m_state.m_appliedCustomProperties.contains(name))
-            return nullptr; // There was a cycle and the value was already resolved, so bail.
-
-        if (!resolvedValue)
-            return createInvalidOrUnset();
-
-        return resolvedValue;
-    }();
-
-    if (!valueToApply) {
-        ASSERT(!m_state.m_inProgressCustomProperties.contains(name));
-        return;
-    }
+    if (!resolvedValue || m_state.m_inCycleCustomProperties.contains(name))
+        resolvedValue = createInvalidOrUnset();
 
+    SetForScope levelScope(m_state.m_currentProperty, &property);
     SetForScope scopedLinkMatchMutation(m_state.m_linkMatch, SelectorChecker::MatchDefault);
-    applyProperty(CSSPropertyCustom, *valueToApply, SelectorChecker::MatchDefault);
+    applyProperty(CSSPropertyCustom, *resolvedValue, SelectorChecker::MatchDefault);
 
     m_state.m_inProgressCustomProperties.remove(name);
     m_state.m_appliedCustomProperties.add(name);
-
-    if (inCycle) {
-        // Resolve this value so that we reset its dependencies.
-        resolveCustomPropertyValueWithVariableReferences(customPropertyValue.get());
-    }
+    m_state.m_inCycleCustomProperties.formUnion(WTFMove(savedInCycleProperties));
 }
 
 inline void Builder::applyCascadeProperty(const PropertyCascade::Property& property)
@@ -257,7 +253,7 @@ void Builder::applyRollbackCascadeProperty(const PropertyCascade::Property& prop
 
 void Builder::applyProperty(CSSPropertyID id, CSSValue& value, SelectorChecker::LinkMatchMask linkMatchMask)
 {
-    ASSERT_WITH_MESSAGE(!isShorthandCSSProperty(id), "Shorthand property id = %d wasn't expanded at parsing time", id);
+    ASSERT_WITH_MESSAGE(!isShorthand(id), "Shorthand property id = %d wasn't expanded at parsing time", id);
 
     auto valueToApply = resolveVariableReferences(id, value);
     auto& style = m_state.style();
@@ -364,7 +360,34 @@ Ref<CSSValue> Builder::resolveVariableReferences(CSSPropertyID propertyID, CSSVa
     if (!value.hasVariableReferences())
         return value;
 
-    auto variableValue = CSSParser { m_state.document() }.parseValueWithVariableReferences(propertyID, value, m_state);
+    auto variableValue = [&]() -> RefPtr<CSSValue> {
+        if (is<CSSPendingSubstitutionValue>(value)) {
+            auto& substitution = downcast<CSSPendingSubstitutionValue>(value);
+            auto shorthandID = substitution.shorthandPropertyId();
+
+            auto resolvedData = substitution.shorthandValue().resolveVariableReferences(m_state);
+            if (!resolvedData)
+                return nullptr;
+
+            ParsedPropertyVector parsedProperties;
+            if (!CSSPropertyParser::parseValue(shorthandID, false, resolvedData->tokens(), substitution.shorthandValue().context(), parsedProperties, StyleRuleType::Style))
+                return nullptr;
+
+            for (auto& property : parsedProperties) {
+                if (property.id() == propertyID)
+                    return property.value();
+            }
+
+            return nullptr;
+        }
+
+        auto& variableReferenceValue = downcast<CSSVariableReferenceValue>(value);
+        auto resolvedData = variableReferenceValue.resolveVariableReferences(m_state);
+        if (!resolvedData)
+            return nullptr;
+
+        return CSSPropertyParser::parseSingleValue(propertyID, resolvedData->tokens(), variableReferenceValue.context());
+    }();
 
     // https://drafts.csswg.org/css-variables-2/#invalid-variables
     // ...as if the property’s value had been specified as the unset keyword.
@@ -378,7 +401,46 @@ RefPtr<CSSCustomPropertyValue> Builder::resolveCustomPropertyValueWithVariableRe
 {
     if (!std::holds_alternative<Ref<CSSVariableReferenceValue>>(value.value()))
         return &value;
-    return CSSParser { m_state.document() }.parseCustomPropertyValueWithVariableReferences(value, m_state);
+
+    auto& variableReferenceValue = std::get<Ref<CSSVariableReferenceValue>>(value.value()).get();
+
+    auto name = value.name();
+    auto* registered = m_state.document().customPropertyRegistry().get(name);
+    auto& syntax = registered ? registered->syntax : CSSCustomPropertySyntax::universal();
+
+    auto resolvedData = variableReferenceValue.resolveVariableReferences(m_state);
+    if (!resolvedData)
+        return nullptr;
+
+    auto dependencies = CSSPropertyParser::collectParsedCustomPropertyValueDependencies(syntax, resolvedData->tokens(), variableReferenceValue.context());
+
+    // https://drafts.css-houdini.org/css-properties-values-api/#dependency-cycles
+    bool hasCycles = false;
+    bool isFontDependent = false;
+
+    auto checkDependencies = [&](auto& propertyDependencies) {
+        for (auto property : propertyDependencies) {
+            if (m_state.m_inProgressProperties.get(property)) {
+                m_state.m_inUnitCycleProperties.set(property);
+                hasCycles = true;
+            }
+            if (property == CSSPropertyFontSize)
+                isFontDependent = true;
+        }
+    };
+
+    checkDependencies(dependencies.properties);
+
+    if (m_state.element() == m_state.document().documentElement())
+        checkDependencies(dependencies.rootProperties);
+
+    if (hasCycles)
+        return nullptr;
+
+    if (isFontDependent)
+        m_state.updateFont();
+
+    return CSSPropertyParser::parseTypedCustomPropertyValue(name, syntax, resolvedData->tokens(), m_state, variableReferenceValue.context());
 }
 
 const PropertyCascade* Builder::ensureRollbackCascadeForRevert()
