@@ -25,12 +25,15 @@
 #include "ExceptionOr.h"
 #include "FormData.h"
 #include "ResourceResponse.h"
+#include "SharedBuffer.h"
 #include "ThreadableLoaderClient.h"
+#include "URLKeepingBlobAlive.h"
 #include "UserGestureIndicator.h"
 #include <wtf/URL.h>
 #include "XMLHttpRequestEventTarget.h"
 #include "XMLHttpRequestProgressEventThrottle.h"
-#include <wtf/Variant.h>
+#include <variant>
+#include <wtf/CancellableTask.h>
 #include <wtf/text/StringBuilder.h>
 
 namespace JSC {
@@ -44,9 +47,9 @@ class Blob;
 class Document;
 class DOMFormData;
 class SecurityOrigin;
-class SharedBuffer;
 class TextResourceDecoder;
 class ThreadableLoader;
+class URLSearchParams;
 class XMLHttpRequestUpload;
 struct OwnedString;
 
@@ -70,18 +73,18 @@ public:
     EventTargetInterface eventTargetInterface() const override { return XMLHttpRequestEventTargetInterfaceType; }
     ScriptExecutionContext* scriptExecutionContext() const override { return ActiveDOMObject::scriptExecutionContext(); }
 
-    using SendTypes = Variant<RefPtr<Document>, RefPtr<Blob>, RefPtr<JSC::ArrayBufferView>, RefPtr<JSC::ArrayBuffer>, RefPtr<DOMFormData>, String>;
+    using SendTypes = std::variant<RefPtr<Document>, RefPtr<Blob>, RefPtr<JSC::ArrayBufferView>, RefPtr<JSC::ArrayBuffer>, RefPtr<DOMFormData>, String, RefPtr<URLSearchParams>>;
 
     const URL& url() const { return m_url; }
     String statusText() const;
     int status() const;
-    State readyState() const;
+    State readyState() const { return static_cast<State>(m_readyState); }
     bool withCredentials() const { return m_includeCredentials; }
     ExceptionOr<void> setWithCredentials(bool);
     ExceptionOr<void> open(const String& method, const String& url);
     ExceptionOr<void> open(const String& method, const URL&, bool async);
     ExceptionOr<void> open(const String& method, const String&, bool async, const String& user, const String& password);
-    ExceptionOr<void> send(Optional<SendTypes>&&);
+    ExceptionOr<void> send(std::optional<SendTypes>&&);
     void abort();
     ExceptionOr<void> setRequestHeader(const String& name, const String& value);
     ExceptionOr<void> overrideMimeType(const String& override);
@@ -90,7 +93,8 @@ public:
     String getResponseHeader(const String& name) const;
     ExceptionOr<OwnedString> responseText();
     String responseTextIgnoringResponseType() const { return m_responseBuilder.toStringPreserveCapacity(); }
-    String responseMIMEType() const;
+    enum class FinalMIMEType : bool { No, Yes };
+    String responseMIMEType(FinalMIMEType = FinalMIMEType::No) const;
 
     Document* optionalResponseXML() const { return m_responseDocument.get(); }
     ExceptionOr<Document*> responseXML();
@@ -114,7 +118,7 @@ public:
         Text = 5,
     };
     ExceptionOr<void> setResponseType(ResponseType);
-    ResponseType responseType() const;
+    ResponseType responseType() const { return static_cast<ResponseType>(m_responseType); }
 
     String responseURL() const;
 
@@ -128,18 +132,20 @@ public:
 
     size_t memoryCost() const;
 
-    WEBCORE_EXPORT void setMaximumIntervalForUserGestureForwarding(double);
-
     using EventTarget::dispatchEvent;
     void dispatchEvent(Event&) override;
 
 private:
+    friend class XMLHttpRequestUpload;
     explicit XMLHttpRequest(ScriptExecutionContext&);
 
+    void updateHasRelevantEventListener();
+    void handleCancellation();
+
     // EventTarget.
     void eventListenersDidChange() final;
 
-    TextEncoding finalResponseCharset() const;
+    PAL::TextEncoding finalResponseCharset() const;
 
     // ActiveDOMObject
     void contextDestroyed() override;
@@ -157,14 +163,14 @@ private:
 
     // ThreadableLoaderClient
     void didSendData(unsigned long long bytesSent, unsigned long long totalBytesToBeSent) override;
-    void didReceiveResponse(unsigned long identifier, const ResourceResponse&) override;
-    void didReceiveData(const char* data, int dataLength) override;
-    void didFinishLoading(unsigned long identifier) override;
+    void didReceiveResponse(ResourceLoaderIdentifier, const ResourceResponse&) override;
+    void didReceiveData(const SharedBuffer&) override;
+    void didFinishLoading(ResourceLoaderIdentifier, const NetworkLoadMetrics&) override;
     void didFail(const ResourceError&) override;
+    void notifyIsDone(bool) final;
 
-    bool responseIsXML() const;
-
-    Optional<ExceptionOr<void>> prepareToSend();
+    std::optional<ExceptionOr<void>> prepareToSend();
+    ExceptionOr<void> send(const URLSearchParams&);
     ExceptionOr<void> send(Document&);
     ExceptionOr<void> send(const String& = { });
     ExceptionOr<void> send(Blob&);
@@ -186,6 +192,8 @@ private:
 
     ExceptionOr<void> createRequest();
 
+    void timeoutTimerFired();
+
     void genericError();
     void networkError();
     void abortError();
@@ -201,7 +209,6 @@ private:
     unsigned m_error : 1;
     unsigned m_uploadListenerFlag : 1;
     unsigned m_uploadComplete : 1;
-    unsigned m_wasAbortedByClient : 1;
     unsigned m_responseCacheIsValid : 1;
     unsigned m_readyState : 3; // State
     unsigned m_responseType : 3; // ResponseType
@@ -210,7 +217,7 @@ private:
 
     std::unique_ptr<XMLHttpRequestUpload> m_upload;
 
-    URL m_url;
+    URLKeepingBlobAlive m_url;
     String m_method;
     HTTPHeaderMap m_requestHeaders;
     RefPtr<FormData> m_requestEntityBody;
@@ -220,7 +227,7 @@ private:
         Ref<XMLHttpRequest> protectedThis; // Keep object alive while loading even if there is no longer a JS wrapper.
         Ref<ThreadableLoader> loader;
     };
-    Optional<LoadingActivity> m_loadingActivity;
+    std::optional<LoadingActivity> m_loadingActivity;
 
     String m_responseEncoding;
 
@@ -230,7 +237,7 @@ private:
 
     RefPtr<Document> m_responseDocument;
 
-    RefPtr<SharedBuffer> m_binaryResponseBuilder;
+    SharedBufferBuilder m_binaryResponseBuilder;
 
     StringBuilder m_responseBuilder;
 
@@ -245,20 +252,11 @@ private:
 
     MonotonicTime m_sendingTime;
 
-    Optional<ExceptionCode> m_exceptionCode;
+    std::optional<ExceptionCode> m_exceptionCode;
     RefPtr<UserGestureToken> m_userGestureToken;
-    Seconds m_maximumIntervalForUserGestureForwarding;
-    bool m_hasRelevantEventListener { false };
+    std::atomic<bool> m_hasRelevantEventListener;
+    TaskCancellationGroup m_abortErrorGroup;
+    bool m_wasDidSendDataCalledForTotalBytes { false };
 };
 
-inline auto XMLHttpRequest::responseType() const -> ResponseType
-{
-    return static_cast<ResponseType>(m_responseType);
-}
-
-inline auto XMLHttpRequest::readyState() const -> State
-{
-    return static_cast<State>(m_readyState);
-}
-
 } // namespace WebCore
