@@ -1,6 +1,7 @@
 /*
  * Copyright (C) 2005 Frerich Raabe <raabe@kde.org>
  * Copyright (C) 2006, 2009, 2013 Apple Inc. All rights reserved.
+ * Copyright (C) 2019 Google Inc. All rights reserved.
  * Copyright (C) 2007 Alexey Proskuryakov <ap@webkit.org>
  *
  * Redistribution and use in source and binary forms, with or without
@@ -28,13 +29,16 @@
 #include "config.h"
 #include "XPathFunctions.h"
 
-#include "Element.h"
+#include "Attribute.h"
+#include "ElementInlines.h"
 #include "ProcessingInstruction.h"
 #include "TreeScope.h"
 #include "XMLNames.h"
 #include "XPathUtil.h"
 #include <wtf/MathExtras.h>
 #include <wtf/NeverDestroyed.h>
+#include <wtf/RobinHoodHashMap.h>
+#include <wtf/SetForScope.h>
 #include <wtf/text/StringBuilder.h>
 
 namespace WebCore {
@@ -45,7 +49,7 @@ static inline bool isWhitespace(UChar c)
     return c == ' ' || c == '\n' || c == '\r' || c == '\t';
 }
 
-#define DEFINE_FUNCTION_CREATOR(Suffix) static std::unique_ptr<Function> createFunction##Suffix() { return std::make_unique<Fun##Suffix>(); }
+#define DEFINE_FUNCTION_CREATOR(Suffix) static std::unique_ptr<Function> createFunction##Suffix() { return makeUnique<Fun##Suffix>(); }
 
 class Interval {
 public:
@@ -63,158 +67,158 @@ private:
 };
 
 class FunLast final : public Function {
-    virtual Value evaluate() const override;
-    virtual Value::Type resultType() const override { return Value::NumberValue; }
+    Value evaluate() const override;
+    Value::Type resultType() const override { return Value::NumberValue; }
 public:
     FunLast() { setIsContextSizeSensitive(true); }
 };
 
 class FunPosition final : public Function {
-    virtual Value evaluate() const override;
-    virtual Value::Type resultType() const override { return Value::NumberValue; }
+    Value evaluate() const override;
+    Value::Type resultType() const override { return Value::NumberValue; }
 public:
     FunPosition() { setIsContextPositionSensitive(true); }
 };
 
 class FunCount final : public Function {
-    virtual Value evaluate() const override;
-    virtual Value::Type resultType() const override { return Value::NumberValue; }
+    Value evaluate() const override;
+    Value::Type resultType() const override { return Value::NumberValue; }
 };
 
 class FunId final : public Function {
-    virtual Value evaluate() const override;
-    virtual Value::Type resultType() const override { return Value::NodeSetValue; }
+    Value evaluate() const override;
+    Value::Type resultType() const override { return Value::NodeSetValue; }
 };
 
 class FunLocalName final : public Function {
-    virtual Value evaluate() const override;
-    virtual Value::Type resultType() const override { return Value::StringValue; }
+    Value evaluate() const override;
+    Value::Type resultType() const override { return Value::StringValue; }
 public:
     FunLocalName() { setIsContextNodeSensitive(true); } // local-name() with no arguments uses context node. 
 };
 
 class FunNamespaceURI final : public Function {
-    virtual Value evaluate() const override;
-    virtual Value::Type resultType() const override { return Value::StringValue; }
+    Value evaluate() const override;
+    Value::Type resultType() const override { return Value::StringValue; }
 public:
     FunNamespaceURI() { setIsContextNodeSensitive(true); } // namespace-uri() with no arguments uses context node. 
 };
 
 class FunName final : public Function {
-    virtual Value evaluate() const override;
-    virtual Value::Type resultType() const override { return Value::StringValue; }
+    Value evaluate() const override;
+    Value::Type resultType() const override { return Value::StringValue; }
 public:
     FunName() { setIsContextNodeSensitive(true); } // name() with no arguments uses context node. 
 };
 
 class FunString final : public Function {
-    virtual Value evaluate() const override;
-    virtual Value::Type resultType() const override { return Value::StringValue; }
+    Value evaluate() const override;
+    Value::Type resultType() const override { return Value::StringValue; }
 public:
     FunString() { setIsContextNodeSensitive(true); } // string() with no arguments uses context node. 
 };
 
 class FunConcat final : public Function {
-    virtual Value evaluate() const override;
-    virtual Value::Type resultType() const override { return Value::StringValue; }
+    Value evaluate() const override;
+    Value::Type resultType() const override { return Value::StringValue; }
 };
 
 class FunStartsWith final : public Function {
-    virtual Value evaluate() const override;
-    virtual Value::Type resultType() const override { return Value::BooleanValue; }
+    Value evaluate() const override;
+    Value::Type resultType() const override { return Value::BooleanValue; }
 };
 
 class FunContains final : public Function {
-    virtual Value evaluate() const override;
-    virtual Value::Type resultType() const override { return Value::BooleanValue; }
+    Value evaluate() const override;
+    Value::Type resultType() const override { return Value::BooleanValue; }
 };
 
 class FunSubstringBefore final : public Function {
-    virtual Value evaluate() const override;
-    virtual Value::Type resultType() const override { return Value::StringValue; }
+    Value evaluate() const override;
+    Value::Type resultType() const override { return Value::StringValue; }
 };
 
 class FunSubstringAfter final : public Function {
-    virtual Value evaluate() const override;
-    virtual Value::Type resultType() const override { return Value::StringValue; }
+    Value evaluate() const override;
+    Value::Type resultType() const override { return Value::StringValue; }
 };
 
 class FunSubstring final : public Function {
-    virtual Value evaluate() const override;
-    virtual Value::Type resultType() const override { return Value::StringValue; }
+    Value evaluate() const override;
+    Value::Type resultType() const override { return Value::StringValue; }
 };
 
 class FunStringLength final : public Function {
-    virtual Value evaluate() const override;
-    virtual Value::Type resultType() const override { return Value::NumberValue; }
+    Value evaluate() const override;
+    Value::Type resultType() const override { return Value::NumberValue; }
 public:
     FunStringLength() { setIsContextNodeSensitive(true); } // string-length() with no arguments uses context node. 
 };
 
 class FunNormalizeSpace final : public Function {
-    virtual Value evaluate() const override;
-    virtual Value::Type resultType() const override { return Value::StringValue; }
+    Value evaluate() const override;
+    Value::Type resultType() const override { return Value::StringValue; }
 public:
     FunNormalizeSpace() { setIsContextNodeSensitive(true); } // normalize-space() with no arguments uses context node. 
 };
 
 class FunTranslate final : public Function {
-    virtual Value evaluate() const override;
-    virtual Value::Type resultType() const override { return Value::StringValue; }
+    Value evaluate() const override;
+    Value::Type resultType() const override { return Value::StringValue; }
 };
 
 class FunBoolean final : public Function {
-    virtual Value evaluate() const override;
-    virtual Value::Type resultType() const override { return Value::BooleanValue; }
+    Value evaluate() const override;
+    Value::Type resultType() const override { return Value::BooleanValue; }
 };
 
 class FunNot : public Function {
-    virtual Value evaluate() const override;
-    virtual Value::Type resultType() const override { return Value::BooleanValue; }
+    Value evaluate() const override;
+    Value::Type resultType() const override { return Value::BooleanValue; }
 };
 
 class FunTrue final : public Function {
-    virtual Value evaluate() const override;
-    virtual Value::Type resultType() const override { return Value::BooleanValue; }
+    Value evaluate() const override;
+    Value::Type resultType() const override { return Value::BooleanValue; }
 };
 
 class FunFalse final : public Function {
-    virtual Value evaluate() const override;
-    virtual Value::Type resultType() const override { return Value::BooleanValue; }
+    Value evaluate() const override;
+    Value::Type resultType() const override { return Value::BooleanValue; }
 };
 
 class FunLang final : public Function {
-    virtual Value evaluate() const override;
-    virtual Value::Type resultType() const override { return Value::BooleanValue; }
+    Value evaluate() const override;
+    Value::Type resultType() const override { return Value::BooleanValue; }
 public:
     FunLang() { setIsContextNodeSensitive(true); } // lang() always works on context node. 
 };
 
 class FunNumber final : public Function {
-    virtual Value evaluate() const override;
-    virtual Value::Type resultType() const override { return Value::NumberValue; }
+    Value evaluate() const override;
+    Value::Type resultType() const override { return Value::NumberValue; }
 public:
     FunNumber() { setIsContextNodeSensitive(true); } // number() with no arguments uses context node. 
 };
 
 class FunSum final : public Function {
-    virtual Value evaluate() const override;
-    virtual Value::Type resultType() const override { return Value::NumberValue; }
+    Value evaluate() const override;
+    Value::Type resultType() const override { return Value::NumberValue; }
 };
 
 class FunFloor final : public Function {
-    virtual Value evaluate() const override;
-    virtual Value::Type resultType() const override { return Value::NumberValue; }
+    Value evaluate() const override;
+    Value::Type resultType() const override { return Value::NumberValue; }
 };
 
 class FunCeiling final : public Function {
-    virtual Value evaluate() const override;
-    virtual Value::Type resultType() const override { return Value::NumberValue; }
+    Value evaluate() const override;
+    Value::Type resultType() const override { return Value::NumberValue; }
 };
 
 class FunRound final : public Function {
-    virtual Value evaluate() const override;
-    virtual Value::Type resultType() const override { return Value::NumberValue; }
+    Value evaluate() const override;
+    Value::Type resultType() const override { return Value::NumberValue; }
 public:
     static double round(double);
 };
@@ -288,10 +292,10 @@ void Function::setArguments(const String& name, Vector<std::unique_ptr<Expressio
     // Functions that use the context node as an implicit argument are context node sensitive when they
     // have no arguments, but when explicit arguments are added, they are no longer context node sensitive.
     // As of this writing, the only exception to this is the "lang" function.
-    if (name != "lang" && !arguments.isEmpty())
+    if (name != "lang"_s && !arguments.isEmpty())
         setIsContextNodeSensitive(false);
 
-    setSubexpressions(std::move(arguments));
+    setSubexpressions(WTFMove(arguments));
 }
 
 Value FunLast::evaluate() const
@@ -309,21 +313,18 @@ Value FunId::evaluate() const
     Value a = argument(0).evaluate();
     StringBuilder idList; // A whitespace-separated list of IDs
 
-    if (a.isNodeSet()) {
-        const NodeSet& nodes = a.toNodeSet();
-        for (size_t i = 0; i < nodes.size(); ++i) {
-            String str = stringValue(nodes[i]);
-            idList.append(str);
+    if (!a.isNodeSet())
+        idList.append(a.toString());
+    else {
+        for (auto& node : a.toNodeSet()) {
+            idList.append(stringValue(node.get()));
             idList.append(' ');
         }
-    } else {
-        String str = a.toString();
-        idList.append(str);
     }
     
     TreeScope& contextScope = evaluationContext().node->treeScope();
     NodeSet result;
-    HashSet<Node*> resultSet;
+    HashSet<Ref<Node>> resultSet;
 
     unsigned startPos = 0;
     unsigned length = idList.length();
@@ -340,8 +341,8 @@ Value FunId::evaluate() const
 
         // If there are several nodes with the same id, id() should return the first one.
         // In WebKit, getElementById behaves so, too, although its behavior in this case is formally undefined.
-        Node* node = contextScope.getElementById(String(idList.deprecatedCharacters() + startPos, endPos - startPos));
-        if (node && resultSet.add(node).isNewEntry)
+        Node* node = contextScope.getElementById(StringView(idList).substring(startPos, endPos - startPos));
+        if (node && resultSet.add(*node).isNewEntry)
             result.append(node);
         
         startPos = endPos;
@@ -349,21 +350,19 @@ Value FunId::evaluate() const
     
     result.markSorted(false);
     
-    return Value(std::move(result));
+    return Value(WTFMove(result));
 }
 
 static inline String expandedNameLocalPart(Node* node)
 {
-    // The local part of an XPath expanded-name matches DOM local name for most node types, except for namespace nodes and processing instruction nodes.
-    ASSERT(node->nodeType() != Node::XPATH_NAMESPACE_NODE); // Not supported yet.
-    if (node->nodeType() == Node::PROCESSING_INSTRUCTION_NODE)
-        return toProcessingInstruction(node)->target();
+    if (is<ProcessingInstruction>(*node))
+        return downcast<ProcessingInstruction>(*node).target();
     return node->localName().string();
 }
 
 static inline String expandedName(Node* node)
 {
-    const AtomicString& prefix = node->prefix();
+    const AtomString& prefix = node->prefix();
     return prefix.isEmpty() ? expandedNameLocalPart(node) : prefix + ":" + expandedNameLocalPart(node);
 }
 
@@ -429,8 +428,9 @@ Value FunConcat::evaluate() const
     result.reserveCapacity(1024);
 
     for (unsigned i = 0, count = argumentCount(); i < count; ++i) {
-        String str(argument(i).evaluate().toString());
-        result.append(str);
+        EvaluationContext clonedContext(Expression::evaluationContext());
+        SetForScope<EvaluationContext> contextForScope(Expression::evaluationContext(), clonedContext);
+        result.append(argument(i).evaluate().toString());
     }
 
     return result.toString();
@@ -438,8 +438,14 @@ Value FunConcat::evaluate() const
 
 Value FunStartsWith::evaluate() const
 {
+    auto clonedContext = Expression::evaluationContext();
+
     String s1 = argument(0).evaluate().toString();
-    String s2 = argument(1).evaluate().toString();
+    String s2;
+    {
+        SetForScope<EvaluationContext> contextForScope(Expression::evaluationContext(), clonedContext);
+        s2 = argument(1).evaluate().toString();
+    }
 
     if (s2.isEmpty())
         return true;
@@ -449,8 +455,14 @@ Value FunStartsWith::evaluate() const
 
 Value FunContains::evaluate() const
 {
+    auto clonedContext = Expression::evaluationContext();
+
     String s1 = argument(0).evaluate().toString();
-    String s2 = argument(1).evaluate().toString();
+    String s2;
+    {
+        SetForScope<EvaluationContext> contextForScope(Expression::evaluationContext(), clonedContext);
+        s2 = argument(1).evaluate().toString();
+    }
 
     if (s2.isEmpty()) 
         return true;
@@ -460,8 +472,15 @@ Value FunContains::evaluate() const
 
 Value FunSubstringBefore::evaluate() const
 {
+    auto clonedContext = Expression::evaluationContext();
+
     String s1 = argument(0).evaluate().toString();
-    String s2 = argument(1).evaluate().toString();
+    String s2;
+
+    {
+        SetForScope<EvaluationContext> contextForScope(Expression::evaluationContext(), clonedContext);
+        s2 = argument(1).evaluate().toString();
+    }
 
     if (s2.isEmpty())
         return emptyString();
@@ -476,8 +495,15 @@ Value FunSubstringBefore::evaluate() const
 
 Value FunSubstringAfter::evaluate() const
 {
+    auto clonedContext = Expression::evaluationContext();
+
     String s1 = argument(0).evaluate().toString();
-    String s2 = argument(1).evaluate().toString();
+    String s2;
+
+    {
+        SetForScope<EvaluationContext> contextForScope(Expression::evaluationContext(), clonedContext);
+        s2 = argument(1).evaluate().toString();
+    }
 
     size_t i = s1.find(s2);
     if (i == notFound)
@@ -488,8 +514,21 @@ Value FunSubstringAfter::evaluate() const
 
 Value FunSubstring::evaluate() const
 {
-    String s = argument(0).evaluate().toString();
-    double doublePos = argument(1).evaluate().toNumber();
+    EvaluationContext clonedContext1(Expression::evaluationContext());
+    EvaluationContext clonedContext2(Expression::evaluationContext());
+
+    String s;
+    double doublePos;
+
+    {
+        SetForScope<EvaluationContext> contextForScope(Expression::evaluationContext(), clonedContext1);
+        s = argument(0).evaluate().toString();
+    }
+    {
+        SetForScope<EvaluationContext> contextForScope(Expression::evaluationContext(), clonedContext2);
+        doublePos = argument(1).evaluate().toNumber();
+    }
+
     if (std::isnan(doublePos))
         return emptyString();
     long pos = static_cast<long>(FunRound::round(doublePos));
@@ -526,20 +565,31 @@ Value FunStringLength::evaluate() const
 
 Value FunNormalizeSpace::evaluate() const
 {
+    // https://www.w3.org/TR/1999/REC-xpath-19991116/#function-normalize-space
     if (!argumentCount()) {
         String s = Value(Expression::evaluationContext().node.get()).toString();
-        return s.simplifyWhiteSpace();
+        return s.simplifyWhiteSpace(isXMLSpace);
     }
-
     String s = argument(0).evaluate().toString();
-    return s.simplifyWhiteSpace();
+    return s.simplifyWhiteSpace(isXMLSpace);
 }
 
 Value FunTranslate::evaluate() const
 {
+    EvaluationContext clonedContext1(Expression::evaluationContext());
+    EvaluationContext clonedContext2(Expression::evaluationContext());
+
     String s1 = argument(0).evaluate().toString();
-    String s2 = argument(1).evaluate().toString();
-    String s3 = argument(2).evaluate().toString();
+    String s2, s3;
+    {
+        SetForScope<EvaluationContext> contextForScope(Expression::evaluationContext(), clonedContext1);
+        s2 = argument(1).evaluate().toString();
+    }
+    {
+        SetForScope<EvaluationContext> contextForScope(Expression::evaluationContext(), clonedContext2);
+        s3 = argument(2).evaluate().toString();
+    }
+
     StringBuilder result;
 
     for (unsigned i1 = 0; i1 < s1.length(); ++i1) {
@@ -574,13 +624,13 @@ Value FunLang::evaluate() const
 {
     String lang = argument(0).evaluate().toString();
 
-    const Attribute* languageAttribute = 0;
+    const Attribute* languageAttribute = nullptr;
     Node* node = evaluationContext().node.get();
     while (node) {
-        if (node->isElementNode()) {
-            Element* element = toElement(node);
-            if (element->hasAttributes())
-                languageAttribute = element->findAttributeByName(XMLNames::langAttr);
+        if (is<Element>(*node)) {
+            Element& element = downcast<Element>(*node);
+            if (element.hasAttributes())
+                languageAttribute = element.findAttributeByName(XMLNames::langAttr);
         }
         if (languageAttribute)
             break;
@@ -592,7 +642,7 @@ Value FunLang::evaluate() const
 
     String langValue = languageAttribute->value();
     while (true) {
-        if (equalIgnoringCase(langValue, lang))
+        if (equalIgnoringASCIICase(langValue, lang))
             return true;
 
         // Remove suffixes one by one.
@@ -628,8 +678,8 @@ Value FunSum::evaluate() const
     // To be really compliant, we should sort the node-set, as floating point addition is not associative.
     // However, this is unlikely to ever become a practical issue, and sorting is slow.
 
-    for (unsigned i = 0; i < nodes.size(); i++)
-        sum += Value(stringValue(nodes[i])).toNumber();
+    for (auto& node : nodes)
+        sum += Value(stringValue(node.get())).toNumber();
     
     return sum;
 }
@@ -665,52 +715,52 @@ struct FunctionMapValue {
     Interval argumentCountInterval;
 };
 
-static void populateFunctionMap(HashMap<String, FunctionMapValue>& functionMap)
+static MemoryCompactLookupOnlyRobinHoodHashMap<String, FunctionMapValue> createFunctionMap()
 {
     struct FunctionMapping {
-        const char* name;
+        ASCIILiteral name;
         FunctionMapValue function;
     };
 
     static const FunctionMapping functions[] = {
-        { "boolean", { createFunctionBoolean, 1 } },
-        { "ceiling", { createFunctionCeiling, 1 } },
-        { "concat", { createFunctionConcat, Interval(2, Interval::Inf) } },
-        { "contains", { createFunctionContains, 2 } },
-        { "count", { createFunctionCount, 1 } },
-        { "false", { createFunctionFalse, 0 } },
-        { "floor", { createFunctionFloor, 1 } },
-        { "id", { createFunctionId, 1 } },
-        { "lang", { createFunctionLang, 1 } },
-        { "last", { createFunctionLast, 0 } },
-        { "local-name", { createFunctionLocalName, Interval(0, 1) } },
-        { "name", { createFunctionName, Interval(0, 1) } },
-        { "namespace-uri", { createFunctionNamespaceURI, Interval(0, 1) } },
-        { "normalize-space", { createFunctionNormalizeSpace, Interval(0, 1) } },
-        { "not", { createFunctionNot, 1 } },
-        { "number", { createFunctionNumber, Interval(0, 1) } },
-        { "position", { createFunctionPosition, 0 } },
-        { "round", { createFunctionRound, 1 } },
-        { "starts-with", { createFunctionStartsWith, 2 } },
-        { "string", { createFunctionString, Interval(0, 1) } },
-        { "string-length", { createFunctionStringLength, Interval(0, 1) } },
-        { "substring", { createFunctionSubstring, Interval(2, 3) } },
-        { "substring-after", { createFunctionSubstringAfter, 2 } },
-        { "substring-before", { createFunctionSubstringBefore, 2 } },
-        { "sum", { createFunctionSum, 1 } },
-        { "translate", { createFunctionTranslate, 3 } },
-        { "true", { createFunctionTrue, 0 } },
+        { "boolean"_s, { createFunctionBoolean, 1 } },
+        { "ceiling"_s, { createFunctionCeiling, 1 } },
+        { "concat"_s, { createFunctionConcat, Interval(2, Interval::Inf) } },
+        { "contains"_s, { createFunctionContains, 2 } },
+        { "count"_s, { createFunctionCount, 1 } },
+        { "false"_s, { createFunctionFalse, 0 } },
+        { "floor"_s, { createFunctionFloor, 1 } },
+        { "id"_s, { createFunctionId, 1 } },
+        { "lang"_s, { createFunctionLang, 1 } },
+        { "last"_s, { createFunctionLast, 0 } },
+        { "local-name"_s, { createFunctionLocalName, Interval(0, 1) } },
+        { "name"_s, { createFunctionName, Interval(0, 1) } },
+        { "namespace-uri"_s, { createFunctionNamespaceURI, Interval(0, 1) } },
+        { "normalize-space"_s, { createFunctionNormalizeSpace, Interval(0, 1) } },
+        { "not"_s, { createFunctionNot, 1 } },
+        { "number"_s, { createFunctionNumber, Interval(0, 1) } },
+        { "position"_s, { createFunctionPosition, 0 } },
+        { "round"_s, { createFunctionRound, 1 } },
+        { "starts-with"_s, { createFunctionStartsWith, 2 } },
+        { "string"_s, { createFunctionString, Interval(0, 1) } },
+        { "string-length"_s, { createFunctionStringLength, Interval(0, 1) } },
+        { "substring"_s, { createFunctionSubstring, Interval(2, 3) } },
+        { "substring-after"_s, { createFunctionSubstringAfter, 2 } },
+        { "substring-before"_s, { createFunctionSubstringBefore, 2 } },
+        { "sum"_s, { createFunctionSum, 1 } },
+        { "translate"_s, { createFunctionTranslate, 3 } },
+        { "true"_s, { createFunctionTrue, 0 } },
     };
 
-    for (size_t i = 0; i < WTF_ARRAY_LENGTH(functions); ++i)
-        functionMap.add(functions[i].name, functions[i].function);
+    MemoryCompactLookupOnlyRobinHoodHashMap<String, FunctionMapValue> map;
+    for (auto& function : functions)
+        map.add(function.name, function.function);
+    return map;
 }
 
 std::unique_ptr<Function> Function::create(const String& name, unsigned numArguments)
 {
-    static NeverDestroyed<HashMap<String, FunctionMapValue>> functionMap;
-    if (functionMap.get().isEmpty())
-        populateFunctionMap(functionMap);
+    static NeverDestroyed functionMap = createFunctionMap();
 
     auto it = functionMap.get().find(name);
     if (it == functionMap.get().end())
@@ -729,9 +779,9 @@ std::unique_ptr<Function> Function::create(const String& name)
 
 std::unique_ptr<Function> Function::create(const String& name, Vector<std::unique_ptr<Expression>> arguments)
 {
-    std::unique_ptr<Function> function = create(name, arguments.size());
+    auto function = create(name, arguments.size());
     if (function)
-        function->setArguments(name, std::move(arguments));
+        function->setArguments(name, WTFMove(arguments));
     return function;
 }
 
