@@ -35,28 +35,40 @@ String toString(TokenType type)
         return "Invalid"_s;
     case TokenType::EndOfFile:
         return "EOF"_s;
+    case TokenType::AbstractFloatLiteral:
+        return "AbstractFloatLiteral"_s;
     case TokenType::IntegerLiteral:
         return "IntegerLiteral"_s;
     case TokenType::IntegerLiteralSigned:
         return "IntegerLiteralSigned"_s;
     case TokenType::IntegerLiteralUnsigned:
         return "IntegerLiteralUnsigned"_s;
-    case TokenType::DecimalFloatLiteral:
-        return "DecimalFloatLiteral"_s;
-    case TokenType::HexFloatLiteral:
-        return "HexFloatLiteral"_s;
+    case TokenType::FloatLiteral:
+        return "FloatLiteral"_s;
     case TokenType::Identifier:
         return "Identifier"_s;
     case TokenType::ReservedWord:
         return "ReservedWord"_s;
     case TokenType::KeywordArray:
         return "array"_s;
+    case TokenType::KeywordConst:
+        return "const"_s;
+    case TokenType::KeywordElse:
+        return "else"_s;
     case TokenType::KeywordStruct:
         return "struct"_s;
     case TokenType::KeywordFn:
         return "fn"_s;
+    case TokenType::KeywordFor:
+        return "for"_s;
     case TokenType::KeywordFunction:
         return "function"_s;
+    case TokenType::KeywordIf:
+        return "if"_s;
+    case TokenType::KeywordLet:
+        return "let"_s;
+    case TokenType::KeywordOverride:
+        return "override"_s;
     case TokenType::KeywordPrivate:
         return "private"_s;
     case TokenType::KeywordRead:
@@ -87,10 +99,20 @@ String toString(TokenType type)
         return "true"_s;
     case TokenType::LiteralFalse:
         return "false"_s;
+    case TokenType::And:
+        return "&"_s;
+    case TokenType::AndAnd:
+        return "&&"_s;
+    case TokenType::AndEq:
+        return "&="_s;
     case TokenType::Arrow:
         return "->"_s;
     case TokenType::Attribute:
         return "@"_s;
+    case TokenType::Bang:
+        return "!"_s;
+    case TokenType::BangEq:
+        return "!="_s;
     case TokenType::BracketLeft:
         return "["_s;
     case TokenType::BracketRight:
@@ -105,18 +127,46 @@ String toString(TokenType type)
         return ","_s;
     case TokenType::Equal:
         return "="_s;
-    case TokenType::GT:
+    case TokenType::EqEq:
+        return "=="_s;
+    case TokenType::Gt:
         return ">"_s;
-    case TokenType::LT:
+    case TokenType::GtEq:
+        return ">="_s;
+    case TokenType::GtGt:
+        return ">>"_s;
+    case TokenType::GtGtEq:
+        return ">>="_s;
+    case TokenType::Lt:
         return "<"_s;
+    case TokenType::LtEq:
+        return "<="_s;
+    case TokenType::LtLt:
+        return "<<"_s;
+    case TokenType::LtLtEq:
+        return "<<="_s;
     case TokenType::Minus:
         return "-"_s;
     case TokenType::MinusMinus:
         return "--"_s;
+    case TokenType::MinusEq:
+        return "-="_s;
+    case TokenType::Modulo:
+        return "%"_s;
+    case TokenType::ModuloEq:
+        return "%="_s;
+    case TokenType::Or:
+        return "|"_s;
+    case TokenType::OrOr:
+        return "||"_s;
+    case TokenType::OrEq:
+        return "|="_s;
     case TokenType::Plus:
         return "+"_s;
     case TokenType::PlusPlus:
         return "++"_s;
+    case TokenType::PlusEq:
+        return "+="_s;
     case TokenType::Period:
         return "."_s;
     case TokenType::ParenLeft:
@@ -125,8 +175,22 @@ String toString(TokenType type)
         return ")"_s;
     case TokenType::Semicolon:
         return ";"_s;
+    case TokenType::Slash:
+        return "/"_s;
+    case TokenType::SlashEq:
+        return "/="_s;
     case TokenType::Star:
         return "*"_s;
+    case TokenType::StarEq:
+        return "*="_s;
+    case TokenType::Tilde:
+        return "~"_s;
+    case TokenType::Underbar:
+        return "_"_s;
+    case TokenType::Xor:
+        return "^"_s;
+    case TokenType::XorEq:
+        return "^="_s;
     }
 }
 
