@@ -31,37 +31,41 @@
 #include "NetworkLoadParameters.h"
 #include <WebCore/AuthenticationChallenge.h>
 #include <wtf/CompletionHandler.h>
-#include <wtf/Optional.h>
 #include <wtf/text/WTFString.h>
 
-#if ENABLE(NETWORK_CAPTURE)
-#include "NetworkCaptureRecorder.h"
-#include "NetworkCaptureReplayer.h"
-#endif
+namespace WebCore {
+class BlobRegistryImpl;
+}
 
 namespace WebKit {
 
-class NetworkLoad final : private NetworkDataTaskClient
-{
+class NetworkLoadScheduler;
+class NetworkProcess;
+
+class NetworkLoad final : private NetworkDataTaskClient {
     WTF_MAKE_FAST_ALLOCATED;
 public:
-    NetworkLoad(NetworkLoadClient&, NetworkLoadParameters&&, NetworkSession&);
+    NetworkLoad(NetworkLoadClient&, WebCore::BlobRegistryImpl*, NetworkLoadParameters&&, NetworkSession&);
+    NetworkLoad(NetworkLoadClient&, NetworkSession&, const Function<RefPtr<NetworkDataTask>(NetworkDataTaskClient&)>&);
     ~NetworkLoad();
 
-    void setDefersLoading(bool);
+    void start();
+    void startWithScheduling();
     void cancel();
 
     bool isAllowedToAskUserForCredentials() const;
 
     const WebCore::ResourceRequest& currentRequest() const { return m_currentRequest; }
-    void clearCurrentRequest() { m_currentRequest = WebCore::ResourceRequest(); }
+    void updateRequestAfterRedirection(WebCore::ResourceRequest&) const;
+    void reprioritizeRequest(WebCore::ResourceLoadPriority);
 
     const NetworkLoadParameters& parameters() const { return m_parameters; }
+    const URL& url() const { return parameters().request.url(); }
+    String attributedBundleIdentifier(WebPageProxyIdentifier);
 
     void continueWillSendRequest(WebCore::ResourceRequest&&);
-    void continueDidReceiveResponse();
 
-    void convertTaskToDownload(PendingDownload&, const WebCore::ResourceRequest&, const WebCore::ResourceResponse&);
+    void convertTaskToDownload(PendingDownload&, const WebCore::ResourceRequest&, const WebCore::ResourceResponse&, ResponseCompletionHandler&&);
     void setPendingDownloadID(DownloadID);
     void setSuggestedFilename(const String&);
     void setPendingDownload(PendingDownload&);
@@ -70,42 +74,33 @@ public:
     bool shouldCaptureExtraNetworkLoadMetrics() const final;
 
     String description() const;
+    void setH2PingCallback(const URL&, CompletionHandler<void(Expected<WTF::Seconds, WebCore::ResourceError>&&)>&&);
 
 private:
-#if ENABLE(NETWORK_CAPTURE)
-    void initializeForRecord(NetworkSession&);
-    void initializeForReplay(NetworkSession&);
-#endif
-    void initialize(NetworkSession&);
-
     // NetworkDataTaskClient
     void willPerformHTTPRedirection(WebCore::ResourceResponse&&, WebCore::ResourceRequest&&, RedirectCompletionHandler&&) final;
-    void didReceiveChallenge(WebCore::AuthenticationChallenge&&, ChallengeCompletionHandler&&) final;
-    void didReceiveResponseNetworkSession(WebCore::ResourceResponse&&, ResponseCompletionHandler&&) final;
-    void didReceiveData(Ref<WebCore::SharedBuffer>&&) final;
+    void didReceiveChallenge(WebCore::AuthenticationChallenge&&, NegotiatedLegacyTLS, ChallengeCompletionHandler&&) final;
+    void didReceiveInformationalResponse(WebCore::ResourceResponse&&) final;
+    void didReceiveResponse(WebCore::ResourceResponse&&, NegotiatedLegacyTLS, PrivateRelayed, ResponseCompletionHandler&&) final;
+    void didReceiveData(const WebCore::SharedBuffer&) final;
     void didCompleteWithError(const WebCore::ResourceError&, const WebCore::NetworkLoadMetrics&) final;
     void didSendData(uint64_t totalBytesSent, uint64_t totalBytesExpectedToSend) final;
     void wasBlocked() final;
     void cannotShowURL() final;
+    void wasBlockedByRestrictions() final;
+    void wasBlockedByDisabledFTP() final;
+    void didNegotiateModernTLS(const URL&) final;
 
-    void notifyDidReceiveResponse(WebCore::ResourceResponse&&, ResponseCompletionHandler&&);
-    void throttleDelayCompleted();
+    void notifyDidReceiveResponse(WebCore::ResourceResponse&&, NegotiatedLegacyTLS, PrivateRelayed, ResponseCompletionHandler&&);
 
     std::reference_wrapper<NetworkLoadClient> m_client;
+    Ref<NetworkProcess> m_networkProcess;
     const NetworkLoadParameters m_parameters;
     CompletionHandler<void(WebCore::ResourceRequest&&)> m_redirectCompletionHandler;
     RefPtr<NetworkDataTask> m_task;
-    ResponseCompletionHandler m_responseCompletionHandler;
-    
-    struct Throttle;
-    std::unique_ptr<Throttle> m_throttle;
+    WeakPtr<NetworkLoadScheduler> m_scheduler;
 
     WebCore::ResourceRequest m_currentRequest; // Updated on redirects.
-
-#if ENABLE(NETWORK_CAPTURE)
-    std::unique_ptr<NetworkCapture::Recorder> m_recorder;
-    std::unique_ptr<NetworkCapture::Replayer> m_replayer;
-#endif
 };
 
 } // namespace WebKit
