@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2017 Apple Inc.  All rights reserved.
+ * Copyright (C) 2017-2022 Apple Inc.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -33,8 +33,12 @@
 #include "NetworkProcess.h"
 #include "WebCoreArgumentCoders.h"
 #include <WebCore/CacheQueryOptions.h>
+#include <WebCore/CrossOriginAccessControl.h>
 #include <WebCore/HTTPParsers.h>
+#include <WebCore/OriginAccessPatterns.h>
+#include <WebCore/RetrieveRecordsOptions.h>
 #include <pal/SessionID.h>
+#include <wtf/CrossThreadCopier.h>
 #include <wtf/MainThread.h>
 #include <wtf/NeverDestroyed.h>
 #include <wtf/UUID.h>
@@ -44,15 +48,14 @@
 #include <wtf/text/StringBuilder.h>
 #include <wtf/text/StringHash.h>
 
-
-using namespace WebCore;
-using namespace WebCore::DOMCacheEngine;
-using namespace WebKit::NetworkCache;
-
 namespace WebKit {
 
 namespace CacheStorage {
 
+using namespace WebCore;
+using namespace WebCore::DOMCacheEngine;
+using namespace NetworkCache;
+
 static inline String computeKeyURL(const URL& url)
 {
     URL keyURL { url };
@@ -65,7 +68,7 @@ static inline Vector<uint64_t> queryCache(const Vector<RecordInformation>* recor
     if (!records)
         return { };
 
-    if (!options.ignoreMethod && request.httpMethod() != "GET")
+    if (!options.ignoreMethod && request.httpMethod() != "GET"_s)
         return { };
 
     Vector<uint64_t> results;
@@ -86,10 +89,9 @@ static inline void updateVaryInformation(RecordInformation& recordInformation, c
     }
 
     varyValue.split(',', [&](StringView view) {
-        if (!recordInformation.hasVaryStar && stripLeadingAndTrailingHTTPSpaces(view) == "*")
+        if (!recordInformation.hasVaryStar && view.trim(isHTTPSpace) == "*"_s)
             recordInformation.hasVaryStar = true;
-        String headerName = view.toString();
-        recordInformation.varyHeaders.add(headerName, request.httpHeaderField(headerName));
+        recordInformation.varyHeaders.add(view.toString(), request.httpHeaderField(view));
     });
 
     if (recordInformation.hasVaryStar)
@@ -98,7 +100,7 @@ static inline void updateVaryInformation(RecordInformation& recordInformation, c
 
 RecordInformation Cache::toRecordInformation(const Record& record)
 {
-    Key key { "record"_s, m_uniqueName, { }, createCanonicalUUIDString(), m_caches.salt() };
+    Key key { "record"_s, m_uniqueName, { }, createVersion4UUIDString(), m_caches.salt() };
     RecordInformation recordInformation { WTFMove(key), MonotonicTime::now().secondsSinceEpoch().milliseconds(), record.identifier, 0 , record.responseBodySize, record.request.url(), false, { } };
 
     updateVaryInformation(recordInformation, record.request, record.response);
@@ -106,7 +108,7 @@ RecordInformation Cache::toRecordInformation(const Record& record)
     return recordInformation;
 }
 
-Cache::Cache(Caches& caches, uint64_t identifier, State state, String&& name, String&& uniqueName)
+Cache::Cache(Caches& caches, DOMCacheIdentifier identifier, State state, String&& name, String&& uniqueName)
     : m_caches(caches)
     , m_state(state)
     , m_identifier(identifier)
@@ -127,35 +129,23 @@ void Cache::clearMemoryRepresentation()
     m_state = State::Uninitialized;
 }
 
-static RecordInformation isolatedCopy(const RecordInformation& information)
+RecordInformation RecordInformation::isolatedCopy() &&
 {
-    auto result = RecordInformation { information.key, information.insertionTime, information.identifier, information.updateResponseCounter, information.size, information.url.isolatedCopy(), information.hasVaryStar, { } };
-    HashMap<String, String> varyHeaders;
-    for (const auto& keyValue : information.varyHeaders)
-        varyHeaders.set(keyValue.key.isolatedCopy(), keyValue.value.isolatedCopy());
-    result.varyHeaders = WTFMove(varyHeaders);
-    return result;
+    return { key, insertionTime, identifier, updateResponseCounter, size, WTFMove(url).isolatedCopy(), hasVaryStar, crossThreadCopy(WTFMove(varyHeaders)) };
 }
 
 struct TraversalResult {
-    uint64_t cacheIdentifier;
+    WebCore::DOMCacheIdentifier cacheIdentifier;
     HashMap<String, Vector<RecordInformation>> records;
     Vector<Key> failedRecords;
+
+    TraversalResult isolatedCopy() &&;
 };
 
-static TraversalResult isolatedCopy(TraversalResult&& result)
+TraversalResult TraversalResult::isolatedCopy() &&
 {
-    HashMap<String, Vector<RecordInformation>> isolatedRecords;
-    for (auto& keyValue : result.records) {
-        auto& recordVector = keyValue.value;
-        for (size_t cptr = 0; cptr < recordVector.size(); cptr++)
-            recordVector[cptr] = isolatedCopy(recordVector[cptr]);
-
-        isolatedRecords.set(keyValue.key.isolatedCopy(), WTFMove(recordVector));
-    }
-
-    // No need to isolate keys since they are isolated through the copy constructor
-    return TraversalResult { result.cacheIdentifier, WTFMove(isolatedRecords), WTFMove(result.failedRecords) };
+    // No need to isolate keys since they are isolated through the copy constructor.
+    return { cacheIdentifier, crossThreadCopy(WTFMove(records)), WTFMove(failedRecords) };
 }
 
 void Cache::open(CompletionCallback&& callback)
@@ -170,9 +160,9 @@ void Cache::open(CompletionCallback&& callback)
     }
     m_state = State::Opening;
     TraversalResult traversalResult { m_identifier, { }, { } };
-    m_caches.readRecordsList(*this, [caches = makeRef(m_caches), callback = WTFMove(callback), traversalResult = WTFMove(traversalResult)](const auto* storageRecord, const auto&) mutable {
+    m_caches.readRecordsList(*this, [caches = Ref { m_caches }, callback = WTFMove(callback), traversalResult = WTFMove(traversalResult)](const auto* storageRecord, const auto&) mutable {
         if (!storageRecord) {
-            RunLoop::main().dispatch([caches = WTFMove(caches), callback = WTFMove(callback), traversalResult = isolatedCopy(WTFMove(traversalResult)) ]() mutable {
+            RunLoop::main().dispatch([caches = WTFMove(caches), callback = WTFMove(callback), traversalResult = WTFMove(traversalResult).isolatedCopy() ]() mutable {
                 for (auto& key : traversalResult.failedRecords)
                     caches->removeCacheEntry(key);
 
@@ -274,7 +264,7 @@ private:
 
 void Cache::retrieveRecord(const RecordInformation& record, Ref<ReadRecordTaskCounter>&& taskCounter)
 {
-    readRecordFromDisk(record, [caches = makeRef(m_caches), identifier = m_identifier, recordIdentifier = record.identifier, updateCounter = record.updateResponseCounter, taskCounter = WTFMove(taskCounter)](Expected<Record, Error>&& result) mutable {
+    readRecordFromDisk(record, [caches = Ref { m_caches }, identifier = m_identifier, recordIdentifier = record.identifier, updateCounter = record.updateResponseCounter, taskCounter = WTFMove(taskCounter)](Expected<Record, Error>&& result) mutable {
         auto* cache = caches->find(identifier);
         if (!cache)
             return;
@@ -282,18 +272,37 @@ void Cache::retrieveRecord(const RecordInformation& record, Ref<ReadRecordTaskCo
     });
 }
 
-void Cache::retrieveRecords(const URL& url, RecordsCallback&& callback)
+void Cache::retrieveRecords(const RetrieveRecordsOptions& options, RecordsCallback&& callback)
 {
     ASSERT(m_state == State::Open);
 
-    auto taskCounter = ReadRecordTaskCounter::create([caches = makeRef(m_caches), identifier = m_identifier, callback = WTFMove(callback)](Vector<Record>&& records, Vector<uint64_t>&& failedRecordIdentifiers) mutable {
+    auto taskCounter = ReadRecordTaskCounter::create([caches = Ref { m_caches }, identifier = m_identifier, options, callback = WTFMove(callback)](Vector<Record>&& records, Vector<uint64_t>&& failedRecordIdentifiers) mutable {
         auto* cache = caches->find(identifier);
         if (cache)
             cache->removeFromRecordList(failedRecordIdentifiers);
+
+        // https://w3c.github.io/ServiceWorker/#dom-cache-matchall (Step 5.4)
+        for (auto& record : records) {
+            if (record.response.type() != ResourceResponse::Type::Opaque)
+                continue;
+
+            if (validateCrossOriginResourcePolicy(options.crossOriginEmbedderPolicy.value, options.sourceOrigin, record.request.url(), record.response, ForNavigation::No, WebCore::EmptyOriginAccessPatterns::singleton())) {
+                callback(makeUnexpected(DOMCacheEngine::Error::CORP));
+                return;
+            }
+        }
+
+        if (!options.shouldProvideResponse) {
+            for (auto& record : records) {
+                record.response = { };
+                record.responseBody = nullptr;
+                record.responseBodySize = 0;
+            }
+        }
         callback(WTFMove(records));
     });
 
-    if (url.isNull()) {
+    if (options.request.url().isNull()) {
         for (auto& records : m_records.values()) {
             for (auto& record : records)
                 retrieveRecord(record, taskCounter.copyRef());
@@ -301,12 +310,18 @@ void Cache::retrieveRecords(const URL& url, RecordsCallback&& callback)
         return;
     }
 
-    auto* records = recordsFromURL(url);
+    if (!options.ignoreMethod && options.request.httpMethod() != "GET"_s)
+        return;
+
+    auto* records = recordsFromURL(options.request.url());
     if (!records)
         return;
 
-    for (auto& record : *records)
-        retrieveRecord(record, taskCounter.copyRef());
+    CacheQueryOptions queryOptions { options.ignoreSearch, options.ignoreMethod, options.ignoreVary };
+    for (auto& record : *records) {
+        if (DOMCacheEngine::queryCacheMatch(options.request, record.url, record.hasVaryStar, record.varyHeaders, queryOptions))
+            retrieveRecord(record, taskCounter.copyRef());
+    }
 }
 
 RecordInformation& Cache::addRecord(Vector<RecordInformation>* records, const Record& record)
@@ -385,7 +400,7 @@ void Cache::storeRecords(Vector<Record>&& records, RecordIdentifiersCallback&& c
         auto* sameURLRecords = recordsFromURL(record.request.url());
         auto matchingRecords = queryCache(sameURLRecords, record.request, options);
 
-        auto position = !matchingRecords.isEmpty() ? sameURLRecords->findMatching([&](const auto& item) { return item.identifier == matchingRecords[0]; }) : notFound;
+        auto position = !matchingRecords.isEmpty() ? sameURLRecords->findIf([&](const auto& item) { return item.identifier == matchingRecords[0]; }) : notFound;
 
         if (position == notFound) {
             record.identifier = ++m_nextRecordIdentifier;
@@ -406,33 +421,37 @@ void Cache::put(Vector<Record>&& records, RecordIdentifiersCallback&& callback)
     ASSERT(m_state == State::Open);
 
     WebCore::CacheQueryOptions options;
-    uint64_t spaceRequired = 0;
+    CheckedUint64 spaceRequired = 0;
 
     for (auto& record : records) {
         auto* sameURLRecords = recordsFromURL(record.request.url());
         auto matchingRecords = queryCache(sameURLRecords, record.request, options);
 
-        auto position = (sameURLRecords && !matchingRecords.isEmpty()) ? sameURLRecords->findMatching([&](const auto& item) { return item.identifier == matchingRecords[0]; }) : notFound;
+        auto position = (sameURLRecords && !matchingRecords.isEmpty()) ? sameURLRecords->findIf([&](const auto& item) { return item.identifier == matchingRecords[0]; }) : notFound;
 
         spaceRequired += record.responseBodySize;
-        if (position != notFound)
-            spaceRequired -= sameURLRecords->at(position).size;
+        if (position != notFound) {
+            uint64_t spaceDecreased = sameURLRecords->at(position).size;
+            if (spaceRequired >= spaceDecreased)
+                spaceRequired -= spaceDecreased;
+        }
     }
 
-    if (m_caches.hasEnoughSpace(spaceRequired)) {
-        storeRecords(WTFMove(records), WTFMove(callback));
+    if (spaceRequired.hasOverflowed()) {
+        callback(makeUnexpected(DOMCacheEngine::Error::QuotaExceeded));
         return;
     }
 
-    m_caches.requestSpace(spaceRequired, [caches = makeRef(m_caches), identifier = m_identifier, records = WTFMove(records), callback = WTFMove(callback)](std::optional<DOMCacheEngine::Error>&& error) mutable {
+    m_caches.requestSpace(spaceRequired, [caches = Ref { m_caches }, identifier = m_identifier, records = WTFMove(records), callback = WTFMove(callback)](std::optional<DOMCacheEngine::Error>&& error) mutable {
         if (error) {
             callback(makeUnexpected(error.value()));
             return;
         }
         auto* cache = caches->find(identifier);
-        if (!cache)
+        if (!cache) {
+            callback(makeUnexpected(DOMCacheEngine::Error::Internal));
             return;
-
+        }
         cache->storeRecords(WTFMove(records), WTFMove(callback));
     });
 }
@@ -449,13 +468,16 @@ void Cache::remove(WebCore::ResourceRequest&& request, WebCore::CacheQueryOption
     }
 
     records->removeAllMatching([this, &recordIdentifiers](auto& item) {
-        bool shouldRemove = recordIdentifiers.findMatching([&item](auto identifier) { return identifier == item.identifier; }) != notFound;
+        bool shouldRemove = recordIdentifiers.findIf([&item](auto identifier) { return identifier == item.identifier; }) != notFound;
         if (shouldRemove)
             this->removeRecordFromDisk(item);
         return shouldRemove;
     });
 
-    callback(WTFMove(recordIdentifiers));
+    // This operation would change caches size, so make sure callback finishes after size file is updated.
+    m_caches.updateSizeFile([callback = WTFMove(callback), recordIdentifiers = WTFMove(recordIdentifiers)]() mutable {
+        callback(WTFMove(recordIdentifiers));
+    });
 }
 
 void Cache::removeFromRecordList(const Vector<uint64_t>& recordIdentifiers)
@@ -466,7 +488,7 @@ void Cache::removeFromRecordList(const Vector<uint64_t>& recordIdentifiers)
     for (auto& records : m_records.values()) {
         auto* cache = this;
         records.removeAllMatching([cache, &recordIdentifiers](const auto& item) {
-            return notFound != recordIdentifiers.findMatching([cache, &item](const auto& identifier) {
+            return notFound != recordIdentifiers.findIf([cache, &item](const auto& identifier) {
                 if (item.identifier != identifier)
                     return false;
                 cache->removeRecordFromDisk(item);
@@ -487,7 +509,7 @@ void Cache::writeRecordToDisk(const RecordInformation& recordInformation, Record
 void Cache::updateRecordToDisk(RecordInformation& existingRecord, Record&& record, Ref<AsynchronousPutTaskCounter>&& taskCounter)
 {
     ++existingRecord.updateResponseCounter;
-    readRecordFromDisk(existingRecord, [caches = makeRef(m_caches), identifier = m_identifier, recordIdentifier = existingRecord.identifier, record = WTFMove(record), taskCounter = WTFMove(taskCounter)](Expected<Record, Error>&& result) mutable {
+    readRecordFromDisk(existingRecord, [caches = Ref { m_caches }, identifier = m_identifier, recordIdentifier = existingRecord.identifier, record = WTFMove(record), taskCounter = WTFMove(taskCounter)](Expected<Record, Error>&& result) mutable {
         if (!result.has_value())
             return;
 
@@ -499,7 +521,7 @@ void Cache::updateRecordToDisk(RecordInformation& existingRecord, Record&& recor
         if (!sameURLRecords)
             return;
 
-        auto position = sameURLRecords->findMatching([&] (const auto& item) { return item.identifier == recordIdentifier; });
+        auto position = sameURLRecords->findIf([&] (const auto& item) { return item.identifier == recordIdentifier; });
         if (position == notFound)
             return;
         auto& recordInfo = sameURLRecords->at(position);
@@ -534,7 +556,7 @@ Storage::Record Cache::encode(const RecordInformation& recordInformation, const
     encoder << recordInformation.insertionTime;
     encoder << recordInformation.size;
     encoder << record.requestHeadersGuard;
-    record.request.encodeWithoutPlatformData(encoder);
+    encoder << record.request;
     record.options.encodePersistent(encoder);
     encoder << record.referrer;
 
@@ -549,52 +571,89 @@ Storage::Record Cache::encode(const RecordInformation& recordInformation, const
     WTF::switchOn(record.responseBody, [](const Ref<WebCore::FormData>& formData) {
         // FIXME: Store form data body.
     }, [&](const Ref<WebCore::SharedBuffer>& buffer) {
-        body = { reinterpret_cast<const uint8_t*>(buffer->data()), buffer->size() };
+        body = { buffer->data(), buffer->size() };
     }, [](const std::nullptr_t&) {
     });
 
     return { recordInformation.key, { }, header, body, { } };
 }
 
-std::optional<Cache::DecodedRecord> Cache::decodeRecordHeader(const Storage::Record& storage)
+static std::optional<WebCore::DOMCacheEngine::Record> decodeDOMCacheRecord(WTF::Persistence::Decoder& decoder)
 {
-    WTF::Persistence::Decoder decoder(storage.header.data(), storage.header.size());
-
-    Record record;
-
-    double insertionTime;
-    if (!decoder.decode(insertionTime))
+    std::optional<FetchHeaders::Guard> requestHeadersGuard;
+    decoder >> requestHeadersGuard;
+    if (!requestHeadersGuard)
+        return std::nullopt;
+    
+    std::optional<ResourceRequest> request;
+    decoder >> request;
+    if (!request)
+        return std::nullopt;
+    
+    FetchOptions options;
+    if (!FetchOptions::decodePersistent(decoder, options))
+        return std::nullopt;
+    
+    std::optional<String> referrer;
+    decoder >> referrer;
+    if (!referrer)
+        return std::nullopt;
+    
+    std::optional<FetchHeaders::Guard> responseHeadersGuard;
+    decoder >> responseHeadersGuard;
+    if (!responseHeadersGuard)
         return std::nullopt;
 
-    uint64_t size;
-    if (!decoder.decode(size))
+    std::optional<ResourceResponse> response;
+    decoder >> response;
+    if (!response)
         return std::nullopt;
-
-    if (!decoder.decode(record.requestHeadersGuard))
-        return std::nullopt;
-
-    if (!record.request.decodeWithoutPlatformData(decoder))
-        return std::nullopt;
-
-    if (!FetchOptions::decodePersistent(decoder, record.options))
-        return std::nullopt;
-
-    if (!decoder.decode(record.referrer))
-        return std::nullopt;
-
-    if (!decoder.decode(record.responseHeadersGuard))
-        return std::nullopt;
-
-    if (!decoder.decode(record.response))
-        return std::nullopt;
-
-    if (!decoder.decode(record.responseBodySize))
+    
+    std::optional<uint64_t> responseBodySize;
+    decoder >> responseBodySize;
+    if (!responseBodySize)
         return std::nullopt;
 
     if (!decoder.verifyChecksum())
         return std::nullopt;
 
-    return DecodedRecord { insertionTime, size, WTFMove(record) };
+    return {{
+        0,
+        0,
+        WTFMove(*requestHeadersGuard),
+        WTFMove(*request),
+        WTFMove(options),
+        WTFMove(*referrer),
+        WTFMove(*responseHeadersGuard),
+        WTFMove(*response),
+        { },
+        WTFMove(*responseBodySize)
+    }};
+}
+
+std::optional<Cache::DecodedRecord> Cache::decodeRecordHeader(const Storage::Record& storage)
+{
+    WTF::Persistence::Decoder decoder(storage.header.span());
+
+    std::optional<double> insertionTime;
+    decoder >> insertionTime;
+    if (!insertionTime)
+        return std::nullopt;
+
+    std::optional<uint64_t> size;
+    decoder >> size;
+    if (!size)
+        return std::nullopt;
+
+    std::optional<WebCore::DOMCacheEngine::Record> record = decodeDOMCacheRecord(decoder);
+    if (!record)
+        return std::nullopt;
+
+    return {{
+        WTFMove(*insertionTime),
+        WTFMove(*size),
+        WTFMove(*record)
+    }};
 }
 
 std::optional<Record> Cache::decode(const Storage::Record& storage)
@@ -607,7 +666,7 @@ std::optional<Record> Cache::decode(const Storage::Record& storage)
     auto record = WTFMove(result->record);
     record.responseBody = WebCore::SharedBuffer::create(storage.body.data(), storage.body.size());
 
-    return WTFMove(record);
+    return record;
 }
 
 Vector<Key> Cache::keys() const
