@@ -85,7 +85,7 @@ enum class SendOption : uint8_t {
 #endif
 };
 
-enum class SendSyncOption {
+enum class SendSyncOption : uint8_t {
     // Use this to inform that this sync call will suspend this process until the user responds with input.
     InformPlatformProcessWillSuspend = 1 << 0,
     UseFullySynchronousModeForTesting = 1 << 1,
@@ -129,12 +129,44 @@ class MachMessage;
 class UnixMessage;
 class WorkQueueMessageReceiver;
 
+struct AsyncReplyIDType;
+using AsyncReplyID = AtomicObjectIdentifier<AsyncReplyIDType>;
+
+template<typename T> struct ConnectionSendSyncResult {
+    std::unique_ptr<Decoder> decoder;
+    std::optional<typename T::ReplyArguments> replyArguments;
+
+    explicit operator bool() const { return !!decoder; }
+
+    typename T::ReplyArguments& reply()
+    {
+        ASSERT(!!replyArguments);
+        return *replyArguments;
+    }
+
+    typename T::ReplyArguments takeReply()
+    {
+        ASSERT(!!replyArguments);
+        return WTFMove(replyArguments).value();
+    }
+
+    template<typename... U>
+    typename T::ReplyArguments takeReplyOr(U&&... defaultValues)
+    {
+        return WTFMove(replyArguments).value_or(typename T::ReplyArguments { std::forward<U>(defaultValues)... });
+    }
+};
+
+struct ConnectionAsyncReplyHandler {
+    CompletionHandler<void(Decoder*)> completionHandler;
+    AsyncReplyID replyID;
+};
+
 class Connection : public ThreadSafeRefCountedAndCanMakeThreadSafeWeakPtr<Connection, WTF::DestructionThread::MainRunLoop> {
 public:
     enum SyncRequestIDType { };
-    using SyncRequestID = ObjectIdentifier<SyncRequestIDType>;
-    enum AsyncReplyIDType { };
-    using AsyncReplyID = ObjectIdentifier<AsyncReplyIDType>;
+    using SyncRequestID = AtomicObjectIdentifier<SyncRequestIDType>;
+    using AsyncReplyID = IPC::AsyncReplyID;
 
     class Client : public MessageReceiver {
     public:
@@ -145,19 +177,48 @@ public:
         virtual ~Client() { }
     };
 
+    struct Identifier;
+    struct Handle {
+        friend struct Identifier;
+        WTF_MAKE_NONCOPYABLE(Handle);
+        Handle() = default;
+        Handle(Handle&&) = default;
+        Handle& operator=(Handle&&) = default;
+
 #if USE(UNIX_DOMAIN_SOCKETS)
-    using Handle = UnixFileDescriptor;
+        Handle(UnixFileDescriptor&& inHandle)
+            : handle(WTFMove(inHandle))
+        { }
+        explicit operator bool() const { return !!handle; }
 #elif OS(WINDOWS)
-    using Handle = Win32Handle;
+        Handle(Win32Handle&& inHandle)
+            : handle(WTFMove(inHandle))
+        { }
+        explicit operator bool() const { return !!handle; }
 #elif OS(DARWIN)
-    using Handle = MachSendRight;
+        Handle(MachSendRight&& sendRight)
+            : handle(WTFMove(sendRight))
+        { }
+        explicit operator bool() const { return MACH_PORT_VALID(handle.sendRight()); }
 #endif
+        void encode(Encoder&);
+        static std::optional<Handle> decode(Decoder&);
+
+    private:
+#if USE(UNIX_DOMAIN_SOCKETS)
+        UnixFileDescriptor handle;
+#elif OS(WINDOWS)
+        Win32Handle handle;
+#elif OS(DARWIN)
+        MachSendRight handle;
+#endif
+    };
 
     struct Identifier {
         Identifier() = default;
 #if USE(UNIX_DOMAIN_SOCKETS)
         explicit Identifier(Handle&& handle)
-            : Identifier(handle.release())
+            : Identifier(handle.handle.release())
         {
         }
         explicit Identifier(int handle)
@@ -168,7 +229,7 @@ public:
         int handle { -1 };
 #elif OS(WINDOWS)
         explicit Identifier(Handle&& handle)
-            : Identifier(handle.release())
+            : Identifier(handle.handle.leak())
         {
         }
         explicit Identifier(HANDLE handle)
@@ -179,7 +240,7 @@ public:
         HANDLE handle { 0 };
 #elif OS(DARWIN)
         explicit Identifier(Handle&& handle)
-            : Identifier(handle.leakSendRight())
+            : Identifier(handle.handle.leakSendRight())
         {
         }
         explicit Identifier(mach_port_t port)
@@ -191,7 +252,7 @@ public:
             , xpcConnection(WTFMove(xpcConnection))
         {
         }
-        operator bool() const { return MACH_PORT_VALID(port); }
+        explicit operator bool() const { return MACH_PORT_VALID(port); }
         mach_port_t port { MACH_PORT_NULL };
         OSObjectPtr<xpc_connection_t> xpcConnection;
 #endif
@@ -217,7 +278,7 @@ public:
     Client* client() const { return m_client; }
 
     enum UniqueIDType { };
-    using UniqueID = ObjectIdentifier<UniqueIDType>;
+    using UniqueID = AtomicObjectIdentifier<UniqueIDType>;
 
     static RefPtr<Connection> connection(UniqueID);
     UniqueID uniqueID() const { return m_uniqueID; }
@@ -233,6 +294,9 @@ public:
     typedef void (*DidCloseOnConnectionWorkQueueCallback)(Connection*);
     void setDidCloseOnConnectionWorkQueueCallback(DidCloseOnConnectionWorkQueueCallback);
 
+    using OutgoingMessageQueueIsGrowingLargeCallback = Function<void()>;
+    void setOutgoingMessageQueueIsGrowingLargeCallback(OutgoingMessageQueueIsGrowingLargeCallback&&);
+
     // Adds a message receive queue. The client should make sure the instance is removed before it goes
     // out of scope.
     // std::nullopt ReceiverMatchSpec matches all receivers.
@@ -265,71 +329,43 @@ public:
     // Sync senders should check the SendSyncResult for true/false in case they need to know if the result was really received.
     // Sync senders should hold on to the SendSyncResult in case they reference the contents of the reply via DataRefererence / ArrayReference.
 
-    template<typename T>
-    struct SendSyncResult {
-        std::unique_ptr<Decoder> decoder;
-        std::optional<typename T::ReplyArguments> replyArguments;
-
-        explicit operator bool() const { return !!decoder; }
-
-        typename T::ReplyArguments& reply()
-        {
-            ASSERT(!!replyArguments);
-            return *replyArguments;
-        }
-
-        typename T::ReplyArguments takeReply()
-        {
-            ASSERT(!!replyArguments);
-            return WTFMove(replyArguments).value();
-        }
-
-        template<typename... U>
-        typename T::ReplyArguments takeReplyOr(U&&... defaultValues)
-        {
-            return WTFMove(replyArguments).value_or(typename T::ReplyArguments { std::forward<U>(defaultValues)... });
-        }
-    };
-
+    template<typename T> using SendSyncResult = ConnectionSendSyncResult<T>;
     template<typename T> SendSyncResult<T> sendSync(T&& message, uint64_t destinationID, Timeout = Timeout::infinity(), OptionSet<SendSyncOption> sendSyncOptions = { }); // Main thread only.
 
     template<typename> bool waitForAndDispatchImmediately(uint64_t destinationID, Timeout, OptionSet<WaitForOption> waitForOptions = { }); // Main thread only.
     template<typename> bool waitForAsyncReplyAndDispatchImmediately(AsyncReplyID, Timeout); // Main thread only.
 
     // Thread-safe.
-    template<typename T, typename C, typename U>
-    AsyncReplyID sendWithAsyncReply(T&& message, C&& completionHandler, ObjectIdentifier<U> destinationID = { }, OptionSet<SendOption> sendOptions = { })
+    template<typename T, typename C>
+    AsyncReplyID sendWithAsyncReply(T&& message, C&& completionHandler, const ObjectIdentifierGenericBase& destinationID, OptionSet<SendOption> sendOptions = { })
     {
         return sendWithAsyncReply<T, C>(WTFMove(message), WTFMove(completionHandler), destinationID.toUInt64(), sendOptions);
     }
 
     // Thread-safe.
-    template<typename T, typename U>
-    bool send(T&& message, ObjectIdentifier<U> destinationID, OptionSet<SendOption> sendOptions = { }, std::optional<Thread::QOS> qos = std::nullopt)
+    template<typename T>
+    bool send(T&& message, const ObjectIdentifierGenericBase& destinationID, OptionSet<SendOption> sendOptions = { }, std::optional<Thread::QOS> qos = std::nullopt)
     {
         return send<T>(WTFMove(message), destinationID.toUInt64(), sendOptions, qos);
     }
 
     // Main thread only.
-    template<typename T, typename U>
-    SendSyncResult<T> sendSync(T&& message, ObjectIdentifier<U> destinationID, Timeout timeout = Timeout::infinity(), OptionSet<SendSyncOption> sendSyncOptions = { })
+    template<typename T>
+    SendSyncResult<T> sendSync(T&& message, const ObjectIdentifierGenericBase& destinationID, Timeout timeout = Timeout::infinity(), OptionSet<SendSyncOption> sendSyncOptions = { })
     {
         return sendSync<T>(WTFMove(message), destinationID.toUInt64(), timeout, sendSyncOptions);
     }
 
     // Main thread only.
-    template<typename T, typename U>
-    bool waitForAndDispatchImmediately(ObjectIdentifier<U> destinationID, Timeout timeout, OptionSet<WaitForOption> waitForOptions = { })
+    template<typename T>
+    bool waitForAndDispatchImmediately(const ObjectIdentifierGenericBase& destinationID, Timeout timeout, OptionSet<WaitForOption> waitForOptions = { })
     {
         return waitForAndDispatchImmediately<T>(destinationID.toUInt64(), timeout, waitForOptions);
     }
 
     bool sendMessage(UniqueRef<Encoder>&&, OptionSet<SendOption> sendOptions, std::optional<Thread::QOS> = std::nullopt);
 
-    struct AsyncReplyHandler {
-        CompletionHandler<void(Decoder*)> completionHandler;
-        AsyncReplyID replyID;
-    };
+    using AsyncReplyHandler = ConnectionAsyncReplyHandler;
     bool sendMessageWithAsyncReply(UniqueRef<Encoder>&&, AsyncReplyHandler, OptionSet<SendOption> sendOptions, std::optional<Thread::QOS> = std::nullopt);
     UniqueRef<Encoder> createSyncMessageEncoder(MessageName, uint64_t destinationID, SyncRequestID&);
     std::unique_ptr<Decoder> sendSyncMessage(SyncRequestID, UniqueRef<Encoder>&&, Timeout, OptionSet<SendSyncOption> sendSyncOptions);
@@ -379,6 +415,8 @@ public:
 
     template<typename T, typename C> static AsyncReplyHandler makeAsyncReplyHandler(C&& completionHandler, ThreadLikeAssertion callThread = CompletionHandlerCallThread::AnyThread);
 
+    CompletionHandler<void(Decoder*)> takeAsyncReplyHandler(AsyncReplyID);
+
 private:
     Connection(Identifier, bool isServer);
     void platformInitialize(Identifier);
@@ -392,7 +430,7 @@ private:
 
     std::unique_ptr<Decoder> waitForMessage(MessageName, uint64_t destinationID, Timeout, OptionSet<WaitForOption>);
 
-    SyncRequestID makeSyncRequestID() { return SyncRequestID::generateThreadSafe(); }
+    SyncRequestID makeSyncRequestID() { return SyncRequestID::generate(); }
     bool pushPendingSyncRequestID(SyncRequestID);
     void popPendingSyncRequestID(SyncRequestID);
     std::unique_ptr<Decoder> waitForSyncReply(SyncRequestID, MessageName, Timeout, OptionSet<SendSyncOption>);
@@ -447,7 +485,6 @@ private:
         void operator()(SyncMessageState*) const;
     };
     void addAsyncReplyHandler(AsyncReplyHandler&&);
-    CompletionHandler<void(Decoder*)> takeAsyncReplyHandler(AsyncReplyID);
     void cancelAsyncReplyHandlers();
 
     static Lock s_connectionMapLock;
@@ -460,6 +497,8 @@ private:
     bool m_onlySendMessagesAsDispatchWhenWaitingForSyncReplyWhenProcessingSuchAMessage { false };
     bool m_shouldExitOnSyncMessageSendFailure { false };
     DidCloseOnConnectionWorkQueueCallback m_didCloseOnConnectionWorkQueueCallback { nullptr };
+    OutgoingMessageQueueIsGrowingLargeCallback m_outgoingMessageQueueIsGrowingLargeCallback;
+    MonotonicTime m_lastOutgoingMessageQueueIsGrowingLargeCallbackCallTime WTF_GUARDED_BY_LOCK(m_outgoingMessagesLock);
 
     Ref<WorkQueue> m_connectionQueue;
     bool m_isConnected { false };
@@ -656,7 +695,7 @@ template<typename T> bool Connection::waitForAsyncReplyAndDispatchImmediately(As
 
     ASSERT(decoder->messageReceiverName() == ReceiverName::AsyncReply);
     ASSERT(decoder->destinationID() == replyID.toUInt64());
-    auto handler = takeAsyncReplyHandler(makeObjectIdentifier<AsyncReplyIDType>(decoder->destinationID()));
+    auto handler = takeAsyncReplyHandler(AtomicObjectIdentifier<AsyncReplyIDType>(decoder->destinationID()));
     if (!handler) {
         ASSERT_NOT_REACHED();
         return false;
@@ -686,7 +725,7 @@ Connection::AsyncReplyHandler Connection::makeAsyncReplyHandler(C&& completionHa
                     cancelReply<T>(WTFMove(completionHandler));
             }, callThread
         },
-        AsyncReplyID::generateThreadSafe()
+        AsyncReplyID::generate()
     };
 }
 
