@@ -26,19 +26,24 @@
 #include "config.h"
 #include "MessageSender.h"
 
+#include "Connection.h"
+
 namespace IPC {
 
 MessageSender::~MessageSender() = default;
 
-bool MessageSender::sendMessage(std::unique_ptr<Encoder> encoder, OptionSet<SendOption> sendOptions, Optional<std::pair<CompletionHandler<void(IPC::Decoder*)>, uint64_t>>&& asyncReplyInfo)
+bool MessageSender::sendMessage(UniqueRef<Encoder>&& encoder, OptionSet<SendOption> sendOptions)
 {
     auto* connection = messageSenderConnection();
     ASSERT(connection);
-
-    if (asyncReplyInfo)
-        IPC::addAsyncReplyHandler(*connection, asyncReplyInfo->second, WTFMove(asyncReplyInfo->first));
-
     return connection->sendMessage(WTFMove(encoder), sendOptions);
 }
 
+bool MessageSender::sendMessageWithAsyncReply(UniqueRef<Encoder>&& encoder, AsyncReplyHandler replyHandler, OptionSet<SendOption> sendOptions)
+{
+    auto* connection = messageSenderConnection();
+    ASSERT(connection);
+    return connection->sendMessageWithAsyncReply(WTFMove(encoder), WTFMove(replyHandler), sendOptions);
+}
+
 } // namespace IPC
