@@ -45,6 +45,8 @@ namespace IPC {
 class Decoder;
 class Encoder;
 class Connection;
+
+template<typename, typename> struct ArgumentCoder;
 }
 
 namespace WebCore {
@@ -57,61 +59,57 @@ namespace WebKit {
 
 enum class MemoryLedger { None, Default, Network, Media, Graphics, Neural };
 
+class SharedMemoryHandle {
+    WTF_MAKE_NONCOPYABLE(SharedMemoryHandle);
+public:
+    using Type =
+#if USE(UNIX_DOMAIN_SOCKETS)
+        UnixFileDescriptor;
+#elif OS(DARWIN)
+        MachSendRight;
+#elif OS(WINDOWS)
+        Win32Handle;
+#endif
+
+    SharedMemoryHandle() = default;
+    SharedMemoryHandle(SharedMemoryHandle&&) = default;
+    SharedMemoryHandle(SharedMemoryHandle::Type&&, size_t);
+
+    SharedMemoryHandle& operator=(SharedMemoryHandle&&) = default;
+
+    bool isNull() const;
+
+    size_t size() const { return m_size; }
+
+    // Take/Set ownership of the memory for jetsam purposes.
+    void takeOwnershipOfMemory(MemoryLedger) const;
+    void setOwnershipOfMemory(const WebCore::ProcessIdentity&, MemoryLedger) const;
+
+#if USE(UNIX_DOMAIN_SOCKETS)
+    UnixFileDescriptor releaseHandle();
+#endif
+
+private:
+    friend struct IPC::ArgumentCoder<SharedMemoryHandle, void>;
+    friend class SharedMemory;
+#if USE(UNIX_DOMAIN_SOCKETS)
+    friend class IPC::Connection;
+#endif
+
+    Type m_handle;
+    size_t m_size { 0 };
+};
+
 class SharedMemory : public ThreadSafeRefCounted<SharedMemory> {
 public:
-    enum class Protection {
-        ReadOnly,
-        ReadWrite
-    };
+    using Handle = SharedMemoryHandle;
 
-    class Handle {
-    public:
-        Handle() = default;
-        Handle(const Handle&) = default;
-        Handle(Handle&&) = default;
-        Handle& operator=(const Handle&) = default;
-        Handle& operator=(Handle&&) = default;
-
-        bool isNull() const;
-
-        size_t size() const { return m_size; }
-
-        // Take/Set ownership of the memory for jetsam purposes.
-        void takeOwnershipOfMemory(MemoryLedger) const;
-        void setOwnershipOfMemory(const WebCore::ProcessIdentity&, MemoryLedger) const;
-
-        void clear();
-
-        void encode(IPC::Encoder&) const;
-        static WARN_UNUSED_RETURN bool decode(IPC::Decoder&, Handle&);
-#if USE(UNIX_DOMAIN_SOCKETS)
-        UnixFileDescriptor releaseHandle();
-#endif
-
-#if PLATFORM(COCOA)
-        explicit Handle(MachSendRight&&, size_t);
-        static std::optional<Handle> create(void* data, size_t, Protection);
-#endif
-
-    private:
-#if USE(UNIX_DOMAIN_SOCKETS)
-        mutable UnixFileDescriptor m_handle;
-#elif OS(DARWIN)
-        mutable MachSendRight m_handle;
-#elif OS(WINDOWS)
-        mutable Win32Handle m_handle;
-#endif
-        size_t m_size { 0 };
-        friend class SharedMemory;
-#if USE(UNIX_DOMAIN_SOCKETS)
-        friend class IPC::Connection;
-#endif
-    };
+    enum class Protection : bool { ReadOnly, ReadWrite };
 
     // FIXME: Change these factory functions to return Ref<SharedMemory> and crash on failure.
     static RefPtr<SharedMemory> allocate(size_t);
     static RefPtr<SharedMemory> copyBuffer(const WebCore::FragmentedSharedBuffer&);
-    static RefPtr<SharedMemory> map(const Handle&, Protection);
+    static RefPtr<SharedMemory> map(Handle&&, Protection);
 #if USE(UNIX_DOMAIN_SOCKETS)
     static RefPtr<SharedMemory> wrapMap(void*, size_t, int fileDescriptor);
 #elif OS(DARWIN)
@@ -143,6 +141,10 @@ public:
     Ref<WebCore::SharedBuffer> createSharedBuffer(size_t) const;
 
 private:
+#if OS(DARWIN)
+    WTF::MachSendRight createSendRight(Protection) const;
+#endif
+
     size_t m_size;
     void* m_data;
 #if PLATFORM(COCOA)
@@ -159,4 +161,13 @@ private:
 #endif
 };
 
+} // namespace WebKit
+
+namespace IPC {
+
+template<> struct ArgumentCoder<WebKit::SharedMemoryHandle, void> {
+    static void encode(Encoder&, WebKit::SharedMemoryHandle&&);
+    static std::optional<WebKit::SharedMemoryHandle> decode(Decoder&);
 };
+
+} // namespace IPC
