@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012-2021 Apple Inc. All rights reserved.
+ * Copyright (C) 2012-2023 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -25,17 +25,20 @@
 
 #pragma once
 
+#include "Connection.h"
 #include "DrawingAreaInfo.h"
 #include "EditorState.h"
-#include "GenericCallback.h"
 #include "PlatformCAAnimationRemote.h"
+#include "PlaybackSessionContextIdentifier.h"
 #include "RemoteLayerBackingStore.h"
 #include "TransactionID.h"
 #include <WebCore/Color.h>
 #include <WebCore/FilterOperations.h>
 #include <WebCore/FloatPoint3D.h>
 #include <WebCore/FloatSize.h>
+#include <WebCore/HTMLMediaElementIdentifier.h>
 #include <WebCore/LayoutMilestone.h>
+#include <WebCore/MediaPlayerEnums.h>
 #include <WebCore/Model.h>
 #include <WebCore/PlatformCALayer.h>
 #include <WebCore/ScrollTypes.h>
@@ -49,6 +52,11 @@
 #include "DynamicViewportSizeUpdate.h"
 #endif
 
+#if ENABLE(THREADED_ANIMATION_RESOLUTION)
+#include <WebCore/AcceleratedEffect.h>
+#include <WebCore/AcceleratedEffectValues.h>
+#endif
+
 namespace IPC {
 class Decoder;
 class Encoder;
@@ -103,6 +111,7 @@ enum class LayerChange : uint64_t {
     DescendentOfSeparatedPortalChanged  = 1LLU << 42,
 #endif
 #endif
+    VideoGravityChanged                 = 1LLU << 44,
 };
 
 class RemoteLayerTreeTransaction {
@@ -115,13 +124,19 @@ public:
         void encode(IPC::Encoder&) const;
         static std::optional<LayerCreationProperties> decode(IPC::Decoder&);
 
-        WebCore::GraphicsLayer::PlatformLayerID layerID;
+        WebCore::PlatformLayerIdentifier layerID;
         WebCore::PlatformCALayer::LayerType type;
+        std::optional<PlaybackSessionContextIdentifier> playerIdentifier;
+        std::optional<WebCore::FloatSize> initialSize;
+        std::optional<WebCore::FloatSize> naturalSize;
 
         uint32_t hostingContextID;
         float hostingDeviceScaleFactor;
         bool preservesFlip;
-        
+
+        // FIXME: This could be a variant<CustomData, HostData, ModelData, NoData>.
+        Markable<WebCore::LayerHostingContextIdentifier> hostIdentifier;
+
 #if ENABLE(MODEL_ELEMENT)
         RefPtr<WebCore::Model> model;
 #endif
@@ -154,20 +169,27 @@ public:
         std::unique_ptr<WebCore::TransformationMatrix> sublayerTransform;
         std::unique_ptr<WebCore::FloatRoundedRect> shapeRoundedRect;
 
-        Vector<WebCore::GraphicsLayer::PlatformLayerID> children;
+        Vector<WebCore::PlatformLayerIdentifier> children;
 
         Vector<std::pair<String, PlatformCAAnimationRemote::Properties>> addedAnimations;
         HashSet<String> keysOfAnimationsToRemove;
+#if ENABLE(THREADED_ANIMATION_RESOLUTION)
+        Vector<Ref<WebCore::AcceleratedEffect>> effects;
+        WebCore::AcceleratedEffectValues baseValues;
+#endif
 
         WebCore::FloatPoint3D position;
         WebCore::FloatPoint3D anchorPoint { 0.5, 0.5, 0 };
         WebCore::FloatRect bounds;
         WebCore::FloatRect contentsRect { 0, 0, 1, 1 };
+        // Used in the WebContent process.
         std::unique_ptr<RemoteLayerBackingStore> backingStore;
+        // Used in the UI process.
+        std::unique_ptr<RemoteLayerBackingStoreProperties> backingStoreProperties;
         std::unique_ptr<WebCore::FilterOperations> filters;
         WebCore::Path shapePath;
-        Markable<WebCore::GraphicsLayer::PlatformLayerID> maskLayerID;
-        Markable<WebCore::GraphicsLayer::PlatformLayerID> clonedLayerID;
+        Markable<WebCore::PlatformLayerIdentifier> maskLayerID;
+        Markable<WebCore::PlatformLayerIdentifier> clonedLayerID;
 #if ENABLE(SCROLLING_THREAD)
         WebCore::ScrollingNodeID scrollingNodeID { 0 };
 #endif
@@ -184,6 +206,7 @@ public:
         WebCore::PlatformCALayer::FilterType magnificationFilter { WebCore::PlatformCALayer::FilterType::Linear };
         WebCore::BlendMode blendMode { WebCore::BlendMode::Normal };
         WebCore::WindRule windRule { WebCore::WindRule::NonZero };
+        WebCore::MediaPlayerVideoGravity videoGravity { WebCore::MediaPlayerVideoGravity::ResizeAspect };
         bool antialiasesEdges { true };
         bool hidden { false };
         bool backingStoreAttached { true };
@@ -212,29 +235,38 @@ public:
     void encode(IPC::Encoder&) const;
     static WARN_UNUSED_RETURN bool decode(IPC::Decoder&, RemoteLayerTreeTransaction&);
 
-    WebCore::GraphicsLayer::PlatformLayerID rootLayerID() const { return m_rootLayerID; }
-    void setRootLayerID(WebCore::GraphicsLayer::PlatformLayerID);
+    WebCore::PlatformLayerIdentifier rootLayerID() const { return m_rootLayerID; }
+    void setRootLayerID(WebCore::PlatformLayerIdentifier);
     void layerPropertiesChanged(PlatformCALayerRemote&);
     void setCreatedLayers(Vector<LayerCreationProperties>);
-    void setDestroyedLayerIDs(Vector<WebCore::GraphicsLayer::PlatformLayerID>);
-    void setLayerIDsWithNewlyUnreachableBackingStore(Vector<WebCore::GraphicsLayer::PlatformLayerID>);
+    void setDestroyedLayerIDs(Vector<WebCore::PlatformLayerIdentifier>);
+    void setLayerIDsWithNewlyUnreachableBackingStore(Vector<WebCore::PlatformLayerIdentifier>);
 
 #if !defined(NDEBUG) || !LOG_DISABLED
     String description() const;
     void dump() const;
 #endif
 
-    typedef HashMap<WebCore::GraphicsLayer::PlatformLayerID, std::unique_ptr<LayerProperties>> LayerPropertiesMap;
+    typedef HashMap<WebCore::PlatformLayerIdentifier, std::unique_ptr<LayerProperties>> LayerPropertiesMap;
+    
+    bool hasAnyLayerChanges() const
+    {
+        return m_changedLayerProperties.size() || m_changedLayers.size() || m_createdLayers.size() || m_destroyedLayerIDs.size() || m_layerIDsWithNewlyUnreachableBackingStore.size();
+    }
 
-    Vector<LayerCreationProperties> createdLayers() const { return m_createdLayers; }
-    Vector<WebCore::GraphicsLayer::PlatformLayerID> destroyedLayers() const { return m_destroyedLayerIDs; }
-    Vector<WebCore::GraphicsLayer::PlatformLayerID> layerIDsWithNewlyUnreachableBackingStore() const { return m_layerIDsWithNewlyUnreachableBackingStore; }
+    const Vector<LayerCreationProperties>& createdLayers() const { return m_createdLayers; }
+    const Vector<WebCore::PlatformLayerIdentifier>& destroyedLayers() const { return m_destroyedLayerIDs; }
+    const Vector<WebCore::PlatformLayerIdentifier>& layerIDsWithNewlyUnreachableBackingStore() const { return m_layerIDsWithNewlyUnreachableBackingStore; }
 
     HashSet<RefPtr<PlatformCALayerRemote>>& changedLayers() { return m_changedLayers; }
 
     const LayerPropertiesMap& changedLayerProperties() const { return m_changedLayerProperties; }
     LayerPropertiesMap& changedLayerProperties() { return m_changedLayerProperties; }
 
+    void setRemoteContextHostedIdentifier(Markable<WebCore::LayerHostingContextIdentifier> identifier) { m_remoteContextHostedIdentifier = identifier; }
+    Markable<WebCore::LayerHostingContextIdentifier> remoteContextHostedIdentifier() const { return m_remoteContextHostedIdentifier; }
+    bool isMainFrameProcessTransaction() const { return !m_remoteContextHostedIdentifier; }
+
     WebCore::IntSize contentsSize() const { return m_contentsSize; }
     void setContentsSize(const WebCore::IntSize& size) { m_contentsSize = size; };
 
@@ -269,8 +301,11 @@ public:
     void setScaleWasSetByUIProcess(bool scaleWasSetByUIProcess) { m_scaleWasSetByUIProcess = scaleWasSetByUIProcess; }
 
 #if PLATFORM(MAC)
-    WebCore::GraphicsLayer::PlatformLayerID pageScalingLayerID() const { return m_pageScalingLayerID.value(); }
-    void setPageScalingLayerID(WebCore::GraphicsLayer::PlatformLayerID layerID) { m_pageScalingLayerID = layerID; }
+    WebCore::PlatformLayerIdentifier pageScalingLayerID() const { return m_pageScalingLayerID.value(); }
+    void setPageScalingLayerID(WebCore::PlatformLayerIdentifier layerID) { m_pageScalingLayerID = layerID; }
+
+    WebCore::PlatformLayerIdentifier scrolledContentsLayerID() const { return m_scrolledContentsLayerID.value(); }
+    void setScrolledContentsLayerID(WebCore::PlatformLayerIdentifier layerID) { m_scrolledContentsLayerID = layerID; }
 #endif
 
     uint64_t renderTreeSize() const { return m_renderTreeSize; }
@@ -309,7 +344,7 @@ public:
     ActivityStateChangeID activityStateChangeID() const { return m_activityStateChangeID; }
     void setActivityStateChangeID(ActivityStateChangeID activityStateChangeID) { m_activityStateChangeID = activityStateChangeID; }
 
-    typedef CallbackID TransactionCallbackID;
+    using TransactionCallbackID = IPC::AsyncReplyID;
     const Vector<TransactionCallbackID>& callbackIDs() const { return m_callbackIDs; }
     void setCallbackIDs(Vector<TransactionCallbackID>&& callbackIDs) { m_callbackIDs = WTFMove(callbackIDs); }
 
@@ -326,14 +361,16 @@ public:
 #endif
 
 private:
-    WebCore::GraphicsLayer::PlatformLayerID m_rootLayerID;
+    WebCore::PlatformLayerIdentifier m_rootLayerID;
     HashSet<RefPtr<PlatformCALayerRemote>> m_changedLayers; // Only used in the Web process.
     LayerPropertiesMap m_changedLayerProperties; // Only used in the UI process.
 
+    Markable<WebCore::LayerHostingContextIdentifier> m_remoteContextHostedIdentifier;
+
     Vector<LayerCreationProperties> m_createdLayers;
-    Vector<WebCore::GraphicsLayer::PlatformLayerID> m_destroyedLayerIDs;
-    Vector<WebCore::GraphicsLayer::PlatformLayerID> m_videoLayerIDsPendingFullscreen;
-    Vector<WebCore::GraphicsLayer::PlatformLayerID> m_layerIDsWithNewlyUnreachableBackingStore;
+    Vector<WebCore::PlatformLayerIdentifier> m_destroyedLayerIDs;
+    Vector<WebCore::PlatformLayerIdentifier> m_videoLayerIDsPendingFullscreen;
+    Vector<WebCore::PlatformLayerIdentifier> m_layerIDsWithNewlyUnreachableBackingStore;
 
     Vector<TransactionCallbackID> m_callbackIDs;
 
@@ -348,7 +385,8 @@ private:
     WebCore::Color m_sampledPageTopColor;
 
 #if PLATFORM(MAC)
-    Markable<WebCore::GraphicsLayer::PlatformLayerID> m_pageScalingLayerID; // Only used for non-delegated scaling.
+    Markable<WebCore::PlatformLayerIdentifier> m_pageScalingLayerID; // Only used for non-delegated scaling.
+    Markable<WebCore::PlatformLayerIdentifier> m_scrolledContentsLayerID;
 #endif
 
     double m_pageScaleFactor { 1 };
