@@ -26,7 +26,7 @@
 
 #pragma once
 
-// FIXME: We should probably move to makeing the WebCore/PlatformFooEvents trivial classes so that
+// FIXME: We should probably move to making the WebCore/PlatformFooEvents trivial classes so that
 // we can use them as the event type.
 
 #include "WebEvent.h"
@@ -34,6 +34,8 @@
 
 #include <wtf/EnumTraits.h>
 #include <wtf/OptionSet.h>
+#include <wtf/RefCounted.h>
+#include <wtf/UUID.h>
 #include <wtf/WallTime.h>
 #include <wtf/text/WTFString.h>
 
@@ -82,8 +84,9 @@ enum class WebEventType : int8_t {
 class WebEvent {
     WTF_MAKE_FAST_ALLOCATED;
 public:
+    WebEvent(WebEventType, OptionSet<WebEventModifier>, WallTime timestamp, UUID authorizationToken);
     WebEvent(WebEventType, OptionSet<WebEventModifier>, WallTime timestamp);
-    
+
     WebEventType type() const { return m_type; }
 
     bool shiftKey() const { return m_modifiers.contains(WebEventModifier::ShiftKey); }
@@ -96,16 +99,16 @@ public:
 
     WallTime timestamp() const { return m_timestamp; }
 
+    UUID authorizationToken() const { return m_authorizationToken; }
+
 protected:
     WebEvent();
 
-    void encode(IPC::Encoder&) const;
-    static WARN_UNUSED_RETURN bool decode(IPC::Decoder&, WebEvent&);
-
 private:
     WebEventType m_type;
     OptionSet<WebEventModifier> m_modifiers;
     WallTime m_timestamp;
+    UUID m_authorizationToken;
 };
 
 WTF::TextStream& operator<<(WTF::TextStream&, WebEventType);
