@@ -86,13 +86,14 @@ struct ImageCopyExternalImage;
 struct ImageCopyTexture;
 struct ImageCopyTextureTagged;
 struct ImageDataLayout;
+class InternalError;
 struct MultisampleState;
 struct ObjectDescriptorBase;
 class OutOfMemoryError;
 struct PipelineDescriptorBase;
 class PipelineLayout;
 struct PipelineLayoutDescriptor;
-struct PresentationConfiguration;
+struct CanvasConfiguration;
 class PresentationContext;
 struct PresentationContextDescriptor;
 struct PrimitiveState;
@@ -163,12 +164,13 @@ struct ImageCopyExternalImage;
 struct ImageCopyTexture;
 struct ImageCopyTextureTagged;
 struct ImageDataLayout;
+struct InternalError;
 struct MultisampleState;
 struct ObjectDescriptorBase;
 struct OutOfMemoryError;
 struct PipelineDescriptorBase;
 struct PipelineLayoutDescriptor;
-struct PresentationConfiguration;
+struct CanvasConfiguration;
 struct PresentationContextDescriptor;
 struct PrimitiveState;
 struct ProgrammableStage;
@@ -235,6 +237,7 @@ public:
     std::optional<PAL::WebGPU::ImageCopyTexture> convertFromBacking(const ImageCopyTexture&);
     std::optional<PAL::WebGPU::ImageCopyTextureTagged> convertFromBacking(const ImageCopyTextureTagged&);
     std::optional<PAL::WebGPU::ImageDataLayout> convertFromBacking(const ImageDataLayout&);
+    RefPtr<PAL::WebGPU::InternalError> convertFromBacking(const InternalError&);
     std::optional<PAL::WebGPU::MultisampleState> convertFromBacking(const MultisampleState&);
     std::optional<PAL::WebGPU::ObjectDescriptorBase> convertFromBacking(const ObjectDescriptorBase&);
     std::optional<PAL::WebGPU::Origin2D> convertFromBacking(const Origin2D&);
@@ -244,7 +247,6 @@ public:
     RefPtr<PAL::WebGPU::OutOfMemoryError> convertFromBacking(const OutOfMemoryError&);
     std::optional<PAL::WebGPU::PipelineDescriptorBase> convertFromBacking(const PipelineDescriptorBase&);
     std::optional<PAL::WebGPU::PipelineLayoutDescriptor> convertFromBacking(const PipelineLayoutDescriptor&);
-    std::optional<PAL::WebGPU::PresentationConfiguration> convertFromBacking(const PresentationConfiguration&);
     std::optional<PAL::WebGPU::PresentationContextDescriptor> convertFromBacking(const PresentationContextDescriptor&);
     std::optional<PAL::WebGPU::PrimitiveState> convertFromBacking(const PrimitiveState&);
     std::optional<PAL::WebGPU::ProgrammableStage> convertFromBacking(const ProgrammableStage&);
