@@ -85,13 +85,14 @@ struct ImageCopyExternalImage;
 struct ImageCopyTexture;
 struct ImageCopyTextureTagged;
 struct ImageDataLayout;
+class InternalError;
 struct MultisampleState;
 struct ObjectDescriptorBase;
 class OutOfMemoryError;
 struct PipelineDescriptorBase;
 class PipelineLayout;
 struct PipelineLayoutDescriptor;
-struct PresentationConfiguration;
+struct CanvasConfiguration;
 class PresentationContext;
 struct PresentationContextDescriptor;
 struct PrimitiveState;
@@ -162,16 +163,18 @@ struct ImageCopyExternalImage;
 struct ImageCopyTexture;
 struct ImageCopyTextureTagged;
 struct ImageDataLayout;
+struct InternalError;
 struct MultisampleState;
 struct ObjectDescriptorBase;
 struct OutOfMemoryError;
 struct PipelineDescriptorBase;
 struct PipelineLayoutDescriptor;
 struct PresentationContextDescriptor;
-struct PresentationConfiguration;
+struct CanvasConfiguration;
 struct PrimitiveState;
 struct ProgrammableStage;
 struct QuerySetDescriptor;
+class RemoteCompositorIntegrationProxy;
 struct RenderBundleDescriptor;
 struct RenderBundleEncoderDescriptor;
 struct RenderPassColorAttachment;
@@ -234,6 +237,7 @@ public:
     std::optional<ImageCopyTexture> convertToBacking(const PAL::WebGPU::ImageCopyTexture&);
     std::optional<ImageCopyTextureTagged> convertToBacking(const PAL::WebGPU::ImageCopyTextureTagged&);
     std::optional<ImageDataLayout> convertToBacking(const PAL::WebGPU::ImageDataLayout&);
+    std::optional<InternalError> convertToBacking(const PAL::WebGPU::InternalError&);
     std::optional<MultisampleState> convertToBacking(const PAL::WebGPU::MultisampleState&);
     std::optional<ObjectDescriptorBase> convertToBacking(const PAL::WebGPU::ObjectDescriptorBase&);
     std::optional<Origin2D> convertToBacking(const PAL::WebGPU::Origin2D&);
@@ -243,7 +247,6 @@ public:
     std::optional<OutOfMemoryError> convertToBacking(const PAL::WebGPU::OutOfMemoryError&);
     std::optional<PipelineDescriptorBase> convertToBacking(const PAL::WebGPU::PipelineDescriptorBase&);
     std::optional<PipelineLayoutDescriptor> convertToBacking(const PAL::WebGPU::PipelineLayoutDescriptor&);
-    std::optional<PresentationConfiguration> convertToBacking(const PAL::WebGPU::PresentationConfiguration&);
     std::optional<PresentationContextDescriptor> convertToBacking(const PAL::WebGPU::PresentationContextDescriptor&);
     std::optional<PrimitiveState> convertToBacking(const PAL::WebGPU::PrimitiveState&);
     std::optional<ProgrammableStage> convertToBacking(const PAL::WebGPU::ProgrammableStage&);
@@ -280,6 +283,7 @@ public:
     virtual WebGPUIdentifier convertToBacking(const PAL::WebGPU::Buffer&) = 0;
     virtual WebGPUIdentifier convertToBacking(const PAL::WebGPU::CommandBuffer&) = 0;
     virtual WebGPUIdentifier convertToBacking(const PAL::WebGPU::CommandEncoder&) = 0;
+    virtual const RemoteCompositorIntegrationProxy& convertToRawBacking(const PAL::WebGPU::CompositorIntegration&) = 0;
     virtual WebGPUIdentifier convertToBacking(const PAL::WebGPU::CompositorIntegration&) = 0;
     virtual WebGPUIdentifier convertToBacking(const PAL::WebGPU::ComputePassEncoder&) = 0;
     virtual WebGPUIdentifier convertToBacking(const PAL::WebGPU::ComputePipeline&) = 0;
