@@ -88,7 +88,8 @@ void WebPageCreationParameters::encode(IPC::Encoder& encoder) const
 #if PLATFORM(MAC)
     encoder << colorSpace;
     encoder << useSystemAppearance;
-    encoder << useFormSemanticContext;
+    encoder << headerBannerHeight;
+    encoder << footerBannerHeight;
 #endif
 
 #if ENABLE(META_VIEWPORT)
@@ -109,6 +110,7 @@ void WebPageCreationParameters::encode(IPC::Encoder& encoder) const
     encoder << keyboardIsAttached;
     encoder << canShowWhileLocked;
     encoder << isCapturingScreen;
+    encoder << insertionPointColor;
 #endif
 #if PLATFORM(COCOA)
     encoder << smartInsertDeleteEnabled;
@@ -182,7 +184,7 @@ void WebPageCreationParameters::encode(IPC::Encoder& encoder) const
     encoder << canUseCredentialStorage;
 
     encoder << httpsUpgradeEnabled;
-#if PLATFORM(IOS)
+#if PLATFORM(IOS) || PLATFORM(VISION)
     encoder << allowsDeprecatedSynchronousXMLHttpRequestDuringUnload;
 #endif
 
@@ -199,11 +201,11 @@ void WebPageCreationParameters::encode(IPC::Encoder& encoder) const
 #endif
 
     encoder << contentSecurityPolicyModeForExtension;
-    encoder << mainFrameIdentifier;
+    encoder << subframeProcessFrameTreeCreationParameters;
 
-#if ENABLE(NETWORK_CONNECTION_INTEGRITY)
-    encoder << lookalikeCharacterStrings;
-    encoder << allowedLookalikeCharacterStrings;
+#if ENABLE(ADVANCED_PRIVACY_PROTECTIONS)
+    encoder << linkDecorationFilteringData;
+    encoder << allowedQueryParametersForAdvancedPrivacyProtections;
 #endif
 
 #if HAVE(MACH_BOOTSTRAP_EXTENSION)
@@ -364,6 +366,10 @@ std::optional<WebPageCreationParameters> WebPageCreationParameters::decode(IPC::
         return std::nullopt;
     if (!decoder.decode(parameters.useFormSemanticContext))
         return std::nullopt;
+    if (!decoder.decode(parameters.headerBannerHeight))
+        return std::nullopt;
+    if (!decoder.decode(parameters.footerBannerHeight))
+        return std::nullopt;
 #endif
 
 #if ENABLE(META_VIEWPORT)
@@ -401,6 +407,8 @@ std::optional<WebPageCreationParameters> WebPageCreationParameters::decode(IPC::
         return std::nullopt;
     if (!decoder.decode(parameters.isCapturingScreen))
         return std::nullopt;
+    if (!decoder.decode(parameters.insertionPointColor))
+        return std::nullopt;
 #endif
 
 #if PLATFORM(COCOA)
@@ -618,7 +626,7 @@ std::optional<WebPageCreationParameters> WebPageCreationParameters::decode(IPC::
     if (!decoder.decode(parameters.httpsUpgradeEnabled))
         return std::nullopt;
 
-#if PLATFORM(IOS)
+#if PLATFORM(IOS) || PLATFORM(VISION)
     if (!decoder.decode(parameters.allowsDeprecatedSynchronousXMLHttpRequestDuringUnload))
         return std::nullopt;
 #endif
@@ -641,21 +649,21 @@ std::optional<WebPageCreationParameters> WebPageCreationParameters::decode(IPC::
     if (!decoder.decode(parameters.contentSecurityPolicyModeForExtension))
         return std::nullopt;
 
-    if (!decoder.decode(parameters.mainFrameIdentifier))
+    if (!decoder.decode(parameters.subframeProcessFrameTreeCreationParameters))
         return std::nullopt;
 
-#if ENABLE(NETWORK_CONNECTION_INTEGRITY)
-    std::optional<Vector<String>> lookalikeCharacterStrings;
-    decoder >> lookalikeCharacterStrings;
-    if (!lookalikeCharacterStrings)
+#if ENABLE(ADVANCED_PRIVACY_PROTECTIONS)
+    std::optional<Vector<WebCore::LinkDecorationFilteringData>> linkDecorationFilteringData;
+    decoder >> linkDecorationFilteringData;
+    if (!linkDecorationFilteringData)
         return std::nullopt;
-    parameters.lookalikeCharacterStrings = WTFMove(*lookalikeCharacterStrings);
+    parameters.linkDecorationFilteringData = WTFMove(*linkDecorationFilteringData);
 
-    std::optional<Vector<WebCore::LookalikeCharactersSanitizationData>> allowedLookalikeCharacterStrings;
-    decoder >> allowedLookalikeCharacterStrings;
-    if (!allowedLookalikeCharacterStrings)
+    std::optional<Vector<WebCore::LinkDecorationFilteringData>> allowedQueryParametersForAdvancedPrivacyProtections;
+    decoder >> allowedQueryParametersForAdvancedPrivacyProtections;
+    if (!allowedQueryParametersForAdvancedPrivacyProtections)
         return std::nullopt;
-    parameters.allowedLookalikeCharacterStrings = WTFMove(*allowedLookalikeCharacterStrings);
+    parameters.allowedQueryParametersForAdvancedPrivacyProtections = WTFMove(*allowedQueryParametersForAdvancedPrivacyProtections);
 #endif
 
 #if HAVE(MACH_BOOTSTRAP_EXTENSION)
