@@ -26,6 +26,7 @@
 #pragma once
 
 #include "DrawingAreaInfo.h"
+#include "FrameTreeCreationParameters.h"
 #include "LayerTreeContext.h"
 #include "SandboxExtension.h"
 #include "SessionState.h"
@@ -42,7 +43,9 @@
 #include <WebCore/FloatSize.h>
 #include <WebCore/FrameIdentifier.h>
 #include <WebCore/HighlightVisibility.h>
+#include <WebCore/IntDegrees.h>
 #include <WebCore/IntSize.h>
+#include <WebCore/LayerHostingContextIdentifier.h>
 #include <WebCore/LayoutMilestone.h>
 #include <WebCore/MediaProducer.h>
 #include <WebCore/PageIdentifier.h>
@@ -58,8 +61,8 @@
 #include <WebCore/ApplicationManifest.h>
 #endif
 
-#if ENABLE(NETWORK_CONNECTION_INTEGRITY)
-#include <WebCore/LookalikeCharactersSanitizationData.h>
+#if ENABLE(ADVANCED_PRIVACY_PROTECTIONS)
+#include <WebCore/LinkDecorationFilteringData.h>
 #endif
 
 #if ENABLE(WK_WEB_EXTENSIONS)
@@ -79,7 +82,7 @@ struct WebPageCreationParameters {
 
     WebCore::IntSize viewSize;
 
-    OptionSet<WebCore::ActivityState::Flag> activityState;
+    OptionSet<WebCore::ActivityState> activityState;
     
     WebPreferencesStore store;
     DrawingAreaType drawingAreaType;
@@ -162,6 +165,8 @@ struct WebPageCreationParameters {
     std::optional<WebCore::DestinationColorSpace> colorSpace;
     bool useSystemAppearance { false };
     bool useFormSemanticContext { false };
+    int headerBannerHeight { 0 };
+    int footerBannerHeight { 0 };
 #endif
 #if ENABLE(META_VIEWPORT)
     bool ignoresViewportScaleLimits;
@@ -176,10 +181,11 @@ struct WebPageCreationParameters {
     WebCore::FloatSize availableScreenSize;
     WebCore::FloatSize overrideScreenSize;
     float textAutosizingWidth;
-    int32_t deviceOrientation { 0 };
+    WebCore::IntDegrees deviceOrientation { 0 };
     bool keyboardIsAttached { false };
     bool canShowWhileLocked { false };
     bool isCapturingScreen { false };
+    WebCore::Color insertionPointColor;
 #endif
 #if PLATFORM(COCOA)
     bool smartInsertDeleteEnabled;
@@ -268,7 +274,7 @@ struct WebPageCreationParameters {
     
     bool httpsUpgradeEnabled { true };
 
-#if PLATFORM(IOS)
+#if PLATFORM(IOS) || PLATFORM(VISION)
     bool allowsDeprecatedSynchronousXMLHttpRequestDuringUnload { false };
 #endif
     
@@ -284,11 +290,11 @@ struct WebPageCreationParameters {
 
     WebCore::ContentSecurityPolicyModeForExtension contentSecurityPolicyModeForExtension { WebCore::ContentSecurityPolicyModeForExtension::None };
 
-    std::optional<WebCore::FrameIdentifier> mainFrameIdentifier;
+    std::optional<FrameTreeCreationParameters> subframeProcessFrameTreeCreationParameters;
 
-#if ENABLE(NETWORK_CONNECTION_INTEGRITY)
-    Vector<String> lookalikeCharacterStrings;
-    Vector<WebCore::LookalikeCharactersSanitizationData> allowedLookalikeCharacterStrings;
+#if ENABLE(ADVANCED_PRIVACY_PROTECTIONS)
+    Vector<WebCore::LinkDecorationFilteringData> linkDecorationFilteringData;
+    Vector<WebCore::LinkDecorationFilteringData> allowedQueryParametersForAdvancedPrivacyProtections;
 #endif
 
 #if HAVE(MACH_BOOTSTRAP_EXTENSION)
