@@ -30,6 +30,7 @@
 #include "DrawingAreaProxyMessages.h"
 #include "WebPageProxy.h"
 #include "WebProcessProxy.h"
+#include <WebCore/ScrollView.h>
 
 #if PLATFORM(COCOA)
 #include <wtf/MachSendRight.h>
@@ -38,21 +39,32 @@
 namespace WebKit {
 using namespace WebCore;
 
-DrawingAreaProxy::DrawingAreaProxy(DrawingAreaType type, WebPageProxy& webPageProxy, WebProcessProxy& process)
+DrawingAreaProxy::DrawingAreaProxy(DrawingAreaType type, WebPageProxy& webPageProxy)
     : m_type(type)
+    , m_identifier(DrawingAreaIdentifier::generate())
     , m_webPageProxy(webPageProxy)
-    , m_process(makeRef(process))
     , m_size(webPageProxy.viewSize())
 #if PLATFORM(MAC)
     , m_viewExposedRectChangedTimer(RunLoop::main(), this, &DrawingAreaProxy::viewExposedRectChangedTimerFired)
 #endif
 {
-    process.addMessageReceiver(Messages::DrawingAreaProxy::messageReceiverName(), m_webPageProxy.pageID(), *this);
 }
 
-DrawingAreaProxy::~DrawingAreaProxy()
+DrawingAreaProxy::~DrawingAreaProxy() = default;
+
+void DrawingAreaProxy::startReceivingMessages(WebProcessProxy& process)
 {
-    process().removeMessageReceiver(Messages::DrawingAreaProxy::messageReceiverName(), m_webPageProxy.pageID());
+    process.addMessageReceiver(Messages::DrawingAreaProxy::messageReceiverName(), m_identifier, *this);
+}
+
+void DrawingAreaProxy::stopReceivingMessages(WebProcessProxy& process)
+{
+    process.removeMessageReceiver(Messages::DrawingAreaProxy::messageReceiverName(), m_identifier);
+}
+
+DelegatedScrollingMode DrawingAreaProxy::delegatedScrollingMode() const
+{
+    return DelegatedScrollingMode::NotDelegated;
 }
 
 bool DrawingAreaProxy::setSize(const IntSize& size, const IntSize& scrollDelta)
@@ -74,27 +86,26 @@ MachSendRight DrawingAreaProxy::createFence()
 #endif
 
 #if PLATFORM(MAC)
-void DrawingAreaProxy::setViewExposedRect(Optional<WebCore::FloatRect> viewExposedRect)
+void DrawingAreaProxy::didChangeViewExposedRect()
 {
-    if (!m_webPageProxy.isValid())
+    if (!m_webPageProxy.hasRunningProcess())
         return;
 
-    m_viewExposedRect = viewExposedRect;
-
     if (!m_viewExposedRectChangedTimer.isActive())
         m_viewExposedRectChangedTimer.startOneShot(0_s);
 }
 
 void DrawingAreaProxy::viewExposedRectChangedTimerFired()
 {
-    if (!m_webPageProxy.isValid())
+    if (!m_webPageProxy.hasRunningProcess())
         return;
 
-    if (m_viewExposedRect == m_lastSentViewExposedRect)
+    auto viewExposedRect = m_webPageProxy.viewExposedRect();
+    if (viewExposedRect == m_lastSentViewExposedRect)
         return;
 
-    process().send(Messages::DrawingArea::SetViewExposedRect(m_viewExposedRect), m_webPageProxy.pageID());
-    m_lastSentViewExposedRect = m_viewExposedRect;
+    m_webPageProxy.send(Messages::DrawingArea::SetViewExposedRect(viewExposedRect), m_identifier);
+    m_lastSentViewExposedRect = viewExposedRect;
 }
 #endif // PLATFORM(MAC)
 
