@@ -36,9 +36,13 @@
 #include <wtf/cocoa/RuntimeApplicationChecksCocoa.h>
 #endif
 
+#define PROCESSTHROTTLER_RELEASE_LOG(msg, ...) RELEASE_LOG(ProcessSuspension, "%p - [PID=%d] ProcessThrottler::" msg, this, m_processIdentifier, ##__VA_ARGS__)
+#define PROCESSTHROTTLER_RELEASE_LOG_WITH_PID(msg, ...) RELEASE_LOG(ProcessSuspension, "%p - [PID=%d] ProcessThrottler::" msg, this, ##__VA_ARGS__)
+
 namespace WebKit {
     
 static const Seconds processSuspensionTimeout { 20_s };
+static const Seconds removeAllAssertionsTimeout { 8_min };
 
 static uint64_t generatePrepareToSuspendRequestID()
 {
@@ -49,6 +53,9 @@ static uint64_t generatePrepareToSuspendRequestID()
 ProcessThrottler::ProcessThrottler(ProcessThrottlerClient& process, bool shouldTakeUIBackgroundAssertion)
     : m_process(process)
     , m_prepareToSuspendTimeoutTimer(RunLoop::main(), this, &ProcessThrottler::prepareToSuspendTimeoutTimerFired)
+    , m_dropNearSuspendedAssertionTimer(RunLoop::main(), this, &ProcessThrottler::dropNearSuspendedAssertionTimerFired)
+    , m_prepareToDropLastAssertionTimeoutTimer(RunLoop::main(), this, &ProcessThrottler::prepareToDropLastAssertionTimeoutTimerFired)
+    , m_pageAllowedToRunInTheBackgroundCounter([this](RefCounterEvent) { numberOfPagesAllowedToRunInTheBackgroundChanged(); })
     , m_shouldTakeUIBackgroundAssertion(shouldTakeUIBackgroundAssertion)
 {
 }
@@ -58,45 +65,30 @@ ProcessThrottler::~ProcessThrottler()
     invalidateAllActivities();
 }
 
-bool ProcessThrottler::addActivity(ForegroundActivity& activity)
+bool ProcessThrottler::addActivity(Activity& activity)
 {
     ASSERT(isMainRunLoop());
     if (!m_allowsActivities) {
         if (!activity.isQuietActivity())
-            PROCESSTHROTTLER_RELEASE_LOG("addActivity: not allowed to add foreground activity %s", activity.name().characters());
+            PROCESSTHROTTLER_RELEASE_LOG("addActivity: not allowed to add %s activity %s", activity.isForeground() ? "foreground" : "background", activity.name().characters());
         return false;
     }
 
-    m_foregroundActivities.add(&activity);
+    if (activity.isForeground())
+        m_foregroundActivities.add(&activity);
+    else
+        m_backgroundActivities.add(&activity);
     updateThrottleStateIfNeeded();
     return true;
 }
 
-bool ProcessThrottler::addActivity(BackgroundActivity& activity)
+void ProcessThrottler::removeActivity(Activity& activity)
 {
     ASSERT(isMainRunLoop());
-    if (!m_allowsActivities) {
-        if (!activity.isQuietActivity())
-            PROCESSTHROTTLER_RELEASE_LOG("addActivity: not allowed to add background activity %s", activity.name().characters());
-        return false;
-    }
-
-    m_backgroundActivities.add(&activity);
-    updateThrottleStateIfNeeded();
-    return true;
-}
-
-void ProcessThrottler::removeActivity(ForegroundActivity& activity)
-{
-    ASSERT(isMainRunLoop());
-    m_foregroundActivities.remove(&activity);
-    updateThrottleStateIfNeeded();
-}
-
-void ProcessThrottler::removeActivity(BackgroundActivity& activity)
-{
-    ASSERT(isMainRunLoop());
-    m_backgroundActivities.remove(&activity);
+    if (activity.isForeground())
+        m_foregroundActivities.remove(&activity);
+    else
+        m_backgroundActivities.remove(&activity);
     updateThrottleStateIfNeeded();
 }
 
@@ -110,7 +102,16 @@ void ProcessThrottler::invalidateAllActivities()
         (*m_backgroundActivities.begin())->invalidate();
     PROCESSTHROTTLER_RELEASE_LOG("invalidateAllActivities: END");
 }
-    
+
+void ProcessThrottler::invalidateAllActivitiesAndDropAssertion()
+{
+    PROCESSTHROTTLER_RELEASE_LOG("invalidateAllActivitiesAndDropAssertion:");
+    invalidateAllActivities();
+    clearPendingRequestToSuspend();
+    m_dropNearSuspendedAssertionTimer.stop();
+    clearAssertion();
+}
+
 ProcessThrottleState ProcessThrottler::expectedThrottleState()
 {
     if (!m_foregroundActivities.isEmpty())
@@ -119,7 +120,7 @@ ProcessThrottleState ProcessThrottler::expectedThrottleState()
         return ProcessThrottleState::Background;
     return ProcessThrottleState::Suspended;
 }
-    
+
 void ProcessThrottler::updateThrottleStateNow()
 {
     setThrottleState(expectedThrottleState());
@@ -133,11 +134,12 @@ String ProcessThrottler::assertionName(ProcessAssertionType type) const
             return "Foreground"_s;
         case ProcessAssertionType::Background:
             return "Background"_s;
-        case ProcessAssertionType::Suspended:
-            return "Suspended"_s;
+        case ProcessAssertionType::NearSuspended:
+            return "NearSuspended"_s;
         case ProcessAssertionType::UnboundedNetworking:
         case ProcessAssertionType::MediaPlayback:
         case ProcessAssertionType::FinishTaskInterruptable:
+        case ProcessAssertionType::BoostedJetsam:
             ASSERT_NOT_REACHED(); // These other assertion types are not used by the ProcessThrottler.
             break;
         }
@@ -147,7 +149,7 @@ String ProcessThrottler::assertionName(ProcessAssertionType type) const
     return makeString(m_process.clientName(), " ", typeString, " Assertion");
 }
 
-std::optional<ProcessAssertionType> ProcessThrottler::assertionTypeForState(ProcessThrottleState state)
+ProcessAssertionType ProcessThrottler::assertionTypeForState(ProcessThrottleState state)
 {
     switch (state) {
     case ProcessThrottleState::Foreground:
@@ -155,7 +157,7 @@ std::optional<ProcessAssertionType> ProcessThrottler::assertionTypeForState(Proc
     case ProcessThrottleState::Background:
         return ProcessAssertionType::Background;
     case ProcessThrottleState::Suspended:
-        return m_shouldTakeSuspendedAssertion ? std::optional(ProcessAssertionType::Suspended) : std::nullopt;
+        return ProcessAssertionType::NearSuspended;
     }
 
     RELEASE_ASSERT_NOT_REACHED();
@@ -164,15 +166,8 @@ std::optional<ProcessAssertionType> ProcessThrottler::assertionTypeForState(Proc
 void ProcessThrottler::setThrottleState(ProcessThrottleState newState)
 {
     m_state = newState;
-    m_process.didChangeThrottleState(newState);
 
-    ProcessAssertionType newType;
-    if (auto assertionType = assertionTypeForState(newState))
-        newType = assertionType.value();
-    else {
-        m_assertion = nullptr;
-        return;
-    }
+    ProcessAssertionType newType = assertionTypeForState(newState);
 
     if (m_assertion && m_assertion->isValid() && m_assertion->type() == newType)
         return;
@@ -182,21 +177,31 @@ void ProcessThrottler::setThrottleState(ProcessThrottleState newState)
     // Keep the previous assertion active until the new assertion is taken asynchronously.
     auto previousAssertion = std::exchange(m_assertion, nullptr);
     if (m_shouldTakeUIBackgroundAssertion) {
-        auto assertion = ProcessAndUIAssertion::create(m_processIdentifier, assertionName(newType), newType, ProcessAssertion::Mode::Async, [previousAssertion = WTFMove(previousAssertion)] { });
+        auto assertion = ProcessAndUIAssertion::create(m_processIdentifier, assertionName(newType), newType, ProcessAssertion::Mode::Async, m_process.environmentIdentifier(), [previousAssertion = WTFMove(previousAssertion)] { });
         assertion->setUIAssertionExpirationHandler([weakThis = WeakPtr { *this }] {
             if (weakThis)
                 weakThis->uiAssertionWillExpireImminently();
         });
         m_assertion = WTFMove(assertion);
     } else
-        m_assertion = ProcessAssertion::create(m_processIdentifier, assertionName(newType), newType, ProcessAssertion::Mode::Async, [previousAssertion = WTFMove(previousAssertion)] { });
+        m_assertion = ProcessAssertion::create(m_processIdentifier, assertionName(newType), newType, ProcessAssertion::Mode::Async, m_process.environmentIdentifier(), [previousAssertion = WTFMove(previousAssertion)] { });
 
     m_assertion->setInvalidationHandler([weakThis = WeakPtr { *this }] {
         if (weakThis)
             weakThis->assertionWasInvalidated();
     });
+
+    if (isHoldingNearSuspendedAssertion()) {
+        if (!m_shouldTakeNearSuspendedAssertion)
+            clearAssertion();
+        else if (m_shouldDropNearSuspendedAssertionAfterDelay)
+            m_dropNearSuspendedAssertionTimer.startOneShot(removeAllAssertionsTimeout);
+    } else
+        m_dropNearSuspendedAssertionTimer.stop();
+
+    m_process.didChangeThrottleState(newState);
 }
-    
+
 void ProcessThrottler::updateThrottleStateIfNeeded()
 {
     if (!m_processIdentifier)
@@ -232,7 +237,17 @@ void ProcessThrottler::didConnectToProcess(ProcessID pid)
 
     m_processIdentifier = pid;
     updateThrottleStateNow();
-    RELEASE_ASSERT(m_assertion || (m_state == ProcessThrottleState::Suspended && !m_shouldTakeSuspendedAssertion));
+    RELEASE_ASSERT(m_assertion || (m_state == ProcessThrottleState::Suspended && !m_shouldTakeNearSuspendedAssertion));
+}
+
+void ProcessThrottler::didDisconnectFromProcess()
+{
+    PROCESSTHROTTLER_RELEASE_LOG_WITH_PID("didDisconnectFromProcess:", m_processIdentifier);
+
+    m_dropNearSuspendedAssertionTimer.stop();
+    clearPendingRequestToSuspend();
+    m_processIdentifier = 0;
+    m_assertion = nullptr;
 }
     
 void ProcessThrottler::prepareToSuspendTimeoutTimerFired()
@@ -241,7 +256,18 @@ void ProcessThrottler::prepareToSuspendTimeoutTimerFired()
     RELEASE_ASSERT(m_pendingRequestToSuspendID);
     updateThrottleStateNow();
 }
-    
+
+void ProcessThrottler::dropNearSuspendedAssertionTimerFired()
+{
+    PROCESSTHROTTLER_RELEASE_LOG("dropNearSuspendedAssertionTimerFired: Removing near-suspended process assertion");
+    RELEASE_ASSERT(isHoldingNearSuspendedAssertion());
+    ASSERT(m_shouldDropNearSuspendedAssertionAfterDelay);
+    if (m_pageAllowedToRunInTheBackgroundCounter.value())
+        PROCESSTHROTTLER_RELEASE_LOG("dropNearSuspendedAssertionTimerFired: Not releasing near-suspended assertion because a page is allowed to run in the background");
+    else
+        clearAssertion();
+}
+
 void ProcessThrottler::processReadyToSuspend()
 {
     PROCESSTHROTTLER_RELEASE_LOG("processReadyToSuspend: Updating process assertion to allow suspension");
@@ -292,18 +318,20 @@ void ProcessThrottler::assertionWasInvalidated()
     invalidateAllActivities();
 }
 
-bool ProcessThrottler::isValidBackgroundActivity(const ProcessThrottler::ActivityVariant& activity)
+bool ProcessThrottler::isValidBackgroundActivity(const ActivityVariant& variant)
 {
-    if (!std::holds_alternative<UniqueRef<ProcessThrottler::BackgroundActivity>>(activity))
+    if (!std::holds_alternative<UniqueRef<Activity>>(variant))
         return false;
-    return std::get<UniqueRef<ProcessThrottler::BackgroundActivity>>(activity)->isValid();
+    auto& activity = std::get<UniqueRef<Activity>>(variant).get();
+    return activity.isValid() && !activity.isForeground();
 }
 
-bool ProcessThrottler::isValidForegroundActivity(const ProcessThrottler::ActivityVariant& activity)
+bool ProcessThrottler::isValidForegroundActivity(const ActivityVariant& variant)
 {
-    if (!std::holds_alternative<UniqueRef<ProcessThrottler::ForegroundActivity>>(activity))
+    if (!std::holds_alternative<UniqueRef<Activity>>(variant))
         return false;
-    return std::get<UniqueRef<ProcessThrottler::ForegroundActivity>>(activity)->isValid();
+    auto& activity = std::get<UniqueRef<Activity>>(variant).get();
+    return activity.isValid() && activity.isForeground();
 }
 
 void ProcessThrottler::setAllowsActivities(bool allow)
@@ -318,36 +346,103 @@ void ProcessThrottler::setAllowsActivities(bool allow)
         invalidateAllActivities();
 }
 
-void ProcessThrottler::setShouldTakeSuspendedAssertion(bool shouldTakeSuspendedAssertion)
+void ProcessThrottler::setShouldTakeNearSuspendedAssertion(bool shouldTakeNearSuspendedAssertion)
 {
-    const bool shouldUpdateAssertion = m_shouldTakeSuspendedAssertion != shouldTakeSuspendedAssertion;
-    m_shouldTakeSuspendedAssertion = shouldTakeSuspendedAssertion;
-    if (shouldUpdateAssertion && m_state == ProcessThrottleState::Suspended)
-        setThrottleState(ProcessThrottleState::Suspended);
+    m_shouldTakeNearSuspendedAssertion = shouldTakeNearSuspendedAssertion;
+    if (shouldTakeNearSuspendedAssertion) {
+        if (!m_assertion && m_processIdentifier) {
+            PROCESSTHROTTLER_RELEASE_LOG_WITH_PID("setShouldTakeNearSuspendedAssertion: Taking near-suspended assertion", m_processIdentifier);
+            setThrottleState(ProcessThrottleState::Suspended);
+        }
+    } else {
+        if (isHoldingNearSuspendedAssertion()) {
+            PROCESSTHROTTLER_RELEASE_LOG_WITH_PID("setShouldTakeNearSuspendedAssertion: Releasing near-suspended assertion", m_processIdentifier);
+            m_dropNearSuspendedAssertionTimer.stop();
+            clearAssertion();
+        }
+    }
 }
 
-ProcessThrottler::TimedActivity::TimedActivity(Seconds timeout, ProcessThrottler::ActivityVariant&& activity)
-    : m_timer(RunLoop::main(), this, &TimedActivity::activityTimedOut)
+void ProcessThrottler::setShouldDropNearSuspendedAssertionAfterDelay(bool shouldDropAfterDelay)
+{
+    if (m_shouldDropNearSuspendedAssertionAfterDelay == shouldDropAfterDelay)
+        return;
+
+    m_shouldDropNearSuspendedAssertionAfterDelay = shouldDropAfterDelay;
+    if (shouldDropAfterDelay) {
+        if (isHoldingNearSuspendedAssertion())
+            m_dropNearSuspendedAssertionTimer.startOneShot(removeAllAssertionsTimeout);
+    } else
+        m_dropNearSuspendedAssertionTimer.stop();
+}
+
+void ProcessThrottler::prepareToDropLastAssertionTimeoutTimerFired()
+{
+    PROCESSTHROTTLER_RELEASE_LOG("prepareToDropLastAssertionTimeoutTimerFired:");
+    m_assertionToClearAfterPrepareToDropLastAssertion = nullptr;
+}
+
+void ProcessThrottler::clearAssertion()
+{
+    if (!m_assertion)
+        return;
+
+    PROCESSTHROTTLER_RELEASE_LOG("clearAssertion:");
+
+    if (!m_prepareToDropLastAssertionTimeoutTimer.isActive())
+        m_prepareToDropLastAssertionTimeoutTimer.startOneShot(10_s);
+
+    m_assertionToClearAfterPrepareToDropLastAssertion = std::exchange(m_assertion, nullptr);
+    m_process.prepareToDropLastAssertion([this, weakThis = WeakPtr { *this }] {
+        if (!weakThis)
+            return;
+        PROCESSTHROTTLER_RELEASE_LOG("clearAssertion: Releasing near-suspended assertion");
+        m_prepareToDropLastAssertionTimeoutTimer.stop();
+        m_assertionToClearAfterPrepareToDropLastAssertion = nullptr;
+    });
+}
+
+PageAllowedToRunInTheBackgroundCounter::Token ProcessThrottler::pageAllowedToRunInTheBackgroundToken()
+{
+    return m_pageAllowedToRunInTheBackgroundCounter.count();
+}
+
+void ProcessThrottler::numberOfPagesAllowedToRunInTheBackgroundChanged()
+{
+    if (m_pageAllowedToRunInTheBackgroundCounter.value())
+        return;
+
+    if (m_dropNearSuspendedAssertionTimer.isActive())
+        return;
+
+    if (isHoldingNearSuspendedAssertion()) {
+        PROCESSTHROTTLER_RELEASE_LOG("numberOfPagesAllowedToRunInTheBackgroundChanged: Releasing near-suspended assertion");
+        clearAssertion();
+    }
+}
+
+ProcessThrottlerTimedActivity::ProcessThrottlerTimedActivity(Seconds timeout, ProcessThrottler::ActivityVariant&& activity)
+    : m_timer(RunLoop::main(), this, &ProcessThrottlerTimedActivity::activityTimedOut)
     , m_timeout(timeout)
     , m_activity(WTFMove(activity))
 {
     updateTimer();
 }
 
-auto ProcessThrottler::TimedActivity::operator=(ProcessThrottler::ActivityVariant&& activity) -> TimedActivity&
+auto ProcessThrottlerTimedActivity::operator=(ProcessThrottler::ActivityVariant&& activity) -> ProcessThrottlerTimedActivity&
 {
     m_activity = WTFMove(activity);
     updateTimer();
     return *this;
 }
 
-void ProcessThrottler::TimedActivity::activityTimedOut()
+void ProcessThrottlerTimedActivity::activityTimedOut()
 {
-    RELEASE_LOG_ERROR(ProcessSuspension, "%p - TimedActivity::activityTimedOut:", this);
+    RELEASE_LOG_ERROR(ProcessSuspension, "%p - ProcessThrottlerTimedActivity::activityTimedOut:", this);
     m_activity = nullptr;
 }
 
-void ProcessThrottler::TimedActivity::updateTimer()
+void ProcessThrottlerTimedActivity::updateTimer()
 {
     if (std::holds_alternative<std::nullptr_t>(m_activity))
         m_timer.stop();
