@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2019 Apple Inc. All rights reserved.
+ * Copyright (C) 2019-2021 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -30,6 +30,7 @@
 
 #include "CtapDriver.h"
 #include <WebCore/ApduResponse.h>
+#include <WebCore/AuthenticatorAttachment.h>
 #include <WebCore/ExceptionData.h>
 #include <WebCore/U2fCommandConstructor.h>
 #include <WebCore/U2fResponseConverter.h>
@@ -52,7 +53,7 @@ U2fAuthenticator::U2fAuthenticator(std::unique_ptr<CtapDriver>&& driver)
 
 void U2fAuthenticator::makeCredential()
 {
-    auto& creationOptions = WTF::get<PublicKeyCredentialCreationOptions>(requestData().options);
+    auto& creationOptions = std::get<PublicKeyCredentialCreationOptions>(requestData().options);
     if (!isConvertibleToU2fRegisterCommand(creationOptions)) {
         receiveRespond(ExceptionData { NotSupportedError, "Cannot convert the request to U2F command."_s });
         return;
@@ -67,7 +68,7 @@ void U2fAuthenticator::makeCredential()
 
 void U2fAuthenticator::checkExcludeList(size_t index)
 {
-    auto& creationOptions = WTF::get<PublicKeyCredentialCreationOptions>(requestData().options);
+    auto& creationOptions = std::get<PublicKeyCredentialCreationOptions>(requestData().options);
     if (index >= creationOptions.excludeCredentials.size()) {
         issueRegisterCommand();
         return;
@@ -79,14 +80,14 @@ void U2fAuthenticator::checkExcludeList(size_t index)
 
 void U2fAuthenticator::issueRegisterCommand()
 {
-    auto u2fCmd = convertToU2fRegisterCommand(requestData().hash, WTF::get<PublicKeyCredentialCreationOptions>(requestData().options));
+    auto u2fCmd = convertToU2fRegisterCommand(requestData().hash, std::get<PublicKeyCredentialCreationOptions>(requestData().options));
     ASSERT(u2fCmd);
     issueNewCommand(WTFMove(*u2fCmd), CommandType::RegisterCommand);
 }
 
 void U2fAuthenticator::getAssertion()
 {
-    if (!isConvertibleToU2fSignCommand(WTF::get<PublicKeyCredentialRequestOptions>(requestData().options))) {
+    if (!isConvertibleToU2fSignCommand(std::get<PublicKeyCredentialRequestOptions>(requestData().options))) {
         receiveRespond(ExceptionData { NotSupportedError, "Cannot convert the request to U2F command."_s });
         return;
     }
@@ -96,14 +97,12 @@ void U2fAuthenticator::getAssertion()
 
 void U2fAuthenticator::issueSignCommand(size_t index)
 {
-    auto& requestOptions = WTF::get<PublicKeyCredentialRequestOptions>(requestData().options);
+    auto& requestOptions = std::get<PublicKeyCredentialRequestOptions>(requestData().options);
     if (index >= requestOptions.allowCredentials.size()) {
-        if (auto* observer = this->observer())
-            observer->authenticatorStatusUpdated(WebAuthenticationStatus::NoCredentialsFound);
-        receiveRespond(ExceptionData { NotAllowedError, "No credentials from the allowCredentials list is found in the authenticator."_s });
+        issueNewCommand(constructBogusU2fRegistrationCommand(), CommandType::BogusCommandNoCredentials);
         return;
     }
-    auto u2fCmd = convertToU2fSignCommand(requestData().hash, requestOptions, requestOptions.allowCredentials[index].idVector, m_isAppId);
+    auto u2fCmd = convertToU2fSignCommand(requestData().hash, requestOptions, requestOptions.allowCredentials[index].id, m_isAppId);
     ASSERT(u2fCmd);
     issueNewCommand(WTFMove(*u2fCmd), CommandType::SignCommand);
 }
@@ -117,7 +116,7 @@ void U2fAuthenticator::issueNewCommand(Vector<uint8_t>&& command, CommandType ty
 
 void U2fAuthenticator::issueCommand(const Vector<uint8_t>& command, CommandType type)
 {
-    driver().transact(Vector<uint8_t>(command), [weakThis = makeWeakPtr(*this), type](Vector<uint8_t>&& data) {
+    driver().transact(Vector<uint8_t>(command), [weakThis = WeakPtr { *this }, type](Vector<uint8_t>&& data) {
         ASSERT(RunLoop::isMain());
         if (!weakThis)
             return;
@@ -140,8 +139,11 @@ void U2fAuthenticator::responseReceived(Vector<uint8_t>&& response, CommandType
     case CommandType::CheckOnlyCommand:
         continueCheckOnlyCommandAfterResponseReceived(WTFMove(*apduResponse));
         return;
-    case CommandType::BogusCommand:
-        continueBogusCommandAfterResponseReceived(WTFMove(*apduResponse));
+    case CommandType::BogusCommandExcludeCredentialsMatch:
+        continueBogusCommandExcludeCredentialsMatchAfterResponseReceived(WTFMove(*apduResponse));
+        return;
+    case CommandType::BogusCommandNoCredentials:
+        continueBogusCommandNoCredentialsAfterResponseReceived(WTFMove(*apduResponse));
         return;
     case CommandType::SignCommand:
         continueSignCommandAfterResponseReceived(WTFMove(*apduResponse));
@@ -154,14 +156,14 @@ void U2fAuthenticator::continueRegisterCommandAfterResponseReceived(ApduResponse
 {
     switch (apduResponse.status()) {
     case ApduResponse::Status::SW_NO_ERROR: {
-        auto& options = WTF::get<PublicKeyCredentialCreationOptions>(requestData().options);
-        auto appId = processGoogleLegacyAppIdSupportExtension(options.extensions);
-        auto response = readU2fRegisterResponse(!appId ? options.rp.id : appId, apduResponse.data(), options.attestation);
+        auto& options = std::get<PublicKeyCredentialCreationOptions>(requestData().options);
+        ASSERT(options.rp.id);
+        auto response = readU2fRegisterResponse(*options.rp.id, apduResponse.data(), AuthenticatorAttachment::CrossPlatform, { driver().transport() }, options.attestation);
         if (!response) {
             receiveRespond(ExceptionData { UnknownError, "Couldn't parse the U2F register response."_s });
             return;
         }
-        receiveRespond(WTFMove(*response));
+        receiveRespond(response.releaseNonNull());
         return;
     }
     case ApduResponse::Status::SW_CONDITIONS_NOT_SATISFIED:
@@ -178,14 +180,14 @@ void U2fAuthenticator::continueCheckOnlyCommandAfterResponseReceived(ApduRespons
     switch (apduResponse.status()) {
     case ApduResponse::Status::SW_NO_ERROR:
     case ApduResponse::Status::SW_CONDITIONS_NOT_SATISFIED:
-        issueNewCommand(constructBogusU2fRegistrationCommand(), CommandType::BogusCommand);
+        issueNewCommand(constructBogusU2fRegistrationCommand(), CommandType::BogusCommandExcludeCredentialsMatch);
         return;
     default:
         checkExcludeList(m_nextListIndex++);
     }
 }
 
-void U2fAuthenticator::continueBogusCommandAfterResponseReceived(ApduResponse&& apduResponse)
+void U2fAuthenticator::continueBogusCommandExcludeCredentialsMatchAfterResponseReceived(ApduResponse&& apduResponse)
 {
     switch (apduResponse.status()) {
     case ApduResponse::Status::SW_NO_ERROR:
@@ -200,25 +202,42 @@ void U2fAuthenticator::continueBogusCommandAfterResponseReceived(ApduResponse&&
     }
 }
 
+void U2fAuthenticator::continueBogusCommandNoCredentialsAfterResponseReceived(ApduResponse&& apduResponse)
+{
+    switch (apduResponse.status()) {
+    case ApduResponse::Status::SW_NO_ERROR:
+        if (auto* observer = this->observer())
+            observer->authenticatorStatusUpdated(WebAuthenticationStatus::NoCredentialsFound);
+        receiveRespond(ExceptionData { NotAllowedError, "No credentials from the allowCredentials list is found in the authenticator."_s });
+        return;
+    case ApduResponse::Status::SW_CONDITIONS_NOT_SATISFIED:
+        // Polling is required during test of user presence.
+        m_retryTimer.startOneShot(Seconds::fromMilliseconds(retryTimeOutValueMs));
+        return;
+    default:
+        receiveRespond(ExceptionData { UnknownError, makeString("Unknown internal error. Error code: ", static_cast<unsigned>(apduResponse.status())) });
+    }
+}
+
 void U2fAuthenticator::continueSignCommandAfterResponseReceived(ApduResponse&& apduResponse)
 {
-    auto& requestOptions = WTF::get<PublicKeyCredentialRequestOptions>(requestData().options);
+    auto& requestOptions = std::get<PublicKeyCredentialRequestOptions>(requestData().options);
     switch (apduResponse.status()) {
     case ApduResponse::Status::SW_NO_ERROR: {
-        Optional<PublicKeyCredentialData> response;
+        RefPtr<AuthenticatorAssertionResponse> response;
         if (m_isAppId) {
             ASSERT(requestOptions.extensions && !requestOptions.extensions->appid.isNull());
-            response = readU2fSignResponse(requestOptions.extensions->appid, requestOptions.allowCredentials[m_nextListIndex - 1].idVector, apduResponse.data());
+            response = readU2fSignResponse(requestOptions.extensions->appid, requestOptions.allowCredentials[m_nextListIndex - 1].id, apduResponse.data(), AuthenticatorAttachment::CrossPlatform);
         } else
-            response = readU2fSignResponse(requestOptions.rpId, requestOptions.allowCredentials[m_nextListIndex - 1].idVector, apduResponse.data());
+            response = readU2fSignResponse(requestOptions.rpId, requestOptions.allowCredentials[m_nextListIndex - 1].id, apduResponse.data(), AuthenticatorAttachment::CrossPlatform);
         if (!response) {
             receiveRespond(ExceptionData { UnknownError, "Couldn't parse the U2F sign response."_s });
             return;
         }
         if (m_isAppId)
-            response->appid = m_isAppId;
+            response->setExtensions({ m_isAppId, std::nullopt, std::nullopt });
 
-        receiveRespond(WTFMove(*response));
+        receiveRespond(response.releaseNonNull());
         return;
     }
     case ApduResponse::Status::SW_CONDITIONS_NOT_SATISFIED:
