@@ -54,15 +54,18 @@
 
 #if PLATFORM(GTK)
 #include "GtkSettingsManager.h"
+#if USE(GBM)
+#include "AcceleratedBackingStoreDMABuf.h"
 #endif
+#endif
+
 
 namespace WebKit {
 
 void WebProcessPool::platformInitialize()
 {
-#if PLATFORM(GTK)
     m_alwaysUsesComplexTextCodePath = true;
-#endif
+
     if (const char* forceComplexText = getenv("WEBKIT_FORCE_COMPLEX_TEXT"))
         m_alwaysUsesComplexTextCodePath = !strcmp(forceComplexText, "1");
 
@@ -83,8 +86,16 @@ void WebProcessPool::platformInitializeWebProcess(const WebProcessProxy& process
     }
 #endif
 
+#if USE(GBM)
+    parameters.renderDeviceFile = WebCore::PlatformDisplay::sharedDisplay().drmRenderNodeFile();
+#endif
+
+#if PLATFORM(GTK) && USE(GBM)
+    parameters.useDMABufSurfaceForCompositing = AcceleratedBackingStoreDMABuf::checkRequirements();
+#endif
+
 #if PLATFORM(WAYLAND)
-    if (WebCore::PlatformDisplay::sharedDisplay().type() == WebCore::PlatformDisplay::Type::Wayland) {
+    if (WebCore::PlatformDisplay::sharedDisplay().type() == WebCore::PlatformDisplay::Type::Wayland && !parameters.useDMABufSurfaceForCompositing) {
         wpe_loader_init("libWPEBackend-fdo-1.0.so.1");
         if (AcceleratedBackingStoreWayland::checkRequirements()) {
             parameters.hostClientFileDescriptor = UnixFileDescriptor { wpe_renderer_host_create_client(), UnixFileDescriptor::Adopt };
@@ -110,6 +121,8 @@ void WebProcessPool::platformInitializeWebProcess(const WebProcessProxy& process
 
     parameters.memoryPressureHandlerConfiguration = m_configuration->memoryPressureHandlerConfiguration();
 
+    parameters.disableFontHintingForTesting = m_configuration->disableFontHintingForTesting();
+
     GApplication* app = g_application_get_default();
     if (app)
         parameters.applicationID = String::fromLatin1(g_application_get_application_id(app));
