@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2020 Apple Inc. All rights reserved.
+ * Copyright (C) 2020-2023 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -30,13 +30,16 @@
 
 #include "GPUConnectionToWebProcess.h"
 #include "GPUProcessConnection.h"
+#include "RemoteGraphicsContextGLInitializationState.h"
 #include "RemoteGraphicsContextGLMessages.h"
 #include "RemoteGraphicsContextGLProxyMessages.h"
+#include "RemoteRenderingBackendProxy.h"
 #include "RemoteVideoFrameObjectHeapProxy.h"
 #include "WebProcess.h"
 #include <WebCore/BitmapImage.h>
 #include <WebCore/GCGLSpan.h>
 #include <WebCore/ImageBuffer.h>
+#include <WebCore/PixelBufferConversion.h>
 #include <wtf/StdLibExtras.h>
 
 #if ENABLE(VIDEO)
@@ -49,8 +52,6 @@ namespace WebKit {
 
 using namespace WebCore;
 
-static constexpr size_t readPixelsInlineSizeLimit = 64 * KB;
-
 namespace {
 
 template<typename... Types, typename... SpanTupleTypes, size_t... Indices>
@@ -70,16 +71,47 @@ IPC::ArrayReferenceTuple<Types...> toArrayReferenceTuple(const GCGLSpanTuple<Spa
 
 }
 
-RemoteGraphicsContextGLProxy::RemoteGraphicsContextGLProxy(IPC::Connection& connection, SerialFunctionDispatcher& dispatcher, const GraphicsContextGLAttributes& attributes, RenderingBackendIdentifier renderingBackend, Ref<RemoteVideoFrameObjectHeapProxy>&& videoFrameObjectHeapProxy)
-    : GraphicsContextGL(attributes)
-    , m_videoFrameObjectHeapProxy(WTFMove(videoFrameObjectHeapProxy))
+RefPtr<RemoteGraphicsContextGLProxy> RemoteGraphicsContextGLProxy::create(IPC::Connection& connection, const WebCore::GraphicsContextGLAttributes& attributes, RemoteRenderingBackendProxy& renderingBackend
+#if ENABLE(VIDEO)
+    , Ref<RemoteVideoFrameObjectHeapProxy>&& videoFrameObjectHeapProxy
+#endif
+    )
 {
-    constexpr unsigned connectionBufferSizeLog2 = 21;
+    constexpr unsigned defaultConnectionBufferSizeLog2 = 21;
+    unsigned connectionBufferSizeLog2 = defaultConnectionBufferSizeLog2;
+    if (attributes.remoteIPCBufferSizeLog2ForTesting)
+        connectionBufferSizeLog2 = attributes.remoteIPCBufferSizeLog2ForTesting;
     auto [clientConnection, serverConnectionHandle] = IPC::StreamClientConnection::create(connectionBufferSizeLog2);
-    m_streamConnection = WTFMove(clientConnection);
-    m_connection = &connection;
-    m_connection->send(Messages::GPUConnectionToWebProcess::CreateGraphicsContextGL(attributes, m_graphicsContextGLIdentifier, renderingBackend, WTFMove(serverConnectionHandle)), 0, IPC::SendOption::DispatchMessageEvenWhenWaitingForSyncReply);
-    m_streamConnection->open(*this, dispatcher);
+    if (!clientConnection)
+        return nullptr;
+    auto instance = platformCreate(connection, clientConnection.releaseNonNull(), attributes
+#if ENABLE(VIDEO)
+        , WTFMove(videoFrameObjectHeapProxy)
+#endif
+    );
+    instance->initializeIPC(WTFMove(serverConnectionHandle), renderingBackend);
+    return instance;
+}
+
+
+RemoteGraphicsContextGLProxy::RemoteGraphicsContextGLProxy(IPC::Connection& connection, RefPtr<IPC::StreamClientConnection> streamConnection, const GraphicsContextGLAttributes& attributes
+#if ENABLE(VIDEO)
+    , Ref<RemoteVideoFrameObjectHeapProxy>&& videoFrameObjectHeapProxy
+#endif
+    )
+    : GraphicsContextGL(attributes)
+    , m_connection(&connection) // NOLINT
+    , m_streamConnection(WTFMove(streamConnection)) // NOLINT
+#if ENABLE(VIDEO)
+    , m_videoFrameObjectHeapProxy(WTFMove(videoFrameObjectHeapProxy))
+#endif
+{
+}
+
+void RemoteGraphicsContextGLProxy::initializeIPC(IPC::StreamServerConnection::Handle&& serverConnectionHandle, RemoteRenderingBackendProxy& renderingBackend)
+{
+    m_connection->send(Messages::GPUConnectionToWebProcess::CreateGraphicsContextGL(contextAttributes(), m_graphicsContextGLIdentifier, renderingBackend.ensureBackendCreated(), WTFMove(serverConnectionHandle)), 0, IPC::SendOption::DispatchMessageEvenWhenWaitingForSyncReply);
+    m_streamConnection->open(*this, renderingBackend.dispatcher());
     // TODO: We must wait until initialized, because at the moment we cannot receive IPC messages
     // during wait while in synchronous stream send. Should be fixed as part of https://bugs.webkit.org/show_bug.cgi?id=217211.
     waitUntilInitialized();
@@ -97,11 +129,7 @@ void RemoteGraphicsContextGLProxy::setContextVisibility(bool)
 
 bool RemoteGraphicsContextGLProxy::isGLES2Compliant() const
 {
-#if ENABLE(WEBGL2)
     return contextAttributes().webGLVersion == GraphicsContextGLWebGLVersion::WebGL2;
-#else
-    return false;
-#endif
 }
 
 void RemoteGraphicsContextGLProxy::markContextChanged()
@@ -142,12 +170,34 @@ bool RemoteGraphicsContextGLProxy::isExtensionEnabled(const String& name)
     return m_availableExtensions.contains(name) || m_enabledExtensions.contains(name);
 }
 
-void RemoteGraphicsContextGLProxy::initialize(const String& availableExtensions, const String& requestableExtensions)
+void RemoteGraphicsContextGLProxy::initialize(const RemoteGraphicsContextGLInitializationState& initializationState)
 {
-    for (auto extension : StringView(availableExtensions).split(' '))
+    for (auto extension : StringView(initializationState.availableExtensions).split(' '))
         m_availableExtensions.add(extension.toString());
-    for (auto extension : StringView(requestableExtensions).split(' '))
+    for (auto extension : StringView(initializationState.requestableExtensions).split(' '))
         m_requestableExtensions.add(extension.toString());
+    m_externalImageTarget = initializationState.externalImageTarget;
+    m_externalImageBindingQuery = initializationState.externalImageBindingQuery;
+}
+
+std::optional<GraphicsContextGL::ExternalImageAttachResult> RemoteGraphicsContextGLProxy::createAndBindExternalImage(GCGLenum, GraphicsContextGL::ExternalImageSource)
+{
+    notImplemented();
+    return { };
+}
+
+GCEGLSync RemoteGraphicsContextGLProxy::createEGLSync(ExternalEGLSyncEvent)
+{
+    notImplemented();
+    return { };
+}
+
+std::tuple<GCGLenum, GCGLenum> RemoteGraphicsContextGLProxy::externalImageTextureBindingPoint()
+{
+    if (isContextLost())
+        return std::make_tuple(0, 0);
+
+    return std::make_tuple(m_externalImageTarget, m_externalImageBindingQuery);
 }
 
 void RemoteGraphicsContextGLProxy::reshape(int width, int height)
@@ -195,7 +245,7 @@ void RemoteGraphicsContextGLProxy::paintCompositedResultsToCanvas(ImageBuffer& b
     }
 }
 
-#if ENABLE(MEDIA_STREAM)
+#if ENABLE(MEDIA_STREAM) || ENABLE(WEB_CODECS)
 RefPtr<WebCore::VideoFrame> RemoteGraphicsContextGLProxy::paintCompositedResultsToVideoFrame()
 {
     ASSERT(isMainRunLoop());
@@ -233,8 +283,8 @@ bool RemoteGraphicsContextGLProxy::copyTextureFromVideoFrame(WebCore::VideoFrame
         auto sendResult = send(Messages::RemoteGraphicsContextGL::SetSharedVideoFrameSemaphore { semaphore });
         if (!sendResult)
             markContextLost();
-    }, [this](auto& handle) {
-        auto sendResult = send(Messages::RemoteGraphicsContextGL::SetSharedVideoFrameMemory { handle });
+    }, [this](auto&& handle) {
+        auto sendResult = send(Messages::RemoteGraphicsContextGL::SetSharedVideoFrameMemory { WTFMove(handle) });
         if (!sendResult)
             markContextLost();
     });
@@ -269,26 +319,16 @@ RefPtr<Image> RemoteGraphicsContextGLProxy::videoFrameToImage(WebCore::VideoFram
 }
 #endif
 
-void RemoteGraphicsContextGLProxy::synthesizeGLError(GCGLenum error)
+GCGLErrorCodeSet RemoteGraphicsContextGLProxy::getErrors()
 {
     if (!isContextLost()) {
-        auto sendResult = send(Messages::RemoteGraphicsContextGL::SynthesizeGLError(error));
+        auto sendResult = sendSync(Messages::RemoteGraphicsContextGL::GetErrors());
         if (!sendResult)
             markContextLost();
-        return;
+        auto [returnValue] = sendResult.takeReplyOr(GCGLErrorCodeSet { });
+        return returnValue;
     }
-}
-
-GCGLenum RemoteGraphicsContextGLProxy::getError()
-{
-    if (!isContextLost()) {
-        auto sendResult = sendSync(Messages::RemoteGraphicsContextGL::GetError());
-        if (!sendResult)
-            markContextLost();
-        auto [returnValue] = sendResult.takeReplyOr(0);
-        return static_cast<GCGLenum>(returnValue);
-    }
-    return NO_ERROR;
+    return { };
 }
 
 void RemoteGraphicsContextGLProxy::simulateEventForTesting(SimulatedEventForTesting event)
@@ -300,59 +340,80 @@ void RemoteGraphicsContextGLProxy::simulateEventForTesting(SimulatedEventForTest
     }
 }
 
-void RemoteGraphicsContextGLProxy::readnPixels(GCGLint x, GCGLint y, GCGLsizei width, GCGLsizei height, GCGLenum format, GCGLenum type, GCGLSpan<GCGLvoid> data)
+void RemoteGraphicsContextGLProxy::readPixels(IntRect rect, GCGLenum format, GCGLenum type, std::span<uint8_t> dataStore, GCGLint alignment, GCGLint rowLength)
 {
-    if (data.size() > readPixelsInlineSizeLimit) {
-        readnPixelsSharedMemory(x, y, width, height, format, type, data);
+    if (isContextLost())
         return;
+    // The structure of `dataStore` is defined by rect, format, type, alignment, rowLength.
+    // We know that `rect.position() < { 0, 0 }` is out of bounds, and thus should not be touched.
+    // ANGLE will return which of the pixels > 0 are out of bounds, and thus should not be touched.
+    // Alignment, rowLength - width should not be touched.
+    // Each `dataStore` row is [oob1*][image][oob2*][remaining rowLength*][alignment*] == dataStoreRowBytes.
+    // GPUP reply row is [image][oob2*].
+    // The calculations here are all non-overflowing, as the caller has validated that.
+
+    unsigned bytesPerGroup = computeBytesPerGroup(format, type);
+    unsigned dataStoreWidth = rowLength > 0 ? rowLength : rect.width();
+    unsigned dataStoreRowBytes = roundUpToMultipleOf(alignment, dataStoreWidth * bytesPerGroup);
+
+    IntSize bottomLeftOutOfBounds;
+    if (rect.x() < 0) {
+        bottomLeftOutOfBounds.setWidth(-rect.x());
+        rect.shiftXEdgeTo(0);
+    }
+    if (rect.y() < 0) {
+        bottomLeftOutOfBounds.setHeight(-rect.y());
+        rect.shiftYEdgeTo(0);
     }
 
-    if (!isContextLost()) {
-        auto sendResult = sendSync(Messages::RemoteGraphicsContextGL::ReadnPixels0(x, y, width, height, format, type, IPC::ArrayReference<uint8_t>(reinterpret_cast<uint8_t*>(data.data()), data.size())));
-        if (sendResult) {
-            auto [dataReply] = sendResult.takeReply();
-            memcpy(data.data(), dataReply.data(), data.size());
-        } else
-            markContextLost();
-    }
-}
+    unsigned replyRowBytes = rect.width() * bytesPerGroup;
+    unsigned replyImageBytes = rect.height() * replyRowBytes;
 
-void RemoteGraphicsContextGLProxy::readnPixels(GCGLint x, GCGLint y, GCGLsizei width, GCGLsizei height, GCGLenum format, GCGLenum type, GCGLintptr offset)
-{
-    if (!isContextLost()) {
-        auto sendResult = send(Messages::RemoteGraphicsContextGL::ReadnPixels1(x, y, width, height, format, type, static_cast<uint64_t>(offset)));
-        if (!sendResult)
-            markContextLost();
+    if (!rect.isEmpty() && !bottomLeftOutOfBounds.isZero()) {
+        // Will not overflow, because rect.size() * { dataStoreRowBytes, bytesPerGroup } is validated and it will fit to the uint32_t.
+        // bottomLeftOutOfBounds must be smaller than rect.size() in case adjusted rect is non-empty.
+        unsigned skipRowBytes = bottomLeftOutOfBounds.width() * bytesPerGroup;
+        dataStore = dataStore.subspan(dataStoreRowBytes * bottomLeftOutOfBounds.height() + skipRowBytes);
     }
-}
 
-void RemoteGraphicsContextGLProxy::readnPixelsSharedMemory(GCGLint x, GCGLint y, GCGLsizei width, GCGLsizei height, GCGLenum format, GCGLenum type, GCGLSpan<GCGLvoid> data)
-{
-    if (!isContextLost()) {
-        std::optional<SharedMemory::Handle> handle;
-#if PLATFORM(COCOA)
-        handle = SharedMemory::Handle::create(data.data(), data.size(), SharedMemory::Protection::ReadWrite);
-#else
-        auto buffer = SharedMemory::allocate(data.size());
-        if (buffer) {
-            handle = buffer->createHandle(SharedMemory::Protection::ReadWrite);
-            memcpy(buffer->data(), data.data(), data.size());
-        }
-#endif
-        if (!handle || handle->isNull()) {
+    static constexpr size_t readPixelsInlineSizeLimit = 64 * KB; // NOTE: when changing, change the value in RemoteGraphicsContextGL too.
+
+    auto copyToData = [&](std::span<const uint8_t> replyData, IntSize readArea) {
+        if (readArea.isEmpty())
+            return;
+        unsigned copyRowBytes = readArea.width() * bytesPerGroup;
+        copyRows(replyRowBytes, replyData, dataStoreRowBytes, dataStore, readArea.height(), copyRowBytes);
+    };
+
+    if (replyImageBytes > readPixelsInlineSizeLimit) {
+        RefPtr<SharedMemory> replyBuffer = SharedMemory::allocate(replyImageBytes);
+        if (!replyBuffer)
+            goto inlineCase;
+        auto handle = replyBuffer->createHandle(SharedMemory::Protection::ReadWrite);
+        if (!handle || handle->isNull())
+            goto inlineCase;
+        auto sendResult = sendSync(Messages::RemoteGraphicsContextGL::ReadPixelsSharedMemory(rect, format, type, WTFMove(*handle)));
+        if (!sendResult) {
             markContextLost();
             return;
         }
-        auto sendResult = sendSync(Messages::RemoteGraphicsContextGL::ReadnPixels2(x, y, width, height, format, type, WTFMove(*handle)));
-        if (sendResult) {
-#if !PLATFORM(COCOA)
-            auto [success] = sendResult.takeReply();
-            if (success)
-                memcpy(data.data(), buffer->data(), data.size());
-#endif
-        } else
-            markContextLost();
+        auto [readArea] = sendResult.takeReply();
+        if (!readArea)
+            return;
+        std::span<const uint8_t> replyData { reinterpret_cast<uint8_t*>(replyBuffer->data()), replyBuffer->size() };
+        copyToData(replyData, *readArea);
+        return;
     }
+inlineCase:
+    auto sendResult = sendSync(Messages::RemoteGraphicsContextGL::ReadPixelsInline(rect, format, type));
+    if (!sendResult) {
+        markContextLost();
+        return;
+    }
+    auto [readArea, inlineReply] = sendResult.takeReply();
+    if (!readArea)
+        return;
+    copyToData(inlineReply, *readArea);
 }
 
 void RemoteGraphicsContextGLProxy::multiDrawArraysANGLE(GCGLenum mode, GCGLSpanTuple<const GCGLint, const GCGLsizei> firstsAndCounts)
@@ -409,18 +470,18 @@ void RemoteGraphicsContextGLProxy::multiDrawElementsInstancedBaseVertexBaseInsta
     }
 }
 
-void RemoteGraphicsContextGLProxy::wasCreated(bool didSucceed, IPC::Semaphore&& wakeUpSemaphore, IPC::Semaphore&& clientWaitSemaphore, String&& availableExtensions, String&& requestedExtensions)
+void RemoteGraphicsContextGLProxy::wasCreated(IPC::Semaphore&& wakeUpSemaphore, IPC::Semaphore&& clientWaitSemaphore, std::optional<RemoteGraphicsContextGLInitializationState>&& initializationState)
 {
     if (isContextLost())
         return;
-    if (!didSucceed) {
+    if (!initializationState) {
         markContextLost();
         return;
     }
     ASSERT(!m_didInitialize);
     m_streamConnection->setSemaphores(WTFMove(wakeUpSemaphore), WTFMove(clientWaitSemaphore));
     m_didInitialize = true;
-    initialize(availableExtensions, requestedExtensions);
+    initialize(initializationState.value());
 }
 
 void RemoteGraphicsContextGLProxy::wasLost()
