@@ -40,6 +40,7 @@
 #include <WebCore/CVUtilities.h>
 #include <WebCore/LibWebRTCDav1dDecoder.h>
 #include <WebCore/LibWebRTCMacros.h>
+#include <WebCore/Page.h>
 #include <WebCore/PlatformMediaSessionManager.h>
 #include <WebCore/VP9UtilitiesCocoa.h>
 #include <WebCore/VideoFrameCV.h>
@@ -96,6 +97,11 @@ std::optional<VideoCodecType> LibWebRTCCodecs::videoCodecTypeFromWebCodec(const
     return { };
 }
 
+void LibWebRTCCodecs::setHasVP9ExtensionSupport(bool hasVP9ExtensionSupport)
+{
+    m_hasVP9ExtensionSupport = hasVP9ExtensionSupport;
+}
+
 #endif
 
 static int32_t releaseVideoDecoder(webrtc::WebKitVideoDecoder::Value decoder)
@@ -190,7 +196,11 @@ void LibWebRTCCodecs::initializeIfNeeded()
     static std::once_flag doInitializationOnce;
     std::call_once(doInitializationOnce, [] {
         callOnMainRunLoopAndWait([] {
+#if HAVE(AUDIT_TOKEN)
+            WebProcess::singleton().ensureGPUProcessConnection().auditToken();
+#else
             WebProcess::singleton().ensureGPUProcessConnection();
+#endif
         });
     });
 }
@@ -294,7 +304,7 @@ LibWebRTCCodecs::Decoder* LibWebRTCCodecs::createDecoderInternal(VideoCodecType
 {
     auto decoder = makeUnique<Decoder>();
     auto* result = decoder.get();
-    decoder->identifier = VideoDecoderIdentifier::generateThreadSafe();
+    decoder->identifier = VideoDecoderIdentifier::generate();
     decoder->type = type;
 
     ensureGPUProcessConnectionAndDispatchToThread([this, decoder = WTFMove(decoder), callback = WTFMove(callback)]() mutable {
@@ -505,7 +515,7 @@ LibWebRTCCodecs::Encoder* LibWebRTCCodecs::createEncoderInternal(VideoCodecType
 {
     auto encoder = makeUnique<Encoder>();
     auto* result = encoder.get();
-    encoder->identifier = VideoEncoderIdentifier::generateThreadSafe();
+    encoder->identifier = VideoEncoderIdentifier::generate();
     encoder->type = type;
     encoder->useAnnexB = useAnnexB;
     encoder->isRealtime = isRealtime;
@@ -587,7 +597,7 @@ template<typename Frame> int32_t LibWebRTCCodecs::encodeFrameInternal(Encoder& e
 
     auto buffer = encoder.sharedVideoFrameWriter.writeBuffer(frame,
         [&](auto& semaphore) { encoder.connection->send(Messages::LibWebRTCCodecsProxy::SetSharedVideoFrameSemaphore { encoder.identifier, semaphore }, 0); },
-        [&](auto& handle) { encoder.connection->send(Messages::LibWebRTCCodecsProxy::SetSharedVideoFrameMemory { encoder.identifier, handle }, 0); });
+        [&](auto&& handle) { encoder.connection->send(Messages::LibWebRTCCodecsProxy::SetSharedVideoFrameMemory { encoder.identifier, WTFMove(handle) }, 0); });
     if (!buffer)
         return WEBRTC_VIDEO_CODEC_ERROR;
 
@@ -709,7 +719,7 @@ void LibWebRTCCodecs::completedEncoding(VideoEncoderIdentifier identifier, IPC::
     webrtc::encoderVideoTaskComplete(encoder->encodedImageCallback, toWebRTCCodecType(encoder->type), data.data(), data.size(), info);
 }
 
-void LibWebRTCCodecs::setEncodingDescription(WebKit::VideoEncoderIdentifier identifier, IPC::DataReference&& data)
+void LibWebRTCCodecs::setEncodingConfiguration(WebKit::VideoEncoderIdentifier identifier, IPC::DataReference&& description, std::optional<WebCore::PlatformVideoColorSpace> colorSpace)
 {
     assertIsCurrent(workQueue());
 
@@ -723,8 +733,12 @@ void LibWebRTCCodecs::setEncodingDescription(WebKit::VideoEncoderIdentifier iden
 
     Locker locker { AdoptLock, encoder->encodedImageCallbackLock };
 
-    if (encoder->descriptionCallback)
-        encoder->descriptionCallback({ data.data(), data.size() });
+    if (encoder->descriptionCallback) {
+        std::optional<Vector<uint8_t>> decoderDescriptionData;
+        if (description.size())
+            decoderDescriptionData = Vector<uint8_t> { description.data(), description.size() };
+        encoder->descriptionCallback(WebCore::VideoEncoderActiveConfiguration { { }, { }, { }, { }, { }, WTFMove(decoderDescriptionData), WTFMove(colorSpace) });
+    }
 }
 
 CVPixelBufferPoolRef LibWebRTCCodecs::pixelBufferPool(size_t width, size_t height, OSType type)
