@@ -30,6 +30,8 @@
 #include <WebCore/FindOptions.h>
 #include <WebCore/IntRect.h>
 #include <WebCore/PageOverlay.h>
+#include <WebCore/SimpleRange.h>
+#include <wtf/CompletionHandler.h>
 #include <wtf/Forward.h>
 #include <wtf/Noncopyable.h>
 #include <wtf/Vector.h>
@@ -39,7 +41,7 @@
 #endif
 
 namespace WebCore {
-class Frame;
+class LocalFrame;
 class Range;
 enum class DidWrap : bool;
 }
@@ -47,18 +49,22 @@ enum class DidWrap : bool;
 namespace WebKit {
 
 class CallbackID;
+class PluginView;
 class WebPage;
 
-class FindController : private WebCore::PageOverlay::Client {
+class FindController final : private WebCore::PageOverlay::Client {
     WTF_MAKE_FAST_ALLOCATED;
     WTF_MAKE_NONCOPYABLE(FindController);
 
 public:
+    enum class TriggerImageAnalysis : bool { No, Yes };
+
     explicit FindController(WebPage*);
     virtual ~FindController();
 
-    void findString(const String&, OptionSet<FindOptions>, unsigned maxMatchCount, CompletionHandler<void(bool)>&&);
+    void findString(const String&, OptionSet<FindOptions>, unsigned maxMatchCount, TriggerImageAnalysis, CompletionHandler<void(bool)>&& = { });
     void findStringMatches(const String&, OptionSet<FindOptions>, unsigned maxMatchCount);
+    void findRectsForStringMatches(const String&, OptionSet<WebKit::FindOptions>, unsigned maxMatchCount, CompletionHandler<void(Vector<WebCore::FloatRect>&&)>&&);
     void getImageForFindMatch(uint32_t matchIndex);
     void selectFindMatch(uint32_t matchIndex);
     void indicateFindMatch(uint32_t matchIndex);
@@ -85,7 +91,7 @@ private:
     void drawRect(WebCore::PageOverlay&, WebCore::GraphicsContext&, const WebCore::IntRect& dirtyRect) override;
 
     Vector<WebCore::FloatRect> rectsForTextMatchesInRect(WebCore::IntRect clipRect);
-    bool updateFindIndicator(WebCore::Frame& selectedFrame, bool isShowingOverlay, bool shouldAnimate = true);
+    bool updateFindIndicator(WebCore::LocalFrame& selectedFrame, bool isShowingOverlay, bool shouldAnimate = true);
 
     enum class FindUIOriginator : uint8_t { FindString, FindStringMatches };
     void updateFindUIAfterPageScroll(bool found, const String&, OptionSet<FindOptions>, unsigned maxMatchCount, WebCore::DidWrap, FindUIOriginator);
@@ -99,6 +105,10 @@ private:
     bool shouldHideFindIndicatorOnScroll() const;
     void didScrollAffectingFindIndicatorPosition();
 
+#if ENABLE(PDFKIT_PLUGIN)
+    PluginView* mainFramePlugIn();
+#endif
+
     WebPage* m_webPage;
     WebCore::PageOverlay* m_findPageOverlay { nullptr };
 
@@ -106,7 +116,7 @@ private:
     // the find indicator isn't showing, but it will never be false when it is showing.
     bool m_isShowingFindIndicator { false };
     WebCore::IntRect m_findIndicatorRect;
-    Vector<RefPtr<WebCore::Range>> m_findMatches;
+    Vector<WebCore::SimpleRange> m_findMatches;
     // Index value is -1 if not found or if number of matches exceeds provided maximum.
     int m_foundStringMatchIndex { -1 };
 
@@ -116,6 +126,4 @@ private:
 #endif
 };
 
-WebCore::FindOptions core(OptionSet<FindOptions>);
-
 } // namespace WebKit
