@@ -26,11 +26,16 @@
 #pragma once
 
 #include "RemoteLayerTreeTransaction.h"
+#include <WebCore/HTMLMediaElementIdentifier.h>
 #include <WebCore/PlatformCALayer.h>
 #include <WebCore/PlatformLayer.h>
 
 namespace WebCore {
 class LayerPool;
+#if ENABLE(THREADED_ANIMATION_RESOLUTION)
+class AcceleratedEffect;
+struct AcceleratedEffectValues;
+#endif
 }
 
 namespace WebKit {
@@ -43,6 +48,9 @@ public:
     static Ref<PlatformCALayerRemote> create(PlatformLayer *, WebCore::PlatformCALayerClient*, RemoteLayerTreeContext&);
 #if ENABLE(MODEL_ELEMENT)
     static Ref<PlatformCALayerRemote> create(Ref<WebCore::Model>, WebCore::PlatformCALayerClient*, RemoteLayerTreeContext&);
+#endif
+#if HAVE(AVKIT)
+    static Ref<PlatformCALayerRemote> create(WebCore::HTMLVideoElement&, WebCore::PlatformCALayerClient*, RemoteLayerTreeContext&);
 #endif
     static Ref<PlatformCALayerRemote> create(const PlatformCALayerRemote&, WebCore::PlatformCALayerClient*, RemoteLayerTreeContext&);
 
@@ -74,6 +82,11 @@ public:
     void animationStarted(const String& key, MonotonicTime beginTime) override;
     void animationEnded(const String& key) override;
 
+#if ENABLE(THREADED_ANIMATION_RESOLUTION)
+    void clearAcceleratedEffectsAndBaseValues() override;
+    void setAcceleratedEffectsAndBaseValues(const WebCore::AcceleratedEffects&, WebCore::AcceleratedEffectValues&) override;
+#endif
+
     void setMask(WebCore::PlatformCALayer*) override;
 
     bool isOpaque() const override;
@@ -124,10 +137,7 @@ public:
     bool hasContents() const override;
     CFTypeRef contents() const override;
     void setContents(CFTypeRef) override;
-#if HAVE(IOSURFACE)
-    void setContents(const WebCore::IOSurface&) override;
-    void setContents(const WTF::MachSendRight&) override;
-#endif
+    void setDelegatedContents(const WebCore::PlatformCALayerDelegatedContents&) override;
     void setContentsRect(const WebCore::FloatRect&) override;
 
     void setMinificationFilter(WebCore::PlatformCALayer::FilterType) override;
@@ -164,6 +174,9 @@ public:
 
     void setAntialiasesEdges(bool) override;
 
+    WebCore::MediaPlayerVideoGravity videoGravity() const override;
+    void setVideoGravity(WebCore::MediaPlayerVideoGravity) override;
+
     // FIXME: Having both shapeRoundedRect and shapePath is redundant. We could use shapePath for everything.
     WebCore::FloatRoundedRect shapeRoundedRect() const override;
     void setShapeRoundedRect(const WebCore::FloatRoundedRect&) override;
@@ -229,11 +242,15 @@ protected:
     void updateClonedLayerProperties(PlatformCALayerRemote& clone, bool copyContents = true) const;
 
 private:
-    bool isPlatformCALayerRemote() const override { return true; }
+    Type type() const override { return Type::Remote; }
     void ensureBackingStore();
     void updateBackingStore();
     void removeSublayer(PlatformCALayerRemote*);
 
+#if ENABLE(CG_DISPLAY_LIST_BACKED_IMAGE_BUFFER)
+    RemoteLayerBackingStore::IncludeDisplayList shouldIncludeDisplayListInBackingStore() const;
+#endif
+
     bool requiresCustomAppearanceUpdateOnBoundsChange() const;
 
     WebCore::LayerPool& layerPool() override;
@@ -252,4 +269,18 @@ private:
 
 } // namespace WebKit
 
-SPECIALIZE_TYPE_TRAITS_PLATFORM_CALAYER(WebKit::PlatformCALayerRemote, isPlatformCALayerRemote())
+SPECIALIZE_TYPE_TRAITS_BEGIN(WebKit::PlatformCALayerRemote)
+static bool isType(const WebCore::PlatformCALayer& layer)
+{
+    switch (layer.type()) {
+    case WebCore::PlatformCALayer::Type::Cocoa:
+        break;
+    case WebCore::PlatformCALayer::Type::Remote:
+    case WebCore::PlatformCALayer::Type::RemoteCustom:
+    case WebCore::PlatformCALayer::Type::RemoteHost:
+    case WebCore::PlatformCALayer::Type::RemoteModel:
+        return true;
+    };
+    return false;
+}
+SPECIALIZE_TYPE_TRAITS_END()
