@@ -28,12 +28,15 @@
 
 #include "APIArray.h"
 #include "DownloadManager.h"
+#include "DrawingArea.h"
 #include "FrameInfoData.h"
+#include "FrameTreeNodeData.h"
 #include "InjectedBundleCSSStyleDeclarationHandle.h"
 #include "InjectedBundleHitTestResult.h"
 #include "InjectedBundleNodeHandle.h"
 #include "InjectedBundleRangeHandle.h"
 #include "InjectedBundleScriptWorld.h"
+#include "MessageSenderInlines.h"
 #include "NetworkConnectionToWebProcessMessages.h"
 #include "NetworkProcessConnection.h"
 #include "PluginView.h"
@@ -42,8 +45,6 @@
 #include "WebChromeClient.h"
 #include "WebCoreArgumentCoders.h"
 #include "WebDocumentLoader.h"
-#include "WebFrameMessages.h"
-#include "WebFrameProxyMessages.h"
 #include "WebImage.h"
 #include "WebPage.h"
 #include "WebPageProxyMessages.h"
@@ -60,12 +61,10 @@
 #include <WebCore/Chrome.h>
 #include <WebCore/DocumentLoader.h>
 #include <WebCore/Editor.h>
-#include <WebCore/ElementChildIterator.h>
+#include <WebCore/ElementChildIteratorInlines.h>
 #include <WebCore/EventHandler.h>
 #include <WebCore/File.h>
-#include <WebCore/Frame.h>
 #include <WebCore/FrameSnapshotting.h>
-#include <WebCore/FrameView.h>
 #include <WebCore/HTMLFormElement.h>
 #include <WebCore/HTMLFrameOwnerElement.h>
 #include <WebCore/HTMLInputElement.h>
@@ -77,6 +76,9 @@
 #include <WebCore/JSElement.h>
 #include <WebCore/JSFile.h>
 #include <WebCore/JSRange.h>
+#include <WebCore/LocalFrame.h>
+#include <WebCore/LocalFrameView.h>
+#include <WebCore/OriginAccessPatterns.h>
 #include <WebCore/Page.h>
 #include <WebCore/PluginDocument.h>
 #include <WebCore/RemoteDOMWindow.h>
@@ -112,32 +114,27 @@ static uint64_t generateListenerID()
     return uniqueListenerID++;
 }
 
+// FIXME: Remove receivedMainFrameIdentifierFromUIProcess in favor of a more correct way of sending frame tree deltas to each process.
 void WebFrame::initWithCoreMainFrame(WebPage& page, Frame& coreFrame, bool receivedMainFrameIdentifierFromUIProcess)
 {
-    ASSERT(!m_frameID);
-    m_frameID = coreFrame.frameID();
-    WebProcess::singleton().addMessageReceiver(Messages::WebFrame::messageReceiverName(), m_frameID.object(), *this);
-    WebProcess::singleton().addWebFrame(frameID(), this);
-
     if (!receivedMainFrameIdentifierFromUIProcess)
         page.send(Messages::WebPageProxy::DidCreateMainFrame(frameID()));
 
     m_coreFrame = coreFrame;
     m_coreFrame->tree().setName(nullAtom());
-    coreFrame.init();
+    if (auto* localFrame = dynamicDowncast<LocalFrame>(coreFrame))
+        localFrame->init();
 }
 
 Ref<WebFrame> WebFrame::createSubframe(WebPage& page, WebFrame& parent, const AtomString& frameName, HTMLFrameOwnerElement& ownerElement)
 {
-    auto frame = create(page);
-    auto coreFrame = Frame::create(page.corePage(), &ownerElement, makeUniqueRef<WebFrameLoaderClient>(frame.get()), WebCore::FrameIdentifier::generate());
+    auto frameID = WebCore::FrameIdentifier::generate();
+    auto frame = create(page, frameID);
+    ASSERT(page.corePage());
+    auto coreFrame = LocalFrame::createSubframe(*page.corePage(), makeUniqueRef<WebLocalFrameLoaderClient>(frame.get(), frame->makeInvalidator()), frameID, ownerElement);
     frame->m_coreFrame = coreFrame.get();
 
-    ASSERT(!frame->m_frameID);
-    frame->m_frameID = coreFrame->frameID();
-    WebProcess::singleton().addMessageReceiver(Messages::WebFrame::messageReceiverName(), frame->m_frameID.object(), frame.get());
-    WebProcess::singleton().addWebFrame(coreFrame->frameID(), frame.ptr());
-    parent.send(Messages::WebFrameProxy::DidCreateSubframe(coreFrame->frameID()));
+    page.send(Messages::WebPageProxy::DidCreateSubframe(parent.frameID(), coreFrame->frameID()));
 
     coreFrame->tree().setName(frameName);
     ASSERT(ownerElement.document().frame());
@@ -146,18 +143,34 @@ Ref<WebFrame> WebFrame::createSubframe(WebPage& page, WebFrame& parent, const At
     return frame;
 }
 
-WebFrame::WebFrame(WebPage& page)
+Ref<WebFrame> WebFrame::createRemoteSubframe(WebPage& page, WebFrame& parent, WebCore::FrameIdentifier frameID, WebCore::ProcessIdentifier remoteProcessIdentifier)
+{
+    auto frame = create(page, frameID);
+    auto client = makeUniqueRef<WebRemoteFrameClient>(frame.copyRef(), frame->makeInvalidator());
+    RELEASE_ASSERT(page.corePage());
+    RELEASE_ASSERT(parent.coreFrame());
+    auto coreFrame = RemoteFrame::createSubframe(*page.corePage(), WTFMove(client), frameID, *parent.coreFrame(), remoteProcessIdentifier);
+    frame->m_coreFrame = coreFrame.get();
+
+    // FIXME: Pass in a name and call FrameTree::setName here.
+    return frame;
+}
+
+WebFrame::WebFrame(WebPage& page, WebCore::FrameIdentifier frameID)
     : m_page(page)
+    , m_frameID(frameID)
 {
 #ifndef NDEBUG
     webFrameCounter.increment();
 #endif
+    ASSERT(!WebProcess::singleton().webFrame(m_frameID));
+    WebProcess::singleton().addWebFrame(m_frameID, this);
 }
 
-WebFrameLoaderClient* WebFrame::frameLoaderClient() const
+WebLocalFrameLoaderClient* WebFrame::frameLoaderClient() const
 {
     auto* localFrame = dynamicDowncast<LocalFrame>(m_coreFrame.get());
-    return localFrame ? static_cast<WebFrameLoaderClient*>(&localFrame->loader().client()) : nullptr;
+    return localFrame ? static_cast<WebLocalFrameLoaderClient*>(&localFrame->loader().client()) : nullptr;
 }
 
 WebFrame::~WebFrame()
@@ -168,7 +181,7 @@ WebFrame::~WebFrame()
     for (auto& completionHandler : willSubmitFormCompletionHandlers.values())
         completionHandler();
 
-    WebProcess::singleton().removeMessageReceiver(Messages::WebFrame::messageReceiverName(), m_frameID.object());
+    ASSERT_WITH_MESSAGE(!WebProcess::singleton().webFrame(m_frameID), "invalidate should have removed this WebFrame before destruction");
 
 #ifndef NDEBUG
     webFrameCounter.decrement();
@@ -180,19 +193,17 @@ WebPage* WebFrame::page() const
     if (!m_coreFrame)
         return nullptr;
 
-    if (auto* page = m_coreFrame->page())
-        return &WebPage::fromCorePage(*page);
-
-    return nullptr;
+    auto* page = m_coreFrame->page();
+    return page ? WebPage::fromCorePage(*page) : nullptr;
 }
 
-WebFrame* WebFrame::fromCoreFrame(const AbstractFrame& frame)
+WebFrame* WebFrame::fromCoreFrame(const Frame& frame)
 {
     if (auto* localFrame = dynamicDowncast<LocalFrame>(frame)) {
-        auto* webFrameLoaderClient = toWebFrameLoaderClient(localFrame->loader().client());
-        if (!webFrameLoaderClient)
+        auto* webLocalFrameLoaderClient = toWebLocalFrameLoaderClient(localFrame->loader().client());
+        if (!webLocalFrameLoaderClient)
             return nullptr;
-        return &webFrameLoaderClient->webFrame();
+        return &webLocalFrameLoaderClient->webFrame();
     }
     if (auto* remoteFrame = dynamicDowncast<RemoteFrame>(frame)) {
         auto& client = static_cast<const WebRemoteFrameClient&>(remoteFrame->client());
@@ -201,17 +212,28 @@ WebFrame* WebFrame::fromCoreFrame(const AbstractFrame& frame)
     return nullptr;
 }
 
-WebCore::Frame* WebFrame::coreFrame() const
+WebCore::LocalFrame* WebFrame::coreLocalFrame() const
 {
     return dynamicDowncast<LocalFrame>(m_coreFrame.get());
 }
 
+WebCore::RemoteFrame* WebFrame::coreRemoteFrame() const
+{
+    return dynamicDowncast<RemoteFrame>(m_coreFrame.get());
+}
+
+WebCore::Frame* WebFrame::coreFrame() const
+{
+    return m_coreFrame.get();
+}
+
 FrameInfoData WebFrame::info() const
 {
     auto* parent = parentFrame();
 
     FrameInfoData info {
         isMainFrame(),
+        is<WebCore::LocalFrame>(coreFrame()) ? FrameType::Local : FrameType::Remote,
         // FIXME: This should use the full request.
         ResourceRequest(url()),
         SecurityOriginData::fromFrame(dynamicDowncast<LocalFrame>(m_coreFrame.get())),
@@ -223,6 +245,32 @@ FrameInfoData WebFrame::info() const
     return info;
 }
 
+FrameTreeNodeData WebFrame::frameTreeData() const
+{
+    FrameTreeNodeData data {
+        info(),
+        { }
+    };
+
+    if (!m_coreFrame) {
+        ASSERT_NOT_REACHED();
+        return data;
+    }
+
+    data.children.reserveInitialCapacity(m_coreFrame->tree().childCount());
+
+    for (auto* child = m_coreFrame->tree().firstChild(); child; child = child->tree().nextSibling()) {
+        auto* childWebFrame = WebFrame::fromCoreFrame(*child);
+        if (!childWebFrame) {
+            ASSERT_NOT_REACHED();
+            continue;
+        }
+        data.children.uncheckedAppend(childWebFrame->frameTreeData());
+    }
+
+    return data;
+}
+
 void WebFrame::getFrameInfo(CompletionHandler<void(FrameInfoData&&)>&& completionHandler)
 {
     completionHandler(info());
@@ -236,10 +284,18 @@ WebCore::FrameIdentifier WebFrame::frameID() const
 
 void WebFrame::invalidate()
 {
+    ASSERT(!WebProcess::singleton().webFrame(m_frameID) || WebProcess::singleton().webFrame(m_frameID) == this);
     WebProcess::singleton().removeWebFrame(frameID(), m_page ? std::optional<WebPageProxyIdentifier>(m_page->webPageProxyIdentifier()) : std::nullopt);
     m_coreFrame = 0;
 }
 
+ScopeExit<Function<void()>> WebFrame::makeInvalidator()
+{
+    return makeScopeExit<Function<void()>>([protectedThis = Ref { *this }] {
+        protectedThis->invalidate();
+    });
+}
+
 uint64_t WebFrame::setUpPolicyListener(WebCore::PolicyCheckIdentifier identifier, WebCore::FramePolicyFunction&& policyFunction, ForNavigationAction forNavigationAction)
 {
     auto policyListenerID = generateListenerID();
@@ -266,42 +322,57 @@ void WebFrame::continueWillSubmitForm(FormSubmitListenerIdentifier listenerID)
         completionHandler();
 }
 
-void WebFrame::didCommitLoadInAnotherProcess()
+void WebFrame::didCommitLoadInAnotherProcess(WebCore::LayerHostingContextIdentifier layerHostingContextIdentifier, WebCore::ProcessIdentifier remoteProcessIdentifier)
 {
     RefPtr coreFrame = m_coreFrame.get();
-    if (!coreFrame)
+    if (!coreFrame) {
+        ASSERT_NOT_REACHED();
         return;
+    }
 
     RefPtr webPage = m_page.get();
-    if (!webPage)
+    if (!webPage) {
+        ASSERT_NOT_REACHED();
         return;
+    }
 
     auto* corePage = webPage->corePage();
-    if (!corePage)
+    if (!corePage) {
+        ASSERT_NOT_REACHED();
         return;
+    }
 
     RefPtr parent = coreFrame->tree().parent();
-    if (!parent)
-        return;
 
     auto* localFrame = dynamicDowncast<WebCore::LocalFrame>(coreFrame.get());
-    if (!localFrame)
+    if (!localFrame) {
+        ASSERT_NOT_REACHED();
         return;
+    }
 
     auto* frameLoaderClient = this->frameLoaderClient();
-    if (!frameLoaderClient)
+    if (!frameLoaderClient) {
+        ASSERT_NOT_REACHED();
         return;
+    }
+
+    RefPtr ownerElement = coreFrame->ownerElement();
 
     auto invalidator = frameLoaderClient->takeFrameInvalidator();
-    RefPtr ownerElement = coreFrame->ownerElement();
     auto* ownerRenderer = localFrame->ownerRenderer();
     localFrame->setView(nullptr);
 
-    parent->tree().removeChild(*coreFrame);
-    coreFrame->disconnectOwnerElement();
+    if (parent)
+        parent->tree().removeChild(*coreFrame);
+    if (ownerElement)
+        coreFrame->disconnectOwnerElement();
     auto client = makeUniqueRef<WebRemoteFrameClient>(*this, WTFMove(invalidator));
+    auto newFrame = ownerElement
+        ? WebCore::RemoteFrame::createSubframeWithContentsInAnotherProcess(*corePage, WTFMove(client), m_frameID, *ownerElement, layerHostingContextIdentifier, remoteProcessIdentifier)
+        : parent
+            ? WebCore::RemoteFrame::createSubframe(*corePage, WTFMove(client), m_frameID, *parent, remoteProcessIdentifier)
+            : WebCore::RemoteFrame::createMainFrame(*corePage, WTFMove(client), m_frameID, remoteProcessIdentifier);
 
-    auto newFrame = WebCore::RemoteFrame::create(*corePage, m_frameID, ownerElement.get(), WTFMove(client));
     auto remoteFrameView = WebCore::RemoteFrameView::create(newFrame);
     // FIXME: We need a corresponding setView(nullptr) during teardown to break the ref cycle.
     newFrame->setView(remoteFrameView.ptr());
@@ -309,9 +380,50 @@ void WebFrame::didCommitLoadInAnotherProcess()
         ownerRenderer->setWidget(remoteFrameView.ptr());
 
     m_coreFrame = newFrame.get();
+
+    // FIXME: This is also done in the WebCore::Frame constructor. Move one to make this more symmetric.
     if (ownerElement) {
-        // FIXME: This is also done in the WebCore::Frame constructor. Move one to make this more symmetric.
         ownerElement->setContentFrame(*m_coreFrame);
+        ownerElement->scheduleInvalidateStyleAndLayerComposition();
+    }
+}
+
+void WebFrame::transitionToLocal(std::optional<WebCore::LayerHostingContextIdentifier> layerHostingContextIdentifier)
+{
+    RefPtr remoteFrame = coreRemoteFrame();
+    if (!remoteFrame) {
+        ASSERT_NOT_REACHED();
+        return;
+    }
+
+    auto* corePage = remoteFrame->page();
+    if (!corePage) {
+        ASSERT_NOT_REACHED();
+        return;
+    }
+
+    auto* parent = remoteFrame->tree().parent();
+    if (!parent) {
+        ASSERT_NOT_REACHED();
+        return;
+    }
+
+    parent->tree().removeChild(*remoteFrame);
+    remoteFrame->disconnectOwnerElement();
+    auto invalidator = static_cast<WebRemoteFrameClient&>(remoteFrame->client()).takeFrameInvalidator();
+
+    auto localFrame = LocalFrame::createSubframeHostedInAnotherProcess(*corePage, makeUniqueRef<WebLocalFrameLoaderClient>(*this, WTFMove(invalidator)), m_frameID, *parent);
+    m_coreFrame = localFrame.ptr();
+    localFrame->init();
+
+    if (layerHostingContextIdentifier)
+        setLayerHostingContextIdentifier(*layerHostingContextIdentifier);
+    if (localFrame->isRootFrame())
+        corePage->addRootFrame(localFrame.get());
+
+    if (auto* webPage = page(); webPage && m_coreFrame->isRootFrame()) {
+        if (auto* drawingArea = webPage->drawingArea())
+            drawingArea->addRootFrame(m_coreFrame->frameID());
     }
 }
 
@@ -445,7 +557,7 @@ String WebFrame::contentsAsString() const
 
     if (isFrameSet()) {
         StringBuilder builder;
-        for (AbstractFrame* child = m_coreFrame->tree().firstChild(); child; child = child->tree().nextSibling()) {
+        for (Frame* child = m_coreFrame->tree().firstChild(); child; child = child->tree().nextSibling()) {
             if (!builder.isEmpty())
                 builder.append(' ');
 
@@ -486,7 +598,7 @@ IntSize WebFrame::size() const
     if (!localFrame)
         return IntSize();
 
-    FrameView* frameView = localFrame->view();
+    auto* frameView = localFrame->view();
     if (!frameView)
         return IntSize();
 
@@ -507,11 +619,12 @@ bool WebFrame::isFrameSet() const
 
 bool WebFrame::isMainFrame() const
 {
-    auto* localFrame = dynamicDowncast<LocalFrame>(m_coreFrame.get());
-    if (!localFrame)
-        return false;
+    return m_coreFrame && m_coreFrame->isMainFrame();
+}
 
-    return localFrame->isMainFrame();
+bool WebFrame::isRootFrame() const
+{
+    return m_coreFrame && m_coreFrame->isRootFrame();
 }
 
 String WebFrame::name() const
@@ -585,7 +698,7 @@ Ref<API::Array> WebFrame::childFrames()
     Vector<RefPtr<API::Object>> vector;
     vector.reserveInitialCapacity(size);
 
-    for (AbstractFrame* child = m_coreFrame->tree().firstChild(); child; child = child->tree().nextSibling()) {
+    for (Frame* child = m_coreFrame->tree().firstChild(); child; child = child->tree().nextSibling()) {
         WebFrame* webFrame = WebFrame::fromCoreFrame(*child);
         ASSERT(webFrame);
         if (!webFrame)
@@ -620,7 +733,7 @@ bool WebFrame::allowsFollowingLink(const URL& url) const
     if (!localFrame)
         return true;
 
-    return localFrame->document()->securityOrigin().canDisplay(url);
+    return localFrame->document()->securityOrigin().canDisplay(url, WebCore::OriginAccessPatternsForWebProcess::singleton());
 }
 
 JSGlobalContextRef WebFrame::jsContext()
@@ -691,7 +804,7 @@ IntRect WebFrame::contentBounds() const
     if (!localFrame)
         return IntRect();
     
-    FrameView* view = localFrame->view();
+    auto* view = localFrame->view();
     if (!view)
         return IntRect();
     
@@ -704,7 +817,7 @@ IntRect WebFrame::visibleContentBounds() const
     if (!localFrame)
         return IntRect();
     
-    FrameView* view = localFrame->view();
+    auto* view = localFrame->view();
     if (!view)
         return IntRect();
     
@@ -718,7 +831,7 @@ IntRect WebFrame::visibleContentBoundsExcludingScrollbars() const
     if (!localFrame)
         return IntRect();
     
-    FrameView* view = localFrame->view();
+    LocalFrameView* view = localFrame->view();
     if (!view)
         return IntRect();
     
@@ -732,7 +845,7 @@ IntSize WebFrame::scrollOffset() const
     if (!localFrame)
         return IntSize();
     
-    FrameView* view = localFrame->view();
+    auto* view = localFrame->view();
     if (!view)
         return IntSize();
 
@@ -745,7 +858,7 @@ bool WebFrame::hasHorizontalScrollbar() const
     if (!localFrame)
         return false;
 
-    FrameView* view = localFrame->view();
+    auto* view = localFrame->view();
     if (!view)
         return false;
 
@@ -758,7 +871,7 @@ bool WebFrame::hasVerticalScrollbar() const
     if (!localFrame)
         return false;
 
-    FrameView* view = localFrame->view();
+    auto* view = localFrame->view();
     if (!view)
         return false;
 
@@ -780,7 +893,7 @@ bool WebFrame::getDocumentBackgroundColor(double* red, double* green, double* bl
     if (!localFrame)
         return false;
 
-    FrameView* view = localFrame->view();
+    auto* view = localFrame->view();
     if (!view)
         return false;
 
@@ -834,13 +947,13 @@ void WebFrame::stopLoading()
 
 WebFrame* WebFrame::frameForContext(JSContextRef context)
 {
-    auto* coreFrame = Frame::fromJSContext(context);
+    auto* coreFrame = LocalFrame::fromJSContext(context);
     return coreFrame ? WebFrame::fromCoreFrame(*coreFrame) : nullptr;
 }
 
 WebFrame* WebFrame::contentFrameForWindowOrFrameElement(JSContextRef context, JSValueRef value)
 {
-    auto* coreFrame = Frame::contentFrameFromWindowOrFrameElement(context, value);
+    auto* coreFrame = LocalFrame::contentFrameFromWindowOrFrameElement(context, value);
     return coreFrame ? WebFrame::fromCoreFrame(*coreFrame) : nullptr;
 }
 
@@ -850,7 +963,7 @@ JSValueRef WebFrame::jsWrapperForWorld(InjectedBundleCSSStyleDeclarationHandle*
     if (!localFrame)
         return nullptr;
 
-    JSDOMWindow* globalObject = localFrame->script().globalObject(world->coreWorld());
+    auto* globalObject = localFrame->script().globalObject(world->coreWorld());
 
     JSLockHolder lock(globalObject);
     return toRef(globalObject, toJS(globalObject, globalObject, cssStyleDeclarationHandle->coreCSSStyleDeclaration()));
@@ -862,7 +975,7 @@ JSValueRef WebFrame::jsWrapperForWorld(InjectedBundleNodeHandle* nodeHandle, Inj
     if (!localFrame)
         return nullptr;
 
-    JSDOMWindow* globalObject = localFrame->script().globalObject(world->coreWorld());
+    auto* globalObject = localFrame->script().globalObject(world->coreWorld());
 
     JSLockHolder lock(globalObject);
     return toRef(globalObject, toJS(globalObject, globalObject, nodeHandle->coreNode()));
@@ -874,7 +987,7 @@ JSValueRef WebFrame::jsWrapperForWorld(InjectedBundleRangeHandle* rangeHandle, I
     if (!localFrame)
         return nullptr;
 
-    JSDOMWindow* globalObject = localFrame->script().globalObject(world->coreWorld());
+    auto* globalObject = localFrame->script().globalObject(world->coreWorld());
 
     JSLockHolder lock(globalObject);
     return toRef(globalObject, toJS(globalObject, globalObject, rangeHandle->coreRange()));
@@ -945,6 +1058,12 @@ String WebFrame::mimeTypeForResourceWithURL(const URL& url) const
     return String();
 }
 
+void WebFrame::updateRemoteFrameSize(WebCore::IntSize size)
+{
+    if (m_page)
+        m_page->send(Messages::WebPageProxy::UpdateRemoteFrameSize(m_frameID, size));
+}
+
 void WebFrame::setTextDirection(const String& direction)
 {
     auto* localFrame = dynamicDowncast<LocalFrame>(m_coreFrame.get());
@@ -968,7 +1087,7 @@ void WebFrame::documentLoaderDetached(uint64_t navigationID)
 #if PLATFORM(COCOA)
 RetainPtr<CFDataRef> WebFrame::webArchiveData(FrameFilterFunction callback, void* context)
 {
-    auto archive = LegacyWebArchive::create(*coreFrame()->document(), [this, callback, context](Frame& frame) -> bool {
+    auto archive = LegacyWebArchive::create(*coreLocalFrame()->document(), [this, callback, context](auto& frame) -> bool {
         if (!callback)
             return true;
 
@@ -987,7 +1106,7 @@ RetainPtr<CFDataRef> WebFrame::webArchiveData(FrameFilterFunction callback, void
 
 RefPtr<WebImage> WebFrame::createSelectionSnapshot() const
 {
-    auto snapshot = snapshotSelection(*coreFrame(), { { WebCore::SnapshotFlags::ForceBlackText, WebCore::SnapshotFlags::Shareable }, PixelFormat::BGRA8, DestinationColorSpace::SRGB() });
+    auto snapshot = snapshotSelection(*coreLocalFrame(), { { WebCore::SnapshotFlags::ForceBlackText, WebCore::SnapshotFlags::Shareable }, PixelFormat::BGRA8, DestinationColorSpace::SRGB() });
     if (!snapshot)
         return nullptr;
 
@@ -1017,18 +1136,32 @@ std::optional<NavigatingToAppBoundDomain> WebFrame::isTopFrameNavigatingToAppBou
     auto* localFrame = dynamicDowncast<LocalFrame>(m_coreFrame.get());
     if (!localFrame)
         return std::nullopt;
-    return fromCoreFrame(localFrame->mainFrame())->isNavigatingToAppBoundDomain();
+    auto* localMainFrame = dynamicDowncast<LocalFrame>(localFrame->mainFrame());
+    if (!localMainFrame)
+        return std::nullopt;
+    return fromCoreFrame(*localMainFrame)->isNavigatingToAppBoundDomain();
 }
 #endif
 
-IPC::Connection* WebFrame::messageSenderConnection() const
+OptionSet<WebCore::AdvancedPrivacyProtections> WebFrame::advancedPrivacyProtections() const
 {
-    return WebProcess::singleton().parentProcessConnection();
-}
+    auto* coreFrame = this->coreLocalFrame();
+    if (!coreFrame)
+        return { };
 
-uint64_t WebFrame::messageSenderDestinationID() const
-{
-    return m_frameID.object().toUInt64();
+    auto* document = coreFrame->document();
+    if (!document)
+        return { };
+
+    auto* topDocumentLoader = document->topDocument().loader();
+    if (!topDocumentLoader)
+        return { };
+
+    auto* policySourceDocumentLoader = topDocumentLoader;
+    if (!policySourceDocumentLoader->request().url().hasSpecialScheme() && document->url().protocolIsInHTTPFamily())
+        policySourceDocumentLoader = document->loader();
+
+    return policySourceDocumentLoader->advancedPrivacyProtections();
 }
 
 } // namespace WebKit
